#include "soapH.h"
SOAP_FMAC3 int SOAP_FMAC4 soap_ignore_element(struct soap *soap)
{
	if (!soap_peek_element(soap))
	{	int t;
		DBGLOG(TEST, SOAP_MESSAGE(fdebug, "Unexpected element '%s' in input (level=%u, %d)\n", soap->tag, soap->level, soap->body));
		if (soap->mustUnderstand && !soap->other)
			return soap->error = SOAP_MUSTUNDERSTAND;
		if (((soap->mode & SOAP_XML_STRICT) && soap->part != SOAP_IN_HEADER) || !soap_match_tag(soap, soap->tag, "SOAP-ENV:"))
		{	DBGLOG(TEST, SOAP_MESSAGE(fdebug, "REJECTING element '%s'\n", soap->tag));
			return soap->error = SOAP_TAG_MISMATCH;
		}
		if (!*soap->id || !soap_getelement(soap, &t))
		{	soap->peeked = 0;
			if (soap->fignore)
				soap->error = soap->fignore(soap, soap->tag);
			else
				soap->error = SOAP_OK;
			DBGLOG(TEST, if (!soap->error) SOAP_MESSAGE(fdebug, "IGNORING element '%s'\n", soap->tag));
			if (!soap->error && soap->body)
			{	soap->level++;
				while (!soap_ignore_element(soap))
					;
				if (soap->error == SOAP_NO_TAG)
					soap->error = soap_element_end_in(soap, NULL);
			}
		}
	}
	return soap->error;
}

#ifndef WITH_NOIDREF
SOAP_FMAC3 int SOAP_FMAC4 soap_putindependent(struct soap *soap)
{
	int i;
	struct soap_plist *pp;
	if (soap->version == 1 && soap->encodingStyle && !(soap->mode & (SOAP_XML_TREE | SOAP_XML_GRAPH)))
		for (i = 0; i < SOAP_PTRHASH; i++)
			for (pp = soap->pht[i]; pp; pp = pp->next)
				if (pp->mark1 == 2 || pp->mark2 == 2)
					if (soap_putelement(soap, pp->ptr, "id", pp->id, pp->type))
						return soap->error;
	return SOAP_OK;
}
#endif

#ifndef WITH_NOIDREF

#ifdef __cplusplus
extern "C" {
#endif
SOAP_FMAC3 int SOAP_FMAC4 soap_putelement(struct soap *soap, const void *ptr, const char *tag, int id, int type)
{
	switch (type)
	{
	case SOAP_TYPE_byte:
		return soap_out_byte(soap, tag, id, (const char *)ptr, "xsd:byte");
	case SOAP_TYPE_short:
		return soap_out_short(soap, tag, id, (const short *)ptr, "xsd:short");
	case SOAP_TYPE_int:
		return soap_out_int(soap, tag, id, (const int *)ptr, "xsd:int");
	case SOAP_TYPE_LONG64:
		return soap_out_LONG64(soap, tag, id, (const LONG64 *)ptr, "xsd:long");
	case SOAP_TYPE_unsignedByte:
		return soap_out_unsignedByte(soap, tag, id, (const unsigned char *)ptr, "xsd:unsignedByte");
	case SOAP_TYPE_unsignedShort:
		return soap_out_unsignedShort(soap, tag, id, (const unsigned short *)ptr, "xsd:unsignedShort");
	case SOAP_TYPE_unsignedInt:
		return soap_out_unsignedInt(soap, tag, id, (const unsigned int *)ptr, "xsd:unsignedInt");
	case SOAP_TYPE_unsignedLONG64:
		return soap_out_unsignedLONG64(soap, tag, id, (const ULONG64 *)ptr, "xsd:unsignedLong");
	case SOAP_TYPE_bool:
		return soap_out_bool(soap, tag, id, (const bool *)ptr, "xsd:boolean");
	case SOAP_TYPE_vbox__StorageControllerType:
		return soap_out_vbox__StorageControllerType(soap, tag, id, (const enum vbox__StorageControllerType *)ptr, "vbox:StorageControllerType");
	case SOAP_TYPE_vbox__StorageBus:
		return soap_out_vbox__StorageBus(soap, tag, id, (const enum vbox__StorageBus *)ptr, "vbox:StorageBus");
	case SOAP_TYPE_vbox__VRDPAuthType:
		return soap_out_vbox__VRDPAuthType(soap, tag, id, (const enum vbox__VRDPAuthType *)ptr, "vbox:VRDPAuthType");
	case SOAP_TYPE_vbox__AudioControllerType:
		return soap_out_vbox__AudioControllerType(soap, tag, id, (const enum vbox__AudioControllerType *)ptr, "vbox:AudioControllerType");
	case SOAP_TYPE_vbox__AudioDriverType:
		return soap_out_vbox__AudioDriverType(soap, tag, id, (const enum vbox__AudioDriverType *)ptr, "vbox:AudioDriverType");
	case SOAP_TYPE_vbox__USBDeviceFilterAction:
		return soap_out_vbox__USBDeviceFilterAction(soap, tag, id, (const enum vbox__USBDeviceFilterAction *)ptr, "vbox:USBDeviceFilterAction");
	case SOAP_TYPE_vbox__USBDeviceState:
		return soap_out_vbox__USBDeviceState(soap, tag, id, (const enum vbox__USBDeviceState *)ptr, "vbox:USBDeviceState");
	case SOAP_TYPE_vbox__PortMode:
		return soap_out_vbox__PortMode(soap, tag, id, (const enum vbox__PortMode *)ptr, "vbox:PortMode");
	case SOAP_TYPE_vbox__NetworkAdapterType:
		return soap_out_vbox__NetworkAdapterType(soap, tag, id, (const enum vbox__NetworkAdapterType *)ptr, "vbox:NetworkAdapterType");
	case SOAP_TYPE_vbox__NetworkAttachmentType:
		return soap_out_vbox__NetworkAttachmentType(soap, tag, id, (const enum vbox__NetworkAttachmentType *)ptr, "vbox:NetworkAttachmentType");
	case SOAP_TYPE_vbox__FramebufferPixelFormat:
		return soap_out_vbox__FramebufferPixelFormat(soap, tag, id, (const enum vbox__FramebufferPixelFormat *)ptr, "vbox:FramebufferPixelFormat");
	case SOAP_TYPE_vbox__MouseButtonState:
		return soap_out_vbox__MouseButtonState(soap, tag, id, (const enum vbox__MouseButtonState *)ptr, "vbox:MouseButtonState");
	case SOAP_TYPE_vbox__HardDiskFormatCapabilities:
		return soap_out_vbox__HardDiskFormatCapabilities(soap, tag, id, (const enum vbox__HardDiskFormatCapabilities *)ptr, "vbox:HardDiskFormatCapabilities");
	case SOAP_TYPE_vbox__DataFlags:
		return soap_out_vbox__DataFlags(soap, tag, id, (const enum vbox__DataFlags *)ptr, "vbox:DataFlags");
	case SOAP_TYPE_vbox__DataType:
		return soap_out_vbox__DataType(soap, tag, id, (const enum vbox__DataType *)ptr, "vbox:DataType");
	case SOAP_TYPE_vbox__HardDiskVariant:
		return soap_out_vbox__HardDiskVariant(soap, tag, id, (const enum vbox__HardDiskVariant *)ptr, "vbox:HardDiskVariant");
	case SOAP_TYPE_vbox__HardDiskType:
		return soap_out_vbox__HardDiskType(soap, tag, id, (const enum vbox__HardDiskType *)ptr, "vbox:HardDiskType");
	case SOAP_TYPE_vbox__MediaState:
		return soap_out_vbox__MediaState(soap, tag, id, (const enum vbox__MediaState *)ptr, "vbox:MediaState");
	case SOAP_TYPE_vbox__HostNetworkInterfaceType:
		return soap_out_vbox__HostNetworkInterfaceType(soap, tag, id, (const enum vbox__HostNetworkInterfaceType *)ptr, "vbox:HostNetworkInterfaceType");
	case SOAP_TYPE_vbox__HostNetworkInterfaceStatus:
		return soap_out_vbox__HostNetworkInterfaceStatus(soap, tag, id, (const enum vbox__HostNetworkInterfaceStatus *)ptr, "vbox:HostNetworkInterfaceStatus");
	case SOAP_TYPE_vbox__HostNetworkInterfaceMediumType:
		return soap_out_vbox__HostNetworkInterfaceMediumType(soap, tag, id, (const enum vbox__HostNetworkInterfaceMediumType *)ptr, "vbox:HostNetworkInterfaceMediumType");
	case SOAP_TYPE_vbox__VirtualSystemDescriptionValueType:
		return soap_out_vbox__VirtualSystemDescriptionValueType(soap, tag, id, (const enum vbox__VirtualSystemDescriptionValueType *)ptr, "vbox:VirtualSystemDescriptionValueType");
	case SOAP_TYPE_vbox__VirtualSystemDescriptionType:
		return soap_out_vbox__VirtualSystemDescriptionType(soap, tag, id, (const enum vbox__VirtualSystemDescriptionType *)ptr, "vbox:VirtualSystemDescriptionType");
	case SOAP_TYPE_vbox__OVFResourceType:
		return soap_out_vbox__OVFResourceType(soap, tag, id, (const enum vbox__OVFResourceType *)ptr, "vbox:OVFResourceType");
	case SOAP_TYPE_vbox__CIMOSType:
		return soap_out_vbox__CIMOSType(soap, tag, id, (const enum vbox__CIMOSType *)ptr, "vbox:CIMOSType");
	case SOAP_TYPE_vbox__VFSFileType:
		return soap_out_vbox__VFSFileType(soap, tag, id, (const enum vbox__VFSFileType *)ptr, "vbox:VFSFileType");
	case SOAP_TYPE_vbox__VFSType:
		return soap_out_vbox__VFSType(soap, tag, id, (const enum vbox__VFSType *)ptr, "vbox:VFSType");
	case SOAP_TYPE_vbox__ProcessorFeature:
		return soap_out_vbox__ProcessorFeature(soap, tag, id, (const enum vbox__ProcessorFeature *)ptr, "vbox:ProcessorFeature");
	case SOAP_TYPE_vbox__DriveState:
		return soap_out_vbox__DriveState(soap, tag, id, (const enum vbox__DriveState *)ptr, "vbox:DriveState");
	case SOAP_TYPE_vbox__BIOSBootMenuMode:
		return soap_out_vbox__BIOSBootMenuMode(soap, tag, id, (const enum vbox__BIOSBootMenuMode *)ptr, "vbox:BIOSBootMenuMode");
	case SOAP_TYPE_vbox__GuestStatisticType:
		return soap_out_vbox__GuestStatisticType(soap, tag, id, (const enum vbox__GuestStatisticType *)ptr, "vbox:GuestStatisticType");
	case SOAP_TYPE_vbox__Scope:
		return soap_out_vbox__Scope(soap, tag, id, (const enum vbox__Scope *)ptr, "vbox:Scope");
	case SOAP_TYPE_vbox__ClipboardMode:
		return soap_out_vbox__ClipboardMode(soap, tag, id, (const enum vbox__ClipboardMode *)ptr, "vbox:ClipboardMode");
	case SOAP_TYPE_vbox__DeviceActivity:
		return soap_out_vbox__DeviceActivity(soap, tag, id, (const enum vbox__DeviceActivity *)ptr, "vbox:DeviceActivity");
	case SOAP_TYPE_vbox__DeviceType:
		return soap_out_vbox__DeviceType(soap, tag, id, (const enum vbox__DeviceType *)ptr, "vbox:DeviceType");
	case SOAP_TYPE_vbox__SessionType:
		return soap_out_vbox__SessionType(soap, tag, id, (const enum vbox__SessionType *)ptr, "vbox:SessionType");
	case SOAP_TYPE_vbox__SessionState:
		return soap_out_vbox__SessionState(soap, tag, id, (const enum vbox__SessionState *)ptr, "vbox:SessionState");
	case SOAP_TYPE_vbox__MachineState:
		return soap_out_vbox__MachineState(soap, tag, id, (const enum vbox__MachineState *)ptr, "vbox:MachineState");
	case SOAP_TYPE_vbox__AccessMode:
		return soap_out_vbox__AccessMode(soap, tag, id, (const enum vbox__AccessMode *)ptr, "vbox:AccessMode");
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREqueryMetricsDataResponse:
		return ((_vbox__IPerformanceCollector_USCOREqueryMetricsDataResponse *)ptr)->soap_out(soap, "vbox:IPerformanceCollector_queryMetricsDataResponse", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREqueryMetricsData:
		return ((_vbox__IPerformanceCollector_USCOREqueryMetricsData *)ptr)->soap_out(soap, "vbox:IPerformanceCollector_queryMetricsData", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREdisableMetricsResponse:
		return ((_vbox__IPerformanceCollector_USCOREdisableMetricsResponse *)ptr)->soap_out(soap, "vbox:IPerformanceCollector_disableMetricsResponse", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREdisableMetrics:
		return ((_vbox__IPerformanceCollector_USCOREdisableMetrics *)ptr)->soap_out(soap, "vbox:IPerformanceCollector_disableMetrics", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREenableMetricsResponse:
		return ((_vbox__IPerformanceCollector_USCOREenableMetricsResponse *)ptr)->soap_out(soap, "vbox:IPerformanceCollector_enableMetricsResponse", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREenableMetrics:
		return ((_vbox__IPerformanceCollector_USCOREenableMetrics *)ptr)->soap_out(soap, "vbox:IPerformanceCollector_enableMetrics", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREsetupMetricsResponse:
		return ((_vbox__IPerformanceCollector_USCOREsetupMetricsResponse *)ptr)->soap_out(soap, "vbox:IPerformanceCollector_setupMetricsResponse", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREsetupMetrics:
		return ((_vbox__IPerformanceCollector_USCOREsetupMetrics *)ptr)->soap_out(soap, "vbox:IPerformanceCollector_setupMetrics", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREgetMetricsResponse:
		return ((_vbox__IPerformanceCollector_USCOREgetMetricsResponse *)ptr)->soap_out(soap, "vbox:IPerformanceCollector_getMetricsResponse", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREgetMetrics:
		return ((_vbox__IPerformanceCollector_USCOREgetMetrics *)ptr)->soap_out(soap, "vbox:IPerformanceCollector_getMetrics", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREgetMetricNamesResponse:
		return ((_vbox__IPerformanceCollector_USCOREgetMetricNamesResponse *)ptr)->soap_out(soap, "vbox:IPerformanceCollector_getMetricNamesResponse", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREgetMetricNames:
		return ((_vbox__IPerformanceCollector_USCOREgetMetricNames *)ptr)->soap_out(soap, "vbox:IPerformanceCollector_getMetricNames", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetMaximumValueResponse:
		return ((_vbox__IPerformanceMetric_USCOREgetMaximumValueResponse *)ptr)->soap_out(soap, "vbox:IPerformanceMetric_getMaximumValueResponse", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetMaximumValue:
		return ((_vbox__IPerformanceMetric_USCOREgetMaximumValue *)ptr)->soap_out(soap, "vbox:IPerformanceMetric_getMaximumValue", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetMinimumValueResponse:
		return ((_vbox__IPerformanceMetric_USCOREgetMinimumValueResponse *)ptr)->soap_out(soap, "vbox:IPerformanceMetric_getMinimumValueResponse", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetMinimumValue:
		return ((_vbox__IPerformanceMetric_USCOREgetMinimumValue *)ptr)->soap_out(soap, "vbox:IPerformanceMetric_getMinimumValue", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetUnitResponse:
		return ((_vbox__IPerformanceMetric_USCOREgetUnitResponse *)ptr)->soap_out(soap, "vbox:IPerformanceMetric_getUnitResponse", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetUnit:
		return ((_vbox__IPerformanceMetric_USCOREgetUnit *)ptr)->soap_out(soap, "vbox:IPerformanceMetric_getUnit", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetCountResponse:
		return ((_vbox__IPerformanceMetric_USCOREgetCountResponse *)ptr)->soap_out(soap, "vbox:IPerformanceMetric_getCountResponse", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetCount:
		return ((_vbox__IPerformanceMetric_USCOREgetCount *)ptr)->soap_out(soap, "vbox:IPerformanceMetric_getCount", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetPeriodResponse:
		return ((_vbox__IPerformanceMetric_USCOREgetPeriodResponse *)ptr)->soap_out(soap, "vbox:IPerformanceMetric_getPeriodResponse", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetPeriod:
		return ((_vbox__IPerformanceMetric_USCOREgetPeriod *)ptr)->soap_out(soap, "vbox:IPerformanceMetric_getPeriod", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetDescriptionResponse:
		return ((_vbox__IPerformanceMetric_USCOREgetDescriptionResponse *)ptr)->soap_out(soap, "vbox:IPerformanceMetric_getDescriptionResponse", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetDescription:
		return ((_vbox__IPerformanceMetric_USCOREgetDescription *)ptr)->soap_out(soap, "vbox:IPerformanceMetric_getDescription", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetObjectResponse:
		return ((_vbox__IPerformanceMetric_USCOREgetObjectResponse *)ptr)->soap_out(soap, "vbox:IPerformanceMetric_getObjectResponse", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetObject:
		return ((_vbox__IPerformanceMetric_USCOREgetObject *)ptr)->soap_out(soap, "vbox:IPerformanceMetric_getObject", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetMetricNameResponse:
		return ((_vbox__IPerformanceMetric_USCOREgetMetricNameResponse *)ptr)->soap_out(soap, "vbox:IPerformanceMetric_getMetricNameResponse", id, NULL);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetMetricName:
		return ((_vbox__IPerformanceMetric_USCOREgetMetricName *)ptr)->soap_out(soap, "vbox:IPerformanceMetric_getMetricName", id, NULL);
	case SOAP_TYPE__vbox__IWebsessionManager_USCORElogoffResponse:
		return ((_vbox__IWebsessionManager_USCORElogoffResponse *)ptr)->soap_out(soap, "vbox:IWebsessionManager_logoffResponse", id, NULL);
	case SOAP_TYPE__vbox__IWebsessionManager_USCORElogoff:
		return ((_vbox__IWebsessionManager_USCORElogoff *)ptr)->soap_out(soap, "vbox:IWebsessionManager_logoff", id, NULL);
	case SOAP_TYPE__vbox__IWebsessionManager_USCOREgetSessionObjectResponse:
		return ((_vbox__IWebsessionManager_USCOREgetSessionObjectResponse *)ptr)->soap_out(soap, "vbox:IWebsessionManager_getSessionObjectResponse", id, NULL);
	case SOAP_TYPE__vbox__IWebsessionManager_USCOREgetSessionObject:
		return ((_vbox__IWebsessionManager_USCOREgetSessionObject *)ptr)->soap_out(soap, "vbox:IWebsessionManager_getSessionObject", id, NULL);
	case SOAP_TYPE__vbox__IWebsessionManager_USCORElogonResponse:
		return ((_vbox__IWebsessionManager_USCORElogonResponse *)ptr)->soap_out(soap, "vbox:IWebsessionManager_logonResponse", id, NULL);
	case SOAP_TYPE__vbox__IWebsessionManager_USCORElogon:
		return ((_vbox__IWebsessionManager_USCORElogon *)ptr)->soap_out(soap, "vbox:IWebsessionManager_logon", id, NULL);
	case SOAP_TYPE__vbox__IManagedObjectRef_USCOREreleaseResponse:
		return ((_vbox__IManagedObjectRef_USCOREreleaseResponse *)ptr)->soap_out(soap, "vbox:IManagedObjectRef_releaseResponse", id, NULL);
	case SOAP_TYPE__vbox__IManagedObjectRef_USCORErelease:
		return ((_vbox__IManagedObjectRef_USCORErelease *)ptr)->soap_out(soap, "vbox:IManagedObjectRef_release", id, NULL);
	case SOAP_TYPE__vbox__IManagedObjectRef_USCOREgetInterfaceNameResponse:
		return ((_vbox__IManagedObjectRef_USCOREgetInterfaceNameResponse *)ptr)->soap_out(soap, "vbox:IManagedObjectRef_getInterfaceNameResponse", id, NULL);
	case SOAP_TYPE__vbox__IManagedObjectRef_USCOREgetInterfaceName:
		return ((_vbox__IManagedObjectRef_USCOREgetInterfaceName *)ptr)->soap_out(soap, "vbox:IManagedObjectRef_getInterfaceName", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCORESetIDEEmulationPortResponse:
		return ((_vbox__IStorageController_USCORESetIDEEmulationPortResponse *)ptr)->soap_out(soap, "vbox:IStorageController_SetIDEEmulationPortResponse", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCORESetIDEEmulationPort:
		return ((_vbox__IStorageController_USCORESetIDEEmulationPort *)ptr)->soap_out(soap, "vbox:IStorageController_SetIDEEmulationPort", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCOREGetIDEEmulationPortResponse:
		return ((_vbox__IStorageController_USCOREGetIDEEmulationPortResponse *)ptr)->soap_out(soap, "vbox:IStorageController_GetIDEEmulationPortResponse", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCOREGetIDEEmulationPort:
		return ((_vbox__IStorageController_USCOREGetIDEEmulationPort *)ptr)->soap_out(soap, "vbox:IStorageController_GetIDEEmulationPort", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCOREsetControllerTypeResponse:
		return ((_vbox__IStorageController_USCOREsetControllerTypeResponse *)ptr)->soap_out(soap, "vbox:IStorageController_setControllerTypeResponse", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCOREsetControllerType:
		return ((_vbox__IStorageController_USCOREsetControllerType *)ptr)->soap_out(soap, "vbox:IStorageController_setControllerType", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetControllerTypeResponse:
		return ((_vbox__IStorageController_USCOREgetControllerTypeResponse *)ptr)->soap_out(soap, "vbox:IStorageController_getControllerTypeResponse", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetControllerType:
		return ((_vbox__IStorageController_USCOREgetControllerType *)ptr)->soap_out(soap, "vbox:IStorageController_getControllerType", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetBusResponse:
		return ((_vbox__IStorageController_USCOREgetBusResponse *)ptr)->soap_out(soap, "vbox:IStorageController_getBusResponse", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetBus:
		return ((_vbox__IStorageController_USCOREgetBus *)ptr)->soap_out(soap, "vbox:IStorageController_getBus", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCOREsetPortCountResponse:
		return ((_vbox__IStorageController_USCOREsetPortCountResponse *)ptr)->soap_out(soap, "vbox:IStorageController_setPortCountResponse", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCOREsetPortCount:
		return ((_vbox__IStorageController_USCOREsetPortCount *)ptr)->soap_out(soap, "vbox:IStorageController_setPortCount", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetPortCountResponse:
		return ((_vbox__IStorageController_USCOREgetPortCountResponse *)ptr)->soap_out(soap, "vbox:IStorageController_getPortCountResponse", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetPortCount:
		return ((_vbox__IStorageController_USCOREgetPortCount *)ptr)->soap_out(soap, "vbox:IStorageController_getPortCount", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCOREsetInstanceResponse:
		return ((_vbox__IStorageController_USCOREsetInstanceResponse *)ptr)->soap_out(soap, "vbox:IStorageController_setInstanceResponse", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCOREsetInstance:
		return ((_vbox__IStorageController_USCOREsetInstance *)ptr)->soap_out(soap, "vbox:IStorageController_setInstance", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetInstanceResponse:
		return ((_vbox__IStorageController_USCOREgetInstanceResponse *)ptr)->soap_out(soap, "vbox:IStorageController_getInstanceResponse", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetInstance:
		return ((_vbox__IStorageController_USCOREgetInstance *)ptr)->soap_out(soap, "vbox:IStorageController_getInstance", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetMaxPortCountResponse:
		return ((_vbox__IStorageController_USCOREgetMaxPortCountResponse *)ptr)->soap_out(soap, "vbox:IStorageController_getMaxPortCountResponse", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetMaxPortCount:
		return ((_vbox__IStorageController_USCOREgetMaxPortCount *)ptr)->soap_out(soap, "vbox:IStorageController_getMaxPortCount", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetMinPortCountResponse:
		return ((_vbox__IStorageController_USCOREgetMinPortCountResponse *)ptr)->soap_out(soap, "vbox:IStorageController_getMinPortCountResponse", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetMinPortCount:
		return ((_vbox__IStorageController_USCOREgetMinPortCount *)ptr)->soap_out(soap, "vbox:IStorageController_getMinPortCount", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetMaxDevicesPerPortCountResponse:
		return ((_vbox__IStorageController_USCOREgetMaxDevicesPerPortCountResponse *)ptr)->soap_out(soap, "vbox:IStorageController_getMaxDevicesPerPortCountResponse", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetMaxDevicesPerPortCount:
		return ((_vbox__IStorageController_USCOREgetMaxDevicesPerPortCount *)ptr)->soap_out(soap, "vbox:IStorageController_getMaxDevicesPerPortCount", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetNameResponse:
		return ((_vbox__IStorageController_USCOREgetNameResponse *)ptr)->soap_out(soap, "vbox:IStorageController_getNameResponse", id, NULL);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetName:
		return ((_vbox__IStorageController_USCOREgetName *)ptr)->soap_out(soap, "vbox:IStorageController_getName", id, NULL);
	case SOAP_TYPE__vbox__ISession_USCOREcloseResponse:
		return ((_vbox__ISession_USCOREcloseResponse *)ptr)->soap_out(soap, "vbox:ISession_closeResponse", id, NULL);
	case SOAP_TYPE__vbox__ISession_USCOREclose:
		return ((_vbox__ISession_USCOREclose *)ptr)->soap_out(soap, "vbox:ISession_close", id, NULL);
	case SOAP_TYPE__vbox__ISession_USCOREgetConsoleResponse:
		return ((_vbox__ISession_USCOREgetConsoleResponse *)ptr)->soap_out(soap, "vbox:ISession_getConsoleResponse", id, NULL);
	case SOAP_TYPE__vbox__ISession_USCOREgetConsole:
		return ((_vbox__ISession_USCOREgetConsole *)ptr)->soap_out(soap, "vbox:ISession_getConsole", id, NULL);
	case SOAP_TYPE__vbox__ISession_USCOREgetMachineResponse:
		return ((_vbox__ISession_USCOREgetMachineResponse *)ptr)->soap_out(soap, "vbox:ISession_getMachineResponse", id, NULL);
	case SOAP_TYPE__vbox__ISession_USCOREgetMachine:
		return ((_vbox__ISession_USCOREgetMachine *)ptr)->soap_out(soap, "vbox:ISession_getMachine", id, NULL);
	case SOAP_TYPE__vbox__ISession_USCOREgetTypeResponse:
		return ((_vbox__ISession_USCOREgetTypeResponse *)ptr)->soap_out(soap, "vbox:ISession_getTypeResponse", id, NULL);
	case SOAP_TYPE__vbox__ISession_USCOREgetType:
		return ((_vbox__ISession_USCOREgetType *)ptr)->soap_out(soap, "vbox:ISession_getType", id, NULL);
	case SOAP_TYPE__vbox__ISession_USCOREgetStateResponse:
		return ((_vbox__ISession_USCOREgetStateResponse *)ptr)->soap_out(soap, "vbox:ISession_getStateResponse", id, NULL);
	case SOAP_TYPE__vbox__ISession_USCOREgetState:
		return ((_vbox__ISession_USCOREgetState *)ptr)->soap_out(soap, "vbox:ISession_getState", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetReuseSingleConnectionResponse:
		return ((_vbox__IVRDPServer_USCOREsetReuseSingleConnectionResponse *)ptr)->soap_out(soap, "vbox:IVRDPServer_setReuseSingleConnectionResponse", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetReuseSingleConnection:
		return ((_vbox__IVRDPServer_USCOREsetReuseSingleConnection *)ptr)->soap_out(soap, "vbox:IVRDPServer_setReuseSingleConnection", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetReuseSingleConnectionResponse:
		return ((_vbox__IVRDPServer_USCOREgetReuseSingleConnectionResponse *)ptr)->soap_out(soap, "vbox:IVRDPServer_getReuseSingleConnectionResponse", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetReuseSingleConnection:
		return ((_vbox__IVRDPServer_USCOREgetReuseSingleConnection *)ptr)->soap_out(soap, "vbox:IVRDPServer_getReuseSingleConnection", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetAllowMultiConnectionResponse:
		return ((_vbox__IVRDPServer_USCOREsetAllowMultiConnectionResponse *)ptr)->soap_out(soap, "vbox:IVRDPServer_setAllowMultiConnectionResponse", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetAllowMultiConnection:
		return ((_vbox__IVRDPServer_USCOREsetAllowMultiConnection *)ptr)->soap_out(soap, "vbox:IVRDPServer_setAllowMultiConnection", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetAllowMultiConnectionResponse:
		return ((_vbox__IVRDPServer_USCOREgetAllowMultiConnectionResponse *)ptr)->soap_out(soap, "vbox:IVRDPServer_getAllowMultiConnectionResponse", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetAllowMultiConnection:
		return ((_vbox__IVRDPServer_USCOREgetAllowMultiConnection *)ptr)->soap_out(soap, "vbox:IVRDPServer_getAllowMultiConnection", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetAuthTimeoutResponse:
		return ((_vbox__IVRDPServer_USCOREsetAuthTimeoutResponse *)ptr)->soap_out(soap, "vbox:IVRDPServer_setAuthTimeoutResponse", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetAuthTimeout:
		return ((_vbox__IVRDPServer_USCOREsetAuthTimeout *)ptr)->soap_out(soap, "vbox:IVRDPServer_setAuthTimeout", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetAuthTimeoutResponse:
		return ((_vbox__IVRDPServer_USCOREgetAuthTimeoutResponse *)ptr)->soap_out(soap, "vbox:IVRDPServer_getAuthTimeoutResponse", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetAuthTimeout:
		return ((_vbox__IVRDPServer_USCOREgetAuthTimeout *)ptr)->soap_out(soap, "vbox:IVRDPServer_getAuthTimeout", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetAuthTypeResponse:
		return ((_vbox__IVRDPServer_USCOREsetAuthTypeResponse *)ptr)->soap_out(soap, "vbox:IVRDPServer_setAuthTypeResponse", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetAuthType:
		return ((_vbox__IVRDPServer_USCOREsetAuthType *)ptr)->soap_out(soap, "vbox:IVRDPServer_setAuthType", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetAuthTypeResponse:
		return ((_vbox__IVRDPServer_USCOREgetAuthTypeResponse *)ptr)->soap_out(soap, "vbox:IVRDPServer_getAuthTypeResponse", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetAuthType:
		return ((_vbox__IVRDPServer_USCOREgetAuthType *)ptr)->soap_out(soap, "vbox:IVRDPServer_getAuthType", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetNetAddressResponse:
		return ((_vbox__IVRDPServer_USCOREsetNetAddressResponse *)ptr)->soap_out(soap, "vbox:IVRDPServer_setNetAddressResponse", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetNetAddress:
		return ((_vbox__IVRDPServer_USCOREsetNetAddress *)ptr)->soap_out(soap, "vbox:IVRDPServer_setNetAddress", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetNetAddressResponse:
		return ((_vbox__IVRDPServer_USCOREgetNetAddressResponse *)ptr)->soap_out(soap, "vbox:IVRDPServer_getNetAddressResponse", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetNetAddress:
		return ((_vbox__IVRDPServer_USCOREgetNetAddress *)ptr)->soap_out(soap, "vbox:IVRDPServer_getNetAddress", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetPortResponse:
		return ((_vbox__IVRDPServer_USCOREsetPortResponse *)ptr)->soap_out(soap, "vbox:IVRDPServer_setPortResponse", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetPort:
		return ((_vbox__IVRDPServer_USCOREsetPort *)ptr)->soap_out(soap, "vbox:IVRDPServer_setPort", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetPortResponse:
		return ((_vbox__IVRDPServer_USCOREgetPortResponse *)ptr)->soap_out(soap, "vbox:IVRDPServer_getPortResponse", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetPort:
		return ((_vbox__IVRDPServer_USCOREgetPort *)ptr)->soap_out(soap, "vbox:IVRDPServer_getPort", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetEnabledResponse:
		return ((_vbox__IVRDPServer_USCOREsetEnabledResponse *)ptr)->soap_out(soap, "vbox:IVRDPServer_setEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetEnabled:
		return ((_vbox__IVRDPServer_USCOREsetEnabled *)ptr)->soap_out(soap, "vbox:IVRDPServer_setEnabled", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetEnabledResponse:
		return ((_vbox__IVRDPServer_USCOREgetEnabledResponse *)ptr)->soap_out(soap, "vbox:IVRDPServer_getEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetEnabled:
		return ((_vbox__IVRDPServer_USCOREgetEnabled *)ptr)->soap_out(soap, "vbox:IVRDPServer_getEnabled", id, NULL);
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREsetAudioDriverResponse:
		return ((_vbox__IAudioAdapter_USCOREsetAudioDriverResponse *)ptr)->soap_out(soap, "vbox:IAudioAdapter_setAudioDriverResponse", id, NULL);
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREsetAudioDriver:
		return ((_vbox__IAudioAdapter_USCOREsetAudioDriver *)ptr)->soap_out(soap, "vbox:IAudioAdapter_setAudioDriver", id, NULL);
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREgetAudioDriverResponse:
		return ((_vbox__IAudioAdapter_USCOREgetAudioDriverResponse *)ptr)->soap_out(soap, "vbox:IAudioAdapter_getAudioDriverResponse", id, NULL);
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREgetAudioDriver:
		return ((_vbox__IAudioAdapter_USCOREgetAudioDriver *)ptr)->soap_out(soap, "vbox:IAudioAdapter_getAudioDriver", id, NULL);
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREsetAudioControllerResponse:
		return ((_vbox__IAudioAdapter_USCOREsetAudioControllerResponse *)ptr)->soap_out(soap, "vbox:IAudioAdapter_setAudioControllerResponse", id, NULL);
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREsetAudioController:
		return ((_vbox__IAudioAdapter_USCOREsetAudioController *)ptr)->soap_out(soap, "vbox:IAudioAdapter_setAudioController", id, NULL);
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREgetAudioControllerResponse:
		return ((_vbox__IAudioAdapter_USCOREgetAudioControllerResponse *)ptr)->soap_out(soap, "vbox:IAudioAdapter_getAudioControllerResponse", id, NULL);
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREgetAudioController:
		return ((_vbox__IAudioAdapter_USCOREgetAudioController *)ptr)->soap_out(soap, "vbox:IAudioAdapter_getAudioController", id, NULL);
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREsetEnabledResponse:
		return ((_vbox__IAudioAdapter_USCOREsetEnabledResponse *)ptr)->soap_out(soap, "vbox:IAudioAdapter_setEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREsetEnabled:
		return ((_vbox__IAudioAdapter_USCOREsetEnabled *)ptr)->soap_out(soap, "vbox:IAudioAdapter_setEnabled", id, NULL);
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREgetEnabledResponse:
		return ((_vbox__IAudioAdapter_USCOREgetEnabledResponse *)ptr)->soap_out(soap, "vbox:IAudioAdapter_getEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREgetEnabled:
		return ((_vbox__IAudioAdapter_USCOREgetEnabled *)ptr)->soap_out(soap, "vbox:IAudioAdapter_getEnabled", id, NULL);
	case SOAP_TYPE__vbox__IHostUSBDeviceFilter_USCOREsetActionResponse:
		return ((_vbox__IHostUSBDeviceFilter_USCOREsetActionResponse *)ptr)->soap_out(soap, "vbox:IHostUSBDeviceFilter_setActionResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostUSBDeviceFilter_USCOREsetAction:
		return ((_vbox__IHostUSBDeviceFilter_USCOREsetAction *)ptr)->soap_out(soap, "vbox:IHostUSBDeviceFilter_setAction", id, NULL);
	case SOAP_TYPE__vbox__IHostUSBDeviceFilter_USCOREgetActionResponse:
		return ((_vbox__IHostUSBDeviceFilter_USCOREgetActionResponse *)ptr)->soap_out(soap, "vbox:IHostUSBDeviceFilter_getActionResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostUSBDeviceFilter_USCOREgetAction:
		return ((_vbox__IHostUSBDeviceFilter_USCOREgetAction *)ptr)->soap_out(soap, "vbox:IHostUSBDeviceFilter_getAction", id, NULL);
	case SOAP_TYPE__vbox__IHostUSBDevice_USCOREgetStateResponse:
		return ((_vbox__IHostUSBDevice_USCOREgetStateResponse *)ptr)->soap_out(soap, "vbox:IHostUSBDevice_getStateResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostUSBDevice_USCOREgetState:
		return ((_vbox__IHostUSBDevice_USCOREgetState *)ptr)->soap_out(soap, "vbox:IHostUSBDevice_getState", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetMaskedInterfacesResponse:
		return ((_vbox__IUSBDeviceFilter_USCOREsetMaskedInterfacesResponse *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_setMaskedInterfacesResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetMaskedInterfaces:
		return ((_vbox__IUSBDeviceFilter_USCOREsetMaskedInterfaces *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_setMaskedInterfaces", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetMaskedInterfacesResponse:
		return ((_vbox__IUSBDeviceFilter_USCOREgetMaskedInterfacesResponse *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_getMaskedInterfacesResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetMaskedInterfaces:
		return ((_vbox__IUSBDeviceFilter_USCOREgetMaskedInterfaces *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_getMaskedInterfaces", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetRemoteResponse:
		return ((_vbox__IUSBDeviceFilter_USCOREsetRemoteResponse *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_setRemoteResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetRemote:
		return ((_vbox__IUSBDeviceFilter_USCOREsetRemote *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_setRemote", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetRemoteResponse:
		return ((_vbox__IUSBDeviceFilter_USCOREgetRemoteResponse *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_getRemoteResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetRemote:
		return ((_vbox__IUSBDeviceFilter_USCOREgetRemote *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_getRemote", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetPortResponse:
		return ((_vbox__IUSBDeviceFilter_USCOREsetPortResponse *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_setPortResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetPort:
		return ((_vbox__IUSBDeviceFilter_USCOREsetPort *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_setPort", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetPortResponse:
		return ((_vbox__IUSBDeviceFilter_USCOREgetPortResponse *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_getPortResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetPort:
		return ((_vbox__IUSBDeviceFilter_USCOREgetPort *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_getPort", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetSerialNumberResponse:
		return ((_vbox__IUSBDeviceFilter_USCOREsetSerialNumberResponse *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_setSerialNumberResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetSerialNumber:
		return ((_vbox__IUSBDeviceFilter_USCOREsetSerialNumber *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_setSerialNumber", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetSerialNumberResponse:
		return ((_vbox__IUSBDeviceFilter_USCOREgetSerialNumberResponse *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_getSerialNumberResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetSerialNumber:
		return ((_vbox__IUSBDeviceFilter_USCOREgetSerialNumber *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_getSerialNumber", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetProductResponse:
		return ((_vbox__IUSBDeviceFilter_USCOREsetProductResponse *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_setProductResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetProduct:
		return ((_vbox__IUSBDeviceFilter_USCOREsetProduct *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_setProduct", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetProductResponse:
		return ((_vbox__IUSBDeviceFilter_USCOREgetProductResponse *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_getProductResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetProduct:
		return ((_vbox__IUSBDeviceFilter_USCOREgetProduct *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_getProduct", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetManufacturerResponse:
		return ((_vbox__IUSBDeviceFilter_USCOREsetManufacturerResponse *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_setManufacturerResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetManufacturer:
		return ((_vbox__IUSBDeviceFilter_USCOREsetManufacturer *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_setManufacturer", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetManufacturerResponse:
		return ((_vbox__IUSBDeviceFilter_USCOREgetManufacturerResponse *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_getManufacturerResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetManufacturer:
		return ((_vbox__IUSBDeviceFilter_USCOREgetManufacturer *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_getManufacturer", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetRevisionResponse:
		return ((_vbox__IUSBDeviceFilter_USCOREsetRevisionResponse *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_setRevisionResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetRevision:
		return ((_vbox__IUSBDeviceFilter_USCOREsetRevision *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_setRevision", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetRevisionResponse:
		return ((_vbox__IUSBDeviceFilter_USCOREgetRevisionResponse *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_getRevisionResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetRevision:
		return ((_vbox__IUSBDeviceFilter_USCOREgetRevision *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_getRevision", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetProductIdResponse:
		return ((_vbox__IUSBDeviceFilter_USCOREsetProductIdResponse *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_setProductIdResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetProductId:
		return ((_vbox__IUSBDeviceFilter_USCOREsetProductId *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_setProductId", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetProductIdResponse:
		return ((_vbox__IUSBDeviceFilter_USCOREgetProductIdResponse *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_getProductIdResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetProductId:
		return ((_vbox__IUSBDeviceFilter_USCOREgetProductId *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_getProductId", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetVendorIdResponse:
		return ((_vbox__IUSBDeviceFilter_USCOREsetVendorIdResponse *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_setVendorIdResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetVendorId:
		return ((_vbox__IUSBDeviceFilter_USCOREsetVendorId *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_setVendorId", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetVendorIdResponse:
		return ((_vbox__IUSBDeviceFilter_USCOREgetVendorIdResponse *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_getVendorIdResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetVendorId:
		return ((_vbox__IUSBDeviceFilter_USCOREgetVendorId *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_getVendorId", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetActiveResponse:
		return ((_vbox__IUSBDeviceFilter_USCOREsetActiveResponse *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_setActiveResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetActive:
		return ((_vbox__IUSBDeviceFilter_USCOREsetActive *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_setActive", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetActiveResponse:
		return ((_vbox__IUSBDeviceFilter_USCOREgetActiveResponse *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_getActiveResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetActive:
		return ((_vbox__IUSBDeviceFilter_USCOREgetActive *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_getActive", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetNameResponse:
		return ((_vbox__IUSBDeviceFilter_USCOREsetNameResponse *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_setNameResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetName:
		return ((_vbox__IUSBDeviceFilter_USCOREsetName *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_setName", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetNameResponse:
		return ((_vbox__IUSBDeviceFilter_USCOREgetNameResponse *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_getNameResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetName:
		return ((_vbox__IUSBDeviceFilter_USCOREgetName *)ptr)->soap_out(soap, "vbox:IUSBDeviceFilter_getName", id, NULL);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetRemoteResponse:
		return ((_vbox__IUSBDevice_USCOREgetRemoteResponse *)ptr)->soap_out(soap, "vbox:IUSBDevice_getRemoteResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetRemote:
		return ((_vbox__IUSBDevice_USCOREgetRemote *)ptr)->soap_out(soap, "vbox:IUSBDevice_getRemote", id, NULL);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetPortVersionResponse:
		return ((_vbox__IUSBDevice_USCOREgetPortVersionResponse *)ptr)->soap_out(soap, "vbox:IUSBDevice_getPortVersionResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetPortVersion:
		return ((_vbox__IUSBDevice_USCOREgetPortVersion *)ptr)->soap_out(soap, "vbox:IUSBDevice_getPortVersion", id, NULL);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetVersionResponse:
		return ((_vbox__IUSBDevice_USCOREgetVersionResponse *)ptr)->soap_out(soap, "vbox:IUSBDevice_getVersionResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetVersion:
		return ((_vbox__IUSBDevice_USCOREgetVersion *)ptr)->soap_out(soap, "vbox:IUSBDevice_getVersion", id, NULL);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetPortResponse:
		return ((_vbox__IUSBDevice_USCOREgetPortResponse *)ptr)->soap_out(soap, "vbox:IUSBDevice_getPortResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetPort:
		return ((_vbox__IUSBDevice_USCOREgetPort *)ptr)->soap_out(soap, "vbox:IUSBDevice_getPort", id, NULL);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetAddressResponse:
		return ((_vbox__IUSBDevice_USCOREgetAddressResponse *)ptr)->soap_out(soap, "vbox:IUSBDevice_getAddressResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetAddress:
		return ((_vbox__IUSBDevice_USCOREgetAddress *)ptr)->soap_out(soap, "vbox:IUSBDevice_getAddress", id, NULL);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetSerialNumberResponse:
		return ((_vbox__IUSBDevice_USCOREgetSerialNumberResponse *)ptr)->soap_out(soap, "vbox:IUSBDevice_getSerialNumberResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetSerialNumber:
		return ((_vbox__IUSBDevice_USCOREgetSerialNumber *)ptr)->soap_out(soap, "vbox:IUSBDevice_getSerialNumber", id, NULL);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetProductResponse:
		return ((_vbox__IUSBDevice_USCOREgetProductResponse *)ptr)->soap_out(soap, "vbox:IUSBDevice_getProductResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetProduct:
		return ((_vbox__IUSBDevice_USCOREgetProduct *)ptr)->soap_out(soap, "vbox:IUSBDevice_getProduct", id, NULL);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetManufacturerResponse:
		return ((_vbox__IUSBDevice_USCOREgetManufacturerResponse *)ptr)->soap_out(soap, "vbox:IUSBDevice_getManufacturerResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetManufacturer:
		return ((_vbox__IUSBDevice_USCOREgetManufacturer *)ptr)->soap_out(soap, "vbox:IUSBDevice_getManufacturer", id, NULL);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetRevisionResponse:
		return ((_vbox__IUSBDevice_USCOREgetRevisionResponse *)ptr)->soap_out(soap, "vbox:IUSBDevice_getRevisionResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetRevision:
		return ((_vbox__IUSBDevice_USCOREgetRevision *)ptr)->soap_out(soap, "vbox:IUSBDevice_getRevision", id, NULL);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetProductIdResponse:
		return ((_vbox__IUSBDevice_USCOREgetProductIdResponse *)ptr)->soap_out(soap, "vbox:IUSBDevice_getProductIdResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetProductId:
		return ((_vbox__IUSBDevice_USCOREgetProductId *)ptr)->soap_out(soap, "vbox:IUSBDevice_getProductId", id, NULL);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetVendorIdResponse:
		return ((_vbox__IUSBDevice_USCOREgetVendorIdResponse *)ptr)->soap_out(soap, "vbox:IUSBDevice_getVendorIdResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetVendorId:
		return ((_vbox__IUSBDevice_USCOREgetVendorId *)ptr)->soap_out(soap, "vbox:IUSBDevice_getVendorId", id, NULL);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetIdResponse:
		return ((_vbox__IUSBDevice_USCOREgetIdResponse *)ptr)->soap_out(soap, "vbox:IUSBDevice_getIdResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetId:
		return ((_vbox__IUSBDevice_USCOREgetId *)ptr)->soap_out(soap, "vbox:IUSBDevice_getId", id, NULL);
	case SOAP_TYPE__vbox__IUSBController_USCOREremoveDeviceFilterResponse:
		return ((_vbox__IUSBController_USCOREremoveDeviceFilterResponse *)ptr)->soap_out(soap, "vbox:IUSBController_removeDeviceFilterResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBController_USCOREremoveDeviceFilter:
		return ((_vbox__IUSBController_USCOREremoveDeviceFilter *)ptr)->soap_out(soap, "vbox:IUSBController_removeDeviceFilter", id, NULL);
	case SOAP_TYPE__vbox__IUSBController_USCOREinsertDeviceFilterResponse:
		return ((_vbox__IUSBController_USCOREinsertDeviceFilterResponse *)ptr)->soap_out(soap, "vbox:IUSBController_insertDeviceFilterResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBController_USCOREinsertDeviceFilter:
		return ((_vbox__IUSBController_USCOREinsertDeviceFilter *)ptr)->soap_out(soap, "vbox:IUSBController_insertDeviceFilter", id, NULL);
	case SOAP_TYPE__vbox__IUSBController_USCOREcreateDeviceFilterResponse:
		return ((_vbox__IUSBController_USCOREcreateDeviceFilterResponse *)ptr)->soap_out(soap, "vbox:IUSBController_createDeviceFilterResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBController_USCOREcreateDeviceFilter:
		return ((_vbox__IUSBController_USCOREcreateDeviceFilter *)ptr)->soap_out(soap, "vbox:IUSBController_createDeviceFilter", id, NULL);
	case SOAP_TYPE__vbox__IUSBController_USCOREgetDeviceFiltersResponse:
		return ((_vbox__IUSBController_USCOREgetDeviceFiltersResponse *)ptr)->soap_out(soap, "vbox:IUSBController_getDeviceFiltersResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBController_USCOREgetDeviceFilters:
		return ((_vbox__IUSBController_USCOREgetDeviceFilters *)ptr)->soap_out(soap, "vbox:IUSBController_getDeviceFilters", id, NULL);
	case SOAP_TYPE__vbox__IUSBController_USCOREgetUSBStandardResponse:
		return ((_vbox__IUSBController_USCOREgetUSBStandardResponse *)ptr)->soap_out(soap, "vbox:IUSBController_getUSBStandardResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBController_USCOREgetUSBStandard:
		return ((_vbox__IUSBController_USCOREgetUSBStandard *)ptr)->soap_out(soap, "vbox:IUSBController_getUSBStandard", id, NULL);
	case SOAP_TYPE__vbox__IUSBController_USCOREsetEnabledEhciResponse:
		return ((_vbox__IUSBController_USCOREsetEnabledEhciResponse *)ptr)->soap_out(soap, "vbox:IUSBController_setEnabledEhciResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBController_USCOREsetEnabledEhci:
		return ((_vbox__IUSBController_USCOREsetEnabledEhci *)ptr)->soap_out(soap, "vbox:IUSBController_setEnabledEhci", id, NULL);
	case SOAP_TYPE__vbox__IUSBController_USCOREgetEnabledEhciResponse:
		return ((_vbox__IUSBController_USCOREgetEnabledEhciResponse *)ptr)->soap_out(soap, "vbox:IUSBController_getEnabledEhciResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBController_USCOREgetEnabledEhci:
		return ((_vbox__IUSBController_USCOREgetEnabledEhci *)ptr)->soap_out(soap, "vbox:IUSBController_getEnabledEhci", id, NULL);
	case SOAP_TYPE__vbox__IUSBController_USCOREsetEnabledResponse:
		return ((_vbox__IUSBController_USCOREsetEnabledResponse *)ptr)->soap_out(soap, "vbox:IUSBController_setEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBController_USCOREsetEnabled:
		return ((_vbox__IUSBController_USCOREsetEnabled *)ptr)->soap_out(soap, "vbox:IUSBController_setEnabled", id, NULL);
	case SOAP_TYPE__vbox__IUSBController_USCOREgetEnabledResponse:
		return ((_vbox__IUSBController_USCOREgetEnabledResponse *)ptr)->soap_out(soap, "vbox:IUSBController_getEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IUSBController_USCOREgetEnabled:
		return ((_vbox__IUSBController_USCOREgetEnabled *)ptr)->soap_out(soap, "vbox:IUSBController_getEnabled", id, NULL);
	case SOAP_TYPE__vbox__IParallelPort_USCOREsetPathResponse:
		return ((_vbox__IParallelPort_USCOREsetPathResponse *)ptr)->soap_out(soap, "vbox:IParallelPort_setPathResponse", id, NULL);
	case SOAP_TYPE__vbox__IParallelPort_USCOREsetPath:
		return ((_vbox__IParallelPort_USCOREsetPath *)ptr)->soap_out(soap, "vbox:IParallelPort_setPath", id, NULL);
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetPathResponse:
		return ((_vbox__IParallelPort_USCOREgetPathResponse *)ptr)->soap_out(soap, "vbox:IParallelPort_getPathResponse", id, NULL);
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetPath:
		return ((_vbox__IParallelPort_USCOREgetPath *)ptr)->soap_out(soap, "vbox:IParallelPort_getPath", id, NULL);
	case SOAP_TYPE__vbox__IParallelPort_USCOREsetIRQResponse:
		return ((_vbox__IParallelPort_USCOREsetIRQResponse *)ptr)->soap_out(soap, "vbox:IParallelPort_setIRQResponse", id, NULL);
	case SOAP_TYPE__vbox__IParallelPort_USCOREsetIRQ:
		return ((_vbox__IParallelPort_USCOREsetIRQ *)ptr)->soap_out(soap, "vbox:IParallelPort_setIRQ", id, NULL);
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetIRQResponse:
		return ((_vbox__IParallelPort_USCOREgetIRQResponse *)ptr)->soap_out(soap, "vbox:IParallelPort_getIRQResponse", id, NULL);
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetIRQ:
		return ((_vbox__IParallelPort_USCOREgetIRQ *)ptr)->soap_out(soap, "vbox:IParallelPort_getIRQ", id, NULL);
	case SOAP_TYPE__vbox__IParallelPort_USCOREsetIOBaseResponse:
		return ((_vbox__IParallelPort_USCOREsetIOBaseResponse *)ptr)->soap_out(soap, "vbox:IParallelPort_setIOBaseResponse", id, NULL);
	case SOAP_TYPE__vbox__IParallelPort_USCOREsetIOBase:
		return ((_vbox__IParallelPort_USCOREsetIOBase *)ptr)->soap_out(soap, "vbox:IParallelPort_setIOBase", id, NULL);
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetIOBaseResponse:
		return ((_vbox__IParallelPort_USCOREgetIOBaseResponse *)ptr)->soap_out(soap, "vbox:IParallelPort_getIOBaseResponse", id, NULL);
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetIOBase:
		return ((_vbox__IParallelPort_USCOREgetIOBase *)ptr)->soap_out(soap, "vbox:IParallelPort_getIOBase", id, NULL);
	case SOAP_TYPE__vbox__IParallelPort_USCOREsetEnabledResponse:
		return ((_vbox__IParallelPort_USCOREsetEnabledResponse *)ptr)->soap_out(soap, "vbox:IParallelPort_setEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IParallelPort_USCOREsetEnabled:
		return ((_vbox__IParallelPort_USCOREsetEnabled *)ptr)->soap_out(soap, "vbox:IParallelPort_setEnabled", id, NULL);
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetEnabledResponse:
		return ((_vbox__IParallelPort_USCOREgetEnabledResponse *)ptr)->soap_out(soap, "vbox:IParallelPort_getEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetEnabled:
		return ((_vbox__IParallelPort_USCOREgetEnabled *)ptr)->soap_out(soap, "vbox:IParallelPort_getEnabled", id, NULL);
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetSlotResponse:
		return ((_vbox__IParallelPort_USCOREgetSlotResponse *)ptr)->soap_out(soap, "vbox:IParallelPort_getSlotResponse", id, NULL);
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetSlot:
		return ((_vbox__IParallelPort_USCOREgetSlot *)ptr)->soap_out(soap, "vbox:IParallelPort_getSlot", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetPathResponse:
		return ((_vbox__ISerialPort_USCOREsetPathResponse *)ptr)->soap_out(soap, "vbox:ISerialPort_setPathResponse", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetPath:
		return ((_vbox__ISerialPort_USCOREsetPath *)ptr)->soap_out(soap, "vbox:ISerialPort_setPath", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetPathResponse:
		return ((_vbox__ISerialPort_USCOREgetPathResponse *)ptr)->soap_out(soap, "vbox:ISerialPort_getPathResponse", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetPath:
		return ((_vbox__ISerialPort_USCOREgetPath *)ptr)->soap_out(soap, "vbox:ISerialPort_getPath", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetServerResponse:
		return ((_vbox__ISerialPort_USCOREsetServerResponse *)ptr)->soap_out(soap, "vbox:ISerialPort_setServerResponse", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetServer:
		return ((_vbox__ISerialPort_USCOREsetServer *)ptr)->soap_out(soap, "vbox:ISerialPort_setServer", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetServerResponse:
		return ((_vbox__ISerialPort_USCOREgetServerResponse *)ptr)->soap_out(soap, "vbox:ISerialPort_getServerResponse", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetServer:
		return ((_vbox__ISerialPort_USCOREgetServer *)ptr)->soap_out(soap, "vbox:ISerialPort_getServer", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetHostModeResponse:
		return ((_vbox__ISerialPort_USCOREsetHostModeResponse *)ptr)->soap_out(soap, "vbox:ISerialPort_setHostModeResponse", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetHostMode:
		return ((_vbox__ISerialPort_USCOREsetHostMode *)ptr)->soap_out(soap, "vbox:ISerialPort_setHostMode", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetHostModeResponse:
		return ((_vbox__ISerialPort_USCOREgetHostModeResponse *)ptr)->soap_out(soap, "vbox:ISerialPort_getHostModeResponse", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetHostMode:
		return ((_vbox__ISerialPort_USCOREgetHostMode *)ptr)->soap_out(soap, "vbox:ISerialPort_getHostMode", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetIRQResponse:
		return ((_vbox__ISerialPort_USCOREsetIRQResponse *)ptr)->soap_out(soap, "vbox:ISerialPort_setIRQResponse", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetIRQ:
		return ((_vbox__ISerialPort_USCOREsetIRQ *)ptr)->soap_out(soap, "vbox:ISerialPort_setIRQ", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetIRQResponse:
		return ((_vbox__ISerialPort_USCOREgetIRQResponse *)ptr)->soap_out(soap, "vbox:ISerialPort_getIRQResponse", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetIRQ:
		return ((_vbox__ISerialPort_USCOREgetIRQ *)ptr)->soap_out(soap, "vbox:ISerialPort_getIRQ", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetIOBaseResponse:
		return ((_vbox__ISerialPort_USCOREsetIOBaseResponse *)ptr)->soap_out(soap, "vbox:ISerialPort_setIOBaseResponse", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetIOBase:
		return ((_vbox__ISerialPort_USCOREsetIOBase *)ptr)->soap_out(soap, "vbox:ISerialPort_setIOBase", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetIOBaseResponse:
		return ((_vbox__ISerialPort_USCOREgetIOBaseResponse *)ptr)->soap_out(soap, "vbox:ISerialPort_getIOBaseResponse", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetIOBase:
		return ((_vbox__ISerialPort_USCOREgetIOBase *)ptr)->soap_out(soap, "vbox:ISerialPort_getIOBase", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetEnabledResponse:
		return ((_vbox__ISerialPort_USCOREsetEnabledResponse *)ptr)->soap_out(soap, "vbox:ISerialPort_setEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetEnabled:
		return ((_vbox__ISerialPort_USCOREsetEnabled *)ptr)->soap_out(soap, "vbox:ISerialPort_setEnabled", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetEnabledResponse:
		return ((_vbox__ISerialPort_USCOREgetEnabledResponse *)ptr)->soap_out(soap, "vbox:ISerialPort_getEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetEnabled:
		return ((_vbox__ISerialPort_USCOREgetEnabled *)ptr)->soap_out(soap, "vbox:ISerialPort_getEnabled", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetSlotResponse:
		return ((_vbox__ISerialPort_USCOREgetSlotResponse *)ptr)->soap_out(soap, "vbox:ISerialPort_getSlotResponse", id, NULL);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetSlot:
		return ((_vbox__ISerialPort_USCOREgetSlot *)ptr)->soap_out(soap, "vbox:ISerialPort_getSlot", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREdetachResponse:
		return ((_vbox__INetworkAdapter_USCOREdetachResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_detachResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREdetach:
		return ((_vbox__INetworkAdapter_USCOREdetach *)ptr)->soap_out(soap, "vbox:INetworkAdapter_detach", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREattachToHostOnlyInterfaceResponse:
		return ((_vbox__INetworkAdapter_USCOREattachToHostOnlyInterfaceResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_attachToHostOnlyInterfaceResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREattachToHostOnlyInterface:
		return ((_vbox__INetworkAdapter_USCOREattachToHostOnlyInterface *)ptr)->soap_out(soap, "vbox:INetworkAdapter_attachToHostOnlyInterface", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREattachToInternalNetworkResponse:
		return ((_vbox__INetworkAdapter_USCOREattachToInternalNetworkResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_attachToInternalNetworkResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREattachToInternalNetwork:
		return ((_vbox__INetworkAdapter_USCOREattachToInternalNetwork *)ptr)->soap_out(soap, "vbox:INetworkAdapter_attachToInternalNetwork", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREattachToBridgedInterfaceResponse:
		return ((_vbox__INetworkAdapter_USCOREattachToBridgedInterfaceResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_attachToBridgedInterfaceResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREattachToBridgedInterface:
		return ((_vbox__INetworkAdapter_USCOREattachToBridgedInterface *)ptr)->soap_out(soap, "vbox:INetworkAdapter_attachToBridgedInterface", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREattachToNATResponse:
		return ((_vbox__INetworkAdapter_USCOREattachToNATResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_attachToNATResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREattachToNAT:
		return ((_vbox__INetworkAdapter_USCOREattachToNAT *)ptr)->soap_out(soap, "vbox:INetworkAdapter_attachToNAT", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetTraceFileResponse:
		return ((_vbox__INetworkAdapter_USCOREsetTraceFileResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_setTraceFileResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetTraceFile:
		return ((_vbox__INetworkAdapter_USCOREsetTraceFile *)ptr)->soap_out(soap, "vbox:INetworkAdapter_setTraceFile", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetTraceFileResponse:
		return ((_vbox__INetworkAdapter_USCOREgetTraceFileResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_getTraceFileResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetTraceFile:
		return ((_vbox__INetworkAdapter_USCOREgetTraceFile *)ptr)->soap_out(soap, "vbox:INetworkAdapter_getTraceFile", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetTraceEnabledResponse:
		return ((_vbox__INetworkAdapter_USCOREsetTraceEnabledResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_setTraceEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetTraceEnabled:
		return ((_vbox__INetworkAdapter_USCOREsetTraceEnabled *)ptr)->soap_out(soap, "vbox:INetworkAdapter_setTraceEnabled", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetTraceEnabledResponse:
		return ((_vbox__INetworkAdapter_USCOREgetTraceEnabledResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_getTraceEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetTraceEnabled:
		return ((_vbox__INetworkAdapter_USCOREgetTraceEnabled *)ptr)->soap_out(soap, "vbox:INetworkAdapter_getTraceEnabled", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetLineSpeedResponse:
		return ((_vbox__INetworkAdapter_USCOREsetLineSpeedResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_setLineSpeedResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetLineSpeed:
		return ((_vbox__INetworkAdapter_USCOREsetLineSpeed *)ptr)->soap_out(soap, "vbox:INetworkAdapter_setLineSpeed", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetLineSpeedResponse:
		return ((_vbox__INetworkAdapter_USCOREgetLineSpeedResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_getLineSpeedResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetLineSpeed:
		return ((_vbox__INetworkAdapter_USCOREgetLineSpeed *)ptr)->soap_out(soap, "vbox:INetworkAdapter_getLineSpeed", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetCableConnectedResponse:
		return ((_vbox__INetworkAdapter_USCOREsetCableConnectedResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_setCableConnectedResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetCableConnected:
		return ((_vbox__INetworkAdapter_USCOREsetCableConnected *)ptr)->soap_out(soap, "vbox:INetworkAdapter_setCableConnected", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetCableConnectedResponse:
		return ((_vbox__INetworkAdapter_USCOREgetCableConnectedResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_getCableConnectedResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetCableConnected:
		return ((_vbox__INetworkAdapter_USCOREgetCableConnected *)ptr)->soap_out(soap, "vbox:INetworkAdapter_getCableConnected", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetNATNetworkResponse:
		return ((_vbox__INetworkAdapter_USCOREsetNATNetworkResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_setNATNetworkResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetNATNetwork:
		return ((_vbox__INetworkAdapter_USCOREsetNATNetwork *)ptr)->soap_out(soap, "vbox:INetworkAdapter_setNATNetwork", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetNATNetworkResponse:
		return ((_vbox__INetworkAdapter_USCOREgetNATNetworkResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_getNATNetworkResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetNATNetwork:
		return ((_vbox__INetworkAdapter_USCOREgetNATNetwork *)ptr)->soap_out(soap, "vbox:INetworkAdapter_getNATNetwork", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetInternalNetworkResponse:
		return ((_vbox__INetworkAdapter_USCOREsetInternalNetworkResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_setInternalNetworkResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetInternalNetwork:
		return ((_vbox__INetworkAdapter_USCOREsetInternalNetwork *)ptr)->soap_out(soap, "vbox:INetworkAdapter_setInternalNetwork", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetInternalNetworkResponse:
		return ((_vbox__INetworkAdapter_USCOREgetInternalNetworkResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_getInternalNetworkResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetInternalNetwork:
		return ((_vbox__INetworkAdapter_USCOREgetInternalNetwork *)ptr)->soap_out(soap, "vbox:INetworkAdapter_getInternalNetwork", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetHostInterfaceResponse:
		return ((_vbox__INetworkAdapter_USCOREsetHostInterfaceResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_setHostInterfaceResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetHostInterface:
		return ((_vbox__INetworkAdapter_USCOREsetHostInterface *)ptr)->soap_out(soap, "vbox:INetworkAdapter_setHostInterface", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetHostInterfaceResponse:
		return ((_vbox__INetworkAdapter_USCOREgetHostInterfaceResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_getHostInterfaceResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetHostInterface:
		return ((_vbox__INetworkAdapter_USCOREgetHostInterface *)ptr)->soap_out(soap, "vbox:INetworkAdapter_getHostInterface", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetAttachmentTypeResponse:
		return ((_vbox__INetworkAdapter_USCOREgetAttachmentTypeResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_getAttachmentTypeResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetAttachmentType:
		return ((_vbox__INetworkAdapter_USCOREgetAttachmentType *)ptr)->soap_out(soap, "vbox:INetworkAdapter_getAttachmentType", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetMACAddressResponse:
		return ((_vbox__INetworkAdapter_USCOREsetMACAddressResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_setMACAddressResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetMACAddress:
		return ((_vbox__INetworkAdapter_USCOREsetMACAddress *)ptr)->soap_out(soap, "vbox:INetworkAdapter_setMACAddress", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetMACAddressResponse:
		return ((_vbox__INetworkAdapter_USCOREgetMACAddressResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_getMACAddressResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetMACAddress:
		return ((_vbox__INetworkAdapter_USCOREgetMACAddress *)ptr)->soap_out(soap, "vbox:INetworkAdapter_getMACAddress", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetEnabledResponse:
		return ((_vbox__INetworkAdapter_USCOREsetEnabledResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_setEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetEnabled:
		return ((_vbox__INetworkAdapter_USCOREsetEnabled *)ptr)->soap_out(soap, "vbox:INetworkAdapter_setEnabled", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetEnabledResponse:
		return ((_vbox__INetworkAdapter_USCOREgetEnabledResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_getEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetEnabled:
		return ((_vbox__INetworkAdapter_USCOREgetEnabled *)ptr)->soap_out(soap, "vbox:INetworkAdapter_getEnabled", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetSlotResponse:
		return ((_vbox__INetworkAdapter_USCOREgetSlotResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_getSlotResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetSlot:
		return ((_vbox__INetworkAdapter_USCOREgetSlot *)ptr)->soap_out(soap, "vbox:INetworkAdapter_getSlot", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetAdapterTypeResponse:
		return ((_vbox__INetworkAdapter_USCOREsetAdapterTypeResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_setAdapterTypeResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetAdapterType:
		return ((_vbox__INetworkAdapter_USCOREsetAdapterType *)ptr)->soap_out(soap, "vbox:INetworkAdapter_setAdapterType", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetAdapterTypeResponse:
		return ((_vbox__INetworkAdapter_USCOREgetAdapterTypeResponse *)ptr)->soap_out(soap, "vbox:INetworkAdapter_getAdapterTypeResponse", id, NULL);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetAdapterType:
		return ((_vbox__INetworkAdapter_USCOREgetAdapterType *)ptr)->soap_out(soap, "vbox:INetworkAdapter_getAdapterType", id, NULL);
	case SOAP_TYPE__vbox__IDisplay_USCOREupdateCompletedResponse:
		return ((_vbox__IDisplay_USCOREupdateCompletedResponse *)ptr)->soap_out(soap, "vbox:IDisplay_updateCompletedResponse", id, NULL);
	case SOAP_TYPE__vbox__IDisplay_USCOREupdateCompleted:
		return ((_vbox__IDisplay_USCOREupdateCompleted *)ptr)->soap_out(soap, "vbox:IDisplay_updateCompleted", id, NULL);
	case SOAP_TYPE__vbox__IDisplay_USCOREresizeCompletedResponse:
		return ((_vbox__IDisplay_USCOREresizeCompletedResponse *)ptr)->soap_out(soap, "vbox:IDisplay_resizeCompletedResponse", id, NULL);
	case SOAP_TYPE__vbox__IDisplay_USCOREresizeCompleted:
		return ((_vbox__IDisplay_USCOREresizeCompleted *)ptr)->soap_out(soap, "vbox:IDisplay_resizeCompleted", id, NULL);
	case SOAP_TYPE__vbox__IDisplay_USCOREinvalidateAndUpdateResponse:
		return ((_vbox__IDisplay_USCOREinvalidateAndUpdateResponse *)ptr)->soap_out(soap, "vbox:IDisplay_invalidateAndUpdateResponse", id, NULL);
	case SOAP_TYPE__vbox__IDisplay_USCOREinvalidateAndUpdate:
		return ((_vbox__IDisplay_USCOREinvalidateAndUpdate *)ptr)->soap_out(soap, "vbox:IDisplay_invalidateAndUpdate", id, NULL);
	case SOAP_TYPE__vbox__IDisplay_USCOREtakeScreenShotSlowResponse:
		return ((_vbox__IDisplay_USCOREtakeScreenShotSlowResponse *)ptr)->soap_out(soap, "vbox:IDisplay_takeScreenShotSlowResponse", id, NULL);
	case SOAP_TYPE__vbox__IDisplay_USCOREtakeScreenShotSlow:
		return ((_vbox__IDisplay_USCOREtakeScreenShotSlow *)ptr)->soap_out(soap, "vbox:IDisplay_takeScreenShotSlow", id, NULL);
	case SOAP_TYPE__vbox__IDisplay_USCOREsetSeamlessModeResponse:
		return ((_vbox__IDisplay_USCOREsetSeamlessModeResponse *)ptr)->soap_out(soap, "vbox:IDisplay_setSeamlessModeResponse", id, NULL);
	case SOAP_TYPE__vbox__IDisplay_USCOREsetSeamlessMode:
		return ((_vbox__IDisplay_USCOREsetSeamlessMode *)ptr)->soap_out(soap, "vbox:IDisplay_setSeamlessMode", id, NULL);
	case SOAP_TYPE__vbox__IDisplay_USCOREsetVideoModeHintResponse:
		return ((_vbox__IDisplay_USCOREsetVideoModeHintResponse *)ptr)->soap_out(soap, "vbox:IDisplay_setVideoModeHintResponse", id, NULL);
	case SOAP_TYPE__vbox__IDisplay_USCOREsetVideoModeHint:
		return ((_vbox__IDisplay_USCOREsetVideoModeHint *)ptr)->soap_out(soap, "vbox:IDisplay_setVideoModeHint", id, NULL);
	case SOAP_TYPE__vbox__IDisplay_USCOREgetBitsPerPixelResponse:
		return ((_vbox__IDisplay_USCOREgetBitsPerPixelResponse *)ptr)->soap_out(soap, "vbox:IDisplay_getBitsPerPixelResponse", id, NULL);
	case SOAP_TYPE__vbox__IDisplay_USCOREgetBitsPerPixel:
		return ((_vbox__IDisplay_USCOREgetBitsPerPixel *)ptr)->soap_out(soap, "vbox:IDisplay_getBitsPerPixel", id, NULL);
	case SOAP_TYPE__vbox__IDisplay_USCOREgetHeightResponse:
		return ((_vbox__IDisplay_USCOREgetHeightResponse *)ptr)->soap_out(soap, "vbox:IDisplay_getHeightResponse", id, NULL);
	case SOAP_TYPE__vbox__IDisplay_USCOREgetHeight:
		return ((_vbox__IDisplay_USCOREgetHeight *)ptr)->soap_out(soap, "vbox:IDisplay_getHeight", id, NULL);
	case SOAP_TYPE__vbox__IDisplay_USCOREgetWidthResponse:
		return ((_vbox__IDisplay_USCOREgetWidthResponse *)ptr)->soap_out(soap, "vbox:IDisplay_getWidthResponse", id, NULL);
	case SOAP_TYPE__vbox__IDisplay_USCOREgetWidth:
		return ((_vbox__IDisplay_USCOREgetWidth *)ptr)->soap_out(soap, "vbox:IDisplay_getWidth", id, NULL);
	case SOAP_TYPE__vbox__IMouse_USCOREputMouseEventAbsoluteResponse:
		return ((_vbox__IMouse_USCOREputMouseEventAbsoluteResponse *)ptr)->soap_out(soap, "vbox:IMouse_putMouseEventAbsoluteResponse", id, NULL);
	case SOAP_TYPE__vbox__IMouse_USCOREputMouseEventAbsolute:
		return ((_vbox__IMouse_USCOREputMouseEventAbsolute *)ptr)->soap_out(soap, "vbox:IMouse_putMouseEventAbsolute", id, NULL);
	case SOAP_TYPE__vbox__IMouse_USCOREputMouseEventResponse:
		return ((_vbox__IMouse_USCOREputMouseEventResponse *)ptr)->soap_out(soap, "vbox:IMouse_putMouseEventResponse", id, NULL);
	case SOAP_TYPE__vbox__IMouse_USCOREputMouseEvent:
		return ((_vbox__IMouse_USCOREputMouseEvent *)ptr)->soap_out(soap, "vbox:IMouse_putMouseEvent", id, NULL);
	case SOAP_TYPE__vbox__IMouse_USCOREgetAbsoluteSupportedResponse:
		return ((_vbox__IMouse_USCOREgetAbsoluteSupportedResponse *)ptr)->soap_out(soap, "vbox:IMouse_getAbsoluteSupportedResponse", id, NULL);
	case SOAP_TYPE__vbox__IMouse_USCOREgetAbsoluteSupported:
		return ((_vbox__IMouse_USCOREgetAbsoluteSupported *)ptr)->soap_out(soap, "vbox:IMouse_getAbsoluteSupported", id, NULL);
	case SOAP_TYPE__vbox__IKeyboard_USCOREputCADResponse:
		return ((_vbox__IKeyboard_USCOREputCADResponse *)ptr)->soap_out(soap, "vbox:IKeyboard_putCADResponse", id, NULL);
	case SOAP_TYPE__vbox__IKeyboard_USCOREputCAD:
		return ((_vbox__IKeyboard_USCOREputCAD *)ptr)->soap_out(soap, "vbox:IKeyboard_putCAD", id, NULL);
	case SOAP_TYPE__vbox__IKeyboard_USCOREputScancodesResponse:
		return ((_vbox__IKeyboard_USCOREputScancodesResponse *)ptr)->soap_out(soap, "vbox:IKeyboard_putScancodesResponse", id, NULL);
	case SOAP_TYPE__vbox__IKeyboard_USCOREputScancodes:
		return ((_vbox__IKeyboard_USCOREputScancodes *)ptr)->soap_out(soap, "vbox:IKeyboard_putScancodes", id, NULL);
	case SOAP_TYPE__vbox__IKeyboard_USCOREputScancodeResponse:
		return ((_vbox__IKeyboard_USCOREputScancodeResponse *)ptr)->soap_out(soap, "vbox:IKeyboard_putScancodeResponse", id, NULL);
	case SOAP_TYPE__vbox__IKeyboard_USCOREputScancode:
		return ((_vbox__IKeyboard_USCOREputScancode *)ptr)->soap_out(soap, "vbox:IKeyboard_putScancode", id, NULL);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREgetHostDriveResponse:
		return ((_vbox__IFloppyDrive_USCOREgetHostDriveResponse *)ptr)->soap_out(soap, "vbox:IFloppyDrive_getHostDriveResponse", id, NULL);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREgetHostDrive:
		return ((_vbox__IFloppyDrive_USCOREgetHostDrive *)ptr)->soap_out(soap, "vbox:IFloppyDrive_getHostDrive", id, NULL);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREgetImageResponse:
		return ((_vbox__IFloppyDrive_USCOREgetImageResponse *)ptr)->soap_out(soap, "vbox:IFloppyDrive_getImageResponse", id, NULL);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREgetImage:
		return ((_vbox__IFloppyDrive_USCOREgetImage *)ptr)->soap_out(soap, "vbox:IFloppyDrive_getImage", id, NULL);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREunmountResponse:
		return ((_vbox__IFloppyDrive_USCOREunmountResponse *)ptr)->soap_out(soap, "vbox:IFloppyDrive_unmountResponse", id, NULL);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREunmount:
		return ((_vbox__IFloppyDrive_USCOREunmount *)ptr)->soap_out(soap, "vbox:IFloppyDrive_unmount", id, NULL);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREcaptureHostDriveResponse:
		return ((_vbox__IFloppyDrive_USCOREcaptureHostDriveResponse *)ptr)->soap_out(soap, "vbox:IFloppyDrive_captureHostDriveResponse", id, NULL);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREcaptureHostDrive:
		return ((_vbox__IFloppyDrive_USCOREcaptureHostDrive *)ptr)->soap_out(soap, "vbox:IFloppyDrive_captureHostDrive", id, NULL);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREmountImageResponse:
		return ((_vbox__IFloppyDrive_USCOREmountImageResponse *)ptr)->soap_out(soap, "vbox:IFloppyDrive_mountImageResponse", id, NULL);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREmountImage:
		return ((_vbox__IFloppyDrive_USCOREmountImage *)ptr)->soap_out(soap, "vbox:IFloppyDrive_mountImage", id, NULL);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREgetStateResponse:
		return ((_vbox__IFloppyDrive_USCOREgetStateResponse *)ptr)->soap_out(soap, "vbox:IFloppyDrive_getStateResponse", id, NULL);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREgetState:
		return ((_vbox__IFloppyDrive_USCOREgetState *)ptr)->soap_out(soap, "vbox:IFloppyDrive_getState", id, NULL);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREsetEnabledResponse:
		return ((_vbox__IFloppyDrive_USCOREsetEnabledResponse *)ptr)->soap_out(soap, "vbox:IFloppyDrive_setEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREsetEnabled:
		return ((_vbox__IFloppyDrive_USCOREsetEnabled *)ptr)->soap_out(soap, "vbox:IFloppyDrive_setEnabled", id, NULL);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREgetEnabledResponse:
		return ((_vbox__IFloppyDrive_USCOREgetEnabledResponse *)ptr)->soap_out(soap, "vbox:IFloppyDrive_getEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREgetEnabled:
		return ((_vbox__IFloppyDrive_USCOREgetEnabled *)ptr)->soap_out(soap, "vbox:IFloppyDrive_getEnabled", id, NULL);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREgetHostDriveResponse:
		return ((_vbox__IDVDDrive_USCOREgetHostDriveResponse *)ptr)->soap_out(soap, "vbox:IDVDDrive_getHostDriveResponse", id, NULL);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREgetHostDrive:
		return ((_vbox__IDVDDrive_USCOREgetHostDrive *)ptr)->soap_out(soap, "vbox:IDVDDrive_getHostDrive", id, NULL);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREgetImageResponse:
		return ((_vbox__IDVDDrive_USCOREgetImageResponse *)ptr)->soap_out(soap, "vbox:IDVDDrive_getImageResponse", id, NULL);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREgetImage:
		return ((_vbox__IDVDDrive_USCOREgetImage *)ptr)->soap_out(soap, "vbox:IDVDDrive_getImage", id, NULL);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREunmountResponse:
		return ((_vbox__IDVDDrive_USCOREunmountResponse *)ptr)->soap_out(soap, "vbox:IDVDDrive_unmountResponse", id, NULL);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREunmount:
		return ((_vbox__IDVDDrive_USCOREunmount *)ptr)->soap_out(soap, "vbox:IDVDDrive_unmount", id, NULL);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREcaptureHostDriveResponse:
		return ((_vbox__IDVDDrive_USCOREcaptureHostDriveResponse *)ptr)->soap_out(soap, "vbox:IDVDDrive_captureHostDriveResponse", id, NULL);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREcaptureHostDrive:
		return ((_vbox__IDVDDrive_USCOREcaptureHostDrive *)ptr)->soap_out(soap, "vbox:IDVDDrive_captureHostDrive", id, NULL);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREmountImageResponse:
		return ((_vbox__IDVDDrive_USCOREmountImageResponse *)ptr)->soap_out(soap, "vbox:IDVDDrive_mountImageResponse", id, NULL);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREmountImage:
		return ((_vbox__IDVDDrive_USCOREmountImage *)ptr)->soap_out(soap, "vbox:IDVDDrive_mountImage", id, NULL);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREsetPassthroughResponse:
		return ((_vbox__IDVDDrive_USCOREsetPassthroughResponse *)ptr)->soap_out(soap, "vbox:IDVDDrive_setPassthroughResponse", id, NULL);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREsetPassthrough:
		return ((_vbox__IDVDDrive_USCOREsetPassthrough *)ptr)->soap_out(soap, "vbox:IDVDDrive_setPassthrough", id, NULL);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREgetPassthroughResponse:
		return ((_vbox__IDVDDrive_USCOREgetPassthroughResponse *)ptr)->soap_out(soap, "vbox:IDVDDrive_getPassthroughResponse", id, NULL);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREgetPassthrough:
		return ((_vbox__IDVDDrive_USCOREgetPassthrough *)ptr)->soap_out(soap, "vbox:IDVDDrive_getPassthrough", id, NULL);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREgetStateResponse:
		return ((_vbox__IDVDDrive_USCOREgetStateResponse *)ptr)->soap_out(soap, "vbox:IDVDDrive_getStateResponse", id, NULL);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREgetState:
		return ((_vbox__IDVDDrive_USCOREgetState *)ptr)->soap_out(soap, "vbox:IDVDDrive_getState", id, NULL);
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREdescribePropertiesResponse:
		return ((_vbox__IHardDiskFormat_USCOREdescribePropertiesResponse *)ptr)->soap_out(soap, "vbox:IHardDiskFormat_describePropertiesResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREdescribeProperties:
		return ((_vbox__IHardDiskFormat_USCOREdescribeProperties *)ptr)->soap_out(soap, "vbox:IHardDiskFormat_describeProperties", id, NULL);
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREgetCapabilitiesResponse:
		return ((_vbox__IHardDiskFormat_USCOREgetCapabilitiesResponse *)ptr)->soap_out(soap, "vbox:IHardDiskFormat_getCapabilitiesResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREgetCapabilities:
		return ((_vbox__IHardDiskFormat_USCOREgetCapabilities *)ptr)->soap_out(soap, "vbox:IHardDiskFormat_getCapabilities", id, NULL);
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREgetFileExtensionsResponse:
		return ((_vbox__IHardDiskFormat_USCOREgetFileExtensionsResponse *)ptr)->soap_out(soap, "vbox:IHardDiskFormat_getFileExtensionsResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREgetFileExtensions:
		return ((_vbox__IHardDiskFormat_USCOREgetFileExtensions *)ptr)->soap_out(soap, "vbox:IHardDiskFormat_getFileExtensions", id, NULL);
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREgetNameResponse:
		return ((_vbox__IHardDiskFormat_USCOREgetNameResponse *)ptr)->soap_out(soap, "vbox:IHardDiskFormat_getNameResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREgetName:
		return ((_vbox__IHardDiskFormat_USCOREgetName *)ptr)->soap_out(soap, "vbox:IHardDiskFormat_getName", id, NULL);
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREgetIdResponse:
		return ((_vbox__IHardDiskFormat_USCOREgetIdResponse *)ptr)->soap_out(soap, "vbox:IHardDiskFormat_getIdResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREgetId:
		return ((_vbox__IHardDiskFormat_USCOREgetId *)ptr)->soap_out(soap, "vbox:IHardDiskFormat_getId", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREresetResponse:
		return ((_vbox__IHardDisk_USCOREresetResponse *)ptr)->soap_out(soap, "vbox:IHardDisk_resetResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREreset:
		return ((_vbox__IHardDisk_USCOREreset *)ptr)->soap_out(soap, "vbox:IHardDisk_reset", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREcompactResponse:
		return ((_vbox__IHardDisk_USCOREcompactResponse *)ptr)->soap_out(soap, "vbox:IHardDisk_compactResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREcompact:
		return ((_vbox__IHardDisk_USCOREcompact *)ptr)->soap_out(soap, "vbox:IHardDisk_compact", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREcloneToResponse:
		return ((_vbox__IHardDisk_USCOREcloneToResponse *)ptr)->soap_out(soap, "vbox:IHardDisk_cloneToResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREcloneTo:
		return ((_vbox__IHardDisk_USCOREcloneTo *)ptr)->soap_out(soap, "vbox:IHardDisk_cloneTo", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREmergeToResponse:
		return ((_vbox__IHardDisk_USCOREmergeToResponse *)ptr)->soap_out(soap, "vbox:IHardDisk_mergeToResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREmergeTo:
		return ((_vbox__IHardDisk_USCOREmergeTo *)ptr)->soap_out(soap, "vbox:IHardDisk_mergeTo", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREcreateDiffStorageResponse:
		return ((_vbox__IHardDisk_USCOREcreateDiffStorageResponse *)ptr)->soap_out(soap, "vbox:IHardDisk_createDiffStorageResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREcreateDiffStorage:
		return ((_vbox__IHardDisk_USCOREcreateDiffStorage *)ptr)->soap_out(soap, "vbox:IHardDisk_createDiffStorage", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREdeleteStorageResponse:
		return ((_vbox__IHardDisk_USCOREdeleteStorageResponse *)ptr)->soap_out(soap, "vbox:IHardDisk_deleteStorageResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREdeleteStorage:
		return ((_vbox__IHardDisk_USCOREdeleteStorage *)ptr)->soap_out(soap, "vbox:IHardDisk_deleteStorage", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREcreateBaseStorageResponse:
		return ((_vbox__IHardDisk_USCOREcreateBaseStorageResponse *)ptr)->soap_out(soap, "vbox:IHardDisk_createBaseStorageResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREcreateBaseStorage:
		return ((_vbox__IHardDisk_USCOREcreateBaseStorage *)ptr)->soap_out(soap, "vbox:IHardDisk_createBaseStorage", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREsetPropertiesResponse:
		return ((_vbox__IHardDisk_USCOREsetPropertiesResponse *)ptr)->soap_out(soap, "vbox:IHardDisk_setPropertiesResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREsetProperties:
		return ((_vbox__IHardDisk_USCOREsetProperties *)ptr)->soap_out(soap, "vbox:IHardDisk_setProperties", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetPropertiesResponse:
		return ((_vbox__IHardDisk_USCOREgetPropertiesResponse *)ptr)->soap_out(soap, "vbox:IHardDisk_getPropertiesResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetProperties:
		return ((_vbox__IHardDisk_USCOREgetProperties *)ptr)->soap_out(soap, "vbox:IHardDisk_getProperties", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREsetPropertyResponse:
		return ((_vbox__IHardDisk_USCOREsetPropertyResponse *)ptr)->soap_out(soap, "vbox:IHardDisk_setPropertyResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREsetProperty:
		return ((_vbox__IHardDisk_USCOREsetProperty *)ptr)->soap_out(soap, "vbox:IHardDisk_setProperty", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetPropertyResponse:
		return ((_vbox__IHardDisk_USCOREgetPropertyResponse *)ptr)->soap_out(soap, "vbox:IHardDisk_getPropertyResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetProperty:
		return ((_vbox__IHardDisk_USCOREgetProperty *)ptr)->soap_out(soap, "vbox:IHardDisk_getProperty", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREsetAutoResetResponse:
		return ((_vbox__IHardDisk_USCOREsetAutoResetResponse *)ptr)->soap_out(soap, "vbox:IHardDisk_setAutoResetResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREsetAutoReset:
		return ((_vbox__IHardDisk_USCOREsetAutoReset *)ptr)->soap_out(soap, "vbox:IHardDisk_setAutoReset", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetAutoResetResponse:
		return ((_vbox__IHardDisk_USCOREgetAutoResetResponse *)ptr)->soap_out(soap, "vbox:IHardDisk_getAutoResetResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetAutoReset:
		return ((_vbox__IHardDisk_USCOREgetAutoReset *)ptr)->soap_out(soap, "vbox:IHardDisk_getAutoReset", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetLogicalSizeResponse:
		return ((_vbox__IHardDisk_USCOREgetLogicalSizeResponse *)ptr)->soap_out(soap, "vbox:IHardDisk_getLogicalSizeResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetLogicalSize:
		return ((_vbox__IHardDisk_USCOREgetLogicalSize *)ptr)->soap_out(soap, "vbox:IHardDisk_getLogicalSize", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetReadOnlyResponse:
		return ((_vbox__IHardDisk_USCOREgetReadOnlyResponse *)ptr)->soap_out(soap, "vbox:IHardDisk_getReadOnlyResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetReadOnly:
		return ((_vbox__IHardDisk_USCOREgetReadOnly *)ptr)->soap_out(soap, "vbox:IHardDisk_getReadOnly", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetRootResponse:
		return ((_vbox__IHardDisk_USCOREgetRootResponse *)ptr)->soap_out(soap, "vbox:IHardDisk_getRootResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetRoot:
		return ((_vbox__IHardDisk_USCOREgetRoot *)ptr)->soap_out(soap, "vbox:IHardDisk_getRoot", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetChildrenResponse:
		return ((_vbox__IHardDisk_USCOREgetChildrenResponse *)ptr)->soap_out(soap, "vbox:IHardDisk_getChildrenResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetChildren:
		return ((_vbox__IHardDisk_USCOREgetChildren *)ptr)->soap_out(soap, "vbox:IHardDisk_getChildren", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetParentResponse:
		return ((_vbox__IHardDisk_USCOREgetParentResponse *)ptr)->soap_out(soap, "vbox:IHardDisk_getParentResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetParent:
		return ((_vbox__IHardDisk_USCOREgetParent *)ptr)->soap_out(soap, "vbox:IHardDisk_getParent", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREsetTypeResponse:
		return ((_vbox__IHardDisk_USCOREsetTypeResponse *)ptr)->soap_out(soap, "vbox:IHardDisk_setTypeResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREsetType:
		return ((_vbox__IHardDisk_USCOREsetType *)ptr)->soap_out(soap, "vbox:IHardDisk_setType", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetTypeResponse:
		return ((_vbox__IHardDisk_USCOREgetTypeResponse *)ptr)->soap_out(soap, "vbox:IHardDisk_getTypeResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetType:
		return ((_vbox__IHardDisk_USCOREgetType *)ptr)->soap_out(soap, "vbox:IHardDisk_getType", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetFormatResponse:
		return ((_vbox__IHardDisk_USCOREgetFormatResponse *)ptr)->soap_out(soap, "vbox:IHardDisk_getFormatResponse", id, NULL);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetFormat:
		return ((_vbox__IHardDisk_USCOREgetFormat *)ptr)->soap_out(soap, "vbox:IHardDisk_getFormat", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREcloseResponse:
		return ((_vbox__IMedium_USCOREcloseResponse *)ptr)->soap_out(soap, "vbox:IMedium_closeResponse", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREclose:
		return ((_vbox__IMedium_USCOREclose *)ptr)->soap_out(soap, "vbox:IMedium_close", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREunlockWriteResponse:
		return ((_vbox__IMedium_USCOREunlockWriteResponse *)ptr)->soap_out(soap, "vbox:IMedium_unlockWriteResponse", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREunlockWrite:
		return ((_vbox__IMedium_USCOREunlockWrite *)ptr)->soap_out(soap, "vbox:IMedium_unlockWrite", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCORElockWriteResponse:
		return ((_vbox__IMedium_USCORElockWriteResponse *)ptr)->soap_out(soap, "vbox:IMedium_lockWriteResponse", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCORElockWrite:
		return ((_vbox__IMedium_USCORElockWrite *)ptr)->soap_out(soap, "vbox:IMedium_lockWrite", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREunlockReadResponse:
		return ((_vbox__IMedium_USCOREunlockReadResponse *)ptr)->soap_out(soap, "vbox:IMedium_unlockReadResponse", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREunlockRead:
		return ((_vbox__IMedium_USCOREunlockRead *)ptr)->soap_out(soap, "vbox:IMedium_unlockRead", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCORElockReadResponse:
		return ((_vbox__IMedium_USCORElockReadResponse *)ptr)->soap_out(soap, "vbox:IMedium_lockReadResponse", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCORElockRead:
		return ((_vbox__IMedium_USCORElockRead *)ptr)->soap_out(soap, "vbox:IMedium_lockRead", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREgetSnapshotIdsResponse:
		return ((_vbox__IMedium_USCOREgetSnapshotIdsResponse *)ptr)->soap_out(soap, "vbox:IMedium_getSnapshotIdsResponse", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREgetSnapshotIds:
		return ((_vbox__IMedium_USCOREgetSnapshotIds *)ptr)->soap_out(soap, "vbox:IMedium_getSnapshotIds", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREgetMachineIdsResponse:
		return ((_vbox__IMedium_USCOREgetMachineIdsResponse *)ptr)->soap_out(soap, "vbox:IMedium_getMachineIdsResponse", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREgetMachineIds:
		return ((_vbox__IMedium_USCOREgetMachineIds *)ptr)->soap_out(soap, "vbox:IMedium_getMachineIds", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREgetLastAccessErrorResponse:
		return ((_vbox__IMedium_USCOREgetLastAccessErrorResponse *)ptr)->soap_out(soap, "vbox:IMedium_getLastAccessErrorResponse", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREgetLastAccessError:
		return ((_vbox__IMedium_USCOREgetLastAccessError *)ptr)->soap_out(soap, "vbox:IMedium_getLastAccessError", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREgetSizeResponse:
		return ((_vbox__IMedium_USCOREgetSizeResponse *)ptr)->soap_out(soap, "vbox:IMedium_getSizeResponse", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREgetSize:
		return ((_vbox__IMedium_USCOREgetSize *)ptr)->soap_out(soap, "vbox:IMedium_getSize", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREgetNameResponse:
		return ((_vbox__IMedium_USCOREgetNameResponse *)ptr)->soap_out(soap, "vbox:IMedium_getNameResponse", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREgetName:
		return ((_vbox__IMedium_USCOREgetName *)ptr)->soap_out(soap, "vbox:IMedium_getName", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREsetLocationResponse:
		return ((_vbox__IMedium_USCOREsetLocationResponse *)ptr)->soap_out(soap, "vbox:IMedium_setLocationResponse", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREsetLocation:
		return ((_vbox__IMedium_USCOREsetLocation *)ptr)->soap_out(soap, "vbox:IMedium_setLocation", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREgetLocationResponse:
		return ((_vbox__IMedium_USCOREgetLocationResponse *)ptr)->soap_out(soap, "vbox:IMedium_getLocationResponse", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREgetLocation:
		return ((_vbox__IMedium_USCOREgetLocation *)ptr)->soap_out(soap, "vbox:IMedium_getLocation", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREgetStateResponse:
		return ((_vbox__IMedium_USCOREgetStateResponse *)ptr)->soap_out(soap, "vbox:IMedium_getStateResponse", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREgetState:
		return ((_vbox__IMedium_USCOREgetState *)ptr)->soap_out(soap, "vbox:IMedium_getState", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREsetDescriptionResponse:
		return ((_vbox__IMedium_USCOREsetDescriptionResponse *)ptr)->soap_out(soap, "vbox:IMedium_setDescriptionResponse", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREsetDescription:
		return ((_vbox__IMedium_USCOREsetDescription *)ptr)->soap_out(soap, "vbox:IMedium_setDescription", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREgetDescriptionResponse:
		return ((_vbox__IMedium_USCOREgetDescriptionResponse *)ptr)->soap_out(soap, "vbox:IMedium_getDescriptionResponse", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREgetDescription:
		return ((_vbox__IMedium_USCOREgetDescription *)ptr)->soap_out(soap, "vbox:IMedium_getDescription", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREgetIdResponse:
		return ((_vbox__IMedium_USCOREgetIdResponse *)ptr)->soap_out(soap, "vbox:IMedium_getIdResponse", id, NULL);
	case SOAP_TYPE__vbox__IMedium_USCOREgetId:
		return ((_vbox__IMedium_USCOREgetId *)ptr)->soap_out(soap, "vbox:IMedium_getId", id, NULL);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetChildrenResponse:
		return ((_vbox__ISnapshot_USCOREgetChildrenResponse *)ptr)->soap_out(soap, "vbox:ISnapshot_getChildrenResponse", id, NULL);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetChildren:
		return ((_vbox__ISnapshot_USCOREgetChildren *)ptr)->soap_out(soap, "vbox:ISnapshot_getChildren", id, NULL);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetParentResponse:
		return ((_vbox__ISnapshot_USCOREgetParentResponse *)ptr)->soap_out(soap, "vbox:ISnapshot_getParentResponse", id, NULL);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetParent:
		return ((_vbox__ISnapshot_USCOREgetParent *)ptr)->soap_out(soap, "vbox:ISnapshot_getParent", id, NULL);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetMachineResponse:
		return ((_vbox__ISnapshot_USCOREgetMachineResponse *)ptr)->soap_out(soap, "vbox:ISnapshot_getMachineResponse", id, NULL);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetMachine:
		return ((_vbox__ISnapshot_USCOREgetMachine *)ptr)->soap_out(soap, "vbox:ISnapshot_getMachine", id, NULL);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetOnlineResponse:
		return ((_vbox__ISnapshot_USCOREgetOnlineResponse *)ptr)->soap_out(soap, "vbox:ISnapshot_getOnlineResponse", id, NULL);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetOnline:
		return ((_vbox__ISnapshot_USCOREgetOnline *)ptr)->soap_out(soap, "vbox:ISnapshot_getOnline", id, NULL);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetTimeStampResponse:
		return ((_vbox__ISnapshot_USCOREgetTimeStampResponse *)ptr)->soap_out(soap, "vbox:ISnapshot_getTimeStampResponse", id, NULL);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetTimeStamp:
		return ((_vbox__ISnapshot_USCOREgetTimeStamp *)ptr)->soap_out(soap, "vbox:ISnapshot_getTimeStamp", id, NULL);
	case SOAP_TYPE__vbox__ISnapshot_USCOREsetDescriptionResponse:
		return ((_vbox__ISnapshot_USCOREsetDescriptionResponse *)ptr)->soap_out(soap, "vbox:ISnapshot_setDescriptionResponse", id, NULL);
	case SOAP_TYPE__vbox__ISnapshot_USCOREsetDescription:
		return ((_vbox__ISnapshot_USCOREsetDescription *)ptr)->soap_out(soap, "vbox:ISnapshot_setDescription", id, NULL);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetDescriptionResponse:
		return ((_vbox__ISnapshot_USCOREgetDescriptionResponse *)ptr)->soap_out(soap, "vbox:ISnapshot_getDescriptionResponse", id, NULL);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetDescription:
		return ((_vbox__ISnapshot_USCOREgetDescription *)ptr)->soap_out(soap, "vbox:ISnapshot_getDescription", id, NULL);
	case SOAP_TYPE__vbox__ISnapshot_USCOREsetNameResponse:
		return ((_vbox__ISnapshot_USCOREsetNameResponse *)ptr)->soap_out(soap, "vbox:ISnapshot_setNameResponse", id, NULL);
	case SOAP_TYPE__vbox__ISnapshot_USCOREsetName:
		return ((_vbox__ISnapshot_USCOREsetName *)ptr)->soap_out(soap, "vbox:ISnapshot_setName", id, NULL);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetNameResponse:
		return ((_vbox__ISnapshot_USCOREgetNameResponse *)ptr)->soap_out(soap, "vbox:ISnapshot_getNameResponse", id, NULL);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetName:
		return ((_vbox__ISnapshot_USCOREgetName *)ptr)->soap_out(soap, "vbox:ISnapshot_getName", id, NULL);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetIdResponse:
		return ((_vbox__ISnapshot_USCOREgetIdResponse *)ptr)->soap_out(soap, "vbox:ISnapshot_getIdResponse", id, NULL);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetId:
		return ((_vbox__ISnapshot_USCOREgetId *)ptr)->soap_out(soap, "vbox:ISnapshot_getId", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREcancelResponse:
		return ((_vbox__IProgress_USCOREcancelResponse *)ptr)->soap_out(soap, "vbox:IProgress_cancelResponse", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREcancel:
		return ((_vbox__IProgress_USCOREcancel *)ptr)->soap_out(soap, "vbox:IProgress_cancel", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREwaitForOperationCompletionResponse:
		return ((_vbox__IProgress_USCOREwaitForOperationCompletionResponse *)ptr)->soap_out(soap, "vbox:IProgress_waitForOperationCompletionResponse", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREwaitForOperationCompletion:
		return ((_vbox__IProgress_USCOREwaitForOperationCompletion *)ptr)->soap_out(soap, "vbox:IProgress_waitForOperationCompletion", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREwaitForCompletionResponse:
		return ((_vbox__IProgress_USCOREwaitForCompletionResponse *)ptr)->soap_out(soap, "vbox:IProgress_waitForCompletionResponse", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREwaitForCompletion:
		return ((_vbox__IProgress_USCOREwaitForCompletion *)ptr)->soap_out(soap, "vbox:IProgress_waitForCompletion", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetOperationPercentResponse:
		return ((_vbox__IProgress_USCOREgetOperationPercentResponse *)ptr)->soap_out(soap, "vbox:IProgress_getOperationPercentResponse", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetOperationPercent:
		return ((_vbox__IProgress_USCOREgetOperationPercent *)ptr)->soap_out(soap, "vbox:IProgress_getOperationPercent", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetOperationDescriptionResponse:
		return ((_vbox__IProgress_USCOREgetOperationDescriptionResponse *)ptr)->soap_out(soap, "vbox:IProgress_getOperationDescriptionResponse", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetOperationDescription:
		return ((_vbox__IProgress_USCOREgetOperationDescription *)ptr)->soap_out(soap, "vbox:IProgress_getOperationDescription", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetOperationResponse:
		return ((_vbox__IProgress_USCOREgetOperationResponse *)ptr)->soap_out(soap, "vbox:IProgress_getOperationResponse", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetOperation:
		return ((_vbox__IProgress_USCOREgetOperation *)ptr)->soap_out(soap, "vbox:IProgress_getOperation", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetOperationCountResponse:
		return ((_vbox__IProgress_USCOREgetOperationCountResponse *)ptr)->soap_out(soap, "vbox:IProgress_getOperationCountResponse", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetOperationCount:
		return ((_vbox__IProgress_USCOREgetOperationCount *)ptr)->soap_out(soap, "vbox:IProgress_getOperationCount", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetErrorInfoResponse:
		return ((_vbox__IProgress_USCOREgetErrorInfoResponse *)ptr)->soap_out(soap, "vbox:IProgress_getErrorInfoResponse", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetErrorInfo:
		return ((_vbox__IProgress_USCOREgetErrorInfo *)ptr)->soap_out(soap, "vbox:IProgress_getErrorInfo", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetResultCodeResponse:
		return ((_vbox__IProgress_USCOREgetResultCodeResponse *)ptr)->soap_out(soap, "vbox:IProgress_getResultCodeResponse", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetResultCode:
		return ((_vbox__IProgress_USCOREgetResultCode *)ptr)->soap_out(soap, "vbox:IProgress_getResultCode", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetCanceledResponse:
		return ((_vbox__IProgress_USCOREgetCanceledResponse *)ptr)->soap_out(soap, "vbox:IProgress_getCanceledResponse", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetCanceled:
		return ((_vbox__IProgress_USCOREgetCanceled *)ptr)->soap_out(soap, "vbox:IProgress_getCanceled", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetCompletedResponse:
		return ((_vbox__IProgress_USCOREgetCompletedResponse *)ptr)->soap_out(soap, "vbox:IProgress_getCompletedResponse", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetCompleted:
		return ((_vbox__IProgress_USCOREgetCompleted *)ptr)->soap_out(soap, "vbox:IProgress_getCompleted", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetTimeRemainingResponse:
		return ((_vbox__IProgress_USCOREgetTimeRemainingResponse *)ptr)->soap_out(soap, "vbox:IProgress_getTimeRemainingResponse", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetTimeRemaining:
		return ((_vbox__IProgress_USCOREgetTimeRemaining *)ptr)->soap_out(soap, "vbox:IProgress_getTimeRemaining", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetPercentResponse:
		return ((_vbox__IProgress_USCOREgetPercentResponse *)ptr)->soap_out(soap, "vbox:IProgress_getPercentResponse", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetPercent:
		return ((_vbox__IProgress_USCOREgetPercent *)ptr)->soap_out(soap, "vbox:IProgress_getPercent", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetCancelableResponse:
		return ((_vbox__IProgress_USCOREgetCancelableResponse *)ptr)->soap_out(soap, "vbox:IProgress_getCancelableResponse", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetCancelable:
		return ((_vbox__IProgress_USCOREgetCancelable *)ptr)->soap_out(soap, "vbox:IProgress_getCancelable", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetInitiatorResponse:
		return ((_vbox__IProgress_USCOREgetInitiatorResponse *)ptr)->soap_out(soap, "vbox:IProgress_getInitiatorResponse", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetInitiator:
		return ((_vbox__IProgress_USCOREgetInitiator *)ptr)->soap_out(soap, "vbox:IProgress_getInitiator", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetDescriptionResponse:
		return ((_vbox__IProgress_USCOREgetDescriptionResponse *)ptr)->soap_out(soap, "vbox:IProgress_getDescriptionResponse", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetDescription:
		return ((_vbox__IProgress_USCOREgetDescription *)ptr)->soap_out(soap, "vbox:IProgress_getDescription", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetIdResponse:
		return ((_vbox__IProgress_USCOREgetIdResponse *)ptr)->soap_out(soap, "vbox:IProgress_getIdResponse", id, NULL);
	case SOAP_TYPE__vbox__IProgress_USCOREgetId:
		return ((_vbox__IProgress_USCOREgetId *)ptr)->soap_out(soap, "vbox:IProgress_getId", id, NULL);
	case SOAP_TYPE__vbox__IGuest_USCOREgetStatisticResponse:
		return ((_vbox__IGuest_USCOREgetStatisticResponse *)ptr)->soap_out(soap, "vbox:IGuest_getStatisticResponse", id, NULL);
	case SOAP_TYPE__vbox__IGuest_USCOREgetStatistic:
		return ((_vbox__IGuest_USCOREgetStatistic *)ptr)->soap_out(soap, "vbox:IGuest_getStatistic", id, NULL);
	case SOAP_TYPE__vbox__IGuest_USCOREsetCredentialsResponse:
		return ((_vbox__IGuest_USCOREsetCredentialsResponse *)ptr)->soap_out(soap, "vbox:IGuest_setCredentialsResponse", id, NULL);
	case SOAP_TYPE__vbox__IGuest_USCOREsetCredentials:
		return ((_vbox__IGuest_USCOREsetCredentials *)ptr)->soap_out(soap, "vbox:IGuest_setCredentials", id, NULL);
	case SOAP_TYPE__vbox__IGuest_USCOREsetStatisticsUpdateIntervalResponse:
		return ((_vbox__IGuest_USCOREsetStatisticsUpdateIntervalResponse *)ptr)->soap_out(soap, "vbox:IGuest_setStatisticsUpdateIntervalResponse", id, NULL);
	case SOAP_TYPE__vbox__IGuest_USCOREsetStatisticsUpdateInterval:
		return ((_vbox__IGuest_USCOREsetStatisticsUpdateInterval *)ptr)->soap_out(soap, "vbox:IGuest_setStatisticsUpdateInterval", id, NULL);
	case SOAP_TYPE__vbox__IGuest_USCOREgetStatisticsUpdateIntervalResponse:
		return ((_vbox__IGuest_USCOREgetStatisticsUpdateIntervalResponse *)ptr)->soap_out(soap, "vbox:IGuest_getStatisticsUpdateIntervalResponse", id, NULL);
	case SOAP_TYPE__vbox__IGuest_USCOREgetStatisticsUpdateInterval:
		return ((_vbox__IGuest_USCOREgetStatisticsUpdateInterval *)ptr)->soap_out(soap, "vbox:IGuest_getStatisticsUpdateInterval", id, NULL);
	case SOAP_TYPE__vbox__IGuest_USCOREsetMemoryBalloonSizeResponse:
		return ((_vbox__IGuest_USCOREsetMemoryBalloonSizeResponse *)ptr)->soap_out(soap, "vbox:IGuest_setMemoryBalloonSizeResponse", id, NULL);
	case SOAP_TYPE__vbox__IGuest_USCOREsetMemoryBalloonSize:
		return ((_vbox__IGuest_USCOREsetMemoryBalloonSize *)ptr)->soap_out(soap, "vbox:IGuest_setMemoryBalloonSize", id, NULL);
	case SOAP_TYPE__vbox__IGuest_USCOREgetMemoryBalloonSizeResponse:
		return ((_vbox__IGuest_USCOREgetMemoryBalloonSizeResponse *)ptr)->soap_out(soap, "vbox:IGuest_getMemoryBalloonSizeResponse", id, NULL);
	case SOAP_TYPE__vbox__IGuest_USCOREgetMemoryBalloonSize:
		return ((_vbox__IGuest_USCOREgetMemoryBalloonSize *)ptr)->soap_out(soap, "vbox:IGuest_getMemoryBalloonSize", id, NULL);
	case SOAP_TYPE__vbox__IGuest_USCOREgetSupportsGraphicsResponse:
		return ((_vbox__IGuest_USCOREgetSupportsGraphicsResponse *)ptr)->soap_out(soap, "vbox:IGuest_getSupportsGraphicsResponse", id, NULL);
	case SOAP_TYPE__vbox__IGuest_USCOREgetSupportsGraphics:
		return ((_vbox__IGuest_USCOREgetSupportsGraphics *)ptr)->soap_out(soap, "vbox:IGuest_getSupportsGraphics", id, NULL);
	case SOAP_TYPE__vbox__IGuest_USCOREgetSupportsSeamlessResponse:
		return ((_vbox__IGuest_USCOREgetSupportsSeamlessResponse *)ptr)->soap_out(soap, "vbox:IGuest_getSupportsSeamlessResponse", id, NULL);
	case SOAP_TYPE__vbox__IGuest_USCOREgetSupportsSeamless:
		return ((_vbox__IGuest_USCOREgetSupportsSeamless *)ptr)->soap_out(soap, "vbox:IGuest_getSupportsSeamless", id, NULL);
	case SOAP_TYPE__vbox__IGuest_USCOREgetAdditionsVersionResponse:
		return ((_vbox__IGuest_USCOREgetAdditionsVersionResponse *)ptr)->soap_out(soap, "vbox:IGuest_getAdditionsVersionResponse", id, NULL);
	case SOAP_TYPE__vbox__IGuest_USCOREgetAdditionsVersion:
		return ((_vbox__IGuest_USCOREgetAdditionsVersion *)ptr)->soap_out(soap, "vbox:IGuest_getAdditionsVersion", id, NULL);
	case SOAP_TYPE__vbox__IGuest_USCOREgetAdditionsActiveResponse:
		return ((_vbox__IGuest_USCOREgetAdditionsActiveResponse *)ptr)->soap_out(soap, "vbox:IGuest_getAdditionsActiveResponse", id, NULL);
	case SOAP_TYPE__vbox__IGuest_USCOREgetAdditionsActive:
		return ((_vbox__IGuest_USCOREgetAdditionsActive *)ptr)->soap_out(soap, "vbox:IGuest_getAdditionsActive", id, NULL);
	case SOAP_TYPE__vbox__IGuest_USCOREgetOSTypeIdResponse:
		return ((_vbox__IGuest_USCOREgetOSTypeIdResponse *)ptr)->soap_out(soap, "vbox:IGuest_getOSTypeIdResponse", id, NULL);
	case SOAP_TYPE__vbox__IGuest_USCOREgetOSTypeId:
		return ((_vbox__IGuest_USCOREgetOSTypeId *)ptr)->soap_out(soap, "vbox:IGuest_getOSTypeId", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetDefaultAudioDriverResponse:
		return ((_vbox__ISystemProperties_USCOREgetDefaultAudioDriverResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_getDefaultAudioDriverResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetDefaultAudioDriver:
		return ((_vbox__ISystemProperties_USCOREgetDefaultAudioDriver *)ptr)->soap_out(soap, "vbox:ISystemProperties_getDefaultAudioDriver", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetLogHistoryCountResponse:
		return ((_vbox__ISystemProperties_USCOREsetLogHistoryCountResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_setLogHistoryCountResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetLogHistoryCount:
		return ((_vbox__ISystemProperties_USCOREsetLogHistoryCount *)ptr)->soap_out(soap, "vbox:ISystemProperties_setLogHistoryCount", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetLogHistoryCountResponse:
		return ((_vbox__ISystemProperties_USCOREgetLogHistoryCountResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_getLogHistoryCountResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetLogHistoryCount:
		return ((_vbox__ISystemProperties_USCOREgetLogHistoryCount *)ptr)->soap_out(soap, "vbox:ISystemProperties_getLogHistoryCount", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetWebServiceAuthLibraryResponse:
		return ((_vbox__ISystemProperties_USCOREsetWebServiceAuthLibraryResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_setWebServiceAuthLibraryResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetWebServiceAuthLibrary:
		return ((_vbox__ISystemProperties_USCOREsetWebServiceAuthLibrary *)ptr)->soap_out(soap, "vbox:ISystemProperties_setWebServiceAuthLibrary", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetWebServiceAuthLibraryResponse:
		return ((_vbox__ISystemProperties_USCOREgetWebServiceAuthLibraryResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_getWebServiceAuthLibraryResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetWebServiceAuthLibrary:
		return ((_vbox__ISystemProperties_USCOREgetWebServiceAuthLibrary *)ptr)->soap_out(soap, "vbox:ISystemProperties_getWebServiceAuthLibrary", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibraryResponse:
		return ((_vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibraryResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_setRemoteDisplayAuthLibraryResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibrary:
		return ((_vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibrary *)ptr)->soap_out(soap, "vbox:ISystemProperties_setRemoteDisplayAuthLibrary", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibraryResponse:
		return ((_vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibraryResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_getRemoteDisplayAuthLibraryResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibrary:
		return ((_vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibrary *)ptr)->soap_out(soap, "vbox:ISystemProperties_getRemoteDisplayAuthLibrary", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetDefaultHardDiskFormatResponse:
		return ((_vbox__ISystemProperties_USCOREsetDefaultHardDiskFormatResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_setDefaultHardDiskFormatResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetDefaultHardDiskFormat:
		return ((_vbox__ISystemProperties_USCOREsetDefaultHardDiskFormat *)ptr)->soap_out(soap, "vbox:ISystemProperties_setDefaultHardDiskFormat", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetDefaultHardDiskFormatResponse:
		return ((_vbox__ISystemProperties_USCOREgetDefaultHardDiskFormatResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_getDefaultHardDiskFormatResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetDefaultHardDiskFormat:
		return ((_vbox__ISystemProperties_USCOREgetDefaultHardDiskFormat *)ptr)->soap_out(soap, "vbox:ISystemProperties_getDefaultHardDiskFormat", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetHardDiskFormatsResponse:
		return ((_vbox__ISystemProperties_USCOREgetHardDiskFormatsResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_getHardDiskFormatsResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetHardDiskFormats:
		return ((_vbox__ISystemProperties_USCOREgetHardDiskFormats *)ptr)->soap_out(soap, "vbox:ISystemProperties_getHardDiskFormats", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetDefaultHardDiskFolderResponse:
		return ((_vbox__ISystemProperties_USCOREsetDefaultHardDiskFolderResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_setDefaultHardDiskFolderResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetDefaultHardDiskFolder:
		return ((_vbox__ISystemProperties_USCOREsetDefaultHardDiskFolder *)ptr)->soap_out(soap, "vbox:ISystemProperties_setDefaultHardDiskFolder", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetDefaultHardDiskFolderResponse:
		return ((_vbox__ISystemProperties_USCOREgetDefaultHardDiskFolderResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_getDefaultHardDiskFolderResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetDefaultHardDiskFolder:
		return ((_vbox__ISystemProperties_USCOREgetDefaultHardDiskFolder *)ptr)->soap_out(soap, "vbox:ISystemProperties_getDefaultHardDiskFolder", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetDefaultMachineFolderResponse:
		return ((_vbox__ISystemProperties_USCOREsetDefaultMachineFolderResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_setDefaultMachineFolderResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetDefaultMachineFolder:
		return ((_vbox__ISystemProperties_USCOREsetDefaultMachineFolder *)ptr)->soap_out(soap, "vbox:ISystemProperties_setDefaultMachineFolder", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetDefaultMachineFolderResponse:
		return ((_vbox__ISystemProperties_USCOREgetDefaultMachineFolderResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_getDefaultMachineFolderResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetDefaultMachineFolder:
		return ((_vbox__ISystemProperties_USCOREgetDefaultMachineFolder *)ptr)->soap_out(soap, "vbox:ISystemProperties_getDefaultMachineFolder", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxBootPositionResponse:
		return ((_vbox__ISystemProperties_USCOREgetMaxBootPositionResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_getMaxBootPositionResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxBootPosition:
		return ((_vbox__ISystemProperties_USCOREgetMaxBootPosition *)ptr)->soap_out(soap, "vbox:ISystemProperties_getMaxBootPosition", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetParallelPortCountResponse:
		return ((_vbox__ISystemProperties_USCOREgetParallelPortCountResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_getParallelPortCountResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetParallelPortCount:
		return ((_vbox__ISystemProperties_USCOREgetParallelPortCount *)ptr)->soap_out(soap, "vbox:ISystemProperties_getParallelPortCount", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetSerialPortCountResponse:
		return ((_vbox__ISystemProperties_USCOREgetSerialPortCountResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_getSerialPortCountResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetSerialPortCount:
		return ((_vbox__ISystemProperties_USCOREgetSerialPortCount *)ptr)->soap_out(soap, "vbox:ISystemProperties_getSerialPortCount", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetNetworkAdapterCountResponse:
		return ((_vbox__ISystemProperties_USCOREgetNetworkAdapterCountResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_getNetworkAdapterCountResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetNetworkAdapterCount:
		return ((_vbox__ISystemProperties_USCOREgetNetworkAdapterCount *)ptr)->soap_out(soap, "vbox:ISystemProperties_getNetworkAdapterCount", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxVDISizeResponse:
		return ((_vbox__ISystemProperties_USCOREgetMaxVDISizeResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_getMaxVDISizeResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxVDISize:
		return ((_vbox__ISystemProperties_USCOREgetMaxVDISize *)ptr)->soap_out(soap, "vbox:ISystemProperties_getMaxVDISize", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxGuestCPUCountResponse:
		return ((_vbox__ISystemProperties_USCOREgetMaxGuestCPUCountResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_getMaxGuestCPUCountResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxGuestCPUCount:
		return ((_vbox__ISystemProperties_USCOREgetMaxGuestCPUCount *)ptr)->soap_out(soap, "vbox:ISystemProperties_getMaxGuestCPUCount", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMinGuestCPUCountResponse:
		return ((_vbox__ISystemProperties_USCOREgetMinGuestCPUCountResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_getMinGuestCPUCountResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMinGuestCPUCount:
		return ((_vbox__ISystemProperties_USCOREgetMinGuestCPUCount *)ptr)->soap_out(soap, "vbox:ISystemProperties_getMinGuestCPUCount", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxGuestVRAMResponse:
		return ((_vbox__ISystemProperties_USCOREgetMaxGuestVRAMResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_getMaxGuestVRAMResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxGuestVRAM:
		return ((_vbox__ISystemProperties_USCOREgetMaxGuestVRAM *)ptr)->soap_out(soap, "vbox:ISystemProperties_getMaxGuestVRAM", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMinGuestVRAMResponse:
		return ((_vbox__ISystemProperties_USCOREgetMinGuestVRAMResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_getMinGuestVRAMResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMinGuestVRAM:
		return ((_vbox__ISystemProperties_USCOREgetMinGuestVRAM *)ptr)->soap_out(soap, "vbox:ISystemProperties_getMinGuestVRAM", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxGuestRAMResponse:
		return ((_vbox__ISystemProperties_USCOREgetMaxGuestRAMResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_getMaxGuestRAMResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxGuestRAM:
		return ((_vbox__ISystemProperties_USCOREgetMaxGuestRAM *)ptr)->soap_out(soap, "vbox:ISystemProperties_getMaxGuestRAM", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMinGuestRAMResponse:
		return ((_vbox__ISystemProperties_USCOREgetMinGuestRAMResponse *)ptr)->soap_out(soap, "vbox:ISystemProperties_getMinGuestRAMResponse", id, NULL);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMinGuestRAM:
		return ((_vbox__ISystemProperties_USCOREgetMinGuestRAM *)ptr)->soap_out(soap, "vbox:ISystemProperties_getMinGuestRAM", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREfindUSBDeviceByAddressResponse:
		return ((_vbox__IHost_USCOREfindUSBDeviceByAddressResponse *)ptr)->soap_out(soap, "vbox:IHost_findUSBDeviceByAddressResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREfindUSBDeviceByAddress:
		return ((_vbox__IHost_USCOREfindUSBDeviceByAddress *)ptr)->soap_out(soap, "vbox:IHost_findUSBDeviceByAddress", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREfindUSBDeviceByIdResponse:
		return ((_vbox__IHost_USCOREfindUSBDeviceByIdResponse *)ptr)->soap_out(soap, "vbox:IHost_findUSBDeviceByIdResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREfindUSBDeviceById:
		return ((_vbox__IHost_USCOREfindUSBDeviceById *)ptr)->soap_out(soap, "vbox:IHost_findUSBDeviceById", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREfindHostNetworkInterfacesOfTypeResponse:
		return ((_vbox__IHost_USCOREfindHostNetworkInterfacesOfTypeResponse *)ptr)->soap_out(soap, "vbox:IHost_findHostNetworkInterfacesOfTypeResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREfindHostNetworkInterfacesOfType:
		return ((_vbox__IHost_USCOREfindHostNetworkInterfacesOfType *)ptr)->soap_out(soap, "vbox:IHost_findHostNetworkInterfacesOfType", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREfindHostNetworkInterfaceByIdResponse:
		return ((_vbox__IHost_USCOREfindHostNetworkInterfaceByIdResponse *)ptr)->soap_out(soap, "vbox:IHost_findHostNetworkInterfaceByIdResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREfindHostNetworkInterfaceById:
		return ((_vbox__IHost_USCOREfindHostNetworkInterfaceById *)ptr)->soap_out(soap, "vbox:IHost_findHostNetworkInterfaceById", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREfindHostNetworkInterfaceByNameResponse:
		return ((_vbox__IHost_USCOREfindHostNetworkInterfaceByNameResponse *)ptr)->soap_out(soap, "vbox:IHost_findHostNetworkInterfaceByNameResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREfindHostNetworkInterfaceByName:
		return ((_vbox__IHost_USCOREfindHostNetworkInterfaceByName *)ptr)->soap_out(soap, "vbox:IHost_findHostNetworkInterfaceByName", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREfindHostFloppyDriveResponse:
		return ((_vbox__IHost_USCOREfindHostFloppyDriveResponse *)ptr)->soap_out(soap, "vbox:IHost_findHostFloppyDriveResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREfindHostFloppyDrive:
		return ((_vbox__IHost_USCOREfindHostFloppyDrive *)ptr)->soap_out(soap, "vbox:IHost_findHostFloppyDrive", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREfindHostDVDDriveResponse:
		return ((_vbox__IHost_USCOREfindHostDVDDriveResponse *)ptr)->soap_out(soap, "vbox:IHost_findHostDVDDriveResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREfindHostDVDDrive:
		return ((_vbox__IHost_USCOREfindHostDVDDrive *)ptr)->soap_out(soap, "vbox:IHost_findHostDVDDrive", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREremoveUSBDeviceFilterResponse:
		return ((_vbox__IHost_USCOREremoveUSBDeviceFilterResponse *)ptr)->soap_out(soap, "vbox:IHost_removeUSBDeviceFilterResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREremoveUSBDeviceFilter:
		return ((_vbox__IHost_USCOREremoveUSBDeviceFilter *)ptr)->soap_out(soap, "vbox:IHost_removeUSBDeviceFilter", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREinsertUSBDeviceFilterResponse:
		return ((_vbox__IHost_USCOREinsertUSBDeviceFilterResponse *)ptr)->soap_out(soap, "vbox:IHost_insertUSBDeviceFilterResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREinsertUSBDeviceFilter:
		return ((_vbox__IHost_USCOREinsertUSBDeviceFilter *)ptr)->soap_out(soap, "vbox:IHost_insertUSBDeviceFilter", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREcreateUSBDeviceFilterResponse:
		return ((_vbox__IHost_USCOREcreateUSBDeviceFilterResponse *)ptr)->soap_out(soap, "vbox:IHost_createUSBDeviceFilterResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREcreateUSBDeviceFilter:
		return ((_vbox__IHost_USCOREcreateUSBDeviceFilter *)ptr)->soap_out(soap, "vbox:IHost_createUSBDeviceFilter", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREremoveHostOnlyNetworkInterfaceResponse:
		return ((_vbox__IHost_USCOREremoveHostOnlyNetworkInterfaceResponse *)ptr)->soap_out(soap, "vbox:IHost_removeHostOnlyNetworkInterfaceResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREremoveHostOnlyNetworkInterface:
		return ((_vbox__IHost_USCOREremoveHostOnlyNetworkInterface *)ptr)->soap_out(soap, "vbox:IHost_removeHostOnlyNetworkInterface", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREcreateHostOnlyNetworkInterfaceResponse:
		return ((_vbox__IHost_USCOREcreateHostOnlyNetworkInterfaceResponse *)ptr)->soap_out(soap, "vbox:IHost_createHostOnlyNetworkInterfaceResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREcreateHostOnlyNetworkInterface:
		return ((_vbox__IHost_USCOREcreateHostOnlyNetworkInterface *)ptr)->soap_out(soap, "vbox:IHost_createHostOnlyNetworkInterface", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorDescriptionResponse:
		return ((_vbox__IHost_USCOREgetProcessorDescriptionResponse *)ptr)->soap_out(soap, "vbox:IHost_getProcessorDescriptionResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorDescription:
		return ((_vbox__IHost_USCOREgetProcessorDescription *)ptr)->soap_out(soap, "vbox:IHost_getProcessorDescription", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorFeatureResponse:
		return ((_vbox__IHost_USCOREgetProcessorFeatureResponse *)ptr)->soap_out(soap, "vbox:IHost_getProcessorFeatureResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorFeature:
		return ((_vbox__IHost_USCOREgetProcessorFeature *)ptr)->soap_out(soap, "vbox:IHost_getProcessorFeature", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorSpeedResponse:
		return ((_vbox__IHost_USCOREgetProcessorSpeedResponse *)ptr)->soap_out(soap, "vbox:IHost_getProcessorSpeedResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorSpeed:
		return ((_vbox__IHost_USCOREgetProcessorSpeed *)ptr)->soap_out(soap, "vbox:IHost_getProcessorSpeed", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetAcceleration3DAvailableResponse:
		return ((_vbox__IHost_USCOREgetAcceleration3DAvailableResponse *)ptr)->soap_out(soap, "vbox:IHost_getAcceleration3DAvailableResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetAcceleration3DAvailable:
		return ((_vbox__IHost_USCOREgetAcceleration3DAvailable *)ptr)->soap_out(soap, "vbox:IHost_getAcceleration3DAvailable", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetUTCTimeResponse:
		return ((_vbox__IHost_USCOREgetUTCTimeResponse *)ptr)->soap_out(soap, "vbox:IHost_getUTCTimeResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetUTCTime:
		return ((_vbox__IHost_USCOREgetUTCTime *)ptr)->soap_out(soap, "vbox:IHost_getUTCTime", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetOSVersionResponse:
		return ((_vbox__IHost_USCOREgetOSVersionResponse *)ptr)->soap_out(soap, "vbox:IHost_getOSVersionResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetOSVersion:
		return ((_vbox__IHost_USCOREgetOSVersion *)ptr)->soap_out(soap, "vbox:IHost_getOSVersion", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetOperatingSystemResponse:
		return ((_vbox__IHost_USCOREgetOperatingSystemResponse *)ptr)->soap_out(soap, "vbox:IHost_getOperatingSystemResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetOperatingSystem:
		return ((_vbox__IHost_USCOREgetOperatingSystem *)ptr)->soap_out(soap, "vbox:IHost_getOperatingSystem", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetMemoryAvailableResponse:
		return ((_vbox__IHost_USCOREgetMemoryAvailableResponse *)ptr)->soap_out(soap, "vbox:IHost_getMemoryAvailableResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetMemoryAvailable:
		return ((_vbox__IHost_USCOREgetMemoryAvailable *)ptr)->soap_out(soap, "vbox:IHost_getMemoryAvailable", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetMemorySizeResponse:
		return ((_vbox__IHost_USCOREgetMemorySizeResponse *)ptr)->soap_out(soap, "vbox:IHost_getMemorySizeResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetMemorySize:
		return ((_vbox__IHost_USCOREgetMemorySize *)ptr)->soap_out(soap, "vbox:IHost_getMemorySize", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorOnlineCountResponse:
		return ((_vbox__IHost_USCOREgetProcessorOnlineCountResponse *)ptr)->soap_out(soap, "vbox:IHost_getProcessorOnlineCountResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorOnlineCount:
		return ((_vbox__IHost_USCOREgetProcessorOnlineCount *)ptr)->soap_out(soap, "vbox:IHost_getProcessorOnlineCount", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorCountResponse:
		return ((_vbox__IHost_USCOREgetProcessorCountResponse *)ptr)->soap_out(soap, "vbox:IHost_getProcessorCountResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorCount:
		return ((_vbox__IHost_USCOREgetProcessorCount *)ptr)->soap_out(soap, "vbox:IHost_getProcessorCount", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetNetworkInterfacesResponse:
		return ((_vbox__IHost_USCOREgetNetworkInterfacesResponse *)ptr)->soap_out(soap, "vbox:IHost_getNetworkInterfacesResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetNetworkInterfaces:
		return ((_vbox__IHost_USCOREgetNetworkInterfaces *)ptr)->soap_out(soap, "vbox:IHost_getNetworkInterfaces", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetUSBDeviceFiltersResponse:
		return ((_vbox__IHost_USCOREgetUSBDeviceFiltersResponse *)ptr)->soap_out(soap, "vbox:IHost_getUSBDeviceFiltersResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetUSBDeviceFilters:
		return ((_vbox__IHost_USCOREgetUSBDeviceFilters *)ptr)->soap_out(soap, "vbox:IHost_getUSBDeviceFilters", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetUSBDevicesResponse:
		return ((_vbox__IHost_USCOREgetUSBDevicesResponse *)ptr)->soap_out(soap, "vbox:IHost_getUSBDevicesResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetUSBDevices:
		return ((_vbox__IHost_USCOREgetUSBDevices *)ptr)->soap_out(soap, "vbox:IHost_getUSBDevices", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetFloppyDrivesResponse:
		return ((_vbox__IHost_USCOREgetFloppyDrivesResponse *)ptr)->soap_out(soap, "vbox:IHost_getFloppyDrivesResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetFloppyDrives:
		return ((_vbox__IHost_USCOREgetFloppyDrives *)ptr)->soap_out(soap, "vbox:IHost_getFloppyDrives", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetDVDDrivesResponse:
		return ((_vbox__IHost_USCOREgetDVDDrivesResponse *)ptr)->soap_out(soap, "vbox:IHost_getDVDDrivesResponse", id, NULL);
	case SOAP_TYPE__vbox__IHost_USCOREgetDVDDrives:
		return ((_vbox__IHost_USCOREgetDVDDrives *)ptr)->soap_out(soap, "vbox:IHost_getDVDDrives", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREdhcpRediscoverResponse:
		return ((_vbox__IHostNetworkInterface_USCOREdhcpRediscoverResponse *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_dhcpRediscoverResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREdhcpRediscover:
		return ((_vbox__IHostNetworkInterface_USCOREdhcpRediscover *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_dhcpRediscover", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREenableDynamicIpConfigResponse:
		return ((_vbox__IHostNetworkInterface_USCOREenableDynamicIpConfigResponse *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_enableDynamicIpConfigResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREenableDynamicIpConfig:
		return ((_vbox__IHostNetworkInterface_USCOREenableDynamicIpConfig *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_enableDynamicIpConfig", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6Response:
		return ((_vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6Response *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_enableStaticIpConfigV6Response", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6:
		return ((_vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6 *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_enableStaticIpConfigV6", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREenableStaticIpConfigResponse:
		return ((_vbox__IHostNetworkInterface_USCOREenableStaticIpConfigResponse *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_enableStaticIpConfigResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREenableStaticIpConfig:
		return ((_vbox__IHostNetworkInterface_USCOREenableStaticIpConfig *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_enableStaticIpConfig", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetInterfaceTypeResponse:
		return ((_vbox__IHostNetworkInterface_USCOREgetInterfaceTypeResponse *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getInterfaceTypeResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetInterfaceType:
		return ((_vbox__IHostNetworkInterface_USCOREgetInterfaceType *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getInterfaceType", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetStatusResponse:
		return ((_vbox__IHostNetworkInterface_USCOREgetStatusResponse *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getStatusResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetStatus:
		return ((_vbox__IHostNetworkInterface_USCOREgetStatus *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getStatus", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetMediumTypeResponse:
		return ((_vbox__IHostNetworkInterface_USCOREgetMediumTypeResponse *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getMediumTypeResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetMediumType:
		return ((_vbox__IHostNetworkInterface_USCOREgetMediumType *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getMediumType", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetHardwareAddressResponse:
		return ((_vbox__IHostNetworkInterface_USCOREgetHardwareAddressResponse *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getHardwareAddressResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetHardwareAddress:
		return ((_vbox__IHostNetworkInterface_USCOREgetHardwareAddress *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getHardwareAddress", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLengthResponse:
		return ((_vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLengthResponse *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getIPV6NetworkMaskPrefixLengthResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLength:
		return ((_vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLength *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getIPV6NetworkMaskPrefixLength", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIPV6AddressResponse:
		return ((_vbox__IHostNetworkInterface_USCOREgetIPV6AddressResponse *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getIPV6AddressResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIPV6Address:
		return ((_vbox__IHostNetworkInterface_USCOREgetIPV6Address *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getIPV6Address", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIPV6SupportedResponse:
		return ((_vbox__IHostNetworkInterface_USCOREgetIPV6SupportedResponse *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getIPV6SupportedResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIPV6Supported:
		return ((_vbox__IHostNetworkInterface_USCOREgetIPV6Supported *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getIPV6Supported", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetNetworkMaskResponse:
		return ((_vbox__IHostNetworkInterface_USCOREgetNetworkMaskResponse *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getNetworkMaskResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetNetworkMask:
		return ((_vbox__IHostNetworkInterface_USCOREgetNetworkMask *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getNetworkMask", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIPAddressResponse:
		return ((_vbox__IHostNetworkInterface_USCOREgetIPAddressResponse *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getIPAddressResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIPAddress:
		return ((_vbox__IHostNetworkInterface_USCOREgetIPAddress *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getIPAddress", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetDhcpEnabledResponse:
		return ((_vbox__IHostNetworkInterface_USCOREgetDhcpEnabledResponse *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getDhcpEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetDhcpEnabled:
		return ((_vbox__IHostNetworkInterface_USCOREgetDhcpEnabled *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getDhcpEnabled", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetNetworkNameResponse:
		return ((_vbox__IHostNetworkInterface_USCOREgetNetworkNameResponse *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getNetworkNameResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetNetworkName:
		return ((_vbox__IHostNetworkInterface_USCOREgetNetworkName *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getNetworkName", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIdResponse:
		return ((_vbox__IHostNetworkInterface_USCOREgetIdResponse *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getIdResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetId:
		return ((_vbox__IHostNetworkInterface_USCOREgetId *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getId", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetNameResponse:
		return ((_vbox__IHostNetworkInterface_USCOREgetNameResponse *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getNameResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetName:
		return ((_vbox__IHostNetworkInterface_USCOREgetName *)ptr)->soap_out(soap, "vbox:IHostNetworkInterface_getName", id, NULL);
	case SOAP_TYPE__vbox__IHostFloppyDrive_USCOREgetUdiResponse:
		return ((_vbox__IHostFloppyDrive_USCOREgetUdiResponse *)ptr)->soap_out(soap, "vbox:IHostFloppyDrive_getUdiResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostFloppyDrive_USCOREgetUdi:
		return ((_vbox__IHostFloppyDrive_USCOREgetUdi *)ptr)->soap_out(soap, "vbox:IHostFloppyDrive_getUdi", id, NULL);
	case SOAP_TYPE__vbox__IHostFloppyDrive_USCOREgetDescriptionResponse:
		return ((_vbox__IHostFloppyDrive_USCOREgetDescriptionResponse *)ptr)->soap_out(soap, "vbox:IHostFloppyDrive_getDescriptionResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostFloppyDrive_USCOREgetDescription:
		return ((_vbox__IHostFloppyDrive_USCOREgetDescription *)ptr)->soap_out(soap, "vbox:IHostFloppyDrive_getDescription", id, NULL);
	case SOAP_TYPE__vbox__IHostFloppyDrive_USCOREgetNameResponse:
		return ((_vbox__IHostFloppyDrive_USCOREgetNameResponse *)ptr)->soap_out(soap, "vbox:IHostFloppyDrive_getNameResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostFloppyDrive_USCOREgetName:
		return ((_vbox__IHostFloppyDrive_USCOREgetName *)ptr)->soap_out(soap, "vbox:IHostFloppyDrive_getName", id, NULL);
	case SOAP_TYPE__vbox__IHostDVDDrive_USCOREgetUdiResponse:
		return ((_vbox__IHostDVDDrive_USCOREgetUdiResponse *)ptr)->soap_out(soap, "vbox:IHostDVDDrive_getUdiResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostDVDDrive_USCOREgetUdi:
		return ((_vbox__IHostDVDDrive_USCOREgetUdi *)ptr)->soap_out(soap, "vbox:IHostDVDDrive_getUdi", id, NULL);
	case SOAP_TYPE__vbox__IHostDVDDrive_USCOREgetDescriptionResponse:
		return ((_vbox__IHostDVDDrive_USCOREgetDescriptionResponse *)ptr)->soap_out(soap, "vbox:IHostDVDDrive_getDescriptionResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostDVDDrive_USCOREgetDescription:
		return ((_vbox__IHostDVDDrive_USCOREgetDescription *)ptr)->soap_out(soap, "vbox:IHostDVDDrive_getDescription", id, NULL);
	case SOAP_TYPE__vbox__IHostDVDDrive_USCOREgetNameResponse:
		return ((_vbox__IHostDVDDrive_USCOREgetNameResponse *)ptr)->soap_out(soap, "vbox:IHostDVDDrive_getNameResponse", id, NULL);
	case SOAP_TYPE__vbox__IHostDVDDrive_USCOREgetName:
		return ((_vbox__IHostDVDDrive_USCOREgetName *)ptr)->soap_out(soap, "vbox:IHostDVDDrive_getName", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREdiscardCurrentSnapshotAndStateResponse:
		return ((_vbox__IConsole_USCOREdiscardCurrentSnapshotAndStateResponse *)ptr)->soap_out(soap, "vbox:IConsole_discardCurrentSnapshotAndStateResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREdiscardCurrentSnapshotAndState:
		return ((_vbox__IConsole_USCOREdiscardCurrentSnapshotAndState *)ptr)->soap_out(soap, "vbox:IConsole_discardCurrentSnapshotAndState", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREdiscardCurrentStateResponse:
		return ((_vbox__IConsole_USCOREdiscardCurrentStateResponse *)ptr)->soap_out(soap, "vbox:IConsole_discardCurrentStateResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREdiscardCurrentState:
		return ((_vbox__IConsole_USCOREdiscardCurrentState *)ptr)->soap_out(soap, "vbox:IConsole_discardCurrentState", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREdiscardSnapshotResponse:
		return ((_vbox__IConsole_USCOREdiscardSnapshotResponse *)ptr)->soap_out(soap, "vbox:IConsole_discardSnapshotResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREdiscardSnapshot:
		return ((_vbox__IConsole_USCOREdiscardSnapshot *)ptr)->soap_out(soap, "vbox:IConsole_discardSnapshot", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREtakeSnapshotResponse:
		return ((_vbox__IConsole_USCOREtakeSnapshotResponse *)ptr)->soap_out(soap, "vbox:IConsole_takeSnapshotResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREtakeSnapshot:
		return ((_vbox__IConsole_USCOREtakeSnapshot *)ptr)->soap_out(soap, "vbox:IConsole_takeSnapshot", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREremoveSharedFolderResponse:
		return ((_vbox__IConsole_USCOREremoveSharedFolderResponse *)ptr)->soap_out(soap, "vbox:IConsole_removeSharedFolderResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREremoveSharedFolder:
		return ((_vbox__IConsole_USCOREremoveSharedFolder *)ptr)->soap_out(soap, "vbox:IConsole_removeSharedFolder", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREcreateSharedFolderResponse:
		return ((_vbox__IConsole_USCOREcreateSharedFolderResponse *)ptr)->soap_out(soap, "vbox:IConsole_createSharedFolderResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREcreateSharedFolder:
		return ((_vbox__IConsole_USCOREcreateSharedFolder *)ptr)->soap_out(soap, "vbox:IConsole_createSharedFolder", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREfindUSBDeviceByIdResponse:
		return ((_vbox__IConsole_USCOREfindUSBDeviceByIdResponse *)ptr)->soap_out(soap, "vbox:IConsole_findUSBDeviceByIdResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREfindUSBDeviceById:
		return ((_vbox__IConsole_USCOREfindUSBDeviceById *)ptr)->soap_out(soap, "vbox:IConsole_findUSBDeviceById", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREfindUSBDeviceByAddressResponse:
		return ((_vbox__IConsole_USCOREfindUSBDeviceByAddressResponse *)ptr)->soap_out(soap, "vbox:IConsole_findUSBDeviceByAddressResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREfindUSBDeviceByAddress:
		return ((_vbox__IConsole_USCOREfindUSBDeviceByAddress *)ptr)->soap_out(soap, "vbox:IConsole_findUSBDeviceByAddress", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREdetachUSBDeviceResponse:
		return ((_vbox__IConsole_USCOREdetachUSBDeviceResponse *)ptr)->soap_out(soap, "vbox:IConsole_detachUSBDeviceResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREdetachUSBDevice:
		return ((_vbox__IConsole_USCOREdetachUSBDevice *)ptr)->soap_out(soap, "vbox:IConsole_detachUSBDevice", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREattachUSBDeviceResponse:
		return ((_vbox__IConsole_USCOREattachUSBDeviceResponse *)ptr)->soap_out(soap, "vbox:IConsole_attachUSBDeviceResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREattachUSBDevice:
		return ((_vbox__IConsole_USCOREattachUSBDevice *)ptr)->soap_out(soap, "vbox:IConsole_attachUSBDevice", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetDeviceActivityResponse:
		return ((_vbox__IConsole_USCOREgetDeviceActivityResponse *)ptr)->soap_out(soap, "vbox:IConsole_getDeviceActivityResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetDeviceActivity:
		return ((_vbox__IConsole_USCOREgetDeviceActivity *)ptr)->soap_out(soap, "vbox:IConsole_getDeviceActivity", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREforgetSavedStateResponse:
		return ((_vbox__IConsole_USCOREforgetSavedStateResponse *)ptr)->soap_out(soap, "vbox:IConsole_forgetSavedStateResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREforgetSavedState:
		return ((_vbox__IConsole_USCOREforgetSavedState *)ptr)->soap_out(soap, "vbox:IConsole_forgetSavedState", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREadoptSavedStateResponse:
		return ((_vbox__IConsole_USCOREadoptSavedStateResponse *)ptr)->soap_out(soap, "vbox:IConsole_adoptSavedStateResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREadoptSavedState:
		return ((_vbox__IConsole_USCOREadoptSavedState *)ptr)->soap_out(soap, "vbox:IConsole_adoptSavedState", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREsaveStateResponse:
		return ((_vbox__IConsole_USCOREsaveStateResponse *)ptr)->soap_out(soap, "vbox:IConsole_saveStateResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREsaveState:
		return ((_vbox__IConsole_USCOREsaveState *)ptr)->soap_out(soap, "vbox:IConsole_saveState", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetGuestEnteredACPIModeResponse:
		return ((_vbox__IConsole_USCOREgetGuestEnteredACPIModeResponse *)ptr)->soap_out(soap, "vbox:IConsole_getGuestEnteredACPIModeResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetGuestEnteredACPIMode:
		return ((_vbox__IConsole_USCOREgetGuestEnteredACPIMode *)ptr)->soap_out(soap, "vbox:IConsole_getGuestEnteredACPIMode", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetPowerButtonHandledResponse:
		return ((_vbox__IConsole_USCOREgetPowerButtonHandledResponse *)ptr)->soap_out(soap, "vbox:IConsole_getPowerButtonHandledResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetPowerButtonHandled:
		return ((_vbox__IConsole_USCOREgetPowerButtonHandled *)ptr)->soap_out(soap, "vbox:IConsole_getPowerButtonHandled", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREsleepButtonResponse:
		return ((_vbox__IConsole_USCOREsleepButtonResponse *)ptr)->soap_out(soap, "vbox:IConsole_sleepButtonResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREsleepButton:
		return ((_vbox__IConsole_USCOREsleepButton *)ptr)->soap_out(soap, "vbox:IConsole_sleepButton", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREpowerButtonResponse:
		return ((_vbox__IConsole_USCOREpowerButtonResponse *)ptr)->soap_out(soap, "vbox:IConsole_powerButtonResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREpowerButton:
		return ((_vbox__IConsole_USCOREpowerButton *)ptr)->soap_out(soap, "vbox:IConsole_powerButton", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREresumeResponse:
		return ((_vbox__IConsole_USCOREresumeResponse *)ptr)->soap_out(soap, "vbox:IConsole_resumeResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREresume:
		return ((_vbox__IConsole_USCOREresume *)ptr)->soap_out(soap, "vbox:IConsole_resume", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREpauseResponse:
		return ((_vbox__IConsole_USCOREpauseResponse *)ptr)->soap_out(soap, "vbox:IConsole_pauseResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREpause:
		return ((_vbox__IConsole_USCOREpause *)ptr)->soap_out(soap, "vbox:IConsole_pause", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREresetResponse:
		return ((_vbox__IConsole_USCOREresetResponse *)ptr)->soap_out(soap, "vbox:IConsole_resetResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREreset:
		return ((_vbox__IConsole_USCOREreset *)ptr)->soap_out(soap, "vbox:IConsole_reset", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREpowerDownResponse:
		return ((_vbox__IConsole_USCOREpowerDownResponse *)ptr)->soap_out(soap, "vbox:IConsole_powerDownResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREpowerDown:
		return ((_vbox__IConsole_USCOREpowerDown *)ptr)->soap_out(soap, "vbox:IConsole_powerDown", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREpowerUpPausedResponse:
		return ((_vbox__IConsole_USCOREpowerUpPausedResponse *)ptr)->soap_out(soap, "vbox:IConsole_powerUpPausedResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREpowerUpPaused:
		return ((_vbox__IConsole_USCOREpowerUpPaused *)ptr)->soap_out(soap, "vbox:IConsole_powerUpPaused", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREpowerUpResponse:
		return ((_vbox__IConsole_USCOREpowerUpResponse *)ptr)->soap_out(soap, "vbox:IConsole_powerUpResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREpowerUp:
		return ((_vbox__IConsole_USCOREpowerUp *)ptr)->soap_out(soap, "vbox:IConsole_powerUp", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetRemoteDisplayInfoResponse:
		return ((_vbox__IConsole_USCOREgetRemoteDisplayInfoResponse *)ptr)->soap_out(soap, "vbox:IConsole_getRemoteDisplayInfoResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetRemoteDisplayInfo:
		return ((_vbox__IConsole_USCOREgetRemoteDisplayInfo *)ptr)->soap_out(soap, "vbox:IConsole_getRemoteDisplayInfo", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetSharedFoldersResponse:
		return ((_vbox__IConsole_USCOREgetSharedFoldersResponse *)ptr)->soap_out(soap, "vbox:IConsole_getSharedFoldersResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetSharedFolders:
		return ((_vbox__IConsole_USCOREgetSharedFolders *)ptr)->soap_out(soap, "vbox:IConsole_getSharedFolders", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetRemoteUSBDevicesResponse:
		return ((_vbox__IConsole_USCOREgetRemoteUSBDevicesResponse *)ptr)->soap_out(soap, "vbox:IConsole_getRemoteUSBDevicesResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetRemoteUSBDevices:
		return ((_vbox__IConsole_USCOREgetRemoteUSBDevices *)ptr)->soap_out(soap, "vbox:IConsole_getRemoteUSBDevices", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetUSBDevicesResponse:
		return ((_vbox__IConsole_USCOREgetUSBDevicesResponse *)ptr)->soap_out(soap, "vbox:IConsole_getUSBDevicesResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetUSBDevices:
		return ((_vbox__IConsole_USCOREgetUSBDevices *)ptr)->soap_out(soap, "vbox:IConsole_getUSBDevices", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetDisplayResponse:
		return ((_vbox__IConsole_USCOREgetDisplayResponse *)ptr)->soap_out(soap, "vbox:IConsole_getDisplayResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetDisplay:
		return ((_vbox__IConsole_USCOREgetDisplay *)ptr)->soap_out(soap, "vbox:IConsole_getDisplay", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetMouseResponse:
		return ((_vbox__IConsole_USCOREgetMouseResponse *)ptr)->soap_out(soap, "vbox:IConsole_getMouseResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetMouse:
		return ((_vbox__IConsole_USCOREgetMouse *)ptr)->soap_out(soap, "vbox:IConsole_getMouse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetKeyboardResponse:
		return ((_vbox__IConsole_USCOREgetKeyboardResponse *)ptr)->soap_out(soap, "vbox:IConsole_getKeyboardResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetKeyboard:
		return ((_vbox__IConsole_USCOREgetKeyboard *)ptr)->soap_out(soap, "vbox:IConsole_getKeyboard", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetGuestResponse:
		return ((_vbox__IConsole_USCOREgetGuestResponse *)ptr)->soap_out(soap, "vbox:IConsole_getGuestResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetGuest:
		return ((_vbox__IConsole_USCOREgetGuest *)ptr)->soap_out(soap, "vbox:IConsole_getGuest", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetStateResponse:
		return ((_vbox__IConsole_USCOREgetStateResponse *)ptr)->soap_out(soap, "vbox:IConsole_getStateResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetState:
		return ((_vbox__IConsole_USCOREgetState *)ptr)->soap_out(soap, "vbox:IConsole_getState", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetMachineResponse:
		return ((_vbox__IConsole_USCOREgetMachineResponse *)ptr)->soap_out(soap, "vbox:IConsole_getMachineResponse", id, NULL);
	case SOAP_TYPE__vbox__IConsole_USCOREgetMachine:
		return ((_vbox__IConsole_USCOREgetMachine *)ptr)->soap_out(soap, "vbox:IConsole_getMachine", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREenumerateGuestPropertiesResponse:
		return ((_vbox__IMachine_USCOREenumerateGuestPropertiesResponse *)ptr)->soap_out(soap, "vbox:IMachine_enumerateGuestPropertiesResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREenumerateGuestProperties:
		return ((_vbox__IMachine_USCOREenumerateGuestProperties *)ptr)->soap_out(soap, "vbox:IMachine_enumerateGuestProperties", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetGuestPropertyValueResponse:
		return ((_vbox__IMachine_USCOREsetGuestPropertyValueResponse *)ptr)->soap_out(soap, "vbox:IMachine_setGuestPropertyValueResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetGuestPropertyValue:
		return ((_vbox__IMachine_USCOREsetGuestPropertyValue *)ptr)->soap_out(soap, "vbox:IMachine_setGuestPropertyValue", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetGuestPropertyResponse:
		return ((_vbox__IMachine_USCOREsetGuestPropertyResponse *)ptr)->soap_out(soap, "vbox:IMachine_setGuestPropertyResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetGuestProperty:
		return ((_vbox__IMachine_USCOREsetGuestProperty *)ptr)->soap_out(soap, "vbox:IMachine_setGuestProperty", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetGuestPropertyTimestampResponse:
		return ((_vbox__IMachine_USCOREgetGuestPropertyTimestampResponse *)ptr)->soap_out(soap, "vbox:IMachine_getGuestPropertyTimestampResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetGuestPropertyTimestamp:
		return ((_vbox__IMachine_USCOREgetGuestPropertyTimestamp *)ptr)->soap_out(soap, "vbox:IMachine_getGuestPropertyTimestamp", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetGuestPropertyValueResponse:
		return ((_vbox__IMachine_USCOREgetGuestPropertyValueResponse *)ptr)->soap_out(soap, "vbox:IMachine_getGuestPropertyValueResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetGuestPropertyValue:
		return ((_vbox__IMachine_USCOREgetGuestPropertyValue *)ptr)->soap_out(soap, "vbox:IMachine_getGuestPropertyValue", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetGuestPropertyResponse:
		return ((_vbox__IMachine_USCOREgetGuestPropertyResponse *)ptr)->soap_out(soap, "vbox:IMachine_getGuestPropertyResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetGuestProperty:
		return ((_vbox__IMachine_USCOREgetGuestProperty *)ptr)->soap_out(soap, "vbox:IMachine_getGuestProperty", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREshowConsoleWindowResponse:
		return ((_vbox__IMachine_USCOREshowConsoleWindowResponse *)ptr)->soap_out(soap, "vbox:IMachine_showConsoleWindowResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREshowConsoleWindow:
		return ((_vbox__IMachine_USCOREshowConsoleWindow *)ptr)->soap_out(soap, "vbox:IMachine_showConsoleWindow", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREcanShowConsoleWindowResponse:
		return ((_vbox__IMachine_USCOREcanShowConsoleWindowResponse *)ptr)->soap_out(soap, "vbox:IMachine_canShowConsoleWindowResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREcanShowConsoleWindow:
		return ((_vbox__IMachine_USCOREcanShowConsoleWindow *)ptr)->soap_out(soap, "vbox:IMachine_canShowConsoleWindow", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREremoveSharedFolderResponse:
		return ((_vbox__IMachine_USCOREremoveSharedFolderResponse *)ptr)->soap_out(soap, "vbox:IMachine_removeSharedFolderResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREremoveSharedFolder:
		return ((_vbox__IMachine_USCOREremoveSharedFolder *)ptr)->soap_out(soap, "vbox:IMachine_removeSharedFolder", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREcreateSharedFolderResponse:
		return ((_vbox__IMachine_USCOREcreateSharedFolderResponse *)ptr)->soap_out(soap, "vbox:IMachine_createSharedFolderResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREcreateSharedFolder:
		return ((_vbox__IMachine_USCOREcreateSharedFolder *)ptr)->soap_out(soap, "vbox:IMachine_createSharedFolder", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetCurrentSnapshotResponse:
		return ((_vbox__IMachine_USCOREsetCurrentSnapshotResponse *)ptr)->soap_out(soap, "vbox:IMachine_setCurrentSnapshotResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetCurrentSnapshot:
		return ((_vbox__IMachine_USCOREsetCurrentSnapshot *)ptr)->soap_out(soap, "vbox:IMachine_setCurrentSnapshot", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREfindSnapshotResponse:
		return ((_vbox__IMachine_USCOREfindSnapshotResponse *)ptr)->soap_out(soap, "vbox:IMachine_findSnapshotResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREfindSnapshot:
		return ((_vbox__IMachine_USCOREfindSnapshot *)ptr)->soap_out(soap, "vbox:IMachine_findSnapshot", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSnapshotResponse:
		return ((_vbox__IMachine_USCOREgetSnapshotResponse *)ptr)->soap_out(soap, "vbox:IMachine_getSnapshotResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSnapshot:
		return ((_vbox__IMachine_USCOREgetSnapshot *)ptr)->soap_out(soap, "vbox:IMachine_getSnapshot", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREexportResponse:
		return ((_vbox__IMachine_USCOREexportResponse *)ptr)->soap_out(soap, "vbox:IMachine_exportResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREexport:
		return ((_vbox__IMachine_USCOREexport *)ptr)->soap_out(soap, "vbox:IMachine_export", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREdeleteSettingsResponse:
		return ((_vbox__IMachine_USCOREdeleteSettingsResponse *)ptr)->soap_out(soap, "vbox:IMachine_deleteSettingsResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREdeleteSettings:
		return ((_vbox__IMachine_USCOREdeleteSettings *)ptr)->soap_out(soap, "vbox:IMachine_deleteSettings", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREdiscardSettingsResponse:
		return ((_vbox__IMachine_USCOREdiscardSettingsResponse *)ptr)->soap_out(soap, "vbox:IMachine_discardSettingsResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREdiscardSettings:
		return ((_vbox__IMachine_USCOREdiscardSettings *)ptr)->soap_out(soap, "vbox:IMachine_discardSettings", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsaveSettingsWithBackupResponse:
		return ((_vbox__IMachine_USCOREsaveSettingsWithBackupResponse *)ptr)->soap_out(soap, "vbox:IMachine_saveSettingsWithBackupResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsaveSettingsWithBackup:
		return ((_vbox__IMachine_USCOREsaveSettingsWithBackup *)ptr)->soap_out(soap, "vbox:IMachine_saveSettingsWithBackup", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsaveSettingsResponse:
		return ((_vbox__IMachine_USCOREsaveSettingsResponse *)ptr)->soap_out(soap, "vbox:IMachine_saveSettingsResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsaveSettings:
		return ((_vbox__IMachine_USCOREsaveSettings *)ptr)->soap_out(soap, "vbox:IMachine_saveSettings", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetExtraDataResponse:
		return ((_vbox__IMachine_USCOREsetExtraDataResponse *)ptr)->soap_out(soap, "vbox:IMachine_setExtraDataResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetExtraData:
		return ((_vbox__IMachine_USCOREsetExtraData *)ptr)->soap_out(soap, "vbox:IMachine_setExtraData", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetExtraDataResponse:
		return ((_vbox__IMachine_USCOREgetExtraDataResponse *)ptr)->soap_out(soap, "vbox:IMachine_getExtraDataResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetExtraData:
		return ((_vbox__IMachine_USCOREgetExtraData *)ptr)->soap_out(soap, "vbox:IMachine_getExtraData", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetNextExtraDataKeyResponse:
		return ((_vbox__IMachine_USCOREgetNextExtraDataKeyResponse *)ptr)->soap_out(soap, "vbox:IMachine_getNextExtraDataKeyResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetNextExtraDataKey:
		return ((_vbox__IMachine_USCOREgetNextExtraDataKey *)ptr)->soap_out(soap, "vbox:IMachine_getNextExtraDataKey", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetParallelPortResponse:
		return ((_vbox__IMachine_USCOREgetParallelPortResponse *)ptr)->soap_out(soap, "vbox:IMachine_getParallelPortResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetParallelPort:
		return ((_vbox__IMachine_USCOREgetParallelPort *)ptr)->soap_out(soap, "vbox:IMachine_getParallelPort", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSerialPortResponse:
		return ((_vbox__IMachine_USCOREgetSerialPortResponse *)ptr)->soap_out(soap, "vbox:IMachine_getSerialPortResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSerialPort:
		return ((_vbox__IMachine_USCOREgetSerialPort *)ptr)->soap_out(soap, "vbox:IMachine_getSerialPort", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREremoveStorageControllerResponse:
		return ((_vbox__IMachine_USCOREremoveStorageControllerResponse *)ptr)->soap_out(soap, "vbox:IMachine_removeStorageControllerResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREremoveStorageController:
		return ((_vbox__IMachine_USCOREremoveStorageController *)ptr)->soap_out(soap, "vbox:IMachine_removeStorageController", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetStorageControllerByNameResponse:
		return ((_vbox__IMachine_USCOREgetStorageControllerByNameResponse *)ptr)->soap_out(soap, "vbox:IMachine_getStorageControllerByNameResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetStorageControllerByName:
		return ((_vbox__IMachine_USCOREgetStorageControllerByName *)ptr)->soap_out(soap, "vbox:IMachine_getStorageControllerByName", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREaddStorageControllerResponse:
		return ((_vbox__IMachine_USCOREaddStorageControllerResponse *)ptr)->soap_out(soap, "vbox:IMachine_addStorageControllerResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREaddStorageController:
		return ((_vbox__IMachine_USCOREaddStorageController *)ptr)->soap_out(soap, "vbox:IMachine_addStorageController", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetNetworkAdapterResponse:
		return ((_vbox__IMachine_USCOREgetNetworkAdapterResponse *)ptr)->soap_out(soap, "vbox:IMachine_getNetworkAdapterResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetNetworkAdapter:
		return ((_vbox__IMachine_USCOREgetNetworkAdapter *)ptr)->soap_out(soap, "vbox:IMachine_getNetworkAdapter", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHardDiskAttachmentsOfControllerResponse:
		return ((_vbox__IMachine_USCOREgetHardDiskAttachmentsOfControllerResponse *)ptr)->soap_out(soap, "vbox:IMachine_getHardDiskAttachmentsOfControllerResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHardDiskAttachmentsOfController:
		return ((_vbox__IMachine_USCOREgetHardDiskAttachmentsOfController *)ptr)->soap_out(soap, "vbox:IMachine_getHardDiskAttachmentsOfController", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREdetachHardDiskResponse:
		return ((_vbox__IMachine_USCOREdetachHardDiskResponse *)ptr)->soap_out(soap, "vbox:IMachine_detachHardDiskResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREdetachHardDisk:
		return ((_vbox__IMachine_USCOREdetachHardDisk *)ptr)->soap_out(soap, "vbox:IMachine_detachHardDisk", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHardDiskResponse:
		return ((_vbox__IMachine_USCOREgetHardDiskResponse *)ptr)->soap_out(soap, "vbox:IMachine_getHardDiskResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHardDisk:
		return ((_vbox__IMachine_USCOREgetHardDisk *)ptr)->soap_out(soap, "vbox:IMachine_getHardDisk", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREattachHardDiskResponse:
		return ((_vbox__IMachine_USCOREattachHardDiskResponse *)ptr)->soap_out(soap, "vbox:IMachine_attachHardDiskResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREattachHardDisk:
		return ((_vbox__IMachine_USCOREattachHardDisk *)ptr)->soap_out(soap, "vbox:IMachine_attachHardDisk", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetBootOrderResponse:
		return ((_vbox__IMachine_USCOREgetBootOrderResponse *)ptr)->soap_out(soap, "vbox:IMachine_getBootOrderResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetBootOrder:
		return ((_vbox__IMachine_USCOREgetBootOrder *)ptr)->soap_out(soap, "vbox:IMachine_getBootOrder", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetBootOrderResponse:
		return ((_vbox__IMachine_USCOREsetBootOrderResponse *)ptr)->soap_out(soap, "vbox:IMachine_setBootOrderResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetBootOrder:
		return ((_vbox__IMachine_USCOREsetBootOrder *)ptr)->soap_out(soap, "vbox:IMachine_setBootOrder", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetGuestPropertyNotificationPatternsResponse:
		return ((_vbox__IMachine_USCOREsetGuestPropertyNotificationPatternsResponse *)ptr)->soap_out(soap, "vbox:IMachine_setGuestPropertyNotificationPatternsResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetGuestPropertyNotificationPatterns:
		return ((_vbox__IMachine_USCOREsetGuestPropertyNotificationPatterns *)ptr)->soap_out(soap, "vbox:IMachine_setGuestPropertyNotificationPatterns", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetGuestPropertyNotificationPatternsResponse:
		return ((_vbox__IMachine_USCOREgetGuestPropertyNotificationPatternsResponse *)ptr)->soap_out(soap, "vbox:IMachine_getGuestPropertyNotificationPatternsResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetGuestPropertyNotificationPatterns:
		return ((_vbox__IMachine_USCOREgetGuestPropertyNotificationPatterns *)ptr)->soap_out(soap, "vbox:IMachine_getGuestPropertyNotificationPatterns", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetClipboardModeResponse:
		return ((_vbox__IMachine_USCOREsetClipboardModeResponse *)ptr)->soap_out(soap, "vbox:IMachine_setClipboardModeResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetClipboardMode:
		return ((_vbox__IMachine_USCOREsetClipboardMode *)ptr)->soap_out(soap, "vbox:IMachine_setClipboardMode", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetClipboardModeResponse:
		return ((_vbox__IMachine_USCOREgetClipboardModeResponse *)ptr)->soap_out(soap, "vbox:IMachine_getClipboardModeResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetClipboardMode:
		return ((_vbox__IMachine_USCOREgetClipboardMode *)ptr)->soap_out(soap, "vbox:IMachine_getClipboardMode", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSharedFoldersResponse:
		return ((_vbox__IMachine_USCOREgetSharedFoldersResponse *)ptr)->soap_out(soap, "vbox:IMachine_getSharedFoldersResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSharedFolders:
		return ((_vbox__IMachine_USCOREgetSharedFolders *)ptr)->soap_out(soap, "vbox:IMachine_getSharedFolders", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetCurrentStateModifiedResponse:
		return ((_vbox__IMachine_USCOREgetCurrentStateModifiedResponse *)ptr)->soap_out(soap, "vbox:IMachine_getCurrentStateModifiedResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetCurrentStateModified:
		return ((_vbox__IMachine_USCOREgetCurrentStateModified *)ptr)->soap_out(soap, "vbox:IMachine_getCurrentStateModified", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSnapshotCountResponse:
		return ((_vbox__IMachine_USCOREgetSnapshotCountResponse *)ptr)->soap_out(soap, "vbox:IMachine_getSnapshotCountResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSnapshotCount:
		return ((_vbox__IMachine_USCOREgetSnapshotCount *)ptr)->soap_out(soap, "vbox:IMachine_getSnapshotCount", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetCurrentSnapshotResponse:
		return ((_vbox__IMachine_USCOREgetCurrentSnapshotResponse *)ptr)->soap_out(soap, "vbox:IMachine_getCurrentSnapshotResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetCurrentSnapshot:
		return ((_vbox__IMachine_USCOREgetCurrentSnapshot *)ptr)->soap_out(soap, "vbox:IMachine_getCurrentSnapshot", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetLogFolderResponse:
		return ((_vbox__IMachine_USCOREgetLogFolderResponse *)ptr)->soap_out(soap, "vbox:IMachine_getLogFolderResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetLogFolder:
		return ((_vbox__IMachine_USCOREgetLogFolder *)ptr)->soap_out(soap, "vbox:IMachine_getLogFolder", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetStateFilePathResponse:
		return ((_vbox__IMachine_USCOREgetStateFilePathResponse *)ptr)->soap_out(soap, "vbox:IMachine_getStateFilePathResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetStateFilePath:
		return ((_vbox__IMachine_USCOREgetStateFilePath *)ptr)->soap_out(soap, "vbox:IMachine_getStateFilePath", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetLastStateChangeResponse:
		return ((_vbox__IMachine_USCOREgetLastStateChangeResponse *)ptr)->soap_out(soap, "vbox:IMachine_getLastStateChangeResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetLastStateChange:
		return ((_vbox__IMachine_USCOREgetLastStateChange *)ptr)->soap_out(soap, "vbox:IMachine_getLastStateChange", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetStateResponse:
		return ((_vbox__IMachine_USCOREgetStateResponse *)ptr)->soap_out(soap, "vbox:IMachine_getStateResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetState:
		return ((_vbox__IMachine_USCOREgetState *)ptr)->soap_out(soap, "vbox:IMachine_getState", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSessionPidResponse:
		return ((_vbox__IMachine_USCOREgetSessionPidResponse *)ptr)->soap_out(soap, "vbox:IMachine_getSessionPidResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSessionPid:
		return ((_vbox__IMachine_USCOREgetSessionPid *)ptr)->soap_out(soap, "vbox:IMachine_getSessionPid", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSessionTypeResponse:
		return ((_vbox__IMachine_USCOREgetSessionTypeResponse *)ptr)->soap_out(soap, "vbox:IMachine_getSessionTypeResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSessionType:
		return ((_vbox__IMachine_USCOREgetSessionType *)ptr)->soap_out(soap, "vbox:IMachine_getSessionType", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSessionStateResponse:
		return ((_vbox__IMachine_USCOREgetSessionStateResponse *)ptr)->soap_out(soap, "vbox:IMachine_getSessionStateResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSessionState:
		return ((_vbox__IMachine_USCOREgetSessionState *)ptr)->soap_out(soap, "vbox:IMachine_getSessionState", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSettingsModifiedResponse:
		return ((_vbox__IMachine_USCOREgetSettingsModifiedResponse *)ptr)->soap_out(soap, "vbox:IMachine_getSettingsModifiedResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSettingsModified:
		return ((_vbox__IMachine_USCOREgetSettingsModified *)ptr)->soap_out(soap, "vbox:IMachine_getSettingsModified", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSettingsFileVersionResponse:
		return ((_vbox__IMachine_USCOREgetSettingsFileVersionResponse *)ptr)->soap_out(soap, "vbox:IMachine_getSettingsFileVersionResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSettingsFileVersion:
		return ((_vbox__IMachine_USCOREgetSettingsFileVersion *)ptr)->soap_out(soap, "vbox:IMachine_getSettingsFileVersion", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSettingsFilePathResponse:
		return ((_vbox__IMachine_USCOREgetSettingsFilePathResponse *)ptr)->soap_out(soap, "vbox:IMachine_getSettingsFilePathResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSettingsFilePath:
		return ((_vbox__IMachine_USCOREgetSettingsFilePath *)ptr)->soap_out(soap, "vbox:IMachine_getSettingsFilePath", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetStorageControllersResponse:
		return ((_vbox__IMachine_USCOREgetStorageControllersResponse *)ptr)->soap_out(soap, "vbox:IMachine_getStorageControllersResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetStorageControllers:
		return ((_vbox__IMachine_USCOREgetStorageControllers *)ptr)->soap_out(soap, "vbox:IMachine_getStorageControllers", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetAudioAdapterResponse:
		return ((_vbox__IMachine_USCOREgetAudioAdapterResponse *)ptr)->soap_out(soap, "vbox:IMachine_getAudioAdapterResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetAudioAdapter:
		return ((_vbox__IMachine_USCOREgetAudioAdapter *)ptr)->soap_out(soap, "vbox:IMachine_getAudioAdapter", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetUSBControllerResponse:
		return ((_vbox__IMachine_USCOREgetUSBControllerResponse *)ptr)->soap_out(soap, "vbox:IMachine_getUSBControllerResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetUSBController:
		return ((_vbox__IMachine_USCOREgetUSBController *)ptr)->soap_out(soap, "vbox:IMachine_getUSBController", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetFloppyDriveResponse:
		return ((_vbox__IMachine_USCOREgetFloppyDriveResponse *)ptr)->soap_out(soap, "vbox:IMachine_getFloppyDriveResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetFloppyDrive:
		return ((_vbox__IMachine_USCOREgetFloppyDrive *)ptr)->soap_out(soap, "vbox:IMachine_getFloppyDrive", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetDVDDriveResponse:
		return ((_vbox__IMachine_USCOREgetDVDDriveResponse *)ptr)->soap_out(soap, "vbox:IMachine_getDVDDriveResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetDVDDrive:
		return ((_vbox__IMachine_USCOREgetDVDDrive *)ptr)->soap_out(soap, "vbox:IMachine_getDVDDrive", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHardDiskAttachmentsResponse:
		return ((_vbox__IMachine_USCOREgetHardDiskAttachmentsResponse *)ptr)->soap_out(soap, "vbox:IMachine_getHardDiskAttachmentsResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHardDiskAttachments:
		return ((_vbox__IMachine_USCOREgetHardDiskAttachments *)ptr)->soap_out(soap, "vbox:IMachine_getHardDiskAttachments", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetVRDPServerResponse:
		return ((_vbox__IMachine_USCOREgetVRDPServerResponse *)ptr)->soap_out(soap, "vbox:IMachine_getVRDPServerResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetVRDPServer:
		return ((_vbox__IMachine_USCOREgetVRDPServer *)ptr)->soap_out(soap, "vbox:IMachine_getVRDPServer", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetSnapshotFolderResponse:
		return ((_vbox__IMachine_USCOREsetSnapshotFolderResponse *)ptr)->soap_out(soap, "vbox:IMachine_setSnapshotFolderResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetSnapshotFolder:
		return ((_vbox__IMachine_USCOREsetSnapshotFolder *)ptr)->soap_out(soap, "vbox:IMachine_setSnapshotFolder", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSnapshotFolderResponse:
		return ((_vbox__IMachine_USCOREgetSnapshotFolderResponse *)ptr)->soap_out(soap, "vbox:IMachine_getSnapshotFolderResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSnapshotFolder:
		return ((_vbox__IMachine_USCOREgetSnapshotFolder *)ptr)->soap_out(soap, "vbox:IMachine_getSnapshotFolder", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetPAEEnabledResponse:
		return ((_vbox__IMachine_USCOREsetPAEEnabledResponse *)ptr)->soap_out(soap, "vbox:IMachine_setPAEEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetPAEEnabled:
		return ((_vbox__IMachine_USCOREsetPAEEnabled *)ptr)->soap_out(soap, "vbox:IMachine_setPAEEnabled", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetPAEEnabledResponse:
		return ((_vbox__IMachine_USCOREgetPAEEnabledResponse *)ptr)->soap_out(soap, "vbox:IMachine_getPAEEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetPAEEnabled:
		return ((_vbox__IMachine_USCOREgetPAEEnabled *)ptr)->soap_out(soap, "vbox:IMachine_getPAEEnabled", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetHWVirtExVPIDEnabledResponse:
		return ((_vbox__IMachine_USCOREsetHWVirtExVPIDEnabledResponse *)ptr)->soap_out(soap, "vbox:IMachine_setHWVirtExVPIDEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetHWVirtExVPIDEnabled:
		return ((_vbox__IMachine_USCOREsetHWVirtExVPIDEnabled *)ptr)->soap_out(soap, "vbox:IMachine_setHWVirtExVPIDEnabled", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHWVirtExVPIDEnabledResponse:
		return ((_vbox__IMachine_USCOREgetHWVirtExVPIDEnabledResponse *)ptr)->soap_out(soap, "vbox:IMachine_getHWVirtExVPIDEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHWVirtExVPIDEnabled:
		return ((_vbox__IMachine_USCOREgetHWVirtExVPIDEnabled *)ptr)->soap_out(soap, "vbox:IMachine_getHWVirtExVPIDEnabled", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabledResponse:
		return ((_vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabledResponse *)ptr)->soap_out(soap, "vbox:IMachine_setHWVirtExNestedPagingEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabled:
		return ((_vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabled *)ptr)->soap_out(soap, "vbox:IMachine_setHWVirtExNestedPagingEnabled", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabledResponse:
		return ((_vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabledResponse *)ptr)->soap_out(soap, "vbox:IMachine_getHWVirtExNestedPagingEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabled:
		return ((_vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabled *)ptr)->soap_out(soap, "vbox:IMachine_getHWVirtExNestedPagingEnabled", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetHWVirtExEnabledResponse:
		return ((_vbox__IMachine_USCOREsetHWVirtExEnabledResponse *)ptr)->soap_out(soap, "vbox:IMachine_setHWVirtExEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetHWVirtExEnabled:
		return ((_vbox__IMachine_USCOREsetHWVirtExEnabled *)ptr)->soap_out(soap, "vbox:IMachine_setHWVirtExEnabled", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHWVirtExEnabledResponse:
		return ((_vbox__IMachine_USCOREgetHWVirtExEnabledResponse *)ptr)->soap_out(soap, "vbox:IMachine_getHWVirtExEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHWVirtExEnabled:
		return ((_vbox__IMachine_USCOREgetHWVirtExEnabled *)ptr)->soap_out(soap, "vbox:IMachine_getHWVirtExEnabled", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetBIOSSettingsResponse:
		return ((_vbox__IMachine_USCOREgetBIOSSettingsResponse *)ptr)->soap_out(soap, "vbox:IMachine_getBIOSSettingsResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetBIOSSettings:
		return ((_vbox__IMachine_USCOREgetBIOSSettings *)ptr)->soap_out(soap, "vbox:IMachine_getBIOSSettings", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetMonitorCountResponse:
		return ((_vbox__IMachine_USCOREsetMonitorCountResponse *)ptr)->soap_out(soap, "vbox:IMachine_setMonitorCountResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetMonitorCount:
		return ((_vbox__IMachine_USCOREsetMonitorCount *)ptr)->soap_out(soap, "vbox:IMachine_setMonitorCount", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetMonitorCountResponse:
		return ((_vbox__IMachine_USCOREgetMonitorCountResponse *)ptr)->soap_out(soap, "vbox:IMachine_getMonitorCountResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetMonitorCount:
		return ((_vbox__IMachine_USCOREgetMonitorCount *)ptr)->soap_out(soap, "vbox:IMachine_getMonitorCount", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetAccelerate3DEnabledResponse:
		return ((_vbox__IMachine_USCOREsetAccelerate3DEnabledResponse *)ptr)->soap_out(soap, "vbox:IMachine_setAccelerate3DEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetAccelerate3DEnabled:
		return ((_vbox__IMachine_USCOREsetAccelerate3DEnabled *)ptr)->soap_out(soap, "vbox:IMachine_setAccelerate3DEnabled", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetAccelerate3DEnabledResponse:
		return ((_vbox__IMachine_USCOREgetAccelerate3DEnabledResponse *)ptr)->soap_out(soap, "vbox:IMachine_getAccelerate3DEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetAccelerate3DEnabled:
		return ((_vbox__IMachine_USCOREgetAccelerate3DEnabled *)ptr)->soap_out(soap, "vbox:IMachine_getAccelerate3DEnabled", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetVRAMSizeResponse:
		return ((_vbox__IMachine_USCOREsetVRAMSizeResponse *)ptr)->soap_out(soap, "vbox:IMachine_setVRAMSizeResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetVRAMSize:
		return ((_vbox__IMachine_USCOREsetVRAMSize *)ptr)->soap_out(soap, "vbox:IMachine_setVRAMSize", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetVRAMSizeResponse:
		return ((_vbox__IMachine_USCOREgetVRAMSizeResponse *)ptr)->soap_out(soap, "vbox:IMachine_getVRAMSizeResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetVRAMSize:
		return ((_vbox__IMachine_USCOREgetVRAMSize *)ptr)->soap_out(soap, "vbox:IMachine_getVRAMSize", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetStatisticsUpdateIntervalResponse:
		return ((_vbox__IMachine_USCOREsetStatisticsUpdateIntervalResponse *)ptr)->soap_out(soap, "vbox:IMachine_setStatisticsUpdateIntervalResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetStatisticsUpdateInterval:
		return ((_vbox__IMachine_USCOREsetStatisticsUpdateInterval *)ptr)->soap_out(soap, "vbox:IMachine_setStatisticsUpdateInterval", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetStatisticsUpdateIntervalResponse:
		return ((_vbox__IMachine_USCOREgetStatisticsUpdateIntervalResponse *)ptr)->soap_out(soap, "vbox:IMachine_getStatisticsUpdateIntervalResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetStatisticsUpdateInterval:
		return ((_vbox__IMachine_USCOREgetStatisticsUpdateInterval *)ptr)->soap_out(soap, "vbox:IMachine_getStatisticsUpdateInterval", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetMemoryBalloonSizeResponse:
		return ((_vbox__IMachine_USCOREsetMemoryBalloonSizeResponse *)ptr)->soap_out(soap, "vbox:IMachine_setMemoryBalloonSizeResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetMemoryBalloonSize:
		return ((_vbox__IMachine_USCOREsetMemoryBalloonSize *)ptr)->soap_out(soap, "vbox:IMachine_setMemoryBalloonSize", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetMemoryBalloonSizeResponse:
		return ((_vbox__IMachine_USCOREgetMemoryBalloonSizeResponse *)ptr)->soap_out(soap, "vbox:IMachine_getMemoryBalloonSizeResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetMemoryBalloonSize:
		return ((_vbox__IMachine_USCOREgetMemoryBalloonSize *)ptr)->soap_out(soap, "vbox:IMachine_getMemoryBalloonSize", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetMemorySizeResponse:
		return ((_vbox__IMachine_USCOREsetMemorySizeResponse *)ptr)->soap_out(soap, "vbox:IMachine_setMemorySizeResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetMemorySize:
		return ((_vbox__IMachine_USCOREsetMemorySize *)ptr)->soap_out(soap, "vbox:IMachine_setMemorySize", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetMemorySizeResponse:
		return ((_vbox__IMachine_USCOREgetMemorySizeResponse *)ptr)->soap_out(soap, "vbox:IMachine_getMemorySizeResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetMemorySize:
		return ((_vbox__IMachine_USCOREgetMemorySize *)ptr)->soap_out(soap, "vbox:IMachine_getMemorySize", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetCPUCountResponse:
		return ((_vbox__IMachine_USCOREsetCPUCountResponse *)ptr)->soap_out(soap, "vbox:IMachine_setCPUCountResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetCPUCount:
		return ((_vbox__IMachine_USCOREsetCPUCount *)ptr)->soap_out(soap, "vbox:IMachine_setCPUCount", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetCPUCountResponse:
		return ((_vbox__IMachine_USCOREgetCPUCountResponse *)ptr)->soap_out(soap, "vbox:IMachine_getCPUCountResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetCPUCount:
		return ((_vbox__IMachine_USCOREgetCPUCount *)ptr)->soap_out(soap, "vbox:IMachine_getCPUCount", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetHardwareVersionResponse:
		return ((_vbox__IMachine_USCOREsetHardwareVersionResponse *)ptr)->soap_out(soap, "vbox:IMachine_setHardwareVersionResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetHardwareVersion:
		return ((_vbox__IMachine_USCOREsetHardwareVersion *)ptr)->soap_out(soap, "vbox:IMachine_setHardwareVersion", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHardwareVersionResponse:
		return ((_vbox__IMachine_USCOREgetHardwareVersionResponse *)ptr)->soap_out(soap, "vbox:IMachine_getHardwareVersionResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHardwareVersion:
		return ((_vbox__IMachine_USCOREgetHardwareVersion *)ptr)->soap_out(soap, "vbox:IMachine_getHardwareVersion", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetOSTypeIdResponse:
		return ((_vbox__IMachine_USCOREsetOSTypeIdResponse *)ptr)->soap_out(soap, "vbox:IMachine_setOSTypeIdResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetOSTypeId:
		return ((_vbox__IMachine_USCOREsetOSTypeId *)ptr)->soap_out(soap, "vbox:IMachine_setOSTypeId", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetOSTypeIdResponse:
		return ((_vbox__IMachine_USCOREgetOSTypeIdResponse *)ptr)->soap_out(soap, "vbox:IMachine_getOSTypeIdResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetOSTypeId:
		return ((_vbox__IMachine_USCOREgetOSTypeId *)ptr)->soap_out(soap, "vbox:IMachine_getOSTypeId", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetIdResponse:
		return ((_vbox__IMachine_USCOREgetIdResponse *)ptr)->soap_out(soap, "vbox:IMachine_getIdResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetId:
		return ((_vbox__IMachine_USCOREgetId *)ptr)->soap_out(soap, "vbox:IMachine_getId", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetDescriptionResponse:
		return ((_vbox__IMachine_USCOREsetDescriptionResponse *)ptr)->soap_out(soap, "vbox:IMachine_setDescriptionResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetDescription:
		return ((_vbox__IMachine_USCOREsetDescription *)ptr)->soap_out(soap, "vbox:IMachine_setDescription", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetDescriptionResponse:
		return ((_vbox__IMachine_USCOREgetDescriptionResponse *)ptr)->soap_out(soap, "vbox:IMachine_getDescriptionResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetDescription:
		return ((_vbox__IMachine_USCOREgetDescription *)ptr)->soap_out(soap, "vbox:IMachine_getDescription", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetNameResponse:
		return ((_vbox__IMachine_USCOREsetNameResponse *)ptr)->soap_out(soap, "vbox:IMachine_setNameResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREsetName:
		return ((_vbox__IMachine_USCOREsetName *)ptr)->soap_out(soap, "vbox:IMachine_setName", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetNameResponse:
		return ((_vbox__IMachine_USCOREgetNameResponse *)ptr)->soap_out(soap, "vbox:IMachine_getNameResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetName:
		return ((_vbox__IMachine_USCOREgetName *)ptr)->soap_out(soap, "vbox:IMachine_getName", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetAccessErrorResponse:
		return ((_vbox__IMachine_USCOREgetAccessErrorResponse *)ptr)->soap_out(soap, "vbox:IMachine_getAccessErrorResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetAccessError:
		return ((_vbox__IMachine_USCOREgetAccessError *)ptr)->soap_out(soap, "vbox:IMachine_getAccessError", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetAccessibleResponse:
		return ((_vbox__IMachine_USCOREgetAccessibleResponse *)ptr)->soap_out(soap, "vbox:IMachine_getAccessibleResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetAccessible:
		return ((_vbox__IMachine_USCOREgetAccessible *)ptr)->soap_out(soap, "vbox:IMachine_getAccessible", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetParentResponse:
		return ((_vbox__IMachine_USCOREgetParentResponse *)ptr)->soap_out(soap, "vbox:IMachine_getParentResponse", id, NULL);
	case SOAP_TYPE__vbox__IMachine_USCOREgetParent:
		return ((_vbox__IMachine_USCOREgetParent *)ptr)->soap_out(soap, "vbox:IMachine_getParent", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetPXEDebugEnabledResponse:
		return ((_vbox__IBIOSSettings_USCOREsetPXEDebugEnabledResponse *)ptr)->soap_out(soap, "vbox:IBIOSSettings_setPXEDebugEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetPXEDebugEnabled:
		return ((_vbox__IBIOSSettings_USCOREsetPXEDebugEnabled *)ptr)->soap_out(soap, "vbox:IBIOSSettings_setPXEDebugEnabled", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetPXEDebugEnabledResponse:
		return ((_vbox__IBIOSSettings_USCOREgetPXEDebugEnabledResponse *)ptr)->soap_out(soap, "vbox:IBIOSSettings_getPXEDebugEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetPXEDebugEnabled:
		return ((_vbox__IBIOSSettings_USCOREgetPXEDebugEnabled *)ptr)->soap_out(soap, "vbox:IBIOSSettings_getPXEDebugEnabled", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetTimeOffsetResponse:
		return ((_vbox__IBIOSSettings_USCOREsetTimeOffsetResponse *)ptr)->soap_out(soap, "vbox:IBIOSSettings_setTimeOffsetResponse", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetTimeOffset:
		return ((_vbox__IBIOSSettings_USCOREsetTimeOffset *)ptr)->soap_out(soap, "vbox:IBIOSSettings_setTimeOffset", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetTimeOffsetResponse:
		return ((_vbox__IBIOSSettings_USCOREgetTimeOffsetResponse *)ptr)->soap_out(soap, "vbox:IBIOSSettings_getTimeOffsetResponse", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetTimeOffset:
		return ((_vbox__IBIOSSettings_USCOREgetTimeOffset *)ptr)->soap_out(soap, "vbox:IBIOSSettings_getTimeOffset", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetIOAPICEnabledResponse:
		return ((_vbox__IBIOSSettings_USCOREsetIOAPICEnabledResponse *)ptr)->soap_out(soap, "vbox:IBIOSSettings_setIOAPICEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetIOAPICEnabled:
		return ((_vbox__IBIOSSettings_USCOREsetIOAPICEnabled *)ptr)->soap_out(soap, "vbox:IBIOSSettings_setIOAPICEnabled", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetIOAPICEnabledResponse:
		return ((_vbox__IBIOSSettings_USCOREgetIOAPICEnabledResponse *)ptr)->soap_out(soap, "vbox:IBIOSSettings_getIOAPICEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetIOAPICEnabled:
		return ((_vbox__IBIOSSettings_USCOREgetIOAPICEnabled *)ptr)->soap_out(soap, "vbox:IBIOSSettings_getIOAPICEnabled", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetACPIEnabledResponse:
		return ((_vbox__IBIOSSettings_USCOREsetACPIEnabledResponse *)ptr)->soap_out(soap, "vbox:IBIOSSettings_setACPIEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetACPIEnabled:
		return ((_vbox__IBIOSSettings_USCOREsetACPIEnabled *)ptr)->soap_out(soap, "vbox:IBIOSSettings_setACPIEnabled", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetACPIEnabledResponse:
		return ((_vbox__IBIOSSettings_USCOREgetACPIEnabledResponse *)ptr)->soap_out(soap, "vbox:IBIOSSettings_getACPIEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetACPIEnabled:
		return ((_vbox__IBIOSSettings_USCOREgetACPIEnabled *)ptr)->soap_out(soap, "vbox:IBIOSSettings_getACPIEnabled", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetBootMenuModeResponse:
		return ((_vbox__IBIOSSettings_USCOREsetBootMenuModeResponse *)ptr)->soap_out(soap, "vbox:IBIOSSettings_setBootMenuModeResponse", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetBootMenuMode:
		return ((_vbox__IBIOSSettings_USCOREsetBootMenuMode *)ptr)->soap_out(soap, "vbox:IBIOSSettings_setBootMenuMode", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetBootMenuModeResponse:
		return ((_vbox__IBIOSSettings_USCOREgetBootMenuModeResponse *)ptr)->soap_out(soap, "vbox:IBIOSSettings_getBootMenuModeResponse", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetBootMenuMode:
		return ((_vbox__IBIOSSettings_USCOREgetBootMenuMode *)ptr)->soap_out(soap, "vbox:IBIOSSettings_getBootMenuMode", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetLogoImagePathResponse:
		return ((_vbox__IBIOSSettings_USCOREsetLogoImagePathResponse *)ptr)->soap_out(soap, "vbox:IBIOSSettings_setLogoImagePathResponse", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetLogoImagePath:
		return ((_vbox__IBIOSSettings_USCOREsetLogoImagePath *)ptr)->soap_out(soap, "vbox:IBIOSSettings_setLogoImagePath", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetLogoImagePathResponse:
		return ((_vbox__IBIOSSettings_USCOREgetLogoImagePathResponse *)ptr)->soap_out(soap, "vbox:IBIOSSettings_getLogoImagePathResponse", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetLogoImagePath:
		return ((_vbox__IBIOSSettings_USCOREgetLogoImagePath *)ptr)->soap_out(soap, "vbox:IBIOSSettings_getLogoImagePath", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetLogoDisplayTimeResponse:
		return ((_vbox__IBIOSSettings_USCOREsetLogoDisplayTimeResponse *)ptr)->soap_out(soap, "vbox:IBIOSSettings_setLogoDisplayTimeResponse", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetLogoDisplayTime:
		return ((_vbox__IBIOSSettings_USCOREsetLogoDisplayTime *)ptr)->soap_out(soap, "vbox:IBIOSSettings_setLogoDisplayTime", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetLogoDisplayTimeResponse:
		return ((_vbox__IBIOSSettings_USCOREgetLogoDisplayTimeResponse *)ptr)->soap_out(soap, "vbox:IBIOSSettings_getLogoDisplayTimeResponse", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetLogoDisplayTime:
		return ((_vbox__IBIOSSettings_USCOREgetLogoDisplayTime *)ptr)->soap_out(soap, "vbox:IBIOSSettings_getLogoDisplayTime", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetLogoFadeOutResponse:
		return ((_vbox__IBIOSSettings_USCOREsetLogoFadeOutResponse *)ptr)->soap_out(soap, "vbox:IBIOSSettings_setLogoFadeOutResponse", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetLogoFadeOut:
		return ((_vbox__IBIOSSettings_USCOREsetLogoFadeOut *)ptr)->soap_out(soap, "vbox:IBIOSSettings_setLogoFadeOut", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetLogoFadeOutResponse:
		return ((_vbox__IBIOSSettings_USCOREgetLogoFadeOutResponse *)ptr)->soap_out(soap, "vbox:IBIOSSettings_getLogoFadeOutResponse", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetLogoFadeOut:
		return ((_vbox__IBIOSSettings_USCOREgetLogoFadeOut *)ptr)->soap_out(soap, "vbox:IBIOSSettings_getLogoFadeOut", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetLogoFadeInResponse:
		return ((_vbox__IBIOSSettings_USCOREsetLogoFadeInResponse *)ptr)->soap_out(soap, "vbox:IBIOSSettings_setLogoFadeInResponse", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetLogoFadeIn:
		return ((_vbox__IBIOSSettings_USCOREsetLogoFadeIn *)ptr)->soap_out(soap, "vbox:IBIOSSettings_setLogoFadeIn", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetLogoFadeInResponse:
		return ((_vbox__IBIOSSettings_USCOREgetLogoFadeInResponse *)ptr)->soap_out(soap, "vbox:IBIOSSettings_getLogoFadeInResponse", id, NULL);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetLogoFadeIn:
		return ((_vbox__IBIOSSettings_USCOREgetLogoFadeIn *)ptr)->soap_out(soap, "vbox:IBIOSSettings_getLogoFadeIn", id, NULL);
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREaddDescriptionResponse:
		return ((_vbox__IVirtualSystemDescription_USCOREaddDescriptionResponse *)ptr)->soap_out(soap, "vbox:IVirtualSystemDescription_addDescriptionResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREaddDescription:
		return ((_vbox__IVirtualSystemDescription_USCOREaddDescription *)ptr)->soap_out(soap, "vbox:IVirtualSystemDescription_addDescription", id, NULL);
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREsetFinalValuesResponse:
		return ((_vbox__IVirtualSystemDescription_USCOREsetFinalValuesResponse *)ptr)->soap_out(soap, "vbox:IVirtualSystemDescription_setFinalValuesResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREsetFinalValues:
		return ((_vbox__IVirtualSystemDescription_USCOREsetFinalValues *)ptr)->soap_out(soap, "vbox:IVirtualSystemDescription_setFinalValues", id, NULL);
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREgetValuesByTypeResponse:
		return ((_vbox__IVirtualSystemDescription_USCOREgetValuesByTypeResponse *)ptr)->soap_out(soap, "vbox:IVirtualSystemDescription_getValuesByTypeResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREgetValuesByType:
		return ((_vbox__IVirtualSystemDescription_USCOREgetValuesByType *)ptr)->soap_out(soap, "vbox:IVirtualSystemDescription_getValuesByType", id, NULL);
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREgetDescriptionByTypeResponse:
		return ((_vbox__IVirtualSystemDescription_USCOREgetDescriptionByTypeResponse *)ptr)->soap_out(soap, "vbox:IVirtualSystemDescription_getDescriptionByTypeResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREgetDescriptionByType:
		return ((_vbox__IVirtualSystemDescription_USCOREgetDescriptionByType *)ptr)->soap_out(soap, "vbox:IVirtualSystemDescription_getDescriptionByType", id, NULL);
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREgetDescriptionResponse:
		return ((_vbox__IVirtualSystemDescription_USCOREgetDescriptionResponse *)ptr)->soap_out(soap, "vbox:IVirtualSystemDescription_getDescriptionResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREgetDescription:
		return ((_vbox__IVirtualSystemDescription_USCOREgetDescription *)ptr)->soap_out(soap, "vbox:IVirtualSystemDescription_getDescription", id, NULL);
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREgetCountResponse:
		return ((_vbox__IVirtualSystemDescription_USCOREgetCountResponse *)ptr)->soap_out(soap, "vbox:IVirtualSystemDescription_getCountResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREgetCount:
		return ((_vbox__IVirtualSystemDescription_USCOREgetCount *)ptr)->soap_out(soap, "vbox:IVirtualSystemDescription_getCount", id, NULL);
	case SOAP_TYPE__vbox__IAppliance_USCOREgetWarningsResponse:
		return ((_vbox__IAppliance_USCOREgetWarningsResponse *)ptr)->soap_out(soap, "vbox:IAppliance_getWarningsResponse", id, NULL);
	case SOAP_TYPE__vbox__IAppliance_USCOREgetWarnings:
		return ((_vbox__IAppliance_USCOREgetWarnings *)ptr)->soap_out(soap, "vbox:IAppliance_getWarnings", id, NULL);
	case SOAP_TYPE__vbox__IAppliance_USCOREwriteResponse:
		return ((_vbox__IAppliance_USCOREwriteResponse *)ptr)->soap_out(soap, "vbox:IAppliance_writeResponse", id, NULL);
	case SOAP_TYPE__vbox__IAppliance_USCOREwrite:
		return ((_vbox__IAppliance_USCOREwrite *)ptr)->soap_out(soap, "vbox:IAppliance_write", id, NULL);
	case SOAP_TYPE__vbox__IAppliance_USCOREcreateVFSExplorerResponse:
		return ((_vbox__IAppliance_USCOREcreateVFSExplorerResponse *)ptr)->soap_out(soap, "vbox:IAppliance_createVFSExplorerResponse", id, NULL);
	case SOAP_TYPE__vbox__IAppliance_USCOREcreateVFSExplorer:
		return ((_vbox__IAppliance_USCOREcreateVFSExplorer *)ptr)->soap_out(soap, "vbox:IAppliance_createVFSExplorer", id, NULL);
	case SOAP_TYPE__vbox__IAppliance_USCOREimportMachinesResponse:
		return ((_vbox__IAppliance_USCOREimportMachinesResponse *)ptr)->soap_out(soap, "vbox:IAppliance_importMachinesResponse", id, NULL);
	case SOAP_TYPE__vbox__IAppliance_USCOREimportMachines:
		return ((_vbox__IAppliance_USCOREimportMachines *)ptr)->soap_out(soap, "vbox:IAppliance_importMachines", id, NULL);
	case SOAP_TYPE__vbox__IAppliance_USCOREinterpretResponse:
		return ((_vbox__IAppliance_USCOREinterpretResponse *)ptr)->soap_out(soap, "vbox:IAppliance_interpretResponse", id, NULL);
	case SOAP_TYPE__vbox__IAppliance_USCOREinterpret:
		return ((_vbox__IAppliance_USCOREinterpret *)ptr)->soap_out(soap, "vbox:IAppliance_interpret", id, NULL);
	case SOAP_TYPE__vbox__IAppliance_USCOREreadResponse:
		return ((_vbox__IAppliance_USCOREreadResponse *)ptr)->soap_out(soap, "vbox:IAppliance_readResponse", id, NULL);
	case SOAP_TYPE__vbox__IAppliance_USCOREread:
		return ((_vbox__IAppliance_USCOREread *)ptr)->soap_out(soap, "vbox:IAppliance_read", id, NULL);
	case SOAP_TYPE__vbox__IAppliance_USCOREgetVirtualSystemDescriptionsResponse:
		return ((_vbox__IAppliance_USCOREgetVirtualSystemDescriptionsResponse *)ptr)->soap_out(soap, "vbox:IAppliance_getVirtualSystemDescriptionsResponse", id, NULL);
	case SOAP_TYPE__vbox__IAppliance_USCOREgetVirtualSystemDescriptions:
		return ((_vbox__IAppliance_USCOREgetVirtualSystemDescriptions *)ptr)->soap_out(soap, "vbox:IAppliance_getVirtualSystemDescriptions", id, NULL);
	case SOAP_TYPE__vbox__IAppliance_USCOREgetDisksResponse:
		return ((_vbox__IAppliance_USCOREgetDisksResponse *)ptr)->soap_out(soap, "vbox:IAppliance_getDisksResponse", id, NULL);
	case SOAP_TYPE__vbox__IAppliance_USCOREgetDisks:
		return ((_vbox__IAppliance_USCOREgetDisks *)ptr)->soap_out(soap, "vbox:IAppliance_getDisks", id, NULL);
	case SOAP_TYPE__vbox__IAppliance_USCOREgetPathResponse:
		return ((_vbox__IAppliance_USCOREgetPathResponse *)ptr)->soap_out(soap, "vbox:IAppliance_getPathResponse", id, NULL);
	case SOAP_TYPE__vbox__IAppliance_USCOREgetPath:
		return ((_vbox__IAppliance_USCOREgetPath *)ptr)->soap_out(soap, "vbox:IAppliance_getPath", id, NULL);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREremoveResponse:
		return ((_vbox__IVFSExplorer_USCOREremoveResponse *)ptr)->soap_out(soap, "vbox:IVFSExplorer_removeResponse", id, NULL);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREremove:
		return ((_vbox__IVFSExplorer_USCOREremove *)ptr)->soap_out(soap, "vbox:IVFSExplorer_remove", id, NULL);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREexistsResponse:
		return ((_vbox__IVFSExplorer_USCOREexistsResponse *)ptr)->soap_out(soap, "vbox:IVFSExplorer_existsResponse", id, NULL);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREexists:
		return ((_vbox__IVFSExplorer_USCOREexists *)ptr)->soap_out(soap, "vbox:IVFSExplorer_exists", id, NULL);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREentryListResponse:
		return ((_vbox__IVFSExplorer_USCOREentryListResponse *)ptr)->soap_out(soap, "vbox:IVFSExplorer_entryListResponse", id, NULL);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREentryList:
		return ((_vbox__IVFSExplorer_USCOREentryList *)ptr)->soap_out(soap, "vbox:IVFSExplorer_entryList", id, NULL);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREcdUpResponse:
		return ((_vbox__IVFSExplorer_USCOREcdUpResponse *)ptr)->soap_out(soap, "vbox:IVFSExplorer_cdUpResponse", id, NULL);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREcdUp:
		return ((_vbox__IVFSExplorer_USCOREcdUp *)ptr)->soap_out(soap, "vbox:IVFSExplorer_cdUp", id, NULL);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREcdResponse:
		return ((_vbox__IVFSExplorer_USCOREcdResponse *)ptr)->soap_out(soap, "vbox:IVFSExplorer_cdResponse", id, NULL);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREcd:
		return ((_vbox__IVFSExplorer_USCOREcd *)ptr)->soap_out(soap, "vbox:IVFSExplorer_cd", id, NULL);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREupdateResponse:
		return ((_vbox__IVFSExplorer_USCOREupdateResponse *)ptr)->soap_out(soap, "vbox:IVFSExplorer_updateResponse", id, NULL);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREupdate:
		return ((_vbox__IVFSExplorer_USCOREupdate *)ptr)->soap_out(soap, "vbox:IVFSExplorer_update", id, NULL);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREgetTypeResponse:
		return ((_vbox__IVFSExplorer_USCOREgetTypeResponse *)ptr)->soap_out(soap, "vbox:IVFSExplorer_getTypeResponse", id, NULL);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREgetType:
		return ((_vbox__IVFSExplorer_USCOREgetType *)ptr)->soap_out(soap, "vbox:IVFSExplorer_getType", id, NULL);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREgetPathResponse:
		return ((_vbox__IVFSExplorer_USCOREgetPathResponse *)ptr)->soap_out(soap, "vbox:IVFSExplorer_getPathResponse", id, NULL);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREgetPath:
		return ((_vbox__IVFSExplorer_USCOREgetPath *)ptr)->soap_out(soap, "vbox:IVFSExplorer_getPath", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREremoveDHCPServerResponse:
		return ((_vbox__IVirtualBox_USCOREremoveDHCPServerResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_removeDHCPServerResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREremoveDHCPServer:
		return ((_vbox__IVirtualBox_USCOREremoveDHCPServer *)ptr)->soap_out(soap, "vbox:IVirtualBox_removeDHCPServer", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindDHCPServerByNetworkNameResponse:
		return ((_vbox__IVirtualBox_USCOREfindDHCPServerByNetworkNameResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_findDHCPServerByNetworkNameResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindDHCPServerByNetworkName:
		return ((_vbox__IVirtualBox_USCOREfindDHCPServerByNetworkName *)ptr)->soap_out(soap, "vbox:IVirtualBox_findDHCPServerByNetworkName", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateDHCPServerResponse:
		return ((_vbox__IVirtualBox_USCOREcreateDHCPServerResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_createDHCPServerResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateDHCPServer:
		return ((_vbox__IVirtualBox_USCOREcreateDHCPServer *)ptr)->soap_out(soap, "vbox:IVirtualBox_createDHCPServer", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREsaveSettingsWithBackupResponse:
		return ((_vbox__IVirtualBox_USCOREsaveSettingsWithBackupResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_saveSettingsWithBackupResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREsaveSettingsWithBackup:
		return ((_vbox__IVirtualBox_USCOREsaveSettingsWithBackup *)ptr)->soap_out(soap, "vbox:IVirtualBox_saveSettingsWithBackup", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREsaveSettingsResponse:
		return ((_vbox__IVirtualBox_USCOREsaveSettingsResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_saveSettingsResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREsaveSettings:
		return ((_vbox__IVirtualBox_USCOREsaveSettings *)ptr)->soap_out(soap, "vbox:IVirtualBox_saveSettings", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREwaitForPropertyChangeResponse:
		return ((_vbox__IVirtualBox_USCOREwaitForPropertyChangeResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_waitForPropertyChangeResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREwaitForPropertyChange:
		return ((_vbox__IVirtualBox_USCOREwaitForPropertyChange *)ptr)->soap_out(soap, "vbox:IVirtualBox_waitForPropertyChange", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenExistingSessionResponse:
		return ((_vbox__IVirtualBox_USCOREopenExistingSessionResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_openExistingSessionResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenExistingSession:
		return ((_vbox__IVirtualBox_USCOREopenExistingSession *)ptr)->soap_out(soap, "vbox:IVirtualBox_openExistingSession", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenRemoteSessionResponse:
		return ((_vbox__IVirtualBox_USCOREopenRemoteSessionResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_openRemoteSessionResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenRemoteSession:
		return ((_vbox__IVirtualBox_USCOREopenRemoteSession *)ptr)->soap_out(soap, "vbox:IVirtualBox_openRemoteSession", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenSessionResponse:
		return ((_vbox__IVirtualBox_USCOREopenSessionResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_openSessionResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenSession:
		return ((_vbox__IVirtualBox_USCOREopenSession *)ptr)->soap_out(soap, "vbox:IVirtualBox_openSession", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREsetExtraDataResponse:
		return ((_vbox__IVirtualBox_USCOREsetExtraDataResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_setExtraDataResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREsetExtraData:
		return ((_vbox__IVirtualBox_USCOREsetExtraData *)ptr)->soap_out(soap, "vbox:IVirtualBox_setExtraData", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetExtraDataResponse:
		return ((_vbox__IVirtualBox_USCOREgetExtraDataResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getExtraDataResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetExtraData:
		return ((_vbox__IVirtualBox_USCOREgetExtraData *)ptr)->soap_out(soap, "vbox:IVirtualBox_getExtraData", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetNextExtraDataKeyResponse:
		return ((_vbox__IVirtualBox_USCOREgetNextExtraDataKeyResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getNextExtraDataKeyResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetNextExtraDataKey:
		return ((_vbox__IVirtualBox_USCOREgetNextExtraDataKey *)ptr)->soap_out(soap, "vbox:IVirtualBox_getNextExtraDataKey", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREremoveSharedFolderResponse:
		return ((_vbox__IVirtualBox_USCOREremoveSharedFolderResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_removeSharedFolderResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREremoveSharedFolder:
		return ((_vbox__IVirtualBox_USCOREremoveSharedFolder *)ptr)->soap_out(soap, "vbox:IVirtualBox_removeSharedFolder", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateSharedFolderResponse:
		return ((_vbox__IVirtualBox_USCOREcreateSharedFolderResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_createSharedFolderResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateSharedFolder:
		return ((_vbox__IVirtualBox_USCOREcreateSharedFolder *)ptr)->soap_out(soap, "vbox:IVirtualBox_createSharedFolder", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetGuestOSTypeResponse:
		return ((_vbox__IVirtualBox_USCOREgetGuestOSTypeResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getGuestOSTypeResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetGuestOSType:
		return ((_vbox__IVirtualBox_USCOREgetGuestOSType *)ptr)->soap_out(soap, "vbox:IVirtualBox_getGuestOSType", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindFloppyImageResponse:
		return ((_vbox__IVirtualBox_USCOREfindFloppyImageResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_findFloppyImageResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindFloppyImage:
		return ((_vbox__IVirtualBox_USCOREfindFloppyImage *)ptr)->soap_out(soap, "vbox:IVirtualBox_findFloppyImage", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetFloppyImageResponse:
		return ((_vbox__IVirtualBox_USCOREgetFloppyImageResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getFloppyImageResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetFloppyImage:
		return ((_vbox__IVirtualBox_USCOREgetFloppyImage *)ptr)->soap_out(soap, "vbox:IVirtualBox_getFloppyImage", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenFloppyImageResponse:
		return ((_vbox__IVirtualBox_USCOREopenFloppyImageResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_openFloppyImageResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenFloppyImage:
		return ((_vbox__IVirtualBox_USCOREopenFloppyImage *)ptr)->soap_out(soap, "vbox:IVirtualBox_openFloppyImage", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindDVDImageResponse:
		return ((_vbox__IVirtualBox_USCOREfindDVDImageResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_findDVDImageResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindDVDImage:
		return ((_vbox__IVirtualBox_USCOREfindDVDImage *)ptr)->soap_out(soap, "vbox:IVirtualBox_findDVDImage", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetDVDImageResponse:
		return ((_vbox__IVirtualBox_USCOREgetDVDImageResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getDVDImageResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetDVDImage:
		return ((_vbox__IVirtualBox_USCOREgetDVDImage *)ptr)->soap_out(soap, "vbox:IVirtualBox_getDVDImage", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenDVDImageResponse:
		return ((_vbox__IVirtualBox_USCOREopenDVDImageResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_openDVDImageResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenDVDImage:
		return ((_vbox__IVirtualBox_USCOREopenDVDImage *)ptr)->soap_out(soap, "vbox:IVirtualBox_openDVDImage", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindHardDiskResponse:
		return ((_vbox__IVirtualBox_USCOREfindHardDiskResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_findHardDiskResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindHardDisk:
		return ((_vbox__IVirtualBox_USCOREfindHardDisk *)ptr)->soap_out(soap, "vbox:IVirtualBox_findHardDisk", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetHardDiskResponse:
		return ((_vbox__IVirtualBox_USCOREgetHardDiskResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getHardDiskResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetHardDisk:
		return ((_vbox__IVirtualBox_USCOREgetHardDisk *)ptr)->soap_out(soap, "vbox:IVirtualBox_getHardDisk", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenHardDiskResponse:
		return ((_vbox__IVirtualBox_USCOREopenHardDiskResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_openHardDiskResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenHardDisk:
		return ((_vbox__IVirtualBox_USCOREopenHardDisk *)ptr)->soap_out(soap, "vbox:IVirtualBox_openHardDisk", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateHardDiskResponse:
		return ((_vbox__IVirtualBox_USCOREcreateHardDiskResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_createHardDiskResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateHardDisk:
		return ((_vbox__IVirtualBox_USCOREcreateHardDisk *)ptr)->soap_out(soap, "vbox:IVirtualBox_createHardDisk", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateApplianceResponse:
		return ((_vbox__IVirtualBox_USCOREcreateApplianceResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_createApplianceResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateAppliance:
		return ((_vbox__IVirtualBox_USCOREcreateAppliance *)ptr)->soap_out(soap, "vbox:IVirtualBox_createAppliance", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREunregisterMachineResponse:
		return ((_vbox__IVirtualBox_USCOREunregisterMachineResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_unregisterMachineResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREunregisterMachine:
		return ((_vbox__IVirtualBox_USCOREunregisterMachine *)ptr)->soap_out(soap, "vbox:IVirtualBox_unregisterMachine", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindMachineResponse:
		return ((_vbox__IVirtualBox_USCOREfindMachineResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_findMachineResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindMachine:
		return ((_vbox__IVirtualBox_USCOREfindMachine *)ptr)->soap_out(soap, "vbox:IVirtualBox_findMachine", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetMachineResponse:
		return ((_vbox__IVirtualBox_USCOREgetMachineResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getMachineResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetMachine:
		return ((_vbox__IVirtualBox_USCOREgetMachine *)ptr)->soap_out(soap, "vbox:IVirtualBox_getMachine", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREregisterMachineResponse:
		return ((_vbox__IVirtualBox_USCOREregisterMachineResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_registerMachineResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREregisterMachine:
		return ((_vbox__IVirtualBox_USCOREregisterMachine *)ptr)->soap_out(soap, "vbox:IVirtualBox_registerMachine", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenMachineResponse:
		return ((_vbox__IVirtualBox_USCOREopenMachineResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_openMachineResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenMachine:
		return ((_vbox__IVirtualBox_USCOREopenMachine *)ptr)->soap_out(soap, "vbox:IVirtualBox_openMachine", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateLegacyMachineResponse:
		return ((_vbox__IVirtualBox_USCOREcreateLegacyMachineResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_createLegacyMachineResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateLegacyMachine:
		return ((_vbox__IVirtualBox_USCOREcreateLegacyMachine *)ptr)->soap_out(soap, "vbox:IVirtualBox_createLegacyMachine", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateMachineResponse:
		return ((_vbox__IVirtualBox_USCOREcreateMachineResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_createMachineResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateMachine:
		return ((_vbox__IVirtualBox_USCOREcreateMachine *)ptr)->soap_out(soap, "vbox:IVirtualBox_createMachine", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetDHCPServersResponse:
		return ((_vbox__IVirtualBox_USCOREgetDHCPServersResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getDHCPServersResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetDHCPServers:
		return ((_vbox__IVirtualBox_USCOREgetDHCPServers *)ptr)->soap_out(soap, "vbox:IVirtualBox_getDHCPServers", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetPerformanceCollectorResponse:
		return ((_vbox__IVirtualBox_USCOREgetPerformanceCollectorResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getPerformanceCollectorResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetPerformanceCollector:
		return ((_vbox__IVirtualBox_USCOREgetPerformanceCollector *)ptr)->soap_out(soap, "vbox:IVirtualBox_getPerformanceCollector", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSharedFoldersResponse:
		return ((_vbox__IVirtualBox_USCOREgetSharedFoldersResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getSharedFoldersResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSharedFolders:
		return ((_vbox__IVirtualBox_USCOREgetSharedFolders *)ptr)->soap_out(soap, "vbox:IVirtualBox_getSharedFolders", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetGuestOSTypesResponse:
		return ((_vbox__IVirtualBox_USCOREgetGuestOSTypesResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getGuestOSTypesResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetGuestOSTypes:
		return ((_vbox__IVirtualBox_USCOREgetGuestOSTypes *)ptr)->soap_out(soap, "vbox:IVirtualBox_getGuestOSTypes", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetProgressOperationsResponse:
		return ((_vbox__IVirtualBox_USCOREgetProgressOperationsResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getProgressOperationsResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetProgressOperations:
		return ((_vbox__IVirtualBox_USCOREgetProgressOperations *)ptr)->soap_out(soap, "vbox:IVirtualBox_getProgressOperations", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetFloppyImagesResponse:
		return ((_vbox__IVirtualBox_USCOREgetFloppyImagesResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getFloppyImagesResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetFloppyImages:
		return ((_vbox__IVirtualBox_USCOREgetFloppyImages *)ptr)->soap_out(soap, "vbox:IVirtualBox_getFloppyImages", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetDVDImagesResponse:
		return ((_vbox__IVirtualBox_USCOREgetDVDImagesResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getDVDImagesResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetDVDImages:
		return ((_vbox__IVirtualBox_USCOREgetDVDImages *)ptr)->soap_out(soap, "vbox:IVirtualBox_getDVDImages", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetHardDisksResponse:
		return ((_vbox__IVirtualBox_USCOREgetHardDisksResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getHardDisksResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetHardDisks:
		return ((_vbox__IVirtualBox_USCOREgetHardDisks *)ptr)->soap_out(soap, "vbox:IVirtualBox_getHardDisks", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetMachinesResponse:
		return ((_vbox__IVirtualBox_USCOREgetMachinesResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getMachinesResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetMachines:
		return ((_vbox__IVirtualBox_USCOREgetMachines *)ptr)->soap_out(soap, "vbox:IVirtualBox_getMachines", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSystemPropertiesResponse:
		return ((_vbox__IVirtualBox_USCOREgetSystemPropertiesResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getSystemPropertiesResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSystemProperties:
		return ((_vbox__IVirtualBox_USCOREgetSystemProperties *)ptr)->soap_out(soap, "vbox:IVirtualBox_getSystemProperties", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetHostResponse:
		return ((_vbox__IVirtualBox_USCOREgetHostResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getHostResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetHost:
		return ((_vbox__IVirtualBox_USCOREgetHost *)ptr)->soap_out(soap, "vbox:IVirtualBox_getHost", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSettingsFormatVersionResponse:
		return ((_vbox__IVirtualBox_USCOREgetSettingsFormatVersionResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getSettingsFormatVersionResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSettingsFormatVersion:
		return ((_vbox__IVirtualBox_USCOREgetSettingsFormatVersion *)ptr)->soap_out(soap, "vbox:IVirtualBox_getSettingsFormatVersion", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSettingsFileVersionResponse:
		return ((_vbox__IVirtualBox_USCOREgetSettingsFileVersionResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getSettingsFileVersionResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSettingsFileVersion:
		return ((_vbox__IVirtualBox_USCOREgetSettingsFileVersion *)ptr)->soap_out(soap, "vbox:IVirtualBox_getSettingsFileVersion", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSettingsFilePathResponse:
		return ((_vbox__IVirtualBox_USCOREgetSettingsFilePathResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getSettingsFilePathResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSettingsFilePath:
		return ((_vbox__IVirtualBox_USCOREgetSettingsFilePath *)ptr)->soap_out(soap, "vbox:IVirtualBox_getSettingsFilePath", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetHomeFolderResponse:
		return ((_vbox__IVirtualBox_USCOREgetHomeFolderResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getHomeFolderResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetHomeFolder:
		return ((_vbox__IVirtualBox_USCOREgetHomeFolder *)ptr)->soap_out(soap, "vbox:IVirtualBox_getHomeFolder", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetPackageTypeResponse:
		return ((_vbox__IVirtualBox_USCOREgetPackageTypeResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getPackageTypeResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetPackageType:
		return ((_vbox__IVirtualBox_USCOREgetPackageType *)ptr)->soap_out(soap, "vbox:IVirtualBox_getPackageType", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetRevisionResponse:
		return ((_vbox__IVirtualBox_USCOREgetRevisionResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getRevisionResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetRevision:
		return ((_vbox__IVirtualBox_USCOREgetRevision *)ptr)->soap_out(soap, "vbox:IVirtualBox_getRevision", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetVersionResponse:
		return ((_vbox__IVirtualBox_USCOREgetVersionResponse *)ptr)->soap_out(soap, "vbox:IVirtualBox_getVersionResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetVersion:
		return ((_vbox__IVirtualBox_USCOREgetVersion *)ptr)->soap_out(soap, "vbox:IVirtualBox_getVersion", id, NULL);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREstopResponse:
		return ((_vbox__IDHCPServer_USCOREstopResponse *)ptr)->soap_out(soap, "vbox:IDHCPServer_stopResponse", id, NULL);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREstop:
		return ((_vbox__IDHCPServer_USCOREstop *)ptr)->soap_out(soap, "vbox:IDHCPServer_stop", id, NULL);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREstartResponse:
		return ((_vbox__IDHCPServer_USCOREstartResponse *)ptr)->soap_out(soap, "vbox:IDHCPServer_startResponse", id, NULL);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREstart:
		return ((_vbox__IDHCPServer_USCOREstart *)ptr)->soap_out(soap, "vbox:IDHCPServer_start", id, NULL);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREsetConfigurationResponse:
		return ((_vbox__IDHCPServer_USCOREsetConfigurationResponse *)ptr)->soap_out(soap, "vbox:IDHCPServer_setConfigurationResponse", id, NULL);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREsetConfiguration:
		return ((_vbox__IDHCPServer_USCOREsetConfiguration *)ptr)->soap_out(soap, "vbox:IDHCPServer_setConfiguration", id, NULL);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetUpperIPResponse:
		return ((_vbox__IDHCPServer_USCOREgetUpperIPResponse *)ptr)->soap_out(soap, "vbox:IDHCPServer_getUpperIPResponse", id, NULL);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetUpperIP:
		return ((_vbox__IDHCPServer_USCOREgetUpperIP *)ptr)->soap_out(soap, "vbox:IDHCPServer_getUpperIP", id, NULL);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetLowerIPResponse:
		return ((_vbox__IDHCPServer_USCOREgetLowerIPResponse *)ptr)->soap_out(soap, "vbox:IDHCPServer_getLowerIPResponse", id, NULL);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetLowerIP:
		return ((_vbox__IDHCPServer_USCOREgetLowerIP *)ptr)->soap_out(soap, "vbox:IDHCPServer_getLowerIP", id, NULL);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetNetworkNameResponse:
		return ((_vbox__IDHCPServer_USCOREgetNetworkNameResponse *)ptr)->soap_out(soap, "vbox:IDHCPServer_getNetworkNameResponse", id, NULL);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetNetworkName:
		return ((_vbox__IDHCPServer_USCOREgetNetworkName *)ptr)->soap_out(soap, "vbox:IDHCPServer_getNetworkName", id, NULL);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetNetworkMaskResponse:
		return ((_vbox__IDHCPServer_USCOREgetNetworkMaskResponse *)ptr)->soap_out(soap, "vbox:IDHCPServer_getNetworkMaskResponse", id, NULL);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetNetworkMask:
		return ((_vbox__IDHCPServer_USCOREgetNetworkMask *)ptr)->soap_out(soap, "vbox:IDHCPServer_getNetworkMask", id, NULL);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetIPAddressResponse:
		return ((_vbox__IDHCPServer_USCOREgetIPAddressResponse *)ptr)->soap_out(soap, "vbox:IDHCPServer_getIPAddressResponse", id, NULL);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetIPAddress:
		return ((_vbox__IDHCPServer_USCOREgetIPAddress *)ptr)->soap_out(soap, "vbox:IDHCPServer_getIPAddress", id, NULL);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREsetEnabledResponse:
		return ((_vbox__IDHCPServer_USCOREsetEnabledResponse *)ptr)->soap_out(soap, "vbox:IDHCPServer_setEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREsetEnabled:
		return ((_vbox__IDHCPServer_USCOREsetEnabled *)ptr)->soap_out(soap, "vbox:IDHCPServer_setEnabled", id, NULL);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetEnabledResponse:
		return ((_vbox__IDHCPServer_USCOREgetEnabledResponse *)ptr)->soap_out(soap, "vbox:IDHCPServer_getEnabledResponse", id, NULL);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetEnabled:
		return ((_vbox__IDHCPServer_USCOREgetEnabled *)ptr)->soap_out(soap, "vbox:IDHCPServer_getEnabled", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetNextResponse:
		return ((_vbox__IVirtualBoxErrorInfo_USCOREgetNextResponse *)ptr)->soap_out(soap, "vbox:IVirtualBoxErrorInfo_getNextResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetNext:
		return ((_vbox__IVirtualBoxErrorInfo_USCOREgetNext *)ptr)->soap_out(soap, "vbox:IVirtualBoxErrorInfo_getNext", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetTextResponse:
		return ((_vbox__IVirtualBoxErrorInfo_USCOREgetTextResponse *)ptr)->soap_out(soap, "vbox:IVirtualBoxErrorInfo_getTextResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetText:
		return ((_vbox__IVirtualBoxErrorInfo_USCOREgetText *)ptr)->soap_out(soap, "vbox:IVirtualBoxErrorInfo_getText", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetComponentResponse:
		return ((_vbox__IVirtualBoxErrorInfo_USCOREgetComponentResponse *)ptr)->soap_out(soap, "vbox:IVirtualBoxErrorInfo_getComponentResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetComponent:
		return ((_vbox__IVirtualBoxErrorInfo_USCOREgetComponent *)ptr)->soap_out(soap, "vbox:IVirtualBoxErrorInfo_getComponent", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceIDResponse:
		return ((_vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceIDResponse *)ptr)->soap_out(soap, "vbox:IVirtualBoxErrorInfo_getInterfaceIDResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceID:
		return ((_vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceID *)ptr)->soap_out(soap, "vbox:IVirtualBoxErrorInfo_getInterfaceID", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetResultCodeResponse:
		return ((_vbox__IVirtualBoxErrorInfo_USCOREgetResultCodeResponse *)ptr)->soap_out(soap, "vbox:IVirtualBoxErrorInfo_getResultCodeResponse", id, NULL);
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetResultCode:
		return ((_vbox__IVirtualBoxErrorInfo_USCOREgetResultCode *)ptr)->soap_out(soap, "vbox:IVirtualBoxErrorInfo_getResultCode", id, NULL);
	case SOAP_TYPE_vbox__ISharedFolder:
		return ((vbox__ISharedFolder *)ptr)->soap_out(soap, tag, id, "vbox:ISharedFolder");
	case SOAP_TYPE_vbox__IHardDiskAttachment:
		return ((vbox__IHardDiskAttachment *)ptr)->soap_out(soap, tag, id, "vbox:IHardDiskAttachment");
	case SOAP_TYPE_vbox__IGuestOSType:
		return ((vbox__IGuestOSType *)ptr)->soap_out(soap, tag, id, "vbox:IGuestOSType");
	case SOAP_TYPE_std__string:
		return soap_out_std__string(soap, tag, id, (const std::string *)ptr, "xsd:string");
	case SOAP_TYPE_vbox__IRemoteDisplayInfo:
		return ((vbox__IRemoteDisplayInfo *)ptr)->soap_out(soap, tag, id, "vbox:IRemoteDisplayInfo");
	case SOAP_TYPE__vbox__RuntimeFault:
		return ((_vbox__RuntimeFault *)ptr)->soap_out(soap, "vbox:RuntimeFault", id, NULL);
	case SOAP_TYPE__vbox__InvalidObjectFault:
		return ((_vbox__InvalidObjectFault *)ptr)->soap_out(soap, "vbox:InvalidObjectFault", id, NULL);
	case SOAP_TYPE_PointerTo_vbox__IPerformanceCollector_USCOREqueryMetricsDataResponse:
		return soap_out_PointerTo_vbox__IPerformanceCollector_USCOREqueryMetricsDataResponse(soap, tag, id, (_vbox__IPerformanceCollector_USCOREqueryMetricsDataResponse *const*)ptr, "vbox:IPerformanceCollector_queryMetricsDataResponse");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceCollector_USCOREqueryMetricsData:
		return soap_out_PointerTo_vbox__IPerformanceCollector_USCOREqueryMetricsData(soap, tag, id, (_vbox__IPerformanceCollector_USCOREqueryMetricsData *const*)ptr, "vbox:IPerformanceCollector_queryMetricsData");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceCollector_USCOREdisableMetricsResponse:
		return soap_out_PointerTo_vbox__IPerformanceCollector_USCOREdisableMetricsResponse(soap, tag, id, (_vbox__IPerformanceCollector_USCOREdisableMetricsResponse *const*)ptr, "vbox:IPerformanceCollector_disableMetricsResponse");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceCollector_USCOREdisableMetrics:
		return soap_out_PointerTo_vbox__IPerformanceCollector_USCOREdisableMetrics(soap, tag, id, (_vbox__IPerformanceCollector_USCOREdisableMetrics *const*)ptr, "vbox:IPerformanceCollector_disableMetrics");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceCollector_USCOREenableMetricsResponse:
		return soap_out_PointerTo_vbox__IPerformanceCollector_USCOREenableMetricsResponse(soap, tag, id, (_vbox__IPerformanceCollector_USCOREenableMetricsResponse *const*)ptr, "vbox:IPerformanceCollector_enableMetricsResponse");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceCollector_USCOREenableMetrics:
		return soap_out_PointerTo_vbox__IPerformanceCollector_USCOREenableMetrics(soap, tag, id, (_vbox__IPerformanceCollector_USCOREenableMetrics *const*)ptr, "vbox:IPerformanceCollector_enableMetrics");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceCollector_USCOREsetupMetricsResponse:
		return soap_out_PointerTo_vbox__IPerformanceCollector_USCOREsetupMetricsResponse(soap, tag, id, (_vbox__IPerformanceCollector_USCOREsetupMetricsResponse *const*)ptr, "vbox:IPerformanceCollector_setupMetricsResponse");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceCollector_USCOREsetupMetrics:
		return soap_out_PointerTo_vbox__IPerformanceCollector_USCOREsetupMetrics(soap, tag, id, (_vbox__IPerformanceCollector_USCOREsetupMetrics *const*)ptr, "vbox:IPerformanceCollector_setupMetrics");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceCollector_USCOREgetMetricsResponse:
		return soap_out_PointerTo_vbox__IPerformanceCollector_USCOREgetMetricsResponse(soap, tag, id, (_vbox__IPerformanceCollector_USCOREgetMetricsResponse *const*)ptr, "vbox:IPerformanceCollector_getMetricsResponse");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceCollector_USCOREgetMetrics:
		return soap_out_PointerTo_vbox__IPerformanceCollector_USCOREgetMetrics(soap, tag, id, (_vbox__IPerformanceCollector_USCOREgetMetrics *const*)ptr, "vbox:IPerformanceCollector_getMetrics");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceCollector_USCOREgetMetricNamesResponse:
		return soap_out_PointerTo_vbox__IPerformanceCollector_USCOREgetMetricNamesResponse(soap, tag, id, (_vbox__IPerformanceCollector_USCOREgetMetricNamesResponse *const*)ptr, "vbox:IPerformanceCollector_getMetricNamesResponse");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceCollector_USCOREgetMetricNames:
		return soap_out_PointerTo_vbox__IPerformanceCollector_USCOREgetMetricNames(soap, tag, id, (_vbox__IPerformanceCollector_USCOREgetMetricNames *const*)ptr, "vbox:IPerformanceCollector_getMetricNames");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetMaximumValueResponse:
		return soap_out_PointerTo_vbox__IPerformanceMetric_USCOREgetMaximumValueResponse(soap, tag, id, (_vbox__IPerformanceMetric_USCOREgetMaximumValueResponse *const*)ptr, "vbox:IPerformanceMetric_getMaximumValueResponse");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetMaximumValue:
		return soap_out_PointerTo_vbox__IPerformanceMetric_USCOREgetMaximumValue(soap, tag, id, (_vbox__IPerformanceMetric_USCOREgetMaximumValue *const*)ptr, "vbox:IPerformanceMetric_getMaximumValue");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetMinimumValueResponse:
		return soap_out_PointerTo_vbox__IPerformanceMetric_USCOREgetMinimumValueResponse(soap, tag, id, (_vbox__IPerformanceMetric_USCOREgetMinimumValueResponse *const*)ptr, "vbox:IPerformanceMetric_getMinimumValueResponse");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetMinimumValue:
		return soap_out_PointerTo_vbox__IPerformanceMetric_USCOREgetMinimumValue(soap, tag, id, (_vbox__IPerformanceMetric_USCOREgetMinimumValue *const*)ptr, "vbox:IPerformanceMetric_getMinimumValue");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetUnitResponse:
		return soap_out_PointerTo_vbox__IPerformanceMetric_USCOREgetUnitResponse(soap, tag, id, (_vbox__IPerformanceMetric_USCOREgetUnitResponse *const*)ptr, "vbox:IPerformanceMetric_getUnitResponse");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetUnit:
		return soap_out_PointerTo_vbox__IPerformanceMetric_USCOREgetUnit(soap, tag, id, (_vbox__IPerformanceMetric_USCOREgetUnit *const*)ptr, "vbox:IPerformanceMetric_getUnit");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetCountResponse:
		return soap_out_PointerTo_vbox__IPerformanceMetric_USCOREgetCountResponse(soap, tag, id, (_vbox__IPerformanceMetric_USCOREgetCountResponse *const*)ptr, "vbox:IPerformanceMetric_getCountResponse");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetCount:
		return soap_out_PointerTo_vbox__IPerformanceMetric_USCOREgetCount(soap, tag, id, (_vbox__IPerformanceMetric_USCOREgetCount *const*)ptr, "vbox:IPerformanceMetric_getCount");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetPeriodResponse:
		return soap_out_PointerTo_vbox__IPerformanceMetric_USCOREgetPeriodResponse(soap, tag, id, (_vbox__IPerformanceMetric_USCOREgetPeriodResponse *const*)ptr, "vbox:IPerformanceMetric_getPeriodResponse");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetPeriod:
		return soap_out_PointerTo_vbox__IPerformanceMetric_USCOREgetPeriod(soap, tag, id, (_vbox__IPerformanceMetric_USCOREgetPeriod *const*)ptr, "vbox:IPerformanceMetric_getPeriod");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetDescriptionResponse:
		return soap_out_PointerTo_vbox__IPerformanceMetric_USCOREgetDescriptionResponse(soap, tag, id, (_vbox__IPerformanceMetric_USCOREgetDescriptionResponse *const*)ptr, "vbox:IPerformanceMetric_getDescriptionResponse");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetDescription:
		return soap_out_PointerTo_vbox__IPerformanceMetric_USCOREgetDescription(soap, tag, id, (_vbox__IPerformanceMetric_USCOREgetDescription *const*)ptr, "vbox:IPerformanceMetric_getDescription");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetObjectResponse:
		return soap_out_PointerTo_vbox__IPerformanceMetric_USCOREgetObjectResponse(soap, tag, id, (_vbox__IPerformanceMetric_USCOREgetObjectResponse *const*)ptr, "vbox:IPerformanceMetric_getObjectResponse");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetObject:
		return soap_out_PointerTo_vbox__IPerformanceMetric_USCOREgetObject(soap, tag, id, (_vbox__IPerformanceMetric_USCOREgetObject *const*)ptr, "vbox:IPerformanceMetric_getObject");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetMetricNameResponse:
		return soap_out_PointerTo_vbox__IPerformanceMetric_USCOREgetMetricNameResponse(soap, tag, id, (_vbox__IPerformanceMetric_USCOREgetMetricNameResponse *const*)ptr, "vbox:IPerformanceMetric_getMetricNameResponse");
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetMetricName:
		return soap_out_PointerTo_vbox__IPerformanceMetric_USCOREgetMetricName(soap, tag, id, (_vbox__IPerformanceMetric_USCOREgetMetricName *const*)ptr, "vbox:IPerformanceMetric_getMetricName");
	case SOAP_TYPE_PointerTo_vbox__IWebsessionManager_USCORElogoffResponse:
		return soap_out_PointerTo_vbox__IWebsessionManager_USCORElogoffResponse(soap, tag, id, (_vbox__IWebsessionManager_USCORElogoffResponse *const*)ptr, "vbox:IWebsessionManager_logoffResponse");
	case SOAP_TYPE_PointerTo_vbox__IWebsessionManager_USCORElogoff:
		return soap_out_PointerTo_vbox__IWebsessionManager_USCORElogoff(soap, tag, id, (_vbox__IWebsessionManager_USCORElogoff *const*)ptr, "vbox:IWebsessionManager_logoff");
	case SOAP_TYPE_PointerTo_vbox__IWebsessionManager_USCOREgetSessionObjectResponse:
		return soap_out_PointerTo_vbox__IWebsessionManager_USCOREgetSessionObjectResponse(soap, tag, id, (_vbox__IWebsessionManager_USCOREgetSessionObjectResponse *const*)ptr, "vbox:IWebsessionManager_getSessionObjectResponse");
	case SOAP_TYPE_PointerTo_vbox__IWebsessionManager_USCOREgetSessionObject:
		return soap_out_PointerTo_vbox__IWebsessionManager_USCOREgetSessionObject(soap, tag, id, (_vbox__IWebsessionManager_USCOREgetSessionObject *const*)ptr, "vbox:IWebsessionManager_getSessionObject");
	case SOAP_TYPE_PointerTo_vbox__IWebsessionManager_USCORElogonResponse:
		return soap_out_PointerTo_vbox__IWebsessionManager_USCORElogonResponse(soap, tag, id, (_vbox__IWebsessionManager_USCORElogonResponse *const*)ptr, "vbox:IWebsessionManager_logonResponse");
	case SOAP_TYPE_PointerTo_vbox__IWebsessionManager_USCORElogon:
		return soap_out_PointerTo_vbox__IWebsessionManager_USCORElogon(soap, tag, id, (_vbox__IWebsessionManager_USCORElogon *const*)ptr, "vbox:IWebsessionManager_logon");
	case SOAP_TYPE_PointerTo_vbox__IManagedObjectRef_USCOREreleaseResponse:
		return soap_out_PointerTo_vbox__IManagedObjectRef_USCOREreleaseResponse(soap, tag, id, (_vbox__IManagedObjectRef_USCOREreleaseResponse *const*)ptr, "vbox:IManagedObjectRef_releaseResponse");
	case SOAP_TYPE_PointerTo_vbox__IManagedObjectRef_USCORErelease:
		return soap_out_PointerTo_vbox__IManagedObjectRef_USCORErelease(soap, tag, id, (_vbox__IManagedObjectRef_USCORErelease *const*)ptr, "vbox:IManagedObjectRef_release");
	case SOAP_TYPE_PointerTo_vbox__IManagedObjectRef_USCOREgetInterfaceNameResponse:
		return soap_out_PointerTo_vbox__IManagedObjectRef_USCOREgetInterfaceNameResponse(soap, tag, id, (_vbox__IManagedObjectRef_USCOREgetInterfaceNameResponse *const*)ptr, "vbox:IManagedObjectRef_getInterfaceNameResponse");
	case SOAP_TYPE_PointerTo_vbox__IManagedObjectRef_USCOREgetInterfaceName:
		return soap_out_PointerTo_vbox__IManagedObjectRef_USCOREgetInterfaceName(soap, tag, id, (_vbox__IManagedObjectRef_USCOREgetInterfaceName *const*)ptr, "vbox:IManagedObjectRef_getInterfaceName");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCORESetIDEEmulationPortResponse:
		return soap_out_PointerTo_vbox__IStorageController_USCORESetIDEEmulationPortResponse(soap, tag, id, (_vbox__IStorageController_USCORESetIDEEmulationPortResponse *const*)ptr, "vbox:IStorageController_SetIDEEmulationPortResponse");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCORESetIDEEmulationPort:
		return soap_out_PointerTo_vbox__IStorageController_USCORESetIDEEmulationPort(soap, tag, id, (_vbox__IStorageController_USCORESetIDEEmulationPort *const*)ptr, "vbox:IStorageController_SetIDEEmulationPort");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREGetIDEEmulationPortResponse:
		return soap_out_PointerTo_vbox__IStorageController_USCOREGetIDEEmulationPortResponse(soap, tag, id, (_vbox__IStorageController_USCOREGetIDEEmulationPortResponse *const*)ptr, "vbox:IStorageController_GetIDEEmulationPortResponse");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREGetIDEEmulationPort:
		return soap_out_PointerTo_vbox__IStorageController_USCOREGetIDEEmulationPort(soap, tag, id, (_vbox__IStorageController_USCOREGetIDEEmulationPort *const*)ptr, "vbox:IStorageController_GetIDEEmulationPort");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREsetControllerTypeResponse:
		return soap_out_PointerTo_vbox__IStorageController_USCOREsetControllerTypeResponse(soap, tag, id, (_vbox__IStorageController_USCOREsetControllerTypeResponse *const*)ptr, "vbox:IStorageController_setControllerTypeResponse");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREsetControllerType:
		return soap_out_PointerTo_vbox__IStorageController_USCOREsetControllerType(soap, tag, id, (_vbox__IStorageController_USCOREsetControllerType *const*)ptr, "vbox:IStorageController_setControllerType");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetControllerTypeResponse:
		return soap_out_PointerTo_vbox__IStorageController_USCOREgetControllerTypeResponse(soap, tag, id, (_vbox__IStorageController_USCOREgetControllerTypeResponse *const*)ptr, "vbox:IStorageController_getControllerTypeResponse");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetControllerType:
		return soap_out_PointerTo_vbox__IStorageController_USCOREgetControllerType(soap, tag, id, (_vbox__IStorageController_USCOREgetControllerType *const*)ptr, "vbox:IStorageController_getControllerType");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetBusResponse:
		return soap_out_PointerTo_vbox__IStorageController_USCOREgetBusResponse(soap, tag, id, (_vbox__IStorageController_USCOREgetBusResponse *const*)ptr, "vbox:IStorageController_getBusResponse");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetBus:
		return soap_out_PointerTo_vbox__IStorageController_USCOREgetBus(soap, tag, id, (_vbox__IStorageController_USCOREgetBus *const*)ptr, "vbox:IStorageController_getBus");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREsetPortCountResponse:
		return soap_out_PointerTo_vbox__IStorageController_USCOREsetPortCountResponse(soap, tag, id, (_vbox__IStorageController_USCOREsetPortCountResponse *const*)ptr, "vbox:IStorageController_setPortCountResponse");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREsetPortCount:
		return soap_out_PointerTo_vbox__IStorageController_USCOREsetPortCount(soap, tag, id, (_vbox__IStorageController_USCOREsetPortCount *const*)ptr, "vbox:IStorageController_setPortCount");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetPortCountResponse:
		return soap_out_PointerTo_vbox__IStorageController_USCOREgetPortCountResponse(soap, tag, id, (_vbox__IStorageController_USCOREgetPortCountResponse *const*)ptr, "vbox:IStorageController_getPortCountResponse");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetPortCount:
		return soap_out_PointerTo_vbox__IStorageController_USCOREgetPortCount(soap, tag, id, (_vbox__IStorageController_USCOREgetPortCount *const*)ptr, "vbox:IStorageController_getPortCount");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREsetInstanceResponse:
		return soap_out_PointerTo_vbox__IStorageController_USCOREsetInstanceResponse(soap, tag, id, (_vbox__IStorageController_USCOREsetInstanceResponse *const*)ptr, "vbox:IStorageController_setInstanceResponse");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREsetInstance:
		return soap_out_PointerTo_vbox__IStorageController_USCOREsetInstance(soap, tag, id, (_vbox__IStorageController_USCOREsetInstance *const*)ptr, "vbox:IStorageController_setInstance");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetInstanceResponse:
		return soap_out_PointerTo_vbox__IStorageController_USCOREgetInstanceResponse(soap, tag, id, (_vbox__IStorageController_USCOREgetInstanceResponse *const*)ptr, "vbox:IStorageController_getInstanceResponse");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetInstance:
		return soap_out_PointerTo_vbox__IStorageController_USCOREgetInstance(soap, tag, id, (_vbox__IStorageController_USCOREgetInstance *const*)ptr, "vbox:IStorageController_getInstance");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetMaxPortCountResponse:
		return soap_out_PointerTo_vbox__IStorageController_USCOREgetMaxPortCountResponse(soap, tag, id, (_vbox__IStorageController_USCOREgetMaxPortCountResponse *const*)ptr, "vbox:IStorageController_getMaxPortCountResponse");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetMaxPortCount:
		return soap_out_PointerTo_vbox__IStorageController_USCOREgetMaxPortCount(soap, tag, id, (_vbox__IStorageController_USCOREgetMaxPortCount *const*)ptr, "vbox:IStorageController_getMaxPortCount");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetMinPortCountResponse:
		return soap_out_PointerTo_vbox__IStorageController_USCOREgetMinPortCountResponse(soap, tag, id, (_vbox__IStorageController_USCOREgetMinPortCountResponse *const*)ptr, "vbox:IStorageController_getMinPortCountResponse");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetMinPortCount:
		return soap_out_PointerTo_vbox__IStorageController_USCOREgetMinPortCount(soap, tag, id, (_vbox__IStorageController_USCOREgetMinPortCount *const*)ptr, "vbox:IStorageController_getMinPortCount");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetMaxDevicesPerPortCountResponse:
		return soap_out_PointerTo_vbox__IStorageController_USCOREgetMaxDevicesPerPortCountResponse(soap, tag, id, (_vbox__IStorageController_USCOREgetMaxDevicesPerPortCountResponse *const*)ptr, "vbox:IStorageController_getMaxDevicesPerPortCountResponse");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetMaxDevicesPerPortCount:
		return soap_out_PointerTo_vbox__IStorageController_USCOREgetMaxDevicesPerPortCount(soap, tag, id, (_vbox__IStorageController_USCOREgetMaxDevicesPerPortCount *const*)ptr, "vbox:IStorageController_getMaxDevicesPerPortCount");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetNameResponse:
		return soap_out_PointerTo_vbox__IStorageController_USCOREgetNameResponse(soap, tag, id, (_vbox__IStorageController_USCOREgetNameResponse *const*)ptr, "vbox:IStorageController_getNameResponse");
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetName:
		return soap_out_PointerTo_vbox__IStorageController_USCOREgetName(soap, tag, id, (_vbox__IStorageController_USCOREgetName *const*)ptr, "vbox:IStorageController_getName");
	case SOAP_TYPE_PointerTo_vbox__ISession_USCOREcloseResponse:
		return soap_out_PointerTo_vbox__ISession_USCOREcloseResponse(soap, tag, id, (_vbox__ISession_USCOREcloseResponse *const*)ptr, "vbox:ISession_closeResponse");
	case SOAP_TYPE_PointerTo_vbox__ISession_USCOREclose:
		return soap_out_PointerTo_vbox__ISession_USCOREclose(soap, tag, id, (_vbox__ISession_USCOREclose *const*)ptr, "vbox:ISession_close");
	case SOAP_TYPE_PointerTo_vbox__ISession_USCOREgetConsoleResponse:
		return soap_out_PointerTo_vbox__ISession_USCOREgetConsoleResponse(soap, tag, id, (_vbox__ISession_USCOREgetConsoleResponse *const*)ptr, "vbox:ISession_getConsoleResponse");
	case SOAP_TYPE_PointerTo_vbox__ISession_USCOREgetConsole:
		return soap_out_PointerTo_vbox__ISession_USCOREgetConsole(soap, tag, id, (_vbox__ISession_USCOREgetConsole *const*)ptr, "vbox:ISession_getConsole");
	case SOAP_TYPE_PointerTo_vbox__ISession_USCOREgetMachineResponse:
		return soap_out_PointerTo_vbox__ISession_USCOREgetMachineResponse(soap, tag, id, (_vbox__ISession_USCOREgetMachineResponse *const*)ptr, "vbox:ISession_getMachineResponse");
	case SOAP_TYPE_PointerTo_vbox__ISession_USCOREgetMachine:
		return soap_out_PointerTo_vbox__ISession_USCOREgetMachine(soap, tag, id, (_vbox__ISession_USCOREgetMachine *const*)ptr, "vbox:ISession_getMachine");
	case SOAP_TYPE_PointerTo_vbox__ISession_USCOREgetTypeResponse:
		return soap_out_PointerTo_vbox__ISession_USCOREgetTypeResponse(soap, tag, id, (_vbox__ISession_USCOREgetTypeResponse *const*)ptr, "vbox:ISession_getTypeResponse");
	case SOAP_TYPE_PointerTo_vbox__ISession_USCOREgetType:
		return soap_out_PointerTo_vbox__ISession_USCOREgetType(soap, tag, id, (_vbox__ISession_USCOREgetType *const*)ptr, "vbox:ISession_getType");
	case SOAP_TYPE_PointerTo_vbox__ISession_USCOREgetStateResponse:
		return soap_out_PointerTo_vbox__ISession_USCOREgetStateResponse(soap, tag, id, (_vbox__ISession_USCOREgetStateResponse *const*)ptr, "vbox:ISession_getStateResponse");
	case SOAP_TYPE_PointerTo_vbox__ISession_USCOREgetState:
		return soap_out_PointerTo_vbox__ISession_USCOREgetState(soap, tag, id, (_vbox__ISession_USCOREgetState *const*)ptr, "vbox:ISession_getState");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetReuseSingleConnectionResponse:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREsetReuseSingleConnectionResponse(soap, tag, id, (_vbox__IVRDPServer_USCOREsetReuseSingleConnectionResponse *const*)ptr, "vbox:IVRDPServer_setReuseSingleConnectionResponse");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetReuseSingleConnection:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREsetReuseSingleConnection(soap, tag, id, (_vbox__IVRDPServer_USCOREsetReuseSingleConnection *const*)ptr, "vbox:IVRDPServer_setReuseSingleConnection");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetReuseSingleConnectionResponse:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREgetReuseSingleConnectionResponse(soap, tag, id, (_vbox__IVRDPServer_USCOREgetReuseSingleConnectionResponse *const*)ptr, "vbox:IVRDPServer_getReuseSingleConnectionResponse");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetReuseSingleConnection:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREgetReuseSingleConnection(soap, tag, id, (_vbox__IVRDPServer_USCOREgetReuseSingleConnection *const*)ptr, "vbox:IVRDPServer_getReuseSingleConnection");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetAllowMultiConnectionResponse:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREsetAllowMultiConnectionResponse(soap, tag, id, (_vbox__IVRDPServer_USCOREsetAllowMultiConnectionResponse *const*)ptr, "vbox:IVRDPServer_setAllowMultiConnectionResponse");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetAllowMultiConnection:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREsetAllowMultiConnection(soap, tag, id, (_vbox__IVRDPServer_USCOREsetAllowMultiConnection *const*)ptr, "vbox:IVRDPServer_setAllowMultiConnection");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetAllowMultiConnectionResponse:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREgetAllowMultiConnectionResponse(soap, tag, id, (_vbox__IVRDPServer_USCOREgetAllowMultiConnectionResponse *const*)ptr, "vbox:IVRDPServer_getAllowMultiConnectionResponse");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetAllowMultiConnection:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREgetAllowMultiConnection(soap, tag, id, (_vbox__IVRDPServer_USCOREgetAllowMultiConnection *const*)ptr, "vbox:IVRDPServer_getAllowMultiConnection");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetAuthTimeoutResponse:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREsetAuthTimeoutResponse(soap, tag, id, (_vbox__IVRDPServer_USCOREsetAuthTimeoutResponse *const*)ptr, "vbox:IVRDPServer_setAuthTimeoutResponse");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetAuthTimeout:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREsetAuthTimeout(soap, tag, id, (_vbox__IVRDPServer_USCOREsetAuthTimeout *const*)ptr, "vbox:IVRDPServer_setAuthTimeout");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetAuthTimeoutResponse:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREgetAuthTimeoutResponse(soap, tag, id, (_vbox__IVRDPServer_USCOREgetAuthTimeoutResponse *const*)ptr, "vbox:IVRDPServer_getAuthTimeoutResponse");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetAuthTimeout:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREgetAuthTimeout(soap, tag, id, (_vbox__IVRDPServer_USCOREgetAuthTimeout *const*)ptr, "vbox:IVRDPServer_getAuthTimeout");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetAuthTypeResponse:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREsetAuthTypeResponse(soap, tag, id, (_vbox__IVRDPServer_USCOREsetAuthTypeResponse *const*)ptr, "vbox:IVRDPServer_setAuthTypeResponse");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetAuthType:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREsetAuthType(soap, tag, id, (_vbox__IVRDPServer_USCOREsetAuthType *const*)ptr, "vbox:IVRDPServer_setAuthType");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetAuthTypeResponse:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREgetAuthTypeResponse(soap, tag, id, (_vbox__IVRDPServer_USCOREgetAuthTypeResponse *const*)ptr, "vbox:IVRDPServer_getAuthTypeResponse");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetAuthType:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREgetAuthType(soap, tag, id, (_vbox__IVRDPServer_USCOREgetAuthType *const*)ptr, "vbox:IVRDPServer_getAuthType");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetNetAddressResponse:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREsetNetAddressResponse(soap, tag, id, (_vbox__IVRDPServer_USCOREsetNetAddressResponse *const*)ptr, "vbox:IVRDPServer_setNetAddressResponse");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetNetAddress:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREsetNetAddress(soap, tag, id, (_vbox__IVRDPServer_USCOREsetNetAddress *const*)ptr, "vbox:IVRDPServer_setNetAddress");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetNetAddressResponse:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREgetNetAddressResponse(soap, tag, id, (_vbox__IVRDPServer_USCOREgetNetAddressResponse *const*)ptr, "vbox:IVRDPServer_getNetAddressResponse");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetNetAddress:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREgetNetAddress(soap, tag, id, (_vbox__IVRDPServer_USCOREgetNetAddress *const*)ptr, "vbox:IVRDPServer_getNetAddress");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetPortResponse:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREsetPortResponse(soap, tag, id, (_vbox__IVRDPServer_USCOREsetPortResponse *const*)ptr, "vbox:IVRDPServer_setPortResponse");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetPort:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREsetPort(soap, tag, id, (_vbox__IVRDPServer_USCOREsetPort *const*)ptr, "vbox:IVRDPServer_setPort");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetPortResponse:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREgetPortResponse(soap, tag, id, (_vbox__IVRDPServer_USCOREgetPortResponse *const*)ptr, "vbox:IVRDPServer_getPortResponse");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetPort:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREgetPort(soap, tag, id, (_vbox__IVRDPServer_USCOREgetPort *const*)ptr, "vbox:IVRDPServer_getPort");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetEnabledResponse:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREsetEnabledResponse(soap, tag, id, (_vbox__IVRDPServer_USCOREsetEnabledResponse *const*)ptr, "vbox:IVRDPServer_setEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetEnabled:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREsetEnabled(soap, tag, id, (_vbox__IVRDPServer_USCOREsetEnabled *const*)ptr, "vbox:IVRDPServer_setEnabled");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetEnabledResponse:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREgetEnabledResponse(soap, tag, id, (_vbox__IVRDPServer_USCOREgetEnabledResponse *const*)ptr, "vbox:IVRDPServer_getEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetEnabled:
		return soap_out_PointerTo_vbox__IVRDPServer_USCOREgetEnabled(soap, tag, id, (_vbox__IVRDPServer_USCOREgetEnabled *const*)ptr, "vbox:IVRDPServer_getEnabled");
	case SOAP_TYPE_PointerTo_vbox__IAudioAdapter_USCOREsetAudioDriverResponse:
		return soap_out_PointerTo_vbox__IAudioAdapter_USCOREsetAudioDriverResponse(soap, tag, id, (_vbox__IAudioAdapter_USCOREsetAudioDriverResponse *const*)ptr, "vbox:IAudioAdapter_setAudioDriverResponse");
	case SOAP_TYPE_PointerTo_vbox__IAudioAdapter_USCOREsetAudioDriver:
		return soap_out_PointerTo_vbox__IAudioAdapter_USCOREsetAudioDriver(soap, tag, id, (_vbox__IAudioAdapter_USCOREsetAudioDriver *const*)ptr, "vbox:IAudioAdapter_setAudioDriver");
	case SOAP_TYPE_PointerTo_vbox__IAudioAdapter_USCOREgetAudioDriverResponse:
		return soap_out_PointerTo_vbox__IAudioAdapter_USCOREgetAudioDriverResponse(soap, tag, id, (_vbox__IAudioAdapter_USCOREgetAudioDriverResponse *const*)ptr, "vbox:IAudioAdapter_getAudioDriverResponse");
	case SOAP_TYPE_PointerTo_vbox__IAudioAdapter_USCOREgetAudioDriver:
		return soap_out_PointerTo_vbox__IAudioAdapter_USCOREgetAudioDriver(soap, tag, id, (_vbox__IAudioAdapter_USCOREgetAudioDriver *const*)ptr, "vbox:IAudioAdapter_getAudioDriver");
	case SOAP_TYPE_PointerTo_vbox__IAudioAdapter_USCOREsetAudioControllerResponse:
		return soap_out_PointerTo_vbox__IAudioAdapter_USCOREsetAudioControllerResponse(soap, tag, id, (_vbox__IAudioAdapter_USCOREsetAudioControllerResponse *const*)ptr, "vbox:IAudioAdapter_setAudioControllerResponse");
	case SOAP_TYPE_PointerTo_vbox__IAudioAdapter_USCOREsetAudioController:
		return soap_out_PointerTo_vbox__IAudioAdapter_USCOREsetAudioController(soap, tag, id, (_vbox__IAudioAdapter_USCOREsetAudioController *const*)ptr, "vbox:IAudioAdapter_setAudioController");
	case SOAP_TYPE_PointerTo_vbox__IAudioAdapter_USCOREgetAudioControllerResponse:
		return soap_out_PointerTo_vbox__IAudioAdapter_USCOREgetAudioControllerResponse(soap, tag, id, (_vbox__IAudioAdapter_USCOREgetAudioControllerResponse *const*)ptr, "vbox:IAudioAdapter_getAudioControllerResponse");
	case SOAP_TYPE_PointerTo_vbox__IAudioAdapter_USCOREgetAudioController:
		return soap_out_PointerTo_vbox__IAudioAdapter_USCOREgetAudioController(soap, tag, id, (_vbox__IAudioAdapter_USCOREgetAudioController *const*)ptr, "vbox:IAudioAdapter_getAudioController");
	case SOAP_TYPE_PointerTo_vbox__IAudioAdapter_USCOREsetEnabledResponse:
		return soap_out_PointerTo_vbox__IAudioAdapter_USCOREsetEnabledResponse(soap, tag, id, (_vbox__IAudioAdapter_USCOREsetEnabledResponse *const*)ptr, "vbox:IAudioAdapter_setEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IAudioAdapter_USCOREsetEnabled:
		return soap_out_PointerTo_vbox__IAudioAdapter_USCOREsetEnabled(soap, tag, id, (_vbox__IAudioAdapter_USCOREsetEnabled *const*)ptr, "vbox:IAudioAdapter_setEnabled");
	case SOAP_TYPE_PointerTo_vbox__IAudioAdapter_USCOREgetEnabledResponse:
		return soap_out_PointerTo_vbox__IAudioAdapter_USCOREgetEnabledResponse(soap, tag, id, (_vbox__IAudioAdapter_USCOREgetEnabledResponse *const*)ptr, "vbox:IAudioAdapter_getEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IAudioAdapter_USCOREgetEnabled:
		return soap_out_PointerTo_vbox__IAudioAdapter_USCOREgetEnabled(soap, tag, id, (_vbox__IAudioAdapter_USCOREgetEnabled *const*)ptr, "vbox:IAudioAdapter_getEnabled");
	case SOAP_TYPE_PointerTo_vbox__IHostUSBDeviceFilter_USCOREsetActionResponse:
		return soap_out_PointerTo_vbox__IHostUSBDeviceFilter_USCOREsetActionResponse(soap, tag, id, (_vbox__IHostUSBDeviceFilter_USCOREsetActionResponse *const*)ptr, "vbox:IHostUSBDeviceFilter_setActionResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostUSBDeviceFilter_USCOREsetAction:
		return soap_out_PointerTo_vbox__IHostUSBDeviceFilter_USCOREsetAction(soap, tag, id, (_vbox__IHostUSBDeviceFilter_USCOREsetAction *const*)ptr, "vbox:IHostUSBDeviceFilter_setAction");
	case SOAP_TYPE_PointerTo_vbox__IHostUSBDeviceFilter_USCOREgetActionResponse:
		return soap_out_PointerTo_vbox__IHostUSBDeviceFilter_USCOREgetActionResponse(soap, tag, id, (_vbox__IHostUSBDeviceFilter_USCOREgetActionResponse *const*)ptr, "vbox:IHostUSBDeviceFilter_getActionResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostUSBDeviceFilter_USCOREgetAction:
		return soap_out_PointerTo_vbox__IHostUSBDeviceFilter_USCOREgetAction(soap, tag, id, (_vbox__IHostUSBDeviceFilter_USCOREgetAction *const*)ptr, "vbox:IHostUSBDeviceFilter_getAction");
	case SOAP_TYPE_PointerTo_vbox__IHostUSBDevice_USCOREgetStateResponse:
		return soap_out_PointerTo_vbox__IHostUSBDevice_USCOREgetStateResponse(soap, tag, id, (_vbox__IHostUSBDevice_USCOREgetStateResponse *const*)ptr, "vbox:IHostUSBDevice_getStateResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostUSBDevice_USCOREgetState:
		return soap_out_PointerTo_vbox__IHostUSBDevice_USCOREgetState(soap, tag, id, (_vbox__IHostUSBDevice_USCOREgetState *const*)ptr, "vbox:IHostUSBDevice_getState");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetMaskedInterfacesResponse:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREsetMaskedInterfacesResponse(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREsetMaskedInterfacesResponse *const*)ptr, "vbox:IUSBDeviceFilter_setMaskedInterfacesResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetMaskedInterfaces:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREsetMaskedInterfaces(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREsetMaskedInterfaces *const*)ptr, "vbox:IUSBDeviceFilter_setMaskedInterfaces");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetMaskedInterfacesResponse:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREgetMaskedInterfacesResponse(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREgetMaskedInterfacesResponse *const*)ptr, "vbox:IUSBDeviceFilter_getMaskedInterfacesResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetMaskedInterfaces:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREgetMaskedInterfaces(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREgetMaskedInterfaces *const*)ptr, "vbox:IUSBDeviceFilter_getMaskedInterfaces");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetRemoteResponse:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREsetRemoteResponse(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREsetRemoteResponse *const*)ptr, "vbox:IUSBDeviceFilter_setRemoteResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetRemote:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREsetRemote(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREsetRemote *const*)ptr, "vbox:IUSBDeviceFilter_setRemote");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetRemoteResponse:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREgetRemoteResponse(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREgetRemoteResponse *const*)ptr, "vbox:IUSBDeviceFilter_getRemoteResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetRemote:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREgetRemote(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREgetRemote *const*)ptr, "vbox:IUSBDeviceFilter_getRemote");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetPortResponse:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREsetPortResponse(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREsetPortResponse *const*)ptr, "vbox:IUSBDeviceFilter_setPortResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetPort:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREsetPort(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREsetPort *const*)ptr, "vbox:IUSBDeviceFilter_setPort");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetPortResponse:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREgetPortResponse(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREgetPortResponse *const*)ptr, "vbox:IUSBDeviceFilter_getPortResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetPort:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREgetPort(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREgetPort *const*)ptr, "vbox:IUSBDeviceFilter_getPort");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetSerialNumberResponse:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREsetSerialNumberResponse(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREsetSerialNumberResponse *const*)ptr, "vbox:IUSBDeviceFilter_setSerialNumberResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetSerialNumber:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREsetSerialNumber(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREsetSerialNumber *const*)ptr, "vbox:IUSBDeviceFilter_setSerialNumber");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetSerialNumberResponse:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREgetSerialNumberResponse(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREgetSerialNumberResponse *const*)ptr, "vbox:IUSBDeviceFilter_getSerialNumberResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetSerialNumber:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREgetSerialNumber(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREgetSerialNumber *const*)ptr, "vbox:IUSBDeviceFilter_getSerialNumber");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetProductResponse:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREsetProductResponse(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREsetProductResponse *const*)ptr, "vbox:IUSBDeviceFilter_setProductResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetProduct:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREsetProduct(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREsetProduct *const*)ptr, "vbox:IUSBDeviceFilter_setProduct");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetProductResponse:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREgetProductResponse(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREgetProductResponse *const*)ptr, "vbox:IUSBDeviceFilter_getProductResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetProduct:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREgetProduct(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREgetProduct *const*)ptr, "vbox:IUSBDeviceFilter_getProduct");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetManufacturerResponse:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREsetManufacturerResponse(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREsetManufacturerResponse *const*)ptr, "vbox:IUSBDeviceFilter_setManufacturerResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetManufacturer:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREsetManufacturer(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREsetManufacturer *const*)ptr, "vbox:IUSBDeviceFilter_setManufacturer");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetManufacturerResponse:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREgetManufacturerResponse(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREgetManufacturerResponse *const*)ptr, "vbox:IUSBDeviceFilter_getManufacturerResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetManufacturer:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREgetManufacturer(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREgetManufacturer *const*)ptr, "vbox:IUSBDeviceFilter_getManufacturer");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetRevisionResponse:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREsetRevisionResponse(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREsetRevisionResponse *const*)ptr, "vbox:IUSBDeviceFilter_setRevisionResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetRevision:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREsetRevision(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREsetRevision *const*)ptr, "vbox:IUSBDeviceFilter_setRevision");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetRevisionResponse:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREgetRevisionResponse(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREgetRevisionResponse *const*)ptr, "vbox:IUSBDeviceFilter_getRevisionResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetRevision:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREgetRevision(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREgetRevision *const*)ptr, "vbox:IUSBDeviceFilter_getRevision");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetProductIdResponse:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREsetProductIdResponse(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREsetProductIdResponse *const*)ptr, "vbox:IUSBDeviceFilter_setProductIdResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetProductId:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREsetProductId(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREsetProductId *const*)ptr, "vbox:IUSBDeviceFilter_setProductId");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetProductIdResponse:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREgetProductIdResponse(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREgetProductIdResponse *const*)ptr, "vbox:IUSBDeviceFilter_getProductIdResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetProductId:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREgetProductId(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREgetProductId *const*)ptr, "vbox:IUSBDeviceFilter_getProductId");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetVendorIdResponse:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREsetVendorIdResponse(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREsetVendorIdResponse *const*)ptr, "vbox:IUSBDeviceFilter_setVendorIdResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetVendorId:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREsetVendorId(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREsetVendorId *const*)ptr, "vbox:IUSBDeviceFilter_setVendorId");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetVendorIdResponse:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREgetVendorIdResponse(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREgetVendorIdResponse *const*)ptr, "vbox:IUSBDeviceFilter_getVendorIdResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetVendorId:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREgetVendorId(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREgetVendorId *const*)ptr, "vbox:IUSBDeviceFilter_getVendorId");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetActiveResponse:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREsetActiveResponse(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREsetActiveResponse *const*)ptr, "vbox:IUSBDeviceFilter_setActiveResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetActive:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREsetActive(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREsetActive *const*)ptr, "vbox:IUSBDeviceFilter_setActive");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetActiveResponse:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREgetActiveResponse(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREgetActiveResponse *const*)ptr, "vbox:IUSBDeviceFilter_getActiveResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetActive:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREgetActive(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREgetActive *const*)ptr, "vbox:IUSBDeviceFilter_getActive");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetNameResponse:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREsetNameResponse(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREsetNameResponse *const*)ptr, "vbox:IUSBDeviceFilter_setNameResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetName:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREsetName(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREsetName *const*)ptr, "vbox:IUSBDeviceFilter_setName");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetNameResponse:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREgetNameResponse(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREgetNameResponse *const*)ptr, "vbox:IUSBDeviceFilter_getNameResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetName:
		return soap_out_PointerTo_vbox__IUSBDeviceFilter_USCOREgetName(soap, tag, id, (_vbox__IUSBDeviceFilter_USCOREgetName *const*)ptr, "vbox:IUSBDeviceFilter_getName");
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetRemoteResponse:
		return soap_out_PointerTo_vbox__IUSBDevice_USCOREgetRemoteResponse(soap, tag, id, (_vbox__IUSBDevice_USCOREgetRemoteResponse *const*)ptr, "vbox:IUSBDevice_getRemoteResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetRemote:
		return soap_out_PointerTo_vbox__IUSBDevice_USCOREgetRemote(soap, tag, id, (_vbox__IUSBDevice_USCOREgetRemote *const*)ptr, "vbox:IUSBDevice_getRemote");
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetPortVersionResponse:
		return soap_out_PointerTo_vbox__IUSBDevice_USCOREgetPortVersionResponse(soap, tag, id, (_vbox__IUSBDevice_USCOREgetPortVersionResponse *const*)ptr, "vbox:IUSBDevice_getPortVersionResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetPortVersion:
		return soap_out_PointerTo_vbox__IUSBDevice_USCOREgetPortVersion(soap, tag, id, (_vbox__IUSBDevice_USCOREgetPortVersion *const*)ptr, "vbox:IUSBDevice_getPortVersion");
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetVersionResponse:
		return soap_out_PointerTo_vbox__IUSBDevice_USCOREgetVersionResponse(soap, tag, id, (_vbox__IUSBDevice_USCOREgetVersionResponse *const*)ptr, "vbox:IUSBDevice_getVersionResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetVersion:
		return soap_out_PointerTo_vbox__IUSBDevice_USCOREgetVersion(soap, tag, id, (_vbox__IUSBDevice_USCOREgetVersion *const*)ptr, "vbox:IUSBDevice_getVersion");
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetPortResponse:
		return soap_out_PointerTo_vbox__IUSBDevice_USCOREgetPortResponse(soap, tag, id, (_vbox__IUSBDevice_USCOREgetPortResponse *const*)ptr, "vbox:IUSBDevice_getPortResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetPort:
		return soap_out_PointerTo_vbox__IUSBDevice_USCOREgetPort(soap, tag, id, (_vbox__IUSBDevice_USCOREgetPort *const*)ptr, "vbox:IUSBDevice_getPort");
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetAddressResponse:
		return soap_out_PointerTo_vbox__IUSBDevice_USCOREgetAddressResponse(soap, tag, id, (_vbox__IUSBDevice_USCOREgetAddressResponse *const*)ptr, "vbox:IUSBDevice_getAddressResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetAddress:
		return soap_out_PointerTo_vbox__IUSBDevice_USCOREgetAddress(soap, tag, id, (_vbox__IUSBDevice_USCOREgetAddress *const*)ptr, "vbox:IUSBDevice_getAddress");
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetSerialNumberResponse:
		return soap_out_PointerTo_vbox__IUSBDevice_USCOREgetSerialNumberResponse(soap, tag, id, (_vbox__IUSBDevice_USCOREgetSerialNumberResponse *const*)ptr, "vbox:IUSBDevice_getSerialNumberResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetSerialNumber:
		return soap_out_PointerTo_vbox__IUSBDevice_USCOREgetSerialNumber(soap, tag, id, (_vbox__IUSBDevice_USCOREgetSerialNumber *const*)ptr, "vbox:IUSBDevice_getSerialNumber");
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetProductResponse:
		return soap_out_PointerTo_vbox__IUSBDevice_USCOREgetProductResponse(soap, tag, id, (_vbox__IUSBDevice_USCOREgetProductResponse *const*)ptr, "vbox:IUSBDevice_getProductResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetProduct:
		return soap_out_PointerTo_vbox__IUSBDevice_USCOREgetProduct(soap, tag, id, (_vbox__IUSBDevice_USCOREgetProduct *const*)ptr, "vbox:IUSBDevice_getProduct");
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetManufacturerResponse:
		return soap_out_PointerTo_vbox__IUSBDevice_USCOREgetManufacturerResponse(soap, tag, id, (_vbox__IUSBDevice_USCOREgetManufacturerResponse *const*)ptr, "vbox:IUSBDevice_getManufacturerResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetManufacturer:
		return soap_out_PointerTo_vbox__IUSBDevice_USCOREgetManufacturer(soap, tag, id, (_vbox__IUSBDevice_USCOREgetManufacturer *const*)ptr, "vbox:IUSBDevice_getManufacturer");
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetRevisionResponse:
		return soap_out_PointerTo_vbox__IUSBDevice_USCOREgetRevisionResponse(soap, tag, id, (_vbox__IUSBDevice_USCOREgetRevisionResponse *const*)ptr, "vbox:IUSBDevice_getRevisionResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetRevision:
		return soap_out_PointerTo_vbox__IUSBDevice_USCOREgetRevision(soap, tag, id, (_vbox__IUSBDevice_USCOREgetRevision *const*)ptr, "vbox:IUSBDevice_getRevision");
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetProductIdResponse:
		return soap_out_PointerTo_vbox__IUSBDevice_USCOREgetProductIdResponse(soap, tag, id, (_vbox__IUSBDevice_USCOREgetProductIdResponse *const*)ptr, "vbox:IUSBDevice_getProductIdResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetProductId:
		return soap_out_PointerTo_vbox__IUSBDevice_USCOREgetProductId(soap, tag, id, (_vbox__IUSBDevice_USCOREgetProductId *const*)ptr, "vbox:IUSBDevice_getProductId");
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetVendorIdResponse:
		return soap_out_PointerTo_vbox__IUSBDevice_USCOREgetVendorIdResponse(soap, tag, id, (_vbox__IUSBDevice_USCOREgetVendorIdResponse *const*)ptr, "vbox:IUSBDevice_getVendorIdResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetVendorId:
		return soap_out_PointerTo_vbox__IUSBDevice_USCOREgetVendorId(soap, tag, id, (_vbox__IUSBDevice_USCOREgetVendorId *const*)ptr, "vbox:IUSBDevice_getVendorId");
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetIdResponse:
		return soap_out_PointerTo_vbox__IUSBDevice_USCOREgetIdResponse(soap, tag, id, (_vbox__IUSBDevice_USCOREgetIdResponse *const*)ptr, "vbox:IUSBDevice_getIdResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetId:
		return soap_out_PointerTo_vbox__IUSBDevice_USCOREgetId(soap, tag, id, (_vbox__IUSBDevice_USCOREgetId *const*)ptr, "vbox:IUSBDevice_getId");
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREremoveDeviceFilterResponse:
		return soap_out_PointerTo_vbox__IUSBController_USCOREremoveDeviceFilterResponse(soap, tag, id, (_vbox__IUSBController_USCOREremoveDeviceFilterResponse *const*)ptr, "vbox:IUSBController_removeDeviceFilterResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREremoveDeviceFilter:
		return soap_out_PointerTo_vbox__IUSBController_USCOREremoveDeviceFilter(soap, tag, id, (_vbox__IUSBController_USCOREremoveDeviceFilter *const*)ptr, "vbox:IUSBController_removeDeviceFilter");
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREinsertDeviceFilterResponse:
		return soap_out_PointerTo_vbox__IUSBController_USCOREinsertDeviceFilterResponse(soap, tag, id, (_vbox__IUSBController_USCOREinsertDeviceFilterResponse *const*)ptr, "vbox:IUSBController_insertDeviceFilterResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREinsertDeviceFilter:
		return soap_out_PointerTo_vbox__IUSBController_USCOREinsertDeviceFilter(soap, tag, id, (_vbox__IUSBController_USCOREinsertDeviceFilter *const*)ptr, "vbox:IUSBController_insertDeviceFilter");
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREcreateDeviceFilterResponse:
		return soap_out_PointerTo_vbox__IUSBController_USCOREcreateDeviceFilterResponse(soap, tag, id, (_vbox__IUSBController_USCOREcreateDeviceFilterResponse *const*)ptr, "vbox:IUSBController_createDeviceFilterResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREcreateDeviceFilter:
		return soap_out_PointerTo_vbox__IUSBController_USCOREcreateDeviceFilter(soap, tag, id, (_vbox__IUSBController_USCOREcreateDeviceFilter *const*)ptr, "vbox:IUSBController_createDeviceFilter");
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREgetDeviceFiltersResponse:
		return soap_out_PointerTo_vbox__IUSBController_USCOREgetDeviceFiltersResponse(soap, tag, id, (_vbox__IUSBController_USCOREgetDeviceFiltersResponse *const*)ptr, "vbox:IUSBController_getDeviceFiltersResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREgetDeviceFilters:
		return soap_out_PointerTo_vbox__IUSBController_USCOREgetDeviceFilters(soap, tag, id, (_vbox__IUSBController_USCOREgetDeviceFilters *const*)ptr, "vbox:IUSBController_getDeviceFilters");
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREgetUSBStandardResponse:
		return soap_out_PointerTo_vbox__IUSBController_USCOREgetUSBStandardResponse(soap, tag, id, (_vbox__IUSBController_USCOREgetUSBStandardResponse *const*)ptr, "vbox:IUSBController_getUSBStandardResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREgetUSBStandard:
		return soap_out_PointerTo_vbox__IUSBController_USCOREgetUSBStandard(soap, tag, id, (_vbox__IUSBController_USCOREgetUSBStandard *const*)ptr, "vbox:IUSBController_getUSBStandard");
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREsetEnabledEhciResponse:
		return soap_out_PointerTo_vbox__IUSBController_USCOREsetEnabledEhciResponse(soap, tag, id, (_vbox__IUSBController_USCOREsetEnabledEhciResponse *const*)ptr, "vbox:IUSBController_setEnabledEhciResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREsetEnabledEhci:
		return soap_out_PointerTo_vbox__IUSBController_USCOREsetEnabledEhci(soap, tag, id, (_vbox__IUSBController_USCOREsetEnabledEhci *const*)ptr, "vbox:IUSBController_setEnabledEhci");
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREgetEnabledEhciResponse:
		return soap_out_PointerTo_vbox__IUSBController_USCOREgetEnabledEhciResponse(soap, tag, id, (_vbox__IUSBController_USCOREgetEnabledEhciResponse *const*)ptr, "vbox:IUSBController_getEnabledEhciResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREgetEnabledEhci:
		return soap_out_PointerTo_vbox__IUSBController_USCOREgetEnabledEhci(soap, tag, id, (_vbox__IUSBController_USCOREgetEnabledEhci *const*)ptr, "vbox:IUSBController_getEnabledEhci");
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREsetEnabledResponse:
		return soap_out_PointerTo_vbox__IUSBController_USCOREsetEnabledResponse(soap, tag, id, (_vbox__IUSBController_USCOREsetEnabledResponse *const*)ptr, "vbox:IUSBController_setEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREsetEnabled:
		return soap_out_PointerTo_vbox__IUSBController_USCOREsetEnabled(soap, tag, id, (_vbox__IUSBController_USCOREsetEnabled *const*)ptr, "vbox:IUSBController_setEnabled");
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREgetEnabledResponse:
		return soap_out_PointerTo_vbox__IUSBController_USCOREgetEnabledResponse(soap, tag, id, (_vbox__IUSBController_USCOREgetEnabledResponse *const*)ptr, "vbox:IUSBController_getEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREgetEnabled:
		return soap_out_PointerTo_vbox__IUSBController_USCOREgetEnabled(soap, tag, id, (_vbox__IUSBController_USCOREgetEnabled *const*)ptr, "vbox:IUSBController_getEnabled");
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREsetPathResponse:
		return soap_out_PointerTo_vbox__IParallelPort_USCOREsetPathResponse(soap, tag, id, (_vbox__IParallelPort_USCOREsetPathResponse *const*)ptr, "vbox:IParallelPort_setPathResponse");
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREsetPath:
		return soap_out_PointerTo_vbox__IParallelPort_USCOREsetPath(soap, tag, id, (_vbox__IParallelPort_USCOREsetPath *const*)ptr, "vbox:IParallelPort_setPath");
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREgetPathResponse:
		return soap_out_PointerTo_vbox__IParallelPort_USCOREgetPathResponse(soap, tag, id, (_vbox__IParallelPort_USCOREgetPathResponse *const*)ptr, "vbox:IParallelPort_getPathResponse");
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREgetPath:
		return soap_out_PointerTo_vbox__IParallelPort_USCOREgetPath(soap, tag, id, (_vbox__IParallelPort_USCOREgetPath *const*)ptr, "vbox:IParallelPort_getPath");
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREsetIRQResponse:
		return soap_out_PointerTo_vbox__IParallelPort_USCOREsetIRQResponse(soap, tag, id, (_vbox__IParallelPort_USCOREsetIRQResponse *const*)ptr, "vbox:IParallelPort_setIRQResponse");
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREsetIRQ:
		return soap_out_PointerTo_vbox__IParallelPort_USCOREsetIRQ(soap, tag, id, (_vbox__IParallelPort_USCOREsetIRQ *const*)ptr, "vbox:IParallelPort_setIRQ");
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREgetIRQResponse:
		return soap_out_PointerTo_vbox__IParallelPort_USCOREgetIRQResponse(soap, tag, id, (_vbox__IParallelPort_USCOREgetIRQResponse *const*)ptr, "vbox:IParallelPort_getIRQResponse");
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREgetIRQ:
		return soap_out_PointerTo_vbox__IParallelPort_USCOREgetIRQ(soap, tag, id, (_vbox__IParallelPort_USCOREgetIRQ *const*)ptr, "vbox:IParallelPort_getIRQ");
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREsetIOBaseResponse:
		return soap_out_PointerTo_vbox__IParallelPort_USCOREsetIOBaseResponse(soap, tag, id, (_vbox__IParallelPort_USCOREsetIOBaseResponse *const*)ptr, "vbox:IParallelPort_setIOBaseResponse");
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREsetIOBase:
		return soap_out_PointerTo_vbox__IParallelPort_USCOREsetIOBase(soap, tag, id, (_vbox__IParallelPort_USCOREsetIOBase *const*)ptr, "vbox:IParallelPort_setIOBase");
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREgetIOBaseResponse:
		return soap_out_PointerTo_vbox__IParallelPort_USCOREgetIOBaseResponse(soap, tag, id, (_vbox__IParallelPort_USCOREgetIOBaseResponse *const*)ptr, "vbox:IParallelPort_getIOBaseResponse");
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREgetIOBase:
		return soap_out_PointerTo_vbox__IParallelPort_USCOREgetIOBase(soap, tag, id, (_vbox__IParallelPort_USCOREgetIOBase *const*)ptr, "vbox:IParallelPort_getIOBase");
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREsetEnabledResponse:
		return soap_out_PointerTo_vbox__IParallelPort_USCOREsetEnabledResponse(soap, tag, id, (_vbox__IParallelPort_USCOREsetEnabledResponse *const*)ptr, "vbox:IParallelPort_setEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREsetEnabled:
		return soap_out_PointerTo_vbox__IParallelPort_USCOREsetEnabled(soap, tag, id, (_vbox__IParallelPort_USCOREsetEnabled *const*)ptr, "vbox:IParallelPort_setEnabled");
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREgetEnabledResponse:
		return soap_out_PointerTo_vbox__IParallelPort_USCOREgetEnabledResponse(soap, tag, id, (_vbox__IParallelPort_USCOREgetEnabledResponse *const*)ptr, "vbox:IParallelPort_getEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREgetEnabled:
		return soap_out_PointerTo_vbox__IParallelPort_USCOREgetEnabled(soap, tag, id, (_vbox__IParallelPort_USCOREgetEnabled *const*)ptr, "vbox:IParallelPort_getEnabled");
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREgetSlotResponse:
		return soap_out_PointerTo_vbox__IParallelPort_USCOREgetSlotResponse(soap, tag, id, (_vbox__IParallelPort_USCOREgetSlotResponse *const*)ptr, "vbox:IParallelPort_getSlotResponse");
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREgetSlot:
		return soap_out_PointerTo_vbox__IParallelPort_USCOREgetSlot(soap, tag, id, (_vbox__IParallelPort_USCOREgetSlot *const*)ptr, "vbox:IParallelPort_getSlot");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREsetPathResponse:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREsetPathResponse(soap, tag, id, (_vbox__ISerialPort_USCOREsetPathResponse *const*)ptr, "vbox:ISerialPort_setPathResponse");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREsetPath:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREsetPath(soap, tag, id, (_vbox__ISerialPort_USCOREsetPath *const*)ptr, "vbox:ISerialPort_setPath");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetPathResponse:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREgetPathResponse(soap, tag, id, (_vbox__ISerialPort_USCOREgetPathResponse *const*)ptr, "vbox:ISerialPort_getPathResponse");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetPath:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREgetPath(soap, tag, id, (_vbox__ISerialPort_USCOREgetPath *const*)ptr, "vbox:ISerialPort_getPath");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREsetServerResponse:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREsetServerResponse(soap, tag, id, (_vbox__ISerialPort_USCOREsetServerResponse *const*)ptr, "vbox:ISerialPort_setServerResponse");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREsetServer:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREsetServer(soap, tag, id, (_vbox__ISerialPort_USCOREsetServer *const*)ptr, "vbox:ISerialPort_setServer");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetServerResponse:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREgetServerResponse(soap, tag, id, (_vbox__ISerialPort_USCOREgetServerResponse *const*)ptr, "vbox:ISerialPort_getServerResponse");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetServer:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREgetServer(soap, tag, id, (_vbox__ISerialPort_USCOREgetServer *const*)ptr, "vbox:ISerialPort_getServer");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREsetHostModeResponse:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREsetHostModeResponse(soap, tag, id, (_vbox__ISerialPort_USCOREsetHostModeResponse *const*)ptr, "vbox:ISerialPort_setHostModeResponse");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREsetHostMode:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREsetHostMode(soap, tag, id, (_vbox__ISerialPort_USCOREsetHostMode *const*)ptr, "vbox:ISerialPort_setHostMode");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetHostModeResponse:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREgetHostModeResponse(soap, tag, id, (_vbox__ISerialPort_USCOREgetHostModeResponse *const*)ptr, "vbox:ISerialPort_getHostModeResponse");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetHostMode:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREgetHostMode(soap, tag, id, (_vbox__ISerialPort_USCOREgetHostMode *const*)ptr, "vbox:ISerialPort_getHostMode");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREsetIRQResponse:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREsetIRQResponse(soap, tag, id, (_vbox__ISerialPort_USCOREsetIRQResponse *const*)ptr, "vbox:ISerialPort_setIRQResponse");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREsetIRQ:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREsetIRQ(soap, tag, id, (_vbox__ISerialPort_USCOREsetIRQ *const*)ptr, "vbox:ISerialPort_setIRQ");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetIRQResponse:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREgetIRQResponse(soap, tag, id, (_vbox__ISerialPort_USCOREgetIRQResponse *const*)ptr, "vbox:ISerialPort_getIRQResponse");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetIRQ:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREgetIRQ(soap, tag, id, (_vbox__ISerialPort_USCOREgetIRQ *const*)ptr, "vbox:ISerialPort_getIRQ");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREsetIOBaseResponse:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREsetIOBaseResponse(soap, tag, id, (_vbox__ISerialPort_USCOREsetIOBaseResponse *const*)ptr, "vbox:ISerialPort_setIOBaseResponse");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREsetIOBase:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREsetIOBase(soap, tag, id, (_vbox__ISerialPort_USCOREsetIOBase *const*)ptr, "vbox:ISerialPort_setIOBase");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetIOBaseResponse:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREgetIOBaseResponse(soap, tag, id, (_vbox__ISerialPort_USCOREgetIOBaseResponse *const*)ptr, "vbox:ISerialPort_getIOBaseResponse");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetIOBase:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREgetIOBase(soap, tag, id, (_vbox__ISerialPort_USCOREgetIOBase *const*)ptr, "vbox:ISerialPort_getIOBase");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREsetEnabledResponse:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREsetEnabledResponse(soap, tag, id, (_vbox__ISerialPort_USCOREsetEnabledResponse *const*)ptr, "vbox:ISerialPort_setEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREsetEnabled:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREsetEnabled(soap, tag, id, (_vbox__ISerialPort_USCOREsetEnabled *const*)ptr, "vbox:ISerialPort_setEnabled");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetEnabledResponse:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREgetEnabledResponse(soap, tag, id, (_vbox__ISerialPort_USCOREgetEnabledResponse *const*)ptr, "vbox:ISerialPort_getEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetEnabled:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREgetEnabled(soap, tag, id, (_vbox__ISerialPort_USCOREgetEnabled *const*)ptr, "vbox:ISerialPort_getEnabled");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetSlotResponse:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREgetSlotResponse(soap, tag, id, (_vbox__ISerialPort_USCOREgetSlotResponse *const*)ptr, "vbox:ISerialPort_getSlotResponse");
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetSlot:
		return soap_out_PointerTo_vbox__ISerialPort_USCOREgetSlot(soap, tag, id, (_vbox__ISerialPort_USCOREgetSlot *const*)ptr, "vbox:ISerialPort_getSlot");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREdetachResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREdetachResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREdetachResponse *const*)ptr, "vbox:INetworkAdapter_detachResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREdetach:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREdetach(soap, tag, id, (_vbox__INetworkAdapter_USCOREdetach *const*)ptr, "vbox:INetworkAdapter_detach");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREattachToHostOnlyInterfaceResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREattachToHostOnlyInterfaceResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREattachToHostOnlyInterfaceResponse *const*)ptr, "vbox:INetworkAdapter_attachToHostOnlyInterfaceResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREattachToHostOnlyInterface:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREattachToHostOnlyInterface(soap, tag, id, (_vbox__INetworkAdapter_USCOREattachToHostOnlyInterface *const*)ptr, "vbox:INetworkAdapter_attachToHostOnlyInterface");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREattachToInternalNetworkResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREattachToInternalNetworkResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREattachToInternalNetworkResponse *const*)ptr, "vbox:INetworkAdapter_attachToInternalNetworkResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREattachToInternalNetwork:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREattachToInternalNetwork(soap, tag, id, (_vbox__INetworkAdapter_USCOREattachToInternalNetwork *const*)ptr, "vbox:INetworkAdapter_attachToInternalNetwork");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREattachToBridgedInterfaceResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREattachToBridgedInterfaceResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREattachToBridgedInterfaceResponse *const*)ptr, "vbox:INetworkAdapter_attachToBridgedInterfaceResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREattachToBridgedInterface:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREattachToBridgedInterface(soap, tag, id, (_vbox__INetworkAdapter_USCOREattachToBridgedInterface *const*)ptr, "vbox:INetworkAdapter_attachToBridgedInterface");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREattachToNATResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREattachToNATResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREattachToNATResponse *const*)ptr, "vbox:INetworkAdapter_attachToNATResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREattachToNAT:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREattachToNAT(soap, tag, id, (_vbox__INetworkAdapter_USCOREattachToNAT *const*)ptr, "vbox:INetworkAdapter_attachToNAT");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetTraceFileResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREsetTraceFileResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREsetTraceFileResponse *const*)ptr, "vbox:INetworkAdapter_setTraceFileResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetTraceFile:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREsetTraceFile(soap, tag, id, (_vbox__INetworkAdapter_USCOREsetTraceFile *const*)ptr, "vbox:INetworkAdapter_setTraceFile");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetTraceFileResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREgetTraceFileResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREgetTraceFileResponse *const*)ptr, "vbox:INetworkAdapter_getTraceFileResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetTraceFile:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREgetTraceFile(soap, tag, id, (_vbox__INetworkAdapter_USCOREgetTraceFile *const*)ptr, "vbox:INetworkAdapter_getTraceFile");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetTraceEnabledResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREsetTraceEnabledResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREsetTraceEnabledResponse *const*)ptr, "vbox:INetworkAdapter_setTraceEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetTraceEnabled:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREsetTraceEnabled(soap, tag, id, (_vbox__INetworkAdapter_USCOREsetTraceEnabled *const*)ptr, "vbox:INetworkAdapter_setTraceEnabled");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetTraceEnabledResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREgetTraceEnabledResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREgetTraceEnabledResponse *const*)ptr, "vbox:INetworkAdapter_getTraceEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetTraceEnabled:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREgetTraceEnabled(soap, tag, id, (_vbox__INetworkAdapter_USCOREgetTraceEnabled *const*)ptr, "vbox:INetworkAdapter_getTraceEnabled");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetLineSpeedResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREsetLineSpeedResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREsetLineSpeedResponse *const*)ptr, "vbox:INetworkAdapter_setLineSpeedResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetLineSpeed:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREsetLineSpeed(soap, tag, id, (_vbox__INetworkAdapter_USCOREsetLineSpeed *const*)ptr, "vbox:INetworkAdapter_setLineSpeed");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetLineSpeedResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREgetLineSpeedResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREgetLineSpeedResponse *const*)ptr, "vbox:INetworkAdapter_getLineSpeedResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetLineSpeed:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREgetLineSpeed(soap, tag, id, (_vbox__INetworkAdapter_USCOREgetLineSpeed *const*)ptr, "vbox:INetworkAdapter_getLineSpeed");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetCableConnectedResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREsetCableConnectedResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREsetCableConnectedResponse *const*)ptr, "vbox:INetworkAdapter_setCableConnectedResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetCableConnected:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREsetCableConnected(soap, tag, id, (_vbox__INetworkAdapter_USCOREsetCableConnected *const*)ptr, "vbox:INetworkAdapter_setCableConnected");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetCableConnectedResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREgetCableConnectedResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREgetCableConnectedResponse *const*)ptr, "vbox:INetworkAdapter_getCableConnectedResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetCableConnected:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREgetCableConnected(soap, tag, id, (_vbox__INetworkAdapter_USCOREgetCableConnected *const*)ptr, "vbox:INetworkAdapter_getCableConnected");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetNATNetworkResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREsetNATNetworkResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREsetNATNetworkResponse *const*)ptr, "vbox:INetworkAdapter_setNATNetworkResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetNATNetwork:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREsetNATNetwork(soap, tag, id, (_vbox__INetworkAdapter_USCOREsetNATNetwork *const*)ptr, "vbox:INetworkAdapter_setNATNetwork");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetNATNetworkResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREgetNATNetworkResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREgetNATNetworkResponse *const*)ptr, "vbox:INetworkAdapter_getNATNetworkResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetNATNetwork:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREgetNATNetwork(soap, tag, id, (_vbox__INetworkAdapter_USCOREgetNATNetwork *const*)ptr, "vbox:INetworkAdapter_getNATNetwork");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetInternalNetworkResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREsetInternalNetworkResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREsetInternalNetworkResponse *const*)ptr, "vbox:INetworkAdapter_setInternalNetworkResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetInternalNetwork:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREsetInternalNetwork(soap, tag, id, (_vbox__INetworkAdapter_USCOREsetInternalNetwork *const*)ptr, "vbox:INetworkAdapter_setInternalNetwork");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetInternalNetworkResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREgetInternalNetworkResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREgetInternalNetworkResponse *const*)ptr, "vbox:INetworkAdapter_getInternalNetworkResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetInternalNetwork:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREgetInternalNetwork(soap, tag, id, (_vbox__INetworkAdapter_USCOREgetInternalNetwork *const*)ptr, "vbox:INetworkAdapter_getInternalNetwork");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetHostInterfaceResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREsetHostInterfaceResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREsetHostInterfaceResponse *const*)ptr, "vbox:INetworkAdapter_setHostInterfaceResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetHostInterface:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREsetHostInterface(soap, tag, id, (_vbox__INetworkAdapter_USCOREsetHostInterface *const*)ptr, "vbox:INetworkAdapter_setHostInterface");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetHostInterfaceResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREgetHostInterfaceResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREgetHostInterfaceResponse *const*)ptr, "vbox:INetworkAdapter_getHostInterfaceResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetHostInterface:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREgetHostInterface(soap, tag, id, (_vbox__INetworkAdapter_USCOREgetHostInterface *const*)ptr, "vbox:INetworkAdapter_getHostInterface");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetAttachmentTypeResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREgetAttachmentTypeResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREgetAttachmentTypeResponse *const*)ptr, "vbox:INetworkAdapter_getAttachmentTypeResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetAttachmentType:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREgetAttachmentType(soap, tag, id, (_vbox__INetworkAdapter_USCOREgetAttachmentType *const*)ptr, "vbox:INetworkAdapter_getAttachmentType");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetMACAddressResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREsetMACAddressResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREsetMACAddressResponse *const*)ptr, "vbox:INetworkAdapter_setMACAddressResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetMACAddress:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREsetMACAddress(soap, tag, id, (_vbox__INetworkAdapter_USCOREsetMACAddress *const*)ptr, "vbox:INetworkAdapter_setMACAddress");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetMACAddressResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREgetMACAddressResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREgetMACAddressResponse *const*)ptr, "vbox:INetworkAdapter_getMACAddressResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetMACAddress:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREgetMACAddress(soap, tag, id, (_vbox__INetworkAdapter_USCOREgetMACAddress *const*)ptr, "vbox:INetworkAdapter_getMACAddress");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetEnabledResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREsetEnabledResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREsetEnabledResponse *const*)ptr, "vbox:INetworkAdapter_setEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetEnabled:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREsetEnabled(soap, tag, id, (_vbox__INetworkAdapter_USCOREsetEnabled *const*)ptr, "vbox:INetworkAdapter_setEnabled");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetEnabledResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREgetEnabledResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREgetEnabledResponse *const*)ptr, "vbox:INetworkAdapter_getEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetEnabled:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREgetEnabled(soap, tag, id, (_vbox__INetworkAdapter_USCOREgetEnabled *const*)ptr, "vbox:INetworkAdapter_getEnabled");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetSlotResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREgetSlotResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREgetSlotResponse *const*)ptr, "vbox:INetworkAdapter_getSlotResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetSlot:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREgetSlot(soap, tag, id, (_vbox__INetworkAdapter_USCOREgetSlot *const*)ptr, "vbox:INetworkAdapter_getSlot");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetAdapterTypeResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREsetAdapterTypeResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREsetAdapterTypeResponse *const*)ptr, "vbox:INetworkAdapter_setAdapterTypeResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetAdapterType:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREsetAdapterType(soap, tag, id, (_vbox__INetworkAdapter_USCOREsetAdapterType *const*)ptr, "vbox:INetworkAdapter_setAdapterType");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetAdapterTypeResponse:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREgetAdapterTypeResponse(soap, tag, id, (_vbox__INetworkAdapter_USCOREgetAdapterTypeResponse *const*)ptr, "vbox:INetworkAdapter_getAdapterTypeResponse");
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetAdapterType:
		return soap_out_PointerTo_vbox__INetworkAdapter_USCOREgetAdapterType(soap, tag, id, (_vbox__INetworkAdapter_USCOREgetAdapterType *const*)ptr, "vbox:INetworkAdapter_getAdapterType");
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREupdateCompletedResponse:
		return soap_out_PointerTo_vbox__IDisplay_USCOREupdateCompletedResponse(soap, tag, id, (_vbox__IDisplay_USCOREupdateCompletedResponse *const*)ptr, "vbox:IDisplay_updateCompletedResponse");
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREupdateCompleted:
		return soap_out_PointerTo_vbox__IDisplay_USCOREupdateCompleted(soap, tag, id, (_vbox__IDisplay_USCOREupdateCompleted *const*)ptr, "vbox:IDisplay_updateCompleted");
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREresizeCompletedResponse:
		return soap_out_PointerTo_vbox__IDisplay_USCOREresizeCompletedResponse(soap, tag, id, (_vbox__IDisplay_USCOREresizeCompletedResponse *const*)ptr, "vbox:IDisplay_resizeCompletedResponse");
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREresizeCompleted:
		return soap_out_PointerTo_vbox__IDisplay_USCOREresizeCompleted(soap, tag, id, (_vbox__IDisplay_USCOREresizeCompleted *const*)ptr, "vbox:IDisplay_resizeCompleted");
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREinvalidateAndUpdateResponse:
		return soap_out_PointerTo_vbox__IDisplay_USCOREinvalidateAndUpdateResponse(soap, tag, id, (_vbox__IDisplay_USCOREinvalidateAndUpdateResponse *const*)ptr, "vbox:IDisplay_invalidateAndUpdateResponse");
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREinvalidateAndUpdate:
		return soap_out_PointerTo_vbox__IDisplay_USCOREinvalidateAndUpdate(soap, tag, id, (_vbox__IDisplay_USCOREinvalidateAndUpdate *const*)ptr, "vbox:IDisplay_invalidateAndUpdate");
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREtakeScreenShotSlowResponse:
		return soap_out_PointerTo_vbox__IDisplay_USCOREtakeScreenShotSlowResponse(soap, tag, id, (_vbox__IDisplay_USCOREtakeScreenShotSlowResponse *const*)ptr, "vbox:IDisplay_takeScreenShotSlowResponse");
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREtakeScreenShotSlow:
		return soap_out_PointerTo_vbox__IDisplay_USCOREtakeScreenShotSlow(soap, tag, id, (_vbox__IDisplay_USCOREtakeScreenShotSlow *const*)ptr, "vbox:IDisplay_takeScreenShotSlow");
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREsetSeamlessModeResponse:
		return soap_out_PointerTo_vbox__IDisplay_USCOREsetSeamlessModeResponse(soap, tag, id, (_vbox__IDisplay_USCOREsetSeamlessModeResponse *const*)ptr, "vbox:IDisplay_setSeamlessModeResponse");
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREsetSeamlessMode:
		return soap_out_PointerTo_vbox__IDisplay_USCOREsetSeamlessMode(soap, tag, id, (_vbox__IDisplay_USCOREsetSeamlessMode *const*)ptr, "vbox:IDisplay_setSeamlessMode");
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREsetVideoModeHintResponse:
		return soap_out_PointerTo_vbox__IDisplay_USCOREsetVideoModeHintResponse(soap, tag, id, (_vbox__IDisplay_USCOREsetVideoModeHintResponse *const*)ptr, "vbox:IDisplay_setVideoModeHintResponse");
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREsetVideoModeHint:
		return soap_out_PointerTo_vbox__IDisplay_USCOREsetVideoModeHint(soap, tag, id, (_vbox__IDisplay_USCOREsetVideoModeHint *const*)ptr, "vbox:IDisplay_setVideoModeHint");
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREgetBitsPerPixelResponse:
		return soap_out_PointerTo_vbox__IDisplay_USCOREgetBitsPerPixelResponse(soap, tag, id, (_vbox__IDisplay_USCOREgetBitsPerPixelResponse *const*)ptr, "vbox:IDisplay_getBitsPerPixelResponse");
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREgetBitsPerPixel:
		return soap_out_PointerTo_vbox__IDisplay_USCOREgetBitsPerPixel(soap, tag, id, (_vbox__IDisplay_USCOREgetBitsPerPixel *const*)ptr, "vbox:IDisplay_getBitsPerPixel");
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREgetHeightResponse:
		return soap_out_PointerTo_vbox__IDisplay_USCOREgetHeightResponse(soap, tag, id, (_vbox__IDisplay_USCOREgetHeightResponse *const*)ptr, "vbox:IDisplay_getHeightResponse");
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREgetHeight:
		return soap_out_PointerTo_vbox__IDisplay_USCOREgetHeight(soap, tag, id, (_vbox__IDisplay_USCOREgetHeight *const*)ptr, "vbox:IDisplay_getHeight");
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREgetWidthResponse:
		return soap_out_PointerTo_vbox__IDisplay_USCOREgetWidthResponse(soap, tag, id, (_vbox__IDisplay_USCOREgetWidthResponse *const*)ptr, "vbox:IDisplay_getWidthResponse");
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREgetWidth:
		return soap_out_PointerTo_vbox__IDisplay_USCOREgetWidth(soap, tag, id, (_vbox__IDisplay_USCOREgetWidth *const*)ptr, "vbox:IDisplay_getWidth");
	case SOAP_TYPE_PointerTo_vbox__IMouse_USCOREputMouseEventAbsoluteResponse:
		return soap_out_PointerTo_vbox__IMouse_USCOREputMouseEventAbsoluteResponse(soap, tag, id, (_vbox__IMouse_USCOREputMouseEventAbsoluteResponse *const*)ptr, "vbox:IMouse_putMouseEventAbsoluteResponse");
	case SOAP_TYPE_PointerTo_vbox__IMouse_USCOREputMouseEventAbsolute:
		return soap_out_PointerTo_vbox__IMouse_USCOREputMouseEventAbsolute(soap, tag, id, (_vbox__IMouse_USCOREputMouseEventAbsolute *const*)ptr, "vbox:IMouse_putMouseEventAbsolute");
	case SOAP_TYPE_PointerTo_vbox__IMouse_USCOREputMouseEventResponse:
		return soap_out_PointerTo_vbox__IMouse_USCOREputMouseEventResponse(soap, tag, id, (_vbox__IMouse_USCOREputMouseEventResponse *const*)ptr, "vbox:IMouse_putMouseEventResponse");
	case SOAP_TYPE_PointerTo_vbox__IMouse_USCOREputMouseEvent:
		return soap_out_PointerTo_vbox__IMouse_USCOREputMouseEvent(soap, tag, id, (_vbox__IMouse_USCOREputMouseEvent *const*)ptr, "vbox:IMouse_putMouseEvent");
	case SOAP_TYPE_PointerTo_vbox__IMouse_USCOREgetAbsoluteSupportedResponse:
		return soap_out_PointerTo_vbox__IMouse_USCOREgetAbsoluteSupportedResponse(soap, tag, id, (_vbox__IMouse_USCOREgetAbsoluteSupportedResponse *const*)ptr, "vbox:IMouse_getAbsoluteSupportedResponse");
	case SOAP_TYPE_PointerTo_vbox__IMouse_USCOREgetAbsoluteSupported:
		return soap_out_PointerTo_vbox__IMouse_USCOREgetAbsoluteSupported(soap, tag, id, (_vbox__IMouse_USCOREgetAbsoluteSupported *const*)ptr, "vbox:IMouse_getAbsoluteSupported");
	case SOAP_TYPE_PointerTo_vbox__IKeyboard_USCOREputCADResponse:
		return soap_out_PointerTo_vbox__IKeyboard_USCOREputCADResponse(soap, tag, id, (_vbox__IKeyboard_USCOREputCADResponse *const*)ptr, "vbox:IKeyboard_putCADResponse");
	case SOAP_TYPE_PointerTo_vbox__IKeyboard_USCOREputCAD:
		return soap_out_PointerTo_vbox__IKeyboard_USCOREputCAD(soap, tag, id, (_vbox__IKeyboard_USCOREputCAD *const*)ptr, "vbox:IKeyboard_putCAD");
	case SOAP_TYPE_PointerTo_vbox__IKeyboard_USCOREputScancodesResponse:
		return soap_out_PointerTo_vbox__IKeyboard_USCOREputScancodesResponse(soap, tag, id, (_vbox__IKeyboard_USCOREputScancodesResponse *const*)ptr, "vbox:IKeyboard_putScancodesResponse");
	case SOAP_TYPE_PointerTo_vbox__IKeyboard_USCOREputScancodes:
		return soap_out_PointerTo_vbox__IKeyboard_USCOREputScancodes(soap, tag, id, (_vbox__IKeyboard_USCOREputScancodes *const*)ptr, "vbox:IKeyboard_putScancodes");
	case SOAP_TYPE_PointerTo_vbox__IKeyboard_USCOREputScancodeResponse:
		return soap_out_PointerTo_vbox__IKeyboard_USCOREputScancodeResponse(soap, tag, id, (_vbox__IKeyboard_USCOREputScancodeResponse *const*)ptr, "vbox:IKeyboard_putScancodeResponse");
	case SOAP_TYPE_PointerTo_vbox__IKeyboard_USCOREputScancode:
		return soap_out_PointerTo_vbox__IKeyboard_USCOREputScancode(soap, tag, id, (_vbox__IKeyboard_USCOREputScancode *const*)ptr, "vbox:IKeyboard_putScancode");
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREgetHostDriveResponse:
		return soap_out_PointerTo_vbox__IFloppyDrive_USCOREgetHostDriveResponse(soap, tag, id, (_vbox__IFloppyDrive_USCOREgetHostDriveResponse *const*)ptr, "vbox:IFloppyDrive_getHostDriveResponse");
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREgetHostDrive:
		return soap_out_PointerTo_vbox__IFloppyDrive_USCOREgetHostDrive(soap, tag, id, (_vbox__IFloppyDrive_USCOREgetHostDrive *const*)ptr, "vbox:IFloppyDrive_getHostDrive");
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREgetImageResponse:
		return soap_out_PointerTo_vbox__IFloppyDrive_USCOREgetImageResponse(soap, tag, id, (_vbox__IFloppyDrive_USCOREgetImageResponse *const*)ptr, "vbox:IFloppyDrive_getImageResponse");
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREgetImage:
		return soap_out_PointerTo_vbox__IFloppyDrive_USCOREgetImage(soap, tag, id, (_vbox__IFloppyDrive_USCOREgetImage *const*)ptr, "vbox:IFloppyDrive_getImage");
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREunmountResponse:
		return soap_out_PointerTo_vbox__IFloppyDrive_USCOREunmountResponse(soap, tag, id, (_vbox__IFloppyDrive_USCOREunmountResponse *const*)ptr, "vbox:IFloppyDrive_unmountResponse");
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREunmount:
		return soap_out_PointerTo_vbox__IFloppyDrive_USCOREunmount(soap, tag, id, (_vbox__IFloppyDrive_USCOREunmount *const*)ptr, "vbox:IFloppyDrive_unmount");
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREcaptureHostDriveResponse:
		return soap_out_PointerTo_vbox__IFloppyDrive_USCOREcaptureHostDriveResponse(soap, tag, id, (_vbox__IFloppyDrive_USCOREcaptureHostDriveResponse *const*)ptr, "vbox:IFloppyDrive_captureHostDriveResponse");
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREcaptureHostDrive:
		return soap_out_PointerTo_vbox__IFloppyDrive_USCOREcaptureHostDrive(soap, tag, id, (_vbox__IFloppyDrive_USCOREcaptureHostDrive *const*)ptr, "vbox:IFloppyDrive_captureHostDrive");
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREmountImageResponse:
		return soap_out_PointerTo_vbox__IFloppyDrive_USCOREmountImageResponse(soap, tag, id, (_vbox__IFloppyDrive_USCOREmountImageResponse *const*)ptr, "vbox:IFloppyDrive_mountImageResponse");
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREmountImage:
		return soap_out_PointerTo_vbox__IFloppyDrive_USCOREmountImage(soap, tag, id, (_vbox__IFloppyDrive_USCOREmountImage *const*)ptr, "vbox:IFloppyDrive_mountImage");
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREgetStateResponse:
		return soap_out_PointerTo_vbox__IFloppyDrive_USCOREgetStateResponse(soap, tag, id, (_vbox__IFloppyDrive_USCOREgetStateResponse *const*)ptr, "vbox:IFloppyDrive_getStateResponse");
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREgetState:
		return soap_out_PointerTo_vbox__IFloppyDrive_USCOREgetState(soap, tag, id, (_vbox__IFloppyDrive_USCOREgetState *const*)ptr, "vbox:IFloppyDrive_getState");
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREsetEnabledResponse:
		return soap_out_PointerTo_vbox__IFloppyDrive_USCOREsetEnabledResponse(soap, tag, id, (_vbox__IFloppyDrive_USCOREsetEnabledResponse *const*)ptr, "vbox:IFloppyDrive_setEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREsetEnabled:
		return soap_out_PointerTo_vbox__IFloppyDrive_USCOREsetEnabled(soap, tag, id, (_vbox__IFloppyDrive_USCOREsetEnabled *const*)ptr, "vbox:IFloppyDrive_setEnabled");
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREgetEnabledResponse:
		return soap_out_PointerTo_vbox__IFloppyDrive_USCOREgetEnabledResponse(soap, tag, id, (_vbox__IFloppyDrive_USCOREgetEnabledResponse *const*)ptr, "vbox:IFloppyDrive_getEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREgetEnabled:
		return soap_out_PointerTo_vbox__IFloppyDrive_USCOREgetEnabled(soap, tag, id, (_vbox__IFloppyDrive_USCOREgetEnabled *const*)ptr, "vbox:IFloppyDrive_getEnabled");
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREgetHostDriveResponse:
		return soap_out_PointerTo_vbox__IDVDDrive_USCOREgetHostDriveResponse(soap, tag, id, (_vbox__IDVDDrive_USCOREgetHostDriveResponse *const*)ptr, "vbox:IDVDDrive_getHostDriveResponse");
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREgetHostDrive:
		return soap_out_PointerTo_vbox__IDVDDrive_USCOREgetHostDrive(soap, tag, id, (_vbox__IDVDDrive_USCOREgetHostDrive *const*)ptr, "vbox:IDVDDrive_getHostDrive");
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREgetImageResponse:
		return soap_out_PointerTo_vbox__IDVDDrive_USCOREgetImageResponse(soap, tag, id, (_vbox__IDVDDrive_USCOREgetImageResponse *const*)ptr, "vbox:IDVDDrive_getImageResponse");
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREgetImage:
		return soap_out_PointerTo_vbox__IDVDDrive_USCOREgetImage(soap, tag, id, (_vbox__IDVDDrive_USCOREgetImage *const*)ptr, "vbox:IDVDDrive_getImage");
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREunmountResponse:
		return soap_out_PointerTo_vbox__IDVDDrive_USCOREunmountResponse(soap, tag, id, (_vbox__IDVDDrive_USCOREunmountResponse *const*)ptr, "vbox:IDVDDrive_unmountResponse");
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREunmount:
		return soap_out_PointerTo_vbox__IDVDDrive_USCOREunmount(soap, tag, id, (_vbox__IDVDDrive_USCOREunmount *const*)ptr, "vbox:IDVDDrive_unmount");
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREcaptureHostDriveResponse:
		return soap_out_PointerTo_vbox__IDVDDrive_USCOREcaptureHostDriveResponse(soap, tag, id, (_vbox__IDVDDrive_USCOREcaptureHostDriveResponse *const*)ptr, "vbox:IDVDDrive_captureHostDriveResponse");
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREcaptureHostDrive:
		return soap_out_PointerTo_vbox__IDVDDrive_USCOREcaptureHostDrive(soap, tag, id, (_vbox__IDVDDrive_USCOREcaptureHostDrive *const*)ptr, "vbox:IDVDDrive_captureHostDrive");
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREmountImageResponse:
		return soap_out_PointerTo_vbox__IDVDDrive_USCOREmountImageResponse(soap, tag, id, (_vbox__IDVDDrive_USCOREmountImageResponse *const*)ptr, "vbox:IDVDDrive_mountImageResponse");
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREmountImage:
		return soap_out_PointerTo_vbox__IDVDDrive_USCOREmountImage(soap, tag, id, (_vbox__IDVDDrive_USCOREmountImage *const*)ptr, "vbox:IDVDDrive_mountImage");
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREsetPassthroughResponse:
		return soap_out_PointerTo_vbox__IDVDDrive_USCOREsetPassthroughResponse(soap, tag, id, (_vbox__IDVDDrive_USCOREsetPassthroughResponse *const*)ptr, "vbox:IDVDDrive_setPassthroughResponse");
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREsetPassthrough:
		return soap_out_PointerTo_vbox__IDVDDrive_USCOREsetPassthrough(soap, tag, id, (_vbox__IDVDDrive_USCOREsetPassthrough *const*)ptr, "vbox:IDVDDrive_setPassthrough");
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREgetPassthroughResponse:
		return soap_out_PointerTo_vbox__IDVDDrive_USCOREgetPassthroughResponse(soap, tag, id, (_vbox__IDVDDrive_USCOREgetPassthroughResponse *const*)ptr, "vbox:IDVDDrive_getPassthroughResponse");
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREgetPassthrough:
		return soap_out_PointerTo_vbox__IDVDDrive_USCOREgetPassthrough(soap, tag, id, (_vbox__IDVDDrive_USCOREgetPassthrough *const*)ptr, "vbox:IDVDDrive_getPassthrough");
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREgetStateResponse:
		return soap_out_PointerTo_vbox__IDVDDrive_USCOREgetStateResponse(soap, tag, id, (_vbox__IDVDDrive_USCOREgetStateResponse *const*)ptr, "vbox:IDVDDrive_getStateResponse");
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREgetState:
		return soap_out_PointerTo_vbox__IDVDDrive_USCOREgetState(soap, tag, id, (_vbox__IDVDDrive_USCOREgetState *const*)ptr, "vbox:IDVDDrive_getState");
	case SOAP_TYPE_PointerTo_vbox__IHardDiskFormat_USCOREdescribePropertiesResponse:
		return soap_out_PointerTo_vbox__IHardDiskFormat_USCOREdescribePropertiesResponse(soap, tag, id, (_vbox__IHardDiskFormat_USCOREdescribePropertiesResponse *const*)ptr, "vbox:IHardDiskFormat_describePropertiesResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDiskFormat_USCOREdescribeProperties:
		return soap_out_PointerTo_vbox__IHardDiskFormat_USCOREdescribeProperties(soap, tag, id, (_vbox__IHardDiskFormat_USCOREdescribeProperties *const*)ptr, "vbox:IHardDiskFormat_describeProperties");
	case SOAP_TYPE_PointerTo_vbox__IHardDiskFormat_USCOREgetCapabilitiesResponse:
		return soap_out_PointerTo_vbox__IHardDiskFormat_USCOREgetCapabilitiesResponse(soap, tag, id, (_vbox__IHardDiskFormat_USCOREgetCapabilitiesResponse *const*)ptr, "vbox:IHardDiskFormat_getCapabilitiesResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDiskFormat_USCOREgetCapabilities:
		return soap_out_PointerTo_vbox__IHardDiskFormat_USCOREgetCapabilities(soap, tag, id, (_vbox__IHardDiskFormat_USCOREgetCapabilities *const*)ptr, "vbox:IHardDiskFormat_getCapabilities");
	case SOAP_TYPE_PointerTo_vbox__IHardDiskFormat_USCOREgetFileExtensionsResponse:
		return soap_out_PointerTo_vbox__IHardDiskFormat_USCOREgetFileExtensionsResponse(soap, tag, id, (_vbox__IHardDiskFormat_USCOREgetFileExtensionsResponse *const*)ptr, "vbox:IHardDiskFormat_getFileExtensionsResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDiskFormat_USCOREgetFileExtensions:
		return soap_out_PointerTo_vbox__IHardDiskFormat_USCOREgetFileExtensions(soap, tag, id, (_vbox__IHardDiskFormat_USCOREgetFileExtensions *const*)ptr, "vbox:IHardDiskFormat_getFileExtensions");
	case SOAP_TYPE_PointerTo_vbox__IHardDiskFormat_USCOREgetNameResponse:
		return soap_out_PointerTo_vbox__IHardDiskFormat_USCOREgetNameResponse(soap, tag, id, (_vbox__IHardDiskFormat_USCOREgetNameResponse *const*)ptr, "vbox:IHardDiskFormat_getNameResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDiskFormat_USCOREgetName:
		return soap_out_PointerTo_vbox__IHardDiskFormat_USCOREgetName(soap, tag, id, (_vbox__IHardDiskFormat_USCOREgetName *const*)ptr, "vbox:IHardDiskFormat_getName");
	case SOAP_TYPE_PointerTo_vbox__IHardDiskFormat_USCOREgetIdResponse:
		return soap_out_PointerTo_vbox__IHardDiskFormat_USCOREgetIdResponse(soap, tag, id, (_vbox__IHardDiskFormat_USCOREgetIdResponse *const*)ptr, "vbox:IHardDiskFormat_getIdResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDiskFormat_USCOREgetId:
		return soap_out_PointerTo_vbox__IHardDiskFormat_USCOREgetId(soap, tag, id, (_vbox__IHardDiskFormat_USCOREgetId *const*)ptr, "vbox:IHardDiskFormat_getId");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREresetResponse:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREresetResponse(soap, tag, id, (_vbox__IHardDisk_USCOREresetResponse *const*)ptr, "vbox:IHardDisk_resetResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREreset:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREreset(soap, tag, id, (_vbox__IHardDisk_USCOREreset *const*)ptr, "vbox:IHardDisk_reset");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREcompactResponse:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREcompactResponse(soap, tag, id, (_vbox__IHardDisk_USCOREcompactResponse *const*)ptr, "vbox:IHardDisk_compactResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREcompact:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREcompact(soap, tag, id, (_vbox__IHardDisk_USCOREcompact *const*)ptr, "vbox:IHardDisk_compact");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREcloneToResponse:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREcloneToResponse(soap, tag, id, (_vbox__IHardDisk_USCOREcloneToResponse *const*)ptr, "vbox:IHardDisk_cloneToResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREcloneTo:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREcloneTo(soap, tag, id, (_vbox__IHardDisk_USCOREcloneTo *const*)ptr, "vbox:IHardDisk_cloneTo");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREmergeToResponse:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREmergeToResponse(soap, tag, id, (_vbox__IHardDisk_USCOREmergeToResponse *const*)ptr, "vbox:IHardDisk_mergeToResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREmergeTo:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREmergeTo(soap, tag, id, (_vbox__IHardDisk_USCOREmergeTo *const*)ptr, "vbox:IHardDisk_mergeTo");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREcreateDiffStorageResponse:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREcreateDiffStorageResponse(soap, tag, id, (_vbox__IHardDisk_USCOREcreateDiffStorageResponse *const*)ptr, "vbox:IHardDisk_createDiffStorageResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREcreateDiffStorage:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREcreateDiffStorage(soap, tag, id, (_vbox__IHardDisk_USCOREcreateDiffStorage *const*)ptr, "vbox:IHardDisk_createDiffStorage");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREdeleteStorageResponse:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREdeleteStorageResponse(soap, tag, id, (_vbox__IHardDisk_USCOREdeleteStorageResponse *const*)ptr, "vbox:IHardDisk_deleteStorageResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREdeleteStorage:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREdeleteStorage(soap, tag, id, (_vbox__IHardDisk_USCOREdeleteStorage *const*)ptr, "vbox:IHardDisk_deleteStorage");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREcreateBaseStorageResponse:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREcreateBaseStorageResponse(soap, tag, id, (_vbox__IHardDisk_USCOREcreateBaseStorageResponse *const*)ptr, "vbox:IHardDisk_createBaseStorageResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREcreateBaseStorage:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREcreateBaseStorage(soap, tag, id, (_vbox__IHardDisk_USCOREcreateBaseStorage *const*)ptr, "vbox:IHardDisk_createBaseStorage");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREsetPropertiesResponse:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREsetPropertiesResponse(soap, tag, id, (_vbox__IHardDisk_USCOREsetPropertiesResponse *const*)ptr, "vbox:IHardDisk_setPropertiesResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREsetProperties:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREsetProperties(soap, tag, id, (_vbox__IHardDisk_USCOREsetProperties *const*)ptr, "vbox:IHardDisk_setProperties");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetPropertiesResponse:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREgetPropertiesResponse(soap, tag, id, (_vbox__IHardDisk_USCOREgetPropertiesResponse *const*)ptr, "vbox:IHardDisk_getPropertiesResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetProperties:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREgetProperties(soap, tag, id, (_vbox__IHardDisk_USCOREgetProperties *const*)ptr, "vbox:IHardDisk_getProperties");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREsetPropertyResponse:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREsetPropertyResponse(soap, tag, id, (_vbox__IHardDisk_USCOREsetPropertyResponse *const*)ptr, "vbox:IHardDisk_setPropertyResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREsetProperty:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREsetProperty(soap, tag, id, (_vbox__IHardDisk_USCOREsetProperty *const*)ptr, "vbox:IHardDisk_setProperty");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetPropertyResponse:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREgetPropertyResponse(soap, tag, id, (_vbox__IHardDisk_USCOREgetPropertyResponse *const*)ptr, "vbox:IHardDisk_getPropertyResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetProperty:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREgetProperty(soap, tag, id, (_vbox__IHardDisk_USCOREgetProperty *const*)ptr, "vbox:IHardDisk_getProperty");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREsetAutoResetResponse:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREsetAutoResetResponse(soap, tag, id, (_vbox__IHardDisk_USCOREsetAutoResetResponse *const*)ptr, "vbox:IHardDisk_setAutoResetResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREsetAutoReset:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREsetAutoReset(soap, tag, id, (_vbox__IHardDisk_USCOREsetAutoReset *const*)ptr, "vbox:IHardDisk_setAutoReset");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetAutoResetResponse:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREgetAutoResetResponse(soap, tag, id, (_vbox__IHardDisk_USCOREgetAutoResetResponse *const*)ptr, "vbox:IHardDisk_getAutoResetResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetAutoReset:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREgetAutoReset(soap, tag, id, (_vbox__IHardDisk_USCOREgetAutoReset *const*)ptr, "vbox:IHardDisk_getAutoReset");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetLogicalSizeResponse:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREgetLogicalSizeResponse(soap, tag, id, (_vbox__IHardDisk_USCOREgetLogicalSizeResponse *const*)ptr, "vbox:IHardDisk_getLogicalSizeResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetLogicalSize:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREgetLogicalSize(soap, tag, id, (_vbox__IHardDisk_USCOREgetLogicalSize *const*)ptr, "vbox:IHardDisk_getLogicalSize");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetReadOnlyResponse:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREgetReadOnlyResponse(soap, tag, id, (_vbox__IHardDisk_USCOREgetReadOnlyResponse *const*)ptr, "vbox:IHardDisk_getReadOnlyResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetReadOnly:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREgetReadOnly(soap, tag, id, (_vbox__IHardDisk_USCOREgetReadOnly *const*)ptr, "vbox:IHardDisk_getReadOnly");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetRootResponse:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREgetRootResponse(soap, tag, id, (_vbox__IHardDisk_USCOREgetRootResponse *const*)ptr, "vbox:IHardDisk_getRootResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetRoot:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREgetRoot(soap, tag, id, (_vbox__IHardDisk_USCOREgetRoot *const*)ptr, "vbox:IHardDisk_getRoot");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetChildrenResponse:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREgetChildrenResponse(soap, tag, id, (_vbox__IHardDisk_USCOREgetChildrenResponse *const*)ptr, "vbox:IHardDisk_getChildrenResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetChildren:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREgetChildren(soap, tag, id, (_vbox__IHardDisk_USCOREgetChildren *const*)ptr, "vbox:IHardDisk_getChildren");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetParentResponse:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREgetParentResponse(soap, tag, id, (_vbox__IHardDisk_USCOREgetParentResponse *const*)ptr, "vbox:IHardDisk_getParentResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetParent:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREgetParent(soap, tag, id, (_vbox__IHardDisk_USCOREgetParent *const*)ptr, "vbox:IHardDisk_getParent");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREsetTypeResponse:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREsetTypeResponse(soap, tag, id, (_vbox__IHardDisk_USCOREsetTypeResponse *const*)ptr, "vbox:IHardDisk_setTypeResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREsetType:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREsetType(soap, tag, id, (_vbox__IHardDisk_USCOREsetType *const*)ptr, "vbox:IHardDisk_setType");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetTypeResponse:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREgetTypeResponse(soap, tag, id, (_vbox__IHardDisk_USCOREgetTypeResponse *const*)ptr, "vbox:IHardDisk_getTypeResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetType:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREgetType(soap, tag, id, (_vbox__IHardDisk_USCOREgetType *const*)ptr, "vbox:IHardDisk_getType");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetFormatResponse:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREgetFormatResponse(soap, tag, id, (_vbox__IHardDisk_USCOREgetFormatResponse *const*)ptr, "vbox:IHardDisk_getFormatResponse");
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetFormat:
		return soap_out_PointerTo_vbox__IHardDisk_USCOREgetFormat(soap, tag, id, (_vbox__IHardDisk_USCOREgetFormat *const*)ptr, "vbox:IHardDisk_getFormat");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREcloseResponse:
		return soap_out_PointerTo_vbox__IMedium_USCOREcloseResponse(soap, tag, id, (_vbox__IMedium_USCOREcloseResponse *const*)ptr, "vbox:IMedium_closeResponse");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREclose:
		return soap_out_PointerTo_vbox__IMedium_USCOREclose(soap, tag, id, (_vbox__IMedium_USCOREclose *const*)ptr, "vbox:IMedium_close");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREunlockWriteResponse:
		return soap_out_PointerTo_vbox__IMedium_USCOREunlockWriteResponse(soap, tag, id, (_vbox__IMedium_USCOREunlockWriteResponse *const*)ptr, "vbox:IMedium_unlockWriteResponse");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREunlockWrite:
		return soap_out_PointerTo_vbox__IMedium_USCOREunlockWrite(soap, tag, id, (_vbox__IMedium_USCOREunlockWrite *const*)ptr, "vbox:IMedium_unlockWrite");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCORElockWriteResponse:
		return soap_out_PointerTo_vbox__IMedium_USCORElockWriteResponse(soap, tag, id, (_vbox__IMedium_USCORElockWriteResponse *const*)ptr, "vbox:IMedium_lockWriteResponse");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCORElockWrite:
		return soap_out_PointerTo_vbox__IMedium_USCORElockWrite(soap, tag, id, (_vbox__IMedium_USCORElockWrite *const*)ptr, "vbox:IMedium_lockWrite");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREunlockReadResponse:
		return soap_out_PointerTo_vbox__IMedium_USCOREunlockReadResponse(soap, tag, id, (_vbox__IMedium_USCOREunlockReadResponse *const*)ptr, "vbox:IMedium_unlockReadResponse");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREunlockRead:
		return soap_out_PointerTo_vbox__IMedium_USCOREunlockRead(soap, tag, id, (_vbox__IMedium_USCOREunlockRead *const*)ptr, "vbox:IMedium_unlockRead");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCORElockReadResponse:
		return soap_out_PointerTo_vbox__IMedium_USCORElockReadResponse(soap, tag, id, (_vbox__IMedium_USCORElockReadResponse *const*)ptr, "vbox:IMedium_lockReadResponse");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCORElockRead:
		return soap_out_PointerTo_vbox__IMedium_USCORElockRead(soap, tag, id, (_vbox__IMedium_USCORElockRead *const*)ptr, "vbox:IMedium_lockRead");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetSnapshotIdsResponse:
		return soap_out_PointerTo_vbox__IMedium_USCOREgetSnapshotIdsResponse(soap, tag, id, (_vbox__IMedium_USCOREgetSnapshotIdsResponse *const*)ptr, "vbox:IMedium_getSnapshotIdsResponse");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetSnapshotIds:
		return soap_out_PointerTo_vbox__IMedium_USCOREgetSnapshotIds(soap, tag, id, (_vbox__IMedium_USCOREgetSnapshotIds *const*)ptr, "vbox:IMedium_getSnapshotIds");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetMachineIdsResponse:
		return soap_out_PointerTo_vbox__IMedium_USCOREgetMachineIdsResponse(soap, tag, id, (_vbox__IMedium_USCOREgetMachineIdsResponse *const*)ptr, "vbox:IMedium_getMachineIdsResponse");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetMachineIds:
		return soap_out_PointerTo_vbox__IMedium_USCOREgetMachineIds(soap, tag, id, (_vbox__IMedium_USCOREgetMachineIds *const*)ptr, "vbox:IMedium_getMachineIds");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetLastAccessErrorResponse:
		return soap_out_PointerTo_vbox__IMedium_USCOREgetLastAccessErrorResponse(soap, tag, id, (_vbox__IMedium_USCOREgetLastAccessErrorResponse *const*)ptr, "vbox:IMedium_getLastAccessErrorResponse");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetLastAccessError:
		return soap_out_PointerTo_vbox__IMedium_USCOREgetLastAccessError(soap, tag, id, (_vbox__IMedium_USCOREgetLastAccessError *const*)ptr, "vbox:IMedium_getLastAccessError");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetSizeResponse:
		return soap_out_PointerTo_vbox__IMedium_USCOREgetSizeResponse(soap, tag, id, (_vbox__IMedium_USCOREgetSizeResponse *const*)ptr, "vbox:IMedium_getSizeResponse");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetSize:
		return soap_out_PointerTo_vbox__IMedium_USCOREgetSize(soap, tag, id, (_vbox__IMedium_USCOREgetSize *const*)ptr, "vbox:IMedium_getSize");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetNameResponse:
		return soap_out_PointerTo_vbox__IMedium_USCOREgetNameResponse(soap, tag, id, (_vbox__IMedium_USCOREgetNameResponse *const*)ptr, "vbox:IMedium_getNameResponse");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetName:
		return soap_out_PointerTo_vbox__IMedium_USCOREgetName(soap, tag, id, (_vbox__IMedium_USCOREgetName *const*)ptr, "vbox:IMedium_getName");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREsetLocationResponse:
		return soap_out_PointerTo_vbox__IMedium_USCOREsetLocationResponse(soap, tag, id, (_vbox__IMedium_USCOREsetLocationResponse *const*)ptr, "vbox:IMedium_setLocationResponse");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREsetLocation:
		return soap_out_PointerTo_vbox__IMedium_USCOREsetLocation(soap, tag, id, (_vbox__IMedium_USCOREsetLocation *const*)ptr, "vbox:IMedium_setLocation");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetLocationResponse:
		return soap_out_PointerTo_vbox__IMedium_USCOREgetLocationResponse(soap, tag, id, (_vbox__IMedium_USCOREgetLocationResponse *const*)ptr, "vbox:IMedium_getLocationResponse");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetLocation:
		return soap_out_PointerTo_vbox__IMedium_USCOREgetLocation(soap, tag, id, (_vbox__IMedium_USCOREgetLocation *const*)ptr, "vbox:IMedium_getLocation");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetStateResponse:
		return soap_out_PointerTo_vbox__IMedium_USCOREgetStateResponse(soap, tag, id, (_vbox__IMedium_USCOREgetStateResponse *const*)ptr, "vbox:IMedium_getStateResponse");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetState:
		return soap_out_PointerTo_vbox__IMedium_USCOREgetState(soap, tag, id, (_vbox__IMedium_USCOREgetState *const*)ptr, "vbox:IMedium_getState");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREsetDescriptionResponse:
		return soap_out_PointerTo_vbox__IMedium_USCOREsetDescriptionResponse(soap, tag, id, (_vbox__IMedium_USCOREsetDescriptionResponse *const*)ptr, "vbox:IMedium_setDescriptionResponse");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREsetDescription:
		return soap_out_PointerTo_vbox__IMedium_USCOREsetDescription(soap, tag, id, (_vbox__IMedium_USCOREsetDescription *const*)ptr, "vbox:IMedium_setDescription");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetDescriptionResponse:
		return soap_out_PointerTo_vbox__IMedium_USCOREgetDescriptionResponse(soap, tag, id, (_vbox__IMedium_USCOREgetDescriptionResponse *const*)ptr, "vbox:IMedium_getDescriptionResponse");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetDescription:
		return soap_out_PointerTo_vbox__IMedium_USCOREgetDescription(soap, tag, id, (_vbox__IMedium_USCOREgetDescription *const*)ptr, "vbox:IMedium_getDescription");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetIdResponse:
		return soap_out_PointerTo_vbox__IMedium_USCOREgetIdResponse(soap, tag, id, (_vbox__IMedium_USCOREgetIdResponse *const*)ptr, "vbox:IMedium_getIdResponse");
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetId:
		return soap_out_PointerTo_vbox__IMedium_USCOREgetId(soap, tag, id, (_vbox__IMedium_USCOREgetId *const*)ptr, "vbox:IMedium_getId");
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetChildrenResponse:
		return soap_out_PointerTo_vbox__ISnapshot_USCOREgetChildrenResponse(soap, tag, id, (_vbox__ISnapshot_USCOREgetChildrenResponse *const*)ptr, "vbox:ISnapshot_getChildrenResponse");
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetChildren:
		return soap_out_PointerTo_vbox__ISnapshot_USCOREgetChildren(soap, tag, id, (_vbox__ISnapshot_USCOREgetChildren *const*)ptr, "vbox:ISnapshot_getChildren");
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetParentResponse:
		return soap_out_PointerTo_vbox__ISnapshot_USCOREgetParentResponse(soap, tag, id, (_vbox__ISnapshot_USCOREgetParentResponse *const*)ptr, "vbox:ISnapshot_getParentResponse");
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetParent:
		return soap_out_PointerTo_vbox__ISnapshot_USCOREgetParent(soap, tag, id, (_vbox__ISnapshot_USCOREgetParent *const*)ptr, "vbox:ISnapshot_getParent");
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetMachineResponse:
		return soap_out_PointerTo_vbox__ISnapshot_USCOREgetMachineResponse(soap, tag, id, (_vbox__ISnapshot_USCOREgetMachineResponse *const*)ptr, "vbox:ISnapshot_getMachineResponse");
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetMachine:
		return soap_out_PointerTo_vbox__ISnapshot_USCOREgetMachine(soap, tag, id, (_vbox__ISnapshot_USCOREgetMachine *const*)ptr, "vbox:ISnapshot_getMachine");
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetOnlineResponse:
		return soap_out_PointerTo_vbox__ISnapshot_USCOREgetOnlineResponse(soap, tag, id, (_vbox__ISnapshot_USCOREgetOnlineResponse *const*)ptr, "vbox:ISnapshot_getOnlineResponse");
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetOnline:
		return soap_out_PointerTo_vbox__ISnapshot_USCOREgetOnline(soap, tag, id, (_vbox__ISnapshot_USCOREgetOnline *const*)ptr, "vbox:ISnapshot_getOnline");
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetTimeStampResponse:
		return soap_out_PointerTo_vbox__ISnapshot_USCOREgetTimeStampResponse(soap, tag, id, (_vbox__ISnapshot_USCOREgetTimeStampResponse *const*)ptr, "vbox:ISnapshot_getTimeStampResponse");
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetTimeStamp:
		return soap_out_PointerTo_vbox__ISnapshot_USCOREgetTimeStamp(soap, tag, id, (_vbox__ISnapshot_USCOREgetTimeStamp *const*)ptr, "vbox:ISnapshot_getTimeStamp");
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREsetDescriptionResponse:
		return soap_out_PointerTo_vbox__ISnapshot_USCOREsetDescriptionResponse(soap, tag, id, (_vbox__ISnapshot_USCOREsetDescriptionResponse *const*)ptr, "vbox:ISnapshot_setDescriptionResponse");
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREsetDescription:
		return soap_out_PointerTo_vbox__ISnapshot_USCOREsetDescription(soap, tag, id, (_vbox__ISnapshot_USCOREsetDescription *const*)ptr, "vbox:ISnapshot_setDescription");
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetDescriptionResponse:
		return soap_out_PointerTo_vbox__ISnapshot_USCOREgetDescriptionResponse(soap, tag, id, (_vbox__ISnapshot_USCOREgetDescriptionResponse *const*)ptr, "vbox:ISnapshot_getDescriptionResponse");
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetDescription:
		return soap_out_PointerTo_vbox__ISnapshot_USCOREgetDescription(soap, tag, id, (_vbox__ISnapshot_USCOREgetDescription *const*)ptr, "vbox:ISnapshot_getDescription");
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREsetNameResponse:
		return soap_out_PointerTo_vbox__ISnapshot_USCOREsetNameResponse(soap, tag, id, (_vbox__ISnapshot_USCOREsetNameResponse *const*)ptr, "vbox:ISnapshot_setNameResponse");
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREsetName:
		return soap_out_PointerTo_vbox__ISnapshot_USCOREsetName(soap, tag, id, (_vbox__ISnapshot_USCOREsetName *const*)ptr, "vbox:ISnapshot_setName");
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetNameResponse:
		return soap_out_PointerTo_vbox__ISnapshot_USCOREgetNameResponse(soap, tag, id, (_vbox__ISnapshot_USCOREgetNameResponse *const*)ptr, "vbox:ISnapshot_getNameResponse");
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetName:
		return soap_out_PointerTo_vbox__ISnapshot_USCOREgetName(soap, tag, id, (_vbox__ISnapshot_USCOREgetName *const*)ptr, "vbox:ISnapshot_getName");
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetIdResponse:
		return soap_out_PointerTo_vbox__ISnapshot_USCOREgetIdResponse(soap, tag, id, (_vbox__ISnapshot_USCOREgetIdResponse *const*)ptr, "vbox:ISnapshot_getIdResponse");
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetId:
		return soap_out_PointerTo_vbox__ISnapshot_USCOREgetId(soap, tag, id, (_vbox__ISnapshot_USCOREgetId *const*)ptr, "vbox:ISnapshot_getId");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREcancelResponse:
		return soap_out_PointerTo_vbox__IProgress_USCOREcancelResponse(soap, tag, id, (_vbox__IProgress_USCOREcancelResponse *const*)ptr, "vbox:IProgress_cancelResponse");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREcancel:
		return soap_out_PointerTo_vbox__IProgress_USCOREcancel(soap, tag, id, (_vbox__IProgress_USCOREcancel *const*)ptr, "vbox:IProgress_cancel");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREwaitForOperationCompletionResponse:
		return soap_out_PointerTo_vbox__IProgress_USCOREwaitForOperationCompletionResponse(soap, tag, id, (_vbox__IProgress_USCOREwaitForOperationCompletionResponse *const*)ptr, "vbox:IProgress_waitForOperationCompletionResponse");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREwaitForOperationCompletion:
		return soap_out_PointerTo_vbox__IProgress_USCOREwaitForOperationCompletion(soap, tag, id, (_vbox__IProgress_USCOREwaitForOperationCompletion *const*)ptr, "vbox:IProgress_waitForOperationCompletion");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREwaitForCompletionResponse:
		return soap_out_PointerTo_vbox__IProgress_USCOREwaitForCompletionResponse(soap, tag, id, (_vbox__IProgress_USCOREwaitForCompletionResponse *const*)ptr, "vbox:IProgress_waitForCompletionResponse");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREwaitForCompletion:
		return soap_out_PointerTo_vbox__IProgress_USCOREwaitForCompletion(soap, tag, id, (_vbox__IProgress_USCOREwaitForCompletion *const*)ptr, "vbox:IProgress_waitForCompletion");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetOperationPercentResponse:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetOperationPercentResponse(soap, tag, id, (_vbox__IProgress_USCOREgetOperationPercentResponse *const*)ptr, "vbox:IProgress_getOperationPercentResponse");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetOperationPercent:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetOperationPercent(soap, tag, id, (_vbox__IProgress_USCOREgetOperationPercent *const*)ptr, "vbox:IProgress_getOperationPercent");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetOperationDescriptionResponse:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetOperationDescriptionResponse(soap, tag, id, (_vbox__IProgress_USCOREgetOperationDescriptionResponse *const*)ptr, "vbox:IProgress_getOperationDescriptionResponse");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetOperationDescription:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetOperationDescription(soap, tag, id, (_vbox__IProgress_USCOREgetOperationDescription *const*)ptr, "vbox:IProgress_getOperationDescription");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetOperationResponse:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetOperationResponse(soap, tag, id, (_vbox__IProgress_USCOREgetOperationResponse *const*)ptr, "vbox:IProgress_getOperationResponse");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetOperation:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetOperation(soap, tag, id, (_vbox__IProgress_USCOREgetOperation *const*)ptr, "vbox:IProgress_getOperation");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetOperationCountResponse:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetOperationCountResponse(soap, tag, id, (_vbox__IProgress_USCOREgetOperationCountResponse *const*)ptr, "vbox:IProgress_getOperationCountResponse");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetOperationCount:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetOperationCount(soap, tag, id, (_vbox__IProgress_USCOREgetOperationCount *const*)ptr, "vbox:IProgress_getOperationCount");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetErrorInfoResponse:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetErrorInfoResponse(soap, tag, id, (_vbox__IProgress_USCOREgetErrorInfoResponse *const*)ptr, "vbox:IProgress_getErrorInfoResponse");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetErrorInfo:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetErrorInfo(soap, tag, id, (_vbox__IProgress_USCOREgetErrorInfo *const*)ptr, "vbox:IProgress_getErrorInfo");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetResultCodeResponse:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetResultCodeResponse(soap, tag, id, (_vbox__IProgress_USCOREgetResultCodeResponse *const*)ptr, "vbox:IProgress_getResultCodeResponse");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetResultCode:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetResultCode(soap, tag, id, (_vbox__IProgress_USCOREgetResultCode *const*)ptr, "vbox:IProgress_getResultCode");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetCanceledResponse:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetCanceledResponse(soap, tag, id, (_vbox__IProgress_USCOREgetCanceledResponse *const*)ptr, "vbox:IProgress_getCanceledResponse");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetCanceled:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetCanceled(soap, tag, id, (_vbox__IProgress_USCOREgetCanceled *const*)ptr, "vbox:IProgress_getCanceled");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetCompletedResponse:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetCompletedResponse(soap, tag, id, (_vbox__IProgress_USCOREgetCompletedResponse *const*)ptr, "vbox:IProgress_getCompletedResponse");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetCompleted:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetCompleted(soap, tag, id, (_vbox__IProgress_USCOREgetCompleted *const*)ptr, "vbox:IProgress_getCompleted");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetTimeRemainingResponse:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetTimeRemainingResponse(soap, tag, id, (_vbox__IProgress_USCOREgetTimeRemainingResponse *const*)ptr, "vbox:IProgress_getTimeRemainingResponse");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetTimeRemaining:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetTimeRemaining(soap, tag, id, (_vbox__IProgress_USCOREgetTimeRemaining *const*)ptr, "vbox:IProgress_getTimeRemaining");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetPercentResponse:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetPercentResponse(soap, tag, id, (_vbox__IProgress_USCOREgetPercentResponse *const*)ptr, "vbox:IProgress_getPercentResponse");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetPercent:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetPercent(soap, tag, id, (_vbox__IProgress_USCOREgetPercent *const*)ptr, "vbox:IProgress_getPercent");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetCancelableResponse:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetCancelableResponse(soap, tag, id, (_vbox__IProgress_USCOREgetCancelableResponse *const*)ptr, "vbox:IProgress_getCancelableResponse");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetCancelable:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetCancelable(soap, tag, id, (_vbox__IProgress_USCOREgetCancelable *const*)ptr, "vbox:IProgress_getCancelable");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetInitiatorResponse:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetInitiatorResponse(soap, tag, id, (_vbox__IProgress_USCOREgetInitiatorResponse *const*)ptr, "vbox:IProgress_getInitiatorResponse");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetInitiator:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetInitiator(soap, tag, id, (_vbox__IProgress_USCOREgetInitiator *const*)ptr, "vbox:IProgress_getInitiator");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetDescriptionResponse:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetDescriptionResponse(soap, tag, id, (_vbox__IProgress_USCOREgetDescriptionResponse *const*)ptr, "vbox:IProgress_getDescriptionResponse");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetDescription:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetDescription(soap, tag, id, (_vbox__IProgress_USCOREgetDescription *const*)ptr, "vbox:IProgress_getDescription");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetIdResponse:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetIdResponse(soap, tag, id, (_vbox__IProgress_USCOREgetIdResponse *const*)ptr, "vbox:IProgress_getIdResponse");
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetId:
		return soap_out_PointerTo_vbox__IProgress_USCOREgetId(soap, tag, id, (_vbox__IProgress_USCOREgetId *const*)ptr, "vbox:IProgress_getId");
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetStatisticResponse:
		return soap_out_PointerTo_vbox__IGuest_USCOREgetStatisticResponse(soap, tag, id, (_vbox__IGuest_USCOREgetStatisticResponse *const*)ptr, "vbox:IGuest_getStatisticResponse");
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetStatistic:
		return soap_out_PointerTo_vbox__IGuest_USCOREgetStatistic(soap, tag, id, (_vbox__IGuest_USCOREgetStatistic *const*)ptr, "vbox:IGuest_getStatistic");
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREsetCredentialsResponse:
		return soap_out_PointerTo_vbox__IGuest_USCOREsetCredentialsResponse(soap, tag, id, (_vbox__IGuest_USCOREsetCredentialsResponse *const*)ptr, "vbox:IGuest_setCredentialsResponse");
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREsetCredentials:
		return soap_out_PointerTo_vbox__IGuest_USCOREsetCredentials(soap, tag, id, (_vbox__IGuest_USCOREsetCredentials *const*)ptr, "vbox:IGuest_setCredentials");
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREsetStatisticsUpdateIntervalResponse:
		return soap_out_PointerTo_vbox__IGuest_USCOREsetStatisticsUpdateIntervalResponse(soap, tag, id, (_vbox__IGuest_USCOREsetStatisticsUpdateIntervalResponse *const*)ptr, "vbox:IGuest_setStatisticsUpdateIntervalResponse");
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREsetStatisticsUpdateInterval:
		return soap_out_PointerTo_vbox__IGuest_USCOREsetStatisticsUpdateInterval(soap, tag, id, (_vbox__IGuest_USCOREsetStatisticsUpdateInterval *const*)ptr, "vbox:IGuest_setStatisticsUpdateInterval");
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetStatisticsUpdateIntervalResponse:
		return soap_out_PointerTo_vbox__IGuest_USCOREgetStatisticsUpdateIntervalResponse(soap, tag, id, (_vbox__IGuest_USCOREgetStatisticsUpdateIntervalResponse *const*)ptr, "vbox:IGuest_getStatisticsUpdateIntervalResponse");
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetStatisticsUpdateInterval:
		return soap_out_PointerTo_vbox__IGuest_USCOREgetStatisticsUpdateInterval(soap, tag, id, (_vbox__IGuest_USCOREgetStatisticsUpdateInterval *const*)ptr, "vbox:IGuest_getStatisticsUpdateInterval");
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREsetMemoryBalloonSizeResponse:
		return soap_out_PointerTo_vbox__IGuest_USCOREsetMemoryBalloonSizeResponse(soap, tag, id, (_vbox__IGuest_USCOREsetMemoryBalloonSizeResponse *const*)ptr, "vbox:IGuest_setMemoryBalloonSizeResponse");
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREsetMemoryBalloonSize:
		return soap_out_PointerTo_vbox__IGuest_USCOREsetMemoryBalloonSize(soap, tag, id, (_vbox__IGuest_USCOREsetMemoryBalloonSize *const*)ptr, "vbox:IGuest_setMemoryBalloonSize");
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetMemoryBalloonSizeResponse:
		return soap_out_PointerTo_vbox__IGuest_USCOREgetMemoryBalloonSizeResponse(soap, tag, id, (_vbox__IGuest_USCOREgetMemoryBalloonSizeResponse *const*)ptr, "vbox:IGuest_getMemoryBalloonSizeResponse");
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetMemoryBalloonSize:
		return soap_out_PointerTo_vbox__IGuest_USCOREgetMemoryBalloonSize(soap, tag, id, (_vbox__IGuest_USCOREgetMemoryBalloonSize *const*)ptr, "vbox:IGuest_getMemoryBalloonSize");
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetSupportsGraphicsResponse:
		return soap_out_PointerTo_vbox__IGuest_USCOREgetSupportsGraphicsResponse(soap, tag, id, (_vbox__IGuest_USCOREgetSupportsGraphicsResponse *const*)ptr, "vbox:IGuest_getSupportsGraphicsResponse");
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetSupportsGraphics:
		return soap_out_PointerTo_vbox__IGuest_USCOREgetSupportsGraphics(soap, tag, id, (_vbox__IGuest_USCOREgetSupportsGraphics *const*)ptr, "vbox:IGuest_getSupportsGraphics");
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetSupportsSeamlessResponse:
		return soap_out_PointerTo_vbox__IGuest_USCOREgetSupportsSeamlessResponse(soap, tag, id, (_vbox__IGuest_USCOREgetSupportsSeamlessResponse *const*)ptr, "vbox:IGuest_getSupportsSeamlessResponse");
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetSupportsSeamless:
		return soap_out_PointerTo_vbox__IGuest_USCOREgetSupportsSeamless(soap, tag, id, (_vbox__IGuest_USCOREgetSupportsSeamless *const*)ptr, "vbox:IGuest_getSupportsSeamless");
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetAdditionsVersionResponse:
		return soap_out_PointerTo_vbox__IGuest_USCOREgetAdditionsVersionResponse(soap, tag, id, (_vbox__IGuest_USCOREgetAdditionsVersionResponse *const*)ptr, "vbox:IGuest_getAdditionsVersionResponse");
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetAdditionsVersion:
		return soap_out_PointerTo_vbox__IGuest_USCOREgetAdditionsVersion(soap, tag, id, (_vbox__IGuest_USCOREgetAdditionsVersion *const*)ptr, "vbox:IGuest_getAdditionsVersion");
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetAdditionsActiveResponse:
		return soap_out_PointerTo_vbox__IGuest_USCOREgetAdditionsActiveResponse(soap, tag, id, (_vbox__IGuest_USCOREgetAdditionsActiveResponse *const*)ptr, "vbox:IGuest_getAdditionsActiveResponse");
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetAdditionsActive:
		return soap_out_PointerTo_vbox__IGuest_USCOREgetAdditionsActive(soap, tag, id, (_vbox__IGuest_USCOREgetAdditionsActive *const*)ptr, "vbox:IGuest_getAdditionsActive");
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetOSTypeIdResponse:
		return soap_out_PointerTo_vbox__IGuest_USCOREgetOSTypeIdResponse(soap, tag, id, (_vbox__IGuest_USCOREgetOSTypeIdResponse *const*)ptr, "vbox:IGuest_getOSTypeIdResponse");
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetOSTypeId:
		return soap_out_PointerTo_vbox__IGuest_USCOREgetOSTypeId(soap, tag, id, (_vbox__IGuest_USCOREgetOSTypeId *const*)ptr, "vbox:IGuest_getOSTypeId");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetDefaultAudioDriverResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetDefaultAudioDriverResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREgetDefaultAudioDriverResponse *const*)ptr, "vbox:ISystemProperties_getDefaultAudioDriverResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetDefaultAudioDriver:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetDefaultAudioDriver(soap, tag, id, (_vbox__ISystemProperties_USCOREgetDefaultAudioDriver *const*)ptr, "vbox:ISystemProperties_getDefaultAudioDriver");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREsetLogHistoryCountResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREsetLogHistoryCountResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREsetLogHistoryCountResponse *const*)ptr, "vbox:ISystemProperties_setLogHistoryCountResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREsetLogHistoryCount:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREsetLogHistoryCount(soap, tag, id, (_vbox__ISystemProperties_USCOREsetLogHistoryCount *const*)ptr, "vbox:ISystemProperties_setLogHistoryCount");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetLogHistoryCountResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetLogHistoryCountResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREgetLogHistoryCountResponse *const*)ptr, "vbox:ISystemProperties_getLogHistoryCountResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetLogHistoryCount:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetLogHistoryCount(soap, tag, id, (_vbox__ISystemProperties_USCOREgetLogHistoryCount *const*)ptr, "vbox:ISystemProperties_getLogHistoryCount");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREsetWebServiceAuthLibraryResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREsetWebServiceAuthLibraryResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREsetWebServiceAuthLibraryResponse *const*)ptr, "vbox:ISystemProperties_setWebServiceAuthLibraryResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREsetWebServiceAuthLibrary:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREsetWebServiceAuthLibrary(soap, tag, id, (_vbox__ISystemProperties_USCOREsetWebServiceAuthLibrary *const*)ptr, "vbox:ISystemProperties_setWebServiceAuthLibrary");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetWebServiceAuthLibraryResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetWebServiceAuthLibraryResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREgetWebServiceAuthLibraryResponse *const*)ptr, "vbox:ISystemProperties_getWebServiceAuthLibraryResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetWebServiceAuthLibrary:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetWebServiceAuthLibrary(soap, tag, id, (_vbox__ISystemProperties_USCOREgetWebServiceAuthLibrary *const*)ptr, "vbox:ISystemProperties_getWebServiceAuthLibrary");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibraryResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibraryResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibraryResponse *const*)ptr, "vbox:ISystemProperties_setRemoteDisplayAuthLibraryResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibrary:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibrary(soap, tag, id, (_vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibrary *const*)ptr, "vbox:ISystemProperties_setRemoteDisplayAuthLibrary");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibraryResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibraryResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibraryResponse *const*)ptr, "vbox:ISystemProperties_getRemoteDisplayAuthLibraryResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibrary:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibrary(soap, tag, id, (_vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibrary *const*)ptr, "vbox:ISystemProperties_getRemoteDisplayAuthLibrary");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREsetDefaultHardDiskFormatResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREsetDefaultHardDiskFormatResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREsetDefaultHardDiskFormatResponse *const*)ptr, "vbox:ISystemProperties_setDefaultHardDiskFormatResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREsetDefaultHardDiskFormat:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREsetDefaultHardDiskFormat(soap, tag, id, (_vbox__ISystemProperties_USCOREsetDefaultHardDiskFormat *const*)ptr, "vbox:ISystemProperties_setDefaultHardDiskFormat");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetDefaultHardDiskFormatResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetDefaultHardDiskFormatResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREgetDefaultHardDiskFormatResponse *const*)ptr, "vbox:ISystemProperties_getDefaultHardDiskFormatResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetDefaultHardDiskFormat:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetDefaultHardDiskFormat(soap, tag, id, (_vbox__ISystemProperties_USCOREgetDefaultHardDiskFormat *const*)ptr, "vbox:ISystemProperties_getDefaultHardDiskFormat");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetHardDiskFormatsResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetHardDiskFormatsResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREgetHardDiskFormatsResponse *const*)ptr, "vbox:ISystemProperties_getHardDiskFormatsResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetHardDiskFormats:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetHardDiskFormats(soap, tag, id, (_vbox__ISystemProperties_USCOREgetHardDiskFormats *const*)ptr, "vbox:ISystemProperties_getHardDiskFormats");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREsetDefaultHardDiskFolderResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREsetDefaultHardDiskFolderResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREsetDefaultHardDiskFolderResponse *const*)ptr, "vbox:ISystemProperties_setDefaultHardDiskFolderResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREsetDefaultHardDiskFolder:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREsetDefaultHardDiskFolder(soap, tag, id, (_vbox__ISystemProperties_USCOREsetDefaultHardDiskFolder *const*)ptr, "vbox:ISystemProperties_setDefaultHardDiskFolder");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetDefaultHardDiskFolderResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetDefaultHardDiskFolderResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREgetDefaultHardDiskFolderResponse *const*)ptr, "vbox:ISystemProperties_getDefaultHardDiskFolderResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetDefaultHardDiskFolder:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetDefaultHardDiskFolder(soap, tag, id, (_vbox__ISystemProperties_USCOREgetDefaultHardDiskFolder *const*)ptr, "vbox:ISystemProperties_getDefaultHardDiskFolder");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREsetDefaultMachineFolderResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREsetDefaultMachineFolderResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREsetDefaultMachineFolderResponse *const*)ptr, "vbox:ISystemProperties_setDefaultMachineFolderResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREsetDefaultMachineFolder:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREsetDefaultMachineFolder(soap, tag, id, (_vbox__ISystemProperties_USCOREsetDefaultMachineFolder *const*)ptr, "vbox:ISystemProperties_setDefaultMachineFolder");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetDefaultMachineFolderResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetDefaultMachineFolderResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREgetDefaultMachineFolderResponse *const*)ptr, "vbox:ISystemProperties_getDefaultMachineFolderResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetDefaultMachineFolder:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetDefaultMachineFolder(soap, tag, id, (_vbox__ISystemProperties_USCOREgetDefaultMachineFolder *const*)ptr, "vbox:ISystemProperties_getDefaultMachineFolder");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMaxBootPositionResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetMaxBootPositionResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREgetMaxBootPositionResponse *const*)ptr, "vbox:ISystemProperties_getMaxBootPositionResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMaxBootPosition:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetMaxBootPosition(soap, tag, id, (_vbox__ISystemProperties_USCOREgetMaxBootPosition *const*)ptr, "vbox:ISystemProperties_getMaxBootPosition");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetParallelPortCountResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetParallelPortCountResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREgetParallelPortCountResponse *const*)ptr, "vbox:ISystemProperties_getParallelPortCountResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetParallelPortCount:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetParallelPortCount(soap, tag, id, (_vbox__ISystemProperties_USCOREgetParallelPortCount *const*)ptr, "vbox:ISystemProperties_getParallelPortCount");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetSerialPortCountResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetSerialPortCountResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREgetSerialPortCountResponse *const*)ptr, "vbox:ISystemProperties_getSerialPortCountResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetSerialPortCount:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetSerialPortCount(soap, tag, id, (_vbox__ISystemProperties_USCOREgetSerialPortCount *const*)ptr, "vbox:ISystemProperties_getSerialPortCount");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetNetworkAdapterCountResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetNetworkAdapterCountResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREgetNetworkAdapterCountResponse *const*)ptr, "vbox:ISystemProperties_getNetworkAdapterCountResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetNetworkAdapterCount:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetNetworkAdapterCount(soap, tag, id, (_vbox__ISystemProperties_USCOREgetNetworkAdapterCount *const*)ptr, "vbox:ISystemProperties_getNetworkAdapterCount");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMaxVDISizeResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetMaxVDISizeResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREgetMaxVDISizeResponse *const*)ptr, "vbox:ISystemProperties_getMaxVDISizeResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMaxVDISize:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetMaxVDISize(soap, tag, id, (_vbox__ISystemProperties_USCOREgetMaxVDISize *const*)ptr, "vbox:ISystemProperties_getMaxVDISize");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMaxGuestCPUCountResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetMaxGuestCPUCountResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREgetMaxGuestCPUCountResponse *const*)ptr, "vbox:ISystemProperties_getMaxGuestCPUCountResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMaxGuestCPUCount:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetMaxGuestCPUCount(soap, tag, id, (_vbox__ISystemProperties_USCOREgetMaxGuestCPUCount *const*)ptr, "vbox:ISystemProperties_getMaxGuestCPUCount");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMinGuestCPUCountResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetMinGuestCPUCountResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREgetMinGuestCPUCountResponse *const*)ptr, "vbox:ISystemProperties_getMinGuestCPUCountResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMinGuestCPUCount:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetMinGuestCPUCount(soap, tag, id, (_vbox__ISystemProperties_USCOREgetMinGuestCPUCount *const*)ptr, "vbox:ISystemProperties_getMinGuestCPUCount");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMaxGuestVRAMResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetMaxGuestVRAMResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREgetMaxGuestVRAMResponse *const*)ptr, "vbox:ISystemProperties_getMaxGuestVRAMResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMaxGuestVRAM:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetMaxGuestVRAM(soap, tag, id, (_vbox__ISystemProperties_USCOREgetMaxGuestVRAM *const*)ptr, "vbox:ISystemProperties_getMaxGuestVRAM");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMinGuestVRAMResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetMinGuestVRAMResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREgetMinGuestVRAMResponse *const*)ptr, "vbox:ISystemProperties_getMinGuestVRAMResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMinGuestVRAM:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetMinGuestVRAM(soap, tag, id, (_vbox__ISystemProperties_USCOREgetMinGuestVRAM *const*)ptr, "vbox:ISystemProperties_getMinGuestVRAM");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMaxGuestRAMResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetMaxGuestRAMResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREgetMaxGuestRAMResponse *const*)ptr, "vbox:ISystemProperties_getMaxGuestRAMResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMaxGuestRAM:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetMaxGuestRAM(soap, tag, id, (_vbox__ISystemProperties_USCOREgetMaxGuestRAM *const*)ptr, "vbox:ISystemProperties_getMaxGuestRAM");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMinGuestRAMResponse:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetMinGuestRAMResponse(soap, tag, id, (_vbox__ISystemProperties_USCOREgetMinGuestRAMResponse *const*)ptr, "vbox:ISystemProperties_getMinGuestRAMResponse");
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMinGuestRAM:
		return soap_out_PointerTo_vbox__ISystemProperties_USCOREgetMinGuestRAM(soap, tag, id, (_vbox__ISystemProperties_USCOREgetMinGuestRAM *const*)ptr, "vbox:ISystemProperties_getMinGuestRAM");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindUSBDeviceByAddressResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREfindUSBDeviceByAddressResponse(soap, tag, id, (_vbox__IHost_USCOREfindUSBDeviceByAddressResponse *const*)ptr, "vbox:IHost_findUSBDeviceByAddressResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindUSBDeviceByAddress:
		return soap_out_PointerTo_vbox__IHost_USCOREfindUSBDeviceByAddress(soap, tag, id, (_vbox__IHost_USCOREfindUSBDeviceByAddress *const*)ptr, "vbox:IHost_findUSBDeviceByAddress");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindUSBDeviceByIdResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREfindUSBDeviceByIdResponse(soap, tag, id, (_vbox__IHost_USCOREfindUSBDeviceByIdResponse *const*)ptr, "vbox:IHost_findUSBDeviceByIdResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindUSBDeviceById:
		return soap_out_PointerTo_vbox__IHost_USCOREfindUSBDeviceById(soap, tag, id, (_vbox__IHost_USCOREfindUSBDeviceById *const*)ptr, "vbox:IHost_findUSBDeviceById");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindHostNetworkInterfacesOfTypeResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREfindHostNetworkInterfacesOfTypeResponse(soap, tag, id, (_vbox__IHost_USCOREfindHostNetworkInterfacesOfTypeResponse *const*)ptr, "vbox:IHost_findHostNetworkInterfacesOfTypeResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindHostNetworkInterfacesOfType:
		return soap_out_PointerTo_vbox__IHost_USCOREfindHostNetworkInterfacesOfType(soap, tag, id, (_vbox__IHost_USCOREfindHostNetworkInterfacesOfType *const*)ptr, "vbox:IHost_findHostNetworkInterfacesOfType");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindHostNetworkInterfaceByIdResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREfindHostNetworkInterfaceByIdResponse(soap, tag, id, (_vbox__IHost_USCOREfindHostNetworkInterfaceByIdResponse *const*)ptr, "vbox:IHost_findHostNetworkInterfaceByIdResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindHostNetworkInterfaceById:
		return soap_out_PointerTo_vbox__IHost_USCOREfindHostNetworkInterfaceById(soap, tag, id, (_vbox__IHost_USCOREfindHostNetworkInterfaceById *const*)ptr, "vbox:IHost_findHostNetworkInterfaceById");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindHostNetworkInterfaceByNameResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREfindHostNetworkInterfaceByNameResponse(soap, tag, id, (_vbox__IHost_USCOREfindHostNetworkInterfaceByNameResponse *const*)ptr, "vbox:IHost_findHostNetworkInterfaceByNameResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindHostNetworkInterfaceByName:
		return soap_out_PointerTo_vbox__IHost_USCOREfindHostNetworkInterfaceByName(soap, tag, id, (_vbox__IHost_USCOREfindHostNetworkInterfaceByName *const*)ptr, "vbox:IHost_findHostNetworkInterfaceByName");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindHostFloppyDriveResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREfindHostFloppyDriveResponse(soap, tag, id, (_vbox__IHost_USCOREfindHostFloppyDriveResponse *const*)ptr, "vbox:IHost_findHostFloppyDriveResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindHostFloppyDrive:
		return soap_out_PointerTo_vbox__IHost_USCOREfindHostFloppyDrive(soap, tag, id, (_vbox__IHost_USCOREfindHostFloppyDrive *const*)ptr, "vbox:IHost_findHostFloppyDrive");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindHostDVDDriveResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREfindHostDVDDriveResponse(soap, tag, id, (_vbox__IHost_USCOREfindHostDVDDriveResponse *const*)ptr, "vbox:IHost_findHostDVDDriveResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindHostDVDDrive:
		return soap_out_PointerTo_vbox__IHost_USCOREfindHostDVDDrive(soap, tag, id, (_vbox__IHost_USCOREfindHostDVDDrive *const*)ptr, "vbox:IHost_findHostDVDDrive");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREremoveUSBDeviceFilterResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREremoveUSBDeviceFilterResponse(soap, tag, id, (_vbox__IHost_USCOREremoveUSBDeviceFilterResponse *const*)ptr, "vbox:IHost_removeUSBDeviceFilterResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREremoveUSBDeviceFilter:
		return soap_out_PointerTo_vbox__IHost_USCOREremoveUSBDeviceFilter(soap, tag, id, (_vbox__IHost_USCOREremoveUSBDeviceFilter *const*)ptr, "vbox:IHost_removeUSBDeviceFilter");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREinsertUSBDeviceFilterResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREinsertUSBDeviceFilterResponse(soap, tag, id, (_vbox__IHost_USCOREinsertUSBDeviceFilterResponse *const*)ptr, "vbox:IHost_insertUSBDeviceFilterResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREinsertUSBDeviceFilter:
		return soap_out_PointerTo_vbox__IHost_USCOREinsertUSBDeviceFilter(soap, tag, id, (_vbox__IHost_USCOREinsertUSBDeviceFilter *const*)ptr, "vbox:IHost_insertUSBDeviceFilter");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREcreateUSBDeviceFilterResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREcreateUSBDeviceFilterResponse(soap, tag, id, (_vbox__IHost_USCOREcreateUSBDeviceFilterResponse *const*)ptr, "vbox:IHost_createUSBDeviceFilterResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREcreateUSBDeviceFilter:
		return soap_out_PointerTo_vbox__IHost_USCOREcreateUSBDeviceFilter(soap, tag, id, (_vbox__IHost_USCOREcreateUSBDeviceFilter *const*)ptr, "vbox:IHost_createUSBDeviceFilter");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREremoveHostOnlyNetworkInterfaceResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREremoveHostOnlyNetworkInterfaceResponse(soap, tag, id, (_vbox__IHost_USCOREremoveHostOnlyNetworkInterfaceResponse *const*)ptr, "vbox:IHost_removeHostOnlyNetworkInterfaceResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREremoveHostOnlyNetworkInterface:
		return soap_out_PointerTo_vbox__IHost_USCOREremoveHostOnlyNetworkInterface(soap, tag, id, (_vbox__IHost_USCOREremoveHostOnlyNetworkInterface *const*)ptr, "vbox:IHost_removeHostOnlyNetworkInterface");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREcreateHostOnlyNetworkInterfaceResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREcreateHostOnlyNetworkInterfaceResponse(soap, tag, id, (_vbox__IHost_USCOREcreateHostOnlyNetworkInterfaceResponse *const*)ptr, "vbox:IHost_createHostOnlyNetworkInterfaceResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREcreateHostOnlyNetworkInterface:
		return soap_out_PointerTo_vbox__IHost_USCOREcreateHostOnlyNetworkInterface(soap, tag, id, (_vbox__IHost_USCOREcreateHostOnlyNetworkInterface *const*)ptr, "vbox:IHost_createHostOnlyNetworkInterface");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetProcessorDescriptionResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREgetProcessorDescriptionResponse(soap, tag, id, (_vbox__IHost_USCOREgetProcessorDescriptionResponse *const*)ptr, "vbox:IHost_getProcessorDescriptionResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetProcessorDescription:
		return soap_out_PointerTo_vbox__IHost_USCOREgetProcessorDescription(soap, tag, id, (_vbox__IHost_USCOREgetProcessorDescription *const*)ptr, "vbox:IHost_getProcessorDescription");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetProcessorFeatureResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREgetProcessorFeatureResponse(soap, tag, id, (_vbox__IHost_USCOREgetProcessorFeatureResponse *const*)ptr, "vbox:IHost_getProcessorFeatureResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetProcessorFeature:
		return soap_out_PointerTo_vbox__IHost_USCOREgetProcessorFeature(soap, tag, id, (_vbox__IHost_USCOREgetProcessorFeature *const*)ptr, "vbox:IHost_getProcessorFeature");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetProcessorSpeedResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREgetProcessorSpeedResponse(soap, tag, id, (_vbox__IHost_USCOREgetProcessorSpeedResponse *const*)ptr, "vbox:IHost_getProcessorSpeedResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetProcessorSpeed:
		return soap_out_PointerTo_vbox__IHost_USCOREgetProcessorSpeed(soap, tag, id, (_vbox__IHost_USCOREgetProcessorSpeed *const*)ptr, "vbox:IHost_getProcessorSpeed");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetAcceleration3DAvailableResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREgetAcceleration3DAvailableResponse(soap, tag, id, (_vbox__IHost_USCOREgetAcceleration3DAvailableResponse *const*)ptr, "vbox:IHost_getAcceleration3DAvailableResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetAcceleration3DAvailable:
		return soap_out_PointerTo_vbox__IHost_USCOREgetAcceleration3DAvailable(soap, tag, id, (_vbox__IHost_USCOREgetAcceleration3DAvailable *const*)ptr, "vbox:IHost_getAcceleration3DAvailable");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetUTCTimeResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREgetUTCTimeResponse(soap, tag, id, (_vbox__IHost_USCOREgetUTCTimeResponse *const*)ptr, "vbox:IHost_getUTCTimeResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetUTCTime:
		return soap_out_PointerTo_vbox__IHost_USCOREgetUTCTime(soap, tag, id, (_vbox__IHost_USCOREgetUTCTime *const*)ptr, "vbox:IHost_getUTCTime");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetOSVersionResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREgetOSVersionResponse(soap, tag, id, (_vbox__IHost_USCOREgetOSVersionResponse *const*)ptr, "vbox:IHost_getOSVersionResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetOSVersion:
		return soap_out_PointerTo_vbox__IHost_USCOREgetOSVersion(soap, tag, id, (_vbox__IHost_USCOREgetOSVersion *const*)ptr, "vbox:IHost_getOSVersion");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetOperatingSystemResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREgetOperatingSystemResponse(soap, tag, id, (_vbox__IHost_USCOREgetOperatingSystemResponse *const*)ptr, "vbox:IHost_getOperatingSystemResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetOperatingSystem:
		return soap_out_PointerTo_vbox__IHost_USCOREgetOperatingSystem(soap, tag, id, (_vbox__IHost_USCOREgetOperatingSystem *const*)ptr, "vbox:IHost_getOperatingSystem");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetMemoryAvailableResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREgetMemoryAvailableResponse(soap, tag, id, (_vbox__IHost_USCOREgetMemoryAvailableResponse *const*)ptr, "vbox:IHost_getMemoryAvailableResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetMemoryAvailable:
		return soap_out_PointerTo_vbox__IHost_USCOREgetMemoryAvailable(soap, tag, id, (_vbox__IHost_USCOREgetMemoryAvailable *const*)ptr, "vbox:IHost_getMemoryAvailable");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetMemorySizeResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREgetMemorySizeResponse(soap, tag, id, (_vbox__IHost_USCOREgetMemorySizeResponse *const*)ptr, "vbox:IHost_getMemorySizeResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetMemorySize:
		return soap_out_PointerTo_vbox__IHost_USCOREgetMemorySize(soap, tag, id, (_vbox__IHost_USCOREgetMemorySize *const*)ptr, "vbox:IHost_getMemorySize");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetProcessorOnlineCountResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREgetProcessorOnlineCountResponse(soap, tag, id, (_vbox__IHost_USCOREgetProcessorOnlineCountResponse *const*)ptr, "vbox:IHost_getProcessorOnlineCountResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetProcessorOnlineCount:
		return soap_out_PointerTo_vbox__IHost_USCOREgetProcessorOnlineCount(soap, tag, id, (_vbox__IHost_USCOREgetProcessorOnlineCount *const*)ptr, "vbox:IHost_getProcessorOnlineCount");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetProcessorCountResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREgetProcessorCountResponse(soap, tag, id, (_vbox__IHost_USCOREgetProcessorCountResponse *const*)ptr, "vbox:IHost_getProcessorCountResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetProcessorCount:
		return soap_out_PointerTo_vbox__IHost_USCOREgetProcessorCount(soap, tag, id, (_vbox__IHost_USCOREgetProcessorCount *const*)ptr, "vbox:IHost_getProcessorCount");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetNetworkInterfacesResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREgetNetworkInterfacesResponse(soap, tag, id, (_vbox__IHost_USCOREgetNetworkInterfacesResponse *const*)ptr, "vbox:IHost_getNetworkInterfacesResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetNetworkInterfaces:
		return soap_out_PointerTo_vbox__IHost_USCOREgetNetworkInterfaces(soap, tag, id, (_vbox__IHost_USCOREgetNetworkInterfaces *const*)ptr, "vbox:IHost_getNetworkInterfaces");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetUSBDeviceFiltersResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREgetUSBDeviceFiltersResponse(soap, tag, id, (_vbox__IHost_USCOREgetUSBDeviceFiltersResponse *const*)ptr, "vbox:IHost_getUSBDeviceFiltersResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetUSBDeviceFilters:
		return soap_out_PointerTo_vbox__IHost_USCOREgetUSBDeviceFilters(soap, tag, id, (_vbox__IHost_USCOREgetUSBDeviceFilters *const*)ptr, "vbox:IHost_getUSBDeviceFilters");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetUSBDevicesResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREgetUSBDevicesResponse(soap, tag, id, (_vbox__IHost_USCOREgetUSBDevicesResponse *const*)ptr, "vbox:IHost_getUSBDevicesResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetUSBDevices:
		return soap_out_PointerTo_vbox__IHost_USCOREgetUSBDevices(soap, tag, id, (_vbox__IHost_USCOREgetUSBDevices *const*)ptr, "vbox:IHost_getUSBDevices");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetFloppyDrivesResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREgetFloppyDrivesResponse(soap, tag, id, (_vbox__IHost_USCOREgetFloppyDrivesResponse *const*)ptr, "vbox:IHost_getFloppyDrivesResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetFloppyDrives:
		return soap_out_PointerTo_vbox__IHost_USCOREgetFloppyDrives(soap, tag, id, (_vbox__IHost_USCOREgetFloppyDrives *const*)ptr, "vbox:IHost_getFloppyDrives");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetDVDDrivesResponse:
		return soap_out_PointerTo_vbox__IHost_USCOREgetDVDDrivesResponse(soap, tag, id, (_vbox__IHost_USCOREgetDVDDrivesResponse *const*)ptr, "vbox:IHost_getDVDDrivesResponse");
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetDVDDrives:
		return soap_out_PointerTo_vbox__IHost_USCOREgetDVDDrives(soap, tag, id, (_vbox__IHost_USCOREgetDVDDrives *const*)ptr, "vbox:IHost_getDVDDrives");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREdhcpRediscoverResponse:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREdhcpRediscoverResponse(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREdhcpRediscoverResponse *const*)ptr, "vbox:IHostNetworkInterface_dhcpRediscoverResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREdhcpRediscover:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREdhcpRediscover(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREdhcpRediscover *const*)ptr, "vbox:IHostNetworkInterface_dhcpRediscover");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREenableDynamicIpConfigResponse:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREenableDynamicIpConfigResponse(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREenableDynamicIpConfigResponse *const*)ptr, "vbox:IHostNetworkInterface_enableDynamicIpConfigResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREenableDynamicIpConfig:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREenableDynamicIpConfig(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREenableDynamicIpConfig *const*)ptr, "vbox:IHostNetworkInterface_enableDynamicIpConfig");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6Response:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6Response(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6Response *const*)ptr, "vbox:IHostNetworkInterface_enableStaticIpConfigV6Response");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6 *const*)ptr, "vbox:IHostNetworkInterface_enableStaticIpConfigV6");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREenableStaticIpConfigResponse:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREenableStaticIpConfigResponse(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREenableStaticIpConfigResponse *const*)ptr, "vbox:IHostNetworkInterface_enableStaticIpConfigResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREenableStaticIpConfig:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREenableStaticIpConfig(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREenableStaticIpConfig *const*)ptr, "vbox:IHostNetworkInterface_enableStaticIpConfig");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetInterfaceTypeResponse:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetInterfaceTypeResponse(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetInterfaceTypeResponse *const*)ptr, "vbox:IHostNetworkInterface_getInterfaceTypeResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetInterfaceType:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetInterfaceType(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetInterfaceType *const*)ptr, "vbox:IHostNetworkInterface_getInterfaceType");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetStatusResponse:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetStatusResponse(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetStatusResponse *const*)ptr, "vbox:IHostNetworkInterface_getStatusResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetStatus:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetStatus(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetStatus *const*)ptr, "vbox:IHostNetworkInterface_getStatus");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetMediumTypeResponse:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetMediumTypeResponse(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetMediumTypeResponse *const*)ptr, "vbox:IHostNetworkInterface_getMediumTypeResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetMediumType:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetMediumType(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetMediumType *const*)ptr, "vbox:IHostNetworkInterface_getMediumType");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetHardwareAddressResponse:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetHardwareAddressResponse(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetHardwareAddressResponse *const*)ptr, "vbox:IHostNetworkInterface_getHardwareAddressResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetHardwareAddress:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetHardwareAddress(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetHardwareAddress *const*)ptr, "vbox:IHostNetworkInterface_getHardwareAddress");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLengthResponse:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLengthResponse(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLengthResponse *const*)ptr, "vbox:IHostNetworkInterface_getIPV6NetworkMaskPrefixLengthResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLength:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLength(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLength *const*)ptr, "vbox:IHostNetworkInterface_getIPV6NetworkMaskPrefixLength");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPV6AddressResponse:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPV6AddressResponse(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetIPV6AddressResponse *const*)ptr, "vbox:IHostNetworkInterface_getIPV6AddressResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPV6Address:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPV6Address(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetIPV6Address *const*)ptr, "vbox:IHostNetworkInterface_getIPV6Address");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPV6SupportedResponse:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPV6SupportedResponse(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetIPV6SupportedResponse *const*)ptr, "vbox:IHostNetworkInterface_getIPV6SupportedResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPV6Supported:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPV6Supported(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetIPV6Supported *const*)ptr, "vbox:IHostNetworkInterface_getIPV6Supported");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetNetworkMaskResponse:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetNetworkMaskResponse(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetNetworkMaskResponse *const*)ptr, "vbox:IHostNetworkInterface_getNetworkMaskResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetNetworkMask:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetNetworkMask(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetNetworkMask *const*)ptr, "vbox:IHostNetworkInterface_getNetworkMask");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPAddressResponse:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPAddressResponse(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetIPAddressResponse *const*)ptr, "vbox:IHostNetworkInterface_getIPAddressResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPAddress:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPAddress(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetIPAddress *const*)ptr, "vbox:IHostNetworkInterface_getIPAddress");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetDhcpEnabledResponse:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetDhcpEnabledResponse(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetDhcpEnabledResponse *const*)ptr, "vbox:IHostNetworkInterface_getDhcpEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetDhcpEnabled:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetDhcpEnabled(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetDhcpEnabled *const*)ptr, "vbox:IHostNetworkInterface_getDhcpEnabled");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetNetworkNameResponse:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetNetworkNameResponse(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetNetworkNameResponse *const*)ptr, "vbox:IHostNetworkInterface_getNetworkNameResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetNetworkName:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetNetworkName(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetNetworkName *const*)ptr, "vbox:IHostNetworkInterface_getNetworkName");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetIdResponse:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetIdResponse(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetIdResponse *const*)ptr, "vbox:IHostNetworkInterface_getIdResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetId:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetId(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetId *const*)ptr, "vbox:IHostNetworkInterface_getId");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetNameResponse:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetNameResponse(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetNameResponse *const*)ptr, "vbox:IHostNetworkInterface_getNameResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetName:
		return soap_out_PointerTo_vbox__IHostNetworkInterface_USCOREgetName(soap, tag, id, (_vbox__IHostNetworkInterface_USCOREgetName *const*)ptr, "vbox:IHostNetworkInterface_getName");
	case SOAP_TYPE_PointerTo_vbox__IHostFloppyDrive_USCOREgetUdiResponse:
		return soap_out_PointerTo_vbox__IHostFloppyDrive_USCOREgetUdiResponse(soap, tag, id, (_vbox__IHostFloppyDrive_USCOREgetUdiResponse *const*)ptr, "vbox:IHostFloppyDrive_getUdiResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostFloppyDrive_USCOREgetUdi:
		return soap_out_PointerTo_vbox__IHostFloppyDrive_USCOREgetUdi(soap, tag, id, (_vbox__IHostFloppyDrive_USCOREgetUdi *const*)ptr, "vbox:IHostFloppyDrive_getUdi");
	case SOAP_TYPE_PointerTo_vbox__IHostFloppyDrive_USCOREgetDescriptionResponse:
		return soap_out_PointerTo_vbox__IHostFloppyDrive_USCOREgetDescriptionResponse(soap, tag, id, (_vbox__IHostFloppyDrive_USCOREgetDescriptionResponse *const*)ptr, "vbox:IHostFloppyDrive_getDescriptionResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostFloppyDrive_USCOREgetDescription:
		return soap_out_PointerTo_vbox__IHostFloppyDrive_USCOREgetDescription(soap, tag, id, (_vbox__IHostFloppyDrive_USCOREgetDescription *const*)ptr, "vbox:IHostFloppyDrive_getDescription");
	case SOAP_TYPE_PointerTo_vbox__IHostFloppyDrive_USCOREgetNameResponse:
		return soap_out_PointerTo_vbox__IHostFloppyDrive_USCOREgetNameResponse(soap, tag, id, (_vbox__IHostFloppyDrive_USCOREgetNameResponse *const*)ptr, "vbox:IHostFloppyDrive_getNameResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostFloppyDrive_USCOREgetName:
		return soap_out_PointerTo_vbox__IHostFloppyDrive_USCOREgetName(soap, tag, id, (_vbox__IHostFloppyDrive_USCOREgetName *const*)ptr, "vbox:IHostFloppyDrive_getName");
	case SOAP_TYPE_PointerTo_vbox__IHostDVDDrive_USCOREgetUdiResponse:
		return soap_out_PointerTo_vbox__IHostDVDDrive_USCOREgetUdiResponse(soap, tag, id, (_vbox__IHostDVDDrive_USCOREgetUdiResponse *const*)ptr, "vbox:IHostDVDDrive_getUdiResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostDVDDrive_USCOREgetUdi:
		return soap_out_PointerTo_vbox__IHostDVDDrive_USCOREgetUdi(soap, tag, id, (_vbox__IHostDVDDrive_USCOREgetUdi *const*)ptr, "vbox:IHostDVDDrive_getUdi");
	case SOAP_TYPE_PointerTo_vbox__IHostDVDDrive_USCOREgetDescriptionResponse:
		return soap_out_PointerTo_vbox__IHostDVDDrive_USCOREgetDescriptionResponse(soap, tag, id, (_vbox__IHostDVDDrive_USCOREgetDescriptionResponse *const*)ptr, "vbox:IHostDVDDrive_getDescriptionResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostDVDDrive_USCOREgetDescription:
		return soap_out_PointerTo_vbox__IHostDVDDrive_USCOREgetDescription(soap, tag, id, (_vbox__IHostDVDDrive_USCOREgetDescription *const*)ptr, "vbox:IHostDVDDrive_getDescription");
	case SOAP_TYPE_PointerTo_vbox__IHostDVDDrive_USCOREgetNameResponse:
		return soap_out_PointerTo_vbox__IHostDVDDrive_USCOREgetNameResponse(soap, tag, id, (_vbox__IHostDVDDrive_USCOREgetNameResponse *const*)ptr, "vbox:IHostDVDDrive_getNameResponse");
	case SOAP_TYPE_PointerTo_vbox__IHostDVDDrive_USCOREgetName:
		return soap_out_PointerTo_vbox__IHostDVDDrive_USCOREgetName(soap, tag, id, (_vbox__IHostDVDDrive_USCOREgetName *const*)ptr, "vbox:IHostDVDDrive_getName");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREdiscardCurrentSnapshotAndStateResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREdiscardCurrentSnapshotAndStateResponse(soap, tag, id, (_vbox__IConsole_USCOREdiscardCurrentSnapshotAndStateResponse *const*)ptr, "vbox:IConsole_discardCurrentSnapshotAndStateResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREdiscardCurrentSnapshotAndState:
		return soap_out_PointerTo_vbox__IConsole_USCOREdiscardCurrentSnapshotAndState(soap, tag, id, (_vbox__IConsole_USCOREdiscardCurrentSnapshotAndState *const*)ptr, "vbox:IConsole_discardCurrentSnapshotAndState");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREdiscardCurrentStateResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREdiscardCurrentStateResponse(soap, tag, id, (_vbox__IConsole_USCOREdiscardCurrentStateResponse *const*)ptr, "vbox:IConsole_discardCurrentStateResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREdiscardCurrentState:
		return soap_out_PointerTo_vbox__IConsole_USCOREdiscardCurrentState(soap, tag, id, (_vbox__IConsole_USCOREdiscardCurrentState *const*)ptr, "vbox:IConsole_discardCurrentState");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREdiscardSnapshotResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREdiscardSnapshotResponse(soap, tag, id, (_vbox__IConsole_USCOREdiscardSnapshotResponse *const*)ptr, "vbox:IConsole_discardSnapshotResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREdiscardSnapshot:
		return soap_out_PointerTo_vbox__IConsole_USCOREdiscardSnapshot(soap, tag, id, (_vbox__IConsole_USCOREdiscardSnapshot *const*)ptr, "vbox:IConsole_discardSnapshot");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREtakeSnapshotResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREtakeSnapshotResponse(soap, tag, id, (_vbox__IConsole_USCOREtakeSnapshotResponse *const*)ptr, "vbox:IConsole_takeSnapshotResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREtakeSnapshot:
		return soap_out_PointerTo_vbox__IConsole_USCOREtakeSnapshot(soap, tag, id, (_vbox__IConsole_USCOREtakeSnapshot *const*)ptr, "vbox:IConsole_takeSnapshot");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREremoveSharedFolderResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREremoveSharedFolderResponse(soap, tag, id, (_vbox__IConsole_USCOREremoveSharedFolderResponse *const*)ptr, "vbox:IConsole_removeSharedFolderResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREremoveSharedFolder:
		return soap_out_PointerTo_vbox__IConsole_USCOREremoveSharedFolder(soap, tag, id, (_vbox__IConsole_USCOREremoveSharedFolder *const*)ptr, "vbox:IConsole_removeSharedFolder");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREcreateSharedFolderResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREcreateSharedFolderResponse(soap, tag, id, (_vbox__IConsole_USCOREcreateSharedFolderResponse *const*)ptr, "vbox:IConsole_createSharedFolderResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREcreateSharedFolder:
		return soap_out_PointerTo_vbox__IConsole_USCOREcreateSharedFolder(soap, tag, id, (_vbox__IConsole_USCOREcreateSharedFolder *const*)ptr, "vbox:IConsole_createSharedFolder");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREfindUSBDeviceByIdResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREfindUSBDeviceByIdResponse(soap, tag, id, (_vbox__IConsole_USCOREfindUSBDeviceByIdResponse *const*)ptr, "vbox:IConsole_findUSBDeviceByIdResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREfindUSBDeviceById:
		return soap_out_PointerTo_vbox__IConsole_USCOREfindUSBDeviceById(soap, tag, id, (_vbox__IConsole_USCOREfindUSBDeviceById *const*)ptr, "vbox:IConsole_findUSBDeviceById");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREfindUSBDeviceByAddressResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREfindUSBDeviceByAddressResponse(soap, tag, id, (_vbox__IConsole_USCOREfindUSBDeviceByAddressResponse *const*)ptr, "vbox:IConsole_findUSBDeviceByAddressResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREfindUSBDeviceByAddress:
		return soap_out_PointerTo_vbox__IConsole_USCOREfindUSBDeviceByAddress(soap, tag, id, (_vbox__IConsole_USCOREfindUSBDeviceByAddress *const*)ptr, "vbox:IConsole_findUSBDeviceByAddress");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREdetachUSBDeviceResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREdetachUSBDeviceResponse(soap, tag, id, (_vbox__IConsole_USCOREdetachUSBDeviceResponse *const*)ptr, "vbox:IConsole_detachUSBDeviceResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREdetachUSBDevice:
		return soap_out_PointerTo_vbox__IConsole_USCOREdetachUSBDevice(soap, tag, id, (_vbox__IConsole_USCOREdetachUSBDevice *const*)ptr, "vbox:IConsole_detachUSBDevice");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREattachUSBDeviceResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREattachUSBDeviceResponse(soap, tag, id, (_vbox__IConsole_USCOREattachUSBDeviceResponse *const*)ptr, "vbox:IConsole_attachUSBDeviceResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREattachUSBDevice:
		return soap_out_PointerTo_vbox__IConsole_USCOREattachUSBDevice(soap, tag, id, (_vbox__IConsole_USCOREattachUSBDevice *const*)ptr, "vbox:IConsole_attachUSBDevice");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetDeviceActivityResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetDeviceActivityResponse(soap, tag, id, (_vbox__IConsole_USCOREgetDeviceActivityResponse *const*)ptr, "vbox:IConsole_getDeviceActivityResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetDeviceActivity:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetDeviceActivity(soap, tag, id, (_vbox__IConsole_USCOREgetDeviceActivity *const*)ptr, "vbox:IConsole_getDeviceActivity");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREforgetSavedStateResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREforgetSavedStateResponse(soap, tag, id, (_vbox__IConsole_USCOREforgetSavedStateResponse *const*)ptr, "vbox:IConsole_forgetSavedStateResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREforgetSavedState:
		return soap_out_PointerTo_vbox__IConsole_USCOREforgetSavedState(soap, tag, id, (_vbox__IConsole_USCOREforgetSavedState *const*)ptr, "vbox:IConsole_forgetSavedState");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREadoptSavedStateResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREadoptSavedStateResponse(soap, tag, id, (_vbox__IConsole_USCOREadoptSavedStateResponse *const*)ptr, "vbox:IConsole_adoptSavedStateResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREadoptSavedState:
		return soap_out_PointerTo_vbox__IConsole_USCOREadoptSavedState(soap, tag, id, (_vbox__IConsole_USCOREadoptSavedState *const*)ptr, "vbox:IConsole_adoptSavedState");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREsaveStateResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREsaveStateResponse(soap, tag, id, (_vbox__IConsole_USCOREsaveStateResponse *const*)ptr, "vbox:IConsole_saveStateResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREsaveState:
		return soap_out_PointerTo_vbox__IConsole_USCOREsaveState(soap, tag, id, (_vbox__IConsole_USCOREsaveState *const*)ptr, "vbox:IConsole_saveState");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetGuestEnteredACPIModeResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetGuestEnteredACPIModeResponse(soap, tag, id, (_vbox__IConsole_USCOREgetGuestEnteredACPIModeResponse *const*)ptr, "vbox:IConsole_getGuestEnteredACPIModeResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetGuestEnteredACPIMode:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetGuestEnteredACPIMode(soap, tag, id, (_vbox__IConsole_USCOREgetGuestEnteredACPIMode *const*)ptr, "vbox:IConsole_getGuestEnteredACPIMode");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetPowerButtonHandledResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetPowerButtonHandledResponse(soap, tag, id, (_vbox__IConsole_USCOREgetPowerButtonHandledResponse *const*)ptr, "vbox:IConsole_getPowerButtonHandledResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetPowerButtonHandled:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetPowerButtonHandled(soap, tag, id, (_vbox__IConsole_USCOREgetPowerButtonHandled *const*)ptr, "vbox:IConsole_getPowerButtonHandled");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREsleepButtonResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREsleepButtonResponse(soap, tag, id, (_vbox__IConsole_USCOREsleepButtonResponse *const*)ptr, "vbox:IConsole_sleepButtonResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREsleepButton:
		return soap_out_PointerTo_vbox__IConsole_USCOREsleepButton(soap, tag, id, (_vbox__IConsole_USCOREsleepButton *const*)ptr, "vbox:IConsole_sleepButton");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREpowerButtonResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREpowerButtonResponse(soap, tag, id, (_vbox__IConsole_USCOREpowerButtonResponse *const*)ptr, "vbox:IConsole_powerButtonResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREpowerButton:
		return soap_out_PointerTo_vbox__IConsole_USCOREpowerButton(soap, tag, id, (_vbox__IConsole_USCOREpowerButton *const*)ptr, "vbox:IConsole_powerButton");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREresumeResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREresumeResponse(soap, tag, id, (_vbox__IConsole_USCOREresumeResponse *const*)ptr, "vbox:IConsole_resumeResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREresume:
		return soap_out_PointerTo_vbox__IConsole_USCOREresume(soap, tag, id, (_vbox__IConsole_USCOREresume *const*)ptr, "vbox:IConsole_resume");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREpauseResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREpauseResponse(soap, tag, id, (_vbox__IConsole_USCOREpauseResponse *const*)ptr, "vbox:IConsole_pauseResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREpause:
		return soap_out_PointerTo_vbox__IConsole_USCOREpause(soap, tag, id, (_vbox__IConsole_USCOREpause *const*)ptr, "vbox:IConsole_pause");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREresetResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREresetResponse(soap, tag, id, (_vbox__IConsole_USCOREresetResponse *const*)ptr, "vbox:IConsole_resetResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREreset:
		return soap_out_PointerTo_vbox__IConsole_USCOREreset(soap, tag, id, (_vbox__IConsole_USCOREreset *const*)ptr, "vbox:IConsole_reset");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREpowerDownResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREpowerDownResponse(soap, tag, id, (_vbox__IConsole_USCOREpowerDownResponse *const*)ptr, "vbox:IConsole_powerDownResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREpowerDown:
		return soap_out_PointerTo_vbox__IConsole_USCOREpowerDown(soap, tag, id, (_vbox__IConsole_USCOREpowerDown *const*)ptr, "vbox:IConsole_powerDown");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREpowerUpPausedResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREpowerUpPausedResponse(soap, tag, id, (_vbox__IConsole_USCOREpowerUpPausedResponse *const*)ptr, "vbox:IConsole_powerUpPausedResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREpowerUpPaused:
		return soap_out_PointerTo_vbox__IConsole_USCOREpowerUpPaused(soap, tag, id, (_vbox__IConsole_USCOREpowerUpPaused *const*)ptr, "vbox:IConsole_powerUpPaused");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREpowerUpResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREpowerUpResponse(soap, tag, id, (_vbox__IConsole_USCOREpowerUpResponse *const*)ptr, "vbox:IConsole_powerUpResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREpowerUp:
		return soap_out_PointerTo_vbox__IConsole_USCOREpowerUp(soap, tag, id, (_vbox__IConsole_USCOREpowerUp *const*)ptr, "vbox:IConsole_powerUp");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetRemoteDisplayInfoResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetRemoteDisplayInfoResponse(soap, tag, id, (_vbox__IConsole_USCOREgetRemoteDisplayInfoResponse *const*)ptr, "vbox:IConsole_getRemoteDisplayInfoResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetRemoteDisplayInfo:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetRemoteDisplayInfo(soap, tag, id, (_vbox__IConsole_USCOREgetRemoteDisplayInfo *const*)ptr, "vbox:IConsole_getRemoteDisplayInfo");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetSharedFoldersResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetSharedFoldersResponse(soap, tag, id, (_vbox__IConsole_USCOREgetSharedFoldersResponse *const*)ptr, "vbox:IConsole_getSharedFoldersResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetSharedFolders:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetSharedFolders(soap, tag, id, (_vbox__IConsole_USCOREgetSharedFolders *const*)ptr, "vbox:IConsole_getSharedFolders");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetRemoteUSBDevicesResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetRemoteUSBDevicesResponse(soap, tag, id, (_vbox__IConsole_USCOREgetRemoteUSBDevicesResponse *const*)ptr, "vbox:IConsole_getRemoteUSBDevicesResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetRemoteUSBDevices:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetRemoteUSBDevices(soap, tag, id, (_vbox__IConsole_USCOREgetRemoteUSBDevices *const*)ptr, "vbox:IConsole_getRemoteUSBDevices");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetUSBDevicesResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetUSBDevicesResponse(soap, tag, id, (_vbox__IConsole_USCOREgetUSBDevicesResponse *const*)ptr, "vbox:IConsole_getUSBDevicesResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetUSBDevices:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetUSBDevices(soap, tag, id, (_vbox__IConsole_USCOREgetUSBDevices *const*)ptr, "vbox:IConsole_getUSBDevices");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetDisplayResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetDisplayResponse(soap, tag, id, (_vbox__IConsole_USCOREgetDisplayResponse *const*)ptr, "vbox:IConsole_getDisplayResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetDisplay:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetDisplay(soap, tag, id, (_vbox__IConsole_USCOREgetDisplay *const*)ptr, "vbox:IConsole_getDisplay");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetMouseResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetMouseResponse(soap, tag, id, (_vbox__IConsole_USCOREgetMouseResponse *const*)ptr, "vbox:IConsole_getMouseResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetMouse:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetMouse(soap, tag, id, (_vbox__IConsole_USCOREgetMouse *const*)ptr, "vbox:IConsole_getMouse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetKeyboardResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetKeyboardResponse(soap, tag, id, (_vbox__IConsole_USCOREgetKeyboardResponse *const*)ptr, "vbox:IConsole_getKeyboardResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetKeyboard:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetKeyboard(soap, tag, id, (_vbox__IConsole_USCOREgetKeyboard *const*)ptr, "vbox:IConsole_getKeyboard");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetGuestResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetGuestResponse(soap, tag, id, (_vbox__IConsole_USCOREgetGuestResponse *const*)ptr, "vbox:IConsole_getGuestResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetGuest:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetGuest(soap, tag, id, (_vbox__IConsole_USCOREgetGuest *const*)ptr, "vbox:IConsole_getGuest");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetStateResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetStateResponse(soap, tag, id, (_vbox__IConsole_USCOREgetStateResponse *const*)ptr, "vbox:IConsole_getStateResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetState:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetState(soap, tag, id, (_vbox__IConsole_USCOREgetState *const*)ptr, "vbox:IConsole_getState");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetMachineResponse:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetMachineResponse(soap, tag, id, (_vbox__IConsole_USCOREgetMachineResponse *const*)ptr, "vbox:IConsole_getMachineResponse");
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetMachine:
		return soap_out_PointerTo_vbox__IConsole_USCOREgetMachine(soap, tag, id, (_vbox__IConsole_USCOREgetMachine *const*)ptr, "vbox:IConsole_getMachine");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREenumerateGuestPropertiesResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREenumerateGuestPropertiesResponse(soap, tag, id, (_vbox__IMachine_USCOREenumerateGuestPropertiesResponse *const*)ptr, "vbox:IMachine_enumerateGuestPropertiesResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREenumerateGuestProperties:
		return soap_out_PointerTo_vbox__IMachine_USCOREenumerateGuestProperties(soap, tag, id, (_vbox__IMachine_USCOREenumerateGuestProperties *const*)ptr, "vbox:IMachine_enumerateGuestProperties");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetGuestPropertyValueResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetGuestPropertyValueResponse(soap, tag, id, (_vbox__IMachine_USCOREsetGuestPropertyValueResponse *const*)ptr, "vbox:IMachine_setGuestPropertyValueResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetGuestPropertyValue:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetGuestPropertyValue(soap, tag, id, (_vbox__IMachine_USCOREsetGuestPropertyValue *const*)ptr, "vbox:IMachine_setGuestPropertyValue");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetGuestPropertyResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetGuestPropertyResponse(soap, tag, id, (_vbox__IMachine_USCOREsetGuestPropertyResponse *const*)ptr, "vbox:IMachine_setGuestPropertyResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetGuestProperty:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetGuestProperty(soap, tag, id, (_vbox__IMachine_USCOREsetGuestProperty *const*)ptr, "vbox:IMachine_setGuestProperty");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetGuestPropertyTimestampResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetGuestPropertyTimestampResponse(soap, tag, id, (_vbox__IMachine_USCOREgetGuestPropertyTimestampResponse *const*)ptr, "vbox:IMachine_getGuestPropertyTimestampResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetGuestPropertyTimestamp:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetGuestPropertyTimestamp(soap, tag, id, (_vbox__IMachine_USCOREgetGuestPropertyTimestamp *const*)ptr, "vbox:IMachine_getGuestPropertyTimestamp");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetGuestPropertyValueResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetGuestPropertyValueResponse(soap, tag, id, (_vbox__IMachine_USCOREgetGuestPropertyValueResponse *const*)ptr, "vbox:IMachine_getGuestPropertyValueResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetGuestPropertyValue:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetGuestPropertyValue(soap, tag, id, (_vbox__IMachine_USCOREgetGuestPropertyValue *const*)ptr, "vbox:IMachine_getGuestPropertyValue");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetGuestPropertyResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetGuestPropertyResponse(soap, tag, id, (_vbox__IMachine_USCOREgetGuestPropertyResponse *const*)ptr, "vbox:IMachine_getGuestPropertyResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetGuestProperty:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetGuestProperty(soap, tag, id, (_vbox__IMachine_USCOREgetGuestProperty *const*)ptr, "vbox:IMachine_getGuestProperty");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREshowConsoleWindowResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREshowConsoleWindowResponse(soap, tag, id, (_vbox__IMachine_USCOREshowConsoleWindowResponse *const*)ptr, "vbox:IMachine_showConsoleWindowResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREshowConsoleWindow:
		return soap_out_PointerTo_vbox__IMachine_USCOREshowConsoleWindow(soap, tag, id, (_vbox__IMachine_USCOREshowConsoleWindow *const*)ptr, "vbox:IMachine_showConsoleWindow");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREcanShowConsoleWindowResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREcanShowConsoleWindowResponse(soap, tag, id, (_vbox__IMachine_USCOREcanShowConsoleWindowResponse *const*)ptr, "vbox:IMachine_canShowConsoleWindowResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREcanShowConsoleWindow:
		return soap_out_PointerTo_vbox__IMachine_USCOREcanShowConsoleWindow(soap, tag, id, (_vbox__IMachine_USCOREcanShowConsoleWindow *const*)ptr, "vbox:IMachine_canShowConsoleWindow");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREremoveSharedFolderResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREremoveSharedFolderResponse(soap, tag, id, (_vbox__IMachine_USCOREremoveSharedFolderResponse *const*)ptr, "vbox:IMachine_removeSharedFolderResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREremoveSharedFolder:
		return soap_out_PointerTo_vbox__IMachine_USCOREremoveSharedFolder(soap, tag, id, (_vbox__IMachine_USCOREremoveSharedFolder *const*)ptr, "vbox:IMachine_removeSharedFolder");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREcreateSharedFolderResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREcreateSharedFolderResponse(soap, tag, id, (_vbox__IMachine_USCOREcreateSharedFolderResponse *const*)ptr, "vbox:IMachine_createSharedFolderResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREcreateSharedFolder:
		return soap_out_PointerTo_vbox__IMachine_USCOREcreateSharedFolder(soap, tag, id, (_vbox__IMachine_USCOREcreateSharedFolder *const*)ptr, "vbox:IMachine_createSharedFolder");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetCurrentSnapshotResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetCurrentSnapshotResponse(soap, tag, id, (_vbox__IMachine_USCOREsetCurrentSnapshotResponse *const*)ptr, "vbox:IMachine_setCurrentSnapshotResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetCurrentSnapshot:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetCurrentSnapshot(soap, tag, id, (_vbox__IMachine_USCOREsetCurrentSnapshot *const*)ptr, "vbox:IMachine_setCurrentSnapshot");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREfindSnapshotResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREfindSnapshotResponse(soap, tag, id, (_vbox__IMachine_USCOREfindSnapshotResponse *const*)ptr, "vbox:IMachine_findSnapshotResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREfindSnapshot:
		return soap_out_PointerTo_vbox__IMachine_USCOREfindSnapshot(soap, tag, id, (_vbox__IMachine_USCOREfindSnapshot *const*)ptr, "vbox:IMachine_findSnapshot");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSnapshotResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetSnapshotResponse(soap, tag, id, (_vbox__IMachine_USCOREgetSnapshotResponse *const*)ptr, "vbox:IMachine_getSnapshotResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSnapshot:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetSnapshot(soap, tag, id, (_vbox__IMachine_USCOREgetSnapshot *const*)ptr, "vbox:IMachine_getSnapshot");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREexportResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREexportResponse(soap, tag, id, (_vbox__IMachine_USCOREexportResponse *const*)ptr, "vbox:IMachine_exportResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREexport:
		return soap_out_PointerTo_vbox__IMachine_USCOREexport(soap, tag, id, (_vbox__IMachine_USCOREexport *const*)ptr, "vbox:IMachine_export");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREdeleteSettingsResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREdeleteSettingsResponse(soap, tag, id, (_vbox__IMachine_USCOREdeleteSettingsResponse *const*)ptr, "vbox:IMachine_deleteSettingsResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREdeleteSettings:
		return soap_out_PointerTo_vbox__IMachine_USCOREdeleteSettings(soap, tag, id, (_vbox__IMachine_USCOREdeleteSettings *const*)ptr, "vbox:IMachine_deleteSettings");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREdiscardSettingsResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREdiscardSettingsResponse(soap, tag, id, (_vbox__IMachine_USCOREdiscardSettingsResponse *const*)ptr, "vbox:IMachine_discardSettingsResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREdiscardSettings:
		return soap_out_PointerTo_vbox__IMachine_USCOREdiscardSettings(soap, tag, id, (_vbox__IMachine_USCOREdiscardSettings *const*)ptr, "vbox:IMachine_discardSettings");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsaveSettingsWithBackupResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsaveSettingsWithBackupResponse(soap, tag, id, (_vbox__IMachine_USCOREsaveSettingsWithBackupResponse *const*)ptr, "vbox:IMachine_saveSettingsWithBackupResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsaveSettingsWithBackup:
		return soap_out_PointerTo_vbox__IMachine_USCOREsaveSettingsWithBackup(soap, tag, id, (_vbox__IMachine_USCOREsaveSettingsWithBackup *const*)ptr, "vbox:IMachine_saveSettingsWithBackup");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsaveSettingsResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsaveSettingsResponse(soap, tag, id, (_vbox__IMachine_USCOREsaveSettingsResponse *const*)ptr, "vbox:IMachine_saveSettingsResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsaveSettings:
		return soap_out_PointerTo_vbox__IMachine_USCOREsaveSettings(soap, tag, id, (_vbox__IMachine_USCOREsaveSettings *const*)ptr, "vbox:IMachine_saveSettings");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetExtraDataResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetExtraDataResponse(soap, tag, id, (_vbox__IMachine_USCOREsetExtraDataResponse *const*)ptr, "vbox:IMachine_setExtraDataResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetExtraData:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetExtraData(soap, tag, id, (_vbox__IMachine_USCOREsetExtraData *const*)ptr, "vbox:IMachine_setExtraData");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetExtraDataResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetExtraDataResponse(soap, tag, id, (_vbox__IMachine_USCOREgetExtraDataResponse *const*)ptr, "vbox:IMachine_getExtraDataResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetExtraData:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetExtraData(soap, tag, id, (_vbox__IMachine_USCOREgetExtraData *const*)ptr, "vbox:IMachine_getExtraData");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetNextExtraDataKeyResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetNextExtraDataKeyResponse(soap, tag, id, (_vbox__IMachine_USCOREgetNextExtraDataKeyResponse *const*)ptr, "vbox:IMachine_getNextExtraDataKeyResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetNextExtraDataKey:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetNextExtraDataKey(soap, tag, id, (_vbox__IMachine_USCOREgetNextExtraDataKey *const*)ptr, "vbox:IMachine_getNextExtraDataKey");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetParallelPortResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetParallelPortResponse(soap, tag, id, (_vbox__IMachine_USCOREgetParallelPortResponse *const*)ptr, "vbox:IMachine_getParallelPortResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetParallelPort:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetParallelPort(soap, tag, id, (_vbox__IMachine_USCOREgetParallelPort *const*)ptr, "vbox:IMachine_getParallelPort");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSerialPortResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetSerialPortResponse(soap, tag, id, (_vbox__IMachine_USCOREgetSerialPortResponse *const*)ptr, "vbox:IMachine_getSerialPortResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSerialPort:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetSerialPort(soap, tag, id, (_vbox__IMachine_USCOREgetSerialPort *const*)ptr, "vbox:IMachine_getSerialPort");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREremoveStorageControllerResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREremoveStorageControllerResponse(soap, tag, id, (_vbox__IMachine_USCOREremoveStorageControllerResponse *const*)ptr, "vbox:IMachine_removeStorageControllerResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREremoveStorageController:
		return soap_out_PointerTo_vbox__IMachine_USCOREremoveStorageController(soap, tag, id, (_vbox__IMachine_USCOREremoveStorageController *const*)ptr, "vbox:IMachine_removeStorageController");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetStorageControllerByNameResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetStorageControllerByNameResponse(soap, tag, id, (_vbox__IMachine_USCOREgetStorageControllerByNameResponse *const*)ptr, "vbox:IMachine_getStorageControllerByNameResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetStorageControllerByName:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetStorageControllerByName(soap, tag, id, (_vbox__IMachine_USCOREgetStorageControllerByName *const*)ptr, "vbox:IMachine_getStorageControllerByName");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREaddStorageControllerResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREaddStorageControllerResponse(soap, tag, id, (_vbox__IMachine_USCOREaddStorageControllerResponse *const*)ptr, "vbox:IMachine_addStorageControllerResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREaddStorageController:
		return soap_out_PointerTo_vbox__IMachine_USCOREaddStorageController(soap, tag, id, (_vbox__IMachine_USCOREaddStorageController *const*)ptr, "vbox:IMachine_addStorageController");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetNetworkAdapterResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetNetworkAdapterResponse(soap, tag, id, (_vbox__IMachine_USCOREgetNetworkAdapterResponse *const*)ptr, "vbox:IMachine_getNetworkAdapterResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetNetworkAdapter:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetNetworkAdapter(soap, tag, id, (_vbox__IMachine_USCOREgetNetworkAdapter *const*)ptr, "vbox:IMachine_getNetworkAdapter");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHardDiskAttachmentsOfControllerResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetHardDiskAttachmentsOfControllerResponse(soap, tag, id, (_vbox__IMachine_USCOREgetHardDiskAttachmentsOfControllerResponse *const*)ptr, "vbox:IMachine_getHardDiskAttachmentsOfControllerResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHardDiskAttachmentsOfController:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetHardDiskAttachmentsOfController(soap, tag, id, (_vbox__IMachine_USCOREgetHardDiskAttachmentsOfController *const*)ptr, "vbox:IMachine_getHardDiskAttachmentsOfController");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREdetachHardDiskResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREdetachHardDiskResponse(soap, tag, id, (_vbox__IMachine_USCOREdetachHardDiskResponse *const*)ptr, "vbox:IMachine_detachHardDiskResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREdetachHardDisk:
		return soap_out_PointerTo_vbox__IMachine_USCOREdetachHardDisk(soap, tag, id, (_vbox__IMachine_USCOREdetachHardDisk *const*)ptr, "vbox:IMachine_detachHardDisk");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHardDiskResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetHardDiskResponse(soap, tag, id, (_vbox__IMachine_USCOREgetHardDiskResponse *const*)ptr, "vbox:IMachine_getHardDiskResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHardDisk:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetHardDisk(soap, tag, id, (_vbox__IMachine_USCOREgetHardDisk *const*)ptr, "vbox:IMachine_getHardDisk");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREattachHardDiskResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREattachHardDiskResponse(soap, tag, id, (_vbox__IMachine_USCOREattachHardDiskResponse *const*)ptr, "vbox:IMachine_attachHardDiskResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREattachHardDisk:
		return soap_out_PointerTo_vbox__IMachine_USCOREattachHardDisk(soap, tag, id, (_vbox__IMachine_USCOREattachHardDisk *const*)ptr, "vbox:IMachine_attachHardDisk");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetBootOrderResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetBootOrderResponse(soap, tag, id, (_vbox__IMachine_USCOREgetBootOrderResponse *const*)ptr, "vbox:IMachine_getBootOrderResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetBootOrder:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetBootOrder(soap, tag, id, (_vbox__IMachine_USCOREgetBootOrder *const*)ptr, "vbox:IMachine_getBootOrder");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetBootOrderResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetBootOrderResponse(soap, tag, id, (_vbox__IMachine_USCOREsetBootOrderResponse *const*)ptr, "vbox:IMachine_setBootOrderResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetBootOrder:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetBootOrder(soap, tag, id, (_vbox__IMachine_USCOREsetBootOrder *const*)ptr, "vbox:IMachine_setBootOrder");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetGuestPropertyNotificationPatternsResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetGuestPropertyNotificationPatternsResponse(soap, tag, id, (_vbox__IMachine_USCOREsetGuestPropertyNotificationPatternsResponse *const*)ptr, "vbox:IMachine_setGuestPropertyNotificationPatternsResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetGuestPropertyNotificationPatterns:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetGuestPropertyNotificationPatterns(soap, tag, id, (_vbox__IMachine_USCOREsetGuestPropertyNotificationPatterns *const*)ptr, "vbox:IMachine_setGuestPropertyNotificationPatterns");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetGuestPropertyNotificationPatternsResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetGuestPropertyNotificationPatternsResponse(soap, tag, id, (_vbox__IMachine_USCOREgetGuestPropertyNotificationPatternsResponse *const*)ptr, "vbox:IMachine_getGuestPropertyNotificationPatternsResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetGuestPropertyNotificationPatterns:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetGuestPropertyNotificationPatterns(soap, tag, id, (_vbox__IMachine_USCOREgetGuestPropertyNotificationPatterns *const*)ptr, "vbox:IMachine_getGuestPropertyNotificationPatterns");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetClipboardModeResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetClipboardModeResponse(soap, tag, id, (_vbox__IMachine_USCOREsetClipboardModeResponse *const*)ptr, "vbox:IMachine_setClipboardModeResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetClipboardMode:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetClipboardMode(soap, tag, id, (_vbox__IMachine_USCOREsetClipboardMode *const*)ptr, "vbox:IMachine_setClipboardMode");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetClipboardModeResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetClipboardModeResponse(soap, tag, id, (_vbox__IMachine_USCOREgetClipboardModeResponse *const*)ptr, "vbox:IMachine_getClipboardModeResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetClipboardMode:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetClipboardMode(soap, tag, id, (_vbox__IMachine_USCOREgetClipboardMode *const*)ptr, "vbox:IMachine_getClipboardMode");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSharedFoldersResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetSharedFoldersResponse(soap, tag, id, (_vbox__IMachine_USCOREgetSharedFoldersResponse *const*)ptr, "vbox:IMachine_getSharedFoldersResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSharedFolders:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetSharedFolders(soap, tag, id, (_vbox__IMachine_USCOREgetSharedFolders *const*)ptr, "vbox:IMachine_getSharedFolders");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetCurrentStateModifiedResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetCurrentStateModifiedResponse(soap, tag, id, (_vbox__IMachine_USCOREgetCurrentStateModifiedResponse *const*)ptr, "vbox:IMachine_getCurrentStateModifiedResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetCurrentStateModified:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetCurrentStateModified(soap, tag, id, (_vbox__IMachine_USCOREgetCurrentStateModified *const*)ptr, "vbox:IMachine_getCurrentStateModified");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSnapshotCountResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetSnapshotCountResponse(soap, tag, id, (_vbox__IMachine_USCOREgetSnapshotCountResponse *const*)ptr, "vbox:IMachine_getSnapshotCountResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSnapshotCount:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetSnapshotCount(soap, tag, id, (_vbox__IMachine_USCOREgetSnapshotCount *const*)ptr, "vbox:IMachine_getSnapshotCount");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetCurrentSnapshotResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetCurrentSnapshotResponse(soap, tag, id, (_vbox__IMachine_USCOREgetCurrentSnapshotResponse *const*)ptr, "vbox:IMachine_getCurrentSnapshotResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetCurrentSnapshot:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetCurrentSnapshot(soap, tag, id, (_vbox__IMachine_USCOREgetCurrentSnapshot *const*)ptr, "vbox:IMachine_getCurrentSnapshot");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetLogFolderResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetLogFolderResponse(soap, tag, id, (_vbox__IMachine_USCOREgetLogFolderResponse *const*)ptr, "vbox:IMachine_getLogFolderResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetLogFolder:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetLogFolder(soap, tag, id, (_vbox__IMachine_USCOREgetLogFolder *const*)ptr, "vbox:IMachine_getLogFolder");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetStateFilePathResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetStateFilePathResponse(soap, tag, id, (_vbox__IMachine_USCOREgetStateFilePathResponse *const*)ptr, "vbox:IMachine_getStateFilePathResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetStateFilePath:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetStateFilePath(soap, tag, id, (_vbox__IMachine_USCOREgetStateFilePath *const*)ptr, "vbox:IMachine_getStateFilePath");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetLastStateChangeResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetLastStateChangeResponse(soap, tag, id, (_vbox__IMachine_USCOREgetLastStateChangeResponse *const*)ptr, "vbox:IMachine_getLastStateChangeResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetLastStateChange:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetLastStateChange(soap, tag, id, (_vbox__IMachine_USCOREgetLastStateChange *const*)ptr, "vbox:IMachine_getLastStateChange");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetStateResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetStateResponse(soap, tag, id, (_vbox__IMachine_USCOREgetStateResponse *const*)ptr, "vbox:IMachine_getStateResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetState:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetState(soap, tag, id, (_vbox__IMachine_USCOREgetState *const*)ptr, "vbox:IMachine_getState");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSessionPidResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetSessionPidResponse(soap, tag, id, (_vbox__IMachine_USCOREgetSessionPidResponse *const*)ptr, "vbox:IMachine_getSessionPidResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSessionPid:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetSessionPid(soap, tag, id, (_vbox__IMachine_USCOREgetSessionPid *const*)ptr, "vbox:IMachine_getSessionPid");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSessionTypeResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetSessionTypeResponse(soap, tag, id, (_vbox__IMachine_USCOREgetSessionTypeResponse *const*)ptr, "vbox:IMachine_getSessionTypeResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSessionType:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetSessionType(soap, tag, id, (_vbox__IMachine_USCOREgetSessionType *const*)ptr, "vbox:IMachine_getSessionType");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSessionStateResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetSessionStateResponse(soap, tag, id, (_vbox__IMachine_USCOREgetSessionStateResponse *const*)ptr, "vbox:IMachine_getSessionStateResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSessionState:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetSessionState(soap, tag, id, (_vbox__IMachine_USCOREgetSessionState *const*)ptr, "vbox:IMachine_getSessionState");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSettingsModifiedResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetSettingsModifiedResponse(soap, tag, id, (_vbox__IMachine_USCOREgetSettingsModifiedResponse *const*)ptr, "vbox:IMachine_getSettingsModifiedResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSettingsModified:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetSettingsModified(soap, tag, id, (_vbox__IMachine_USCOREgetSettingsModified *const*)ptr, "vbox:IMachine_getSettingsModified");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSettingsFileVersionResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetSettingsFileVersionResponse(soap, tag, id, (_vbox__IMachine_USCOREgetSettingsFileVersionResponse *const*)ptr, "vbox:IMachine_getSettingsFileVersionResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSettingsFileVersion:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetSettingsFileVersion(soap, tag, id, (_vbox__IMachine_USCOREgetSettingsFileVersion *const*)ptr, "vbox:IMachine_getSettingsFileVersion");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSettingsFilePathResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetSettingsFilePathResponse(soap, tag, id, (_vbox__IMachine_USCOREgetSettingsFilePathResponse *const*)ptr, "vbox:IMachine_getSettingsFilePathResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSettingsFilePath:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetSettingsFilePath(soap, tag, id, (_vbox__IMachine_USCOREgetSettingsFilePath *const*)ptr, "vbox:IMachine_getSettingsFilePath");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetStorageControllersResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetStorageControllersResponse(soap, tag, id, (_vbox__IMachine_USCOREgetStorageControllersResponse *const*)ptr, "vbox:IMachine_getStorageControllersResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetStorageControllers:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetStorageControllers(soap, tag, id, (_vbox__IMachine_USCOREgetStorageControllers *const*)ptr, "vbox:IMachine_getStorageControllers");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetAudioAdapterResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetAudioAdapterResponse(soap, tag, id, (_vbox__IMachine_USCOREgetAudioAdapterResponse *const*)ptr, "vbox:IMachine_getAudioAdapterResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetAudioAdapter:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetAudioAdapter(soap, tag, id, (_vbox__IMachine_USCOREgetAudioAdapter *const*)ptr, "vbox:IMachine_getAudioAdapter");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetUSBControllerResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetUSBControllerResponse(soap, tag, id, (_vbox__IMachine_USCOREgetUSBControllerResponse *const*)ptr, "vbox:IMachine_getUSBControllerResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetUSBController:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetUSBController(soap, tag, id, (_vbox__IMachine_USCOREgetUSBController *const*)ptr, "vbox:IMachine_getUSBController");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetFloppyDriveResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetFloppyDriveResponse(soap, tag, id, (_vbox__IMachine_USCOREgetFloppyDriveResponse *const*)ptr, "vbox:IMachine_getFloppyDriveResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetFloppyDrive:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetFloppyDrive(soap, tag, id, (_vbox__IMachine_USCOREgetFloppyDrive *const*)ptr, "vbox:IMachine_getFloppyDrive");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetDVDDriveResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetDVDDriveResponse(soap, tag, id, (_vbox__IMachine_USCOREgetDVDDriveResponse *const*)ptr, "vbox:IMachine_getDVDDriveResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetDVDDrive:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetDVDDrive(soap, tag, id, (_vbox__IMachine_USCOREgetDVDDrive *const*)ptr, "vbox:IMachine_getDVDDrive");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHardDiskAttachmentsResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetHardDiskAttachmentsResponse(soap, tag, id, (_vbox__IMachine_USCOREgetHardDiskAttachmentsResponse *const*)ptr, "vbox:IMachine_getHardDiskAttachmentsResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHardDiskAttachments:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetHardDiskAttachments(soap, tag, id, (_vbox__IMachine_USCOREgetHardDiskAttachments *const*)ptr, "vbox:IMachine_getHardDiskAttachments");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetVRDPServerResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetVRDPServerResponse(soap, tag, id, (_vbox__IMachine_USCOREgetVRDPServerResponse *const*)ptr, "vbox:IMachine_getVRDPServerResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetVRDPServer:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetVRDPServer(soap, tag, id, (_vbox__IMachine_USCOREgetVRDPServer *const*)ptr, "vbox:IMachine_getVRDPServer");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetSnapshotFolderResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetSnapshotFolderResponse(soap, tag, id, (_vbox__IMachine_USCOREsetSnapshotFolderResponse *const*)ptr, "vbox:IMachine_setSnapshotFolderResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetSnapshotFolder:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetSnapshotFolder(soap, tag, id, (_vbox__IMachine_USCOREsetSnapshotFolder *const*)ptr, "vbox:IMachine_setSnapshotFolder");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSnapshotFolderResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetSnapshotFolderResponse(soap, tag, id, (_vbox__IMachine_USCOREgetSnapshotFolderResponse *const*)ptr, "vbox:IMachine_getSnapshotFolderResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSnapshotFolder:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetSnapshotFolder(soap, tag, id, (_vbox__IMachine_USCOREgetSnapshotFolder *const*)ptr, "vbox:IMachine_getSnapshotFolder");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetPAEEnabledResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetPAEEnabledResponse(soap, tag, id, (_vbox__IMachine_USCOREsetPAEEnabledResponse *const*)ptr, "vbox:IMachine_setPAEEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetPAEEnabled:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetPAEEnabled(soap, tag, id, (_vbox__IMachine_USCOREsetPAEEnabled *const*)ptr, "vbox:IMachine_setPAEEnabled");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetPAEEnabledResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetPAEEnabledResponse(soap, tag, id, (_vbox__IMachine_USCOREgetPAEEnabledResponse *const*)ptr, "vbox:IMachine_getPAEEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetPAEEnabled:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetPAEEnabled(soap, tag, id, (_vbox__IMachine_USCOREgetPAEEnabled *const*)ptr, "vbox:IMachine_getPAEEnabled");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetHWVirtExVPIDEnabledResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetHWVirtExVPIDEnabledResponse(soap, tag, id, (_vbox__IMachine_USCOREsetHWVirtExVPIDEnabledResponse *const*)ptr, "vbox:IMachine_setHWVirtExVPIDEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetHWVirtExVPIDEnabled:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetHWVirtExVPIDEnabled(soap, tag, id, (_vbox__IMachine_USCOREsetHWVirtExVPIDEnabled *const*)ptr, "vbox:IMachine_setHWVirtExVPIDEnabled");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHWVirtExVPIDEnabledResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetHWVirtExVPIDEnabledResponse(soap, tag, id, (_vbox__IMachine_USCOREgetHWVirtExVPIDEnabledResponse *const*)ptr, "vbox:IMachine_getHWVirtExVPIDEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHWVirtExVPIDEnabled:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetHWVirtExVPIDEnabled(soap, tag, id, (_vbox__IMachine_USCOREgetHWVirtExVPIDEnabled *const*)ptr, "vbox:IMachine_getHWVirtExVPIDEnabled");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabledResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabledResponse(soap, tag, id, (_vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabledResponse *const*)ptr, "vbox:IMachine_setHWVirtExNestedPagingEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabled:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabled(soap, tag, id, (_vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabled *const*)ptr, "vbox:IMachine_setHWVirtExNestedPagingEnabled");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabledResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabledResponse(soap, tag, id, (_vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabledResponse *const*)ptr, "vbox:IMachine_getHWVirtExNestedPagingEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabled:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabled(soap, tag, id, (_vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabled *const*)ptr, "vbox:IMachine_getHWVirtExNestedPagingEnabled");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetHWVirtExEnabledResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetHWVirtExEnabledResponse(soap, tag, id, (_vbox__IMachine_USCOREsetHWVirtExEnabledResponse *const*)ptr, "vbox:IMachine_setHWVirtExEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetHWVirtExEnabled:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetHWVirtExEnabled(soap, tag, id, (_vbox__IMachine_USCOREsetHWVirtExEnabled *const*)ptr, "vbox:IMachine_setHWVirtExEnabled");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHWVirtExEnabledResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetHWVirtExEnabledResponse(soap, tag, id, (_vbox__IMachine_USCOREgetHWVirtExEnabledResponse *const*)ptr, "vbox:IMachine_getHWVirtExEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHWVirtExEnabled:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetHWVirtExEnabled(soap, tag, id, (_vbox__IMachine_USCOREgetHWVirtExEnabled *const*)ptr, "vbox:IMachine_getHWVirtExEnabled");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetBIOSSettingsResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetBIOSSettingsResponse(soap, tag, id, (_vbox__IMachine_USCOREgetBIOSSettingsResponse *const*)ptr, "vbox:IMachine_getBIOSSettingsResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetBIOSSettings:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetBIOSSettings(soap, tag, id, (_vbox__IMachine_USCOREgetBIOSSettings *const*)ptr, "vbox:IMachine_getBIOSSettings");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetMonitorCountResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetMonitorCountResponse(soap, tag, id, (_vbox__IMachine_USCOREsetMonitorCountResponse *const*)ptr, "vbox:IMachine_setMonitorCountResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetMonitorCount:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetMonitorCount(soap, tag, id, (_vbox__IMachine_USCOREsetMonitorCount *const*)ptr, "vbox:IMachine_setMonitorCount");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetMonitorCountResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetMonitorCountResponse(soap, tag, id, (_vbox__IMachine_USCOREgetMonitorCountResponse *const*)ptr, "vbox:IMachine_getMonitorCountResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetMonitorCount:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetMonitorCount(soap, tag, id, (_vbox__IMachine_USCOREgetMonitorCount *const*)ptr, "vbox:IMachine_getMonitorCount");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetAccelerate3DEnabledResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetAccelerate3DEnabledResponse(soap, tag, id, (_vbox__IMachine_USCOREsetAccelerate3DEnabledResponse *const*)ptr, "vbox:IMachine_setAccelerate3DEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetAccelerate3DEnabled:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetAccelerate3DEnabled(soap, tag, id, (_vbox__IMachine_USCOREsetAccelerate3DEnabled *const*)ptr, "vbox:IMachine_setAccelerate3DEnabled");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetAccelerate3DEnabledResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetAccelerate3DEnabledResponse(soap, tag, id, (_vbox__IMachine_USCOREgetAccelerate3DEnabledResponse *const*)ptr, "vbox:IMachine_getAccelerate3DEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetAccelerate3DEnabled:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetAccelerate3DEnabled(soap, tag, id, (_vbox__IMachine_USCOREgetAccelerate3DEnabled *const*)ptr, "vbox:IMachine_getAccelerate3DEnabled");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetVRAMSizeResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetVRAMSizeResponse(soap, tag, id, (_vbox__IMachine_USCOREsetVRAMSizeResponse *const*)ptr, "vbox:IMachine_setVRAMSizeResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetVRAMSize:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetVRAMSize(soap, tag, id, (_vbox__IMachine_USCOREsetVRAMSize *const*)ptr, "vbox:IMachine_setVRAMSize");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetVRAMSizeResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetVRAMSizeResponse(soap, tag, id, (_vbox__IMachine_USCOREgetVRAMSizeResponse *const*)ptr, "vbox:IMachine_getVRAMSizeResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetVRAMSize:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetVRAMSize(soap, tag, id, (_vbox__IMachine_USCOREgetVRAMSize *const*)ptr, "vbox:IMachine_getVRAMSize");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetStatisticsUpdateIntervalResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetStatisticsUpdateIntervalResponse(soap, tag, id, (_vbox__IMachine_USCOREsetStatisticsUpdateIntervalResponse *const*)ptr, "vbox:IMachine_setStatisticsUpdateIntervalResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetStatisticsUpdateInterval:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetStatisticsUpdateInterval(soap, tag, id, (_vbox__IMachine_USCOREsetStatisticsUpdateInterval *const*)ptr, "vbox:IMachine_setStatisticsUpdateInterval");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetStatisticsUpdateIntervalResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetStatisticsUpdateIntervalResponse(soap, tag, id, (_vbox__IMachine_USCOREgetStatisticsUpdateIntervalResponse *const*)ptr, "vbox:IMachine_getStatisticsUpdateIntervalResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetStatisticsUpdateInterval:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetStatisticsUpdateInterval(soap, tag, id, (_vbox__IMachine_USCOREgetStatisticsUpdateInterval *const*)ptr, "vbox:IMachine_getStatisticsUpdateInterval");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetMemoryBalloonSizeResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetMemoryBalloonSizeResponse(soap, tag, id, (_vbox__IMachine_USCOREsetMemoryBalloonSizeResponse *const*)ptr, "vbox:IMachine_setMemoryBalloonSizeResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetMemoryBalloonSize:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetMemoryBalloonSize(soap, tag, id, (_vbox__IMachine_USCOREsetMemoryBalloonSize *const*)ptr, "vbox:IMachine_setMemoryBalloonSize");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetMemoryBalloonSizeResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetMemoryBalloonSizeResponse(soap, tag, id, (_vbox__IMachine_USCOREgetMemoryBalloonSizeResponse *const*)ptr, "vbox:IMachine_getMemoryBalloonSizeResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetMemoryBalloonSize:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetMemoryBalloonSize(soap, tag, id, (_vbox__IMachine_USCOREgetMemoryBalloonSize *const*)ptr, "vbox:IMachine_getMemoryBalloonSize");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetMemorySizeResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetMemorySizeResponse(soap, tag, id, (_vbox__IMachine_USCOREsetMemorySizeResponse *const*)ptr, "vbox:IMachine_setMemorySizeResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetMemorySize:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetMemorySize(soap, tag, id, (_vbox__IMachine_USCOREsetMemorySize *const*)ptr, "vbox:IMachine_setMemorySize");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetMemorySizeResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetMemorySizeResponse(soap, tag, id, (_vbox__IMachine_USCOREgetMemorySizeResponse *const*)ptr, "vbox:IMachine_getMemorySizeResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetMemorySize:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetMemorySize(soap, tag, id, (_vbox__IMachine_USCOREgetMemorySize *const*)ptr, "vbox:IMachine_getMemorySize");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetCPUCountResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetCPUCountResponse(soap, tag, id, (_vbox__IMachine_USCOREsetCPUCountResponse *const*)ptr, "vbox:IMachine_setCPUCountResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetCPUCount:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetCPUCount(soap, tag, id, (_vbox__IMachine_USCOREsetCPUCount *const*)ptr, "vbox:IMachine_setCPUCount");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetCPUCountResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetCPUCountResponse(soap, tag, id, (_vbox__IMachine_USCOREgetCPUCountResponse *const*)ptr, "vbox:IMachine_getCPUCountResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetCPUCount:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetCPUCount(soap, tag, id, (_vbox__IMachine_USCOREgetCPUCount *const*)ptr, "vbox:IMachine_getCPUCount");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetHardwareVersionResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetHardwareVersionResponse(soap, tag, id, (_vbox__IMachine_USCOREsetHardwareVersionResponse *const*)ptr, "vbox:IMachine_setHardwareVersionResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetHardwareVersion:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetHardwareVersion(soap, tag, id, (_vbox__IMachine_USCOREsetHardwareVersion *const*)ptr, "vbox:IMachine_setHardwareVersion");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHardwareVersionResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetHardwareVersionResponse(soap, tag, id, (_vbox__IMachine_USCOREgetHardwareVersionResponse *const*)ptr, "vbox:IMachine_getHardwareVersionResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHardwareVersion:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetHardwareVersion(soap, tag, id, (_vbox__IMachine_USCOREgetHardwareVersion *const*)ptr, "vbox:IMachine_getHardwareVersion");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetOSTypeIdResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetOSTypeIdResponse(soap, tag, id, (_vbox__IMachine_USCOREsetOSTypeIdResponse *const*)ptr, "vbox:IMachine_setOSTypeIdResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetOSTypeId:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetOSTypeId(soap, tag, id, (_vbox__IMachine_USCOREsetOSTypeId *const*)ptr, "vbox:IMachine_setOSTypeId");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetOSTypeIdResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetOSTypeIdResponse(soap, tag, id, (_vbox__IMachine_USCOREgetOSTypeIdResponse *const*)ptr, "vbox:IMachine_getOSTypeIdResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetOSTypeId:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetOSTypeId(soap, tag, id, (_vbox__IMachine_USCOREgetOSTypeId *const*)ptr, "vbox:IMachine_getOSTypeId");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetIdResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetIdResponse(soap, tag, id, (_vbox__IMachine_USCOREgetIdResponse *const*)ptr, "vbox:IMachine_getIdResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetId:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetId(soap, tag, id, (_vbox__IMachine_USCOREgetId *const*)ptr, "vbox:IMachine_getId");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetDescriptionResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetDescriptionResponse(soap, tag, id, (_vbox__IMachine_USCOREsetDescriptionResponse *const*)ptr, "vbox:IMachine_setDescriptionResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetDescription:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetDescription(soap, tag, id, (_vbox__IMachine_USCOREsetDescription *const*)ptr, "vbox:IMachine_setDescription");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetDescriptionResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetDescriptionResponse(soap, tag, id, (_vbox__IMachine_USCOREgetDescriptionResponse *const*)ptr, "vbox:IMachine_getDescriptionResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetDescription:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetDescription(soap, tag, id, (_vbox__IMachine_USCOREgetDescription *const*)ptr, "vbox:IMachine_getDescription");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetNameResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetNameResponse(soap, tag, id, (_vbox__IMachine_USCOREsetNameResponse *const*)ptr, "vbox:IMachine_setNameResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetName:
		return soap_out_PointerTo_vbox__IMachine_USCOREsetName(soap, tag, id, (_vbox__IMachine_USCOREsetName *const*)ptr, "vbox:IMachine_setName");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetNameResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetNameResponse(soap, tag, id, (_vbox__IMachine_USCOREgetNameResponse *const*)ptr, "vbox:IMachine_getNameResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetName:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetName(soap, tag, id, (_vbox__IMachine_USCOREgetName *const*)ptr, "vbox:IMachine_getName");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetAccessErrorResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetAccessErrorResponse(soap, tag, id, (_vbox__IMachine_USCOREgetAccessErrorResponse *const*)ptr, "vbox:IMachine_getAccessErrorResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetAccessError:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetAccessError(soap, tag, id, (_vbox__IMachine_USCOREgetAccessError *const*)ptr, "vbox:IMachine_getAccessError");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetAccessibleResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetAccessibleResponse(soap, tag, id, (_vbox__IMachine_USCOREgetAccessibleResponse *const*)ptr, "vbox:IMachine_getAccessibleResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetAccessible:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetAccessible(soap, tag, id, (_vbox__IMachine_USCOREgetAccessible *const*)ptr, "vbox:IMachine_getAccessible");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetParentResponse:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetParentResponse(soap, tag, id, (_vbox__IMachine_USCOREgetParentResponse *const*)ptr, "vbox:IMachine_getParentResponse");
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetParent:
		return soap_out_PointerTo_vbox__IMachine_USCOREgetParent(soap, tag, id, (_vbox__IMachine_USCOREgetParent *const*)ptr, "vbox:IMachine_getParent");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetPXEDebugEnabledResponse:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREsetPXEDebugEnabledResponse(soap, tag, id, (_vbox__IBIOSSettings_USCOREsetPXEDebugEnabledResponse *const*)ptr, "vbox:IBIOSSettings_setPXEDebugEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetPXEDebugEnabled:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREsetPXEDebugEnabled(soap, tag, id, (_vbox__IBIOSSettings_USCOREsetPXEDebugEnabled *const*)ptr, "vbox:IBIOSSettings_setPXEDebugEnabled");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetPXEDebugEnabledResponse:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREgetPXEDebugEnabledResponse(soap, tag, id, (_vbox__IBIOSSettings_USCOREgetPXEDebugEnabledResponse *const*)ptr, "vbox:IBIOSSettings_getPXEDebugEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetPXEDebugEnabled:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREgetPXEDebugEnabled(soap, tag, id, (_vbox__IBIOSSettings_USCOREgetPXEDebugEnabled *const*)ptr, "vbox:IBIOSSettings_getPXEDebugEnabled");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetTimeOffsetResponse:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREsetTimeOffsetResponse(soap, tag, id, (_vbox__IBIOSSettings_USCOREsetTimeOffsetResponse *const*)ptr, "vbox:IBIOSSettings_setTimeOffsetResponse");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetTimeOffset:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREsetTimeOffset(soap, tag, id, (_vbox__IBIOSSettings_USCOREsetTimeOffset *const*)ptr, "vbox:IBIOSSettings_setTimeOffset");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetTimeOffsetResponse:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREgetTimeOffsetResponse(soap, tag, id, (_vbox__IBIOSSettings_USCOREgetTimeOffsetResponse *const*)ptr, "vbox:IBIOSSettings_getTimeOffsetResponse");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetTimeOffset:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREgetTimeOffset(soap, tag, id, (_vbox__IBIOSSettings_USCOREgetTimeOffset *const*)ptr, "vbox:IBIOSSettings_getTimeOffset");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetIOAPICEnabledResponse:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREsetIOAPICEnabledResponse(soap, tag, id, (_vbox__IBIOSSettings_USCOREsetIOAPICEnabledResponse *const*)ptr, "vbox:IBIOSSettings_setIOAPICEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetIOAPICEnabled:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREsetIOAPICEnabled(soap, tag, id, (_vbox__IBIOSSettings_USCOREsetIOAPICEnabled *const*)ptr, "vbox:IBIOSSettings_setIOAPICEnabled");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetIOAPICEnabledResponse:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREgetIOAPICEnabledResponse(soap, tag, id, (_vbox__IBIOSSettings_USCOREgetIOAPICEnabledResponse *const*)ptr, "vbox:IBIOSSettings_getIOAPICEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetIOAPICEnabled:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREgetIOAPICEnabled(soap, tag, id, (_vbox__IBIOSSettings_USCOREgetIOAPICEnabled *const*)ptr, "vbox:IBIOSSettings_getIOAPICEnabled");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetACPIEnabledResponse:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREsetACPIEnabledResponse(soap, tag, id, (_vbox__IBIOSSettings_USCOREsetACPIEnabledResponse *const*)ptr, "vbox:IBIOSSettings_setACPIEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetACPIEnabled:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREsetACPIEnabled(soap, tag, id, (_vbox__IBIOSSettings_USCOREsetACPIEnabled *const*)ptr, "vbox:IBIOSSettings_setACPIEnabled");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetACPIEnabledResponse:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREgetACPIEnabledResponse(soap, tag, id, (_vbox__IBIOSSettings_USCOREgetACPIEnabledResponse *const*)ptr, "vbox:IBIOSSettings_getACPIEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetACPIEnabled:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREgetACPIEnabled(soap, tag, id, (_vbox__IBIOSSettings_USCOREgetACPIEnabled *const*)ptr, "vbox:IBIOSSettings_getACPIEnabled");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetBootMenuModeResponse:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREsetBootMenuModeResponse(soap, tag, id, (_vbox__IBIOSSettings_USCOREsetBootMenuModeResponse *const*)ptr, "vbox:IBIOSSettings_setBootMenuModeResponse");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetBootMenuMode:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREsetBootMenuMode(soap, tag, id, (_vbox__IBIOSSettings_USCOREsetBootMenuMode *const*)ptr, "vbox:IBIOSSettings_setBootMenuMode");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetBootMenuModeResponse:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREgetBootMenuModeResponse(soap, tag, id, (_vbox__IBIOSSettings_USCOREgetBootMenuModeResponse *const*)ptr, "vbox:IBIOSSettings_getBootMenuModeResponse");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetBootMenuMode:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREgetBootMenuMode(soap, tag, id, (_vbox__IBIOSSettings_USCOREgetBootMenuMode *const*)ptr, "vbox:IBIOSSettings_getBootMenuMode");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetLogoImagePathResponse:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREsetLogoImagePathResponse(soap, tag, id, (_vbox__IBIOSSettings_USCOREsetLogoImagePathResponse *const*)ptr, "vbox:IBIOSSettings_setLogoImagePathResponse");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetLogoImagePath:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREsetLogoImagePath(soap, tag, id, (_vbox__IBIOSSettings_USCOREsetLogoImagePath *const*)ptr, "vbox:IBIOSSettings_setLogoImagePath");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetLogoImagePathResponse:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREgetLogoImagePathResponse(soap, tag, id, (_vbox__IBIOSSettings_USCOREgetLogoImagePathResponse *const*)ptr, "vbox:IBIOSSettings_getLogoImagePathResponse");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetLogoImagePath:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREgetLogoImagePath(soap, tag, id, (_vbox__IBIOSSettings_USCOREgetLogoImagePath *const*)ptr, "vbox:IBIOSSettings_getLogoImagePath");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetLogoDisplayTimeResponse:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREsetLogoDisplayTimeResponse(soap, tag, id, (_vbox__IBIOSSettings_USCOREsetLogoDisplayTimeResponse *const*)ptr, "vbox:IBIOSSettings_setLogoDisplayTimeResponse");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetLogoDisplayTime:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREsetLogoDisplayTime(soap, tag, id, (_vbox__IBIOSSettings_USCOREsetLogoDisplayTime *const*)ptr, "vbox:IBIOSSettings_setLogoDisplayTime");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetLogoDisplayTimeResponse:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREgetLogoDisplayTimeResponse(soap, tag, id, (_vbox__IBIOSSettings_USCOREgetLogoDisplayTimeResponse *const*)ptr, "vbox:IBIOSSettings_getLogoDisplayTimeResponse");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetLogoDisplayTime:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREgetLogoDisplayTime(soap, tag, id, (_vbox__IBIOSSettings_USCOREgetLogoDisplayTime *const*)ptr, "vbox:IBIOSSettings_getLogoDisplayTime");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetLogoFadeOutResponse:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREsetLogoFadeOutResponse(soap, tag, id, (_vbox__IBIOSSettings_USCOREsetLogoFadeOutResponse *const*)ptr, "vbox:IBIOSSettings_setLogoFadeOutResponse");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetLogoFadeOut:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREsetLogoFadeOut(soap, tag, id, (_vbox__IBIOSSettings_USCOREsetLogoFadeOut *const*)ptr, "vbox:IBIOSSettings_setLogoFadeOut");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetLogoFadeOutResponse:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREgetLogoFadeOutResponse(soap, tag, id, (_vbox__IBIOSSettings_USCOREgetLogoFadeOutResponse *const*)ptr, "vbox:IBIOSSettings_getLogoFadeOutResponse");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetLogoFadeOut:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREgetLogoFadeOut(soap, tag, id, (_vbox__IBIOSSettings_USCOREgetLogoFadeOut *const*)ptr, "vbox:IBIOSSettings_getLogoFadeOut");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetLogoFadeInResponse:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREsetLogoFadeInResponse(soap, tag, id, (_vbox__IBIOSSettings_USCOREsetLogoFadeInResponse *const*)ptr, "vbox:IBIOSSettings_setLogoFadeInResponse");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetLogoFadeIn:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREsetLogoFadeIn(soap, tag, id, (_vbox__IBIOSSettings_USCOREsetLogoFadeIn *const*)ptr, "vbox:IBIOSSettings_setLogoFadeIn");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetLogoFadeInResponse:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREgetLogoFadeInResponse(soap, tag, id, (_vbox__IBIOSSettings_USCOREgetLogoFadeInResponse *const*)ptr, "vbox:IBIOSSettings_getLogoFadeInResponse");
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetLogoFadeIn:
		return soap_out_PointerTo_vbox__IBIOSSettings_USCOREgetLogoFadeIn(soap, tag, id, (_vbox__IBIOSSettings_USCOREgetLogoFadeIn *const*)ptr, "vbox:IBIOSSettings_getLogoFadeIn");
	case SOAP_TYPE_PointerTo_vbox__IVirtualSystemDescription_USCOREaddDescriptionResponse:
		return soap_out_PointerTo_vbox__IVirtualSystemDescription_USCOREaddDescriptionResponse(soap, tag, id, (_vbox__IVirtualSystemDescription_USCOREaddDescriptionResponse *const*)ptr, "vbox:IVirtualSystemDescription_addDescriptionResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualSystemDescription_USCOREaddDescription:
		return soap_out_PointerTo_vbox__IVirtualSystemDescription_USCOREaddDescription(soap, tag, id, (_vbox__IVirtualSystemDescription_USCOREaddDescription *const*)ptr, "vbox:IVirtualSystemDescription_addDescription");
	case SOAP_TYPE_PointerTo_vbox__IVirtualSystemDescription_USCOREsetFinalValuesResponse:
		return soap_out_PointerTo_vbox__IVirtualSystemDescription_USCOREsetFinalValuesResponse(soap, tag, id, (_vbox__IVirtualSystemDescription_USCOREsetFinalValuesResponse *const*)ptr, "vbox:IVirtualSystemDescription_setFinalValuesResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualSystemDescription_USCOREsetFinalValues:
		return soap_out_PointerTo_vbox__IVirtualSystemDescription_USCOREsetFinalValues(soap, tag, id, (_vbox__IVirtualSystemDescription_USCOREsetFinalValues *const*)ptr, "vbox:IVirtualSystemDescription_setFinalValues");
	case SOAP_TYPE_PointerTo_vbox__IVirtualSystemDescription_USCOREgetValuesByTypeResponse:
		return soap_out_PointerTo_vbox__IVirtualSystemDescription_USCOREgetValuesByTypeResponse(soap, tag, id, (_vbox__IVirtualSystemDescription_USCOREgetValuesByTypeResponse *const*)ptr, "vbox:IVirtualSystemDescription_getValuesByTypeResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualSystemDescription_USCOREgetValuesByType:
		return soap_out_PointerTo_vbox__IVirtualSystemDescription_USCOREgetValuesByType(soap, tag, id, (_vbox__IVirtualSystemDescription_USCOREgetValuesByType *const*)ptr, "vbox:IVirtualSystemDescription_getValuesByType");
	case SOAP_TYPE_PointerTo_vbox__IVirtualSystemDescription_USCOREgetDescriptionByTypeResponse:
		return soap_out_PointerTo_vbox__IVirtualSystemDescription_USCOREgetDescriptionByTypeResponse(soap, tag, id, (_vbox__IVirtualSystemDescription_USCOREgetDescriptionByTypeResponse *const*)ptr, "vbox:IVirtualSystemDescription_getDescriptionByTypeResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualSystemDescription_USCOREgetDescriptionByType:
		return soap_out_PointerTo_vbox__IVirtualSystemDescription_USCOREgetDescriptionByType(soap, tag, id, (_vbox__IVirtualSystemDescription_USCOREgetDescriptionByType *const*)ptr, "vbox:IVirtualSystemDescription_getDescriptionByType");
	case SOAP_TYPE_PointerTo_vbox__IVirtualSystemDescription_USCOREgetDescriptionResponse:
		return soap_out_PointerTo_vbox__IVirtualSystemDescription_USCOREgetDescriptionResponse(soap, tag, id, (_vbox__IVirtualSystemDescription_USCOREgetDescriptionResponse *const*)ptr, "vbox:IVirtualSystemDescription_getDescriptionResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualSystemDescription_USCOREgetDescription:
		return soap_out_PointerTo_vbox__IVirtualSystemDescription_USCOREgetDescription(soap, tag, id, (_vbox__IVirtualSystemDescription_USCOREgetDescription *const*)ptr, "vbox:IVirtualSystemDescription_getDescription");
	case SOAP_TYPE_PointerTo_vbox__IVirtualSystemDescription_USCOREgetCountResponse:
		return soap_out_PointerTo_vbox__IVirtualSystemDescription_USCOREgetCountResponse(soap, tag, id, (_vbox__IVirtualSystemDescription_USCOREgetCountResponse *const*)ptr, "vbox:IVirtualSystemDescription_getCountResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualSystemDescription_USCOREgetCount:
		return soap_out_PointerTo_vbox__IVirtualSystemDescription_USCOREgetCount(soap, tag, id, (_vbox__IVirtualSystemDescription_USCOREgetCount *const*)ptr, "vbox:IVirtualSystemDescription_getCount");
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREgetWarningsResponse:
		return soap_out_PointerTo_vbox__IAppliance_USCOREgetWarningsResponse(soap, tag, id, (_vbox__IAppliance_USCOREgetWarningsResponse *const*)ptr, "vbox:IAppliance_getWarningsResponse");
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREgetWarnings:
		return soap_out_PointerTo_vbox__IAppliance_USCOREgetWarnings(soap, tag, id, (_vbox__IAppliance_USCOREgetWarnings *const*)ptr, "vbox:IAppliance_getWarnings");
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREwriteResponse:
		return soap_out_PointerTo_vbox__IAppliance_USCOREwriteResponse(soap, tag, id, (_vbox__IAppliance_USCOREwriteResponse *const*)ptr, "vbox:IAppliance_writeResponse");
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREwrite:
		return soap_out_PointerTo_vbox__IAppliance_USCOREwrite(soap, tag, id, (_vbox__IAppliance_USCOREwrite *const*)ptr, "vbox:IAppliance_write");
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREcreateVFSExplorerResponse:
		return soap_out_PointerTo_vbox__IAppliance_USCOREcreateVFSExplorerResponse(soap, tag, id, (_vbox__IAppliance_USCOREcreateVFSExplorerResponse *const*)ptr, "vbox:IAppliance_createVFSExplorerResponse");
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREcreateVFSExplorer:
		return soap_out_PointerTo_vbox__IAppliance_USCOREcreateVFSExplorer(soap, tag, id, (_vbox__IAppliance_USCOREcreateVFSExplorer *const*)ptr, "vbox:IAppliance_createVFSExplorer");
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREimportMachinesResponse:
		return soap_out_PointerTo_vbox__IAppliance_USCOREimportMachinesResponse(soap, tag, id, (_vbox__IAppliance_USCOREimportMachinesResponse *const*)ptr, "vbox:IAppliance_importMachinesResponse");
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREimportMachines:
		return soap_out_PointerTo_vbox__IAppliance_USCOREimportMachines(soap, tag, id, (_vbox__IAppliance_USCOREimportMachines *const*)ptr, "vbox:IAppliance_importMachines");
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREinterpretResponse:
		return soap_out_PointerTo_vbox__IAppliance_USCOREinterpretResponse(soap, tag, id, (_vbox__IAppliance_USCOREinterpretResponse *const*)ptr, "vbox:IAppliance_interpretResponse");
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREinterpret:
		return soap_out_PointerTo_vbox__IAppliance_USCOREinterpret(soap, tag, id, (_vbox__IAppliance_USCOREinterpret *const*)ptr, "vbox:IAppliance_interpret");
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREreadResponse:
		return soap_out_PointerTo_vbox__IAppliance_USCOREreadResponse(soap, tag, id, (_vbox__IAppliance_USCOREreadResponse *const*)ptr, "vbox:IAppliance_readResponse");
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREread:
		return soap_out_PointerTo_vbox__IAppliance_USCOREread(soap, tag, id, (_vbox__IAppliance_USCOREread *const*)ptr, "vbox:IAppliance_read");
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREgetVirtualSystemDescriptionsResponse:
		return soap_out_PointerTo_vbox__IAppliance_USCOREgetVirtualSystemDescriptionsResponse(soap, tag, id, (_vbox__IAppliance_USCOREgetVirtualSystemDescriptionsResponse *const*)ptr, "vbox:IAppliance_getVirtualSystemDescriptionsResponse");
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREgetVirtualSystemDescriptions:
		return soap_out_PointerTo_vbox__IAppliance_USCOREgetVirtualSystemDescriptions(soap, tag, id, (_vbox__IAppliance_USCOREgetVirtualSystemDescriptions *const*)ptr, "vbox:IAppliance_getVirtualSystemDescriptions");
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREgetDisksResponse:
		return soap_out_PointerTo_vbox__IAppliance_USCOREgetDisksResponse(soap, tag, id, (_vbox__IAppliance_USCOREgetDisksResponse *const*)ptr, "vbox:IAppliance_getDisksResponse");
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREgetDisks:
		return soap_out_PointerTo_vbox__IAppliance_USCOREgetDisks(soap, tag, id, (_vbox__IAppliance_USCOREgetDisks *const*)ptr, "vbox:IAppliance_getDisks");
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREgetPathResponse:
		return soap_out_PointerTo_vbox__IAppliance_USCOREgetPathResponse(soap, tag, id, (_vbox__IAppliance_USCOREgetPathResponse *const*)ptr, "vbox:IAppliance_getPathResponse");
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREgetPath:
		return soap_out_PointerTo_vbox__IAppliance_USCOREgetPath(soap, tag, id, (_vbox__IAppliance_USCOREgetPath *const*)ptr, "vbox:IAppliance_getPath");
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREremoveResponse:
		return soap_out_PointerTo_vbox__IVFSExplorer_USCOREremoveResponse(soap, tag, id, (_vbox__IVFSExplorer_USCOREremoveResponse *const*)ptr, "vbox:IVFSExplorer_removeResponse");
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREremove:
		return soap_out_PointerTo_vbox__IVFSExplorer_USCOREremove(soap, tag, id, (_vbox__IVFSExplorer_USCOREremove *const*)ptr, "vbox:IVFSExplorer_remove");
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREexistsResponse:
		return soap_out_PointerTo_vbox__IVFSExplorer_USCOREexistsResponse(soap, tag, id, (_vbox__IVFSExplorer_USCOREexistsResponse *const*)ptr, "vbox:IVFSExplorer_existsResponse");
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREexists:
		return soap_out_PointerTo_vbox__IVFSExplorer_USCOREexists(soap, tag, id, (_vbox__IVFSExplorer_USCOREexists *const*)ptr, "vbox:IVFSExplorer_exists");
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREentryListResponse:
		return soap_out_PointerTo_vbox__IVFSExplorer_USCOREentryListResponse(soap, tag, id, (_vbox__IVFSExplorer_USCOREentryListResponse *const*)ptr, "vbox:IVFSExplorer_entryListResponse");
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREentryList:
		return soap_out_PointerTo_vbox__IVFSExplorer_USCOREentryList(soap, tag, id, (_vbox__IVFSExplorer_USCOREentryList *const*)ptr, "vbox:IVFSExplorer_entryList");
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREcdUpResponse:
		return soap_out_PointerTo_vbox__IVFSExplorer_USCOREcdUpResponse(soap, tag, id, (_vbox__IVFSExplorer_USCOREcdUpResponse *const*)ptr, "vbox:IVFSExplorer_cdUpResponse");
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREcdUp:
		return soap_out_PointerTo_vbox__IVFSExplorer_USCOREcdUp(soap, tag, id, (_vbox__IVFSExplorer_USCOREcdUp *const*)ptr, "vbox:IVFSExplorer_cdUp");
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREcdResponse:
		return soap_out_PointerTo_vbox__IVFSExplorer_USCOREcdResponse(soap, tag, id, (_vbox__IVFSExplorer_USCOREcdResponse *const*)ptr, "vbox:IVFSExplorer_cdResponse");
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREcd:
		return soap_out_PointerTo_vbox__IVFSExplorer_USCOREcd(soap, tag, id, (_vbox__IVFSExplorer_USCOREcd *const*)ptr, "vbox:IVFSExplorer_cd");
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREupdateResponse:
		return soap_out_PointerTo_vbox__IVFSExplorer_USCOREupdateResponse(soap, tag, id, (_vbox__IVFSExplorer_USCOREupdateResponse *const*)ptr, "vbox:IVFSExplorer_updateResponse");
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREupdate:
		return soap_out_PointerTo_vbox__IVFSExplorer_USCOREupdate(soap, tag, id, (_vbox__IVFSExplorer_USCOREupdate *const*)ptr, "vbox:IVFSExplorer_update");
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREgetTypeResponse:
		return soap_out_PointerTo_vbox__IVFSExplorer_USCOREgetTypeResponse(soap, tag, id, (_vbox__IVFSExplorer_USCOREgetTypeResponse *const*)ptr, "vbox:IVFSExplorer_getTypeResponse");
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREgetType:
		return soap_out_PointerTo_vbox__IVFSExplorer_USCOREgetType(soap, tag, id, (_vbox__IVFSExplorer_USCOREgetType *const*)ptr, "vbox:IVFSExplorer_getType");
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREgetPathResponse:
		return soap_out_PointerTo_vbox__IVFSExplorer_USCOREgetPathResponse(soap, tag, id, (_vbox__IVFSExplorer_USCOREgetPathResponse *const*)ptr, "vbox:IVFSExplorer_getPathResponse");
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREgetPath:
		return soap_out_PointerTo_vbox__IVFSExplorer_USCOREgetPath(soap, tag, id, (_vbox__IVFSExplorer_USCOREgetPath *const*)ptr, "vbox:IVFSExplorer_getPath");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREremoveDHCPServerResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREremoveDHCPServerResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREremoveDHCPServerResponse *const*)ptr, "vbox:IVirtualBox_removeDHCPServerResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREremoveDHCPServer:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREremoveDHCPServer(soap, tag, id, (_vbox__IVirtualBox_USCOREremoveDHCPServer *const*)ptr, "vbox:IVirtualBox_removeDHCPServer");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREfindDHCPServerByNetworkNameResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREfindDHCPServerByNetworkNameResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREfindDHCPServerByNetworkNameResponse *const*)ptr, "vbox:IVirtualBox_findDHCPServerByNetworkNameResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREfindDHCPServerByNetworkName:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREfindDHCPServerByNetworkName(soap, tag, id, (_vbox__IVirtualBox_USCOREfindDHCPServerByNetworkName *const*)ptr, "vbox:IVirtualBox_findDHCPServerByNetworkName");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREcreateDHCPServerResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREcreateDHCPServerResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREcreateDHCPServerResponse *const*)ptr, "vbox:IVirtualBox_createDHCPServerResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREcreateDHCPServer:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREcreateDHCPServer(soap, tag, id, (_vbox__IVirtualBox_USCOREcreateDHCPServer *const*)ptr, "vbox:IVirtualBox_createDHCPServer");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREsaveSettingsWithBackupResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREsaveSettingsWithBackupResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREsaveSettingsWithBackupResponse *const*)ptr, "vbox:IVirtualBox_saveSettingsWithBackupResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREsaveSettingsWithBackup:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREsaveSettingsWithBackup(soap, tag, id, (_vbox__IVirtualBox_USCOREsaveSettingsWithBackup *const*)ptr, "vbox:IVirtualBox_saveSettingsWithBackup");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREsaveSettingsResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREsaveSettingsResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREsaveSettingsResponse *const*)ptr, "vbox:IVirtualBox_saveSettingsResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREsaveSettings:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREsaveSettings(soap, tag, id, (_vbox__IVirtualBox_USCOREsaveSettings *const*)ptr, "vbox:IVirtualBox_saveSettings");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREwaitForPropertyChangeResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREwaitForPropertyChangeResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREwaitForPropertyChangeResponse *const*)ptr, "vbox:IVirtualBox_waitForPropertyChangeResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREwaitForPropertyChange:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREwaitForPropertyChange(soap, tag, id, (_vbox__IVirtualBox_USCOREwaitForPropertyChange *const*)ptr, "vbox:IVirtualBox_waitForPropertyChange");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenExistingSessionResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREopenExistingSessionResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREopenExistingSessionResponse *const*)ptr, "vbox:IVirtualBox_openExistingSessionResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenExistingSession:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREopenExistingSession(soap, tag, id, (_vbox__IVirtualBox_USCOREopenExistingSession *const*)ptr, "vbox:IVirtualBox_openExistingSession");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenRemoteSessionResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREopenRemoteSessionResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREopenRemoteSessionResponse *const*)ptr, "vbox:IVirtualBox_openRemoteSessionResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenRemoteSession:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREopenRemoteSession(soap, tag, id, (_vbox__IVirtualBox_USCOREopenRemoteSession *const*)ptr, "vbox:IVirtualBox_openRemoteSession");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenSessionResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREopenSessionResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREopenSessionResponse *const*)ptr, "vbox:IVirtualBox_openSessionResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenSession:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREopenSession(soap, tag, id, (_vbox__IVirtualBox_USCOREopenSession *const*)ptr, "vbox:IVirtualBox_openSession");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREsetExtraDataResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREsetExtraDataResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREsetExtraDataResponse *const*)ptr, "vbox:IVirtualBox_setExtraDataResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREsetExtraData:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREsetExtraData(soap, tag, id, (_vbox__IVirtualBox_USCOREsetExtraData *const*)ptr, "vbox:IVirtualBox_setExtraData");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetExtraDataResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetExtraDataResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetExtraDataResponse *const*)ptr, "vbox:IVirtualBox_getExtraDataResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetExtraData:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetExtraData(soap, tag, id, (_vbox__IVirtualBox_USCOREgetExtraData *const*)ptr, "vbox:IVirtualBox_getExtraData");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetNextExtraDataKeyResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetNextExtraDataKeyResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetNextExtraDataKeyResponse *const*)ptr, "vbox:IVirtualBox_getNextExtraDataKeyResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetNextExtraDataKey:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetNextExtraDataKey(soap, tag, id, (_vbox__IVirtualBox_USCOREgetNextExtraDataKey *const*)ptr, "vbox:IVirtualBox_getNextExtraDataKey");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREremoveSharedFolderResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREremoveSharedFolderResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREremoveSharedFolderResponse *const*)ptr, "vbox:IVirtualBox_removeSharedFolderResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREremoveSharedFolder:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREremoveSharedFolder(soap, tag, id, (_vbox__IVirtualBox_USCOREremoveSharedFolder *const*)ptr, "vbox:IVirtualBox_removeSharedFolder");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREcreateSharedFolderResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREcreateSharedFolderResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREcreateSharedFolderResponse *const*)ptr, "vbox:IVirtualBox_createSharedFolderResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREcreateSharedFolder:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREcreateSharedFolder(soap, tag, id, (_vbox__IVirtualBox_USCOREcreateSharedFolder *const*)ptr, "vbox:IVirtualBox_createSharedFolder");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetGuestOSTypeResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetGuestOSTypeResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetGuestOSTypeResponse *const*)ptr, "vbox:IVirtualBox_getGuestOSTypeResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetGuestOSType:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetGuestOSType(soap, tag, id, (_vbox__IVirtualBox_USCOREgetGuestOSType *const*)ptr, "vbox:IVirtualBox_getGuestOSType");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREfindFloppyImageResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREfindFloppyImageResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREfindFloppyImageResponse *const*)ptr, "vbox:IVirtualBox_findFloppyImageResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREfindFloppyImage:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREfindFloppyImage(soap, tag, id, (_vbox__IVirtualBox_USCOREfindFloppyImage *const*)ptr, "vbox:IVirtualBox_findFloppyImage");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetFloppyImageResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetFloppyImageResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetFloppyImageResponse *const*)ptr, "vbox:IVirtualBox_getFloppyImageResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetFloppyImage:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetFloppyImage(soap, tag, id, (_vbox__IVirtualBox_USCOREgetFloppyImage *const*)ptr, "vbox:IVirtualBox_getFloppyImage");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenFloppyImageResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREopenFloppyImageResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREopenFloppyImageResponse *const*)ptr, "vbox:IVirtualBox_openFloppyImageResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenFloppyImage:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREopenFloppyImage(soap, tag, id, (_vbox__IVirtualBox_USCOREopenFloppyImage *const*)ptr, "vbox:IVirtualBox_openFloppyImage");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREfindDVDImageResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREfindDVDImageResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREfindDVDImageResponse *const*)ptr, "vbox:IVirtualBox_findDVDImageResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREfindDVDImage:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREfindDVDImage(soap, tag, id, (_vbox__IVirtualBox_USCOREfindDVDImage *const*)ptr, "vbox:IVirtualBox_findDVDImage");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetDVDImageResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetDVDImageResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetDVDImageResponse *const*)ptr, "vbox:IVirtualBox_getDVDImageResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetDVDImage:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetDVDImage(soap, tag, id, (_vbox__IVirtualBox_USCOREgetDVDImage *const*)ptr, "vbox:IVirtualBox_getDVDImage");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenDVDImageResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREopenDVDImageResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREopenDVDImageResponse *const*)ptr, "vbox:IVirtualBox_openDVDImageResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenDVDImage:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREopenDVDImage(soap, tag, id, (_vbox__IVirtualBox_USCOREopenDVDImage *const*)ptr, "vbox:IVirtualBox_openDVDImage");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREfindHardDiskResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREfindHardDiskResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREfindHardDiskResponse *const*)ptr, "vbox:IVirtualBox_findHardDiskResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREfindHardDisk:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREfindHardDisk(soap, tag, id, (_vbox__IVirtualBox_USCOREfindHardDisk *const*)ptr, "vbox:IVirtualBox_findHardDisk");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetHardDiskResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetHardDiskResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetHardDiskResponse *const*)ptr, "vbox:IVirtualBox_getHardDiskResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetHardDisk:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetHardDisk(soap, tag, id, (_vbox__IVirtualBox_USCOREgetHardDisk *const*)ptr, "vbox:IVirtualBox_getHardDisk");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenHardDiskResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREopenHardDiskResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREopenHardDiskResponse *const*)ptr, "vbox:IVirtualBox_openHardDiskResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenHardDisk:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREopenHardDisk(soap, tag, id, (_vbox__IVirtualBox_USCOREopenHardDisk *const*)ptr, "vbox:IVirtualBox_openHardDisk");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREcreateHardDiskResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREcreateHardDiskResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREcreateHardDiskResponse *const*)ptr, "vbox:IVirtualBox_createHardDiskResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREcreateHardDisk:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREcreateHardDisk(soap, tag, id, (_vbox__IVirtualBox_USCOREcreateHardDisk *const*)ptr, "vbox:IVirtualBox_createHardDisk");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREcreateApplianceResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREcreateApplianceResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREcreateApplianceResponse *const*)ptr, "vbox:IVirtualBox_createApplianceResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREcreateAppliance:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREcreateAppliance(soap, tag, id, (_vbox__IVirtualBox_USCOREcreateAppliance *const*)ptr, "vbox:IVirtualBox_createAppliance");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREunregisterMachineResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREunregisterMachineResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREunregisterMachineResponse *const*)ptr, "vbox:IVirtualBox_unregisterMachineResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREunregisterMachine:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREunregisterMachine(soap, tag, id, (_vbox__IVirtualBox_USCOREunregisterMachine *const*)ptr, "vbox:IVirtualBox_unregisterMachine");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREfindMachineResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREfindMachineResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREfindMachineResponse *const*)ptr, "vbox:IVirtualBox_findMachineResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREfindMachine:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREfindMachine(soap, tag, id, (_vbox__IVirtualBox_USCOREfindMachine *const*)ptr, "vbox:IVirtualBox_findMachine");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetMachineResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetMachineResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetMachineResponse *const*)ptr, "vbox:IVirtualBox_getMachineResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetMachine:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetMachine(soap, tag, id, (_vbox__IVirtualBox_USCOREgetMachine *const*)ptr, "vbox:IVirtualBox_getMachine");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREregisterMachineResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREregisterMachineResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREregisterMachineResponse *const*)ptr, "vbox:IVirtualBox_registerMachineResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREregisterMachine:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREregisterMachine(soap, tag, id, (_vbox__IVirtualBox_USCOREregisterMachine *const*)ptr, "vbox:IVirtualBox_registerMachine");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenMachineResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREopenMachineResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREopenMachineResponse *const*)ptr, "vbox:IVirtualBox_openMachineResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenMachine:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREopenMachine(soap, tag, id, (_vbox__IVirtualBox_USCOREopenMachine *const*)ptr, "vbox:IVirtualBox_openMachine");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREcreateLegacyMachineResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREcreateLegacyMachineResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREcreateLegacyMachineResponse *const*)ptr, "vbox:IVirtualBox_createLegacyMachineResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREcreateLegacyMachine:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREcreateLegacyMachine(soap, tag, id, (_vbox__IVirtualBox_USCOREcreateLegacyMachine *const*)ptr, "vbox:IVirtualBox_createLegacyMachine");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREcreateMachineResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREcreateMachineResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREcreateMachineResponse *const*)ptr, "vbox:IVirtualBox_createMachineResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREcreateMachine:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREcreateMachine(soap, tag, id, (_vbox__IVirtualBox_USCOREcreateMachine *const*)ptr, "vbox:IVirtualBox_createMachine");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetDHCPServersResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetDHCPServersResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetDHCPServersResponse *const*)ptr, "vbox:IVirtualBox_getDHCPServersResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetDHCPServers:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetDHCPServers(soap, tag, id, (_vbox__IVirtualBox_USCOREgetDHCPServers *const*)ptr, "vbox:IVirtualBox_getDHCPServers");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetPerformanceCollectorResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetPerformanceCollectorResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetPerformanceCollectorResponse *const*)ptr, "vbox:IVirtualBox_getPerformanceCollectorResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetPerformanceCollector:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetPerformanceCollector(soap, tag, id, (_vbox__IVirtualBox_USCOREgetPerformanceCollector *const*)ptr, "vbox:IVirtualBox_getPerformanceCollector");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetSharedFoldersResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetSharedFoldersResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetSharedFoldersResponse *const*)ptr, "vbox:IVirtualBox_getSharedFoldersResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetSharedFolders:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetSharedFolders(soap, tag, id, (_vbox__IVirtualBox_USCOREgetSharedFolders *const*)ptr, "vbox:IVirtualBox_getSharedFolders");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetGuestOSTypesResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetGuestOSTypesResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetGuestOSTypesResponse *const*)ptr, "vbox:IVirtualBox_getGuestOSTypesResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetGuestOSTypes:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetGuestOSTypes(soap, tag, id, (_vbox__IVirtualBox_USCOREgetGuestOSTypes *const*)ptr, "vbox:IVirtualBox_getGuestOSTypes");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetProgressOperationsResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetProgressOperationsResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetProgressOperationsResponse *const*)ptr, "vbox:IVirtualBox_getProgressOperationsResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetProgressOperations:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetProgressOperations(soap, tag, id, (_vbox__IVirtualBox_USCOREgetProgressOperations *const*)ptr, "vbox:IVirtualBox_getProgressOperations");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetFloppyImagesResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetFloppyImagesResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetFloppyImagesResponse *const*)ptr, "vbox:IVirtualBox_getFloppyImagesResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetFloppyImages:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetFloppyImages(soap, tag, id, (_vbox__IVirtualBox_USCOREgetFloppyImages *const*)ptr, "vbox:IVirtualBox_getFloppyImages");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetDVDImagesResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetDVDImagesResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetDVDImagesResponse *const*)ptr, "vbox:IVirtualBox_getDVDImagesResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetDVDImages:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetDVDImages(soap, tag, id, (_vbox__IVirtualBox_USCOREgetDVDImages *const*)ptr, "vbox:IVirtualBox_getDVDImages");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetHardDisksResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetHardDisksResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetHardDisksResponse *const*)ptr, "vbox:IVirtualBox_getHardDisksResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetHardDisks:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetHardDisks(soap, tag, id, (_vbox__IVirtualBox_USCOREgetHardDisks *const*)ptr, "vbox:IVirtualBox_getHardDisks");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetMachinesResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetMachinesResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetMachinesResponse *const*)ptr, "vbox:IVirtualBox_getMachinesResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetMachines:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetMachines(soap, tag, id, (_vbox__IVirtualBox_USCOREgetMachines *const*)ptr, "vbox:IVirtualBox_getMachines");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetSystemPropertiesResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetSystemPropertiesResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetSystemPropertiesResponse *const*)ptr, "vbox:IVirtualBox_getSystemPropertiesResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetSystemProperties:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetSystemProperties(soap, tag, id, (_vbox__IVirtualBox_USCOREgetSystemProperties *const*)ptr, "vbox:IVirtualBox_getSystemProperties");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetHostResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetHostResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetHostResponse *const*)ptr, "vbox:IVirtualBox_getHostResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetHost:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetHost(soap, tag, id, (_vbox__IVirtualBox_USCOREgetHost *const*)ptr, "vbox:IVirtualBox_getHost");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetSettingsFormatVersionResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetSettingsFormatVersionResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetSettingsFormatVersionResponse *const*)ptr, "vbox:IVirtualBox_getSettingsFormatVersionResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetSettingsFormatVersion:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetSettingsFormatVersion(soap, tag, id, (_vbox__IVirtualBox_USCOREgetSettingsFormatVersion *const*)ptr, "vbox:IVirtualBox_getSettingsFormatVersion");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetSettingsFileVersionResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetSettingsFileVersionResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetSettingsFileVersionResponse *const*)ptr, "vbox:IVirtualBox_getSettingsFileVersionResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetSettingsFileVersion:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetSettingsFileVersion(soap, tag, id, (_vbox__IVirtualBox_USCOREgetSettingsFileVersion *const*)ptr, "vbox:IVirtualBox_getSettingsFileVersion");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetSettingsFilePathResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetSettingsFilePathResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetSettingsFilePathResponse *const*)ptr, "vbox:IVirtualBox_getSettingsFilePathResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetSettingsFilePath:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetSettingsFilePath(soap, tag, id, (_vbox__IVirtualBox_USCOREgetSettingsFilePath *const*)ptr, "vbox:IVirtualBox_getSettingsFilePath");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetHomeFolderResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetHomeFolderResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetHomeFolderResponse *const*)ptr, "vbox:IVirtualBox_getHomeFolderResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetHomeFolder:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetHomeFolder(soap, tag, id, (_vbox__IVirtualBox_USCOREgetHomeFolder *const*)ptr, "vbox:IVirtualBox_getHomeFolder");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetPackageTypeResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetPackageTypeResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetPackageTypeResponse *const*)ptr, "vbox:IVirtualBox_getPackageTypeResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetPackageType:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetPackageType(soap, tag, id, (_vbox__IVirtualBox_USCOREgetPackageType *const*)ptr, "vbox:IVirtualBox_getPackageType");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetRevisionResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetRevisionResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetRevisionResponse *const*)ptr, "vbox:IVirtualBox_getRevisionResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetRevision:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetRevision(soap, tag, id, (_vbox__IVirtualBox_USCOREgetRevision *const*)ptr, "vbox:IVirtualBox_getRevision");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetVersionResponse:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetVersionResponse(soap, tag, id, (_vbox__IVirtualBox_USCOREgetVersionResponse *const*)ptr, "vbox:IVirtualBox_getVersionResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetVersion:
		return soap_out_PointerTo_vbox__IVirtualBox_USCOREgetVersion(soap, tag, id, (_vbox__IVirtualBox_USCOREgetVersion *const*)ptr, "vbox:IVirtualBox_getVersion");
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREstopResponse:
		return soap_out_PointerTo_vbox__IDHCPServer_USCOREstopResponse(soap, tag, id, (_vbox__IDHCPServer_USCOREstopResponse *const*)ptr, "vbox:IDHCPServer_stopResponse");
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREstop:
		return soap_out_PointerTo_vbox__IDHCPServer_USCOREstop(soap, tag, id, (_vbox__IDHCPServer_USCOREstop *const*)ptr, "vbox:IDHCPServer_stop");
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREstartResponse:
		return soap_out_PointerTo_vbox__IDHCPServer_USCOREstartResponse(soap, tag, id, (_vbox__IDHCPServer_USCOREstartResponse *const*)ptr, "vbox:IDHCPServer_startResponse");
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREstart:
		return soap_out_PointerTo_vbox__IDHCPServer_USCOREstart(soap, tag, id, (_vbox__IDHCPServer_USCOREstart *const*)ptr, "vbox:IDHCPServer_start");
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREsetConfigurationResponse:
		return soap_out_PointerTo_vbox__IDHCPServer_USCOREsetConfigurationResponse(soap, tag, id, (_vbox__IDHCPServer_USCOREsetConfigurationResponse *const*)ptr, "vbox:IDHCPServer_setConfigurationResponse");
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREsetConfiguration:
		return soap_out_PointerTo_vbox__IDHCPServer_USCOREsetConfiguration(soap, tag, id, (_vbox__IDHCPServer_USCOREsetConfiguration *const*)ptr, "vbox:IDHCPServer_setConfiguration");
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREgetUpperIPResponse:
		return soap_out_PointerTo_vbox__IDHCPServer_USCOREgetUpperIPResponse(soap, tag, id, (_vbox__IDHCPServer_USCOREgetUpperIPResponse *const*)ptr, "vbox:IDHCPServer_getUpperIPResponse");
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREgetUpperIP:
		return soap_out_PointerTo_vbox__IDHCPServer_USCOREgetUpperIP(soap, tag, id, (_vbox__IDHCPServer_USCOREgetUpperIP *const*)ptr, "vbox:IDHCPServer_getUpperIP");
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREgetLowerIPResponse:
		return soap_out_PointerTo_vbox__IDHCPServer_USCOREgetLowerIPResponse(soap, tag, id, (_vbox__IDHCPServer_USCOREgetLowerIPResponse *const*)ptr, "vbox:IDHCPServer_getLowerIPResponse");
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREgetLowerIP:
		return soap_out_PointerTo_vbox__IDHCPServer_USCOREgetLowerIP(soap, tag, id, (_vbox__IDHCPServer_USCOREgetLowerIP *const*)ptr, "vbox:IDHCPServer_getLowerIP");
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREgetNetworkNameResponse:
		return soap_out_PointerTo_vbox__IDHCPServer_USCOREgetNetworkNameResponse(soap, tag, id, (_vbox__IDHCPServer_USCOREgetNetworkNameResponse *const*)ptr, "vbox:IDHCPServer_getNetworkNameResponse");
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREgetNetworkName:
		return soap_out_PointerTo_vbox__IDHCPServer_USCOREgetNetworkName(soap, tag, id, (_vbox__IDHCPServer_USCOREgetNetworkName *const*)ptr, "vbox:IDHCPServer_getNetworkName");
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREgetNetworkMaskResponse:
		return soap_out_PointerTo_vbox__IDHCPServer_USCOREgetNetworkMaskResponse(soap, tag, id, (_vbox__IDHCPServer_USCOREgetNetworkMaskResponse *const*)ptr, "vbox:IDHCPServer_getNetworkMaskResponse");
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREgetNetworkMask:
		return soap_out_PointerTo_vbox__IDHCPServer_USCOREgetNetworkMask(soap, tag, id, (_vbox__IDHCPServer_USCOREgetNetworkMask *const*)ptr, "vbox:IDHCPServer_getNetworkMask");
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREgetIPAddressResponse:
		return soap_out_PointerTo_vbox__IDHCPServer_USCOREgetIPAddressResponse(soap, tag, id, (_vbox__IDHCPServer_USCOREgetIPAddressResponse *const*)ptr, "vbox:IDHCPServer_getIPAddressResponse");
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREgetIPAddress:
		return soap_out_PointerTo_vbox__IDHCPServer_USCOREgetIPAddress(soap, tag, id, (_vbox__IDHCPServer_USCOREgetIPAddress *const*)ptr, "vbox:IDHCPServer_getIPAddress");
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREsetEnabledResponse:
		return soap_out_PointerTo_vbox__IDHCPServer_USCOREsetEnabledResponse(soap, tag, id, (_vbox__IDHCPServer_USCOREsetEnabledResponse *const*)ptr, "vbox:IDHCPServer_setEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREsetEnabled:
		return soap_out_PointerTo_vbox__IDHCPServer_USCOREsetEnabled(soap, tag, id, (_vbox__IDHCPServer_USCOREsetEnabled *const*)ptr, "vbox:IDHCPServer_setEnabled");
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREgetEnabledResponse:
		return soap_out_PointerTo_vbox__IDHCPServer_USCOREgetEnabledResponse(soap, tag, id, (_vbox__IDHCPServer_USCOREgetEnabledResponse *const*)ptr, "vbox:IDHCPServer_getEnabledResponse");
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREgetEnabled:
		return soap_out_PointerTo_vbox__IDHCPServer_USCOREgetEnabled(soap, tag, id, (_vbox__IDHCPServer_USCOREgetEnabled *const*)ptr, "vbox:IDHCPServer_getEnabled");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetNextResponse:
		return soap_out_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetNextResponse(soap, tag, id, (_vbox__IVirtualBoxErrorInfo_USCOREgetNextResponse *const*)ptr, "vbox:IVirtualBoxErrorInfo_getNextResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetNext:
		return soap_out_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetNext(soap, tag, id, (_vbox__IVirtualBoxErrorInfo_USCOREgetNext *const*)ptr, "vbox:IVirtualBoxErrorInfo_getNext");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetTextResponse:
		return soap_out_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetTextResponse(soap, tag, id, (_vbox__IVirtualBoxErrorInfo_USCOREgetTextResponse *const*)ptr, "vbox:IVirtualBoxErrorInfo_getTextResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetText:
		return soap_out_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetText(soap, tag, id, (_vbox__IVirtualBoxErrorInfo_USCOREgetText *const*)ptr, "vbox:IVirtualBoxErrorInfo_getText");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetComponentResponse:
		return soap_out_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetComponentResponse(soap, tag, id, (_vbox__IVirtualBoxErrorInfo_USCOREgetComponentResponse *const*)ptr, "vbox:IVirtualBoxErrorInfo_getComponentResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetComponent:
		return soap_out_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetComponent(soap, tag, id, (_vbox__IVirtualBoxErrorInfo_USCOREgetComponent *const*)ptr, "vbox:IVirtualBoxErrorInfo_getComponent");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceIDResponse:
		return soap_out_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceIDResponse(soap, tag, id, (_vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceIDResponse *const*)ptr, "vbox:IVirtualBoxErrorInfo_getInterfaceIDResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceID:
		return soap_out_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceID(soap, tag, id, (_vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceID *const*)ptr, "vbox:IVirtualBoxErrorInfo_getInterfaceID");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetResultCodeResponse:
		return soap_out_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetResultCodeResponse(soap, tag, id, (_vbox__IVirtualBoxErrorInfo_USCOREgetResultCodeResponse *const*)ptr, "vbox:IVirtualBoxErrorInfo_getResultCodeResponse");
	case SOAP_TYPE_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetResultCode:
		return soap_out_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetResultCode(soap, tag, id, (_vbox__IVirtualBoxErrorInfo_USCOREgetResultCode *const*)ptr, "vbox:IVirtualBoxErrorInfo_getResultCode");
	case SOAP_TYPE_PointerTovbox__IRemoteDisplayInfo:
		return soap_out_PointerTovbox__IRemoteDisplayInfo(soap, tag, id, (vbox__IRemoteDisplayInfo *const*)ptr, "vbox:IRemoteDisplayInfo");
	case SOAP_TYPE_PointerTovbox__IHardDiskAttachment:
		return soap_out_PointerTovbox__IHardDiskAttachment(soap, tag, id, (vbox__IHardDiskAttachment *const*)ptr, "vbox:IHardDiskAttachment");
	case SOAP_TYPE_PointerTovbox__ISharedFolder:
		return soap_out_PointerTovbox__ISharedFolder(soap, tag, id, (vbox__ISharedFolder *const*)ptr, "vbox:ISharedFolder");
	case SOAP_TYPE_PointerTovbox__IGuestOSType:
		return soap_out_PointerTovbox__IGuestOSType(soap, tag, id, (vbox__IGuestOSType *const*)ptr, "vbox:IGuestOSType");
	case SOAP_TYPE_PointerTo_vbox__RuntimeFault:
		return soap_out_PointerTo_vbox__RuntimeFault(soap, tag, id, (_vbox__RuntimeFault *const*)ptr, "vbox:RuntimeFault");
	case SOAP_TYPE_PointerTo_vbox__InvalidObjectFault:
		return soap_out_PointerTo_vbox__InvalidObjectFault(soap, tag, id, (_vbox__InvalidObjectFault *const*)ptr, "vbox:InvalidObjectFault");
	case SOAP_TYPE__QName:
		return soap_out_string(soap, tag, id, (char*const*)&ptr, "xsd:QName");
	case SOAP_TYPE_string:
		return soap_out_string(soap, tag, id, (char*const*)&ptr, "xsd:string");
	}
	return SOAP_OK;
}

#ifdef __cplusplus
}
#endif
#endif

#ifndef WITH_NOIDREF

#ifdef __cplusplus
extern "C" {
#endif
SOAP_FMAC3 void SOAP_FMAC4 soap_markelement(struct soap *soap, const void *ptr, int type)
{
	(void)soap; (void)ptr; (void)type; /* appease -Wall -Werror */
	switch (type)
	{
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREqueryMetricsDataResponse:
		((_vbox__IPerformanceCollector_USCOREqueryMetricsDataResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREqueryMetricsData:
		((_vbox__IPerformanceCollector_USCOREqueryMetricsData *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREdisableMetricsResponse:
		((_vbox__IPerformanceCollector_USCOREdisableMetricsResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREdisableMetrics:
		((_vbox__IPerformanceCollector_USCOREdisableMetrics *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREenableMetricsResponse:
		((_vbox__IPerformanceCollector_USCOREenableMetricsResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREenableMetrics:
		((_vbox__IPerformanceCollector_USCOREenableMetrics *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREsetupMetricsResponse:
		((_vbox__IPerformanceCollector_USCOREsetupMetricsResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREsetupMetrics:
		((_vbox__IPerformanceCollector_USCOREsetupMetrics *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREgetMetricsResponse:
		((_vbox__IPerformanceCollector_USCOREgetMetricsResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREgetMetrics:
		((_vbox__IPerformanceCollector_USCOREgetMetrics *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREgetMetricNamesResponse:
		((_vbox__IPerformanceCollector_USCOREgetMetricNamesResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREgetMetricNames:
		((_vbox__IPerformanceCollector_USCOREgetMetricNames *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetMaximumValueResponse:
		((_vbox__IPerformanceMetric_USCOREgetMaximumValueResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetMaximumValue:
		((_vbox__IPerformanceMetric_USCOREgetMaximumValue *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetMinimumValueResponse:
		((_vbox__IPerformanceMetric_USCOREgetMinimumValueResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetMinimumValue:
		((_vbox__IPerformanceMetric_USCOREgetMinimumValue *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetUnitResponse:
		((_vbox__IPerformanceMetric_USCOREgetUnitResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetUnit:
		((_vbox__IPerformanceMetric_USCOREgetUnit *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetCountResponse:
		((_vbox__IPerformanceMetric_USCOREgetCountResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetCount:
		((_vbox__IPerformanceMetric_USCOREgetCount *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetPeriodResponse:
		((_vbox__IPerformanceMetric_USCOREgetPeriodResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetPeriod:
		((_vbox__IPerformanceMetric_USCOREgetPeriod *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetDescriptionResponse:
		((_vbox__IPerformanceMetric_USCOREgetDescriptionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetDescription:
		((_vbox__IPerformanceMetric_USCOREgetDescription *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetObjectResponse:
		((_vbox__IPerformanceMetric_USCOREgetObjectResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetObject:
		((_vbox__IPerformanceMetric_USCOREgetObject *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetMetricNameResponse:
		((_vbox__IPerformanceMetric_USCOREgetMetricNameResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetMetricName:
		((_vbox__IPerformanceMetric_USCOREgetMetricName *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IWebsessionManager_USCORElogoffResponse:
		((_vbox__IWebsessionManager_USCORElogoffResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IWebsessionManager_USCORElogoff:
		((_vbox__IWebsessionManager_USCORElogoff *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IWebsessionManager_USCOREgetSessionObjectResponse:
		((_vbox__IWebsessionManager_USCOREgetSessionObjectResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IWebsessionManager_USCOREgetSessionObject:
		((_vbox__IWebsessionManager_USCOREgetSessionObject *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IWebsessionManager_USCORElogonResponse:
		((_vbox__IWebsessionManager_USCORElogonResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IWebsessionManager_USCORElogon:
		((_vbox__IWebsessionManager_USCORElogon *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IManagedObjectRef_USCOREreleaseResponse:
		((_vbox__IManagedObjectRef_USCOREreleaseResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IManagedObjectRef_USCORErelease:
		((_vbox__IManagedObjectRef_USCORErelease *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IManagedObjectRef_USCOREgetInterfaceNameResponse:
		((_vbox__IManagedObjectRef_USCOREgetInterfaceNameResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IManagedObjectRef_USCOREgetInterfaceName:
		((_vbox__IManagedObjectRef_USCOREgetInterfaceName *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCORESetIDEEmulationPortResponse:
		((_vbox__IStorageController_USCORESetIDEEmulationPortResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCORESetIDEEmulationPort:
		((_vbox__IStorageController_USCORESetIDEEmulationPort *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCOREGetIDEEmulationPortResponse:
		((_vbox__IStorageController_USCOREGetIDEEmulationPortResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCOREGetIDEEmulationPort:
		((_vbox__IStorageController_USCOREGetIDEEmulationPort *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCOREsetControllerTypeResponse:
		((_vbox__IStorageController_USCOREsetControllerTypeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCOREsetControllerType:
		((_vbox__IStorageController_USCOREsetControllerType *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCOREgetControllerTypeResponse:
		((_vbox__IStorageController_USCOREgetControllerTypeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCOREgetControllerType:
		((_vbox__IStorageController_USCOREgetControllerType *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCOREgetBusResponse:
		((_vbox__IStorageController_USCOREgetBusResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCOREgetBus:
		((_vbox__IStorageController_USCOREgetBus *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCOREsetPortCountResponse:
		((_vbox__IStorageController_USCOREsetPortCountResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCOREsetPortCount:
		((_vbox__IStorageController_USCOREsetPortCount *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCOREgetPortCountResponse:
		((_vbox__IStorageController_USCOREgetPortCountResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCOREgetPortCount:
		((_vbox__IStorageController_USCOREgetPortCount *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCOREsetInstanceResponse:
		((_vbox__IStorageController_USCOREsetInstanceResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCOREsetInstance:
		((_vbox__IStorageController_USCOREsetInstance *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCOREgetInstanceResponse:
		((_vbox__IStorageController_USCOREgetInstanceResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCOREgetInstance:
		((_vbox__IStorageController_USCOREgetInstance *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCOREgetMaxPortCountResponse:
		((_vbox__IStorageController_USCOREgetMaxPortCountResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCOREgetMaxPortCount:
		((_vbox__IStorageController_USCOREgetMaxPortCount *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCOREgetMinPortCountResponse:
		((_vbox__IStorageController_USCOREgetMinPortCountResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCOREgetMinPortCount:
		((_vbox__IStorageController_USCOREgetMinPortCount *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCOREgetMaxDevicesPerPortCountResponse:
		((_vbox__IStorageController_USCOREgetMaxDevicesPerPortCountResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCOREgetMaxDevicesPerPortCount:
		((_vbox__IStorageController_USCOREgetMaxDevicesPerPortCount *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCOREgetNameResponse:
		((_vbox__IStorageController_USCOREgetNameResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IStorageController_USCOREgetName:
		((_vbox__IStorageController_USCOREgetName *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISession_USCOREcloseResponse:
		((_vbox__ISession_USCOREcloseResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISession_USCOREclose:
		((_vbox__ISession_USCOREclose *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISession_USCOREgetConsoleResponse:
		((_vbox__ISession_USCOREgetConsoleResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISession_USCOREgetConsole:
		((_vbox__ISession_USCOREgetConsole *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISession_USCOREgetMachineResponse:
		((_vbox__ISession_USCOREgetMachineResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISession_USCOREgetMachine:
		((_vbox__ISession_USCOREgetMachine *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISession_USCOREgetTypeResponse:
		((_vbox__ISession_USCOREgetTypeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISession_USCOREgetType:
		((_vbox__ISession_USCOREgetType *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISession_USCOREgetStateResponse:
		((_vbox__ISession_USCOREgetStateResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISession_USCOREgetState:
		((_vbox__ISession_USCOREgetState *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetReuseSingleConnectionResponse:
		((_vbox__IVRDPServer_USCOREsetReuseSingleConnectionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetReuseSingleConnection:
		((_vbox__IVRDPServer_USCOREsetReuseSingleConnection *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetReuseSingleConnectionResponse:
		((_vbox__IVRDPServer_USCOREgetReuseSingleConnectionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetReuseSingleConnection:
		((_vbox__IVRDPServer_USCOREgetReuseSingleConnection *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetAllowMultiConnectionResponse:
		((_vbox__IVRDPServer_USCOREsetAllowMultiConnectionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetAllowMultiConnection:
		((_vbox__IVRDPServer_USCOREsetAllowMultiConnection *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetAllowMultiConnectionResponse:
		((_vbox__IVRDPServer_USCOREgetAllowMultiConnectionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetAllowMultiConnection:
		((_vbox__IVRDPServer_USCOREgetAllowMultiConnection *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetAuthTimeoutResponse:
		((_vbox__IVRDPServer_USCOREsetAuthTimeoutResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetAuthTimeout:
		((_vbox__IVRDPServer_USCOREsetAuthTimeout *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetAuthTimeoutResponse:
		((_vbox__IVRDPServer_USCOREgetAuthTimeoutResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetAuthTimeout:
		((_vbox__IVRDPServer_USCOREgetAuthTimeout *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetAuthTypeResponse:
		((_vbox__IVRDPServer_USCOREsetAuthTypeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetAuthType:
		((_vbox__IVRDPServer_USCOREsetAuthType *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetAuthTypeResponse:
		((_vbox__IVRDPServer_USCOREgetAuthTypeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetAuthType:
		((_vbox__IVRDPServer_USCOREgetAuthType *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetNetAddressResponse:
		((_vbox__IVRDPServer_USCOREsetNetAddressResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetNetAddress:
		((_vbox__IVRDPServer_USCOREsetNetAddress *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetNetAddressResponse:
		((_vbox__IVRDPServer_USCOREgetNetAddressResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetNetAddress:
		((_vbox__IVRDPServer_USCOREgetNetAddress *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetPortResponse:
		((_vbox__IVRDPServer_USCOREsetPortResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetPort:
		((_vbox__IVRDPServer_USCOREsetPort *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetPortResponse:
		((_vbox__IVRDPServer_USCOREgetPortResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetPort:
		((_vbox__IVRDPServer_USCOREgetPort *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetEnabledResponse:
		((_vbox__IVRDPServer_USCOREsetEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetEnabled:
		((_vbox__IVRDPServer_USCOREsetEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetEnabledResponse:
		((_vbox__IVRDPServer_USCOREgetEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetEnabled:
		((_vbox__IVRDPServer_USCOREgetEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREsetAudioDriverResponse:
		((_vbox__IAudioAdapter_USCOREsetAudioDriverResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREsetAudioDriver:
		((_vbox__IAudioAdapter_USCOREsetAudioDriver *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREgetAudioDriverResponse:
		((_vbox__IAudioAdapter_USCOREgetAudioDriverResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREgetAudioDriver:
		((_vbox__IAudioAdapter_USCOREgetAudioDriver *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREsetAudioControllerResponse:
		((_vbox__IAudioAdapter_USCOREsetAudioControllerResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREsetAudioController:
		((_vbox__IAudioAdapter_USCOREsetAudioController *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREgetAudioControllerResponse:
		((_vbox__IAudioAdapter_USCOREgetAudioControllerResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREgetAudioController:
		((_vbox__IAudioAdapter_USCOREgetAudioController *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREsetEnabledResponse:
		((_vbox__IAudioAdapter_USCOREsetEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREsetEnabled:
		((_vbox__IAudioAdapter_USCOREsetEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREgetEnabledResponse:
		((_vbox__IAudioAdapter_USCOREgetEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREgetEnabled:
		((_vbox__IAudioAdapter_USCOREgetEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostUSBDeviceFilter_USCOREsetActionResponse:
		((_vbox__IHostUSBDeviceFilter_USCOREsetActionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostUSBDeviceFilter_USCOREsetAction:
		((_vbox__IHostUSBDeviceFilter_USCOREsetAction *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostUSBDeviceFilter_USCOREgetActionResponse:
		((_vbox__IHostUSBDeviceFilter_USCOREgetActionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostUSBDeviceFilter_USCOREgetAction:
		((_vbox__IHostUSBDeviceFilter_USCOREgetAction *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostUSBDevice_USCOREgetStateResponse:
		((_vbox__IHostUSBDevice_USCOREgetStateResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostUSBDevice_USCOREgetState:
		((_vbox__IHostUSBDevice_USCOREgetState *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetMaskedInterfacesResponse:
		((_vbox__IUSBDeviceFilter_USCOREsetMaskedInterfacesResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetMaskedInterfaces:
		((_vbox__IUSBDeviceFilter_USCOREsetMaskedInterfaces *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetMaskedInterfacesResponse:
		((_vbox__IUSBDeviceFilter_USCOREgetMaskedInterfacesResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetMaskedInterfaces:
		((_vbox__IUSBDeviceFilter_USCOREgetMaskedInterfaces *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetRemoteResponse:
		((_vbox__IUSBDeviceFilter_USCOREsetRemoteResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetRemote:
		((_vbox__IUSBDeviceFilter_USCOREsetRemote *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetRemoteResponse:
		((_vbox__IUSBDeviceFilter_USCOREgetRemoteResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetRemote:
		((_vbox__IUSBDeviceFilter_USCOREgetRemote *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetPortResponse:
		((_vbox__IUSBDeviceFilter_USCOREsetPortResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetPort:
		((_vbox__IUSBDeviceFilter_USCOREsetPort *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetPortResponse:
		((_vbox__IUSBDeviceFilter_USCOREgetPortResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetPort:
		((_vbox__IUSBDeviceFilter_USCOREgetPort *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetSerialNumberResponse:
		((_vbox__IUSBDeviceFilter_USCOREsetSerialNumberResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetSerialNumber:
		((_vbox__IUSBDeviceFilter_USCOREsetSerialNumber *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetSerialNumberResponse:
		((_vbox__IUSBDeviceFilter_USCOREgetSerialNumberResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetSerialNumber:
		((_vbox__IUSBDeviceFilter_USCOREgetSerialNumber *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetProductResponse:
		((_vbox__IUSBDeviceFilter_USCOREsetProductResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetProduct:
		((_vbox__IUSBDeviceFilter_USCOREsetProduct *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetProductResponse:
		((_vbox__IUSBDeviceFilter_USCOREgetProductResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetProduct:
		((_vbox__IUSBDeviceFilter_USCOREgetProduct *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetManufacturerResponse:
		((_vbox__IUSBDeviceFilter_USCOREsetManufacturerResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetManufacturer:
		((_vbox__IUSBDeviceFilter_USCOREsetManufacturer *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetManufacturerResponse:
		((_vbox__IUSBDeviceFilter_USCOREgetManufacturerResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetManufacturer:
		((_vbox__IUSBDeviceFilter_USCOREgetManufacturer *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetRevisionResponse:
		((_vbox__IUSBDeviceFilter_USCOREsetRevisionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetRevision:
		((_vbox__IUSBDeviceFilter_USCOREsetRevision *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetRevisionResponse:
		((_vbox__IUSBDeviceFilter_USCOREgetRevisionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetRevision:
		((_vbox__IUSBDeviceFilter_USCOREgetRevision *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetProductIdResponse:
		((_vbox__IUSBDeviceFilter_USCOREsetProductIdResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetProductId:
		((_vbox__IUSBDeviceFilter_USCOREsetProductId *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetProductIdResponse:
		((_vbox__IUSBDeviceFilter_USCOREgetProductIdResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetProductId:
		((_vbox__IUSBDeviceFilter_USCOREgetProductId *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetVendorIdResponse:
		((_vbox__IUSBDeviceFilter_USCOREsetVendorIdResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetVendorId:
		((_vbox__IUSBDeviceFilter_USCOREsetVendorId *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetVendorIdResponse:
		((_vbox__IUSBDeviceFilter_USCOREgetVendorIdResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetVendorId:
		((_vbox__IUSBDeviceFilter_USCOREgetVendorId *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetActiveResponse:
		((_vbox__IUSBDeviceFilter_USCOREsetActiveResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetActive:
		((_vbox__IUSBDeviceFilter_USCOREsetActive *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetActiveResponse:
		((_vbox__IUSBDeviceFilter_USCOREgetActiveResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetActive:
		((_vbox__IUSBDeviceFilter_USCOREgetActive *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetNameResponse:
		((_vbox__IUSBDeviceFilter_USCOREsetNameResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetName:
		((_vbox__IUSBDeviceFilter_USCOREsetName *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetNameResponse:
		((_vbox__IUSBDeviceFilter_USCOREgetNameResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetName:
		((_vbox__IUSBDeviceFilter_USCOREgetName *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetRemoteResponse:
		((_vbox__IUSBDevice_USCOREgetRemoteResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetRemote:
		((_vbox__IUSBDevice_USCOREgetRemote *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetPortVersionResponse:
		((_vbox__IUSBDevice_USCOREgetPortVersionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetPortVersion:
		((_vbox__IUSBDevice_USCOREgetPortVersion *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetVersionResponse:
		((_vbox__IUSBDevice_USCOREgetVersionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetVersion:
		((_vbox__IUSBDevice_USCOREgetVersion *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetPortResponse:
		((_vbox__IUSBDevice_USCOREgetPortResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetPort:
		((_vbox__IUSBDevice_USCOREgetPort *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetAddressResponse:
		((_vbox__IUSBDevice_USCOREgetAddressResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetAddress:
		((_vbox__IUSBDevice_USCOREgetAddress *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetSerialNumberResponse:
		((_vbox__IUSBDevice_USCOREgetSerialNumberResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetSerialNumber:
		((_vbox__IUSBDevice_USCOREgetSerialNumber *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetProductResponse:
		((_vbox__IUSBDevice_USCOREgetProductResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetProduct:
		((_vbox__IUSBDevice_USCOREgetProduct *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetManufacturerResponse:
		((_vbox__IUSBDevice_USCOREgetManufacturerResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetManufacturer:
		((_vbox__IUSBDevice_USCOREgetManufacturer *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetRevisionResponse:
		((_vbox__IUSBDevice_USCOREgetRevisionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetRevision:
		((_vbox__IUSBDevice_USCOREgetRevision *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetProductIdResponse:
		((_vbox__IUSBDevice_USCOREgetProductIdResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetProductId:
		((_vbox__IUSBDevice_USCOREgetProductId *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetVendorIdResponse:
		((_vbox__IUSBDevice_USCOREgetVendorIdResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetVendorId:
		((_vbox__IUSBDevice_USCOREgetVendorId *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetIdResponse:
		((_vbox__IUSBDevice_USCOREgetIdResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetId:
		((_vbox__IUSBDevice_USCOREgetId *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBController_USCOREremoveDeviceFilterResponse:
		((_vbox__IUSBController_USCOREremoveDeviceFilterResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBController_USCOREremoveDeviceFilter:
		((_vbox__IUSBController_USCOREremoveDeviceFilter *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBController_USCOREinsertDeviceFilterResponse:
		((_vbox__IUSBController_USCOREinsertDeviceFilterResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBController_USCOREinsertDeviceFilter:
		((_vbox__IUSBController_USCOREinsertDeviceFilter *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBController_USCOREcreateDeviceFilterResponse:
		((_vbox__IUSBController_USCOREcreateDeviceFilterResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBController_USCOREcreateDeviceFilter:
		((_vbox__IUSBController_USCOREcreateDeviceFilter *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBController_USCOREgetDeviceFiltersResponse:
		((_vbox__IUSBController_USCOREgetDeviceFiltersResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBController_USCOREgetDeviceFilters:
		((_vbox__IUSBController_USCOREgetDeviceFilters *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBController_USCOREgetUSBStandardResponse:
		((_vbox__IUSBController_USCOREgetUSBStandardResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBController_USCOREgetUSBStandard:
		((_vbox__IUSBController_USCOREgetUSBStandard *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBController_USCOREsetEnabledEhciResponse:
		((_vbox__IUSBController_USCOREsetEnabledEhciResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBController_USCOREsetEnabledEhci:
		((_vbox__IUSBController_USCOREsetEnabledEhci *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBController_USCOREgetEnabledEhciResponse:
		((_vbox__IUSBController_USCOREgetEnabledEhciResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBController_USCOREgetEnabledEhci:
		((_vbox__IUSBController_USCOREgetEnabledEhci *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBController_USCOREsetEnabledResponse:
		((_vbox__IUSBController_USCOREsetEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBController_USCOREsetEnabled:
		((_vbox__IUSBController_USCOREsetEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBController_USCOREgetEnabledResponse:
		((_vbox__IUSBController_USCOREgetEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IUSBController_USCOREgetEnabled:
		((_vbox__IUSBController_USCOREgetEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IParallelPort_USCOREsetPathResponse:
		((_vbox__IParallelPort_USCOREsetPathResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IParallelPort_USCOREsetPath:
		((_vbox__IParallelPort_USCOREsetPath *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetPathResponse:
		((_vbox__IParallelPort_USCOREgetPathResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetPath:
		((_vbox__IParallelPort_USCOREgetPath *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IParallelPort_USCOREsetIRQResponse:
		((_vbox__IParallelPort_USCOREsetIRQResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IParallelPort_USCOREsetIRQ:
		((_vbox__IParallelPort_USCOREsetIRQ *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetIRQResponse:
		((_vbox__IParallelPort_USCOREgetIRQResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetIRQ:
		((_vbox__IParallelPort_USCOREgetIRQ *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IParallelPort_USCOREsetIOBaseResponse:
		((_vbox__IParallelPort_USCOREsetIOBaseResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IParallelPort_USCOREsetIOBase:
		((_vbox__IParallelPort_USCOREsetIOBase *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetIOBaseResponse:
		((_vbox__IParallelPort_USCOREgetIOBaseResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetIOBase:
		((_vbox__IParallelPort_USCOREgetIOBase *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IParallelPort_USCOREsetEnabledResponse:
		((_vbox__IParallelPort_USCOREsetEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IParallelPort_USCOREsetEnabled:
		((_vbox__IParallelPort_USCOREsetEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetEnabledResponse:
		((_vbox__IParallelPort_USCOREgetEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetEnabled:
		((_vbox__IParallelPort_USCOREgetEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetSlotResponse:
		((_vbox__IParallelPort_USCOREgetSlotResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetSlot:
		((_vbox__IParallelPort_USCOREgetSlot *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetPathResponse:
		((_vbox__ISerialPort_USCOREsetPathResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetPath:
		((_vbox__ISerialPort_USCOREsetPath *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetPathResponse:
		((_vbox__ISerialPort_USCOREgetPathResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetPath:
		((_vbox__ISerialPort_USCOREgetPath *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetServerResponse:
		((_vbox__ISerialPort_USCOREsetServerResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetServer:
		((_vbox__ISerialPort_USCOREsetServer *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetServerResponse:
		((_vbox__ISerialPort_USCOREgetServerResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetServer:
		((_vbox__ISerialPort_USCOREgetServer *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetHostModeResponse:
		((_vbox__ISerialPort_USCOREsetHostModeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetHostMode:
		((_vbox__ISerialPort_USCOREsetHostMode *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetHostModeResponse:
		((_vbox__ISerialPort_USCOREgetHostModeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetHostMode:
		((_vbox__ISerialPort_USCOREgetHostMode *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetIRQResponse:
		((_vbox__ISerialPort_USCOREsetIRQResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetIRQ:
		((_vbox__ISerialPort_USCOREsetIRQ *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetIRQResponse:
		((_vbox__ISerialPort_USCOREgetIRQResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetIRQ:
		((_vbox__ISerialPort_USCOREgetIRQ *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetIOBaseResponse:
		((_vbox__ISerialPort_USCOREsetIOBaseResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetIOBase:
		((_vbox__ISerialPort_USCOREsetIOBase *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetIOBaseResponse:
		((_vbox__ISerialPort_USCOREgetIOBaseResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetIOBase:
		((_vbox__ISerialPort_USCOREgetIOBase *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetEnabledResponse:
		((_vbox__ISerialPort_USCOREsetEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetEnabled:
		((_vbox__ISerialPort_USCOREsetEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetEnabledResponse:
		((_vbox__ISerialPort_USCOREgetEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetEnabled:
		((_vbox__ISerialPort_USCOREgetEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetSlotResponse:
		((_vbox__ISerialPort_USCOREgetSlotResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetSlot:
		((_vbox__ISerialPort_USCOREgetSlot *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREdetachResponse:
		((_vbox__INetworkAdapter_USCOREdetachResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREdetach:
		((_vbox__INetworkAdapter_USCOREdetach *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREattachToHostOnlyInterfaceResponse:
		((_vbox__INetworkAdapter_USCOREattachToHostOnlyInterfaceResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREattachToHostOnlyInterface:
		((_vbox__INetworkAdapter_USCOREattachToHostOnlyInterface *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREattachToInternalNetworkResponse:
		((_vbox__INetworkAdapter_USCOREattachToInternalNetworkResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREattachToInternalNetwork:
		((_vbox__INetworkAdapter_USCOREattachToInternalNetwork *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREattachToBridgedInterfaceResponse:
		((_vbox__INetworkAdapter_USCOREattachToBridgedInterfaceResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREattachToBridgedInterface:
		((_vbox__INetworkAdapter_USCOREattachToBridgedInterface *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREattachToNATResponse:
		((_vbox__INetworkAdapter_USCOREattachToNATResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREattachToNAT:
		((_vbox__INetworkAdapter_USCOREattachToNAT *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetTraceFileResponse:
		((_vbox__INetworkAdapter_USCOREsetTraceFileResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetTraceFile:
		((_vbox__INetworkAdapter_USCOREsetTraceFile *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetTraceFileResponse:
		((_vbox__INetworkAdapter_USCOREgetTraceFileResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetTraceFile:
		((_vbox__INetworkAdapter_USCOREgetTraceFile *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetTraceEnabledResponse:
		((_vbox__INetworkAdapter_USCOREsetTraceEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetTraceEnabled:
		((_vbox__INetworkAdapter_USCOREsetTraceEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetTraceEnabledResponse:
		((_vbox__INetworkAdapter_USCOREgetTraceEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetTraceEnabled:
		((_vbox__INetworkAdapter_USCOREgetTraceEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetLineSpeedResponse:
		((_vbox__INetworkAdapter_USCOREsetLineSpeedResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetLineSpeed:
		((_vbox__INetworkAdapter_USCOREsetLineSpeed *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetLineSpeedResponse:
		((_vbox__INetworkAdapter_USCOREgetLineSpeedResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetLineSpeed:
		((_vbox__INetworkAdapter_USCOREgetLineSpeed *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetCableConnectedResponse:
		((_vbox__INetworkAdapter_USCOREsetCableConnectedResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetCableConnected:
		((_vbox__INetworkAdapter_USCOREsetCableConnected *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetCableConnectedResponse:
		((_vbox__INetworkAdapter_USCOREgetCableConnectedResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetCableConnected:
		((_vbox__INetworkAdapter_USCOREgetCableConnected *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetNATNetworkResponse:
		((_vbox__INetworkAdapter_USCOREsetNATNetworkResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetNATNetwork:
		((_vbox__INetworkAdapter_USCOREsetNATNetwork *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetNATNetworkResponse:
		((_vbox__INetworkAdapter_USCOREgetNATNetworkResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetNATNetwork:
		((_vbox__INetworkAdapter_USCOREgetNATNetwork *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetInternalNetworkResponse:
		((_vbox__INetworkAdapter_USCOREsetInternalNetworkResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetInternalNetwork:
		((_vbox__INetworkAdapter_USCOREsetInternalNetwork *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetInternalNetworkResponse:
		((_vbox__INetworkAdapter_USCOREgetInternalNetworkResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetInternalNetwork:
		((_vbox__INetworkAdapter_USCOREgetInternalNetwork *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetHostInterfaceResponse:
		((_vbox__INetworkAdapter_USCOREsetHostInterfaceResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetHostInterface:
		((_vbox__INetworkAdapter_USCOREsetHostInterface *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetHostInterfaceResponse:
		((_vbox__INetworkAdapter_USCOREgetHostInterfaceResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetHostInterface:
		((_vbox__INetworkAdapter_USCOREgetHostInterface *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetAttachmentTypeResponse:
		((_vbox__INetworkAdapter_USCOREgetAttachmentTypeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetAttachmentType:
		((_vbox__INetworkAdapter_USCOREgetAttachmentType *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetMACAddressResponse:
		((_vbox__INetworkAdapter_USCOREsetMACAddressResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetMACAddress:
		((_vbox__INetworkAdapter_USCOREsetMACAddress *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetMACAddressResponse:
		((_vbox__INetworkAdapter_USCOREgetMACAddressResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetMACAddress:
		((_vbox__INetworkAdapter_USCOREgetMACAddress *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetEnabledResponse:
		((_vbox__INetworkAdapter_USCOREsetEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetEnabled:
		((_vbox__INetworkAdapter_USCOREsetEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetEnabledResponse:
		((_vbox__INetworkAdapter_USCOREgetEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetEnabled:
		((_vbox__INetworkAdapter_USCOREgetEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetSlotResponse:
		((_vbox__INetworkAdapter_USCOREgetSlotResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetSlot:
		((_vbox__INetworkAdapter_USCOREgetSlot *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetAdapterTypeResponse:
		((_vbox__INetworkAdapter_USCOREsetAdapterTypeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetAdapterType:
		((_vbox__INetworkAdapter_USCOREsetAdapterType *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetAdapterTypeResponse:
		((_vbox__INetworkAdapter_USCOREgetAdapterTypeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetAdapterType:
		((_vbox__INetworkAdapter_USCOREgetAdapterType *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDisplay_USCOREupdateCompletedResponse:
		((_vbox__IDisplay_USCOREupdateCompletedResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDisplay_USCOREupdateCompleted:
		((_vbox__IDisplay_USCOREupdateCompleted *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDisplay_USCOREresizeCompletedResponse:
		((_vbox__IDisplay_USCOREresizeCompletedResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDisplay_USCOREresizeCompleted:
		((_vbox__IDisplay_USCOREresizeCompleted *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDisplay_USCOREinvalidateAndUpdateResponse:
		((_vbox__IDisplay_USCOREinvalidateAndUpdateResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDisplay_USCOREinvalidateAndUpdate:
		((_vbox__IDisplay_USCOREinvalidateAndUpdate *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDisplay_USCOREtakeScreenShotSlowResponse:
		((_vbox__IDisplay_USCOREtakeScreenShotSlowResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDisplay_USCOREtakeScreenShotSlow:
		((_vbox__IDisplay_USCOREtakeScreenShotSlow *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDisplay_USCOREsetSeamlessModeResponse:
		((_vbox__IDisplay_USCOREsetSeamlessModeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDisplay_USCOREsetSeamlessMode:
		((_vbox__IDisplay_USCOREsetSeamlessMode *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDisplay_USCOREsetVideoModeHintResponse:
		((_vbox__IDisplay_USCOREsetVideoModeHintResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDisplay_USCOREsetVideoModeHint:
		((_vbox__IDisplay_USCOREsetVideoModeHint *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDisplay_USCOREgetBitsPerPixelResponse:
		((_vbox__IDisplay_USCOREgetBitsPerPixelResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDisplay_USCOREgetBitsPerPixel:
		((_vbox__IDisplay_USCOREgetBitsPerPixel *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDisplay_USCOREgetHeightResponse:
		((_vbox__IDisplay_USCOREgetHeightResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDisplay_USCOREgetHeight:
		((_vbox__IDisplay_USCOREgetHeight *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDisplay_USCOREgetWidthResponse:
		((_vbox__IDisplay_USCOREgetWidthResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDisplay_USCOREgetWidth:
		((_vbox__IDisplay_USCOREgetWidth *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMouse_USCOREputMouseEventAbsoluteResponse:
		((_vbox__IMouse_USCOREputMouseEventAbsoluteResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMouse_USCOREputMouseEventAbsolute:
		((_vbox__IMouse_USCOREputMouseEventAbsolute *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMouse_USCOREputMouseEventResponse:
		((_vbox__IMouse_USCOREputMouseEventResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMouse_USCOREputMouseEvent:
		((_vbox__IMouse_USCOREputMouseEvent *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMouse_USCOREgetAbsoluteSupportedResponse:
		((_vbox__IMouse_USCOREgetAbsoluteSupportedResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMouse_USCOREgetAbsoluteSupported:
		((_vbox__IMouse_USCOREgetAbsoluteSupported *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IKeyboard_USCOREputCADResponse:
		((_vbox__IKeyboard_USCOREputCADResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IKeyboard_USCOREputCAD:
		((_vbox__IKeyboard_USCOREputCAD *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IKeyboard_USCOREputScancodesResponse:
		((_vbox__IKeyboard_USCOREputScancodesResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IKeyboard_USCOREputScancodes:
		((_vbox__IKeyboard_USCOREputScancodes *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IKeyboard_USCOREputScancodeResponse:
		((_vbox__IKeyboard_USCOREputScancodeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IKeyboard_USCOREputScancode:
		((_vbox__IKeyboard_USCOREputScancode *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREgetHostDriveResponse:
		((_vbox__IFloppyDrive_USCOREgetHostDriveResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREgetHostDrive:
		((_vbox__IFloppyDrive_USCOREgetHostDrive *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREgetImageResponse:
		((_vbox__IFloppyDrive_USCOREgetImageResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREgetImage:
		((_vbox__IFloppyDrive_USCOREgetImage *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREunmountResponse:
		((_vbox__IFloppyDrive_USCOREunmountResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREunmount:
		((_vbox__IFloppyDrive_USCOREunmount *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREcaptureHostDriveResponse:
		((_vbox__IFloppyDrive_USCOREcaptureHostDriveResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREcaptureHostDrive:
		((_vbox__IFloppyDrive_USCOREcaptureHostDrive *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREmountImageResponse:
		((_vbox__IFloppyDrive_USCOREmountImageResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREmountImage:
		((_vbox__IFloppyDrive_USCOREmountImage *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREgetStateResponse:
		((_vbox__IFloppyDrive_USCOREgetStateResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREgetState:
		((_vbox__IFloppyDrive_USCOREgetState *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREsetEnabledResponse:
		((_vbox__IFloppyDrive_USCOREsetEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREsetEnabled:
		((_vbox__IFloppyDrive_USCOREsetEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREgetEnabledResponse:
		((_vbox__IFloppyDrive_USCOREgetEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREgetEnabled:
		((_vbox__IFloppyDrive_USCOREgetEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDVDDrive_USCOREgetHostDriveResponse:
		((_vbox__IDVDDrive_USCOREgetHostDriveResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDVDDrive_USCOREgetHostDrive:
		((_vbox__IDVDDrive_USCOREgetHostDrive *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDVDDrive_USCOREgetImageResponse:
		((_vbox__IDVDDrive_USCOREgetImageResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDVDDrive_USCOREgetImage:
		((_vbox__IDVDDrive_USCOREgetImage *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDVDDrive_USCOREunmountResponse:
		((_vbox__IDVDDrive_USCOREunmountResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDVDDrive_USCOREunmount:
		((_vbox__IDVDDrive_USCOREunmount *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDVDDrive_USCOREcaptureHostDriveResponse:
		((_vbox__IDVDDrive_USCOREcaptureHostDriveResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDVDDrive_USCOREcaptureHostDrive:
		((_vbox__IDVDDrive_USCOREcaptureHostDrive *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDVDDrive_USCOREmountImageResponse:
		((_vbox__IDVDDrive_USCOREmountImageResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDVDDrive_USCOREmountImage:
		((_vbox__IDVDDrive_USCOREmountImage *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDVDDrive_USCOREsetPassthroughResponse:
		((_vbox__IDVDDrive_USCOREsetPassthroughResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDVDDrive_USCOREsetPassthrough:
		((_vbox__IDVDDrive_USCOREsetPassthrough *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDVDDrive_USCOREgetPassthroughResponse:
		((_vbox__IDVDDrive_USCOREgetPassthroughResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDVDDrive_USCOREgetPassthrough:
		((_vbox__IDVDDrive_USCOREgetPassthrough *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDVDDrive_USCOREgetStateResponse:
		((_vbox__IDVDDrive_USCOREgetStateResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDVDDrive_USCOREgetState:
		((_vbox__IDVDDrive_USCOREgetState *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREdescribePropertiesResponse:
		((_vbox__IHardDiskFormat_USCOREdescribePropertiesResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREdescribeProperties:
		((_vbox__IHardDiskFormat_USCOREdescribeProperties *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREgetCapabilitiesResponse:
		((_vbox__IHardDiskFormat_USCOREgetCapabilitiesResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREgetCapabilities:
		((_vbox__IHardDiskFormat_USCOREgetCapabilities *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREgetFileExtensionsResponse:
		((_vbox__IHardDiskFormat_USCOREgetFileExtensionsResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREgetFileExtensions:
		((_vbox__IHardDiskFormat_USCOREgetFileExtensions *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREgetNameResponse:
		((_vbox__IHardDiskFormat_USCOREgetNameResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREgetName:
		((_vbox__IHardDiskFormat_USCOREgetName *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREgetIdResponse:
		((_vbox__IHardDiskFormat_USCOREgetIdResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREgetId:
		((_vbox__IHardDiskFormat_USCOREgetId *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREresetResponse:
		((_vbox__IHardDisk_USCOREresetResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREreset:
		((_vbox__IHardDisk_USCOREreset *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREcompactResponse:
		((_vbox__IHardDisk_USCOREcompactResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREcompact:
		((_vbox__IHardDisk_USCOREcompact *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREcloneToResponse:
		((_vbox__IHardDisk_USCOREcloneToResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREcloneTo:
		((_vbox__IHardDisk_USCOREcloneTo *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREmergeToResponse:
		((_vbox__IHardDisk_USCOREmergeToResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREmergeTo:
		((_vbox__IHardDisk_USCOREmergeTo *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREcreateDiffStorageResponse:
		((_vbox__IHardDisk_USCOREcreateDiffStorageResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREcreateDiffStorage:
		((_vbox__IHardDisk_USCOREcreateDiffStorage *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREdeleteStorageResponse:
		((_vbox__IHardDisk_USCOREdeleteStorageResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREdeleteStorage:
		((_vbox__IHardDisk_USCOREdeleteStorage *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREcreateBaseStorageResponse:
		((_vbox__IHardDisk_USCOREcreateBaseStorageResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREcreateBaseStorage:
		((_vbox__IHardDisk_USCOREcreateBaseStorage *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREsetPropertiesResponse:
		((_vbox__IHardDisk_USCOREsetPropertiesResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREsetProperties:
		((_vbox__IHardDisk_USCOREsetProperties *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetPropertiesResponse:
		((_vbox__IHardDisk_USCOREgetPropertiesResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetProperties:
		((_vbox__IHardDisk_USCOREgetProperties *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREsetPropertyResponse:
		((_vbox__IHardDisk_USCOREsetPropertyResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREsetProperty:
		((_vbox__IHardDisk_USCOREsetProperty *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetPropertyResponse:
		((_vbox__IHardDisk_USCOREgetPropertyResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetProperty:
		((_vbox__IHardDisk_USCOREgetProperty *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREsetAutoResetResponse:
		((_vbox__IHardDisk_USCOREsetAutoResetResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREsetAutoReset:
		((_vbox__IHardDisk_USCOREsetAutoReset *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetAutoResetResponse:
		((_vbox__IHardDisk_USCOREgetAutoResetResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetAutoReset:
		((_vbox__IHardDisk_USCOREgetAutoReset *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetLogicalSizeResponse:
		((_vbox__IHardDisk_USCOREgetLogicalSizeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetLogicalSize:
		((_vbox__IHardDisk_USCOREgetLogicalSize *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetReadOnlyResponse:
		((_vbox__IHardDisk_USCOREgetReadOnlyResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetReadOnly:
		((_vbox__IHardDisk_USCOREgetReadOnly *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetRootResponse:
		((_vbox__IHardDisk_USCOREgetRootResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetRoot:
		((_vbox__IHardDisk_USCOREgetRoot *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetChildrenResponse:
		((_vbox__IHardDisk_USCOREgetChildrenResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetChildren:
		((_vbox__IHardDisk_USCOREgetChildren *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetParentResponse:
		((_vbox__IHardDisk_USCOREgetParentResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetParent:
		((_vbox__IHardDisk_USCOREgetParent *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREsetTypeResponse:
		((_vbox__IHardDisk_USCOREsetTypeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREsetType:
		((_vbox__IHardDisk_USCOREsetType *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetTypeResponse:
		((_vbox__IHardDisk_USCOREgetTypeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetType:
		((_vbox__IHardDisk_USCOREgetType *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetFormatResponse:
		((_vbox__IHardDisk_USCOREgetFormatResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetFormat:
		((_vbox__IHardDisk_USCOREgetFormat *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREcloseResponse:
		((_vbox__IMedium_USCOREcloseResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREclose:
		((_vbox__IMedium_USCOREclose *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREunlockWriteResponse:
		((_vbox__IMedium_USCOREunlockWriteResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREunlockWrite:
		((_vbox__IMedium_USCOREunlockWrite *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCORElockWriteResponse:
		((_vbox__IMedium_USCORElockWriteResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCORElockWrite:
		((_vbox__IMedium_USCORElockWrite *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREunlockReadResponse:
		((_vbox__IMedium_USCOREunlockReadResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREunlockRead:
		((_vbox__IMedium_USCOREunlockRead *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCORElockReadResponse:
		((_vbox__IMedium_USCORElockReadResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCORElockRead:
		((_vbox__IMedium_USCORElockRead *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREgetSnapshotIdsResponse:
		((_vbox__IMedium_USCOREgetSnapshotIdsResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREgetSnapshotIds:
		((_vbox__IMedium_USCOREgetSnapshotIds *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREgetMachineIdsResponse:
		((_vbox__IMedium_USCOREgetMachineIdsResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREgetMachineIds:
		((_vbox__IMedium_USCOREgetMachineIds *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREgetLastAccessErrorResponse:
		((_vbox__IMedium_USCOREgetLastAccessErrorResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREgetLastAccessError:
		((_vbox__IMedium_USCOREgetLastAccessError *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREgetSizeResponse:
		((_vbox__IMedium_USCOREgetSizeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREgetSize:
		((_vbox__IMedium_USCOREgetSize *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREgetNameResponse:
		((_vbox__IMedium_USCOREgetNameResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREgetName:
		((_vbox__IMedium_USCOREgetName *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREsetLocationResponse:
		((_vbox__IMedium_USCOREsetLocationResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREsetLocation:
		((_vbox__IMedium_USCOREsetLocation *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREgetLocationResponse:
		((_vbox__IMedium_USCOREgetLocationResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREgetLocation:
		((_vbox__IMedium_USCOREgetLocation *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREgetStateResponse:
		((_vbox__IMedium_USCOREgetStateResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREgetState:
		((_vbox__IMedium_USCOREgetState *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREsetDescriptionResponse:
		((_vbox__IMedium_USCOREsetDescriptionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREsetDescription:
		((_vbox__IMedium_USCOREsetDescription *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREgetDescriptionResponse:
		((_vbox__IMedium_USCOREgetDescriptionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREgetDescription:
		((_vbox__IMedium_USCOREgetDescription *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREgetIdResponse:
		((_vbox__IMedium_USCOREgetIdResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMedium_USCOREgetId:
		((_vbox__IMedium_USCOREgetId *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetChildrenResponse:
		((_vbox__ISnapshot_USCOREgetChildrenResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetChildren:
		((_vbox__ISnapshot_USCOREgetChildren *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetParentResponse:
		((_vbox__ISnapshot_USCOREgetParentResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetParent:
		((_vbox__ISnapshot_USCOREgetParent *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetMachineResponse:
		((_vbox__ISnapshot_USCOREgetMachineResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetMachine:
		((_vbox__ISnapshot_USCOREgetMachine *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetOnlineResponse:
		((_vbox__ISnapshot_USCOREgetOnlineResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetOnline:
		((_vbox__ISnapshot_USCOREgetOnline *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetTimeStampResponse:
		((_vbox__ISnapshot_USCOREgetTimeStampResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetTimeStamp:
		((_vbox__ISnapshot_USCOREgetTimeStamp *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISnapshot_USCOREsetDescriptionResponse:
		((_vbox__ISnapshot_USCOREsetDescriptionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISnapshot_USCOREsetDescription:
		((_vbox__ISnapshot_USCOREsetDescription *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetDescriptionResponse:
		((_vbox__ISnapshot_USCOREgetDescriptionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetDescription:
		((_vbox__ISnapshot_USCOREgetDescription *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISnapshot_USCOREsetNameResponse:
		((_vbox__ISnapshot_USCOREsetNameResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISnapshot_USCOREsetName:
		((_vbox__ISnapshot_USCOREsetName *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetNameResponse:
		((_vbox__ISnapshot_USCOREgetNameResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetName:
		((_vbox__ISnapshot_USCOREgetName *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetIdResponse:
		((_vbox__ISnapshot_USCOREgetIdResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetId:
		((_vbox__ISnapshot_USCOREgetId *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREcancelResponse:
		((_vbox__IProgress_USCOREcancelResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREcancel:
		((_vbox__IProgress_USCOREcancel *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREwaitForOperationCompletionResponse:
		((_vbox__IProgress_USCOREwaitForOperationCompletionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREwaitForOperationCompletion:
		((_vbox__IProgress_USCOREwaitForOperationCompletion *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREwaitForCompletionResponse:
		((_vbox__IProgress_USCOREwaitForCompletionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREwaitForCompletion:
		((_vbox__IProgress_USCOREwaitForCompletion *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetOperationPercentResponse:
		((_vbox__IProgress_USCOREgetOperationPercentResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetOperationPercent:
		((_vbox__IProgress_USCOREgetOperationPercent *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetOperationDescriptionResponse:
		((_vbox__IProgress_USCOREgetOperationDescriptionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetOperationDescription:
		((_vbox__IProgress_USCOREgetOperationDescription *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetOperationResponse:
		((_vbox__IProgress_USCOREgetOperationResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetOperation:
		((_vbox__IProgress_USCOREgetOperation *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetOperationCountResponse:
		((_vbox__IProgress_USCOREgetOperationCountResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetOperationCount:
		((_vbox__IProgress_USCOREgetOperationCount *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetErrorInfoResponse:
		((_vbox__IProgress_USCOREgetErrorInfoResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetErrorInfo:
		((_vbox__IProgress_USCOREgetErrorInfo *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetResultCodeResponse:
		((_vbox__IProgress_USCOREgetResultCodeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetResultCode:
		((_vbox__IProgress_USCOREgetResultCode *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetCanceledResponse:
		((_vbox__IProgress_USCOREgetCanceledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetCanceled:
		((_vbox__IProgress_USCOREgetCanceled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetCompletedResponse:
		((_vbox__IProgress_USCOREgetCompletedResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetCompleted:
		((_vbox__IProgress_USCOREgetCompleted *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetTimeRemainingResponse:
		((_vbox__IProgress_USCOREgetTimeRemainingResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetTimeRemaining:
		((_vbox__IProgress_USCOREgetTimeRemaining *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetPercentResponse:
		((_vbox__IProgress_USCOREgetPercentResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetPercent:
		((_vbox__IProgress_USCOREgetPercent *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetCancelableResponse:
		((_vbox__IProgress_USCOREgetCancelableResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetCancelable:
		((_vbox__IProgress_USCOREgetCancelable *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetInitiatorResponse:
		((_vbox__IProgress_USCOREgetInitiatorResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetInitiator:
		((_vbox__IProgress_USCOREgetInitiator *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetDescriptionResponse:
		((_vbox__IProgress_USCOREgetDescriptionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetDescription:
		((_vbox__IProgress_USCOREgetDescription *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetIdResponse:
		((_vbox__IProgress_USCOREgetIdResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IProgress_USCOREgetId:
		((_vbox__IProgress_USCOREgetId *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IGuest_USCOREgetStatisticResponse:
		((_vbox__IGuest_USCOREgetStatisticResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IGuest_USCOREgetStatistic:
		((_vbox__IGuest_USCOREgetStatistic *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IGuest_USCOREsetCredentialsResponse:
		((_vbox__IGuest_USCOREsetCredentialsResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IGuest_USCOREsetCredentials:
		((_vbox__IGuest_USCOREsetCredentials *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IGuest_USCOREsetStatisticsUpdateIntervalResponse:
		((_vbox__IGuest_USCOREsetStatisticsUpdateIntervalResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IGuest_USCOREsetStatisticsUpdateInterval:
		((_vbox__IGuest_USCOREsetStatisticsUpdateInterval *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IGuest_USCOREgetStatisticsUpdateIntervalResponse:
		((_vbox__IGuest_USCOREgetStatisticsUpdateIntervalResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IGuest_USCOREgetStatisticsUpdateInterval:
		((_vbox__IGuest_USCOREgetStatisticsUpdateInterval *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IGuest_USCOREsetMemoryBalloonSizeResponse:
		((_vbox__IGuest_USCOREsetMemoryBalloonSizeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IGuest_USCOREsetMemoryBalloonSize:
		((_vbox__IGuest_USCOREsetMemoryBalloonSize *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IGuest_USCOREgetMemoryBalloonSizeResponse:
		((_vbox__IGuest_USCOREgetMemoryBalloonSizeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IGuest_USCOREgetMemoryBalloonSize:
		((_vbox__IGuest_USCOREgetMemoryBalloonSize *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IGuest_USCOREgetSupportsGraphicsResponse:
		((_vbox__IGuest_USCOREgetSupportsGraphicsResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IGuest_USCOREgetSupportsGraphics:
		((_vbox__IGuest_USCOREgetSupportsGraphics *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IGuest_USCOREgetSupportsSeamlessResponse:
		((_vbox__IGuest_USCOREgetSupportsSeamlessResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IGuest_USCOREgetSupportsSeamless:
		((_vbox__IGuest_USCOREgetSupportsSeamless *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IGuest_USCOREgetAdditionsVersionResponse:
		((_vbox__IGuest_USCOREgetAdditionsVersionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IGuest_USCOREgetAdditionsVersion:
		((_vbox__IGuest_USCOREgetAdditionsVersion *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IGuest_USCOREgetAdditionsActiveResponse:
		((_vbox__IGuest_USCOREgetAdditionsActiveResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IGuest_USCOREgetAdditionsActive:
		((_vbox__IGuest_USCOREgetAdditionsActive *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IGuest_USCOREgetOSTypeIdResponse:
		((_vbox__IGuest_USCOREgetOSTypeIdResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IGuest_USCOREgetOSTypeId:
		((_vbox__IGuest_USCOREgetOSTypeId *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetDefaultAudioDriverResponse:
		((_vbox__ISystemProperties_USCOREgetDefaultAudioDriverResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetDefaultAudioDriver:
		((_vbox__ISystemProperties_USCOREgetDefaultAudioDriver *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetLogHistoryCountResponse:
		((_vbox__ISystemProperties_USCOREsetLogHistoryCountResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetLogHistoryCount:
		((_vbox__ISystemProperties_USCOREsetLogHistoryCount *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetLogHistoryCountResponse:
		((_vbox__ISystemProperties_USCOREgetLogHistoryCountResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetLogHistoryCount:
		((_vbox__ISystemProperties_USCOREgetLogHistoryCount *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetWebServiceAuthLibraryResponse:
		((_vbox__ISystemProperties_USCOREsetWebServiceAuthLibraryResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetWebServiceAuthLibrary:
		((_vbox__ISystemProperties_USCOREsetWebServiceAuthLibrary *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetWebServiceAuthLibraryResponse:
		((_vbox__ISystemProperties_USCOREgetWebServiceAuthLibraryResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetWebServiceAuthLibrary:
		((_vbox__ISystemProperties_USCOREgetWebServiceAuthLibrary *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibraryResponse:
		((_vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibraryResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibrary:
		((_vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibrary *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibraryResponse:
		((_vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibraryResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibrary:
		((_vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibrary *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetDefaultHardDiskFormatResponse:
		((_vbox__ISystemProperties_USCOREsetDefaultHardDiskFormatResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetDefaultHardDiskFormat:
		((_vbox__ISystemProperties_USCOREsetDefaultHardDiskFormat *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetDefaultHardDiskFormatResponse:
		((_vbox__ISystemProperties_USCOREgetDefaultHardDiskFormatResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetDefaultHardDiskFormat:
		((_vbox__ISystemProperties_USCOREgetDefaultHardDiskFormat *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetHardDiskFormatsResponse:
		((_vbox__ISystemProperties_USCOREgetHardDiskFormatsResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetHardDiskFormats:
		((_vbox__ISystemProperties_USCOREgetHardDiskFormats *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetDefaultHardDiskFolderResponse:
		((_vbox__ISystemProperties_USCOREsetDefaultHardDiskFolderResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetDefaultHardDiskFolder:
		((_vbox__ISystemProperties_USCOREsetDefaultHardDiskFolder *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetDefaultHardDiskFolderResponse:
		((_vbox__ISystemProperties_USCOREgetDefaultHardDiskFolderResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetDefaultHardDiskFolder:
		((_vbox__ISystemProperties_USCOREgetDefaultHardDiskFolder *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetDefaultMachineFolderResponse:
		((_vbox__ISystemProperties_USCOREsetDefaultMachineFolderResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetDefaultMachineFolder:
		((_vbox__ISystemProperties_USCOREsetDefaultMachineFolder *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetDefaultMachineFolderResponse:
		((_vbox__ISystemProperties_USCOREgetDefaultMachineFolderResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetDefaultMachineFolder:
		((_vbox__ISystemProperties_USCOREgetDefaultMachineFolder *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxBootPositionResponse:
		((_vbox__ISystemProperties_USCOREgetMaxBootPositionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxBootPosition:
		((_vbox__ISystemProperties_USCOREgetMaxBootPosition *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetParallelPortCountResponse:
		((_vbox__ISystemProperties_USCOREgetParallelPortCountResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetParallelPortCount:
		((_vbox__ISystemProperties_USCOREgetParallelPortCount *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetSerialPortCountResponse:
		((_vbox__ISystemProperties_USCOREgetSerialPortCountResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetSerialPortCount:
		((_vbox__ISystemProperties_USCOREgetSerialPortCount *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetNetworkAdapterCountResponse:
		((_vbox__ISystemProperties_USCOREgetNetworkAdapterCountResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetNetworkAdapterCount:
		((_vbox__ISystemProperties_USCOREgetNetworkAdapterCount *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxVDISizeResponse:
		((_vbox__ISystemProperties_USCOREgetMaxVDISizeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxVDISize:
		((_vbox__ISystemProperties_USCOREgetMaxVDISize *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxGuestCPUCountResponse:
		((_vbox__ISystemProperties_USCOREgetMaxGuestCPUCountResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxGuestCPUCount:
		((_vbox__ISystemProperties_USCOREgetMaxGuestCPUCount *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMinGuestCPUCountResponse:
		((_vbox__ISystemProperties_USCOREgetMinGuestCPUCountResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMinGuestCPUCount:
		((_vbox__ISystemProperties_USCOREgetMinGuestCPUCount *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxGuestVRAMResponse:
		((_vbox__ISystemProperties_USCOREgetMaxGuestVRAMResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxGuestVRAM:
		((_vbox__ISystemProperties_USCOREgetMaxGuestVRAM *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMinGuestVRAMResponse:
		((_vbox__ISystemProperties_USCOREgetMinGuestVRAMResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMinGuestVRAM:
		((_vbox__ISystemProperties_USCOREgetMinGuestVRAM *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxGuestRAMResponse:
		((_vbox__ISystemProperties_USCOREgetMaxGuestRAMResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxGuestRAM:
		((_vbox__ISystemProperties_USCOREgetMaxGuestRAM *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMinGuestRAMResponse:
		((_vbox__ISystemProperties_USCOREgetMinGuestRAMResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMinGuestRAM:
		((_vbox__ISystemProperties_USCOREgetMinGuestRAM *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREfindUSBDeviceByAddressResponse:
		((_vbox__IHost_USCOREfindUSBDeviceByAddressResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREfindUSBDeviceByAddress:
		((_vbox__IHost_USCOREfindUSBDeviceByAddress *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREfindUSBDeviceByIdResponse:
		((_vbox__IHost_USCOREfindUSBDeviceByIdResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREfindUSBDeviceById:
		((_vbox__IHost_USCOREfindUSBDeviceById *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREfindHostNetworkInterfacesOfTypeResponse:
		((_vbox__IHost_USCOREfindHostNetworkInterfacesOfTypeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREfindHostNetworkInterfacesOfType:
		((_vbox__IHost_USCOREfindHostNetworkInterfacesOfType *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREfindHostNetworkInterfaceByIdResponse:
		((_vbox__IHost_USCOREfindHostNetworkInterfaceByIdResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREfindHostNetworkInterfaceById:
		((_vbox__IHost_USCOREfindHostNetworkInterfaceById *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREfindHostNetworkInterfaceByNameResponse:
		((_vbox__IHost_USCOREfindHostNetworkInterfaceByNameResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREfindHostNetworkInterfaceByName:
		((_vbox__IHost_USCOREfindHostNetworkInterfaceByName *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREfindHostFloppyDriveResponse:
		((_vbox__IHost_USCOREfindHostFloppyDriveResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREfindHostFloppyDrive:
		((_vbox__IHost_USCOREfindHostFloppyDrive *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREfindHostDVDDriveResponse:
		((_vbox__IHost_USCOREfindHostDVDDriveResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREfindHostDVDDrive:
		((_vbox__IHost_USCOREfindHostDVDDrive *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREremoveUSBDeviceFilterResponse:
		((_vbox__IHost_USCOREremoveUSBDeviceFilterResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREremoveUSBDeviceFilter:
		((_vbox__IHost_USCOREremoveUSBDeviceFilter *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREinsertUSBDeviceFilterResponse:
		((_vbox__IHost_USCOREinsertUSBDeviceFilterResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREinsertUSBDeviceFilter:
		((_vbox__IHost_USCOREinsertUSBDeviceFilter *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREcreateUSBDeviceFilterResponse:
		((_vbox__IHost_USCOREcreateUSBDeviceFilterResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREcreateUSBDeviceFilter:
		((_vbox__IHost_USCOREcreateUSBDeviceFilter *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREremoveHostOnlyNetworkInterfaceResponse:
		((_vbox__IHost_USCOREremoveHostOnlyNetworkInterfaceResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREremoveHostOnlyNetworkInterface:
		((_vbox__IHost_USCOREremoveHostOnlyNetworkInterface *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREcreateHostOnlyNetworkInterfaceResponse:
		((_vbox__IHost_USCOREcreateHostOnlyNetworkInterfaceResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREcreateHostOnlyNetworkInterface:
		((_vbox__IHost_USCOREcreateHostOnlyNetworkInterface *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorDescriptionResponse:
		((_vbox__IHost_USCOREgetProcessorDescriptionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorDescription:
		((_vbox__IHost_USCOREgetProcessorDescription *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorFeatureResponse:
		((_vbox__IHost_USCOREgetProcessorFeatureResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorFeature:
		((_vbox__IHost_USCOREgetProcessorFeature *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorSpeedResponse:
		((_vbox__IHost_USCOREgetProcessorSpeedResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorSpeed:
		((_vbox__IHost_USCOREgetProcessorSpeed *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetAcceleration3DAvailableResponse:
		((_vbox__IHost_USCOREgetAcceleration3DAvailableResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetAcceleration3DAvailable:
		((_vbox__IHost_USCOREgetAcceleration3DAvailable *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetUTCTimeResponse:
		((_vbox__IHost_USCOREgetUTCTimeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetUTCTime:
		((_vbox__IHost_USCOREgetUTCTime *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetOSVersionResponse:
		((_vbox__IHost_USCOREgetOSVersionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetOSVersion:
		((_vbox__IHost_USCOREgetOSVersion *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetOperatingSystemResponse:
		((_vbox__IHost_USCOREgetOperatingSystemResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetOperatingSystem:
		((_vbox__IHost_USCOREgetOperatingSystem *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetMemoryAvailableResponse:
		((_vbox__IHost_USCOREgetMemoryAvailableResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetMemoryAvailable:
		((_vbox__IHost_USCOREgetMemoryAvailable *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetMemorySizeResponse:
		((_vbox__IHost_USCOREgetMemorySizeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetMemorySize:
		((_vbox__IHost_USCOREgetMemorySize *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorOnlineCountResponse:
		((_vbox__IHost_USCOREgetProcessorOnlineCountResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorOnlineCount:
		((_vbox__IHost_USCOREgetProcessorOnlineCount *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorCountResponse:
		((_vbox__IHost_USCOREgetProcessorCountResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorCount:
		((_vbox__IHost_USCOREgetProcessorCount *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetNetworkInterfacesResponse:
		((_vbox__IHost_USCOREgetNetworkInterfacesResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetNetworkInterfaces:
		((_vbox__IHost_USCOREgetNetworkInterfaces *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetUSBDeviceFiltersResponse:
		((_vbox__IHost_USCOREgetUSBDeviceFiltersResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetUSBDeviceFilters:
		((_vbox__IHost_USCOREgetUSBDeviceFilters *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetUSBDevicesResponse:
		((_vbox__IHost_USCOREgetUSBDevicesResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetUSBDevices:
		((_vbox__IHost_USCOREgetUSBDevices *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetFloppyDrivesResponse:
		((_vbox__IHost_USCOREgetFloppyDrivesResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetFloppyDrives:
		((_vbox__IHost_USCOREgetFloppyDrives *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetDVDDrivesResponse:
		((_vbox__IHost_USCOREgetDVDDrivesResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHost_USCOREgetDVDDrives:
		((_vbox__IHost_USCOREgetDVDDrives *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREdhcpRediscoverResponse:
		((_vbox__IHostNetworkInterface_USCOREdhcpRediscoverResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREdhcpRediscover:
		((_vbox__IHostNetworkInterface_USCOREdhcpRediscover *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREenableDynamicIpConfigResponse:
		((_vbox__IHostNetworkInterface_USCOREenableDynamicIpConfigResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREenableDynamicIpConfig:
		((_vbox__IHostNetworkInterface_USCOREenableDynamicIpConfig *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6Response:
		((_vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6Response *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6:
		((_vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6 *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREenableStaticIpConfigResponse:
		((_vbox__IHostNetworkInterface_USCOREenableStaticIpConfigResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREenableStaticIpConfig:
		((_vbox__IHostNetworkInterface_USCOREenableStaticIpConfig *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetInterfaceTypeResponse:
		((_vbox__IHostNetworkInterface_USCOREgetInterfaceTypeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetInterfaceType:
		((_vbox__IHostNetworkInterface_USCOREgetInterfaceType *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetStatusResponse:
		((_vbox__IHostNetworkInterface_USCOREgetStatusResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetStatus:
		((_vbox__IHostNetworkInterface_USCOREgetStatus *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetMediumTypeResponse:
		((_vbox__IHostNetworkInterface_USCOREgetMediumTypeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetMediumType:
		((_vbox__IHostNetworkInterface_USCOREgetMediumType *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetHardwareAddressResponse:
		((_vbox__IHostNetworkInterface_USCOREgetHardwareAddressResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetHardwareAddress:
		((_vbox__IHostNetworkInterface_USCOREgetHardwareAddress *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLengthResponse:
		((_vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLengthResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLength:
		((_vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLength *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIPV6AddressResponse:
		((_vbox__IHostNetworkInterface_USCOREgetIPV6AddressResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIPV6Address:
		((_vbox__IHostNetworkInterface_USCOREgetIPV6Address *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIPV6SupportedResponse:
		((_vbox__IHostNetworkInterface_USCOREgetIPV6SupportedResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIPV6Supported:
		((_vbox__IHostNetworkInterface_USCOREgetIPV6Supported *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetNetworkMaskResponse:
		((_vbox__IHostNetworkInterface_USCOREgetNetworkMaskResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetNetworkMask:
		((_vbox__IHostNetworkInterface_USCOREgetNetworkMask *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIPAddressResponse:
		((_vbox__IHostNetworkInterface_USCOREgetIPAddressResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIPAddress:
		((_vbox__IHostNetworkInterface_USCOREgetIPAddress *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetDhcpEnabledResponse:
		((_vbox__IHostNetworkInterface_USCOREgetDhcpEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetDhcpEnabled:
		((_vbox__IHostNetworkInterface_USCOREgetDhcpEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetNetworkNameResponse:
		((_vbox__IHostNetworkInterface_USCOREgetNetworkNameResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetNetworkName:
		((_vbox__IHostNetworkInterface_USCOREgetNetworkName *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIdResponse:
		((_vbox__IHostNetworkInterface_USCOREgetIdResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetId:
		((_vbox__IHostNetworkInterface_USCOREgetId *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetNameResponse:
		((_vbox__IHostNetworkInterface_USCOREgetNameResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetName:
		((_vbox__IHostNetworkInterface_USCOREgetName *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostFloppyDrive_USCOREgetUdiResponse:
		((_vbox__IHostFloppyDrive_USCOREgetUdiResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostFloppyDrive_USCOREgetUdi:
		((_vbox__IHostFloppyDrive_USCOREgetUdi *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostFloppyDrive_USCOREgetDescriptionResponse:
		((_vbox__IHostFloppyDrive_USCOREgetDescriptionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostFloppyDrive_USCOREgetDescription:
		((_vbox__IHostFloppyDrive_USCOREgetDescription *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostFloppyDrive_USCOREgetNameResponse:
		((_vbox__IHostFloppyDrive_USCOREgetNameResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostFloppyDrive_USCOREgetName:
		((_vbox__IHostFloppyDrive_USCOREgetName *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostDVDDrive_USCOREgetUdiResponse:
		((_vbox__IHostDVDDrive_USCOREgetUdiResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostDVDDrive_USCOREgetUdi:
		((_vbox__IHostDVDDrive_USCOREgetUdi *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostDVDDrive_USCOREgetDescriptionResponse:
		((_vbox__IHostDVDDrive_USCOREgetDescriptionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostDVDDrive_USCOREgetDescription:
		((_vbox__IHostDVDDrive_USCOREgetDescription *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostDVDDrive_USCOREgetNameResponse:
		((_vbox__IHostDVDDrive_USCOREgetNameResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IHostDVDDrive_USCOREgetName:
		((_vbox__IHostDVDDrive_USCOREgetName *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREdiscardCurrentSnapshotAndStateResponse:
		((_vbox__IConsole_USCOREdiscardCurrentSnapshotAndStateResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREdiscardCurrentSnapshotAndState:
		((_vbox__IConsole_USCOREdiscardCurrentSnapshotAndState *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREdiscardCurrentStateResponse:
		((_vbox__IConsole_USCOREdiscardCurrentStateResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREdiscardCurrentState:
		((_vbox__IConsole_USCOREdiscardCurrentState *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREdiscardSnapshotResponse:
		((_vbox__IConsole_USCOREdiscardSnapshotResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREdiscardSnapshot:
		((_vbox__IConsole_USCOREdiscardSnapshot *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREtakeSnapshotResponse:
		((_vbox__IConsole_USCOREtakeSnapshotResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREtakeSnapshot:
		((_vbox__IConsole_USCOREtakeSnapshot *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREremoveSharedFolderResponse:
		((_vbox__IConsole_USCOREremoveSharedFolderResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREremoveSharedFolder:
		((_vbox__IConsole_USCOREremoveSharedFolder *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREcreateSharedFolderResponse:
		((_vbox__IConsole_USCOREcreateSharedFolderResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREcreateSharedFolder:
		((_vbox__IConsole_USCOREcreateSharedFolder *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREfindUSBDeviceByIdResponse:
		((_vbox__IConsole_USCOREfindUSBDeviceByIdResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREfindUSBDeviceById:
		((_vbox__IConsole_USCOREfindUSBDeviceById *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREfindUSBDeviceByAddressResponse:
		((_vbox__IConsole_USCOREfindUSBDeviceByAddressResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREfindUSBDeviceByAddress:
		((_vbox__IConsole_USCOREfindUSBDeviceByAddress *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREdetachUSBDeviceResponse:
		((_vbox__IConsole_USCOREdetachUSBDeviceResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREdetachUSBDevice:
		((_vbox__IConsole_USCOREdetachUSBDevice *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREattachUSBDeviceResponse:
		((_vbox__IConsole_USCOREattachUSBDeviceResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREattachUSBDevice:
		((_vbox__IConsole_USCOREattachUSBDevice *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetDeviceActivityResponse:
		((_vbox__IConsole_USCOREgetDeviceActivityResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetDeviceActivity:
		((_vbox__IConsole_USCOREgetDeviceActivity *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREforgetSavedStateResponse:
		((_vbox__IConsole_USCOREforgetSavedStateResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREforgetSavedState:
		((_vbox__IConsole_USCOREforgetSavedState *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREadoptSavedStateResponse:
		((_vbox__IConsole_USCOREadoptSavedStateResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREadoptSavedState:
		((_vbox__IConsole_USCOREadoptSavedState *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREsaveStateResponse:
		((_vbox__IConsole_USCOREsaveStateResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREsaveState:
		((_vbox__IConsole_USCOREsaveState *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetGuestEnteredACPIModeResponse:
		((_vbox__IConsole_USCOREgetGuestEnteredACPIModeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetGuestEnteredACPIMode:
		((_vbox__IConsole_USCOREgetGuestEnteredACPIMode *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetPowerButtonHandledResponse:
		((_vbox__IConsole_USCOREgetPowerButtonHandledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetPowerButtonHandled:
		((_vbox__IConsole_USCOREgetPowerButtonHandled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREsleepButtonResponse:
		((_vbox__IConsole_USCOREsleepButtonResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREsleepButton:
		((_vbox__IConsole_USCOREsleepButton *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREpowerButtonResponse:
		((_vbox__IConsole_USCOREpowerButtonResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREpowerButton:
		((_vbox__IConsole_USCOREpowerButton *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREresumeResponse:
		((_vbox__IConsole_USCOREresumeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREresume:
		((_vbox__IConsole_USCOREresume *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREpauseResponse:
		((_vbox__IConsole_USCOREpauseResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREpause:
		((_vbox__IConsole_USCOREpause *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREresetResponse:
		((_vbox__IConsole_USCOREresetResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREreset:
		((_vbox__IConsole_USCOREreset *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREpowerDownResponse:
		((_vbox__IConsole_USCOREpowerDownResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREpowerDown:
		((_vbox__IConsole_USCOREpowerDown *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREpowerUpPausedResponse:
		((_vbox__IConsole_USCOREpowerUpPausedResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREpowerUpPaused:
		((_vbox__IConsole_USCOREpowerUpPaused *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREpowerUpResponse:
		((_vbox__IConsole_USCOREpowerUpResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREpowerUp:
		((_vbox__IConsole_USCOREpowerUp *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetRemoteDisplayInfoResponse:
		((_vbox__IConsole_USCOREgetRemoteDisplayInfoResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetRemoteDisplayInfo:
		((_vbox__IConsole_USCOREgetRemoteDisplayInfo *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetSharedFoldersResponse:
		((_vbox__IConsole_USCOREgetSharedFoldersResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetSharedFolders:
		((_vbox__IConsole_USCOREgetSharedFolders *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetRemoteUSBDevicesResponse:
		((_vbox__IConsole_USCOREgetRemoteUSBDevicesResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetRemoteUSBDevices:
		((_vbox__IConsole_USCOREgetRemoteUSBDevices *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetUSBDevicesResponse:
		((_vbox__IConsole_USCOREgetUSBDevicesResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetUSBDevices:
		((_vbox__IConsole_USCOREgetUSBDevices *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetDisplayResponse:
		((_vbox__IConsole_USCOREgetDisplayResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetDisplay:
		((_vbox__IConsole_USCOREgetDisplay *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetMouseResponse:
		((_vbox__IConsole_USCOREgetMouseResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetMouse:
		((_vbox__IConsole_USCOREgetMouse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetKeyboardResponse:
		((_vbox__IConsole_USCOREgetKeyboardResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetKeyboard:
		((_vbox__IConsole_USCOREgetKeyboard *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetGuestResponse:
		((_vbox__IConsole_USCOREgetGuestResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetGuest:
		((_vbox__IConsole_USCOREgetGuest *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetStateResponse:
		((_vbox__IConsole_USCOREgetStateResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetState:
		((_vbox__IConsole_USCOREgetState *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetMachineResponse:
		((_vbox__IConsole_USCOREgetMachineResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IConsole_USCOREgetMachine:
		((_vbox__IConsole_USCOREgetMachine *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREenumerateGuestPropertiesResponse:
		((_vbox__IMachine_USCOREenumerateGuestPropertiesResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREenumerateGuestProperties:
		((_vbox__IMachine_USCOREenumerateGuestProperties *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetGuestPropertyValueResponse:
		((_vbox__IMachine_USCOREsetGuestPropertyValueResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetGuestPropertyValue:
		((_vbox__IMachine_USCOREsetGuestPropertyValue *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetGuestPropertyResponse:
		((_vbox__IMachine_USCOREsetGuestPropertyResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetGuestProperty:
		((_vbox__IMachine_USCOREsetGuestProperty *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetGuestPropertyTimestampResponse:
		((_vbox__IMachine_USCOREgetGuestPropertyTimestampResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetGuestPropertyTimestamp:
		((_vbox__IMachine_USCOREgetGuestPropertyTimestamp *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetGuestPropertyValueResponse:
		((_vbox__IMachine_USCOREgetGuestPropertyValueResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetGuestPropertyValue:
		((_vbox__IMachine_USCOREgetGuestPropertyValue *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetGuestPropertyResponse:
		((_vbox__IMachine_USCOREgetGuestPropertyResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetGuestProperty:
		((_vbox__IMachine_USCOREgetGuestProperty *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREshowConsoleWindowResponse:
		((_vbox__IMachine_USCOREshowConsoleWindowResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREshowConsoleWindow:
		((_vbox__IMachine_USCOREshowConsoleWindow *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREcanShowConsoleWindowResponse:
		((_vbox__IMachine_USCOREcanShowConsoleWindowResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREcanShowConsoleWindow:
		((_vbox__IMachine_USCOREcanShowConsoleWindow *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREremoveSharedFolderResponse:
		((_vbox__IMachine_USCOREremoveSharedFolderResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREremoveSharedFolder:
		((_vbox__IMachine_USCOREremoveSharedFolder *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREcreateSharedFolderResponse:
		((_vbox__IMachine_USCOREcreateSharedFolderResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREcreateSharedFolder:
		((_vbox__IMachine_USCOREcreateSharedFolder *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetCurrentSnapshotResponse:
		((_vbox__IMachine_USCOREsetCurrentSnapshotResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetCurrentSnapshot:
		((_vbox__IMachine_USCOREsetCurrentSnapshot *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREfindSnapshotResponse:
		((_vbox__IMachine_USCOREfindSnapshotResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREfindSnapshot:
		((_vbox__IMachine_USCOREfindSnapshot *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetSnapshotResponse:
		((_vbox__IMachine_USCOREgetSnapshotResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetSnapshot:
		((_vbox__IMachine_USCOREgetSnapshot *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREexportResponse:
		((_vbox__IMachine_USCOREexportResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREexport:
		((_vbox__IMachine_USCOREexport *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREdeleteSettingsResponse:
		((_vbox__IMachine_USCOREdeleteSettingsResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREdeleteSettings:
		((_vbox__IMachine_USCOREdeleteSettings *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREdiscardSettingsResponse:
		((_vbox__IMachine_USCOREdiscardSettingsResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREdiscardSettings:
		((_vbox__IMachine_USCOREdiscardSettings *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsaveSettingsWithBackupResponse:
		((_vbox__IMachine_USCOREsaveSettingsWithBackupResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsaveSettingsWithBackup:
		((_vbox__IMachine_USCOREsaveSettingsWithBackup *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsaveSettingsResponse:
		((_vbox__IMachine_USCOREsaveSettingsResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsaveSettings:
		((_vbox__IMachine_USCOREsaveSettings *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetExtraDataResponse:
		((_vbox__IMachine_USCOREsetExtraDataResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetExtraData:
		((_vbox__IMachine_USCOREsetExtraData *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetExtraDataResponse:
		((_vbox__IMachine_USCOREgetExtraDataResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetExtraData:
		((_vbox__IMachine_USCOREgetExtraData *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetNextExtraDataKeyResponse:
		((_vbox__IMachine_USCOREgetNextExtraDataKeyResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetNextExtraDataKey:
		((_vbox__IMachine_USCOREgetNextExtraDataKey *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetParallelPortResponse:
		((_vbox__IMachine_USCOREgetParallelPortResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetParallelPort:
		((_vbox__IMachine_USCOREgetParallelPort *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetSerialPortResponse:
		((_vbox__IMachine_USCOREgetSerialPortResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetSerialPort:
		((_vbox__IMachine_USCOREgetSerialPort *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREremoveStorageControllerResponse:
		((_vbox__IMachine_USCOREremoveStorageControllerResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREremoveStorageController:
		((_vbox__IMachine_USCOREremoveStorageController *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetStorageControllerByNameResponse:
		((_vbox__IMachine_USCOREgetStorageControllerByNameResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetStorageControllerByName:
		((_vbox__IMachine_USCOREgetStorageControllerByName *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREaddStorageControllerResponse:
		((_vbox__IMachine_USCOREaddStorageControllerResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREaddStorageController:
		((_vbox__IMachine_USCOREaddStorageController *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetNetworkAdapterResponse:
		((_vbox__IMachine_USCOREgetNetworkAdapterResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetNetworkAdapter:
		((_vbox__IMachine_USCOREgetNetworkAdapter *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetHardDiskAttachmentsOfControllerResponse:
		((_vbox__IMachine_USCOREgetHardDiskAttachmentsOfControllerResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetHardDiskAttachmentsOfController:
		((_vbox__IMachine_USCOREgetHardDiskAttachmentsOfController *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREdetachHardDiskResponse:
		((_vbox__IMachine_USCOREdetachHardDiskResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREdetachHardDisk:
		((_vbox__IMachine_USCOREdetachHardDisk *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetHardDiskResponse:
		((_vbox__IMachine_USCOREgetHardDiskResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetHardDisk:
		((_vbox__IMachine_USCOREgetHardDisk *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREattachHardDiskResponse:
		((_vbox__IMachine_USCOREattachHardDiskResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREattachHardDisk:
		((_vbox__IMachine_USCOREattachHardDisk *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetBootOrderResponse:
		((_vbox__IMachine_USCOREgetBootOrderResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetBootOrder:
		((_vbox__IMachine_USCOREgetBootOrder *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetBootOrderResponse:
		((_vbox__IMachine_USCOREsetBootOrderResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetBootOrder:
		((_vbox__IMachine_USCOREsetBootOrder *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetGuestPropertyNotificationPatternsResponse:
		((_vbox__IMachine_USCOREsetGuestPropertyNotificationPatternsResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetGuestPropertyNotificationPatterns:
		((_vbox__IMachine_USCOREsetGuestPropertyNotificationPatterns *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetGuestPropertyNotificationPatternsResponse:
		((_vbox__IMachine_USCOREgetGuestPropertyNotificationPatternsResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetGuestPropertyNotificationPatterns:
		((_vbox__IMachine_USCOREgetGuestPropertyNotificationPatterns *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetClipboardModeResponse:
		((_vbox__IMachine_USCOREsetClipboardModeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetClipboardMode:
		((_vbox__IMachine_USCOREsetClipboardMode *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetClipboardModeResponse:
		((_vbox__IMachine_USCOREgetClipboardModeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetClipboardMode:
		((_vbox__IMachine_USCOREgetClipboardMode *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetSharedFoldersResponse:
		((_vbox__IMachine_USCOREgetSharedFoldersResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetSharedFolders:
		((_vbox__IMachine_USCOREgetSharedFolders *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetCurrentStateModifiedResponse:
		((_vbox__IMachine_USCOREgetCurrentStateModifiedResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetCurrentStateModified:
		((_vbox__IMachine_USCOREgetCurrentStateModified *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetSnapshotCountResponse:
		((_vbox__IMachine_USCOREgetSnapshotCountResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetSnapshotCount:
		((_vbox__IMachine_USCOREgetSnapshotCount *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetCurrentSnapshotResponse:
		((_vbox__IMachine_USCOREgetCurrentSnapshotResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetCurrentSnapshot:
		((_vbox__IMachine_USCOREgetCurrentSnapshot *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetLogFolderResponse:
		((_vbox__IMachine_USCOREgetLogFolderResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetLogFolder:
		((_vbox__IMachine_USCOREgetLogFolder *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetStateFilePathResponse:
		((_vbox__IMachine_USCOREgetStateFilePathResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetStateFilePath:
		((_vbox__IMachine_USCOREgetStateFilePath *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetLastStateChangeResponse:
		((_vbox__IMachine_USCOREgetLastStateChangeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetLastStateChange:
		((_vbox__IMachine_USCOREgetLastStateChange *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetStateResponse:
		((_vbox__IMachine_USCOREgetStateResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetState:
		((_vbox__IMachine_USCOREgetState *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetSessionPidResponse:
		((_vbox__IMachine_USCOREgetSessionPidResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetSessionPid:
		((_vbox__IMachine_USCOREgetSessionPid *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetSessionTypeResponse:
		((_vbox__IMachine_USCOREgetSessionTypeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetSessionType:
		((_vbox__IMachine_USCOREgetSessionType *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetSessionStateResponse:
		((_vbox__IMachine_USCOREgetSessionStateResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetSessionState:
		((_vbox__IMachine_USCOREgetSessionState *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetSettingsModifiedResponse:
		((_vbox__IMachine_USCOREgetSettingsModifiedResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetSettingsModified:
		((_vbox__IMachine_USCOREgetSettingsModified *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetSettingsFileVersionResponse:
		((_vbox__IMachine_USCOREgetSettingsFileVersionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetSettingsFileVersion:
		((_vbox__IMachine_USCOREgetSettingsFileVersion *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetSettingsFilePathResponse:
		((_vbox__IMachine_USCOREgetSettingsFilePathResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetSettingsFilePath:
		((_vbox__IMachine_USCOREgetSettingsFilePath *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetStorageControllersResponse:
		((_vbox__IMachine_USCOREgetStorageControllersResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetStorageControllers:
		((_vbox__IMachine_USCOREgetStorageControllers *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetAudioAdapterResponse:
		((_vbox__IMachine_USCOREgetAudioAdapterResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetAudioAdapter:
		((_vbox__IMachine_USCOREgetAudioAdapter *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetUSBControllerResponse:
		((_vbox__IMachine_USCOREgetUSBControllerResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetUSBController:
		((_vbox__IMachine_USCOREgetUSBController *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetFloppyDriveResponse:
		((_vbox__IMachine_USCOREgetFloppyDriveResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetFloppyDrive:
		((_vbox__IMachine_USCOREgetFloppyDrive *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetDVDDriveResponse:
		((_vbox__IMachine_USCOREgetDVDDriveResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetDVDDrive:
		((_vbox__IMachine_USCOREgetDVDDrive *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetHardDiskAttachmentsResponse:
		((_vbox__IMachine_USCOREgetHardDiskAttachmentsResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetHardDiskAttachments:
		((_vbox__IMachine_USCOREgetHardDiskAttachments *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetVRDPServerResponse:
		((_vbox__IMachine_USCOREgetVRDPServerResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetVRDPServer:
		((_vbox__IMachine_USCOREgetVRDPServer *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetSnapshotFolderResponse:
		((_vbox__IMachine_USCOREsetSnapshotFolderResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetSnapshotFolder:
		((_vbox__IMachine_USCOREsetSnapshotFolder *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetSnapshotFolderResponse:
		((_vbox__IMachine_USCOREgetSnapshotFolderResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetSnapshotFolder:
		((_vbox__IMachine_USCOREgetSnapshotFolder *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetPAEEnabledResponse:
		((_vbox__IMachine_USCOREsetPAEEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetPAEEnabled:
		((_vbox__IMachine_USCOREsetPAEEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetPAEEnabledResponse:
		((_vbox__IMachine_USCOREgetPAEEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetPAEEnabled:
		((_vbox__IMachine_USCOREgetPAEEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetHWVirtExVPIDEnabledResponse:
		((_vbox__IMachine_USCOREsetHWVirtExVPIDEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetHWVirtExVPIDEnabled:
		((_vbox__IMachine_USCOREsetHWVirtExVPIDEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetHWVirtExVPIDEnabledResponse:
		((_vbox__IMachine_USCOREgetHWVirtExVPIDEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetHWVirtExVPIDEnabled:
		((_vbox__IMachine_USCOREgetHWVirtExVPIDEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabledResponse:
		((_vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabled:
		((_vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabledResponse:
		((_vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabled:
		((_vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetHWVirtExEnabledResponse:
		((_vbox__IMachine_USCOREsetHWVirtExEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetHWVirtExEnabled:
		((_vbox__IMachine_USCOREsetHWVirtExEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetHWVirtExEnabledResponse:
		((_vbox__IMachine_USCOREgetHWVirtExEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetHWVirtExEnabled:
		((_vbox__IMachine_USCOREgetHWVirtExEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetBIOSSettingsResponse:
		((_vbox__IMachine_USCOREgetBIOSSettingsResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetBIOSSettings:
		((_vbox__IMachine_USCOREgetBIOSSettings *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetMonitorCountResponse:
		((_vbox__IMachine_USCOREsetMonitorCountResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetMonitorCount:
		((_vbox__IMachine_USCOREsetMonitorCount *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetMonitorCountResponse:
		((_vbox__IMachine_USCOREgetMonitorCountResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetMonitorCount:
		((_vbox__IMachine_USCOREgetMonitorCount *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetAccelerate3DEnabledResponse:
		((_vbox__IMachine_USCOREsetAccelerate3DEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetAccelerate3DEnabled:
		((_vbox__IMachine_USCOREsetAccelerate3DEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetAccelerate3DEnabledResponse:
		((_vbox__IMachine_USCOREgetAccelerate3DEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetAccelerate3DEnabled:
		((_vbox__IMachine_USCOREgetAccelerate3DEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetVRAMSizeResponse:
		((_vbox__IMachine_USCOREsetVRAMSizeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetVRAMSize:
		((_vbox__IMachine_USCOREsetVRAMSize *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetVRAMSizeResponse:
		((_vbox__IMachine_USCOREgetVRAMSizeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetVRAMSize:
		((_vbox__IMachine_USCOREgetVRAMSize *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetStatisticsUpdateIntervalResponse:
		((_vbox__IMachine_USCOREsetStatisticsUpdateIntervalResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetStatisticsUpdateInterval:
		((_vbox__IMachine_USCOREsetStatisticsUpdateInterval *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetStatisticsUpdateIntervalResponse:
		((_vbox__IMachine_USCOREgetStatisticsUpdateIntervalResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetStatisticsUpdateInterval:
		((_vbox__IMachine_USCOREgetStatisticsUpdateInterval *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetMemoryBalloonSizeResponse:
		((_vbox__IMachine_USCOREsetMemoryBalloonSizeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetMemoryBalloonSize:
		((_vbox__IMachine_USCOREsetMemoryBalloonSize *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetMemoryBalloonSizeResponse:
		((_vbox__IMachine_USCOREgetMemoryBalloonSizeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetMemoryBalloonSize:
		((_vbox__IMachine_USCOREgetMemoryBalloonSize *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetMemorySizeResponse:
		((_vbox__IMachine_USCOREsetMemorySizeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetMemorySize:
		((_vbox__IMachine_USCOREsetMemorySize *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetMemorySizeResponse:
		((_vbox__IMachine_USCOREgetMemorySizeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetMemorySize:
		((_vbox__IMachine_USCOREgetMemorySize *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetCPUCountResponse:
		((_vbox__IMachine_USCOREsetCPUCountResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetCPUCount:
		((_vbox__IMachine_USCOREsetCPUCount *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetCPUCountResponse:
		((_vbox__IMachine_USCOREgetCPUCountResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetCPUCount:
		((_vbox__IMachine_USCOREgetCPUCount *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetHardwareVersionResponse:
		((_vbox__IMachine_USCOREsetHardwareVersionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetHardwareVersion:
		((_vbox__IMachine_USCOREsetHardwareVersion *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetHardwareVersionResponse:
		((_vbox__IMachine_USCOREgetHardwareVersionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetHardwareVersion:
		((_vbox__IMachine_USCOREgetHardwareVersion *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetOSTypeIdResponse:
		((_vbox__IMachine_USCOREsetOSTypeIdResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetOSTypeId:
		((_vbox__IMachine_USCOREsetOSTypeId *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetOSTypeIdResponse:
		((_vbox__IMachine_USCOREgetOSTypeIdResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetOSTypeId:
		((_vbox__IMachine_USCOREgetOSTypeId *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetIdResponse:
		((_vbox__IMachine_USCOREgetIdResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetId:
		((_vbox__IMachine_USCOREgetId *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetDescriptionResponse:
		((_vbox__IMachine_USCOREsetDescriptionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetDescription:
		((_vbox__IMachine_USCOREsetDescription *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetDescriptionResponse:
		((_vbox__IMachine_USCOREgetDescriptionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetDescription:
		((_vbox__IMachine_USCOREgetDescription *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetNameResponse:
		((_vbox__IMachine_USCOREsetNameResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREsetName:
		((_vbox__IMachine_USCOREsetName *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetNameResponse:
		((_vbox__IMachine_USCOREgetNameResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetName:
		((_vbox__IMachine_USCOREgetName *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetAccessErrorResponse:
		((_vbox__IMachine_USCOREgetAccessErrorResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetAccessError:
		((_vbox__IMachine_USCOREgetAccessError *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetAccessibleResponse:
		((_vbox__IMachine_USCOREgetAccessibleResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetAccessible:
		((_vbox__IMachine_USCOREgetAccessible *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetParentResponse:
		((_vbox__IMachine_USCOREgetParentResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IMachine_USCOREgetParent:
		((_vbox__IMachine_USCOREgetParent *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetPXEDebugEnabledResponse:
		((_vbox__IBIOSSettings_USCOREsetPXEDebugEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetPXEDebugEnabled:
		((_vbox__IBIOSSettings_USCOREsetPXEDebugEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetPXEDebugEnabledResponse:
		((_vbox__IBIOSSettings_USCOREgetPXEDebugEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetPXEDebugEnabled:
		((_vbox__IBIOSSettings_USCOREgetPXEDebugEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetTimeOffsetResponse:
		((_vbox__IBIOSSettings_USCOREsetTimeOffsetResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetTimeOffset:
		((_vbox__IBIOSSettings_USCOREsetTimeOffset *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetTimeOffsetResponse:
		((_vbox__IBIOSSettings_USCOREgetTimeOffsetResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetTimeOffset:
		((_vbox__IBIOSSettings_USCOREgetTimeOffset *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetIOAPICEnabledResponse:
		((_vbox__IBIOSSettings_USCOREsetIOAPICEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetIOAPICEnabled:
		((_vbox__IBIOSSettings_USCOREsetIOAPICEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetIOAPICEnabledResponse:
		((_vbox__IBIOSSettings_USCOREgetIOAPICEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetIOAPICEnabled:
		((_vbox__IBIOSSettings_USCOREgetIOAPICEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetACPIEnabledResponse:
		((_vbox__IBIOSSettings_USCOREsetACPIEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetACPIEnabled:
		((_vbox__IBIOSSettings_USCOREsetACPIEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetACPIEnabledResponse:
		((_vbox__IBIOSSettings_USCOREgetACPIEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetACPIEnabled:
		((_vbox__IBIOSSettings_USCOREgetACPIEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetBootMenuModeResponse:
		((_vbox__IBIOSSettings_USCOREsetBootMenuModeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetBootMenuMode:
		((_vbox__IBIOSSettings_USCOREsetBootMenuMode *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetBootMenuModeResponse:
		((_vbox__IBIOSSettings_USCOREgetBootMenuModeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetBootMenuMode:
		((_vbox__IBIOSSettings_USCOREgetBootMenuMode *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetLogoImagePathResponse:
		((_vbox__IBIOSSettings_USCOREsetLogoImagePathResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetLogoImagePath:
		((_vbox__IBIOSSettings_USCOREsetLogoImagePath *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetLogoImagePathResponse:
		((_vbox__IBIOSSettings_USCOREgetLogoImagePathResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetLogoImagePath:
		((_vbox__IBIOSSettings_USCOREgetLogoImagePath *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetLogoDisplayTimeResponse:
		((_vbox__IBIOSSettings_USCOREsetLogoDisplayTimeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetLogoDisplayTime:
		((_vbox__IBIOSSettings_USCOREsetLogoDisplayTime *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetLogoDisplayTimeResponse:
		((_vbox__IBIOSSettings_USCOREgetLogoDisplayTimeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetLogoDisplayTime:
		((_vbox__IBIOSSettings_USCOREgetLogoDisplayTime *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetLogoFadeOutResponse:
		((_vbox__IBIOSSettings_USCOREsetLogoFadeOutResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetLogoFadeOut:
		((_vbox__IBIOSSettings_USCOREsetLogoFadeOut *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetLogoFadeOutResponse:
		((_vbox__IBIOSSettings_USCOREgetLogoFadeOutResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetLogoFadeOut:
		((_vbox__IBIOSSettings_USCOREgetLogoFadeOut *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetLogoFadeInResponse:
		((_vbox__IBIOSSettings_USCOREsetLogoFadeInResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetLogoFadeIn:
		((_vbox__IBIOSSettings_USCOREsetLogoFadeIn *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetLogoFadeInResponse:
		((_vbox__IBIOSSettings_USCOREgetLogoFadeInResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetLogoFadeIn:
		((_vbox__IBIOSSettings_USCOREgetLogoFadeIn *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREaddDescriptionResponse:
		((_vbox__IVirtualSystemDescription_USCOREaddDescriptionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREaddDescription:
		((_vbox__IVirtualSystemDescription_USCOREaddDescription *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREsetFinalValuesResponse:
		((_vbox__IVirtualSystemDescription_USCOREsetFinalValuesResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREsetFinalValues:
		((_vbox__IVirtualSystemDescription_USCOREsetFinalValues *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREgetValuesByTypeResponse:
		((_vbox__IVirtualSystemDescription_USCOREgetValuesByTypeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREgetValuesByType:
		((_vbox__IVirtualSystemDescription_USCOREgetValuesByType *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREgetDescriptionByTypeResponse:
		((_vbox__IVirtualSystemDescription_USCOREgetDescriptionByTypeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREgetDescriptionByType:
		((_vbox__IVirtualSystemDescription_USCOREgetDescriptionByType *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREgetDescriptionResponse:
		((_vbox__IVirtualSystemDescription_USCOREgetDescriptionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREgetDescription:
		((_vbox__IVirtualSystemDescription_USCOREgetDescription *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREgetCountResponse:
		((_vbox__IVirtualSystemDescription_USCOREgetCountResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREgetCount:
		((_vbox__IVirtualSystemDescription_USCOREgetCount *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAppliance_USCOREgetWarningsResponse:
		((_vbox__IAppliance_USCOREgetWarningsResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAppliance_USCOREgetWarnings:
		((_vbox__IAppliance_USCOREgetWarnings *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAppliance_USCOREwriteResponse:
		((_vbox__IAppliance_USCOREwriteResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAppliance_USCOREwrite:
		((_vbox__IAppliance_USCOREwrite *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAppliance_USCOREcreateVFSExplorerResponse:
		((_vbox__IAppliance_USCOREcreateVFSExplorerResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAppliance_USCOREcreateVFSExplorer:
		((_vbox__IAppliance_USCOREcreateVFSExplorer *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAppliance_USCOREimportMachinesResponse:
		((_vbox__IAppliance_USCOREimportMachinesResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAppliance_USCOREimportMachines:
		((_vbox__IAppliance_USCOREimportMachines *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAppliance_USCOREinterpretResponse:
		((_vbox__IAppliance_USCOREinterpretResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAppliance_USCOREinterpret:
		((_vbox__IAppliance_USCOREinterpret *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAppliance_USCOREreadResponse:
		((_vbox__IAppliance_USCOREreadResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAppliance_USCOREread:
		((_vbox__IAppliance_USCOREread *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAppliance_USCOREgetVirtualSystemDescriptionsResponse:
		((_vbox__IAppliance_USCOREgetVirtualSystemDescriptionsResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAppliance_USCOREgetVirtualSystemDescriptions:
		((_vbox__IAppliance_USCOREgetVirtualSystemDescriptions *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAppliance_USCOREgetDisksResponse:
		((_vbox__IAppliance_USCOREgetDisksResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAppliance_USCOREgetDisks:
		((_vbox__IAppliance_USCOREgetDisks *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAppliance_USCOREgetPathResponse:
		((_vbox__IAppliance_USCOREgetPathResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IAppliance_USCOREgetPath:
		((_vbox__IAppliance_USCOREgetPath *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREremoveResponse:
		((_vbox__IVFSExplorer_USCOREremoveResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREremove:
		((_vbox__IVFSExplorer_USCOREremove *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREexistsResponse:
		((_vbox__IVFSExplorer_USCOREexistsResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREexists:
		((_vbox__IVFSExplorer_USCOREexists *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREentryListResponse:
		((_vbox__IVFSExplorer_USCOREentryListResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREentryList:
		((_vbox__IVFSExplorer_USCOREentryList *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREcdUpResponse:
		((_vbox__IVFSExplorer_USCOREcdUpResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREcdUp:
		((_vbox__IVFSExplorer_USCOREcdUp *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREcdResponse:
		((_vbox__IVFSExplorer_USCOREcdResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREcd:
		((_vbox__IVFSExplorer_USCOREcd *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREupdateResponse:
		((_vbox__IVFSExplorer_USCOREupdateResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREupdate:
		((_vbox__IVFSExplorer_USCOREupdate *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREgetTypeResponse:
		((_vbox__IVFSExplorer_USCOREgetTypeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREgetType:
		((_vbox__IVFSExplorer_USCOREgetType *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREgetPathResponse:
		((_vbox__IVFSExplorer_USCOREgetPathResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREgetPath:
		((_vbox__IVFSExplorer_USCOREgetPath *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREremoveDHCPServerResponse:
		((_vbox__IVirtualBox_USCOREremoveDHCPServerResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREremoveDHCPServer:
		((_vbox__IVirtualBox_USCOREremoveDHCPServer *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindDHCPServerByNetworkNameResponse:
		((_vbox__IVirtualBox_USCOREfindDHCPServerByNetworkNameResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindDHCPServerByNetworkName:
		((_vbox__IVirtualBox_USCOREfindDHCPServerByNetworkName *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateDHCPServerResponse:
		((_vbox__IVirtualBox_USCOREcreateDHCPServerResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateDHCPServer:
		((_vbox__IVirtualBox_USCOREcreateDHCPServer *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREsaveSettingsWithBackupResponse:
		((_vbox__IVirtualBox_USCOREsaveSettingsWithBackupResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREsaveSettingsWithBackup:
		((_vbox__IVirtualBox_USCOREsaveSettingsWithBackup *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREsaveSettingsResponse:
		((_vbox__IVirtualBox_USCOREsaveSettingsResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREsaveSettings:
		((_vbox__IVirtualBox_USCOREsaveSettings *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREwaitForPropertyChangeResponse:
		((_vbox__IVirtualBox_USCOREwaitForPropertyChangeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREwaitForPropertyChange:
		((_vbox__IVirtualBox_USCOREwaitForPropertyChange *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenExistingSessionResponse:
		((_vbox__IVirtualBox_USCOREopenExistingSessionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenExistingSession:
		((_vbox__IVirtualBox_USCOREopenExistingSession *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenRemoteSessionResponse:
		((_vbox__IVirtualBox_USCOREopenRemoteSessionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenRemoteSession:
		((_vbox__IVirtualBox_USCOREopenRemoteSession *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenSessionResponse:
		((_vbox__IVirtualBox_USCOREopenSessionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenSession:
		((_vbox__IVirtualBox_USCOREopenSession *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREsetExtraDataResponse:
		((_vbox__IVirtualBox_USCOREsetExtraDataResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREsetExtraData:
		((_vbox__IVirtualBox_USCOREsetExtraData *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetExtraDataResponse:
		((_vbox__IVirtualBox_USCOREgetExtraDataResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetExtraData:
		((_vbox__IVirtualBox_USCOREgetExtraData *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetNextExtraDataKeyResponse:
		((_vbox__IVirtualBox_USCOREgetNextExtraDataKeyResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetNextExtraDataKey:
		((_vbox__IVirtualBox_USCOREgetNextExtraDataKey *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREremoveSharedFolderResponse:
		((_vbox__IVirtualBox_USCOREremoveSharedFolderResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREremoveSharedFolder:
		((_vbox__IVirtualBox_USCOREremoveSharedFolder *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateSharedFolderResponse:
		((_vbox__IVirtualBox_USCOREcreateSharedFolderResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateSharedFolder:
		((_vbox__IVirtualBox_USCOREcreateSharedFolder *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetGuestOSTypeResponse:
		((_vbox__IVirtualBox_USCOREgetGuestOSTypeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetGuestOSType:
		((_vbox__IVirtualBox_USCOREgetGuestOSType *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindFloppyImageResponse:
		((_vbox__IVirtualBox_USCOREfindFloppyImageResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindFloppyImage:
		((_vbox__IVirtualBox_USCOREfindFloppyImage *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetFloppyImageResponse:
		((_vbox__IVirtualBox_USCOREgetFloppyImageResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetFloppyImage:
		((_vbox__IVirtualBox_USCOREgetFloppyImage *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenFloppyImageResponse:
		((_vbox__IVirtualBox_USCOREopenFloppyImageResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenFloppyImage:
		((_vbox__IVirtualBox_USCOREopenFloppyImage *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindDVDImageResponse:
		((_vbox__IVirtualBox_USCOREfindDVDImageResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindDVDImage:
		((_vbox__IVirtualBox_USCOREfindDVDImage *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetDVDImageResponse:
		((_vbox__IVirtualBox_USCOREgetDVDImageResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetDVDImage:
		((_vbox__IVirtualBox_USCOREgetDVDImage *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenDVDImageResponse:
		((_vbox__IVirtualBox_USCOREopenDVDImageResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenDVDImage:
		((_vbox__IVirtualBox_USCOREopenDVDImage *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindHardDiskResponse:
		((_vbox__IVirtualBox_USCOREfindHardDiskResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindHardDisk:
		((_vbox__IVirtualBox_USCOREfindHardDisk *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetHardDiskResponse:
		((_vbox__IVirtualBox_USCOREgetHardDiskResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetHardDisk:
		((_vbox__IVirtualBox_USCOREgetHardDisk *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenHardDiskResponse:
		((_vbox__IVirtualBox_USCOREopenHardDiskResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenHardDisk:
		((_vbox__IVirtualBox_USCOREopenHardDisk *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateHardDiskResponse:
		((_vbox__IVirtualBox_USCOREcreateHardDiskResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateHardDisk:
		((_vbox__IVirtualBox_USCOREcreateHardDisk *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateApplianceResponse:
		((_vbox__IVirtualBox_USCOREcreateApplianceResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateAppliance:
		((_vbox__IVirtualBox_USCOREcreateAppliance *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREunregisterMachineResponse:
		((_vbox__IVirtualBox_USCOREunregisterMachineResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREunregisterMachine:
		((_vbox__IVirtualBox_USCOREunregisterMachine *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindMachineResponse:
		((_vbox__IVirtualBox_USCOREfindMachineResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindMachine:
		((_vbox__IVirtualBox_USCOREfindMachine *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetMachineResponse:
		((_vbox__IVirtualBox_USCOREgetMachineResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetMachine:
		((_vbox__IVirtualBox_USCOREgetMachine *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREregisterMachineResponse:
		((_vbox__IVirtualBox_USCOREregisterMachineResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREregisterMachine:
		((_vbox__IVirtualBox_USCOREregisterMachine *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenMachineResponse:
		((_vbox__IVirtualBox_USCOREopenMachineResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenMachine:
		((_vbox__IVirtualBox_USCOREopenMachine *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateLegacyMachineResponse:
		((_vbox__IVirtualBox_USCOREcreateLegacyMachineResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateLegacyMachine:
		((_vbox__IVirtualBox_USCOREcreateLegacyMachine *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateMachineResponse:
		((_vbox__IVirtualBox_USCOREcreateMachineResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateMachine:
		((_vbox__IVirtualBox_USCOREcreateMachine *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetDHCPServersResponse:
		((_vbox__IVirtualBox_USCOREgetDHCPServersResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetDHCPServers:
		((_vbox__IVirtualBox_USCOREgetDHCPServers *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetPerformanceCollectorResponse:
		((_vbox__IVirtualBox_USCOREgetPerformanceCollectorResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetPerformanceCollector:
		((_vbox__IVirtualBox_USCOREgetPerformanceCollector *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSharedFoldersResponse:
		((_vbox__IVirtualBox_USCOREgetSharedFoldersResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSharedFolders:
		((_vbox__IVirtualBox_USCOREgetSharedFolders *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetGuestOSTypesResponse:
		((_vbox__IVirtualBox_USCOREgetGuestOSTypesResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetGuestOSTypes:
		((_vbox__IVirtualBox_USCOREgetGuestOSTypes *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetProgressOperationsResponse:
		((_vbox__IVirtualBox_USCOREgetProgressOperationsResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetProgressOperations:
		((_vbox__IVirtualBox_USCOREgetProgressOperations *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetFloppyImagesResponse:
		((_vbox__IVirtualBox_USCOREgetFloppyImagesResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetFloppyImages:
		((_vbox__IVirtualBox_USCOREgetFloppyImages *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetDVDImagesResponse:
		((_vbox__IVirtualBox_USCOREgetDVDImagesResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetDVDImages:
		((_vbox__IVirtualBox_USCOREgetDVDImages *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetHardDisksResponse:
		((_vbox__IVirtualBox_USCOREgetHardDisksResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetHardDisks:
		((_vbox__IVirtualBox_USCOREgetHardDisks *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetMachinesResponse:
		((_vbox__IVirtualBox_USCOREgetMachinesResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetMachines:
		((_vbox__IVirtualBox_USCOREgetMachines *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSystemPropertiesResponse:
		((_vbox__IVirtualBox_USCOREgetSystemPropertiesResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSystemProperties:
		((_vbox__IVirtualBox_USCOREgetSystemProperties *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetHostResponse:
		((_vbox__IVirtualBox_USCOREgetHostResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetHost:
		((_vbox__IVirtualBox_USCOREgetHost *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSettingsFormatVersionResponse:
		((_vbox__IVirtualBox_USCOREgetSettingsFormatVersionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSettingsFormatVersion:
		((_vbox__IVirtualBox_USCOREgetSettingsFormatVersion *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSettingsFileVersionResponse:
		((_vbox__IVirtualBox_USCOREgetSettingsFileVersionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSettingsFileVersion:
		((_vbox__IVirtualBox_USCOREgetSettingsFileVersion *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSettingsFilePathResponse:
		((_vbox__IVirtualBox_USCOREgetSettingsFilePathResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSettingsFilePath:
		((_vbox__IVirtualBox_USCOREgetSettingsFilePath *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetHomeFolderResponse:
		((_vbox__IVirtualBox_USCOREgetHomeFolderResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetHomeFolder:
		((_vbox__IVirtualBox_USCOREgetHomeFolder *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetPackageTypeResponse:
		((_vbox__IVirtualBox_USCOREgetPackageTypeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetPackageType:
		((_vbox__IVirtualBox_USCOREgetPackageType *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetRevisionResponse:
		((_vbox__IVirtualBox_USCOREgetRevisionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetRevision:
		((_vbox__IVirtualBox_USCOREgetRevision *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetVersionResponse:
		((_vbox__IVirtualBox_USCOREgetVersionResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetVersion:
		((_vbox__IVirtualBox_USCOREgetVersion *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDHCPServer_USCOREstopResponse:
		((_vbox__IDHCPServer_USCOREstopResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDHCPServer_USCOREstop:
		((_vbox__IDHCPServer_USCOREstop *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDHCPServer_USCOREstartResponse:
		((_vbox__IDHCPServer_USCOREstartResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDHCPServer_USCOREstart:
		((_vbox__IDHCPServer_USCOREstart *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDHCPServer_USCOREsetConfigurationResponse:
		((_vbox__IDHCPServer_USCOREsetConfigurationResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDHCPServer_USCOREsetConfiguration:
		((_vbox__IDHCPServer_USCOREsetConfiguration *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetUpperIPResponse:
		((_vbox__IDHCPServer_USCOREgetUpperIPResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetUpperIP:
		((_vbox__IDHCPServer_USCOREgetUpperIP *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetLowerIPResponse:
		((_vbox__IDHCPServer_USCOREgetLowerIPResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetLowerIP:
		((_vbox__IDHCPServer_USCOREgetLowerIP *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetNetworkNameResponse:
		((_vbox__IDHCPServer_USCOREgetNetworkNameResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetNetworkName:
		((_vbox__IDHCPServer_USCOREgetNetworkName *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetNetworkMaskResponse:
		((_vbox__IDHCPServer_USCOREgetNetworkMaskResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetNetworkMask:
		((_vbox__IDHCPServer_USCOREgetNetworkMask *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetIPAddressResponse:
		((_vbox__IDHCPServer_USCOREgetIPAddressResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetIPAddress:
		((_vbox__IDHCPServer_USCOREgetIPAddress *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDHCPServer_USCOREsetEnabledResponse:
		((_vbox__IDHCPServer_USCOREsetEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDHCPServer_USCOREsetEnabled:
		((_vbox__IDHCPServer_USCOREsetEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetEnabledResponse:
		((_vbox__IDHCPServer_USCOREgetEnabledResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetEnabled:
		((_vbox__IDHCPServer_USCOREgetEnabled *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetNextResponse:
		((_vbox__IVirtualBoxErrorInfo_USCOREgetNextResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetNext:
		((_vbox__IVirtualBoxErrorInfo_USCOREgetNext *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetTextResponse:
		((_vbox__IVirtualBoxErrorInfo_USCOREgetTextResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetText:
		((_vbox__IVirtualBoxErrorInfo_USCOREgetText *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetComponentResponse:
		((_vbox__IVirtualBoxErrorInfo_USCOREgetComponentResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetComponent:
		((_vbox__IVirtualBoxErrorInfo_USCOREgetComponent *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceIDResponse:
		((_vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceIDResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceID:
		((_vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceID *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetResultCodeResponse:
		((_vbox__IVirtualBoxErrorInfo_USCOREgetResultCodeResponse *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetResultCode:
		((_vbox__IVirtualBoxErrorInfo_USCOREgetResultCode *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE_vbox__ISharedFolder:
		((vbox__ISharedFolder *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE_vbox__IHardDiskAttachment:
		((vbox__IHardDiskAttachment *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE_vbox__IGuestOSType:
		((vbox__IGuestOSType *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE_std__string:
		soap_serialize_std__string(soap, (const std::string *)ptr);
		break;
	case SOAP_TYPE_vbox__IRemoteDisplayInfo:
		((vbox__IRemoteDisplayInfo *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__RuntimeFault:
		((_vbox__RuntimeFault *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE__vbox__InvalidObjectFault:
		((_vbox__InvalidObjectFault *)ptr)->soap_serialize(soap);
		break;
	case SOAP_TYPE___vbox__IPerformanceCollector_USCOREqueryMetricsData:
		soap_serialize___vbox__IPerformanceCollector_USCOREqueryMetricsData(soap, (const struct __vbox__IPerformanceCollector_USCOREqueryMetricsData *)ptr);
		break;
	case SOAP_TYPE___vbox__IPerformanceCollector_USCOREdisableMetrics:
		soap_serialize___vbox__IPerformanceCollector_USCOREdisableMetrics(soap, (const struct __vbox__IPerformanceCollector_USCOREdisableMetrics *)ptr);
		break;
	case SOAP_TYPE___vbox__IPerformanceCollector_USCOREenableMetrics:
		soap_serialize___vbox__IPerformanceCollector_USCOREenableMetrics(soap, (const struct __vbox__IPerformanceCollector_USCOREenableMetrics *)ptr);
		break;
	case SOAP_TYPE___vbox__IPerformanceCollector_USCOREsetupMetrics:
		soap_serialize___vbox__IPerformanceCollector_USCOREsetupMetrics(soap, (const struct __vbox__IPerformanceCollector_USCOREsetupMetrics *)ptr);
		break;
	case SOAP_TYPE___vbox__IPerformanceCollector_USCOREgetMetrics:
		soap_serialize___vbox__IPerformanceCollector_USCOREgetMetrics(soap, (const struct __vbox__IPerformanceCollector_USCOREgetMetrics *)ptr);
		break;
	case SOAP_TYPE___vbox__IPerformanceCollector_USCOREgetMetricNames:
		soap_serialize___vbox__IPerformanceCollector_USCOREgetMetricNames(soap, (const struct __vbox__IPerformanceCollector_USCOREgetMetricNames *)ptr);
		break;
	case SOAP_TYPE___vbox__IPerformanceMetric_USCOREgetMaximumValue:
		soap_serialize___vbox__IPerformanceMetric_USCOREgetMaximumValue(soap, (const struct __vbox__IPerformanceMetric_USCOREgetMaximumValue *)ptr);
		break;
	case SOAP_TYPE___vbox__IPerformanceMetric_USCOREgetMinimumValue:
		soap_serialize___vbox__IPerformanceMetric_USCOREgetMinimumValue(soap, (const struct __vbox__IPerformanceMetric_USCOREgetMinimumValue *)ptr);
		break;
	case SOAP_TYPE___vbox__IPerformanceMetric_USCOREgetUnit:
		soap_serialize___vbox__IPerformanceMetric_USCOREgetUnit(soap, (const struct __vbox__IPerformanceMetric_USCOREgetUnit *)ptr);
		break;
	case SOAP_TYPE___vbox__IPerformanceMetric_USCOREgetCount:
		soap_serialize___vbox__IPerformanceMetric_USCOREgetCount(soap, (const struct __vbox__IPerformanceMetric_USCOREgetCount *)ptr);
		break;
	case SOAP_TYPE___vbox__IPerformanceMetric_USCOREgetPeriod:
		soap_serialize___vbox__IPerformanceMetric_USCOREgetPeriod(soap, (const struct __vbox__IPerformanceMetric_USCOREgetPeriod *)ptr);
		break;
	case SOAP_TYPE___vbox__IPerformanceMetric_USCOREgetDescription:
		soap_serialize___vbox__IPerformanceMetric_USCOREgetDescription(soap, (const struct __vbox__IPerformanceMetric_USCOREgetDescription *)ptr);
		break;
	case SOAP_TYPE___vbox__IPerformanceMetric_USCOREgetObject:
		soap_serialize___vbox__IPerformanceMetric_USCOREgetObject(soap, (const struct __vbox__IPerformanceMetric_USCOREgetObject *)ptr);
		break;
	case SOAP_TYPE___vbox__IPerformanceMetric_USCOREgetMetricName:
		soap_serialize___vbox__IPerformanceMetric_USCOREgetMetricName(soap, (const struct __vbox__IPerformanceMetric_USCOREgetMetricName *)ptr);
		break;
	case SOAP_TYPE___vbox__IWebsessionManager_USCORElogoff:
		soap_serialize___vbox__IWebsessionManager_USCORElogoff(soap, (const struct __vbox__IWebsessionManager_USCORElogoff *)ptr);
		break;
	case SOAP_TYPE___vbox__IWebsessionManager_USCOREgetSessionObject:
		soap_serialize___vbox__IWebsessionManager_USCOREgetSessionObject(soap, (const struct __vbox__IWebsessionManager_USCOREgetSessionObject *)ptr);
		break;
	case SOAP_TYPE___vbox__IWebsessionManager_USCORElogon:
		soap_serialize___vbox__IWebsessionManager_USCORElogon(soap, (const struct __vbox__IWebsessionManager_USCORElogon *)ptr);
		break;
	case SOAP_TYPE___vbox__IManagedObjectRef_USCORErelease:
		soap_serialize___vbox__IManagedObjectRef_USCORErelease(soap, (const struct __vbox__IManagedObjectRef_USCORErelease *)ptr);
		break;
	case SOAP_TYPE___vbox__IManagedObjectRef_USCOREgetInterfaceName:
		soap_serialize___vbox__IManagedObjectRef_USCOREgetInterfaceName(soap, (const struct __vbox__IManagedObjectRef_USCOREgetInterfaceName *)ptr);
		break;
	case SOAP_TYPE___vbox__IStorageController_USCORESetIDEEmulationPort:
		soap_serialize___vbox__IStorageController_USCORESetIDEEmulationPort(soap, (const struct __vbox__IStorageController_USCORESetIDEEmulationPort *)ptr);
		break;
	case SOAP_TYPE___vbox__IStorageController_USCOREGetIDEEmulationPort:
		soap_serialize___vbox__IStorageController_USCOREGetIDEEmulationPort(soap, (const struct __vbox__IStorageController_USCOREGetIDEEmulationPort *)ptr);
		break;
	case SOAP_TYPE___vbox__IStorageController_USCOREsetControllerType:
		soap_serialize___vbox__IStorageController_USCOREsetControllerType(soap, (const struct __vbox__IStorageController_USCOREsetControllerType *)ptr);
		break;
	case SOAP_TYPE___vbox__IStorageController_USCOREgetControllerType:
		soap_serialize___vbox__IStorageController_USCOREgetControllerType(soap, (const struct __vbox__IStorageController_USCOREgetControllerType *)ptr);
		break;
	case SOAP_TYPE___vbox__IStorageController_USCOREgetBus:
		soap_serialize___vbox__IStorageController_USCOREgetBus(soap, (const struct __vbox__IStorageController_USCOREgetBus *)ptr);
		break;
	case SOAP_TYPE___vbox__IStorageController_USCOREsetPortCount:
		soap_serialize___vbox__IStorageController_USCOREsetPortCount(soap, (const struct __vbox__IStorageController_USCOREsetPortCount *)ptr);
		break;
	case SOAP_TYPE___vbox__IStorageController_USCOREgetPortCount:
		soap_serialize___vbox__IStorageController_USCOREgetPortCount(soap, (const struct __vbox__IStorageController_USCOREgetPortCount *)ptr);
		break;
	case SOAP_TYPE___vbox__IStorageController_USCOREsetInstance:
		soap_serialize___vbox__IStorageController_USCOREsetInstance(soap, (const struct __vbox__IStorageController_USCOREsetInstance *)ptr);
		break;
	case SOAP_TYPE___vbox__IStorageController_USCOREgetInstance:
		soap_serialize___vbox__IStorageController_USCOREgetInstance(soap, (const struct __vbox__IStorageController_USCOREgetInstance *)ptr);
		break;
	case SOAP_TYPE___vbox__IStorageController_USCOREgetMaxPortCount:
		soap_serialize___vbox__IStorageController_USCOREgetMaxPortCount(soap, (const struct __vbox__IStorageController_USCOREgetMaxPortCount *)ptr);
		break;
	case SOAP_TYPE___vbox__IStorageController_USCOREgetMinPortCount:
		soap_serialize___vbox__IStorageController_USCOREgetMinPortCount(soap, (const struct __vbox__IStorageController_USCOREgetMinPortCount *)ptr);
		break;
	case SOAP_TYPE___vbox__IStorageController_USCOREgetMaxDevicesPerPortCount:
		soap_serialize___vbox__IStorageController_USCOREgetMaxDevicesPerPortCount(soap, (const struct __vbox__IStorageController_USCOREgetMaxDevicesPerPortCount *)ptr);
		break;
	case SOAP_TYPE___vbox__IStorageController_USCOREgetName:
		soap_serialize___vbox__IStorageController_USCOREgetName(soap, (const struct __vbox__IStorageController_USCOREgetName *)ptr);
		break;
	case SOAP_TYPE___vbox__ISession_USCOREclose:
		soap_serialize___vbox__ISession_USCOREclose(soap, (const struct __vbox__ISession_USCOREclose *)ptr);
		break;
	case SOAP_TYPE___vbox__ISession_USCOREgetConsole:
		soap_serialize___vbox__ISession_USCOREgetConsole(soap, (const struct __vbox__ISession_USCOREgetConsole *)ptr);
		break;
	case SOAP_TYPE___vbox__ISession_USCOREgetMachine:
		soap_serialize___vbox__ISession_USCOREgetMachine(soap, (const struct __vbox__ISession_USCOREgetMachine *)ptr);
		break;
	case SOAP_TYPE___vbox__ISession_USCOREgetType:
		soap_serialize___vbox__ISession_USCOREgetType(soap, (const struct __vbox__ISession_USCOREgetType *)ptr);
		break;
	case SOAP_TYPE___vbox__ISession_USCOREgetState:
		soap_serialize___vbox__ISession_USCOREgetState(soap, (const struct __vbox__ISession_USCOREgetState *)ptr);
		break;
	case SOAP_TYPE___vbox__IVRDPServer_USCOREsetReuseSingleConnection:
		soap_serialize___vbox__IVRDPServer_USCOREsetReuseSingleConnection(soap, (const struct __vbox__IVRDPServer_USCOREsetReuseSingleConnection *)ptr);
		break;
	case SOAP_TYPE___vbox__IVRDPServer_USCOREgetReuseSingleConnection:
		soap_serialize___vbox__IVRDPServer_USCOREgetReuseSingleConnection(soap, (const struct __vbox__IVRDPServer_USCOREgetReuseSingleConnection *)ptr);
		break;
	case SOAP_TYPE___vbox__IVRDPServer_USCOREsetAllowMultiConnection:
		soap_serialize___vbox__IVRDPServer_USCOREsetAllowMultiConnection(soap, (const struct __vbox__IVRDPServer_USCOREsetAllowMultiConnection *)ptr);
		break;
	case SOAP_TYPE___vbox__IVRDPServer_USCOREgetAllowMultiConnection:
		soap_serialize___vbox__IVRDPServer_USCOREgetAllowMultiConnection(soap, (const struct __vbox__IVRDPServer_USCOREgetAllowMultiConnection *)ptr);
		break;
	case SOAP_TYPE___vbox__IVRDPServer_USCOREsetAuthTimeout:
		soap_serialize___vbox__IVRDPServer_USCOREsetAuthTimeout(soap, (const struct __vbox__IVRDPServer_USCOREsetAuthTimeout *)ptr);
		break;
	case SOAP_TYPE___vbox__IVRDPServer_USCOREgetAuthTimeout:
		soap_serialize___vbox__IVRDPServer_USCOREgetAuthTimeout(soap, (const struct __vbox__IVRDPServer_USCOREgetAuthTimeout *)ptr);
		break;
	case SOAP_TYPE___vbox__IVRDPServer_USCOREsetAuthType:
		soap_serialize___vbox__IVRDPServer_USCOREsetAuthType(soap, (const struct __vbox__IVRDPServer_USCOREsetAuthType *)ptr);
		break;
	case SOAP_TYPE___vbox__IVRDPServer_USCOREgetAuthType:
		soap_serialize___vbox__IVRDPServer_USCOREgetAuthType(soap, (const struct __vbox__IVRDPServer_USCOREgetAuthType *)ptr);
		break;
	case SOAP_TYPE___vbox__IVRDPServer_USCOREsetNetAddress:
		soap_serialize___vbox__IVRDPServer_USCOREsetNetAddress(soap, (const struct __vbox__IVRDPServer_USCOREsetNetAddress *)ptr);
		break;
	case SOAP_TYPE___vbox__IVRDPServer_USCOREgetNetAddress:
		soap_serialize___vbox__IVRDPServer_USCOREgetNetAddress(soap, (const struct __vbox__IVRDPServer_USCOREgetNetAddress *)ptr);
		break;
	case SOAP_TYPE___vbox__IVRDPServer_USCOREsetPort:
		soap_serialize___vbox__IVRDPServer_USCOREsetPort(soap, (const struct __vbox__IVRDPServer_USCOREsetPort *)ptr);
		break;
	case SOAP_TYPE___vbox__IVRDPServer_USCOREgetPort:
		soap_serialize___vbox__IVRDPServer_USCOREgetPort(soap, (const struct __vbox__IVRDPServer_USCOREgetPort *)ptr);
		break;
	case SOAP_TYPE___vbox__IVRDPServer_USCOREsetEnabled:
		soap_serialize___vbox__IVRDPServer_USCOREsetEnabled(soap, (const struct __vbox__IVRDPServer_USCOREsetEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IVRDPServer_USCOREgetEnabled:
		soap_serialize___vbox__IVRDPServer_USCOREgetEnabled(soap, (const struct __vbox__IVRDPServer_USCOREgetEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IAudioAdapter_USCOREsetAudioDriver:
		soap_serialize___vbox__IAudioAdapter_USCOREsetAudioDriver(soap, (const struct __vbox__IAudioAdapter_USCOREsetAudioDriver *)ptr);
		break;
	case SOAP_TYPE___vbox__IAudioAdapter_USCOREgetAudioDriver:
		soap_serialize___vbox__IAudioAdapter_USCOREgetAudioDriver(soap, (const struct __vbox__IAudioAdapter_USCOREgetAudioDriver *)ptr);
		break;
	case SOAP_TYPE___vbox__IAudioAdapter_USCOREsetAudioController:
		soap_serialize___vbox__IAudioAdapter_USCOREsetAudioController(soap, (const struct __vbox__IAudioAdapter_USCOREsetAudioController *)ptr);
		break;
	case SOAP_TYPE___vbox__IAudioAdapter_USCOREgetAudioController:
		soap_serialize___vbox__IAudioAdapter_USCOREgetAudioController(soap, (const struct __vbox__IAudioAdapter_USCOREgetAudioController *)ptr);
		break;
	case SOAP_TYPE___vbox__IAudioAdapter_USCOREsetEnabled:
		soap_serialize___vbox__IAudioAdapter_USCOREsetEnabled(soap, (const struct __vbox__IAudioAdapter_USCOREsetEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IAudioAdapter_USCOREgetEnabled:
		soap_serialize___vbox__IAudioAdapter_USCOREgetEnabled(soap, (const struct __vbox__IAudioAdapter_USCOREgetEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostUSBDeviceFilter_USCOREsetAction:
		soap_serialize___vbox__IHostUSBDeviceFilter_USCOREsetAction(soap, (const struct __vbox__IHostUSBDeviceFilter_USCOREsetAction *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostUSBDeviceFilter_USCOREgetAction:
		soap_serialize___vbox__IHostUSBDeviceFilter_USCOREgetAction(soap, (const struct __vbox__IHostUSBDeviceFilter_USCOREgetAction *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostUSBDevice_USCOREgetState:
		soap_serialize___vbox__IHostUSBDevice_USCOREgetState(soap, (const struct __vbox__IHostUSBDevice_USCOREgetState *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREsetMaskedInterfaces:
		soap_serialize___vbox__IUSBDeviceFilter_USCOREsetMaskedInterfaces(soap, (const struct __vbox__IUSBDeviceFilter_USCOREsetMaskedInterfaces *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREgetMaskedInterfaces:
		soap_serialize___vbox__IUSBDeviceFilter_USCOREgetMaskedInterfaces(soap, (const struct __vbox__IUSBDeviceFilter_USCOREgetMaskedInterfaces *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREsetRemote:
		soap_serialize___vbox__IUSBDeviceFilter_USCOREsetRemote(soap, (const struct __vbox__IUSBDeviceFilter_USCOREsetRemote *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREgetRemote:
		soap_serialize___vbox__IUSBDeviceFilter_USCOREgetRemote(soap, (const struct __vbox__IUSBDeviceFilter_USCOREgetRemote *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREsetPort:
		soap_serialize___vbox__IUSBDeviceFilter_USCOREsetPort(soap, (const struct __vbox__IUSBDeviceFilter_USCOREsetPort *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREgetPort:
		soap_serialize___vbox__IUSBDeviceFilter_USCOREgetPort(soap, (const struct __vbox__IUSBDeviceFilter_USCOREgetPort *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREsetSerialNumber:
		soap_serialize___vbox__IUSBDeviceFilter_USCOREsetSerialNumber(soap, (const struct __vbox__IUSBDeviceFilter_USCOREsetSerialNumber *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREgetSerialNumber:
		soap_serialize___vbox__IUSBDeviceFilter_USCOREgetSerialNumber(soap, (const struct __vbox__IUSBDeviceFilter_USCOREgetSerialNumber *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREsetProduct:
		soap_serialize___vbox__IUSBDeviceFilter_USCOREsetProduct(soap, (const struct __vbox__IUSBDeviceFilter_USCOREsetProduct *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREgetProduct:
		soap_serialize___vbox__IUSBDeviceFilter_USCOREgetProduct(soap, (const struct __vbox__IUSBDeviceFilter_USCOREgetProduct *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREsetManufacturer:
		soap_serialize___vbox__IUSBDeviceFilter_USCOREsetManufacturer(soap, (const struct __vbox__IUSBDeviceFilter_USCOREsetManufacturer *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREgetManufacturer:
		soap_serialize___vbox__IUSBDeviceFilter_USCOREgetManufacturer(soap, (const struct __vbox__IUSBDeviceFilter_USCOREgetManufacturer *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREsetRevision:
		soap_serialize___vbox__IUSBDeviceFilter_USCOREsetRevision(soap, (const struct __vbox__IUSBDeviceFilter_USCOREsetRevision *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREgetRevision:
		soap_serialize___vbox__IUSBDeviceFilter_USCOREgetRevision(soap, (const struct __vbox__IUSBDeviceFilter_USCOREgetRevision *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREsetProductId:
		soap_serialize___vbox__IUSBDeviceFilter_USCOREsetProductId(soap, (const struct __vbox__IUSBDeviceFilter_USCOREsetProductId *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREgetProductId:
		soap_serialize___vbox__IUSBDeviceFilter_USCOREgetProductId(soap, (const struct __vbox__IUSBDeviceFilter_USCOREgetProductId *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREsetVendorId:
		soap_serialize___vbox__IUSBDeviceFilter_USCOREsetVendorId(soap, (const struct __vbox__IUSBDeviceFilter_USCOREsetVendorId *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREgetVendorId:
		soap_serialize___vbox__IUSBDeviceFilter_USCOREgetVendorId(soap, (const struct __vbox__IUSBDeviceFilter_USCOREgetVendorId *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREsetActive:
		soap_serialize___vbox__IUSBDeviceFilter_USCOREsetActive(soap, (const struct __vbox__IUSBDeviceFilter_USCOREsetActive *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREgetActive:
		soap_serialize___vbox__IUSBDeviceFilter_USCOREgetActive(soap, (const struct __vbox__IUSBDeviceFilter_USCOREgetActive *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREsetName:
		soap_serialize___vbox__IUSBDeviceFilter_USCOREsetName(soap, (const struct __vbox__IUSBDeviceFilter_USCOREsetName *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREgetName:
		soap_serialize___vbox__IUSBDeviceFilter_USCOREgetName(soap, (const struct __vbox__IUSBDeviceFilter_USCOREgetName *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDevice_USCOREgetRemote:
		soap_serialize___vbox__IUSBDevice_USCOREgetRemote(soap, (const struct __vbox__IUSBDevice_USCOREgetRemote *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDevice_USCOREgetPortVersion:
		soap_serialize___vbox__IUSBDevice_USCOREgetPortVersion(soap, (const struct __vbox__IUSBDevice_USCOREgetPortVersion *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDevice_USCOREgetVersion:
		soap_serialize___vbox__IUSBDevice_USCOREgetVersion(soap, (const struct __vbox__IUSBDevice_USCOREgetVersion *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDevice_USCOREgetPort:
		soap_serialize___vbox__IUSBDevice_USCOREgetPort(soap, (const struct __vbox__IUSBDevice_USCOREgetPort *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDevice_USCOREgetAddress:
		soap_serialize___vbox__IUSBDevice_USCOREgetAddress(soap, (const struct __vbox__IUSBDevice_USCOREgetAddress *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDevice_USCOREgetSerialNumber:
		soap_serialize___vbox__IUSBDevice_USCOREgetSerialNumber(soap, (const struct __vbox__IUSBDevice_USCOREgetSerialNumber *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDevice_USCOREgetProduct:
		soap_serialize___vbox__IUSBDevice_USCOREgetProduct(soap, (const struct __vbox__IUSBDevice_USCOREgetProduct *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDevice_USCOREgetManufacturer:
		soap_serialize___vbox__IUSBDevice_USCOREgetManufacturer(soap, (const struct __vbox__IUSBDevice_USCOREgetManufacturer *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDevice_USCOREgetRevision:
		soap_serialize___vbox__IUSBDevice_USCOREgetRevision(soap, (const struct __vbox__IUSBDevice_USCOREgetRevision *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDevice_USCOREgetProductId:
		soap_serialize___vbox__IUSBDevice_USCOREgetProductId(soap, (const struct __vbox__IUSBDevice_USCOREgetProductId *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDevice_USCOREgetVendorId:
		soap_serialize___vbox__IUSBDevice_USCOREgetVendorId(soap, (const struct __vbox__IUSBDevice_USCOREgetVendorId *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBDevice_USCOREgetId:
		soap_serialize___vbox__IUSBDevice_USCOREgetId(soap, (const struct __vbox__IUSBDevice_USCOREgetId *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBController_USCOREremoveDeviceFilter:
		soap_serialize___vbox__IUSBController_USCOREremoveDeviceFilter(soap, (const struct __vbox__IUSBController_USCOREremoveDeviceFilter *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBController_USCOREinsertDeviceFilter:
		soap_serialize___vbox__IUSBController_USCOREinsertDeviceFilter(soap, (const struct __vbox__IUSBController_USCOREinsertDeviceFilter *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBController_USCOREcreateDeviceFilter:
		soap_serialize___vbox__IUSBController_USCOREcreateDeviceFilter(soap, (const struct __vbox__IUSBController_USCOREcreateDeviceFilter *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBController_USCOREgetDeviceFilters:
		soap_serialize___vbox__IUSBController_USCOREgetDeviceFilters(soap, (const struct __vbox__IUSBController_USCOREgetDeviceFilters *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBController_USCOREgetUSBStandard:
		soap_serialize___vbox__IUSBController_USCOREgetUSBStandard(soap, (const struct __vbox__IUSBController_USCOREgetUSBStandard *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBController_USCOREsetEnabledEhci:
		soap_serialize___vbox__IUSBController_USCOREsetEnabledEhci(soap, (const struct __vbox__IUSBController_USCOREsetEnabledEhci *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBController_USCOREgetEnabledEhci:
		soap_serialize___vbox__IUSBController_USCOREgetEnabledEhci(soap, (const struct __vbox__IUSBController_USCOREgetEnabledEhci *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBController_USCOREsetEnabled:
		soap_serialize___vbox__IUSBController_USCOREsetEnabled(soap, (const struct __vbox__IUSBController_USCOREsetEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IUSBController_USCOREgetEnabled:
		soap_serialize___vbox__IUSBController_USCOREgetEnabled(soap, (const struct __vbox__IUSBController_USCOREgetEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IParallelPort_USCOREsetPath:
		soap_serialize___vbox__IParallelPort_USCOREsetPath(soap, (const struct __vbox__IParallelPort_USCOREsetPath *)ptr);
		break;
	case SOAP_TYPE___vbox__IParallelPort_USCOREgetPath:
		soap_serialize___vbox__IParallelPort_USCOREgetPath(soap, (const struct __vbox__IParallelPort_USCOREgetPath *)ptr);
		break;
	case SOAP_TYPE___vbox__IParallelPort_USCOREsetIRQ:
		soap_serialize___vbox__IParallelPort_USCOREsetIRQ(soap, (const struct __vbox__IParallelPort_USCOREsetIRQ *)ptr);
		break;
	case SOAP_TYPE___vbox__IParallelPort_USCOREgetIRQ:
		soap_serialize___vbox__IParallelPort_USCOREgetIRQ(soap, (const struct __vbox__IParallelPort_USCOREgetIRQ *)ptr);
		break;
	case SOAP_TYPE___vbox__IParallelPort_USCOREsetIOBase:
		soap_serialize___vbox__IParallelPort_USCOREsetIOBase(soap, (const struct __vbox__IParallelPort_USCOREsetIOBase *)ptr);
		break;
	case SOAP_TYPE___vbox__IParallelPort_USCOREgetIOBase:
		soap_serialize___vbox__IParallelPort_USCOREgetIOBase(soap, (const struct __vbox__IParallelPort_USCOREgetIOBase *)ptr);
		break;
	case SOAP_TYPE___vbox__IParallelPort_USCOREsetEnabled:
		soap_serialize___vbox__IParallelPort_USCOREsetEnabled(soap, (const struct __vbox__IParallelPort_USCOREsetEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IParallelPort_USCOREgetEnabled:
		soap_serialize___vbox__IParallelPort_USCOREgetEnabled(soap, (const struct __vbox__IParallelPort_USCOREgetEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IParallelPort_USCOREgetSlot:
		soap_serialize___vbox__IParallelPort_USCOREgetSlot(soap, (const struct __vbox__IParallelPort_USCOREgetSlot *)ptr);
		break;
	case SOAP_TYPE___vbox__ISerialPort_USCOREsetPath:
		soap_serialize___vbox__ISerialPort_USCOREsetPath(soap, (const struct __vbox__ISerialPort_USCOREsetPath *)ptr);
		break;
	case SOAP_TYPE___vbox__ISerialPort_USCOREgetPath:
		soap_serialize___vbox__ISerialPort_USCOREgetPath(soap, (const struct __vbox__ISerialPort_USCOREgetPath *)ptr);
		break;
	case SOAP_TYPE___vbox__ISerialPort_USCOREsetServer:
		soap_serialize___vbox__ISerialPort_USCOREsetServer(soap, (const struct __vbox__ISerialPort_USCOREsetServer *)ptr);
		break;
	case SOAP_TYPE___vbox__ISerialPort_USCOREgetServer:
		soap_serialize___vbox__ISerialPort_USCOREgetServer(soap, (const struct __vbox__ISerialPort_USCOREgetServer *)ptr);
		break;
	case SOAP_TYPE___vbox__ISerialPort_USCOREsetHostMode:
		soap_serialize___vbox__ISerialPort_USCOREsetHostMode(soap, (const struct __vbox__ISerialPort_USCOREsetHostMode *)ptr);
		break;
	case SOAP_TYPE___vbox__ISerialPort_USCOREgetHostMode:
		soap_serialize___vbox__ISerialPort_USCOREgetHostMode(soap, (const struct __vbox__ISerialPort_USCOREgetHostMode *)ptr);
		break;
	case SOAP_TYPE___vbox__ISerialPort_USCOREsetIRQ:
		soap_serialize___vbox__ISerialPort_USCOREsetIRQ(soap, (const struct __vbox__ISerialPort_USCOREsetIRQ *)ptr);
		break;
	case SOAP_TYPE___vbox__ISerialPort_USCOREgetIRQ:
		soap_serialize___vbox__ISerialPort_USCOREgetIRQ(soap, (const struct __vbox__ISerialPort_USCOREgetIRQ *)ptr);
		break;
	case SOAP_TYPE___vbox__ISerialPort_USCOREsetIOBase:
		soap_serialize___vbox__ISerialPort_USCOREsetIOBase(soap, (const struct __vbox__ISerialPort_USCOREsetIOBase *)ptr);
		break;
	case SOAP_TYPE___vbox__ISerialPort_USCOREgetIOBase:
		soap_serialize___vbox__ISerialPort_USCOREgetIOBase(soap, (const struct __vbox__ISerialPort_USCOREgetIOBase *)ptr);
		break;
	case SOAP_TYPE___vbox__ISerialPort_USCOREsetEnabled:
		soap_serialize___vbox__ISerialPort_USCOREsetEnabled(soap, (const struct __vbox__ISerialPort_USCOREsetEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__ISerialPort_USCOREgetEnabled:
		soap_serialize___vbox__ISerialPort_USCOREgetEnabled(soap, (const struct __vbox__ISerialPort_USCOREgetEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__ISerialPort_USCOREgetSlot:
		soap_serialize___vbox__ISerialPort_USCOREgetSlot(soap, (const struct __vbox__ISerialPort_USCOREgetSlot *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREdetach:
		soap_serialize___vbox__INetworkAdapter_USCOREdetach(soap, (const struct __vbox__INetworkAdapter_USCOREdetach *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREattachToHostOnlyInterface:
		soap_serialize___vbox__INetworkAdapter_USCOREattachToHostOnlyInterface(soap, (const struct __vbox__INetworkAdapter_USCOREattachToHostOnlyInterface *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREattachToInternalNetwork:
		soap_serialize___vbox__INetworkAdapter_USCOREattachToInternalNetwork(soap, (const struct __vbox__INetworkAdapter_USCOREattachToInternalNetwork *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREattachToBridgedInterface:
		soap_serialize___vbox__INetworkAdapter_USCOREattachToBridgedInterface(soap, (const struct __vbox__INetworkAdapter_USCOREattachToBridgedInterface *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREattachToNAT:
		soap_serialize___vbox__INetworkAdapter_USCOREattachToNAT(soap, (const struct __vbox__INetworkAdapter_USCOREattachToNAT *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREsetTraceFile:
		soap_serialize___vbox__INetworkAdapter_USCOREsetTraceFile(soap, (const struct __vbox__INetworkAdapter_USCOREsetTraceFile *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREgetTraceFile:
		soap_serialize___vbox__INetworkAdapter_USCOREgetTraceFile(soap, (const struct __vbox__INetworkAdapter_USCOREgetTraceFile *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREsetTraceEnabled:
		soap_serialize___vbox__INetworkAdapter_USCOREsetTraceEnabled(soap, (const struct __vbox__INetworkAdapter_USCOREsetTraceEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREgetTraceEnabled:
		soap_serialize___vbox__INetworkAdapter_USCOREgetTraceEnabled(soap, (const struct __vbox__INetworkAdapter_USCOREgetTraceEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREsetLineSpeed:
		soap_serialize___vbox__INetworkAdapter_USCOREsetLineSpeed(soap, (const struct __vbox__INetworkAdapter_USCOREsetLineSpeed *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREgetLineSpeed:
		soap_serialize___vbox__INetworkAdapter_USCOREgetLineSpeed(soap, (const struct __vbox__INetworkAdapter_USCOREgetLineSpeed *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREsetCableConnected:
		soap_serialize___vbox__INetworkAdapter_USCOREsetCableConnected(soap, (const struct __vbox__INetworkAdapter_USCOREsetCableConnected *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREgetCableConnected:
		soap_serialize___vbox__INetworkAdapter_USCOREgetCableConnected(soap, (const struct __vbox__INetworkAdapter_USCOREgetCableConnected *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREsetNATNetwork:
		soap_serialize___vbox__INetworkAdapter_USCOREsetNATNetwork(soap, (const struct __vbox__INetworkAdapter_USCOREsetNATNetwork *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREgetNATNetwork:
		soap_serialize___vbox__INetworkAdapter_USCOREgetNATNetwork(soap, (const struct __vbox__INetworkAdapter_USCOREgetNATNetwork *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREsetInternalNetwork:
		soap_serialize___vbox__INetworkAdapter_USCOREsetInternalNetwork(soap, (const struct __vbox__INetworkAdapter_USCOREsetInternalNetwork *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREgetInternalNetwork:
		soap_serialize___vbox__INetworkAdapter_USCOREgetInternalNetwork(soap, (const struct __vbox__INetworkAdapter_USCOREgetInternalNetwork *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREsetHostInterface:
		soap_serialize___vbox__INetworkAdapter_USCOREsetHostInterface(soap, (const struct __vbox__INetworkAdapter_USCOREsetHostInterface *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREgetHostInterface:
		soap_serialize___vbox__INetworkAdapter_USCOREgetHostInterface(soap, (const struct __vbox__INetworkAdapter_USCOREgetHostInterface *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREgetAttachmentType:
		soap_serialize___vbox__INetworkAdapter_USCOREgetAttachmentType(soap, (const struct __vbox__INetworkAdapter_USCOREgetAttachmentType *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREsetMACAddress:
		soap_serialize___vbox__INetworkAdapter_USCOREsetMACAddress(soap, (const struct __vbox__INetworkAdapter_USCOREsetMACAddress *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREgetMACAddress:
		soap_serialize___vbox__INetworkAdapter_USCOREgetMACAddress(soap, (const struct __vbox__INetworkAdapter_USCOREgetMACAddress *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREsetEnabled:
		soap_serialize___vbox__INetworkAdapter_USCOREsetEnabled(soap, (const struct __vbox__INetworkAdapter_USCOREsetEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREgetEnabled:
		soap_serialize___vbox__INetworkAdapter_USCOREgetEnabled(soap, (const struct __vbox__INetworkAdapter_USCOREgetEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREgetSlot:
		soap_serialize___vbox__INetworkAdapter_USCOREgetSlot(soap, (const struct __vbox__INetworkAdapter_USCOREgetSlot *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREsetAdapterType:
		soap_serialize___vbox__INetworkAdapter_USCOREsetAdapterType(soap, (const struct __vbox__INetworkAdapter_USCOREsetAdapterType *)ptr);
		break;
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREgetAdapterType:
		soap_serialize___vbox__INetworkAdapter_USCOREgetAdapterType(soap, (const struct __vbox__INetworkAdapter_USCOREgetAdapterType *)ptr);
		break;
	case SOAP_TYPE___vbox__IDisplay_USCOREupdateCompleted:
		soap_serialize___vbox__IDisplay_USCOREupdateCompleted(soap, (const struct __vbox__IDisplay_USCOREupdateCompleted *)ptr);
		break;
	case SOAP_TYPE___vbox__IDisplay_USCOREresizeCompleted:
		soap_serialize___vbox__IDisplay_USCOREresizeCompleted(soap, (const struct __vbox__IDisplay_USCOREresizeCompleted *)ptr);
		break;
	case SOAP_TYPE___vbox__IDisplay_USCOREinvalidateAndUpdate:
		soap_serialize___vbox__IDisplay_USCOREinvalidateAndUpdate(soap, (const struct __vbox__IDisplay_USCOREinvalidateAndUpdate *)ptr);
		break;
	case SOAP_TYPE___vbox__IDisplay_USCOREtakeScreenShotSlow:
		soap_serialize___vbox__IDisplay_USCOREtakeScreenShotSlow(soap, (const struct __vbox__IDisplay_USCOREtakeScreenShotSlow *)ptr);
		break;
	case SOAP_TYPE___vbox__IDisplay_USCOREsetSeamlessMode:
		soap_serialize___vbox__IDisplay_USCOREsetSeamlessMode(soap, (const struct __vbox__IDisplay_USCOREsetSeamlessMode *)ptr);
		break;
	case SOAP_TYPE___vbox__IDisplay_USCOREsetVideoModeHint:
		soap_serialize___vbox__IDisplay_USCOREsetVideoModeHint(soap, (const struct __vbox__IDisplay_USCOREsetVideoModeHint *)ptr);
		break;
	case SOAP_TYPE___vbox__IDisplay_USCOREgetBitsPerPixel:
		soap_serialize___vbox__IDisplay_USCOREgetBitsPerPixel(soap, (const struct __vbox__IDisplay_USCOREgetBitsPerPixel *)ptr);
		break;
	case SOAP_TYPE___vbox__IDisplay_USCOREgetHeight:
		soap_serialize___vbox__IDisplay_USCOREgetHeight(soap, (const struct __vbox__IDisplay_USCOREgetHeight *)ptr);
		break;
	case SOAP_TYPE___vbox__IDisplay_USCOREgetWidth:
		soap_serialize___vbox__IDisplay_USCOREgetWidth(soap, (const struct __vbox__IDisplay_USCOREgetWidth *)ptr);
		break;
	case SOAP_TYPE___vbox__IMouse_USCOREputMouseEventAbsolute:
		soap_serialize___vbox__IMouse_USCOREputMouseEventAbsolute(soap, (const struct __vbox__IMouse_USCOREputMouseEventAbsolute *)ptr);
		break;
	case SOAP_TYPE___vbox__IMouse_USCOREputMouseEvent:
		soap_serialize___vbox__IMouse_USCOREputMouseEvent(soap, (const struct __vbox__IMouse_USCOREputMouseEvent *)ptr);
		break;
	case SOAP_TYPE___vbox__IMouse_USCOREgetAbsoluteSupported:
		soap_serialize___vbox__IMouse_USCOREgetAbsoluteSupported(soap, (const struct __vbox__IMouse_USCOREgetAbsoluteSupported *)ptr);
		break;
	case SOAP_TYPE___vbox__IKeyboard_USCOREputCAD:
		soap_serialize___vbox__IKeyboard_USCOREputCAD(soap, (const struct __vbox__IKeyboard_USCOREputCAD *)ptr);
		break;
	case SOAP_TYPE___vbox__IKeyboard_USCOREputScancodes:
		soap_serialize___vbox__IKeyboard_USCOREputScancodes(soap, (const struct __vbox__IKeyboard_USCOREputScancodes *)ptr);
		break;
	case SOAP_TYPE___vbox__IKeyboard_USCOREputScancode:
		soap_serialize___vbox__IKeyboard_USCOREputScancode(soap, (const struct __vbox__IKeyboard_USCOREputScancode *)ptr);
		break;
	case SOAP_TYPE___vbox__IFloppyDrive_USCOREgetHostDrive:
		soap_serialize___vbox__IFloppyDrive_USCOREgetHostDrive(soap, (const struct __vbox__IFloppyDrive_USCOREgetHostDrive *)ptr);
		break;
	case SOAP_TYPE___vbox__IFloppyDrive_USCOREgetImage:
		soap_serialize___vbox__IFloppyDrive_USCOREgetImage(soap, (const struct __vbox__IFloppyDrive_USCOREgetImage *)ptr);
		break;
	case SOAP_TYPE___vbox__IFloppyDrive_USCOREunmount:
		soap_serialize___vbox__IFloppyDrive_USCOREunmount(soap, (const struct __vbox__IFloppyDrive_USCOREunmount *)ptr);
		break;
	case SOAP_TYPE___vbox__IFloppyDrive_USCOREcaptureHostDrive:
		soap_serialize___vbox__IFloppyDrive_USCOREcaptureHostDrive(soap, (const struct __vbox__IFloppyDrive_USCOREcaptureHostDrive *)ptr);
		break;
	case SOAP_TYPE___vbox__IFloppyDrive_USCOREmountImage:
		soap_serialize___vbox__IFloppyDrive_USCOREmountImage(soap, (const struct __vbox__IFloppyDrive_USCOREmountImage *)ptr);
		break;
	case SOAP_TYPE___vbox__IFloppyDrive_USCOREgetState:
		soap_serialize___vbox__IFloppyDrive_USCOREgetState(soap, (const struct __vbox__IFloppyDrive_USCOREgetState *)ptr);
		break;
	case SOAP_TYPE___vbox__IFloppyDrive_USCOREsetEnabled:
		soap_serialize___vbox__IFloppyDrive_USCOREsetEnabled(soap, (const struct __vbox__IFloppyDrive_USCOREsetEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IFloppyDrive_USCOREgetEnabled:
		soap_serialize___vbox__IFloppyDrive_USCOREgetEnabled(soap, (const struct __vbox__IFloppyDrive_USCOREgetEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IDVDDrive_USCOREgetHostDrive:
		soap_serialize___vbox__IDVDDrive_USCOREgetHostDrive(soap, (const struct __vbox__IDVDDrive_USCOREgetHostDrive *)ptr);
		break;
	case SOAP_TYPE___vbox__IDVDDrive_USCOREgetImage:
		soap_serialize___vbox__IDVDDrive_USCOREgetImage(soap, (const struct __vbox__IDVDDrive_USCOREgetImage *)ptr);
		break;
	case SOAP_TYPE___vbox__IDVDDrive_USCOREunmount:
		soap_serialize___vbox__IDVDDrive_USCOREunmount(soap, (const struct __vbox__IDVDDrive_USCOREunmount *)ptr);
		break;
	case SOAP_TYPE___vbox__IDVDDrive_USCOREcaptureHostDrive:
		soap_serialize___vbox__IDVDDrive_USCOREcaptureHostDrive(soap, (const struct __vbox__IDVDDrive_USCOREcaptureHostDrive *)ptr);
		break;
	case SOAP_TYPE___vbox__IDVDDrive_USCOREmountImage:
		soap_serialize___vbox__IDVDDrive_USCOREmountImage(soap, (const struct __vbox__IDVDDrive_USCOREmountImage *)ptr);
		break;
	case SOAP_TYPE___vbox__IDVDDrive_USCOREsetPassthrough:
		soap_serialize___vbox__IDVDDrive_USCOREsetPassthrough(soap, (const struct __vbox__IDVDDrive_USCOREsetPassthrough *)ptr);
		break;
	case SOAP_TYPE___vbox__IDVDDrive_USCOREgetPassthrough:
		soap_serialize___vbox__IDVDDrive_USCOREgetPassthrough(soap, (const struct __vbox__IDVDDrive_USCOREgetPassthrough *)ptr);
		break;
	case SOAP_TYPE___vbox__IDVDDrive_USCOREgetState:
		soap_serialize___vbox__IDVDDrive_USCOREgetState(soap, (const struct __vbox__IDVDDrive_USCOREgetState *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDiskFormat_USCOREdescribeProperties:
		soap_serialize___vbox__IHardDiskFormat_USCOREdescribeProperties(soap, (const struct __vbox__IHardDiskFormat_USCOREdescribeProperties *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDiskFormat_USCOREgetCapabilities:
		soap_serialize___vbox__IHardDiskFormat_USCOREgetCapabilities(soap, (const struct __vbox__IHardDiskFormat_USCOREgetCapabilities *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDiskFormat_USCOREgetFileExtensions:
		soap_serialize___vbox__IHardDiskFormat_USCOREgetFileExtensions(soap, (const struct __vbox__IHardDiskFormat_USCOREgetFileExtensions *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDiskFormat_USCOREgetName:
		soap_serialize___vbox__IHardDiskFormat_USCOREgetName(soap, (const struct __vbox__IHardDiskFormat_USCOREgetName *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDiskFormat_USCOREgetId:
		soap_serialize___vbox__IHardDiskFormat_USCOREgetId(soap, (const struct __vbox__IHardDiskFormat_USCOREgetId *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDisk_USCOREreset:
		soap_serialize___vbox__IHardDisk_USCOREreset(soap, (const struct __vbox__IHardDisk_USCOREreset *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDisk_USCOREcompact:
		soap_serialize___vbox__IHardDisk_USCOREcompact(soap, (const struct __vbox__IHardDisk_USCOREcompact *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDisk_USCOREcloneTo:
		soap_serialize___vbox__IHardDisk_USCOREcloneTo(soap, (const struct __vbox__IHardDisk_USCOREcloneTo *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDisk_USCOREmergeTo:
		soap_serialize___vbox__IHardDisk_USCOREmergeTo(soap, (const struct __vbox__IHardDisk_USCOREmergeTo *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDisk_USCOREcreateDiffStorage:
		soap_serialize___vbox__IHardDisk_USCOREcreateDiffStorage(soap, (const struct __vbox__IHardDisk_USCOREcreateDiffStorage *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDisk_USCOREdeleteStorage:
		soap_serialize___vbox__IHardDisk_USCOREdeleteStorage(soap, (const struct __vbox__IHardDisk_USCOREdeleteStorage *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDisk_USCOREcreateBaseStorage:
		soap_serialize___vbox__IHardDisk_USCOREcreateBaseStorage(soap, (const struct __vbox__IHardDisk_USCOREcreateBaseStorage *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDisk_USCOREsetProperties:
		soap_serialize___vbox__IHardDisk_USCOREsetProperties(soap, (const struct __vbox__IHardDisk_USCOREsetProperties *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDisk_USCOREgetProperties:
		soap_serialize___vbox__IHardDisk_USCOREgetProperties(soap, (const struct __vbox__IHardDisk_USCOREgetProperties *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDisk_USCOREsetProperty:
		soap_serialize___vbox__IHardDisk_USCOREsetProperty(soap, (const struct __vbox__IHardDisk_USCOREsetProperty *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDisk_USCOREgetProperty:
		soap_serialize___vbox__IHardDisk_USCOREgetProperty(soap, (const struct __vbox__IHardDisk_USCOREgetProperty *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDisk_USCOREsetAutoReset:
		soap_serialize___vbox__IHardDisk_USCOREsetAutoReset(soap, (const struct __vbox__IHardDisk_USCOREsetAutoReset *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDisk_USCOREgetAutoReset:
		soap_serialize___vbox__IHardDisk_USCOREgetAutoReset(soap, (const struct __vbox__IHardDisk_USCOREgetAutoReset *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDisk_USCOREgetLogicalSize:
		soap_serialize___vbox__IHardDisk_USCOREgetLogicalSize(soap, (const struct __vbox__IHardDisk_USCOREgetLogicalSize *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDisk_USCOREgetReadOnly:
		soap_serialize___vbox__IHardDisk_USCOREgetReadOnly(soap, (const struct __vbox__IHardDisk_USCOREgetReadOnly *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDisk_USCOREgetRoot:
		soap_serialize___vbox__IHardDisk_USCOREgetRoot(soap, (const struct __vbox__IHardDisk_USCOREgetRoot *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDisk_USCOREgetChildren:
		soap_serialize___vbox__IHardDisk_USCOREgetChildren(soap, (const struct __vbox__IHardDisk_USCOREgetChildren *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDisk_USCOREgetParent:
		soap_serialize___vbox__IHardDisk_USCOREgetParent(soap, (const struct __vbox__IHardDisk_USCOREgetParent *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDisk_USCOREsetType:
		soap_serialize___vbox__IHardDisk_USCOREsetType(soap, (const struct __vbox__IHardDisk_USCOREsetType *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDisk_USCOREgetType:
		soap_serialize___vbox__IHardDisk_USCOREgetType(soap, (const struct __vbox__IHardDisk_USCOREgetType *)ptr);
		break;
	case SOAP_TYPE___vbox__IHardDisk_USCOREgetFormat:
		soap_serialize___vbox__IHardDisk_USCOREgetFormat(soap, (const struct __vbox__IHardDisk_USCOREgetFormat *)ptr);
		break;
	case SOAP_TYPE___vbox__IMedium_USCOREclose:
		soap_serialize___vbox__IMedium_USCOREclose(soap, (const struct __vbox__IMedium_USCOREclose *)ptr);
		break;
	case SOAP_TYPE___vbox__IMedium_USCOREunlockWrite:
		soap_serialize___vbox__IMedium_USCOREunlockWrite(soap, (const struct __vbox__IMedium_USCOREunlockWrite *)ptr);
		break;
	case SOAP_TYPE___vbox__IMedium_USCORElockWrite:
		soap_serialize___vbox__IMedium_USCORElockWrite(soap, (const struct __vbox__IMedium_USCORElockWrite *)ptr);
		break;
	case SOAP_TYPE___vbox__IMedium_USCOREunlockRead:
		soap_serialize___vbox__IMedium_USCOREunlockRead(soap, (const struct __vbox__IMedium_USCOREunlockRead *)ptr);
		break;
	case SOAP_TYPE___vbox__IMedium_USCORElockRead:
		soap_serialize___vbox__IMedium_USCORElockRead(soap, (const struct __vbox__IMedium_USCORElockRead *)ptr);
		break;
	case SOAP_TYPE___vbox__IMedium_USCOREgetSnapshotIds:
		soap_serialize___vbox__IMedium_USCOREgetSnapshotIds(soap, (const struct __vbox__IMedium_USCOREgetSnapshotIds *)ptr);
		break;
	case SOAP_TYPE___vbox__IMedium_USCOREgetMachineIds:
		soap_serialize___vbox__IMedium_USCOREgetMachineIds(soap, (const struct __vbox__IMedium_USCOREgetMachineIds *)ptr);
		break;
	case SOAP_TYPE___vbox__IMedium_USCOREgetLastAccessError:
		soap_serialize___vbox__IMedium_USCOREgetLastAccessError(soap, (const struct __vbox__IMedium_USCOREgetLastAccessError *)ptr);
		break;
	case SOAP_TYPE___vbox__IMedium_USCOREgetSize:
		soap_serialize___vbox__IMedium_USCOREgetSize(soap, (const struct __vbox__IMedium_USCOREgetSize *)ptr);
		break;
	case SOAP_TYPE___vbox__IMedium_USCOREgetName:
		soap_serialize___vbox__IMedium_USCOREgetName(soap, (const struct __vbox__IMedium_USCOREgetName *)ptr);
		break;
	case SOAP_TYPE___vbox__IMedium_USCOREsetLocation:
		soap_serialize___vbox__IMedium_USCOREsetLocation(soap, (const struct __vbox__IMedium_USCOREsetLocation *)ptr);
		break;
	case SOAP_TYPE___vbox__IMedium_USCOREgetLocation:
		soap_serialize___vbox__IMedium_USCOREgetLocation(soap, (const struct __vbox__IMedium_USCOREgetLocation *)ptr);
		break;
	case SOAP_TYPE___vbox__IMedium_USCOREgetState:
		soap_serialize___vbox__IMedium_USCOREgetState(soap, (const struct __vbox__IMedium_USCOREgetState *)ptr);
		break;
	case SOAP_TYPE___vbox__IMedium_USCOREsetDescription:
		soap_serialize___vbox__IMedium_USCOREsetDescription(soap, (const struct __vbox__IMedium_USCOREsetDescription *)ptr);
		break;
	case SOAP_TYPE___vbox__IMedium_USCOREgetDescription:
		soap_serialize___vbox__IMedium_USCOREgetDescription(soap, (const struct __vbox__IMedium_USCOREgetDescription *)ptr);
		break;
	case SOAP_TYPE___vbox__IMedium_USCOREgetId:
		soap_serialize___vbox__IMedium_USCOREgetId(soap, (const struct __vbox__IMedium_USCOREgetId *)ptr);
		break;
	case SOAP_TYPE___vbox__ISnapshot_USCOREgetChildren:
		soap_serialize___vbox__ISnapshot_USCOREgetChildren(soap, (const struct __vbox__ISnapshot_USCOREgetChildren *)ptr);
		break;
	case SOAP_TYPE___vbox__ISnapshot_USCOREgetParent:
		soap_serialize___vbox__ISnapshot_USCOREgetParent(soap, (const struct __vbox__ISnapshot_USCOREgetParent *)ptr);
		break;
	case SOAP_TYPE___vbox__ISnapshot_USCOREgetMachine:
		soap_serialize___vbox__ISnapshot_USCOREgetMachine(soap, (const struct __vbox__ISnapshot_USCOREgetMachine *)ptr);
		break;
	case SOAP_TYPE___vbox__ISnapshot_USCOREgetOnline:
		soap_serialize___vbox__ISnapshot_USCOREgetOnline(soap, (const struct __vbox__ISnapshot_USCOREgetOnline *)ptr);
		break;
	case SOAP_TYPE___vbox__ISnapshot_USCOREgetTimeStamp:
		soap_serialize___vbox__ISnapshot_USCOREgetTimeStamp(soap, (const struct __vbox__ISnapshot_USCOREgetTimeStamp *)ptr);
		break;
	case SOAP_TYPE___vbox__ISnapshot_USCOREsetDescription:
		soap_serialize___vbox__ISnapshot_USCOREsetDescription(soap, (const struct __vbox__ISnapshot_USCOREsetDescription *)ptr);
		break;
	case SOAP_TYPE___vbox__ISnapshot_USCOREgetDescription:
		soap_serialize___vbox__ISnapshot_USCOREgetDescription(soap, (const struct __vbox__ISnapshot_USCOREgetDescription *)ptr);
		break;
	case SOAP_TYPE___vbox__ISnapshot_USCOREsetName:
		soap_serialize___vbox__ISnapshot_USCOREsetName(soap, (const struct __vbox__ISnapshot_USCOREsetName *)ptr);
		break;
	case SOAP_TYPE___vbox__ISnapshot_USCOREgetName:
		soap_serialize___vbox__ISnapshot_USCOREgetName(soap, (const struct __vbox__ISnapshot_USCOREgetName *)ptr);
		break;
	case SOAP_TYPE___vbox__ISnapshot_USCOREgetId:
		soap_serialize___vbox__ISnapshot_USCOREgetId(soap, (const struct __vbox__ISnapshot_USCOREgetId *)ptr);
		break;
	case SOAP_TYPE___vbox__IProgress_USCOREcancel:
		soap_serialize___vbox__IProgress_USCOREcancel(soap, (const struct __vbox__IProgress_USCOREcancel *)ptr);
		break;
	case SOAP_TYPE___vbox__IProgress_USCOREwaitForOperationCompletion:
		soap_serialize___vbox__IProgress_USCOREwaitForOperationCompletion(soap, (const struct __vbox__IProgress_USCOREwaitForOperationCompletion *)ptr);
		break;
	case SOAP_TYPE___vbox__IProgress_USCOREwaitForCompletion:
		soap_serialize___vbox__IProgress_USCOREwaitForCompletion(soap, (const struct __vbox__IProgress_USCOREwaitForCompletion *)ptr);
		break;
	case SOAP_TYPE___vbox__IProgress_USCOREgetOperationPercent:
		soap_serialize___vbox__IProgress_USCOREgetOperationPercent(soap, (const struct __vbox__IProgress_USCOREgetOperationPercent *)ptr);
		break;
	case SOAP_TYPE___vbox__IProgress_USCOREgetOperationDescription:
		soap_serialize___vbox__IProgress_USCOREgetOperationDescription(soap, (const struct __vbox__IProgress_USCOREgetOperationDescription *)ptr);
		break;
	case SOAP_TYPE___vbox__IProgress_USCOREgetOperation:
		soap_serialize___vbox__IProgress_USCOREgetOperation(soap, (const struct __vbox__IProgress_USCOREgetOperation *)ptr);
		break;
	case SOAP_TYPE___vbox__IProgress_USCOREgetOperationCount:
		soap_serialize___vbox__IProgress_USCOREgetOperationCount(soap, (const struct __vbox__IProgress_USCOREgetOperationCount *)ptr);
		break;
	case SOAP_TYPE___vbox__IProgress_USCOREgetErrorInfo:
		soap_serialize___vbox__IProgress_USCOREgetErrorInfo(soap, (const struct __vbox__IProgress_USCOREgetErrorInfo *)ptr);
		break;
	case SOAP_TYPE___vbox__IProgress_USCOREgetResultCode:
		soap_serialize___vbox__IProgress_USCOREgetResultCode(soap, (const struct __vbox__IProgress_USCOREgetResultCode *)ptr);
		break;
	case SOAP_TYPE___vbox__IProgress_USCOREgetCanceled:
		soap_serialize___vbox__IProgress_USCOREgetCanceled(soap, (const struct __vbox__IProgress_USCOREgetCanceled *)ptr);
		break;
	case SOAP_TYPE___vbox__IProgress_USCOREgetCompleted:
		soap_serialize___vbox__IProgress_USCOREgetCompleted(soap, (const struct __vbox__IProgress_USCOREgetCompleted *)ptr);
		break;
	case SOAP_TYPE___vbox__IProgress_USCOREgetTimeRemaining:
		soap_serialize___vbox__IProgress_USCOREgetTimeRemaining(soap, (const struct __vbox__IProgress_USCOREgetTimeRemaining *)ptr);
		break;
	case SOAP_TYPE___vbox__IProgress_USCOREgetPercent:
		soap_serialize___vbox__IProgress_USCOREgetPercent(soap, (const struct __vbox__IProgress_USCOREgetPercent *)ptr);
		break;
	case SOAP_TYPE___vbox__IProgress_USCOREgetCancelable:
		soap_serialize___vbox__IProgress_USCOREgetCancelable(soap, (const struct __vbox__IProgress_USCOREgetCancelable *)ptr);
		break;
	case SOAP_TYPE___vbox__IProgress_USCOREgetInitiator:
		soap_serialize___vbox__IProgress_USCOREgetInitiator(soap, (const struct __vbox__IProgress_USCOREgetInitiator *)ptr);
		break;
	case SOAP_TYPE___vbox__IProgress_USCOREgetDescription:
		soap_serialize___vbox__IProgress_USCOREgetDescription(soap, (const struct __vbox__IProgress_USCOREgetDescription *)ptr);
		break;
	case SOAP_TYPE___vbox__IProgress_USCOREgetId:
		soap_serialize___vbox__IProgress_USCOREgetId(soap, (const struct __vbox__IProgress_USCOREgetId *)ptr);
		break;
	case SOAP_TYPE___vbox__IGuest_USCOREgetStatistic:
		soap_serialize___vbox__IGuest_USCOREgetStatistic(soap, (const struct __vbox__IGuest_USCOREgetStatistic *)ptr);
		break;
	case SOAP_TYPE___vbox__IGuest_USCOREsetCredentials:
		soap_serialize___vbox__IGuest_USCOREsetCredentials(soap, (const struct __vbox__IGuest_USCOREsetCredentials *)ptr);
		break;
	case SOAP_TYPE___vbox__IGuest_USCOREsetStatisticsUpdateInterval:
		soap_serialize___vbox__IGuest_USCOREsetStatisticsUpdateInterval(soap, (const struct __vbox__IGuest_USCOREsetStatisticsUpdateInterval *)ptr);
		break;
	case SOAP_TYPE___vbox__IGuest_USCOREgetStatisticsUpdateInterval:
		soap_serialize___vbox__IGuest_USCOREgetStatisticsUpdateInterval(soap, (const struct __vbox__IGuest_USCOREgetStatisticsUpdateInterval *)ptr);
		break;
	case SOAP_TYPE___vbox__IGuest_USCOREsetMemoryBalloonSize:
		soap_serialize___vbox__IGuest_USCOREsetMemoryBalloonSize(soap, (const struct __vbox__IGuest_USCOREsetMemoryBalloonSize *)ptr);
		break;
	case SOAP_TYPE___vbox__IGuest_USCOREgetMemoryBalloonSize:
		soap_serialize___vbox__IGuest_USCOREgetMemoryBalloonSize(soap, (const struct __vbox__IGuest_USCOREgetMemoryBalloonSize *)ptr);
		break;
	case SOAP_TYPE___vbox__IGuest_USCOREgetSupportsGraphics:
		soap_serialize___vbox__IGuest_USCOREgetSupportsGraphics(soap, (const struct __vbox__IGuest_USCOREgetSupportsGraphics *)ptr);
		break;
	case SOAP_TYPE___vbox__IGuest_USCOREgetSupportsSeamless:
		soap_serialize___vbox__IGuest_USCOREgetSupportsSeamless(soap, (const struct __vbox__IGuest_USCOREgetSupportsSeamless *)ptr);
		break;
	case SOAP_TYPE___vbox__IGuest_USCOREgetAdditionsVersion:
		soap_serialize___vbox__IGuest_USCOREgetAdditionsVersion(soap, (const struct __vbox__IGuest_USCOREgetAdditionsVersion *)ptr);
		break;
	case SOAP_TYPE___vbox__IGuest_USCOREgetAdditionsActive:
		soap_serialize___vbox__IGuest_USCOREgetAdditionsActive(soap, (const struct __vbox__IGuest_USCOREgetAdditionsActive *)ptr);
		break;
	case SOAP_TYPE___vbox__IGuest_USCOREgetOSTypeId:
		soap_serialize___vbox__IGuest_USCOREgetOSTypeId(soap, (const struct __vbox__IGuest_USCOREgetOSTypeId *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetDefaultAudioDriver:
		soap_serialize___vbox__ISystemProperties_USCOREgetDefaultAudioDriver(soap, (const struct __vbox__ISystemProperties_USCOREgetDefaultAudioDriver *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREsetLogHistoryCount:
		soap_serialize___vbox__ISystemProperties_USCOREsetLogHistoryCount(soap, (const struct __vbox__ISystemProperties_USCOREsetLogHistoryCount *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetLogHistoryCount:
		soap_serialize___vbox__ISystemProperties_USCOREgetLogHistoryCount(soap, (const struct __vbox__ISystemProperties_USCOREgetLogHistoryCount *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREsetWebServiceAuthLibrary:
		soap_serialize___vbox__ISystemProperties_USCOREsetWebServiceAuthLibrary(soap, (const struct __vbox__ISystemProperties_USCOREsetWebServiceAuthLibrary *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetWebServiceAuthLibrary:
		soap_serialize___vbox__ISystemProperties_USCOREgetWebServiceAuthLibrary(soap, (const struct __vbox__ISystemProperties_USCOREgetWebServiceAuthLibrary *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibrary:
		soap_serialize___vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibrary(soap, (const struct __vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibrary *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibrary:
		soap_serialize___vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibrary(soap, (const struct __vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibrary *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREsetDefaultHardDiskFormat:
		soap_serialize___vbox__ISystemProperties_USCOREsetDefaultHardDiskFormat(soap, (const struct __vbox__ISystemProperties_USCOREsetDefaultHardDiskFormat *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetDefaultHardDiskFormat:
		soap_serialize___vbox__ISystemProperties_USCOREgetDefaultHardDiskFormat(soap, (const struct __vbox__ISystemProperties_USCOREgetDefaultHardDiskFormat *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetHardDiskFormats:
		soap_serialize___vbox__ISystemProperties_USCOREgetHardDiskFormats(soap, (const struct __vbox__ISystemProperties_USCOREgetHardDiskFormats *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREsetDefaultHardDiskFolder:
		soap_serialize___vbox__ISystemProperties_USCOREsetDefaultHardDiskFolder(soap, (const struct __vbox__ISystemProperties_USCOREsetDefaultHardDiskFolder *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetDefaultHardDiskFolder:
		soap_serialize___vbox__ISystemProperties_USCOREgetDefaultHardDiskFolder(soap, (const struct __vbox__ISystemProperties_USCOREgetDefaultHardDiskFolder *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREsetDefaultMachineFolder:
		soap_serialize___vbox__ISystemProperties_USCOREsetDefaultMachineFolder(soap, (const struct __vbox__ISystemProperties_USCOREsetDefaultMachineFolder *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetDefaultMachineFolder:
		soap_serialize___vbox__ISystemProperties_USCOREgetDefaultMachineFolder(soap, (const struct __vbox__ISystemProperties_USCOREgetDefaultMachineFolder *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetMaxBootPosition:
		soap_serialize___vbox__ISystemProperties_USCOREgetMaxBootPosition(soap, (const struct __vbox__ISystemProperties_USCOREgetMaxBootPosition *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetParallelPortCount:
		soap_serialize___vbox__ISystemProperties_USCOREgetParallelPortCount(soap, (const struct __vbox__ISystemProperties_USCOREgetParallelPortCount *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetSerialPortCount:
		soap_serialize___vbox__ISystemProperties_USCOREgetSerialPortCount(soap, (const struct __vbox__ISystemProperties_USCOREgetSerialPortCount *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetNetworkAdapterCount:
		soap_serialize___vbox__ISystemProperties_USCOREgetNetworkAdapterCount(soap, (const struct __vbox__ISystemProperties_USCOREgetNetworkAdapterCount *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetMaxVDISize:
		soap_serialize___vbox__ISystemProperties_USCOREgetMaxVDISize(soap, (const struct __vbox__ISystemProperties_USCOREgetMaxVDISize *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetMaxGuestCPUCount:
		soap_serialize___vbox__ISystemProperties_USCOREgetMaxGuestCPUCount(soap, (const struct __vbox__ISystemProperties_USCOREgetMaxGuestCPUCount *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetMinGuestCPUCount:
		soap_serialize___vbox__ISystemProperties_USCOREgetMinGuestCPUCount(soap, (const struct __vbox__ISystemProperties_USCOREgetMinGuestCPUCount *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetMaxGuestVRAM:
		soap_serialize___vbox__ISystemProperties_USCOREgetMaxGuestVRAM(soap, (const struct __vbox__ISystemProperties_USCOREgetMaxGuestVRAM *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetMinGuestVRAM:
		soap_serialize___vbox__ISystemProperties_USCOREgetMinGuestVRAM(soap, (const struct __vbox__ISystemProperties_USCOREgetMinGuestVRAM *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetMaxGuestRAM:
		soap_serialize___vbox__ISystemProperties_USCOREgetMaxGuestRAM(soap, (const struct __vbox__ISystemProperties_USCOREgetMaxGuestRAM *)ptr);
		break;
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetMinGuestRAM:
		soap_serialize___vbox__ISystemProperties_USCOREgetMinGuestRAM(soap, (const struct __vbox__ISystemProperties_USCOREgetMinGuestRAM *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREfindUSBDeviceByAddress:
		soap_serialize___vbox__IHost_USCOREfindUSBDeviceByAddress(soap, (const struct __vbox__IHost_USCOREfindUSBDeviceByAddress *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREfindUSBDeviceById:
		soap_serialize___vbox__IHost_USCOREfindUSBDeviceById(soap, (const struct __vbox__IHost_USCOREfindUSBDeviceById *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREfindHostNetworkInterfacesOfType:
		soap_serialize___vbox__IHost_USCOREfindHostNetworkInterfacesOfType(soap, (const struct __vbox__IHost_USCOREfindHostNetworkInterfacesOfType *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREfindHostNetworkInterfaceById:
		soap_serialize___vbox__IHost_USCOREfindHostNetworkInterfaceById(soap, (const struct __vbox__IHost_USCOREfindHostNetworkInterfaceById *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREfindHostNetworkInterfaceByName:
		soap_serialize___vbox__IHost_USCOREfindHostNetworkInterfaceByName(soap, (const struct __vbox__IHost_USCOREfindHostNetworkInterfaceByName *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREfindHostFloppyDrive:
		soap_serialize___vbox__IHost_USCOREfindHostFloppyDrive(soap, (const struct __vbox__IHost_USCOREfindHostFloppyDrive *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREfindHostDVDDrive:
		soap_serialize___vbox__IHost_USCOREfindHostDVDDrive(soap, (const struct __vbox__IHost_USCOREfindHostDVDDrive *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREremoveUSBDeviceFilter:
		soap_serialize___vbox__IHost_USCOREremoveUSBDeviceFilter(soap, (const struct __vbox__IHost_USCOREremoveUSBDeviceFilter *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREinsertUSBDeviceFilter:
		soap_serialize___vbox__IHost_USCOREinsertUSBDeviceFilter(soap, (const struct __vbox__IHost_USCOREinsertUSBDeviceFilter *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREcreateUSBDeviceFilter:
		soap_serialize___vbox__IHost_USCOREcreateUSBDeviceFilter(soap, (const struct __vbox__IHost_USCOREcreateUSBDeviceFilter *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREremoveHostOnlyNetworkInterface:
		soap_serialize___vbox__IHost_USCOREremoveHostOnlyNetworkInterface(soap, (const struct __vbox__IHost_USCOREremoveHostOnlyNetworkInterface *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREcreateHostOnlyNetworkInterface:
		soap_serialize___vbox__IHost_USCOREcreateHostOnlyNetworkInterface(soap, (const struct __vbox__IHost_USCOREcreateHostOnlyNetworkInterface *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREgetProcessorDescription:
		soap_serialize___vbox__IHost_USCOREgetProcessorDescription(soap, (const struct __vbox__IHost_USCOREgetProcessorDescription *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREgetProcessorFeature:
		soap_serialize___vbox__IHost_USCOREgetProcessorFeature(soap, (const struct __vbox__IHost_USCOREgetProcessorFeature *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREgetProcessorSpeed:
		soap_serialize___vbox__IHost_USCOREgetProcessorSpeed(soap, (const struct __vbox__IHost_USCOREgetProcessorSpeed *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREgetAcceleration3DAvailable:
		soap_serialize___vbox__IHost_USCOREgetAcceleration3DAvailable(soap, (const struct __vbox__IHost_USCOREgetAcceleration3DAvailable *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREgetUTCTime:
		soap_serialize___vbox__IHost_USCOREgetUTCTime(soap, (const struct __vbox__IHost_USCOREgetUTCTime *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREgetOSVersion:
		soap_serialize___vbox__IHost_USCOREgetOSVersion(soap, (const struct __vbox__IHost_USCOREgetOSVersion *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREgetOperatingSystem:
		soap_serialize___vbox__IHost_USCOREgetOperatingSystem(soap, (const struct __vbox__IHost_USCOREgetOperatingSystem *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREgetMemoryAvailable:
		soap_serialize___vbox__IHost_USCOREgetMemoryAvailable(soap, (const struct __vbox__IHost_USCOREgetMemoryAvailable *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREgetMemorySize:
		soap_serialize___vbox__IHost_USCOREgetMemorySize(soap, (const struct __vbox__IHost_USCOREgetMemorySize *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREgetProcessorOnlineCount:
		soap_serialize___vbox__IHost_USCOREgetProcessorOnlineCount(soap, (const struct __vbox__IHost_USCOREgetProcessorOnlineCount *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREgetProcessorCount:
		soap_serialize___vbox__IHost_USCOREgetProcessorCount(soap, (const struct __vbox__IHost_USCOREgetProcessorCount *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREgetNetworkInterfaces:
		soap_serialize___vbox__IHost_USCOREgetNetworkInterfaces(soap, (const struct __vbox__IHost_USCOREgetNetworkInterfaces *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREgetUSBDeviceFilters:
		soap_serialize___vbox__IHost_USCOREgetUSBDeviceFilters(soap, (const struct __vbox__IHost_USCOREgetUSBDeviceFilters *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREgetUSBDevices:
		soap_serialize___vbox__IHost_USCOREgetUSBDevices(soap, (const struct __vbox__IHost_USCOREgetUSBDevices *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREgetFloppyDrives:
		soap_serialize___vbox__IHost_USCOREgetFloppyDrives(soap, (const struct __vbox__IHost_USCOREgetFloppyDrives *)ptr);
		break;
	case SOAP_TYPE___vbox__IHost_USCOREgetDVDDrives:
		soap_serialize___vbox__IHost_USCOREgetDVDDrives(soap, (const struct __vbox__IHost_USCOREgetDVDDrives *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREdhcpRediscover:
		soap_serialize___vbox__IHostNetworkInterface_USCOREdhcpRediscover(soap, (const struct __vbox__IHostNetworkInterface_USCOREdhcpRediscover *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREenableDynamicIpConfig:
		soap_serialize___vbox__IHostNetworkInterface_USCOREenableDynamicIpConfig(soap, (const struct __vbox__IHostNetworkInterface_USCOREenableDynamicIpConfig *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6:
		soap_serialize___vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6(soap, (const struct __vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6 *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREenableStaticIpConfig:
		soap_serialize___vbox__IHostNetworkInterface_USCOREenableStaticIpConfig(soap, (const struct __vbox__IHostNetworkInterface_USCOREenableStaticIpConfig *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetInterfaceType:
		soap_serialize___vbox__IHostNetworkInterface_USCOREgetInterfaceType(soap, (const struct __vbox__IHostNetworkInterface_USCOREgetInterfaceType *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetStatus:
		soap_serialize___vbox__IHostNetworkInterface_USCOREgetStatus(soap, (const struct __vbox__IHostNetworkInterface_USCOREgetStatus *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetMediumType:
		soap_serialize___vbox__IHostNetworkInterface_USCOREgetMediumType(soap, (const struct __vbox__IHostNetworkInterface_USCOREgetMediumType *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetHardwareAddress:
		soap_serialize___vbox__IHostNetworkInterface_USCOREgetHardwareAddress(soap, (const struct __vbox__IHostNetworkInterface_USCOREgetHardwareAddress *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLength:
		soap_serialize___vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLength(soap, (const struct __vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLength *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetIPV6Address:
		soap_serialize___vbox__IHostNetworkInterface_USCOREgetIPV6Address(soap, (const struct __vbox__IHostNetworkInterface_USCOREgetIPV6Address *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetIPV6Supported:
		soap_serialize___vbox__IHostNetworkInterface_USCOREgetIPV6Supported(soap, (const struct __vbox__IHostNetworkInterface_USCOREgetIPV6Supported *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetNetworkMask:
		soap_serialize___vbox__IHostNetworkInterface_USCOREgetNetworkMask(soap, (const struct __vbox__IHostNetworkInterface_USCOREgetNetworkMask *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetIPAddress:
		soap_serialize___vbox__IHostNetworkInterface_USCOREgetIPAddress(soap, (const struct __vbox__IHostNetworkInterface_USCOREgetIPAddress *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetDhcpEnabled:
		soap_serialize___vbox__IHostNetworkInterface_USCOREgetDhcpEnabled(soap, (const struct __vbox__IHostNetworkInterface_USCOREgetDhcpEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetNetworkName:
		soap_serialize___vbox__IHostNetworkInterface_USCOREgetNetworkName(soap, (const struct __vbox__IHostNetworkInterface_USCOREgetNetworkName *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetId:
		soap_serialize___vbox__IHostNetworkInterface_USCOREgetId(soap, (const struct __vbox__IHostNetworkInterface_USCOREgetId *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetName:
		soap_serialize___vbox__IHostNetworkInterface_USCOREgetName(soap, (const struct __vbox__IHostNetworkInterface_USCOREgetName *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostFloppyDrive_USCOREgetUdi:
		soap_serialize___vbox__IHostFloppyDrive_USCOREgetUdi(soap, (const struct __vbox__IHostFloppyDrive_USCOREgetUdi *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostFloppyDrive_USCOREgetDescription:
		soap_serialize___vbox__IHostFloppyDrive_USCOREgetDescription(soap, (const struct __vbox__IHostFloppyDrive_USCOREgetDescription *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostFloppyDrive_USCOREgetName:
		soap_serialize___vbox__IHostFloppyDrive_USCOREgetName(soap, (const struct __vbox__IHostFloppyDrive_USCOREgetName *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostDVDDrive_USCOREgetUdi:
		soap_serialize___vbox__IHostDVDDrive_USCOREgetUdi(soap, (const struct __vbox__IHostDVDDrive_USCOREgetUdi *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostDVDDrive_USCOREgetDescription:
		soap_serialize___vbox__IHostDVDDrive_USCOREgetDescription(soap, (const struct __vbox__IHostDVDDrive_USCOREgetDescription *)ptr);
		break;
	case SOAP_TYPE___vbox__IHostDVDDrive_USCOREgetName:
		soap_serialize___vbox__IHostDVDDrive_USCOREgetName(soap, (const struct __vbox__IHostDVDDrive_USCOREgetName *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREdiscardCurrentSnapshotAndState:
		soap_serialize___vbox__IConsole_USCOREdiscardCurrentSnapshotAndState(soap, (const struct __vbox__IConsole_USCOREdiscardCurrentSnapshotAndState *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREdiscardCurrentState:
		soap_serialize___vbox__IConsole_USCOREdiscardCurrentState(soap, (const struct __vbox__IConsole_USCOREdiscardCurrentState *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREdiscardSnapshot:
		soap_serialize___vbox__IConsole_USCOREdiscardSnapshot(soap, (const struct __vbox__IConsole_USCOREdiscardSnapshot *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREtakeSnapshot:
		soap_serialize___vbox__IConsole_USCOREtakeSnapshot(soap, (const struct __vbox__IConsole_USCOREtakeSnapshot *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREremoveSharedFolder:
		soap_serialize___vbox__IConsole_USCOREremoveSharedFolder(soap, (const struct __vbox__IConsole_USCOREremoveSharedFolder *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREcreateSharedFolder:
		soap_serialize___vbox__IConsole_USCOREcreateSharedFolder(soap, (const struct __vbox__IConsole_USCOREcreateSharedFolder *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREfindUSBDeviceById:
		soap_serialize___vbox__IConsole_USCOREfindUSBDeviceById(soap, (const struct __vbox__IConsole_USCOREfindUSBDeviceById *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREfindUSBDeviceByAddress:
		soap_serialize___vbox__IConsole_USCOREfindUSBDeviceByAddress(soap, (const struct __vbox__IConsole_USCOREfindUSBDeviceByAddress *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREdetachUSBDevice:
		soap_serialize___vbox__IConsole_USCOREdetachUSBDevice(soap, (const struct __vbox__IConsole_USCOREdetachUSBDevice *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREattachUSBDevice:
		soap_serialize___vbox__IConsole_USCOREattachUSBDevice(soap, (const struct __vbox__IConsole_USCOREattachUSBDevice *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREgetDeviceActivity:
		soap_serialize___vbox__IConsole_USCOREgetDeviceActivity(soap, (const struct __vbox__IConsole_USCOREgetDeviceActivity *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREforgetSavedState:
		soap_serialize___vbox__IConsole_USCOREforgetSavedState(soap, (const struct __vbox__IConsole_USCOREforgetSavedState *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREadoptSavedState:
		soap_serialize___vbox__IConsole_USCOREadoptSavedState(soap, (const struct __vbox__IConsole_USCOREadoptSavedState *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREsaveState:
		soap_serialize___vbox__IConsole_USCOREsaveState(soap, (const struct __vbox__IConsole_USCOREsaveState *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREgetGuestEnteredACPIMode:
		soap_serialize___vbox__IConsole_USCOREgetGuestEnteredACPIMode(soap, (const struct __vbox__IConsole_USCOREgetGuestEnteredACPIMode *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREgetPowerButtonHandled:
		soap_serialize___vbox__IConsole_USCOREgetPowerButtonHandled(soap, (const struct __vbox__IConsole_USCOREgetPowerButtonHandled *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREsleepButton:
		soap_serialize___vbox__IConsole_USCOREsleepButton(soap, (const struct __vbox__IConsole_USCOREsleepButton *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREpowerButton:
		soap_serialize___vbox__IConsole_USCOREpowerButton(soap, (const struct __vbox__IConsole_USCOREpowerButton *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREresume:
		soap_serialize___vbox__IConsole_USCOREresume(soap, (const struct __vbox__IConsole_USCOREresume *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREpause:
		soap_serialize___vbox__IConsole_USCOREpause(soap, (const struct __vbox__IConsole_USCOREpause *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREreset:
		soap_serialize___vbox__IConsole_USCOREreset(soap, (const struct __vbox__IConsole_USCOREreset *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREpowerDown:
		soap_serialize___vbox__IConsole_USCOREpowerDown(soap, (const struct __vbox__IConsole_USCOREpowerDown *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREpowerUpPaused:
		soap_serialize___vbox__IConsole_USCOREpowerUpPaused(soap, (const struct __vbox__IConsole_USCOREpowerUpPaused *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREpowerUp:
		soap_serialize___vbox__IConsole_USCOREpowerUp(soap, (const struct __vbox__IConsole_USCOREpowerUp *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREgetRemoteDisplayInfo:
		soap_serialize___vbox__IConsole_USCOREgetRemoteDisplayInfo(soap, (const struct __vbox__IConsole_USCOREgetRemoteDisplayInfo *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREgetSharedFolders:
		soap_serialize___vbox__IConsole_USCOREgetSharedFolders(soap, (const struct __vbox__IConsole_USCOREgetSharedFolders *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREgetRemoteUSBDevices:
		soap_serialize___vbox__IConsole_USCOREgetRemoteUSBDevices(soap, (const struct __vbox__IConsole_USCOREgetRemoteUSBDevices *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREgetUSBDevices:
		soap_serialize___vbox__IConsole_USCOREgetUSBDevices(soap, (const struct __vbox__IConsole_USCOREgetUSBDevices *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREgetDisplay:
		soap_serialize___vbox__IConsole_USCOREgetDisplay(soap, (const struct __vbox__IConsole_USCOREgetDisplay *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREgetMouse:
		soap_serialize___vbox__IConsole_USCOREgetMouse(soap, (const struct __vbox__IConsole_USCOREgetMouse *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREgetKeyboard:
		soap_serialize___vbox__IConsole_USCOREgetKeyboard(soap, (const struct __vbox__IConsole_USCOREgetKeyboard *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREgetGuest:
		soap_serialize___vbox__IConsole_USCOREgetGuest(soap, (const struct __vbox__IConsole_USCOREgetGuest *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREgetState:
		soap_serialize___vbox__IConsole_USCOREgetState(soap, (const struct __vbox__IConsole_USCOREgetState *)ptr);
		break;
	case SOAP_TYPE___vbox__IConsole_USCOREgetMachine:
		soap_serialize___vbox__IConsole_USCOREgetMachine(soap, (const struct __vbox__IConsole_USCOREgetMachine *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREenumerateGuestProperties:
		soap_serialize___vbox__IMachine_USCOREenumerateGuestProperties(soap, (const struct __vbox__IMachine_USCOREenumerateGuestProperties *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsetGuestPropertyValue:
		soap_serialize___vbox__IMachine_USCOREsetGuestPropertyValue(soap, (const struct __vbox__IMachine_USCOREsetGuestPropertyValue *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsetGuestProperty:
		soap_serialize___vbox__IMachine_USCOREsetGuestProperty(soap, (const struct __vbox__IMachine_USCOREsetGuestProperty *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetGuestPropertyTimestamp:
		soap_serialize___vbox__IMachine_USCOREgetGuestPropertyTimestamp(soap, (const struct __vbox__IMachine_USCOREgetGuestPropertyTimestamp *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetGuestPropertyValue:
		soap_serialize___vbox__IMachine_USCOREgetGuestPropertyValue(soap, (const struct __vbox__IMachine_USCOREgetGuestPropertyValue *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetGuestProperty:
		soap_serialize___vbox__IMachine_USCOREgetGuestProperty(soap, (const struct __vbox__IMachine_USCOREgetGuestProperty *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREshowConsoleWindow:
		soap_serialize___vbox__IMachine_USCOREshowConsoleWindow(soap, (const struct __vbox__IMachine_USCOREshowConsoleWindow *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREcanShowConsoleWindow:
		soap_serialize___vbox__IMachine_USCOREcanShowConsoleWindow(soap, (const struct __vbox__IMachine_USCOREcanShowConsoleWindow *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREremoveSharedFolder:
		soap_serialize___vbox__IMachine_USCOREremoveSharedFolder(soap, (const struct __vbox__IMachine_USCOREremoveSharedFolder *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREcreateSharedFolder:
		soap_serialize___vbox__IMachine_USCOREcreateSharedFolder(soap, (const struct __vbox__IMachine_USCOREcreateSharedFolder *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsetCurrentSnapshot:
		soap_serialize___vbox__IMachine_USCOREsetCurrentSnapshot(soap, (const struct __vbox__IMachine_USCOREsetCurrentSnapshot *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREfindSnapshot:
		soap_serialize___vbox__IMachine_USCOREfindSnapshot(soap, (const struct __vbox__IMachine_USCOREfindSnapshot *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetSnapshot:
		soap_serialize___vbox__IMachine_USCOREgetSnapshot(soap, (const struct __vbox__IMachine_USCOREgetSnapshot *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREexport:
		soap_serialize___vbox__IMachine_USCOREexport(soap, (const struct __vbox__IMachine_USCOREexport *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREdeleteSettings:
		soap_serialize___vbox__IMachine_USCOREdeleteSettings(soap, (const struct __vbox__IMachine_USCOREdeleteSettings *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREdiscardSettings:
		soap_serialize___vbox__IMachine_USCOREdiscardSettings(soap, (const struct __vbox__IMachine_USCOREdiscardSettings *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsaveSettingsWithBackup:
		soap_serialize___vbox__IMachine_USCOREsaveSettingsWithBackup(soap, (const struct __vbox__IMachine_USCOREsaveSettingsWithBackup *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsaveSettings:
		soap_serialize___vbox__IMachine_USCOREsaveSettings(soap, (const struct __vbox__IMachine_USCOREsaveSettings *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsetExtraData:
		soap_serialize___vbox__IMachine_USCOREsetExtraData(soap, (const struct __vbox__IMachine_USCOREsetExtraData *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetExtraData:
		soap_serialize___vbox__IMachine_USCOREgetExtraData(soap, (const struct __vbox__IMachine_USCOREgetExtraData *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetNextExtraDataKey:
		soap_serialize___vbox__IMachine_USCOREgetNextExtraDataKey(soap, (const struct __vbox__IMachine_USCOREgetNextExtraDataKey *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetParallelPort:
		soap_serialize___vbox__IMachine_USCOREgetParallelPort(soap, (const struct __vbox__IMachine_USCOREgetParallelPort *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetSerialPort:
		soap_serialize___vbox__IMachine_USCOREgetSerialPort(soap, (const struct __vbox__IMachine_USCOREgetSerialPort *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREremoveStorageController:
		soap_serialize___vbox__IMachine_USCOREremoveStorageController(soap, (const struct __vbox__IMachine_USCOREremoveStorageController *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetStorageControllerByName:
		soap_serialize___vbox__IMachine_USCOREgetStorageControllerByName(soap, (const struct __vbox__IMachine_USCOREgetStorageControllerByName *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREaddStorageController:
		soap_serialize___vbox__IMachine_USCOREaddStorageController(soap, (const struct __vbox__IMachine_USCOREaddStorageController *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetNetworkAdapter:
		soap_serialize___vbox__IMachine_USCOREgetNetworkAdapter(soap, (const struct __vbox__IMachine_USCOREgetNetworkAdapter *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetHardDiskAttachmentsOfController:
		soap_serialize___vbox__IMachine_USCOREgetHardDiskAttachmentsOfController(soap, (const struct __vbox__IMachine_USCOREgetHardDiskAttachmentsOfController *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREdetachHardDisk:
		soap_serialize___vbox__IMachine_USCOREdetachHardDisk(soap, (const struct __vbox__IMachine_USCOREdetachHardDisk *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetHardDisk:
		soap_serialize___vbox__IMachine_USCOREgetHardDisk(soap, (const struct __vbox__IMachine_USCOREgetHardDisk *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREattachHardDisk:
		soap_serialize___vbox__IMachine_USCOREattachHardDisk(soap, (const struct __vbox__IMachine_USCOREattachHardDisk *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetBootOrder:
		soap_serialize___vbox__IMachine_USCOREgetBootOrder(soap, (const struct __vbox__IMachine_USCOREgetBootOrder *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsetBootOrder:
		soap_serialize___vbox__IMachine_USCOREsetBootOrder(soap, (const struct __vbox__IMachine_USCOREsetBootOrder *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsetGuestPropertyNotificationPatterns:
		soap_serialize___vbox__IMachine_USCOREsetGuestPropertyNotificationPatterns(soap, (const struct __vbox__IMachine_USCOREsetGuestPropertyNotificationPatterns *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetGuestPropertyNotificationPatterns:
		soap_serialize___vbox__IMachine_USCOREgetGuestPropertyNotificationPatterns(soap, (const struct __vbox__IMachine_USCOREgetGuestPropertyNotificationPatterns *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsetClipboardMode:
		soap_serialize___vbox__IMachine_USCOREsetClipboardMode(soap, (const struct __vbox__IMachine_USCOREsetClipboardMode *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetClipboardMode:
		soap_serialize___vbox__IMachine_USCOREgetClipboardMode(soap, (const struct __vbox__IMachine_USCOREgetClipboardMode *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetSharedFolders:
		soap_serialize___vbox__IMachine_USCOREgetSharedFolders(soap, (const struct __vbox__IMachine_USCOREgetSharedFolders *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetCurrentStateModified:
		soap_serialize___vbox__IMachine_USCOREgetCurrentStateModified(soap, (const struct __vbox__IMachine_USCOREgetCurrentStateModified *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetSnapshotCount:
		soap_serialize___vbox__IMachine_USCOREgetSnapshotCount(soap, (const struct __vbox__IMachine_USCOREgetSnapshotCount *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetCurrentSnapshot:
		soap_serialize___vbox__IMachine_USCOREgetCurrentSnapshot(soap, (const struct __vbox__IMachine_USCOREgetCurrentSnapshot *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetLogFolder:
		soap_serialize___vbox__IMachine_USCOREgetLogFolder(soap, (const struct __vbox__IMachine_USCOREgetLogFolder *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetStateFilePath:
		soap_serialize___vbox__IMachine_USCOREgetStateFilePath(soap, (const struct __vbox__IMachine_USCOREgetStateFilePath *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetLastStateChange:
		soap_serialize___vbox__IMachine_USCOREgetLastStateChange(soap, (const struct __vbox__IMachine_USCOREgetLastStateChange *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetState:
		soap_serialize___vbox__IMachine_USCOREgetState(soap, (const struct __vbox__IMachine_USCOREgetState *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetSessionPid:
		soap_serialize___vbox__IMachine_USCOREgetSessionPid(soap, (const struct __vbox__IMachine_USCOREgetSessionPid *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetSessionType:
		soap_serialize___vbox__IMachine_USCOREgetSessionType(soap, (const struct __vbox__IMachine_USCOREgetSessionType *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetSessionState:
		soap_serialize___vbox__IMachine_USCOREgetSessionState(soap, (const struct __vbox__IMachine_USCOREgetSessionState *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetSettingsModified:
		soap_serialize___vbox__IMachine_USCOREgetSettingsModified(soap, (const struct __vbox__IMachine_USCOREgetSettingsModified *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetSettingsFileVersion:
		soap_serialize___vbox__IMachine_USCOREgetSettingsFileVersion(soap, (const struct __vbox__IMachine_USCOREgetSettingsFileVersion *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetSettingsFilePath:
		soap_serialize___vbox__IMachine_USCOREgetSettingsFilePath(soap, (const struct __vbox__IMachine_USCOREgetSettingsFilePath *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetStorageControllers:
		soap_serialize___vbox__IMachine_USCOREgetStorageControllers(soap, (const struct __vbox__IMachine_USCOREgetStorageControllers *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetAudioAdapter:
		soap_serialize___vbox__IMachine_USCOREgetAudioAdapter(soap, (const struct __vbox__IMachine_USCOREgetAudioAdapter *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetUSBController:
		soap_serialize___vbox__IMachine_USCOREgetUSBController(soap, (const struct __vbox__IMachine_USCOREgetUSBController *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetFloppyDrive:
		soap_serialize___vbox__IMachine_USCOREgetFloppyDrive(soap, (const struct __vbox__IMachine_USCOREgetFloppyDrive *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetDVDDrive:
		soap_serialize___vbox__IMachine_USCOREgetDVDDrive(soap, (const struct __vbox__IMachine_USCOREgetDVDDrive *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetHardDiskAttachments:
		soap_serialize___vbox__IMachine_USCOREgetHardDiskAttachments(soap, (const struct __vbox__IMachine_USCOREgetHardDiskAttachments *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetVRDPServer:
		soap_serialize___vbox__IMachine_USCOREgetVRDPServer(soap, (const struct __vbox__IMachine_USCOREgetVRDPServer *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsetSnapshotFolder:
		soap_serialize___vbox__IMachine_USCOREsetSnapshotFolder(soap, (const struct __vbox__IMachine_USCOREsetSnapshotFolder *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetSnapshotFolder:
		soap_serialize___vbox__IMachine_USCOREgetSnapshotFolder(soap, (const struct __vbox__IMachine_USCOREgetSnapshotFolder *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsetPAEEnabled:
		soap_serialize___vbox__IMachine_USCOREsetPAEEnabled(soap, (const struct __vbox__IMachine_USCOREsetPAEEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetPAEEnabled:
		soap_serialize___vbox__IMachine_USCOREgetPAEEnabled(soap, (const struct __vbox__IMachine_USCOREgetPAEEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsetHWVirtExVPIDEnabled:
		soap_serialize___vbox__IMachine_USCOREsetHWVirtExVPIDEnabled(soap, (const struct __vbox__IMachine_USCOREsetHWVirtExVPIDEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetHWVirtExVPIDEnabled:
		soap_serialize___vbox__IMachine_USCOREgetHWVirtExVPIDEnabled(soap, (const struct __vbox__IMachine_USCOREgetHWVirtExVPIDEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabled:
		soap_serialize___vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabled(soap, (const struct __vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabled:
		soap_serialize___vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabled(soap, (const struct __vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsetHWVirtExEnabled:
		soap_serialize___vbox__IMachine_USCOREsetHWVirtExEnabled(soap, (const struct __vbox__IMachine_USCOREsetHWVirtExEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetHWVirtExEnabled:
		soap_serialize___vbox__IMachine_USCOREgetHWVirtExEnabled(soap, (const struct __vbox__IMachine_USCOREgetHWVirtExEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetBIOSSettings:
		soap_serialize___vbox__IMachine_USCOREgetBIOSSettings(soap, (const struct __vbox__IMachine_USCOREgetBIOSSettings *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsetMonitorCount:
		soap_serialize___vbox__IMachine_USCOREsetMonitorCount(soap, (const struct __vbox__IMachine_USCOREsetMonitorCount *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetMonitorCount:
		soap_serialize___vbox__IMachine_USCOREgetMonitorCount(soap, (const struct __vbox__IMachine_USCOREgetMonitorCount *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsetAccelerate3DEnabled:
		soap_serialize___vbox__IMachine_USCOREsetAccelerate3DEnabled(soap, (const struct __vbox__IMachine_USCOREsetAccelerate3DEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetAccelerate3DEnabled:
		soap_serialize___vbox__IMachine_USCOREgetAccelerate3DEnabled(soap, (const struct __vbox__IMachine_USCOREgetAccelerate3DEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsetVRAMSize:
		soap_serialize___vbox__IMachine_USCOREsetVRAMSize(soap, (const struct __vbox__IMachine_USCOREsetVRAMSize *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetVRAMSize:
		soap_serialize___vbox__IMachine_USCOREgetVRAMSize(soap, (const struct __vbox__IMachine_USCOREgetVRAMSize *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsetStatisticsUpdateInterval:
		soap_serialize___vbox__IMachine_USCOREsetStatisticsUpdateInterval(soap, (const struct __vbox__IMachine_USCOREsetStatisticsUpdateInterval *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetStatisticsUpdateInterval:
		soap_serialize___vbox__IMachine_USCOREgetStatisticsUpdateInterval(soap, (const struct __vbox__IMachine_USCOREgetStatisticsUpdateInterval *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsetMemoryBalloonSize:
		soap_serialize___vbox__IMachine_USCOREsetMemoryBalloonSize(soap, (const struct __vbox__IMachine_USCOREsetMemoryBalloonSize *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetMemoryBalloonSize:
		soap_serialize___vbox__IMachine_USCOREgetMemoryBalloonSize(soap, (const struct __vbox__IMachine_USCOREgetMemoryBalloonSize *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsetMemorySize:
		soap_serialize___vbox__IMachine_USCOREsetMemorySize(soap, (const struct __vbox__IMachine_USCOREsetMemorySize *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetMemorySize:
		soap_serialize___vbox__IMachine_USCOREgetMemorySize(soap, (const struct __vbox__IMachine_USCOREgetMemorySize *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsetCPUCount:
		soap_serialize___vbox__IMachine_USCOREsetCPUCount(soap, (const struct __vbox__IMachine_USCOREsetCPUCount *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetCPUCount:
		soap_serialize___vbox__IMachine_USCOREgetCPUCount(soap, (const struct __vbox__IMachine_USCOREgetCPUCount *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsetHardwareVersion:
		soap_serialize___vbox__IMachine_USCOREsetHardwareVersion(soap, (const struct __vbox__IMachine_USCOREsetHardwareVersion *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetHardwareVersion:
		soap_serialize___vbox__IMachine_USCOREgetHardwareVersion(soap, (const struct __vbox__IMachine_USCOREgetHardwareVersion *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsetOSTypeId:
		soap_serialize___vbox__IMachine_USCOREsetOSTypeId(soap, (const struct __vbox__IMachine_USCOREsetOSTypeId *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetOSTypeId:
		soap_serialize___vbox__IMachine_USCOREgetOSTypeId(soap, (const struct __vbox__IMachine_USCOREgetOSTypeId *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetId:
		soap_serialize___vbox__IMachine_USCOREgetId(soap, (const struct __vbox__IMachine_USCOREgetId *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsetDescription:
		soap_serialize___vbox__IMachine_USCOREsetDescription(soap, (const struct __vbox__IMachine_USCOREsetDescription *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetDescription:
		soap_serialize___vbox__IMachine_USCOREgetDescription(soap, (const struct __vbox__IMachine_USCOREgetDescription *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREsetName:
		soap_serialize___vbox__IMachine_USCOREsetName(soap, (const struct __vbox__IMachine_USCOREsetName *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetName:
		soap_serialize___vbox__IMachine_USCOREgetName(soap, (const struct __vbox__IMachine_USCOREgetName *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetAccessError:
		soap_serialize___vbox__IMachine_USCOREgetAccessError(soap, (const struct __vbox__IMachine_USCOREgetAccessError *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetAccessible:
		soap_serialize___vbox__IMachine_USCOREgetAccessible(soap, (const struct __vbox__IMachine_USCOREgetAccessible *)ptr);
		break;
	case SOAP_TYPE___vbox__IMachine_USCOREgetParent:
		soap_serialize___vbox__IMachine_USCOREgetParent(soap, (const struct __vbox__IMachine_USCOREgetParent *)ptr);
		break;
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREsetPXEDebugEnabled:
		soap_serialize___vbox__IBIOSSettings_USCOREsetPXEDebugEnabled(soap, (const struct __vbox__IBIOSSettings_USCOREsetPXEDebugEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREgetPXEDebugEnabled:
		soap_serialize___vbox__IBIOSSettings_USCOREgetPXEDebugEnabled(soap, (const struct __vbox__IBIOSSettings_USCOREgetPXEDebugEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREsetTimeOffset:
		soap_serialize___vbox__IBIOSSettings_USCOREsetTimeOffset(soap, (const struct __vbox__IBIOSSettings_USCOREsetTimeOffset *)ptr);
		break;
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREgetTimeOffset:
		soap_serialize___vbox__IBIOSSettings_USCOREgetTimeOffset(soap, (const struct __vbox__IBIOSSettings_USCOREgetTimeOffset *)ptr);
		break;
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREsetIOAPICEnabled:
		soap_serialize___vbox__IBIOSSettings_USCOREsetIOAPICEnabled(soap, (const struct __vbox__IBIOSSettings_USCOREsetIOAPICEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREgetIOAPICEnabled:
		soap_serialize___vbox__IBIOSSettings_USCOREgetIOAPICEnabled(soap, (const struct __vbox__IBIOSSettings_USCOREgetIOAPICEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREsetACPIEnabled:
		soap_serialize___vbox__IBIOSSettings_USCOREsetACPIEnabled(soap, (const struct __vbox__IBIOSSettings_USCOREsetACPIEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREgetACPIEnabled:
		soap_serialize___vbox__IBIOSSettings_USCOREgetACPIEnabled(soap, (const struct __vbox__IBIOSSettings_USCOREgetACPIEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREsetBootMenuMode:
		soap_serialize___vbox__IBIOSSettings_USCOREsetBootMenuMode(soap, (const struct __vbox__IBIOSSettings_USCOREsetBootMenuMode *)ptr);
		break;
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREgetBootMenuMode:
		soap_serialize___vbox__IBIOSSettings_USCOREgetBootMenuMode(soap, (const struct __vbox__IBIOSSettings_USCOREgetBootMenuMode *)ptr);
		break;
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREsetLogoImagePath:
		soap_serialize___vbox__IBIOSSettings_USCOREsetLogoImagePath(soap, (const struct __vbox__IBIOSSettings_USCOREsetLogoImagePath *)ptr);
		break;
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREgetLogoImagePath:
		soap_serialize___vbox__IBIOSSettings_USCOREgetLogoImagePath(soap, (const struct __vbox__IBIOSSettings_USCOREgetLogoImagePath *)ptr);
		break;
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREsetLogoDisplayTime:
		soap_serialize___vbox__IBIOSSettings_USCOREsetLogoDisplayTime(soap, (const struct __vbox__IBIOSSettings_USCOREsetLogoDisplayTime *)ptr);
		break;
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREgetLogoDisplayTime:
		soap_serialize___vbox__IBIOSSettings_USCOREgetLogoDisplayTime(soap, (const struct __vbox__IBIOSSettings_USCOREgetLogoDisplayTime *)ptr);
		break;
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREsetLogoFadeOut:
		soap_serialize___vbox__IBIOSSettings_USCOREsetLogoFadeOut(soap, (const struct __vbox__IBIOSSettings_USCOREsetLogoFadeOut *)ptr);
		break;
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREgetLogoFadeOut:
		soap_serialize___vbox__IBIOSSettings_USCOREgetLogoFadeOut(soap, (const struct __vbox__IBIOSSettings_USCOREgetLogoFadeOut *)ptr);
		break;
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREsetLogoFadeIn:
		soap_serialize___vbox__IBIOSSettings_USCOREsetLogoFadeIn(soap, (const struct __vbox__IBIOSSettings_USCOREsetLogoFadeIn *)ptr);
		break;
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREgetLogoFadeIn:
		soap_serialize___vbox__IBIOSSettings_USCOREgetLogoFadeIn(soap, (const struct __vbox__IBIOSSettings_USCOREgetLogoFadeIn *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualSystemDescription_USCOREaddDescription:
		soap_serialize___vbox__IVirtualSystemDescription_USCOREaddDescription(soap, (const struct __vbox__IVirtualSystemDescription_USCOREaddDescription *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualSystemDescription_USCOREsetFinalValues:
		soap_serialize___vbox__IVirtualSystemDescription_USCOREsetFinalValues(soap, (const struct __vbox__IVirtualSystemDescription_USCOREsetFinalValues *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualSystemDescription_USCOREgetValuesByType:
		soap_serialize___vbox__IVirtualSystemDescription_USCOREgetValuesByType(soap, (const struct __vbox__IVirtualSystemDescription_USCOREgetValuesByType *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualSystemDescription_USCOREgetDescriptionByType:
		soap_serialize___vbox__IVirtualSystemDescription_USCOREgetDescriptionByType(soap, (const struct __vbox__IVirtualSystemDescription_USCOREgetDescriptionByType *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualSystemDescription_USCOREgetDescription:
		soap_serialize___vbox__IVirtualSystemDescription_USCOREgetDescription(soap, (const struct __vbox__IVirtualSystemDescription_USCOREgetDescription *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualSystemDescription_USCOREgetCount:
		soap_serialize___vbox__IVirtualSystemDescription_USCOREgetCount(soap, (const struct __vbox__IVirtualSystemDescription_USCOREgetCount *)ptr);
		break;
	case SOAP_TYPE___vbox__IAppliance_USCOREgetWarnings:
		soap_serialize___vbox__IAppliance_USCOREgetWarnings(soap, (const struct __vbox__IAppliance_USCOREgetWarnings *)ptr);
		break;
	case SOAP_TYPE___vbox__IAppliance_USCOREwrite:
		soap_serialize___vbox__IAppliance_USCOREwrite(soap, (const struct __vbox__IAppliance_USCOREwrite *)ptr);
		break;
	case SOAP_TYPE___vbox__IAppliance_USCOREcreateVFSExplorer:
		soap_serialize___vbox__IAppliance_USCOREcreateVFSExplorer(soap, (const struct __vbox__IAppliance_USCOREcreateVFSExplorer *)ptr);
		break;
	case SOAP_TYPE___vbox__IAppliance_USCOREimportMachines:
		soap_serialize___vbox__IAppliance_USCOREimportMachines(soap, (const struct __vbox__IAppliance_USCOREimportMachines *)ptr);
		break;
	case SOAP_TYPE___vbox__IAppliance_USCOREinterpret:
		soap_serialize___vbox__IAppliance_USCOREinterpret(soap, (const struct __vbox__IAppliance_USCOREinterpret *)ptr);
		break;
	case SOAP_TYPE___vbox__IAppliance_USCOREread:
		soap_serialize___vbox__IAppliance_USCOREread(soap, (const struct __vbox__IAppliance_USCOREread *)ptr);
		break;
	case SOAP_TYPE___vbox__IAppliance_USCOREgetVirtualSystemDescriptions:
		soap_serialize___vbox__IAppliance_USCOREgetVirtualSystemDescriptions(soap, (const struct __vbox__IAppliance_USCOREgetVirtualSystemDescriptions *)ptr);
		break;
	case SOAP_TYPE___vbox__IAppliance_USCOREgetDisks:
		soap_serialize___vbox__IAppliance_USCOREgetDisks(soap, (const struct __vbox__IAppliance_USCOREgetDisks *)ptr);
		break;
	case SOAP_TYPE___vbox__IAppliance_USCOREgetPath:
		soap_serialize___vbox__IAppliance_USCOREgetPath(soap, (const struct __vbox__IAppliance_USCOREgetPath *)ptr);
		break;
	case SOAP_TYPE___vbox__IVFSExplorer_USCOREremove:
		soap_serialize___vbox__IVFSExplorer_USCOREremove(soap, (const struct __vbox__IVFSExplorer_USCOREremove *)ptr);
		break;
	case SOAP_TYPE___vbox__IVFSExplorer_USCOREexists:
		soap_serialize___vbox__IVFSExplorer_USCOREexists(soap, (const struct __vbox__IVFSExplorer_USCOREexists *)ptr);
		break;
	case SOAP_TYPE___vbox__IVFSExplorer_USCOREentryList:
		soap_serialize___vbox__IVFSExplorer_USCOREentryList(soap, (const struct __vbox__IVFSExplorer_USCOREentryList *)ptr);
		break;
	case SOAP_TYPE___vbox__IVFSExplorer_USCOREcdUp:
		soap_serialize___vbox__IVFSExplorer_USCOREcdUp(soap, (const struct __vbox__IVFSExplorer_USCOREcdUp *)ptr);
		break;
	case SOAP_TYPE___vbox__IVFSExplorer_USCOREcd:
		soap_serialize___vbox__IVFSExplorer_USCOREcd(soap, (const struct __vbox__IVFSExplorer_USCOREcd *)ptr);
		break;
	case SOAP_TYPE___vbox__IVFSExplorer_USCOREupdate:
		soap_serialize___vbox__IVFSExplorer_USCOREupdate(soap, (const struct __vbox__IVFSExplorer_USCOREupdate *)ptr);
		break;
	case SOAP_TYPE___vbox__IVFSExplorer_USCOREgetType:
		soap_serialize___vbox__IVFSExplorer_USCOREgetType(soap, (const struct __vbox__IVFSExplorer_USCOREgetType *)ptr);
		break;
	case SOAP_TYPE___vbox__IVFSExplorer_USCOREgetPath:
		soap_serialize___vbox__IVFSExplorer_USCOREgetPath(soap, (const struct __vbox__IVFSExplorer_USCOREgetPath *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREremoveDHCPServer:
		soap_serialize___vbox__IVirtualBox_USCOREremoveDHCPServer(soap, (const struct __vbox__IVirtualBox_USCOREremoveDHCPServer *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREfindDHCPServerByNetworkName:
		soap_serialize___vbox__IVirtualBox_USCOREfindDHCPServerByNetworkName(soap, (const struct __vbox__IVirtualBox_USCOREfindDHCPServerByNetworkName *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREcreateDHCPServer:
		soap_serialize___vbox__IVirtualBox_USCOREcreateDHCPServer(soap, (const struct __vbox__IVirtualBox_USCOREcreateDHCPServer *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREsaveSettingsWithBackup:
		soap_serialize___vbox__IVirtualBox_USCOREsaveSettingsWithBackup(soap, (const struct __vbox__IVirtualBox_USCOREsaveSettingsWithBackup *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREsaveSettings:
		soap_serialize___vbox__IVirtualBox_USCOREsaveSettings(soap, (const struct __vbox__IVirtualBox_USCOREsaveSettings *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREwaitForPropertyChange:
		soap_serialize___vbox__IVirtualBox_USCOREwaitForPropertyChange(soap, (const struct __vbox__IVirtualBox_USCOREwaitForPropertyChange *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREopenExistingSession:
		soap_serialize___vbox__IVirtualBox_USCOREopenExistingSession(soap, (const struct __vbox__IVirtualBox_USCOREopenExistingSession *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREopenRemoteSession:
		soap_serialize___vbox__IVirtualBox_USCOREopenRemoteSession(soap, (const struct __vbox__IVirtualBox_USCOREopenRemoteSession *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREopenSession:
		soap_serialize___vbox__IVirtualBox_USCOREopenSession(soap, (const struct __vbox__IVirtualBox_USCOREopenSession *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREsetExtraData:
		soap_serialize___vbox__IVirtualBox_USCOREsetExtraData(soap, (const struct __vbox__IVirtualBox_USCOREsetExtraData *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetExtraData:
		soap_serialize___vbox__IVirtualBox_USCOREgetExtraData(soap, (const struct __vbox__IVirtualBox_USCOREgetExtraData *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetNextExtraDataKey:
		soap_serialize___vbox__IVirtualBox_USCOREgetNextExtraDataKey(soap, (const struct __vbox__IVirtualBox_USCOREgetNextExtraDataKey *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREremoveSharedFolder:
		soap_serialize___vbox__IVirtualBox_USCOREremoveSharedFolder(soap, (const struct __vbox__IVirtualBox_USCOREremoveSharedFolder *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREcreateSharedFolder:
		soap_serialize___vbox__IVirtualBox_USCOREcreateSharedFolder(soap, (const struct __vbox__IVirtualBox_USCOREcreateSharedFolder *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetGuestOSType:
		soap_serialize___vbox__IVirtualBox_USCOREgetGuestOSType(soap, (const struct __vbox__IVirtualBox_USCOREgetGuestOSType *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREfindFloppyImage:
		soap_serialize___vbox__IVirtualBox_USCOREfindFloppyImage(soap, (const struct __vbox__IVirtualBox_USCOREfindFloppyImage *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetFloppyImage:
		soap_serialize___vbox__IVirtualBox_USCOREgetFloppyImage(soap, (const struct __vbox__IVirtualBox_USCOREgetFloppyImage *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREopenFloppyImage:
		soap_serialize___vbox__IVirtualBox_USCOREopenFloppyImage(soap, (const struct __vbox__IVirtualBox_USCOREopenFloppyImage *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREfindDVDImage:
		soap_serialize___vbox__IVirtualBox_USCOREfindDVDImage(soap, (const struct __vbox__IVirtualBox_USCOREfindDVDImage *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetDVDImage:
		soap_serialize___vbox__IVirtualBox_USCOREgetDVDImage(soap, (const struct __vbox__IVirtualBox_USCOREgetDVDImage *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREopenDVDImage:
		soap_serialize___vbox__IVirtualBox_USCOREopenDVDImage(soap, (const struct __vbox__IVirtualBox_USCOREopenDVDImage *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREfindHardDisk:
		soap_serialize___vbox__IVirtualBox_USCOREfindHardDisk(soap, (const struct __vbox__IVirtualBox_USCOREfindHardDisk *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetHardDisk:
		soap_serialize___vbox__IVirtualBox_USCOREgetHardDisk(soap, (const struct __vbox__IVirtualBox_USCOREgetHardDisk *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREopenHardDisk:
		soap_serialize___vbox__IVirtualBox_USCOREopenHardDisk(soap, (const struct __vbox__IVirtualBox_USCOREopenHardDisk *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREcreateHardDisk:
		soap_serialize___vbox__IVirtualBox_USCOREcreateHardDisk(soap, (const struct __vbox__IVirtualBox_USCOREcreateHardDisk *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREcreateAppliance:
		soap_serialize___vbox__IVirtualBox_USCOREcreateAppliance(soap, (const struct __vbox__IVirtualBox_USCOREcreateAppliance *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREunregisterMachine:
		soap_serialize___vbox__IVirtualBox_USCOREunregisterMachine(soap, (const struct __vbox__IVirtualBox_USCOREunregisterMachine *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREfindMachine:
		soap_serialize___vbox__IVirtualBox_USCOREfindMachine(soap, (const struct __vbox__IVirtualBox_USCOREfindMachine *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetMachine:
		soap_serialize___vbox__IVirtualBox_USCOREgetMachine(soap, (const struct __vbox__IVirtualBox_USCOREgetMachine *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREregisterMachine:
		soap_serialize___vbox__IVirtualBox_USCOREregisterMachine(soap, (const struct __vbox__IVirtualBox_USCOREregisterMachine *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREopenMachine:
		soap_serialize___vbox__IVirtualBox_USCOREopenMachine(soap, (const struct __vbox__IVirtualBox_USCOREopenMachine *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREcreateLegacyMachine:
		soap_serialize___vbox__IVirtualBox_USCOREcreateLegacyMachine(soap, (const struct __vbox__IVirtualBox_USCOREcreateLegacyMachine *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREcreateMachine:
		soap_serialize___vbox__IVirtualBox_USCOREcreateMachine(soap, (const struct __vbox__IVirtualBox_USCOREcreateMachine *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetDHCPServers:
		soap_serialize___vbox__IVirtualBox_USCOREgetDHCPServers(soap, (const struct __vbox__IVirtualBox_USCOREgetDHCPServers *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetPerformanceCollector:
		soap_serialize___vbox__IVirtualBox_USCOREgetPerformanceCollector(soap, (const struct __vbox__IVirtualBox_USCOREgetPerformanceCollector *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetSharedFolders:
		soap_serialize___vbox__IVirtualBox_USCOREgetSharedFolders(soap, (const struct __vbox__IVirtualBox_USCOREgetSharedFolders *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetGuestOSTypes:
		soap_serialize___vbox__IVirtualBox_USCOREgetGuestOSTypes(soap, (const struct __vbox__IVirtualBox_USCOREgetGuestOSTypes *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetProgressOperations:
		soap_serialize___vbox__IVirtualBox_USCOREgetProgressOperations(soap, (const struct __vbox__IVirtualBox_USCOREgetProgressOperations *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetFloppyImages:
		soap_serialize___vbox__IVirtualBox_USCOREgetFloppyImages(soap, (const struct __vbox__IVirtualBox_USCOREgetFloppyImages *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetDVDImages:
		soap_serialize___vbox__IVirtualBox_USCOREgetDVDImages(soap, (const struct __vbox__IVirtualBox_USCOREgetDVDImages *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetHardDisks:
		soap_serialize___vbox__IVirtualBox_USCOREgetHardDisks(soap, (const struct __vbox__IVirtualBox_USCOREgetHardDisks *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetMachines:
		soap_serialize___vbox__IVirtualBox_USCOREgetMachines(soap, (const struct __vbox__IVirtualBox_USCOREgetMachines *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetSystemProperties:
		soap_serialize___vbox__IVirtualBox_USCOREgetSystemProperties(soap, (const struct __vbox__IVirtualBox_USCOREgetSystemProperties *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetHost:
		soap_serialize___vbox__IVirtualBox_USCOREgetHost(soap, (const struct __vbox__IVirtualBox_USCOREgetHost *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetSettingsFormatVersion:
		soap_serialize___vbox__IVirtualBox_USCOREgetSettingsFormatVersion(soap, (const struct __vbox__IVirtualBox_USCOREgetSettingsFormatVersion *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetSettingsFileVersion:
		soap_serialize___vbox__IVirtualBox_USCOREgetSettingsFileVersion(soap, (const struct __vbox__IVirtualBox_USCOREgetSettingsFileVersion *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetSettingsFilePath:
		soap_serialize___vbox__IVirtualBox_USCOREgetSettingsFilePath(soap, (const struct __vbox__IVirtualBox_USCOREgetSettingsFilePath *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetHomeFolder:
		soap_serialize___vbox__IVirtualBox_USCOREgetHomeFolder(soap, (const struct __vbox__IVirtualBox_USCOREgetHomeFolder *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetPackageType:
		soap_serialize___vbox__IVirtualBox_USCOREgetPackageType(soap, (const struct __vbox__IVirtualBox_USCOREgetPackageType *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetRevision:
		soap_serialize___vbox__IVirtualBox_USCOREgetRevision(soap, (const struct __vbox__IVirtualBox_USCOREgetRevision *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetVersion:
		soap_serialize___vbox__IVirtualBox_USCOREgetVersion(soap, (const struct __vbox__IVirtualBox_USCOREgetVersion *)ptr);
		break;
	case SOAP_TYPE___vbox__IDHCPServer_USCOREstop:
		soap_serialize___vbox__IDHCPServer_USCOREstop(soap, (const struct __vbox__IDHCPServer_USCOREstop *)ptr);
		break;
	case SOAP_TYPE___vbox__IDHCPServer_USCOREstart:
		soap_serialize___vbox__IDHCPServer_USCOREstart(soap, (const struct __vbox__IDHCPServer_USCOREstart *)ptr);
		break;
	case SOAP_TYPE___vbox__IDHCPServer_USCOREsetConfiguration:
		soap_serialize___vbox__IDHCPServer_USCOREsetConfiguration(soap, (const struct __vbox__IDHCPServer_USCOREsetConfiguration *)ptr);
		break;
	case SOAP_TYPE___vbox__IDHCPServer_USCOREgetUpperIP:
		soap_serialize___vbox__IDHCPServer_USCOREgetUpperIP(soap, (const struct __vbox__IDHCPServer_USCOREgetUpperIP *)ptr);
		break;
	case SOAP_TYPE___vbox__IDHCPServer_USCOREgetLowerIP:
		soap_serialize___vbox__IDHCPServer_USCOREgetLowerIP(soap, (const struct __vbox__IDHCPServer_USCOREgetLowerIP *)ptr);
		break;
	case SOAP_TYPE___vbox__IDHCPServer_USCOREgetNetworkName:
		soap_serialize___vbox__IDHCPServer_USCOREgetNetworkName(soap, (const struct __vbox__IDHCPServer_USCOREgetNetworkName *)ptr);
		break;
	case SOAP_TYPE___vbox__IDHCPServer_USCOREgetNetworkMask:
		soap_serialize___vbox__IDHCPServer_USCOREgetNetworkMask(soap, (const struct __vbox__IDHCPServer_USCOREgetNetworkMask *)ptr);
		break;
	case SOAP_TYPE___vbox__IDHCPServer_USCOREgetIPAddress:
		soap_serialize___vbox__IDHCPServer_USCOREgetIPAddress(soap, (const struct __vbox__IDHCPServer_USCOREgetIPAddress *)ptr);
		break;
	case SOAP_TYPE___vbox__IDHCPServer_USCOREsetEnabled:
		soap_serialize___vbox__IDHCPServer_USCOREsetEnabled(soap, (const struct __vbox__IDHCPServer_USCOREsetEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IDHCPServer_USCOREgetEnabled:
		soap_serialize___vbox__IDHCPServer_USCOREgetEnabled(soap, (const struct __vbox__IDHCPServer_USCOREgetEnabled *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBoxErrorInfo_USCOREgetNext:
		soap_serialize___vbox__IVirtualBoxErrorInfo_USCOREgetNext(soap, (const struct __vbox__IVirtualBoxErrorInfo_USCOREgetNext *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBoxErrorInfo_USCOREgetText:
		soap_serialize___vbox__IVirtualBoxErrorInfo_USCOREgetText(soap, (const struct __vbox__IVirtualBoxErrorInfo_USCOREgetText *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBoxErrorInfo_USCOREgetComponent:
		soap_serialize___vbox__IVirtualBoxErrorInfo_USCOREgetComponent(soap, (const struct __vbox__IVirtualBoxErrorInfo_USCOREgetComponent *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceID:
		soap_serialize___vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceID(soap, (const struct __vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceID *)ptr);
		break;
	case SOAP_TYPE___vbox__IVirtualBoxErrorInfo_USCOREgetResultCode:
		soap_serialize___vbox__IVirtualBoxErrorInfo_USCOREgetResultCode(soap, (const struct __vbox__IVirtualBoxErrorInfo_USCOREgetResultCode *)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceCollector_USCOREqueryMetricsDataResponse:
		soap_serialize_PointerTo_vbox__IPerformanceCollector_USCOREqueryMetricsDataResponse(soap, (_vbox__IPerformanceCollector_USCOREqueryMetricsDataResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceCollector_USCOREqueryMetricsData:
		soap_serialize_PointerTo_vbox__IPerformanceCollector_USCOREqueryMetricsData(soap, (_vbox__IPerformanceCollector_USCOREqueryMetricsData *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceCollector_USCOREdisableMetricsResponse:
		soap_serialize_PointerTo_vbox__IPerformanceCollector_USCOREdisableMetricsResponse(soap, (_vbox__IPerformanceCollector_USCOREdisableMetricsResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceCollector_USCOREdisableMetrics:
		soap_serialize_PointerTo_vbox__IPerformanceCollector_USCOREdisableMetrics(soap, (_vbox__IPerformanceCollector_USCOREdisableMetrics *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceCollector_USCOREenableMetricsResponse:
		soap_serialize_PointerTo_vbox__IPerformanceCollector_USCOREenableMetricsResponse(soap, (_vbox__IPerformanceCollector_USCOREenableMetricsResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceCollector_USCOREenableMetrics:
		soap_serialize_PointerTo_vbox__IPerformanceCollector_USCOREenableMetrics(soap, (_vbox__IPerformanceCollector_USCOREenableMetrics *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceCollector_USCOREsetupMetricsResponse:
		soap_serialize_PointerTo_vbox__IPerformanceCollector_USCOREsetupMetricsResponse(soap, (_vbox__IPerformanceCollector_USCOREsetupMetricsResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceCollector_USCOREsetupMetrics:
		soap_serialize_PointerTo_vbox__IPerformanceCollector_USCOREsetupMetrics(soap, (_vbox__IPerformanceCollector_USCOREsetupMetrics *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceCollector_USCOREgetMetricsResponse:
		soap_serialize_PointerTo_vbox__IPerformanceCollector_USCOREgetMetricsResponse(soap, (_vbox__IPerformanceCollector_USCOREgetMetricsResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceCollector_USCOREgetMetrics:
		soap_serialize_PointerTo_vbox__IPerformanceCollector_USCOREgetMetrics(soap, (_vbox__IPerformanceCollector_USCOREgetMetrics *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceCollector_USCOREgetMetricNamesResponse:
		soap_serialize_PointerTo_vbox__IPerformanceCollector_USCOREgetMetricNamesResponse(soap, (_vbox__IPerformanceCollector_USCOREgetMetricNamesResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceCollector_USCOREgetMetricNames:
		soap_serialize_PointerTo_vbox__IPerformanceCollector_USCOREgetMetricNames(soap, (_vbox__IPerformanceCollector_USCOREgetMetricNames *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetMaximumValueResponse:
		soap_serialize_PointerTo_vbox__IPerformanceMetric_USCOREgetMaximumValueResponse(soap, (_vbox__IPerformanceMetric_USCOREgetMaximumValueResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetMaximumValue:
		soap_serialize_PointerTo_vbox__IPerformanceMetric_USCOREgetMaximumValue(soap, (_vbox__IPerformanceMetric_USCOREgetMaximumValue *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetMinimumValueResponse:
		soap_serialize_PointerTo_vbox__IPerformanceMetric_USCOREgetMinimumValueResponse(soap, (_vbox__IPerformanceMetric_USCOREgetMinimumValueResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetMinimumValue:
		soap_serialize_PointerTo_vbox__IPerformanceMetric_USCOREgetMinimumValue(soap, (_vbox__IPerformanceMetric_USCOREgetMinimumValue *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetUnitResponse:
		soap_serialize_PointerTo_vbox__IPerformanceMetric_USCOREgetUnitResponse(soap, (_vbox__IPerformanceMetric_USCOREgetUnitResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetUnit:
		soap_serialize_PointerTo_vbox__IPerformanceMetric_USCOREgetUnit(soap, (_vbox__IPerformanceMetric_USCOREgetUnit *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetCountResponse:
		soap_serialize_PointerTo_vbox__IPerformanceMetric_USCOREgetCountResponse(soap, (_vbox__IPerformanceMetric_USCOREgetCountResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetCount:
		soap_serialize_PointerTo_vbox__IPerformanceMetric_USCOREgetCount(soap, (_vbox__IPerformanceMetric_USCOREgetCount *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetPeriodResponse:
		soap_serialize_PointerTo_vbox__IPerformanceMetric_USCOREgetPeriodResponse(soap, (_vbox__IPerformanceMetric_USCOREgetPeriodResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetPeriod:
		soap_serialize_PointerTo_vbox__IPerformanceMetric_USCOREgetPeriod(soap, (_vbox__IPerformanceMetric_USCOREgetPeriod *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetDescriptionResponse:
		soap_serialize_PointerTo_vbox__IPerformanceMetric_USCOREgetDescriptionResponse(soap, (_vbox__IPerformanceMetric_USCOREgetDescriptionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetDescription:
		soap_serialize_PointerTo_vbox__IPerformanceMetric_USCOREgetDescription(soap, (_vbox__IPerformanceMetric_USCOREgetDescription *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetObjectResponse:
		soap_serialize_PointerTo_vbox__IPerformanceMetric_USCOREgetObjectResponse(soap, (_vbox__IPerformanceMetric_USCOREgetObjectResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetObject:
		soap_serialize_PointerTo_vbox__IPerformanceMetric_USCOREgetObject(soap, (_vbox__IPerformanceMetric_USCOREgetObject *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetMetricNameResponse:
		soap_serialize_PointerTo_vbox__IPerformanceMetric_USCOREgetMetricNameResponse(soap, (_vbox__IPerformanceMetric_USCOREgetMetricNameResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IPerformanceMetric_USCOREgetMetricName:
		soap_serialize_PointerTo_vbox__IPerformanceMetric_USCOREgetMetricName(soap, (_vbox__IPerformanceMetric_USCOREgetMetricName *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IWebsessionManager_USCORElogoffResponse:
		soap_serialize_PointerTo_vbox__IWebsessionManager_USCORElogoffResponse(soap, (_vbox__IWebsessionManager_USCORElogoffResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IWebsessionManager_USCORElogoff:
		soap_serialize_PointerTo_vbox__IWebsessionManager_USCORElogoff(soap, (_vbox__IWebsessionManager_USCORElogoff *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IWebsessionManager_USCOREgetSessionObjectResponse:
		soap_serialize_PointerTo_vbox__IWebsessionManager_USCOREgetSessionObjectResponse(soap, (_vbox__IWebsessionManager_USCOREgetSessionObjectResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IWebsessionManager_USCOREgetSessionObject:
		soap_serialize_PointerTo_vbox__IWebsessionManager_USCOREgetSessionObject(soap, (_vbox__IWebsessionManager_USCOREgetSessionObject *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IWebsessionManager_USCORElogonResponse:
		soap_serialize_PointerTo_vbox__IWebsessionManager_USCORElogonResponse(soap, (_vbox__IWebsessionManager_USCORElogonResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IWebsessionManager_USCORElogon:
		soap_serialize_PointerTo_vbox__IWebsessionManager_USCORElogon(soap, (_vbox__IWebsessionManager_USCORElogon *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IManagedObjectRef_USCOREreleaseResponse:
		soap_serialize_PointerTo_vbox__IManagedObjectRef_USCOREreleaseResponse(soap, (_vbox__IManagedObjectRef_USCOREreleaseResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IManagedObjectRef_USCORErelease:
		soap_serialize_PointerTo_vbox__IManagedObjectRef_USCORErelease(soap, (_vbox__IManagedObjectRef_USCORErelease *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IManagedObjectRef_USCOREgetInterfaceNameResponse:
		soap_serialize_PointerTo_vbox__IManagedObjectRef_USCOREgetInterfaceNameResponse(soap, (_vbox__IManagedObjectRef_USCOREgetInterfaceNameResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IManagedObjectRef_USCOREgetInterfaceName:
		soap_serialize_PointerTo_vbox__IManagedObjectRef_USCOREgetInterfaceName(soap, (_vbox__IManagedObjectRef_USCOREgetInterfaceName *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCORESetIDEEmulationPortResponse:
		soap_serialize_PointerTo_vbox__IStorageController_USCORESetIDEEmulationPortResponse(soap, (_vbox__IStorageController_USCORESetIDEEmulationPortResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCORESetIDEEmulationPort:
		soap_serialize_PointerTo_vbox__IStorageController_USCORESetIDEEmulationPort(soap, (_vbox__IStorageController_USCORESetIDEEmulationPort *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREGetIDEEmulationPortResponse:
		soap_serialize_PointerTo_vbox__IStorageController_USCOREGetIDEEmulationPortResponse(soap, (_vbox__IStorageController_USCOREGetIDEEmulationPortResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREGetIDEEmulationPort:
		soap_serialize_PointerTo_vbox__IStorageController_USCOREGetIDEEmulationPort(soap, (_vbox__IStorageController_USCOREGetIDEEmulationPort *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREsetControllerTypeResponse:
		soap_serialize_PointerTo_vbox__IStorageController_USCOREsetControllerTypeResponse(soap, (_vbox__IStorageController_USCOREsetControllerTypeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREsetControllerType:
		soap_serialize_PointerTo_vbox__IStorageController_USCOREsetControllerType(soap, (_vbox__IStorageController_USCOREsetControllerType *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetControllerTypeResponse:
		soap_serialize_PointerTo_vbox__IStorageController_USCOREgetControllerTypeResponse(soap, (_vbox__IStorageController_USCOREgetControllerTypeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetControllerType:
		soap_serialize_PointerTo_vbox__IStorageController_USCOREgetControllerType(soap, (_vbox__IStorageController_USCOREgetControllerType *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetBusResponse:
		soap_serialize_PointerTo_vbox__IStorageController_USCOREgetBusResponse(soap, (_vbox__IStorageController_USCOREgetBusResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetBus:
		soap_serialize_PointerTo_vbox__IStorageController_USCOREgetBus(soap, (_vbox__IStorageController_USCOREgetBus *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREsetPortCountResponse:
		soap_serialize_PointerTo_vbox__IStorageController_USCOREsetPortCountResponse(soap, (_vbox__IStorageController_USCOREsetPortCountResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREsetPortCount:
		soap_serialize_PointerTo_vbox__IStorageController_USCOREsetPortCount(soap, (_vbox__IStorageController_USCOREsetPortCount *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetPortCountResponse:
		soap_serialize_PointerTo_vbox__IStorageController_USCOREgetPortCountResponse(soap, (_vbox__IStorageController_USCOREgetPortCountResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetPortCount:
		soap_serialize_PointerTo_vbox__IStorageController_USCOREgetPortCount(soap, (_vbox__IStorageController_USCOREgetPortCount *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREsetInstanceResponse:
		soap_serialize_PointerTo_vbox__IStorageController_USCOREsetInstanceResponse(soap, (_vbox__IStorageController_USCOREsetInstanceResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREsetInstance:
		soap_serialize_PointerTo_vbox__IStorageController_USCOREsetInstance(soap, (_vbox__IStorageController_USCOREsetInstance *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetInstanceResponse:
		soap_serialize_PointerTo_vbox__IStorageController_USCOREgetInstanceResponse(soap, (_vbox__IStorageController_USCOREgetInstanceResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetInstance:
		soap_serialize_PointerTo_vbox__IStorageController_USCOREgetInstance(soap, (_vbox__IStorageController_USCOREgetInstance *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetMaxPortCountResponse:
		soap_serialize_PointerTo_vbox__IStorageController_USCOREgetMaxPortCountResponse(soap, (_vbox__IStorageController_USCOREgetMaxPortCountResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetMaxPortCount:
		soap_serialize_PointerTo_vbox__IStorageController_USCOREgetMaxPortCount(soap, (_vbox__IStorageController_USCOREgetMaxPortCount *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetMinPortCountResponse:
		soap_serialize_PointerTo_vbox__IStorageController_USCOREgetMinPortCountResponse(soap, (_vbox__IStorageController_USCOREgetMinPortCountResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetMinPortCount:
		soap_serialize_PointerTo_vbox__IStorageController_USCOREgetMinPortCount(soap, (_vbox__IStorageController_USCOREgetMinPortCount *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetMaxDevicesPerPortCountResponse:
		soap_serialize_PointerTo_vbox__IStorageController_USCOREgetMaxDevicesPerPortCountResponse(soap, (_vbox__IStorageController_USCOREgetMaxDevicesPerPortCountResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetMaxDevicesPerPortCount:
		soap_serialize_PointerTo_vbox__IStorageController_USCOREgetMaxDevicesPerPortCount(soap, (_vbox__IStorageController_USCOREgetMaxDevicesPerPortCount *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetNameResponse:
		soap_serialize_PointerTo_vbox__IStorageController_USCOREgetNameResponse(soap, (_vbox__IStorageController_USCOREgetNameResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IStorageController_USCOREgetName:
		soap_serialize_PointerTo_vbox__IStorageController_USCOREgetName(soap, (_vbox__IStorageController_USCOREgetName *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISession_USCOREcloseResponse:
		soap_serialize_PointerTo_vbox__ISession_USCOREcloseResponse(soap, (_vbox__ISession_USCOREcloseResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISession_USCOREclose:
		soap_serialize_PointerTo_vbox__ISession_USCOREclose(soap, (_vbox__ISession_USCOREclose *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISession_USCOREgetConsoleResponse:
		soap_serialize_PointerTo_vbox__ISession_USCOREgetConsoleResponse(soap, (_vbox__ISession_USCOREgetConsoleResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISession_USCOREgetConsole:
		soap_serialize_PointerTo_vbox__ISession_USCOREgetConsole(soap, (_vbox__ISession_USCOREgetConsole *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISession_USCOREgetMachineResponse:
		soap_serialize_PointerTo_vbox__ISession_USCOREgetMachineResponse(soap, (_vbox__ISession_USCOREgetMachineResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISession_USCOREgetMachine:
		soap_serialize_PointerTo_vbox__ISession_USCOREgetMachine(soap, (_vbox__ISession_USCOREgetMachine *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISession_USCOREgetTypeResponse:
		soap_serialize_PointerTo_vbox__ISession_USCOREgetTypeResponse(soap, (_vbox__ISession_USCOREgetTypeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISession_USCOREgetType:
		soap_serialize_PointerTo_vbox__ISession_USCOREgetType(soap, (_vbox__ISession_USCOREgetType *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISession_USCOREgetStateResponse:
		soap_serialize_PointerTo_vbox__ISession_USCOREgetStateResponse(soap, (_vbox__ISession_USCOREgetStateResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISession_USCOREgetState:
		soap_serialize_PointerTo_vbox__ISession_USCOREgetState(soap, (_vbox__ISession_USCOREgetState *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetReuseSingleConnectionResponse:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREsetReuseSingleConnectionResponse(soap, (_vbox__IVRDPServer_USCOREsetReuseSingleConnectionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetReuseSingleConnection:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREsetReuseSingleConnection(soap, (_vbox__IVRDPServer_USCOREsetReuseSingleConnection *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetReuseSingleConnectionResponse:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREgetReuseSingleConnectionResponse(soap, (_vbox__IVRDPServer_USCOREgetReuseSingleConnectionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetReuseSingleConnection:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREgetReuseSingleConnection(soap, (_vbox__IVRDPServer_USCOREgetReuseSingleConnection *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetAllowMultiConnectionResponse:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREsetAllowMultiConnectionResponse(soap, (_vbox__IVRDPServer_USCOREsetAllowMultiConnectionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetAllowMultiConnection:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREsetAllowMultiConnection(soap, (_vbox__IVRDPServer_USCOREsetAllowMultiConnection *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetAllowMultiConnectionResponse:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREgetAllowMultiConnectionResponse(soap, (_vbox__IVRDPServer_USCOREgetAllowMultiConnectionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetAllowMultiConnection:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREgetAllowMultiConnection(soap, (_vbox__IVRDPServer_USCOREgetAllowMultiConnection *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetAuthTimeoutResponse:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREsetAuthTimeoutResponse(soap, (_vbox__IVRDPServer_USCOREsetAuthTimeoutResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetAuthTimeout:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREsetAuthTimeout(soap, (_vbox__IVRDPServer_USCOREsetAuthTimeout *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetAuthTimeoutResponse:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREgetAuthTimeoutResponse(soap, (_vbox__IVRDPServer_USCOREgetAuthTimeoutResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetAuthTimeout:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREgetAuthTimeout(soap, (_vbox__IVRDPServer_USCOREgetAuthTimeout *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetAuthTypeResponse:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREsetAuthTypeResponse(soap, (_vbox__IVRDPServer_USCOREsetAuthTypeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetAuthType:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREsetAuthType(soap, (_vbox__IVRDPServer_USCOREsetAuthType *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetAuthTypeResponse:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREgetAuthTypeResponse(soap, (_vbox__IVRDPServer_USCOREgetAuthTypeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetAuthType:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREgetAuthType(soap, (_vbox__IVRDPServer_USCOREgetAuthType *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetNetAddressResponse:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREsetNetAddressResponse(soap, (_vbox__IVRDPServer_USCOREsetNetAddressResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetNetAddress:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREsetNetAddress(soap, (_vbox__IVRDPServer_USCOREsetNetAddress *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetNetAddressResponse:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREgetNetAddressResponse(soap, (_vbox__IVRDPServer_USCOREgetNetAddressResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetNetAddress:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREgetNetAddress(soap, (_vbox__IVRDPServer_USCOREgetNetAddress *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetPortResponse:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREsetPortResponse(soap, (_vbox__IVRDPServer_USCOREsetPortResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetPort:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREsetPort(soap, (_vbox__IVRDPServer_USCOREsetPort *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetPortResponse:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREgetPortResponse(soap, (_vbox__IVRDPServer_USCOREgetPortResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetPort:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREgetPort(soap, (_vbox__IVRDPServer_USCOREgetPort *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetEnabledResponse:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREsetEnabledResponse(soap, (_vbox__IVRDPServer_USCOREsetEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREsetEnabled:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREsetEnabled(soap, (_vbox__IVRDPServer_USCOREsetEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetEnabledResponse:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREgetEnabledResponse(soap, (_vbox__IVRDPServer_USCOREgetEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVRDPServer_USCOREgetEnabled:
		soap_serialize_PointerTo_vbox__IVRDPServer_USCOREgetEnabled(soap, (_vbox__IVRDPServer_USCOREgetEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAudioAdapter_USCOREsetAudioDriverResponse:
		soap_serialize_PointerTo_vbox__IAudioAdapter_USCOREsetAudioDriverResponse(soap, (_vbox__IAudioAdapter_USCOREsetAudioDriverResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAudioAdapter_USCOREsetAudioDriver:
		soap_serialize_PointerTo_vbox__IAudioAdapter_USCOREsetAudioDriver(soap, (_vbox__IAudioAdapter_USCOREsetAudioDriver *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAudioAdapter_USCOREgetAudioDriverResponse:
		soap_serialize_PointerTo_vbox__IAudioAdapter_USCOREgetAudioDriverResponse(soap, (_vbox__IAudioAdapter_USCOREgetAudioDriverResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAudioAdapter_USCOREgetAudioDriver:
		soap_serialize_PointerTo_vbox__IAudioAdapter_USCOREgetAudioDriver(soap, (_vbox__IAudioAdapter_USCOREgetAudioDriver *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAudioAdapter_USCOREsetAudioControllerResponse:
		soap_serialize_PointerTo_vbox__IAudioAdapter_USCOREsetAudioControllerResponse(soap, (_vbox__IAudioAdapter_USCOREsetAudioControllerResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAudioAdapter_USCOREsetAudioController:
		soap_serialize_PointerTo_vbox__IAudioAdapter_USCOREsetAudioController(soap, (_vbox__IAudioAdapter_USCOREsetAudioController *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAudioAdapter_USCOREgetAudioControllerResponse:
		soap_serialize_PointerTo_vbox__IAudioAdapter_USCOREgetAudioControllerResponse(soap, (_vbox__IAudioAdapter_USCOREgetAudioControllerResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAudioAdapter_USCOREgetAudioController:
		soap_serialize_PointerTo_vbox__IAudioAdapter_USCOREgetAudioController(soap, (_vbox__IAudioAdapter_USCOREgetAudioController *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAudioAdapter_USCOREsetEnabledResponse:
		soap_serialize_PointerTo_vbox__IAudioAdapter_USCOREsetEnabledResponse(soap, (_vbox__IAudioAdapter_USCOREsetEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAudioAdapter_USCOREsetEnabled:
		soap_serialize_PointerTo_vbox__IAudioAdapter_USCOREsetEnabled(soap, (_vbox__IAudioAdapter_USCOREsetEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAudioAdapter_USCOREgetEnabledResponse:
		soap_serialize_PointerTo_vbox__IAudioAdapter_USCOREgetEnabledResponse(soap, (_vbox__IAudioAdapter_USCOREgetEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAudioAdapter_USCOREgetEnabled:
		soap_serialize_PointerTo_vbox__IAudioAdapter_USCOREgetEnabled(soap, (_vbox__IAudioAdapter_USCOREgetEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostUSBDeviceFilter_USCOREsetActionResponse:
		soap_serialize_PointerTo_vbox__IHostUSBDeviceFilter_USCOREsetActionResponse(soap, (_vbox__IHostUSBDeviceFilter_USCOREsetActionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostUSBDeviceFilter_USCOREsetAction:
		soap_serialize_PointerTo_vbox__IHostUSBDeviceFilter_USCOREsetAction(soap, (_vbox__IHostUSBDeviceFilter_USCOREsetAction *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostUSBDeviceFilter_USCOREgetActionResponse:
		soap_serialize_PointerTo_vbox__IHostUSBDeviceFilter_USCOREgetActionResponse(soap, (_vbox__IHostUSBDeviceFilter_USCOREgetActionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostUSBDeviceFilter_USCOREgetAction:
		soap_serialize_PointerTo_vbox__IHostUSBDeviceFilter_USCOREgetAction(soap, (_vbox__IHostUSBDeviceFilter_USCOREgetAction *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostUSBDevice_USCOREgetStateResponse:
		soap_serialize_PointerTo_vbox__IHostUSBDevice_USCOREgetStateResponse(soap, (_vbox__IHostUSBDevice_USCOREgetStateResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostUSBDevice_USCOREgetState:
		soap_serialize_PointerTo_vbox__IHostUSBDevice_USCOREgetState(soap, (_vbox__IHostUSBDevice_USCOREgetState *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetMaskedInterfacesResponse:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREsetMaskedInterfacesResponse(soap, (_vbox__IUSBDeviceFilter_USCOREsetMaskedInterfacesResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetMaskedInterfaces:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREsetMaskedInterfaces(soap, (_vbox__IUSBDeviceFilter_USCOREsetMaskedInterfaces *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetMaskedInterfacesResponse:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREgetMaskedInterfacesResponse(soap, (_vbox__IUSBDeviceFilter_USCOREgetMaskedInterfacesResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetMaskedInterfaces:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREgetMaskedInterfaces(soap, (_vbox__IUSBDeviceFilter_USCOREgetMaskedInterfaces *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetRemoteResponse:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREsetRemoteResponse(soap, (_vbox__IUSBDeviceFilter_USCOREsetRemoteResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetRemote:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREsetRemote(soap, (_vbox__IUSBDeviceFilter_USCOREsetRemote *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetRemoteResponse:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREgetRemoteResponse(soap, (_vbox__IUSBDeviceFilter_USCOREgetRemoteResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetRemote:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREgetRemote(soap, (_vbox__IUSBDeviceFilter_USCOREgetRemote *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetPortResponse:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREsetPortResponse(soap, (_vbox__IUSBDeviceFilter_USCOREsetPortResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetPort:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREsetPort(soap, (_vbox__IUSBDeviceFilter_USCOREsetPort *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetPortResponse:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREgetPortResponse(soap, (_vbox__IUSBDeviceFilter_USCOREgetPortResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetPort:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREgetPort(soap, (_vbox__IUSBDeviceFilter_USCOREgetPort *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetSerialNumberResponse:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREsetSerialNumberResponse(soap, (_vbox__IUSBDeviceFilter_USCOREsetSerialNumberResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetSerialNumber:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREsetSerialNumber(soap, (_vbox__IUSBDeviceFilter_USCOREsetSerialNumber *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetSerialNumberResponse:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREgetSerialNumberResponse(soap, (_vbox__IUSBDeviceFilter_USCOREgetSerialNumberResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetSerialNumber:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREgetSerialNumber(soap, (_vbox__IUSBDeviceFilter_USCOREgetSerialNumber *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetProductResponse:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREsetProductResponse(soap, (_vbox__IUSBDeviceFilter_USCOREsetProductResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetProduct:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREsetProduct(soap, (_vbox__IUSBDeviceFilter_USCOREsetProduct *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetProductResponse:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREgetProductResponse(soap, (_vbox__IUSBDeviceFilter_USCOREgetProductResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetProduct:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREgetProduct(soap, (_vbox__IUSBDeviceFilter_USCOREgetProduct *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetManufacturerResponse:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREsetManufacturerResponse(soap, (_vbox__IUSBDeviceFilter_USCOREsetManufacturerResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetManufacturer:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREsetManufacturer(soap, (_vbox__IUSBDeviceFilter_USCOREsetManufacturer *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetManufacturerResponse:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREgetManufacturerResponse(soap, (_vbox__IUSBDeviceFilter_USCOREgetManufacturerResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetManufacturer:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREgetManufacturer(soap, (_vbox__IUSBDeviceFilter_USCOREgetManufacturer *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetRevisionResponse:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREsetRevisionResponse(soap, (_vbox__IUSBDeviceFilter_USCOREsetRevisionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetRevision:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREsetRevision(soap, (_vbox__IUSBDeviceFilter_USCOREsetRevision *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetRevisionResponse:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREgetRevisionResponse(soap, (_vbox__IUSBDeviceFilter_USCOREgetRevisionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetRevision:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREgetRevision(soap, (_vbox__IUSBDeviceFilter_USCOREgetRevision *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetProductIdResponse:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREsetProductIdResponse(soap, (_vbox__IUSBDeviceFilter_USCOREsetProductIdResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetProductId:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREsetProductId(soap, (_vbox__IUSBDeviceFilter_USCOREsetProductId *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetProductIdResponse:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREgetProductIdResponse(soap, (_vbox__IUSBDeviceFilter_USCOREgetProductIdResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetProductId:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREgetProductId(soap, (_vbox__IUSBDeviceFilter_USCOREgetProductId *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetVendorIdResponse:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREsetVendorIdResponse(soap, (_vbox__IUSBDeviceFilter_USCOREsetVendorIdResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetVendorId:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREsetVendorId(soap, (_vbox__IUSBDeviceFilter_USCOREsetVendorId *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetVendorIdResponse:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREgetVendorIdResponse(soap, (_vbox__IUSBDeviceFilter_USCOREgetVendorIdResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetVendorId:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREgetVendorId(soap, (_vbox__IUSBDeviceFilter_USCOREgetVendorId *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetActiveResponse:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREsetActiveResponse(soap, (_vbox__IUSBDeviceFilter_USCOREsetActiveResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetActive:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREsetActive(soap, (_vbox__IUSBDeviceFilter_USCOREsetActive *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetActiveResponse:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREgetActiveResponse(soap, (_vbox__IUSBDeviceFilter_USCOREgetActiveResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetActive:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREgetActive(soap, (_vbox__IUSBDeviceFilter_USCOREgetActive *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetNameResponse:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREsetNameResponse(soap, (_vbox__IUSBDeviceFilter_USCOREsetNameResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREsetName:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREsetName(soap, (_vbox__IUSBDeviceFilter_USCOREsetName *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetNameResponse:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREgetNameResponse(soap, (_vbox__IUSBDeviceFilter_USCOREgetNameResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDeviceFilter_USCOREgetName:
		soap_serialize_PointerTo_vbox__IUSBDeviceFilter_USCOREgetName(soap, (_vbox__IUSBDeviceFilter_USCOREgetName *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetRemoteResponse:
		soap_serialize_PointerTo_vbox__IUSBDevice_USCOREgetRemoteResponse(soap, (_vbox__IUSBDevice_USCOREgetRemoteResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetRemote:
		soap_serialize_PointerTo_vbox__IUSBDevice_USCOREgetRemote(soap, (_vbox__IUSBDevice_USCOREgetRemote *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetPortVersionResponse:
		soap_serialize_PointerTo_vbox__IUSBDevice_USCOREgetPortVersionResponse(soap, (_vbox__IUSBDevice_USCOREgetPortVersionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetPortVersion:
		soap_serialize_PointerTo_vbox__IUSBDevice_USCOREgetPortVersion(soap, (_vbox__IUSBDevice_USCOREgetPortVersion *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetVersionResponse:
		soap_serialize_PointerTo_vbox__IUSBDevice_USCOREgetVersionResponse(soap, (_vbox__IUSBDevice_USCOREgetVersionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetVersion:
		soap_serialize_PointerTo_vbox__IUSBDevice_USCOREgetVersion(soap, (_vbox__IUSBDevice_USCOREgetVersion *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetPortResponse:
		soap_serialize_PointerTo_vbox__IUSBDevice_USCOREgetPortResponse(soap, (_vbox__IUSBDevice_USCOREgetPortResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetPort:
		soap_serialize_PointerTo_vbox__IUSBDevice_USCOREgetPort(soap, (_vbox__IUSBDevice_USCOREgetPort *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetAddressResponse:
		soap_serialize_PointerTo_vbox__IUSBDevice_USCOREgetAddressResponse(soap, (_vbox__IUSBDevice_USCOREgetAddressResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetAddress:
		soap_serialize_PointerTo_vbox__IUSBDevice_USCOREgetAddress(soap, (_vbox__IUSBDevice_USCOREgetAddress *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetSerialNumberResponse:
		soap_serialize_PointerTo_vbox__IUSBDevice_USCOREgetSerialNumberResponse(soap, (_vbox__IUSBDevice_USCOREgetSerialNumberResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetSerialNumber:
		soap_serialize_PointerTo_vbox__IUSBDevice_USCOREgetSerialNumber(soap, (_vbox__IUSBDevice_USCOREgetSerialNumber *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetProductResponse:
		soap_serialize_PointerTo_vbox__IUSBDevice_USCOREgetProductResponse(soap, (_vbox__IUSBDevice_USCOREgetProductResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetProduct:
		soap_serialize_PointerTo_vbox__IUSBDevice_USCOREgetProduct(soap, (_vbox__IUSBDevice_USCOREgetProduct *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetManufacturerResponse:
		soap_serialize_PointerTo_vbox__IUSBDevice_USCOREgetManufacturerResponse(soap, (_vbox__IUSBDevice_USCOREgetManufacturerResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetManufacturer:
		soap_serialize_PointerTo_vbox__IUSBDevice_USCOREgetManufacturer(soap, (_vbox__IUSBDevice_USCOREgetManufacturer *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetRevisionResponse:
		soap_serialize_PointerTo_vbox__IUSBDevice_USCOREgetRevisionResponse(soap, (_vbox__IUSBDevice_USCOREgetRevisionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetRevision:
		soap_serialize_PointerTo_vbox__IUSBDevice_USCOREgetRevision(soap, (_vbox__IUSBDevice_USCOREgetRevision *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetProductIdResponse:
		soap_serialize_PointerTo_vbox__IUSBDevice_USCOREgetProductIdResponse(soap, (_vbox__IUSBDevice_USCOREgetProductIdResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetProductId:
		soap_serialize_PointerTo_vbox__IUSBDevice_USCOREgetProductId(soap, (_vbox__IUSBDevice_USCOREgetProductId *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetVendorIdResponse:
		soap_serialize_PointerTo_vbox__IUSBDevice_USCOREgetVendorIdResponse(soap, (_vbox__IUSBDevice_USCOREgetVendorIdResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetVendorId:
		soap_serialize_PointerTo_vbox__IUSBDevice_USCOREgetVendorId(soap, (_vbox__IUSBDevice_USCOREgetVendorId *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetIdResponse:
		soap_serialize_PointerTo_vbox__IUSBDevice_USCOREgetIdResponse(soap, (_vbox__IUSBDevice_USCOREgetIdResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBDevice_USCOREgetId:
		soap_serialize_PointerTo_vbox__IUSBDevice_USCOREgetId(soap, (_vbox__IUSBDevice_USCOREgetId *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREremoveDeviceFilterResponse:
		soap_serialize_PointerTo_vbox__IUSBController_USCOREremoveDeviceFilterResponse(soap, (_vbox__IUSBController_USCOREremoveDeviceFilterResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREremoveDeviceFilter:
		soap_serialize_PointerTo_vbox__IUSBController_USCOREremoveDeviceFilter(soap, (_vbox__IUSBController_USCOREremoveDeviceFilter *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREinsertDeviceFilterResponse:
		soap_serialize_PointerTo_vbox__IUSBController_USCOREinsertDeviceFilterResponse(soap, (_vbox__IUSBController_USCOREinsertDeviceFilterResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREinsertDeviceFilter:
		soap_serialize_PointerTo_vbox__IUSBController_USCOREinsertDeviceFilter(soap, (_vbox__IUSBController_USCOREinsertDeviceFilter *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREcreateDeviceFilterResponse:
		soap_serialize_PointerTo_vbox__IUSBController_USCOREcreateDeviceFilterResponse(soap, (_vbox__IUSBController_USCOREcreateDeviceFilterResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREcreateDeviceFilter:
		soap_serialize_PointerTo_vbox__IUSBController_USCOREcreateDeviceFilter(soap, (_vbox__IUSBController_USCOREcreateDeviceFilter *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREgetDeviceFiltersResponse:
		soap_serialize_PointerTo_vbox__IUSBController_USCOREgetDeviceFiltersResponse(soap, (_vbox__IUSBController_USCOREgetDeviceFiltersResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREgetDeviceFilters:
		soap_serialize_PointerTo_vbox__IUSBController_USCOREgetDeviceFilters(soap, (_vbox__IUSBController_USCOREgetDeviceFilters *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREgetUSBStandardResponse:
		soap_serialize_PointerTo_vbox__IUSBController_USCOREgetUSBStandardResponse(soap, (_vbox__IUSBController_USCOREgetUSBStandardResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREgetUSBStandard:
		soap_serialize_PointerTo_vbox__IUSBController_USCOREgetUSBStandard(soap, (_vbox__IUSBController_USCOREgetUSBStandard *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREsetEnabledEhciResponse:
		soap_serialize_PointerTo_vbox__IUSBController_USCOREsetEnabledEhciResponse(soap, (_vbox__IUSBController_USCOREsetEnabledEhciResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREsetEnabledEhci:
		soap_serialize_PointerTo_vbox__IUSBController_USCOREsetEnabledEhci(soap, (_vbox__IUSBController_USCOREsetEnabledEhci *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREgetEnabledEhciResponse:
		soap_serialize_PointerTo_vbox__IUSBController_USCOREgetEnabledEhciResponse(soap, (_vbox__IUSBController_USCOREgetEnabledEhciResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREgetEnabledEhci:
		soap_serialize_PointerTo_vbox__IUSBController_USCOREgetEnabledEhci(soap, (_vbox__IUSBController_USCOREgetEnabledEhci *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREsetEnabledResponse:
		soap_serialize_PointerTo_vbox__IUSBController_USCOREsetEnabledResponse(soap, (_vbox__IUSBController_USCOREsetEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREsetEnabled:
		soap_serialize_PointerTo_vbox__IUSBController_USCOREsetEnabled(soap, (_vbox__IUSBController_USCOREsetEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREgetEnabledResponse:
		soap_serialize_PointerTo_vbox__IUSBController_USCOREgetEnabledResponse(soap, (_vbox__IUSBController_USCOREgetEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IUSBController_USCOREgetEnabled:
		soap_serialize_PointerTo_vbox__IUSBController_USCOREgetEnabled(soap, (_vbox__IUSBController_USCOREgetEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREsetPathResponse:
		soap_serialize_PointerTo_vbox__IParallelPort_USCOREsetPathResponse(soap, (_vbox__IParallelPort_USCOREsetPathResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREsetPath:
		soap_serialize_PointerTo_vbox__IParallelPort_USCOREsetPath(soap, (_vbox__IParallelPort_USCOREsetPath *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREgetPathResponse:
		soap_serialize_PointerTo_vbox__IParallelPort_USCOREgetPathResponse(soap, (_vbox__IParallelPort_USCOREgetPathResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREgetPath:
		soap_serialize_PointerTo_vbox__IParallelPort_USCOREgetPath(soap, (_vbox__IParallelPort_USCOREgetPath *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREsetIRQResponse:
		soap_serialize_PointerTo_vbox__IParallelPort_USCOREsetIRQResponse(soap, (_vbox__IParallelPort_USCOREsetIRQResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREsetIRQ:
		soap_serialize_PointerTo_vbox__IParallelPort_USCOREsetIRQ(soap, (_vbox__IParallelPort_USCOREsetIRQ *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREgetIRQResponse:
		soap_serialize_PointerTo_vbox__IParallelPort_USCOREgetIRQResponse(soap, (_vbox__IParallelPort_USCOREgetIRQResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREgetIRQ:
		soap_serialize_PointerTo_vbox__IParallelPort_USCOREgetIRQ(soap, (_vbox__IParallelPort_USCOREgetIRQ *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREsetIOBaseResponse:
		soap_serialize_PointerTo_vbox__IParallelPort_USCOREsetIOBaseResponse(soap, (_vbox__IParallelPort_USCOREsetIOBaseResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREsetIOBase:
		soap_serialize_PointerTo_vbox__IParallelPort_USCOREsetIOBase(soap, (_vbox__IParallelPort_USCOREsetIOBase *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREgetIOBaseResponse:
		soap_serialize_PointerTo_vbox__IParallelPort_USCOREgetIOBaseResponse(soap, (_vbox__IParallelPort_USCOREgetIOBaseResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREgetIOBase:
		soap_serialize_PointerTo_vbox__IParallelPort_USCOREgetIOBase(soap, (_vbox__IParallelPort_USCOREgetIOBase *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREsetEnabledResponse:
		soap_serialize_PointerTo_vbox__IParallelPort_USCOREsetEnabledResponse(soap, (_vbox__IParallelPort_USCOREsetEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREsetEnabled:
		soap_serialize_PointerTo_vbox__IParallelPort_USCOREsetEnabled(soap, (_vbox__IParallelPort_USCOREsetEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREgetEnabledResponse:
		soap_serialize_PointerTo_vbox__IParallelPort_USCOREgetEnabledResponse(soap, (_vbox__IParallelPort_USCOREgetEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREgetEnabled:
		soap_serialize_PointerTo_vbox__IParallelPort_USCOREgetEnabled(soap, (_vbox__IParallelPort_USCOREgetEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREgetSlotResponse:
		soap_serialize_PointerTo_vbox__IParallelPort_USCOREgetSlotResponse(soap, (_vbox__IParallelPort_USCOREgetSlotResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IParallelPort_USCOREgetSlot:
		soap_serialize_PointerTo_vbox__IParallelPort_USCOREgetSlot(soap, (_vbox__IParallelPort_USCOREgetSlot *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREsetPathResponse:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREsetPathResponse(soap, (_vbox__ISerialPort_USCOREsetPathResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREsetPath:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREsetPath(soap, (_vbox__ISerialPort_USCOREsetPath *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetPathResponse:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREgetPathResponse(soap, (_vbox__ISerialPort_USCOREgetPathResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetPath:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREgetPath(soap, (_vbox__ISerialPort_USCOREgetPath *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREsetServerResponse:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREsetServerResponse(soap, (_vbox__ISerialPort_USCOREsetServerResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREsetServer:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREsetServer(soap, (_vbox__ISerialPort_USCOREsetServer *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetServerResponse:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREgetServerResponse(soap, (_vbox__ISerialPort_USCOREgetServerResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetServer:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREgetServer(soap, (_vbox__ISerialPort_USCOREgetServer *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREsetHostModeResponse:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREsetHostModeResponse(soap, (_vbox__ISerialPort_USCOREsetHostModeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREsetHostMode:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREsetHostMode(soap, (_vbox__ISerialPort_USCOREsetHostMode *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetHostModeResponse:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREgetHostModeResponse(soap, (_vbox__ISerialPort_USCOREgetHostModeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetHostMode:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREgetHostMode(soap, (_vbox__ISerialPort_USCOREgetHostMode *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREsetIRQResponse:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREsetIRQResponse(soap, (_vbox__ISerialPort_USCOREsetIRQResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREsetIRQ:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREsetIRQ(soap, (_vbox__ISerialPort_USCOREsetIRQ *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetIRQResponse:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREgetIRQResponse(soap, (_vbox__ISerialPort_USCOREgetIRQResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetIRQ:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREgetIRQ(soap, (_vbox__ISerialPort_USCOREgetIRQ *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREsetIOBaseResponse:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREsetIOBaseResponse(soap, (_vbox__ISerialPort_USCOREsetIOBaseResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREsetIOBase:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREsetIOBase(soap, (_vbox__ISerialPort_USCOREsetIOBase *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetIOBaseResponse:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREgetIOBaseResponse(soap, (_vbox__ISerialPort_USCOREgetIOBaseResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetIOBase:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREgetIOBase(soap, (_vbox__ISerialPort_USCOREgetIOBase *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREsetEnabledResponse:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREsetEnabledResponse(soap, (_vbox__ISerialPort_USCOREsetEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREsetEnabled:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREsetEnabled(soap, (_vbox__ISerialPort_USCOREsetEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetEnabledResponse:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREgetEnabledResponse(soap, (_vbox__ISerialPort_USCOREgetEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetEnabled:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREgetEnabled(soap, (_vbox__ISerialPort_USCOREgetEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetSlotResponse:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREgetSlotResponse(soap, (_vbox__ISerialPort_USCOREgetSlotResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISerialPort_USCOREgetSlot:
		soap_serialize_PointerTo_vbox__ISerialPort_USCOREgetSlot(soap, (_vbox__ISerialPort_USCOREgetSlot *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREdetachResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREdetachResponse(soap, (_vbox__INetworkAdapter_USCOREdetachResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREdetach:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREdetach(soap, (_vbox__INetworkAdapter_USCOREdetach *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREattachToHostOnlyInterfaceResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREattachToHostOnlyInterfaceResponse(soap, (_vbox__INetworkAdapter_USCOREattachToHostOnlyInterfaceResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREattachToHostOnlyInterface:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREattachToHostOnlyInterface(soap, (_vbox__INetworkAdapter_USCOREattachToHostOnlyInterface *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREattachToInternalNetworkResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREattachToInternalNetworkResponse(soap, (_vbox__INetworkAdapter_USCOREattachToInternalNetworkResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREattachToInternalNetwork:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREattachToInternalNetwork(soap, (_vbox__INetworkAdapter_USCOREattachToInternalNetwork *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREattachToBridgedInterfaceResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREattachToBridgedInterfaceResponse(soap, (_vbox__INetworkAdapter_USCOREattachToBridgedInterfaceResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREattachToBridgedInterface:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREattachToBridgedInterface(soap, (_vbox__INetworkAdapter_USCOREattachToBridgedInterface *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREattachToNATResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREattachToNATResponse(soap, (_vbox__INetworkAdapter_USCOREattachToNATResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREattachToNAT:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREattachToNAT(soap, (_vbox__INetworkAdapter_USCOREattachToNAT *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetTraceFileResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREsetTraceFileResponse(soap, (_vbox__INetworkAdapter_USCOREsetTraceFileResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetTraceFile:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREsetTraceFile(soap, (_vbox__INetworkAdapter_USCOREsetTraceFile *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetTraceFileResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREgetTraceFileResponse(soap, (_vbox__INetworkAdapter_USCOREgetTraceFileResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetTraceFile:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREgetTraceFile(soap, (_vbox__INetworkAdapter_USCOREgetTraceFile *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetTraceEnabledResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREsetTraceEnabledResponse(soap, (_vbox__INetworkAdapter_USCOREsetTraceEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetTraceEnabled:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREsetTraceEnabled(soap, (_vbox__INetworkAdapter_USCOREsetTraceEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetTraceEnabledResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREgetTraceEnabledResponse(soap, (_vbox__INetworkAdapter_USCOREgetTraceEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetTraceEnabled:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREgetTraceEnabled(soap, (_vbox__INetworkAdapter_USCOREgetTraceEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetLineSpeedResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREsetLineSpeedResponse(soap, (_vbox__INetworkAdapter_USCOREsetLineSpeedResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetLineSpeed:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREsetLineSpeed(soap, (_vbox__INetworkAdapter_USCOREsetLineSpeed *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetLineSpeedResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREgetLineSpeedResponse(soap, (_vbox__INetworkAdapter_USCOREgetLineSpeedResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetLineSpeed:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREgetLineSpeed(soap, (_vbox__INetworkAdapter_USCOREgetLineSpeed *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetCableConnectedResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREsetCableConnectedResponse(soap, (_vbox__INetworkAdapter_USCOREsetCableConnectedResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetCableConnected:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREsetCableConnected(soap, (_vbox__INetworkAdapter_USCOREsetCableConnected *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetCableConnectedResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREgetCableConnectedResponse(soap, (_vbox__INetworkAdapter_USCOREgetCableConnectedResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetCableConnected:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREgetCableConnected(soap, (_vbox__INetworkAdapter_USCOREgetCableConnected *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetNATNetworkResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREsetNATNetworkResponse(soap, (_vbox__INetworkAdapter_USCOREsetNATNetworkResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetNATNetwork:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREsetNATNetwork(soap, (_vbox__INetworkAdapter_USCOREsetNATNetwork *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetNATNetworkResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREgetNATNetworkResponse(soap, (_vbox__INetworkAdapter_USCOREgetNATNetworkResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetNATNetwork:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREgetNATNetwork(soap, (_vbox__INetworkAdapter_USCOREgetNATNetwork *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetInternalNetworkResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREsetInternalNetworkResponse(soap, (_vbox__INetworkAdapter_USCOREsetInternalNetworkResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetInternalNetwork:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREsetInternalNetwork(soap, (_vbox__INetworkAdapter_USCOREsetInternalNetwork *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetInternalNetworkResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREgetInternalNetworkResponse(soap, (_vbox__INetworkAdapter_USCOREgetInternalNetworkResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetInternalNetwork:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREgetInternalNetwork(soap, (_vbox__INetworkAdapter_USCOREgetInternalNetwork *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetHostInterfaceResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREsetHostInterfaceResponse(soap, (_vbox__INetworkAdapter_USCOREsetHostInterfaceResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetHostInterface:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREsetHostInterface(soap, (_vbox__INetworkAdapter_USCOREsetHostInterface *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetHostInterfaceResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREgetHostInterfaceResponse(soap, (_vbox__INetworkAdapter_USCOREgetHostInterfaceResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetHostInterface:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREgetHostInterface(soap, (_vbox__INetworkAdapter_USCOREgetHostInterface *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetAttachmentTypeResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREgetAttachmentTypeResponse(soap, (_vbox__INetworkAdapter_USCOREgetAttachmentTypeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetAttachmentType:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREgetAttachmentType(soap, (_vbox__INetworkAdapter_USCOREgetAttachmentType *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetMACAddressResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREsetMACAddressResponse(soap, (_vbox__INetworkAdapter_USCOREsetMACAddressResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetMACAddress:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREsetMACAddress(soap, (_vbox__INetworkAdapter_USCOREsetMACAddress *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetMACAddressResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREgetMACAddressResponse(soap, (_vbox__INetworkAdapter_USCOREgetMACAddressResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetMACAddress:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREgetMACAddress(soap, (_vbox__INetworkAdapter_USCOREgetMACAddress *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetEnabledResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREsetEnabledResponse(soap, (_vbox__INetworkAdapter_USCOREsetEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetEnabled:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREsetEnabled(soap, (_vbox__INetworkAdapter_USCOREsetEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetEnabledResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREgetEnabledResponse(soap, (_vbox__INetworkAdapter_USCOREgetEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetEnabled:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREgetEnabled(soap, (_vbox__INetworkAdapter_USCOREgetEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetSlotResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREgetSlotResponse(soap, (_vbox__INetworkAdapter_USCOREgetSlotResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetSlot:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREgetSlot(soap, (_vbox__INetworkAdapter_USCOREgetSlot *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetAdapterTypeResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREsetAdapterTypeResponse(soap, (_vbox__INetworkAdapter_USCOREsetAdapterTypeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREsetAdapterType:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREsetAdapterType(soap, (_vbox__INetworkAdapter_USCOREsetAdapterType *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetAdapterTypeResponse:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREgetAdapterTypeResponse(soap, (_vbox__INetworkAdapter_USCOREgetAdapterTypeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__INetworkAdapter_USCOREgetAdapterType:
		soap_serialize_PointerTo_vbox__INetworkAdapter_USCOREgetAdapterType(soap, (_vbox__INetworkAdapter_USCOREgetAdapterType *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREupdateCompletedResponse:
		soap_serialize_PointerTo_vbox__IDisplay_USCOREupdateCompletedResponse(soap, (_vbox__IDisplay_USCOREupdateCompletedResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREupdateCompleted:
		soap_serialize_PointerTo_vbox__IDisplay_USCOREupdateCompleted(soap, (_vbox__IDisplay_USCOREupdateCompleted *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREresizeCompletedResponse:
		soap_serialize_PointerTo_vbox__IDisplay_USCOREresizeCompletedResponse(soap, (_vbox__IDisplay_USCOREresizeCompletedResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREresizeCompleted:
		soap_serialize_PointerTo_vbox__IDisplay_USCOREresizeCompleted(soap, (_vbox__IDisplay_USCOREresizeCompleted *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREinvalidateAndUpdateResponse:
		soap_serialize_PointerTo_vbox__IDisplay_USCOREinvalidateAndUpdateResponse(soap, (_vbox__IDisplay_USCOREinvalidateAndUpdateResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREinvalidateAndUpdate:
		soap_serialize_PointerTo_vbox__IDisplay_USCOREinvalidateAndUpdate(soap, (_vbox__IDisplay_USCOREinvalidateAndUpdate *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREtakeScreenShotSlowResponse:
		soap_serialize_PointerTo_vbox__IDisplay_USCOREtakeScreenShotSlowResponse(soap, (_vbox__IDisplay_USCOREtakeScreenShotSlowResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREtakeScreenShotSlow:
		soap_serialize_PointerTo_vbox__IDisplay_USCOREtakeScreenShotSlow(soap, (_vbox__IDisplay_USCOREtakeScreenShotSlow *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREsetSeamlessModeResponse:
		soap_serialize_PointerTo_vbox__IDisplay_USCOREsetSeamlessModeResponse(soap, (_vbox__IDisplay_USCOREsetSeamlessModeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREsetSeamlessMode:
		soap_serialize_PointerTo_vbox__IDisplay_USCOREsetSeamlessMode(soap, (_vbox__IDisplay_USCOREsetSeamlessMode *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREsetVideoModeHintResponse:
		soap_serialize_PointerTo_vbox__IDisplay_USCOREsetVideoModeHintResponse(soap, (_vbox__IDisplay_USCOREsetVideoModeHintResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREsetVideoModeHint:
		soap_serialize_PointerTo_vbox__IDisplay_USCOREsetVideoModeHint(soap, (_vbox__IDisplay_USCOREsetVideoModeHint *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREgetBitsPerPixelResponse:
		soap_serialize_PointerTo_vbox__IDisplay_USCOREgetBitsPerPixelResponse(soap, (_vbox__IDisplay_USCOREgetBitsPerPixelResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREgetBitsPerPixel:
		soap_serialize_PointerTo_vbox__IDisplay_USCOREgetBitsPerPixel(soap, (_vbox__IDisplay_USCOREgetBitsPerPixel *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREgetHeightResponse:
		soap_serialize_PointerTo_vbox__IDisplay_USCOREgetHeightResponse(soap, (_vbox__IDisplay_USCOREgetHeightResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREgetHeight:
		soap_serialize_PointerTo_vbox__IDisplay_USCOREgetHeight(soap, (_vbox__IDisplay_USCOREgetHeight *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREgetWidthResponse:
		soap_serialize_PointerTo_vbox__IDisplay_USCOREgetWidthResponse(soap, (_vbox__IDisplay_USCOREgetWidthResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDisplay_USCOREgetWidth:
		soap_serialize_PointerTo_vbox__IDisplay_USCOREgetWidth(soap, (_vbox__IDisplay_USCOREgetWidth *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMouse_USCOREputMouseEventAbsoluteResponse:
		soap_serialize_PointerTo_vbox__IMouse_USCOREputMouseEventAbsoluteResponse(soap, (_vbox__IMouse_USCOREputMouseEventAbsoluteResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMouse_USCOREputMouseEventAbsolute:
		soap_serialize_PointerTo_vbox__IMouse_USCOREputMouseEventAbsolute(soap, (_vbox__IMouse_USCOREputMouseEventAbsolute *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMouse_USCOREputMouseEventResponse:
		soap_serialize_PointerTo_vbox__IMouse_USCOREputMouseEventResponse(soap, (_vbox__IMouse_USCOREputMouseEventResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMouse_USCOREputMouseEvent:
		soap_serialize_PointerTo_vbox__IMouse_USCOREputMouseEvent(soap, (_vbox__IMouse_USCOREputMouseEvent *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMouse_USCOREgetAbsoluteSupportedResponse:
		soap_serialize_PointerTo_vbox__IMouse_USCOREgetAbsoluteSupportedResponse(soap, (_vbox__IMouse_USCOREgetAbsoluteSupportedResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMouse_USCOREgetAbsoluteSupported:
		soap_serialize_PointerTo_vbox__IMouse_USCOREgetAbsoluteSupported(soap, (_vbox__IMouse_USCOREgetAbsoluteSupported *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IKeyboard_USCOREputCADResponse:
		soap_serialize_PointerTo_vbox__IKeyboard_USCOREputCADResponse(soap, (_vbox__IKeyboard_USCOREputCADResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IKeyboard_USCOREputCAD:
		soap_serialize_PointerTo_vbox__IKeyboard_USCOREputCAD(soap, (_vbox__IKeyboard_USCOREputCAD *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IKeyboard_USCOREputScancodesResponse:
		soap_serialize_PointerTo_vbox__IKeyboard_USCOREputScancodesResponse(soap, (_vbox__IKeyboard_USCOREputScancodesResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IKeyboard_USCOREputScancodes:
		soap_serialize_PointerTo_vbox__IKeyboard_USCOREputScancodes(soap, (_vbox__IKeyboard_USCOREputScancodes *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IKeyboard_USCOREputScancodeResponse:
		soap_serialize_PointerTo_vbox__IKeyboard_USCOREputScancodeResponse(soap, (_vbox__IKeyboard_USCOREputScancodeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IKeyboard_USCOREputScancode:
		soap_serialize_PointerTo_vbox__IKeyboard_USCOREputScancode(soap, (_vbox__IKeyboard_USCOREputScancode *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREgetHostDriveResponse:
		soap_serialize_PointerTo_vbox__IFloppyDrive_USCOREgetHostDriveResponse(soap, (_vbox__IFloppyDrive_USCOREgetHostDriveResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREgetHostDrive:
		soap_serialize_PointerTo_vbox__IFloppyDrive_USCOREgetHostDrive(soap, (_vbox__IFloppyDrive_USCOREgetHostDrive *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREgetImageResponse:
		soap_serialize_PointerTo_vbox__IFloppyDrive_USCOREgetImageResponse(soap, (_vbox__IFloppyDrive_USCOREgetImageResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREgetImage:
		soap_serialize_PointerTo_vbox__IFloppyDrive_USCOREgetImage(soap, (_vbox__IFloppyDrive_USCOREgetImage *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREunmountResponse:
		soap_serialize_PointerTo_vbox__IFloppyDrive_USCOREunmountResponse(soap, (_vbox__IFloppyDrive_USCOREunmountResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREunmount:
		soap_serialize_PointerTo_vbox__IFloppyDrive_USCOREunmount(soap, (_vbox__IFloppyDrive_USCOREunmount *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREcaptureHostDriveResponse:
		soap_serialize_PointerTo_vbox__IFloppyDrive_USCOREcaptureHostDriveResponse(soap, (_vbox__IFloppyDrive_USCOREcaptureHostDriveResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREcaptureHostDrive:
		soap_serialize_PointerTo_vbox__IFloppyDrive_USCOREcaptureHostDrive(soap, (_vbox__IFloppyDrive_USCOREcaptureHostDrive *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREmountImageResponse:
		soap_serialize_PointerTo_vbox__IFloppyDrive_USCOREmountImageResponse(soap, (_vbox__IFloppyDrive_USCOREmountImageResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREmountImage:
		soap_serialize_PointerTo_vbox__IFloppyDrive_USCOREmountImage(soap, (_vbox__IFloppyDrive_USCOREmountImage *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREgetStateResponse:
		soap_serialize_PointerTo_vbox__IFloppyDrive_USCOREgetStateResponse(soap, (_vbox__IFloppyDrive_USCOREgetStateResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREgetState:
		soap_serialize_PointerTo_vbox__IFloppyDrive_USCOREgetState(soap, (_vbox__IFloppyDrive_USCOREgetState *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREsetEnabledResponse:
		soap_serialize_PointerTo_vbox__IFloppyDrive_USCOREsetEnabledResponse(soap, (_vbox__IFloppyDrive_USCOREsetEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREsetEnabled:
		soap_serialize_PointerTo_vbox__IFloppyDrive_USCOREsetEnabled(soap, (_vbox__IFloppyDrive_USCOREsetEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREgetEnabledResponse:
		soap_serialize_PointerTo_vbox__IFloppyDrive_USCOREgetEnabledResponse(soap, (_vbox__IFloppyDrive_USCOREgetEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IFloppyDrive_USCOREgetEnabled:
		soap_serialize_PointerTo_vbox__IFloppyDrive_USCOREgetEnabled(soap, (_vbox__IFloppyDrive_USCOREgetEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREgetHostDriveResponse:
		soap_serialize_PointerTo_vbox__IDVDDrive_USCOREgetHostDriveResponse(soap, (_vbox__IDVDDrive_USCOREgetHostDriveResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREgetHostDrive:
		soap_serialize_PointerTo_vbox__IDVDDrive_USCOREgetHostDrive(soap, (_vbox__IDVDDrive_USCOREgetHostDrive *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREgetImageResponse:
		soap_serialize_PointerTo_vbox__IDVDDrive_USCOREgetImageResponse(soap, (_vbox__IDVDDrive_USCOREgetImageResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREgetImage:
		soap_serialize_PointerTo_vbox__IDVDDrive_USCOREgetImage(soap, (_vbox__IDVDDrive_USCOREgetImage *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREunmountResponse:
		soap_serialize_PointerTo_vbox__IDVDDrive_USCOREunmountResponse(soap, (_vbox__IDVDDrive_USCOREunmountResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREunmount:
		soap_serialize_PointerTo_vbox__IDVDDrive_USCOREunmount(soap, (_vbox__IDVDDrive_USCOREunmount *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREcaptureHostDriveResponse:
		soap_serialize_PointerTo_vbox__IDVDDrive_USCOREcaptureHostDriveResponse(soap, (_vbox__IDVDDrive_USCOREcaptureHostDriveResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREcaptureHostDrive:
		soap_serialize_PointerTo_vbox__IDVDDrive_USCOREcaptureHostDrive(soap, (_vbox__IDVDDrive_USCOREcaptureHostDrive *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREmountImageResponse:
		soap_serialize_PointerTo_vbox__IDVDDrive_USCOREmountImageResponse(soap, (_vbox__IDVDDrive_USCOREmountImageResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREmountImage:
		soap_serialize_PointerTo_vbox__IDVDDrive_USCOREmountImage(soap, (_vbox__IDVDDrive_USCOREmountImage *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREsetPassthroughResponse:
		soap_serialize_PointerTo_vbox__IDVDDrive_USCOREsetPassthroughResponse(soap, (_vbox__IDVDDrive_USCOREsetPassthroughResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREsetPassthrough:
		soap_serialize_PointerTo_vbox__IDVDDrive_USCOREsetPassthrough(soap, (_vbox__IDVDDrive_USCOREsetPassthrough *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREgetPassthroughResponse:
		soap_serialize_PointerTo_vbox__IDVDDrive_USCOREgetPassthroughResponse(soap, (_vbox__IDVDDrive_USCOREgetPassthroughResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREgetPassthrough:
		soap_serialize_PointerTo_vbox__IDVDDrive_USCOREgetPassthrough(soap, (_vbox__IDVDDrive_USCOREgetPassthrough *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREgetStateResponse:
		soap_serialize_PointerTo_vbox__IDVDDrive_USCOREgetStateResponse(soap, (_vbox__IDVDDrive_USCOREgetStateResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDVDDrive_USCOREgetState:
		soap_serialize_PointerTo_vbox__IDVDDrive_USCOREgetState(soap, (_vbox__IDVDDrive_USCOREgetState *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDiskFormat_USCOREdescribePropertiesResponse:
		soap_serialize_PointerTo_vbox__IHardDiskFormat_USCOREdescribePropertiesResponse(soap, (_vbox__IHardDiskFormat_USCOREdescribePropertiesResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDiskFormat_USCOREdescribeProperties:
		soap_serialize_PointerTo_vbox__IHardDiskFormat_USCOREdescribeProperties(soap, (_vbox__IHardDiskFormat_USCOREdescribeProperties *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDiskFormat_USCOREgetCapabilitiesResponse:
		soap_serialize_PointerTo_vbox__IHardDiskFormat_USCOREgetCapabilitiesResponse(soap, (_vbox__IHardDiskFormat_USCOREgetCapabilitiesResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDiskFormat_USCOREgetCapabilities:
		soap_serialize_PointerTo_vbox__IHardDiskFormat_USCOREgetCapabilities(soap, (_vbox__IHardDiskFormat_USCOREgetCapabilities *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDiskFormat_USCOREgetFileExtensionsResponse:
		soap_serialize_PointerTo_vbox__IHardDiskFormat_USCOREgetFileExtensionsResponse(soap, (_vbox__IHardDiskFormat_USCOREgetFileExtensionsResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDiskFormat_USCOREgetFileExtensions:
		soap_serialize_PointerTo_vbox__IHardDiskFormat_USCOREgetFileExtensions(soap, (_vbox__IHardDiskFormat_USCOREgetFileExtensions *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDiskFormat_USCOREgetNameResponse:
		soap_serialize_PointerTo_vbox__IHardDiskFormat_USCOREgetNameResponse(soap, (_vbox__IHardDiskFormat_USCOREgetNameResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDiskFormat_USCOREgetName:
		soap_serialize_PointerTo_vbox__IHardDiskFormat_USCOREgetName(soap, (_vbox__IHardDiskFormat_USCOREgetName *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDiskFormat_USCOREgetIdResponse:
		soap_serialize_PointerTo_vbox__IHardDiskFormat_USCOREgetIdResponse(soap, (_vbox__IHardDiskFormat_USCOREgetIdResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDiskFormat_USCOREgetId:
		soap_serialize_PointerTo_vbox__IHardDiskFormat_USCOREgetId(soap, (_vbox__IHardDiskFormat_USCOREgetId *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREresetResponse:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREresetResponse(soap, (_vbox__IHardDisk_USCOREresetResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREreset:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREreset(soap, (_vbox__IHardDisk_USCOREreset *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREcompactResponse:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREcompactResponse(soap, (_vbox__IHardDisk_USCOREcompactResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREcompact:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREcompact(soap, (_vbox__IHardDisk_USCOREcompact *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREcloneToResponse:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREcloneToResponse(soap, (_vbox__IHardDisk_USCOREcloneToResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREcloneTo:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREcloneTo(soap, (_vbox__IHardDisk_USCOREcloneTo *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREmergeToResponse:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREmergeToResponse(soap, (_vbox__IHardDisk_USCOREmergeToResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREmergeTo:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREmergeTo(soap, (_vbox__IHardDisk_USCOREmergeTo *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREcreateDiffStorageResponse:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREcreateDiffStorageResponse(soap, (_vbox__IHardDisk_USCOREcreateDiffStorageResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREcreateDiffStorage:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREcreateDiffStorage(soap, (_vbox__IHardDisk_USCOREcreateDiffStorage *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREdeleteStorageResponse:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREdeleteStorageResponse(soap, (_vbox__IHardDisk_USCOREdeleteStorageResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREdeleteStorage:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREdeleteStorage(soap, (_vbox__IHardDisk_USCOREdeleteStorage *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREcreateBaseStorageResponse:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREcreateBaseStorageResponse(soap, (_vbox__IHardDisk_USCOREcreateBaseStorageResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREcreateBaseStorage:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREcreateBaseStorage(soap, (_vbox__IHardDisk_USCOREcreateBaseStorage *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREsetPropertiesResponse:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREsetPropertiesResponse(soap, (_vbox__IHardDisk_USCOREsetPropertiesResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREsetProperties:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREsetProperties(soap, (_vbox__IHardDisk_USCOREsetProperties *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetPropertiesResponse:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREgetPropertiesResponse(soap, (_vbox__IHardDisk_USCOREgetPropertiesResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetProperties:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREgetProperties(soap, (_vbox__IHardDisk_USCOREgetProperties *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREsetPropertyResponse:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREsetPropertyResponse(soap, (_vbox__IHardDisk_USCOREsetPropertyResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREsetProperty:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREsetProperty(soap, (_vbox__IHardDisk_USCOREsetProperty *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetPropertyResponse:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREgetPropertyResponse(soap, (_vbox__IHardDisk_USCOREgetPropertyResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetProperty:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREgetProperty(soap, (_vbox__IHardDisk_USCOREgetProperty *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREsetAutoResetResponse:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREsetAutoResetResponse(soap, (_vbox__IHardDisk_USCOREsetAutoResetResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREsetAutoReset:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREsetAutoReset(soap, (_vbox__IHardDisk_USCOREsetAutoReset *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetAutoResetResponse:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREgetAutoResetResponse(soap, (_vbox__IHardDisk_USCOREgetAutoResetResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetAutoReset:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREgetAutoReset(soap, (_vbox__IHardDisk_USCOREgetAutoReset *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetLogicalSizeResponse:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREgetLogicalSizeResponse(soap, (_vbox__IHardDisk_USCOREgetLogicalSizeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetLogicalSize:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREgetLogicalSize(soap, (_vbox__IHardDisk_USCOREgetLogicalSize *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetReadOnlyResponse:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREgetReadOnlyResponse(soap, (_vbox__IHardDisk_USCOREgetReadOnlyResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetReadOnly:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREgetReadOnly(soap, (_vbox__IHardDisk_USCOREgetReadOnly *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetRootResponse:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREgetRootResponse(soap, (_vbox__IHardDisk_USCOREgetRootResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetRoot:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREgetRoot(soap, (_vbox__IHardDisk_USCOREgetRoot *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetChildrenResponse:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREgetChildrenResponse(soap, (_vbox__IHardDisk_USCOREgetChildrenResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetChildren:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREgetChildren(soap, (_vbox__IHardDisk_USCOREgetChildren *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetParentResponse:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREgetParentResponse(soap, (_vbox__IHardDisk_USCOREgetParentResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetParent:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREgetParent(soap, (_vbox__IHardDisk_USCOREgetParent *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREsetTypeResponse:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREsetTypeResponse(soap, (_vbox__IHardDisk_USCOREsetTypeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREsetType:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREsetType(soap, (_vbox__IHardDisk_USCOREsetType *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetTypeResponse:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREgetTypeResponse(soap, (_vbox__IHardDisk_USCOREgetTypeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetType:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREgetType(soap, (_vbox__IHardDisk_USCOREgetType *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetFormatResponse:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREgetFormatResponse(soap, (_vbox__IHardDisk_USCOREgetFormatResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHardDisk_USCOREgetFormat:
		soap_serialize_PointerTo_vbox__IHardDisk_USCOREgetFormat(soap, (_vbox__IHardDisk_USCOREgetFormat *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREcloseResponse:
		soap_serialize_PointerTo_vbox__IMedium_USCOREcloseResponse(soap, (_vbox__IMedium_USCOREcloseResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREclose:
		soap_serialize_PointerTo_vbox__IMedium_USCOREclose(soap, (_vbox__IMedium_USCOREclose *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREunlockWriteResponse:
		soap_serialize_PointerTo_vbox__IMedium_USCOREunlockWriteResponse(soap, (_vbox__IMedium_USCOREunlockWriteResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREunlockWrite:
		soap_serialize_PointerTo_vbox__IMedium_USCOREunlockWrite(soap, (_vbox__IMedium_USCOREunlockWrite *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCORElockWriteResponse:
		soap_serialize_PointerTo_vbox__IMedium_USCORElockWriteResponse(soap, (_vbox__IMedium_USCORElockWriteResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCORElockWrite:
		soap_serialize_PointerTo_vbox__IMedium_USCORElockWrite(soap, (_vbox__IMedium_USCORElockWrite *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREunlockReadResponse:
		soap_serialize_PointerTo_vbox__IMedium_USCOREunlockReadResponse(soap, (_vbox__IMedium_USCOREunlockReadResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREunlockRead:
		soap_serialize_PointerTo_vbox__IMedium_USCOREunlockRead(soap, (_vbox__IMedium_USCOREunlockRead *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCORElockReadResponse:
		soap_serialize_PointerTo_vbox__IMedium_USCORElockReadResponse(soap, (_vbox__IMedium_USCORElockReadResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCORElockRead:
		soap_serialize_PointerTo_vbox__IMedium_USCORElockRead(soap, (_vbox__IMedium_USCORElockRead *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetSnapshotIdsResponse:
		soap_serialize_PointerTo_vbox__IMedium_USCOREgetSnapshotIdsResponse(soap, (_vbox__IMedium_USCOREgetSnapshotIdsResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetSnapshotIds:
		soap_serialize_PointerTo_vbox__IMedium_USCOREgetSnapshotIds(soap, (_vbox__IMedium_USCOREgetSnapshotIds *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetMachineIdsResponse:
		soap_serialize_PointerTo_vbox__IMedium_USCOREgetMachineIdsResponse(soap, (_vbox__IMedium_USCOREgetMachineIdsResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetMachineIds:
		soap_serialize_PointerTo_vbox__IMedium_USCOREgetMachineIds(soap, (_vbox__IMedium_USCOREgetMachineIds *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetLastAccessErrorResponse:
		soap_serialize_PointerTo_vbox__IMedium_USCOREgetLastAccessErrorResponse(soap, (_vbox__IMedium_USCOREgetLastAccessErrorResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetLastAccessError:
		soap_serialize_PointerTo_vbox__IMedium_USCOREgetLastAccessError(soap, (_vbox__IMedium_USCOREgetLastAccessError *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetSizeResponse:
		soap_serialize_PointerTo_vbox__IMedium_USCOREgetSizeResponse(soap, (_vbox__IMedium_USCOREgetSizeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetSize:
		soap_serialize_PointerTo_vbox__IMedium_USCOREgetSize(soap, (_vbox__IMedium_USCOREgetSize *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetNameResponse:
		soap_serialize_PointerTo_vbox__IMedium_USCOREgetNameResponse(soap, (_vbox__IMedium_USCOREgetNameResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetName:
		soap_serialize_PointerTo_vbox__IMedium_USCOREgetName(soap, (_vbox__IMedium_USCOREgetName *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREsetLocationResponse:
		soap_serialize_PointerTo_vbox__IMedium_USCOREsetLocationResponse(soap, (_vbox__IMedium_USCOREsetLocationResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREsetLocation:
		soap_serialize_PointerTo_vbox__IMedium_USCOREsetLocation(soap, (_vbox__IMedium_USCOREsetLocation *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetLocationResponse:
		soap_serialize_PointerTo_vbox__IMedium_USCOREgetLocationResponse(soap, (_vbox__IMedium_USCOREgetLocationResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetLocation:
		soap_serialize_PointerTo_vbox__IMedium_USCOREgetLocation(soap, (_vbox__IMedium_USCOREgetLocation *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetStateResponse:
		soap_serialize_PointerTo_vbox__IMedium_USCOREgetStateResponse(soap, (_vbox__IMedium_USCOREgetStateResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetState:
		soap_serialize_PointerTo_vbox__IMedium_USCOREgetState(soap, (_vbox__IMedium_USCOREgetState *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREsetDescriptionResponse:
		soap_serialize_PointerTo_vbox__IMedium_USCOREsetDescriptionResponse(soap, (_vbox__IMedium_USCOREsetDescriptionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREsetDescription:
		soap_serialize_PointerTo_vbox__IMedium_USCOREsetDescription(soap, (_vbox__IMedium_USCOREsetDescription *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetDescriptionResponse:
		soap_serialize_PointerTo_vbox__IMedium_USCOREgetDescriptionResponse(soap, (_vbox__IMedium_USCOREgetDescriptionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetDescription:
		soap_serialize_PointerTo_vbox__IMedium_USCOREgetDescription(soap, (_vbox__IMedium_USCOREgetDescription *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetIdResponse:
		soap_serialize_PointerTo_vbox__IMedium_USCOREgetIdResponse(soap, (_vbox__IMedium_USCOREgetIdResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMedium_USCOREgetId:
		soap_serialize_PointerTo_vbox__IMedium_USCOREgetId(soap, (_vbox__IMedium_USCOREgetId *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetChildrenResponse:
		soap_serialize_PointerTo_vbox__ISnapshot_USCOREgetChildrenResponse(soap, (_vbox__ISnapshot_USCOREgetChildrenResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetChildren:
		soap_serialize_PointerTo_vbox__ISnapshot_USCOREgetChildren(soap, (_vbox__ISnapshot_USCOREgetChildren *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetParentResponse:
		soap_serialize_PointerTo_vbox__ISnapshot_USCOREgetParentResponse(soap, (_vbox__ISnapshot_USCOREgetParentResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetParent:
		soap_serialize_PointerTo_vbox__ISnapshot_USCOREgetParent(soap, (_vbox__ISnapshot_USCOREgetParent *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetMachineResponse:
		soap_serialize_PointerTo_vbox__ISnapshot_USCOREgetMachineResponse(soap, (_vbox__ISnapshot_USCOREgetMachineResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetMachine:
		soap_serialize_PointerTo_vbox__ISnapshot_USCOREgetMachine(soap, (_vbox__ISnapshot_USCOREgetMachine *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetOnlineResponse:
		soap_serialize_PointerTo_vbox__ISnapshot_USCOREgetOnlineResponse(soap, (_vbox__ISnapshot_USCOREgetOnlineResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetOnline:
		soap_serialize_PointerTo_vbox__ISnapshot_USCOREgetOnline(soap, (_vbox__ISnapshot_USCOREgetOnline *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetTimeStampResponse:
		soap_serialize_PointerTo_vbox__ISnapshot_USCOREgetTimeStampResponse(soap, (_vbox__ISnapshot_USCOREgetTimeStampResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetTimeStamp:
		soap_serialize_PointerTo_vbox__ISnapshot_USCOREgetTimeStamp(soap, (_vbox__ISnapshot_USCOREgetTimeStamp *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREsetDescriptionResponse:
		soap_serialize_PointerTo_vbox__ISnapshot_USCOREsetDescriptionResponse(soap, (_vbox__ISnapshot_USCOREsetDescriptionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREsetDescription:
		soap_serialize_PointerTo_vbox__ISnapshot_USCOREsetDescription(soap, (_vbox__ISnapshot_USCOREsetDescription *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetDescriptionResponse:
		soap_serialize_PointerTo_vbox__ISnapshot_USCOREgetDescriptionResponse(soap, (_vbox__ISnapshot_USCOREgetDescriptionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetDescription:
		soap_serialize_PointerTo_vbox__ISnapshot_USCOREgetDescription(soap, (_vbox__ISnapshot_USCOREgetDescription *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREsetNameResponse:
		soap_serialize_PointerTo_vbox__ISnapshot_USCOREsetNameResponse(soap, (_vbox__ISnapshot_USCOREsetNameResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREsetName:
		soap_serialize_PointerTo_vbox__ISnapshot_USCOREsetName(soap, (_vbox__ISnapshot_USCOREsetName *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetNameResponse:
		soap_serialize_PointerTo_vbox__ISnapshot_USCOREgetNameResponse(soap, (_vbox__ISnapshot_USCOREgetNameResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetName:
		soap_serialize_PointerTo_vbox__ISnapshot_USCOREgetName(soap, (_vbox__ISnapshot_USCOREgetName *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetIdResponse:
		soap_serialize_PointerTo_vbox__ISnapshot_USCOREgetIdResponse(soap, (_vbox__ISnapshot_USCOREgetIdResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISnapshot_USCOREgetId:
		soap_serialize_PointerTo_vbox__ISnapshot_USCOREgetId(soap, (_vbox__ISnapshot_USCOREgetId *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREcancelResponse:
		soap_serialize_PointerTo_vbox__IProgress_USCOREcancelResponse(soap, (_vbox__IProgress_USCOREcancelResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREcancel:
		soap_serialize_PointerTo_vbox__IProgress_USCOREcancel(soap, (_vbox__IProgress_USCOREcancel *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREwaitForOperationCompletionResponse:
		soap_serialize_PointerTo_vbox__IProgress_USCOREwaitForOperationCompletionResponse(soap, (_vbox__IProgress_USCOREwaitForOperationCompletionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREwaitForOperationCompletion:
		soap_serialize_PointerTo_vbox__IProgress_USCOREwaitForOperationCompletion(soap, (_vbox__IProgress_USCOREwaitForOperationCompletion *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREwaitForCompletionResponse:
		soap_serialize_PointerTo_vbox__IProgress_USCOREwaitForCompletionResponse(soap, (_vbox__IProgress_USCOREwaitForCompletionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREwaitForCompletion:
		soap_serialize_PointerTo_vbox__IProgress_USCOREwaitForCompletion(soap, (_vbox__IProgress_USCOREwaitForCompletion *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetOperationPercentResponse:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetOperationPercentResponse(soap, (_vbox__IProgress_USCOREgetOperationPercentResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetOperationPercent:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetOperationPercent(soap, (_vbox__IProgress_USCOREgetOperationPercent *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetOperationDescriptionResponse:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetOperationDescriptionResponse(soap, (_vbox__IProgress_USCOREgetOperationDescriptionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetOperationDescription:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetOperationDescription(soap, (_vbox__IProgress_USCOREgetOperationDescription *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetOperationResponse:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetOperationResponse(soap, (_vbox__IProgress_USCOREgetOperationResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetOperation:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetOperation(soap, (_vbox__IProgress_USCOREgetOperation *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetOperationCountResponse:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetOperationCountResponse(soap, (_vbox__IProgress_USCOREgetOperationCountResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetOperationCount:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetOperationCount(soap, (_vbox__IProgress_USCOREgetOperationCount *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetErrorInfoResponse:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetErrorInfoResponse(soap, (_vbox__IProgress_USCOREgetErrorInfoResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetErrorInfo:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetErrorInfo(soap, (_vbox__IProgress_USCOREgetErrorInfo *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetResultCodeResponse:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetResultCodeResponse(soap, (_vbox__IProgress_USCOREgetResultCodeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetResultCode:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetResultCode(soap, (_vbox__IProgress_USCOREgetResultCode *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetCanceledResponse:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetCanceledResponse(soap, (_vbox__IProgress_USCOREgetCanceledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetCanceled:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetCanceled(soap, (_vbox__IProgress_USCOREgetCanceled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetCompletedResponse:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetCompletedResponse(soap, (_vbox__IProgress_USCOREgetCompletedResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetCompleted:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetCompleted(soap, (_vbox__IProgress_USCOREgetCompleted *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetTimeRemainingResponse:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetTimeRemainingResponse(soap, (_vbox__IProgress_USCOREgetTimeRemainingResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetTimeRemaining:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetTimeRemaining(soap, (_vbox__IProgress_USCOREgetTimeRemaining *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetPercentResponse:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetPercentResponse(soap, (_vbox__IProgress_USCOREgetPercentResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetPercent:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetPercent(soap, (_vbox__IProgress_USCOREgetPercent *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetCancelableResponse:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetCancelableResponse(soap, (_vbox__IProgress_USCOREgetCancelableResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetCancelable:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetCancelable(soap, (_vbox__IProgress_USCOREgetCancelable *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetInitiatorResponse:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetInitiatorResponse(soap, (_vbox__IProgress_USCOREgetInitiatorResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetInitiator:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetInitiator(soap, (_vbox__IProgress_USCOREgetInitiator *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetDescriptionResponse:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetDescriptionResponse(soap, (_vbox__IProgress_USCOREgetDescriptionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetDescription:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetDescription(soap, (_vbox__IProgress_USCOREgetDescription *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetIdResponse:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetIdResponse(soap, (_vbox__IProgress_USCOREgetIdResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IProgress_USCOREgetId:
		soap_serialize_PointerTo_vbox__IProgress_USCOREgetId(soap, (_vbox__IProgress_USCOREgetId *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetStatisticResponse:
		soap_serialize_PointerTo_vbox__IGuest_USCOREgetStatisticResponse(soap, (_vbox__IGuest_USCOREgetStatisticResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetStatistic:
		soap_serialize_PointerTo_vbox__IGuest_USCOREgetStatistic(soap, (_vbox__IGuest_USCOREgetStatistic *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREsetCredentialsResponse:
		soap_serialize_PointerTo_vbox__IGuest_USCOREsetCredentialsResponse(soap, (_vbox__IGuest_USCOREsetCredentialsResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREsetCredentials:
		soap_serialize_PointerTo_vbox__IGuest_USCOREsetCredentials(soap, (_vbox__IGuest_USCOREsetCredentials *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREsetStatisticsUpdateIntervalResponse:
		soap_serialize_PointerTo_vbox__IGuest_USCOREsetStatisticsUpdateIntervalResponse(soap, (_vbox__IGuest_USCOREsetStatisticsUpdateIntervalResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREsetStatisticsUpdateInterval:
		soap_serialize_PointerTo_vbox__IGuest_USCOREsetStatisticsUpdateInterval(soap, (_vbox__IGuest_USCOREsetStatisticsUpdateInterval *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetStatisticsUpdateIntervalResponse:
		soap_serialize_PointerTo_vbox__IGuest_USCOREgetStatisticsUpdateIntervalResponse(soap, (_vbox__IGuest_USCOREgetStatisticsUpdateIntervalResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetStatisticsUpdateInterval:
		soap_serialize_PointerTo_vbox__IGuest_USCOREgetStatisticsUpdateInterval(soap, (_vbox__IGuest_USCOREgetStatisticsUpdateInterval *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREsetMemoryBalloonSizeResponse:
		soap_serialize_PointerTo_vbox__IGuest_USCOREsetMemoryBalloonSizeResponse(soap, (_vbox__IGuest_USCOREsetMemoryBalloonSizeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREsetMemoryBalloonSize:
		soap_serialize_PointerTo_vbox__IGuest_USCOREsetMemoryBalloonSize(soap, (_vbox__IGuest_USCOREsetMemoryBalloonSize *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetMemoryBalloonSizeResponse:
		soap_serialize_PointerTo_vbox__IGuest_USCOREgetMemoryBalloonSizeResponse(soap, (_vbox__IGuest_USCOREgetMemoryBalloonSizeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetMemoryBalloonSize:
		soap_serialize_PointerTo_vbox__IGuest_USCOREgetMemoryBalloonSize(soap, (_vbox__IGuest_USCOREgetMemoryBalloonSize *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetSupportsGraphicsResponse:
		soap_serialize_PointerTo_vbox__IGuest_USCOREgetSupportsGraphicsResponse(soap, (_vbox__IGuest_USCOREgetSupportsGraphicsResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetSupportsGraphics:
		soap_serialize_PointerTo_vbox__IGuest_USCOREgetSupportsGraphics(soap, (_vbox__IGuest_USCOREgetSupportsGraphics *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetSupportsSeamlessResponse:
		soap_serialize_PointerTo_vbox__IGuest_USCOREgetSupportsSeamlessResponse(soap, (_vbox__IGuest_USCOREgetSupportsSeamlessResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetSupportsSeamless:
		soap_serialize_PointerTo_vbox__IGuest_USCOREgetSupportsSeamless(soap, (_vbox__IGuest_USCOREgetSupportsSeamless *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetAdditionsVersionResponse:
		soap_serialize_PointerTo_vbox__IGuest_USCOREgetAdditionsVersionResponse(soap, (_vbox__IGuest_USCOREgetAdditionsVersionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetAdditionsVersion:
		soap_serialize_PointerTo_vbox__IGuest_USCOREgetAdditionsVersion(soap, (_vbox__IGuest_USCOREgetAdditionsVersion *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetAdditionsActiveResponse:
		soap_serialize_PointerTo_vbox__IGuest_USCOREgetAdditionsActiveResponse(soap, (_vbox__IGuest_USCOREgetAdditionsActiveResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetAdditionsActive:
		soap_serialize_PointerTo_vbox__IGuest_USCOREgetAdditionsActive(soap, (_vbox__IGuest_USCOREgetAdditionsActive *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetOSTypeIdResponse:
		soap_serialize_PointerTo_vbox__IGuest_USCOREgetOSTypeIdResponse(soap, (_vbox__IGuest_USCOREgetOSTypeIdResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IGuest_USCOREgetOSTypeId:
		soap_serialize_PointerTo_vbox__IGuest_USCOREgetOSTypeId(soap, (_vbox__IGuest_USCOREgetOSTypeId *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetDefaultAudioDriverResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetDefaultAudioDriverResponse(soap, (_vbox__ISystemProperties_USCOREgetDefaultAudioDriverResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetDefaultAudioDriver:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetDefaultAudioDriver(soap, (_vbox__ISystemProperties_USCOREgetDefaultAudioDriver *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREsetLogHistoryCountResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREsetLogHistoryCountResponse(soap, (_vbox__ISystemProperties_USCOREsetLogHistoryCountResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREsetLogHistoryCount:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREsetLogHistoryCount(soap, (_vbox__ISystemProperties_USCOREsetLogHistoryCount *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetLogHistoryCountResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetLogHistoryCountResponse(soap, (_vbox__ISystemProperties_USCOREgetLogHistoryCountResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetLogHistoryCount:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetLogHistoryCount(soap, (_vbox__ISystemProperties_USCOREgetLogHistoryCount *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREsetWebServiceAuthLibraryResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREsetWebServiceAuthLibraryResponse(soap, (_vbox__ISystemProperties_USCOREsetWebServiceAuthLibraryResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREsetWebServiceAuthLibrary:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREsetWebServiceAuthLibrary(soap, (_vbox__ISystemProperties_USCOREsetWebServiceAuthLibrary *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetWebServiceAuthLibraryResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetWebServiceAuthLibraryResponse(soap, (_vbox__ISystemProperties_USCOREgetWebServiceAuthLibraryResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetWebServiceAuthLibrary:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetWebServiceAuthLibrary(soap, (_vbox__ISystemProperties_USCOREgetWebServiceAuthLibrary *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibraryResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibraryResponse(soap, (_vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibraryResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibrary:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibrary(soap, (_vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibrary *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibraryResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibraryResponse(soap, (_vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibraryResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibrary:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibrary(soap, (_vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibrary *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREsetDefaultHardDiskFormatResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREsetDefaultHardDiskFormatResponse(soap, (_vbox__ISystemProperties_USCOREsetDefaultHardDiskFormatResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREsetDefaultHardDiskFormat:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREsetDefaultHardDiskFormat(soap, (_vbox__ISystemProperties_USCOREsetDefaultHardDiskFormat *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetDefaultHardDiskFormatResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetDefaultHardDiskFormatResponse(soap, (_vbox__ISystemProperties_USCOREgetDefaultHardDiskFormatResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetDefaultHardDiskFormat:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetDefaultHardDiskFormat(soap, (_vbox__ISystemProperties_USCOREgetDefaultHardDiskFormat *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetHardDiskFormatsResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetHardDiskFormatsResponse(soap, (_vbox__ISystemProperties_USCOREgetHardDiskFormatsResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetHardDiskFormats:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetHardDiskFormats(soap, (_vbox__ISystemProperties_USCOREgetHardDiskFormats *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREsetDefaultHardDiskFolderResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREsetDefaultHardDiskFolderResponse(soap, (_vbox__ISystemProperties_USCOREsetDefaultHardDiskFolderResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREsetDefaultHardDiskFolder:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREsetDefaultHardDiskFolder(soap, (_vbox__ISystemProperties_USCOREsetDefaultHardDiskFolder *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetDefaultHardDiskFolderResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetDefaultHardDiskFolderResponse(soap, (_vbox__ISystemProperties_USCOREgetDefaultHardDiskFolderResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetDefaultHardDiskFolder:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetDefaultHardDiskFolder(soap, (_vbox__ISystemProperties_USCOREgetDefaultHardDiskFolder *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREsetDefaultMachineFolderResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREsetDefaultMachineFolderResponse(soap, (_vbox__ISystemProperties_USCOREsetDefaultMachineFolderResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREsetDefaultMachineFolder:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREsetDefaultMachineFolder(soap, (_vbox__ISystemProperties_USCOREsetDefaultMachineFolder *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetDefaultMachineFolderResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetDefaultMachineFolderResponse(soap, (_vbox__ISystemProperties_USCOREgetDefaultMachineFolderResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetDefaultMachineFolder:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetDefaultMachineFolder(soap, (_vbox__ISystemProperties_USCOREgetDefaultMachineFolder *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMaxBootPositionResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetMaxBootPositionResponse(soap, (_vbox__ISystemProperties_USCOREgetMaxBootPositionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMaxBootPosition:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetMaxBootPosition(soap, (_vbox__ISystemProperties_USCOREgetMaxBootPosition *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetParallelPortCountResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetParallelPortCountResponse(soap, (_vbox__ISystemProperties_USCOREgetParallelPortCountResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetParallelPortCount:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetParallelPortCount(soap, (_vbox__ISystemProperties_USCOREgetParallelPortCount *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetSerialPortCountResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetSerialPortCountResponse(soap, (_vbox__ISystemProperties_USCOREgetSerialPortCountResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetSerialPortCount:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetSerialPortCount(soap, (_vbox__ISystemProperties_USCOREgetSerialPortCount *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetNetworkAdapterCountResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetNetworkAdapterCountResponse(soap, (_vbox__ISystemProperties_USCOREgetNetworkAdapterCountResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetNetworkAdapterCount:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetNetworkAdapterCount(soap, (_vbox__ISystemProperties_USCOREgetNetworkAdapterCount *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMaxVDISizeResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetMaxVDISizeResponse(soap, (_vbox__ISystemProperties_USCOREgetMaxVDISizeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMaxVDISize:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetMaxVDISize(soap, (_vbox__ISystemProperties_USCOREgetMaxVDISize *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMaxGuestCPUCountResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetMaxGuestCPUCountResponse(soap, (_vbox__ISystemProperties_USCOREgetMaxGuestCPUCountResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMaxGuestCPUCount:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetMaxGuestCPUCount(soap, (_vbox__ISystemProperties_USCOREgetMaxGuestCPUCount *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMinGuestCPUCountResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetMinGuestCPUCountResponse(soap, (_vbox__ISystemProperties_USCOREgetMinGuestCPUCountResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMinGuestCPUCount:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetMinGuestCPUCount(soap, (_vbox__ISystemProperties_USCOREgetMinGuestCPUCount *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMaxGuestVRAMResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetMaxGuestVRAMResponse(soap, (_vbox__ISystemProperties_USCOREgetMaxGuestVRAMResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMaxGuestVRAM:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetMaxGuestVRAM(soap, (_vbox__ISystemProperties_USCOREgetMaxGuestVRAM *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMinGuestVRAMResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetMinGuestVRAMResponse(soap, (_vbox__ISystemProperties_USCOREgetMinGuestVRAMResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMinGuestVRAM:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetMinGuestVRAM(soap, (_vbox__ISystemProperties_USCOREgetMinGuestVRAM *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMaxGuestRAMResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetMaxGuestRAMResponse(soap, (_vbox__ISystemProperties_USCOREgetMaxGuestRAMResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMaxGuestRAM:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetMaxGuestRAM(soap, (_vbox__ISystemProperties_USCOREgetMaxGuestRAM *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMinGuestRAMResponse:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetMinGuestRAMResponse(soap, (_vbox__ISystemProperties_USCOREgetMinGuestRAMResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__ISystemProperties_USCOREgetMinGuestRAM:
		soap_serialize_PointerTo_vbox__ISystemProperties_USCOREgetMinGuestRAM(soap, (_vbox__ISystemProperties_USCOREgetMinGuestRAM *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindUSBDeviceByAddressResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREfindUSBDeviceByAddressResponse(soap, (_vbox__IHost_USCOREfindUSBDeviceByAddressResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindUSBDeviceByAddress:
		soap_serialize_PointerTo_vbox__IHost_USCOREfindUSBDeviceByAddress(soap, (_vbox__IHost_USCOREfindUSBDeviceByAddress *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindUSBDeviceByIdResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREfindUSBDeviceByIdResponse(soap, (_vbox__IHost_USCOREfindUSBDeviceByIdResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindUSBDeviceById:
		soap_serialize_PointerTo_vbox__IHost_USCOREfindUSBDeviceById(soap, (_vbox__IHost_USCOREfindUSBDeviceById *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindHostNetworkInterfacesOfTypeResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREfindHostNetworkInterfacesOfTypeResponse(soap, (_vbox__IHost_USCOREfindHostNetworkInterfacesOfTypeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindHostNetworkInterfacesOfType:
		soap_serialize_PointerTo_vbox__IHost_USCOREfindHostNetworkInterfacesOfType(soap, (_vbox__IHost_USCOREfindHostNetworkInterfacesOfType *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindHostNetworkInterfaceByIdResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREfindHostNetworkInterfaceByIdResponse(soap, (_vbox__IHost_USCOREfindHostNetworkInterfaceByIdResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindHostNetworkInterfaceById:
		soap_serialize_PointerTo_vbox__IHost_USCOREfindHostNetworkInterfaceById(soap, (_vbox__IHost_USCOREfindHostNetworkInterfaceById *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindHostNetworkInterfaceByNameResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREfindHostNetworkInterfaceByNameResponse(soap, (_vbox__IHost_USCOREfindHostNetworkInterfaceByNameResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindHostNetworkInterfaceByName:
		soap_serialize_PointerTo_vbox__IHost_USCOREfindHostNetworkInterfaceByName(soap, (_vbox__IHost_USCOREfindHostNetworkInterfaceByName *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindHostFloppyDriveResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREfindHostFloppyDriveResponse(soap, (_vbox__IHost_USCOREfindHostFloppyDriveResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindHostFloppyDrive:
		soap_serialize_PointerTo_vbox__IHost_USCOREfindHostFloppyDrive(soap, (_vbox__IHost_USCOREfindHostFloppyDrive *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindHostDVDDriveResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREfindHostDVDDriveResponse(soap, (_vbox__IHost_USCOREfindHostDVDDriveResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREfindHostDVDDrive:
		soap_serialize_PointerTo_vbox__IHost_USCOREfindHostDVDDrive(soap, (_vbox__IHost_USCOREfindHostDVDDrive *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREremoveUSBDeviceFilterResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREremoveUSBDeviceFilterResponse(soap, (_vbox__IHost_USCOREremoveUSBDeviceFilterResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREremoveUSBDeviceFilter:
		soap_serialize_PointerTo_vbox__IHost_USCOREremoveUSBDeviceFilter(soap, (_vbox__IHost_USCOREremoveUSBDeviceFilter *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREinsertUSBDeviceFilterResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREinsertUSBDeviceFilterResponse(soap, (_vbox__IHost_USCOREinsertUSBDeviceFilterResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREinsertUSBDeviceFilter:
		soap_serialize_PointerTo_vbox__IHost_USCOREinsertUSBDeviceFilter(soap, (_vbox__IHost_USCOREinsertUSBDeviceFilter *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREcreateUSBDeviceFilterResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREcreateUSBDeviceFilterResponse(soap, (_vbox__IHost_USCOREcreateUSBDeviceFilterResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREcreateUSBDeviceFilter:
		soap_serialize_PointerTo_vbox__IHost_USCOREcreateUSBDeviceFilter(soap, (_vbox__IHost_USCOREcreateUSBDeviceFilter *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREremoveHostOnlyNetworkInterfaceResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREremoveHostOnlyNetworkInterfaceResponse(soap, (_vbox__IHost_USCOREremoveHostOnlyNetworkInterfaceResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREremoveHostOnlyNetworkInterface:
		soap_serialize_PointerTo_vbox__IHost_USCOREremoveHostOnlyNetworkInterface(soap, (_vbox__IHost_USCOREremoveHostOnlyNetworkInterface *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREcreateHostOnlyNetworkInterfaceResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREcreateHostOnlyNetworkInterfaceResponse(soap, (_vbox__IHost_USCOREcreateHostOnlyNetworkInterfaceResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREcreateHostOnlyNetworkInterface:
		soap_serialize_PointerTo_vbox__IHost_USCOREcreateHostOnlyNetworkInterface(soap, (_vbox__IHost_USCOREcreateHostOnlyNetworkInterface *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetProcessorDescriptionResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetProcessorDescriptionResponse(soap, (_vbox__IHost_USCOREgetProcessorDescriptionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetProcessorDescription:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetProcessorDescription(soap, (_vbox__IHost_USCOREgetProcessorDescription *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetProcessorFeatureResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetProcessorFeatureResponse(soap, (_vbox__IHost_USCOREgetProcessorFeatureResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetProcessorFeature:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetProcessorFeature(soap, (_vbox__IHost_USCOREgetProcessorFeature *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetProcessorSpeedResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetProcessorSpeedResponse(soap, (_vbox__IHost_USCOREgetProcessorSpeedResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetProcessorSpeed:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetProcessorSpeed(soap, (_vbox__IHost_USCOREgetProcessorSpeed *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetAcceleration3DAvailableResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetAcceleration3DAvailableResponse(soap, (_vbox__IHost_USCOREgetAcceleration3DAvailableResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetAcceleration3DAvailable:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetAcceleration3DAvailable(soap, (_vbox__IHost_USCOREgetAcceleration3DAvailable *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetUTCTimeResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetUTCTimeResponse(soap, (_vbox__IHost_USCOREgetUTCTimeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetUTCTime:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetUTCTime(soap, (_vbox__IHost_USCOREgetUTCTime *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetOSVersionResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetOSVersionResponse(soap, (_vbox__IHost_USCOREgetOSVersionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetOSVersion:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetOSVersion(soap, (_vbox__IHost_USCOREgetOSVersion *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetOperatingSystemResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetOperatingSystemResponse(soap, (_vbox__IHost_USCOREgetOperatingSystemResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetOperatingSystem:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetOperatingSystem(soap, (_vbox__IHost_USCOREgetOperatingSystem *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetMemoryAvailableResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetMemoryAvailableResponse(soap, (_vbox__IHost_USCOREgetMemoryAvailableResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetMemoryAvailable:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetMemoryAvailable(soap, (_vbox__IHost_USCOREgetMemoryAvailable *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetMemorySizeResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetMemorySizeResponse(soap, (_vbox__IHost_USCOREgetMemorySizeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetMemorySize:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetMemorySize(soap, (_vbox__IHost_USCOREgetMemorySize *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetProcessorOnlineCountResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetProcessorOnlineCountResponse(soap, (_vbox__IHost_USCOREgetProcessorOnlineCountResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetProcessorOnlineCount:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetProcessorOnlineCount(soap, (_vbox__IHost_USCOREgetProcessorOnlineCount *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetProcessorCountResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetProcessorCountResponse(soap, (_vbox__IHost_USCOREgetProcessorCountResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetProcessorCount:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetProcessorCount(soap, (_vbox__IHost_USCOREgetProcessorCount *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetNetworkInterfacesResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetNetworkInterfacesResponse(soap, (_vbox__IHost_USCOREgetNetworkInterfacesResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetNetworkInterfaces:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetNetworkInterfaces(soap, (_vbox__IHost_USCOREgetNetworkInterfaces *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetUSBDeviceFiltersResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetUSBDeviceFiltersResponse(soap, (_vbox__IHost_USCOREgetUSBDeviceFiltersResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetUSBDeviceFilters:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetUSBDeviceFilters(soap, (_vbox__IHost_USCOREgetUSBDeviceFilters *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetUSBDevicesResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetUSBDevicesResponse(soap, (_vbox__IHost_USCOREgetUSBDevicesResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetUSBDevices:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetUSBDevices(soap, (_vbox__IHost_USCOREgetUSBDevices *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetFloppyDrivesResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetFloppyDrivesResponse(soap, (_vbox__IHost_USCOREgetFloppyDrivesResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetFloppyDrives:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetFloppyDrives(soap, (_vbox__IHost_USCOREgetFloppyDrives *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetDVDDrivesResponse:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetDVDDrivesResponse(soap, (_vbox__IHost_USCOREgetDVDDrivesResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHost_USCOREgetDVDDrives:
		soap_serialize_PointerTo_vbox__IHost_USCOREgetDVDDrives(soap, (_vbox__IHost_USCOREgetDVDDrives *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREdhcpRediscoverResponse:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREdhcpRediscoverResponse(soap, (_vbox__IHostNetworkInterface_USCOREdhcpRediscoverResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREdhcpRediscover:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREdhcpRediscover(soap, (_vbox__IHostNetworkInterface_USCOREdhcpRediscover *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREenableDynamicIpConfigResponse:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREenableDynamicIpConfigResponse(soap, (_vbox__IHostNetworkInterface_USCOREenableDynamicIpConfigResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREenableDynamicIpConfig:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREenableDynamicIpConfig(soap, (_vbox__IHostNetworkInterface_USCOREenableDynamicIpConfig *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6Response:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6Response(soap, (_vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6Response *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6(soap, (_vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6 *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREenableStaticIpConfigResponse:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREenableStaticIpConfigResponse(soap, (_vbox__IHostNetworkInterface_USCOREenableStaticIpConfigResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREenableStaticIpConfig:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREenableStaticIpConfig(soap, (_vbox__IHostNetworkInterface_USCOREenableStaticIpConfig *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetInterfaceTypeResponse:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetInterfaceTypeResponse(soap, (_vbox__IHostNetworkInterface_USCOREgetInterfaceTypeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetInterfaceType:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetInterfaceType(soap, (_vbox__IHostNetworkInterface_USCOREgetInterfaceType *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetStatusResponse:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetStatusResponse(soap, (_vbox__IHostNetworkInterface_USCOREgetStatusResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetStatus:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetStatus(soap, (_vbox__IHostNetworkInterface_USCOREgetStatus *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetMediumTypeResponse:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetMediumTypeResponse(soap, (_vbox__IHostNetworkInterface_USCOREgetMediumTypeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetMediumType:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetMediumType(soap, (_vbox__IHostNetworkInterface_USCOREgetMediumType *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetHardwareAddressResponse:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetHardwareAddressResponse(soap, (_vbox__IHostNetworkInterface_USCOREgetHardwareAddressResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetHardwareAddress:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetHardwareAddress(soap, (_vbox__IHostNetworkInterface_USCOREgetHardwareAddress *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLengthResponse:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLengthResponse(soap, (_vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLengthResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLength:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLength(soap, (_vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLength *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPV6AddressResponse:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPV6AddressResponse(soap, (_vbox__IHostNetworkInterface_USCOREgetIPV6AddressResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPV6Address:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPV6Address(soap, (_vbox__IHostNetworkInterface_USCOREgetIPV6Address *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPV6SupportedResponse:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPV6SupportedResponse(soap, (_vbox__IHostNetworkInterface_USCOREgetIPV6SupportedResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPV6Supported:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPV6Supported(soap, (_vbox__IHostNetworkInterface_USCOREgetIPV6Supported *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetNetworkMaskResponse:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetNetworkMaskResponse(soap, (_vbox__IHostNetworkInterface_USCOREgetNetworkMaskResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetNetworkMask:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetNetworkMask(soap, (_vbox__IHostNetworkInterface_USCOREgetNetworkMask *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPAddressResponse:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPAddressResponse(soap, (_vbox__IHostNetworkInterface_USCOREgetIPAddressResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPAddress:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetIPAddress(soap, (_vbox__IHostNetworkInterface_USCOREgetIPAddress *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetDhcpEnabledResponse:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetDhcpEnabledResponse(soap, (_vbox__IHostNetworkInterface_USCOREgetDhcpEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetDhcpEnabled:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetDhcpEnabled(soap, (_vbox__IHostNetworkInterface_USCOREgetDhcpEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetNetworkNameResponse:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetNetworkNameResponse(soap, (_vbox__IHostNetworkInterface_USCOREgetNetworkNameResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetNetworkName:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetNetworkName(soap, (_vbox__IHostNetworkInterface_USCOREgetNetworkName *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetIdResponse:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetIdResponse(soap, (_vbox__IHostNetworkInterface_USCOREgetIdResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetId:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetId(soap, (_vbox__IHostNetworkInterface_USCOREgetId *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetNameResponse:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetNameResponse(soap, (_vbox__IHostNetworkInterface_USCOREgetNameResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostNetworkInterface_USCOREgetName:
		soap_serialize_PointerTo_vbox__IHostNetworkInterface_USCOREgetName(soap, (_vbox__IHostNetworkInterface_USCOREgetName *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostFloppyDrive_USCOREgetUdiResponse:
		soap_serialize_PointerTo_vbox__IHostFloppyDrive_USCOREgetUdiResponse(soap, (_vbox__IHostFloppyDrive_USCOREgetUdiResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostFloppyDrive_USCOREgetUdi:
		soap_serialize_PointerTo_vbox__IHostFloppyDrive_USCOREgetUdi(soap, (_vbox__IHostFloppyDrive_USCOREgetUdi *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostFloppyDrive_USCOREgetDescriptionResponse:
		soap_serialize_PointerTo_vbox__IHostFloppyDrive_USCOREgetDescriptionResponse(soap, (_vbox__IHostFloppyDrive_USCOREgetDescriptionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostFloppyDrive_USCOREgetDescription:
		soap_serialize_PointerTo_vbox__IHostFloppyDrive_USCOREgetDescription(soap, (_vbox__IHostFloppyDrive_USCOREgetDescription *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostFloppyDrive_USCOREgetNameResponse:
		soap_serialize_PointerTo_vbox__IHostFloppyDrive_USCOREgetNameResponse(soap, (_vbox__IHostFloppyDrive_USCOREgetNameResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostFloppyDrive_USCOREgetName:
		soap_serialize_PointerTo_vbox__IHostFloppyDrive_USCOREgetName(soap, (_vbox__IHostFloppyDrive_USCOREgetName *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostDVDDrive_USCOREgetUdiResponse:
		soap_serialize_PointerTo_vbox__IHostDVDDrive_USCOREgetUdiResponse(soap, (_vbox__IHostDVDDrive_USCOREgetUdiResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostDVDDrive_USCOREgetUdi:
		soap_serialize_PointerTo_vbox__IHostDVDDrive_USCOREgetUdi(soap, (_vbox__IHostDVDDrive_USCOREgetUdi *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostDVDDrive_USCOREgetDescriptionResponse:
		soap_serialize_PointerTo_vbox__IHostDVDDrive_USCOREgetDescriptionResponse(soap, (_vbox__IHostDVDDrive_USCOREgetDescriptionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostDVDDrive_USCOREgetDescription:
		soap_serialize_PointerTo_vbox__IHostDVDDrive_USCOREgetDescription(soap, (_vbox__IHostDVDDrive_USCOREgetDescription *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostDVDDrive_USCOREgetNameResponse:
		soap_serialize_PointerTo_vbox__IHostDVDDrive_USCOREgetNameResponse(soap, (_vbox__IHostDVDDrive_USCOREgetNameResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IHostDVDDrive_USCOREgetName:
		soap_serialize_PointerTo_vbox__IHostDVDDrive_USCOREgetName(soap, (_vbox__IHostDVDDrive_USCOREgetName *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREdiscardCurrentSnapshotAndStateResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREdiscardCurrentSnapshotAndStateResponse(soap, (_vbox__IConsole_USCOREdiscardCurrentSnapshotAndStateResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREdiscardCurrentSnapshotAndState:
		soap_serialize_PointerTo_vbox__IConsole_USCOREdiscardCurrentSnapshotAndState(soap, (_vbox__IConsole_USCOREdiscardCurrentSnapshotAndState *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREdiscardCurrentStateResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREdiscardCurrentStateResponse(soap, (_vbox__IConsole_USCOREdiscardCurrentStateResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREdiscardCurrentState:
		soap_serialize_PointerTo_vbox__IConsole_USCOREdiscardCurrentState(soap, (_vbox__IConsole_USCOREdiscardCurrentState *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREdiscardSnapshotResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREdiscardSnapshotResponse(soap, (_vbox__IConsole_USCOREdiscardSnapshotResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREdiscardSnapshot:
		soap_serialize_PointerTo_vbox__IConsole_USCOREdiscardSnapshot(soap, (_vbox__IConsole_USCOREdiscardSnapshot *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREtakeSnapshotResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREtakeSnapshotResponse(soap, (_vbox__IConsole_USCOREtakeSnapshotResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREtakeSnapshot:
		soap_serialize_PointerTo_vbox__IConsole_USCOREtakeSnapshot(soap, (_vbox__IConsole_USCOREtakeSnapshot *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREremoveSharedFolderResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREremoveSharedFolderResponse(soap, (_vbox__IConsole_USCOREremoveSharedFolderResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREremoveSharedFolder:
		soap_serialize_PointerTo_vbox__IConsole_USCOREremoveSharedFolder(soap, (_vbox__IConsole_USCOREremoveSharedFolder *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREcreateSharedFolderResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREcreateSharedFolderResponse(soap, (_vbox__IConsole_USCOREcreateSharedFolderResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREcreateSharedFolder:
		soap_serialize_PointerTo_vbox__IConsole_USCOREcreateSharedFolder(soap, (_vbox__IConsole_USCOREcreateSharedFolder *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREfindUSBDeviceByIdResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREfindUSBDeviceByIdResponse(soap, (_vbox__IConsole_USCOREfindUSBDeviceByIdResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREfindUSBDeviceById:
		soap_serialize_PointerTo_vbox__IConsole_USCOREfindUSBDeviceById(soap, (_vbox__IConsole_USCOREfindUSBDeviceById *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREfindUSBDeviceByAddressResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREfindUSBDeviceByAddressResponse(soap, (_vbox__IConsole_USCOREfindUSBDeviceByAddressResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREfindUSBDeviceByAddress:
		soap_serialize_PointerTo_vbox__IConsole_USCOREfindUSBDeviceByAddress(soap, (_vbox__IConsole_USCOREfindUSBDeviceByAddress *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREdetachUSBDeviceResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREdetachUSBDeviceResponse(soap, (_vbox__IConsole_USCOREdetachUSBDeviceResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREdetachUSBDevice:
		soap_serialize_PointerTo_vbox__IConsole_USCOREdetachUSBDevice(soap, (_vbox__IConsole_USCOREdetachUSBDevice *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREattachUSBDeviceResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREattachUSBDeviceResponse(soap, (_vbox__IConsole_USCOREattachUSBDeviceResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREattachUSBDevice:
		soap_serialize_PointerTo_vbox__IConsole_USCOREattachUSBDevice(soap, (_vbox__IConsole_USCOREattachUSBDevice *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetDeviceActivityResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetDeviceActivityResponse(soap, (_vbox__IConsole_USCOREgetDeviceActivityResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetDeviceActivity:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetDeviceActivity(soap, (_vbox__IConsole_USCOREgetDeviceActivity *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREforgetSavedStateResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREforgetSavedStateResponse(soap, (_vbox__IConsole_USCOREforgetSavedStateResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREforgetSavedState:
		soap_serialize_PointerTo_vbox__IConsole_USCOREforgetSavedState(soap, (_vbox__IConsole_USCOREforgetSavedState *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREadoptSavedStateResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREadoptSavedStateResponse(soap, (_vbox__IConsole_USCOREadoptSavedStateResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREadoptSavedState:
		soap_serialize_PointerTo_vbox__IConsole_USCOREadoptSavedState(soap, (_vbox__IConsole_USCOREadoptSavedState *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREsaveStateResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREsaveStateResponse(soap, (_vbox__IConsole_USCOREsaveStateResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREsaveState:
		soap_serialize_PointerTo_vbox__IConsole_USCOREsaveState(soap, (_vbox__IConsole_USCOREsaveState *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetGuestEnteredACPIModeResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetGuestEnteredACPIModeResponse(soap, (_vbox__IConsole_USCOREgetGuestEnteredACPIModeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetGuestEnteredACPIMode:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetGuestEnteredACPIMode(soap, (_vbox__IConsole_USCOREgetGuestEnteredACPIMode *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetPowerButtonHandledResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetPowerButtonHandledResponse(soap, (_vbox__IConsole_USCOREgetPowerButtonHandledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetPowerButtonHandled:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetPowerButtonHandled(soap, (_vbox__IConsole_USCOREgetPowerButtonHandled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREsleepButtonResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREsleepButtonResponse(soap, (_vbox__IConsole_USCOREsleepButtonResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREsleepButton:
		soap_serialize_PointerTo_vbox__IConsole_USCOREsleepButton(soap, (_vbox__IConsole_USCOREsleepButton *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREpowerButtonResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREpowerButtonResponse(soap, (_vbox__IConsole_USCOREpowerButtonResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREpowerButton:
		soap_serialize_PointerTo_vbox__IConsole_USCOREpowerButton(soap, (_vbox__IConsole_USCOREpowerButton *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREresumeResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREresumeResponse(soap, (_vbox__IConsole_USCOREresumeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREresume:
		soap_serialize_PointerTo_vbox__IConsole_USCOREresume(soap, (_vbox__IConsole_USCOREresume *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREpauseResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREpauseResponse(soap, (_vbox__IConsole_USCOREpauseResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREpause:
		soap_serialize_PointerTo_vbox__IConsole_USCOREpause(soap, (_vbox__IConsole_USCOREpause *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREresetResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREresetResponse(soap, (_vbox__IConsole_USCOREresetResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREreset:
		soap_serialize_PointerTo_vbox__IConsole_USCOREreset(soap, (_vbox__IConsole_USCOREreset *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREpowerDownResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREpowerDownResponse(soap, (_vbox__IConsole_USCOREpowerDownResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREpowerDown:
		soap_serialize_PointerTo_vbox__IConsole_USCOREpowerDown(soap, (_vbox__IConsole_USCOREpowerDown *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREpowerUpPausedResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREpowerUpPausedResponse(soap, (_vbox__IConsole_USCOREpowerUpPausedResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREpowerUpPaused:
		soap_serialize_PointerTo_vbox__IConsole_USCOREpowerUpPaused(soap, (_vbox__IConsole_USCOREpowerUpPaused *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREpowerUpResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREpowerUpResponse(soap, (_vbox__IConsole_USCOREpowerUpResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREpowerUp:
		soap_serialize_PointerTo_vbox__IConsole_USCOREpowerUp(soap, (_vbox__IConsole_USCOREpowerUp *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetRemoteDisplayInfoResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetRemoteDisplayInfoResponse(soap, (_vbox__IConsole_USCOREgetRemoteDisplayInfoResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetRemoteDisplayInfo:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetRemoteDisplayInfo(soap, (_vbox__IConsole_USCOREgetRemoteDisplayInfo *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetSharedFoldersResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetSharedFoldersResponse(soap, (_vbox__IConsole_USCOREgetSharedFoldersResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetSharedFolders:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetSharedFolders(soap, (_vbox__IConsole_USCOREgetSharedFolders *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetRemoteUSBDevicesResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetRemoteUSBDevicesResponse(soap, (_vbox__IConsole_USCOREgetRemoteUSBDevicesResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetRemoteUSBDevices:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetRemoteUSBDevices(soap, (_vbox__IConsole_USCOREgetRemoteUSBDevices *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetUSBDevicesResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetUSBDevicesResponse(soap, (_vbox__IConsole_USCOREgetUSBDevicesResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetUSBDevices:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetUSBDevices(soap, (_vbox__IConsole_USCOREgetUSBDevices *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetDisplayResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetDisplayResponse(soap, (_vbox__IConsole_USCOREgetDisplayResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetDisplay:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetDisplay(soap, (_vbox__IConsole_USCOREgetDisplay *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetMouseResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetMouseResponse(soap, (_vbox__IConsole_USCOREgetMouseResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetMouse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetMouse(soap, (_vbox__IConsole_USCOREgetMouse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetKeyboardResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetKeyboardResponse(soap, (_vbox__IConsole_USCOREgetKeyboardResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetKeyboard:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetKeyboard(soap, (_vbox__IConsole_USCOREgetKeyboard *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetGuestResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetGuestResponse(soap, (_vbox__IConsole_USCOREgetGuestResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetGuest:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetGuest(soap, (_vbox__IConsole_USCOREgetGuest *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetStateResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetStateResponse(soap, (_vbox__IConsole_USCOREgetStateResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetState:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetState(soap, (_vbox__IConsole_USCOREgetState *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetMachineResponse:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetMachineResponse(soap, (_vbox__IConsole_USCOREgetMachineResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IConsole_USCOREgetMachine:
		soap_serialize_PointerTo_vbox__IConsole_USCOREgetMachine(soap, (_vbox__IConsole_USCOREgetMachine *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREenumerateGuestPropertiesResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREenumerateGuestPropertiesResponse(soap, (_vbox__IMachine_USCOREenumerateGuestPropertiesResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREenumerateGuestProperties:
		soap_serialize_PointerTo_vbox__IMachine_USCOREenumerateGuestProperties(soap, (_vbox__IMachine_USCOREenumerateGuestProperties *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetGuestPropertyValueResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetGuestPropertyValueResponse(soap, (_vbox__IMachine_USCOREsetGuestPropertyValueResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetGuestPropertyValue:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetGuestPropertyValue(soap, (_vbox__IMachine_USCOREsetGuestPropertyValue *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetGuestPropertyResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetGuestPropertyResponse(soap, (_vbox__IMachine_USCOREsetGuestPropertyResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetGuestProperty:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetGuestProperty(soap, (_vbox__IMachine_USCOREsetGuestProperty *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetGuestPropertyTimestampResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetGuestPropertyTimestampResponse(soap, (_vbox__IMachine_USCOREgetGuestPropertyTimestampResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetGuestPropertyTimestamp:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetGuestPropertyTimestamp(soap, (_vbox__IMachine_USCOREgetGuestPropertyTimestamp *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetGuestPropertyValueResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetGuestPropertyValueResponse(soap, (_vbox__IMachine_USCOREgetGuestPropertyValueResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetGuestPropertyValue:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetGuestPropertyValue(soap, (_vbox__IMachine_USCOREgetGuestPropertyValue *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetGuestPropertyResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetGuestPropertyResponse(soap, (_vbox__IMachine_USCOREgetGuestPropertyResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetGuestProperty:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetGuestProperty(soap, (_vbox__IMachine_USCOREgetGuestProperty *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREshowConsoleWindowResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREshowConsoleWindowResponse(soap, (_vbox__IMachine_USCOREshowConsoleWindowResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREshowConsoleWindow:
		soap_serialize_PointerTo_vbox__IMachine_USCOREshowConsoleWindow(soap, (_vbox__IMachine_USCOREshowConsoleWindow *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREcanShowConsoleWindowResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREcanShowConsoleWindowResponse(soap, (_vbox__IMachine_USCOREcanShowConsoleWindowResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREcanShowConsoleWindow:
		soap_serialize_PointerTo_vbox__IMachine_USCOREcanShowConsoleWindow(soap, (_vbox__IMachine_USCOREcanShowConsoleWindow *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREremoveSharedFolderResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREremoveSharedFolderResponse(soap, (_vbox__IMachine_USCOREremoveSharedFolderResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREremoveSharedFolder:
		soap_serialize_PointerTo_vbox__IMachine_USCOREremoveSharedFolder(soap, (_vbox__IMachine_USCOREremoveSharedFolder *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREcreateSharedFolderResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREcreateSharedFolderResponse(soap, (_vbox__IMachine_USCOREcreateSharedFolderResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREcreateSharedFolder:
		soap_serialize_PointerTo_vbox__IMachine_USCOREcreateSharedFolder(soap, (_vbox__IMachine_USCOREcreateSharedFolder *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetCurrentSnapshotResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetCurrentSnapshotResponse(soap, (_vbox__IMachine_USCOREsetCurrentSnapshotResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetCurrentSnapshot:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetCurrentSnapshot(soap, (_vbox__IMachine_USCOREsetCurrentSnapshot *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREfindSnapshotResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREfindSnapshotResponse(soap, (_vbox__IMachine_USCOREfindSnapshotResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREfindSnapshot:
		soap_serialize_PointerTo_vbox__IMachine_USCOREfindSnapshot(soap, (_vbox__IMachine_USCOREfindSnapshot *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSnapshotResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetSnapshotResponse(soap, (_vbox__IMachine_USCOREgetSnapshotResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSnapshot:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetSnapshot(soap, (_vbox__IMachine_USCOREgetSnapshot *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREexportResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREexportResponse(soap, (_vbox__IMachine_USCOREexportResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREexport:
		soap_serialize_PointerTo_vbox__IMachine_USCOREexport(soap, (_vbox__IMachine_USCOREexport *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREdeleteSettingsResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREdeleteSettingsResponse(soap, (_vbox__IMachine_USCOREdeleteSettingsResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREdeleteSettings:
		soap_serialize_PointerTo_vbox__IMachine_USCOREdeleteSettings(soap, (_vbox__IMachine_USCOREdeleteSettings *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREdiscardSettingsResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREdiscardSettingsResponse(soap, (_vbox__IMachine_USCOREdiscardSettingsResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREdiscardSettings:
		soap_serialize_PointerTo_vbox__IMachine_USCOREdiscardSettings(soap, (_vbox__IMachine_USCOREdiscardSettings *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsaveSettingsWithBackupResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsaveSettingsWithBackupResponse(soap, (_vbox__IMachine_USCOREsaveSettingsWithBackupResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsaveSettingsWithBackup:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsaveSettingsWithBackup(soap, (_vbox__IMachine_USCOREsaveSettingsWithBackup *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsaveSettingsResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsaveSettingsResponse(soap, (_vbox__IMachine_USCOREsaveSettingsResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsaveSettings:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsaveSettings(soap, (_vbox__IMachine_USCOREsaveSettings *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetExtraDataResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetExtraDataResponse(soap, (_vbox__IMachine_USCOREsetExtraDataResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetExtraData:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetExtraData(soap, (_vbox__IMachine_USCOREsetExtraData *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetExtraDataResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetExtraDataResponse(soap, (_vbox__IMachine_USCOREgetExtraDataResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetExtraData:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetExtraData(soap, (_vbox__IMachine_USCOREgetExtraData *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetNextExtraDataKeyResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetNextExtraDataKeyResponse(soap, (_vbox__IMachine_USCOREgetNextExtraDataKeyResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetNextExtraDataKey:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetNextExtraDataKey(soap, (_vbox__IMachine_USCOREgetNextExtraDataKey *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetParallelPortResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetParallelPortResponse(soap, (_vbox__IMachine_USCOREgetParallelPortResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetParallelPort:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetParallelPort(soap, (_vbox__IMachine_USCOREgetParallelPort *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSerialPortResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetSerialPortResponse(soap, (_vbox__IMachine_USCOREgetSerialPortResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSerialPort:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetSerialPort(soap, (_vbox__IMachine_USCOREgetSerialPort *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREremoveStorageControllerResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREremoveStorageControllerResponse(soap, (_vbox__IMachine_USCOREremoveStorageControllerResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREremoveStorageController:
		soap_serialize_PointerTo_vbox__IMachine_USCOREremoveStorageController(soap, (_vbox__IMachine_USCOREremoveStorageController *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetStorageControllerByNameResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetStorageControllerByNameResponse(soap, (_vbox__IMachine_USCOREgetStorageControllerByNameResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetStorageControllerByName:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetStorageControllerByName(soap, (_vbox__IMachine_USCOREgetStorageControllerByName *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREaddStorageControllerResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREaddStorageControllerResponse(soap, (_vbox__IMachine_USCOREaddStorageControllerResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREaddStorageController:
		soap_serialize_PointerTo_vbox__IMachine_USCOREaddStorageController(soap, (_vbox__IMachine_USCOREaddStorageController *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetNetworkAdapterResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetNetworkAdapterResponse(soap, (_vbox__IMachine_USCOREgetNetworkAdapterResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetNetworkAdapter:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetNetworkAdapter(soap, (_vbox__IMachine_USCOREgetNetworkAdapter *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHardDiskAttachmentsOfControllerResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetHardDiskAttachmentsOfControllerResponse(soap, (_vbox__IMachine_USCOREgetHardDiskAttachmentsOfControllerResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHardDiskAttachmentsOfController:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetHardDiskAttachmentsOfController(soap, (_vbox__IMachine_USCOREgetHardDiskAttachmentsOfController *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREdetachHardDiskResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREdetachHardDiskResponse(soap, (_vbox__IMachine_USCOREdetachHardDiskResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREdetachHardDisk:
		soap_serialize_PointerTo_vbox__IMachine_USCOREdetachHardDisk(soap, (_vbox__IMachine_USCOREdetachHardDisk *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHardDiskResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetHardDiskResponse(soap, (_vbox__IMachine_USCOREgetHardDiskResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHardDisk:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetHardDisk(soap, (_vbox__IMachine_USCOREgetHardDisk *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREattachHardDiskResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREattachHardDiskResponse(soap, (_vbox__IMachine_USCOREattachHardDiskResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREattachHardDisk:
		soap_serialize_PointerTo_vbox__IMachine_USCOREattachHardDisk(soap, (_vbox__IMachine_USCOREattachHardDisk *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetBootOrderResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetBootOrderResponse(soap, (_vbox__IMachine_USCOREgetBootOrderResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetBootOrder:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetBootOrder(soap, (_vbox__IMachine_USCOREgetBootOrder *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetBootOrderResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetBootOrderResponse(soap, (_vbox__IMachine_USCOREsetBootOrderResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetBootOrder:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetBootOrder(soap, (_vbox__IMachine_USCOREsetBootOrder *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetGuestPropertyNotificationPatternsResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetGuestPropertyNotificationPatternsResponse(soap, (_vbox__IMachine_USCOREsetGuestPropertyNotificationPatternsResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetGuestPropertyNotificationPatterns:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetGuestPropertyNotificationPatterns(soap, (_vbox__IMachine_USCOREsetGuestPropertyNotificationPatterns *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetGuestPropertyNotificationPatternsResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetGuestPropertyNotificationPatternsResponse(soap, (_vbox__IMachine_USCOREgetGuestPropertyNotificationPatternsResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetGuestPropertyNotificationPatterns:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetGuestPropertyNotificationPatterns(soap, (_vbox__IMachine_USCOREgetGuestPropertyNotificationPatterns *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetClipboardModeResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetClipboardModeResponse(soap, (_vbox__IMachine_USCOREsetClipboardModeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetClipboardMode:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetClipboardMode(soap, (_vbox__IMachine_USCOREsetClipboardMode *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetClipboardModeResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetClipboardModeResponse(soap, (_vbox__IMachine_USCOREgetClipboardModeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetClipboardMode:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetClipboardMode(soap, (_vbox__IMachine_USCOREgetClipboardMode *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSharedFoldersResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetSharedFoldersResponse(soap, (_vbox__IMachine_USCOREgetSharedFoldersResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSharedFolders:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetSharedFolders(soap, (_vbox__IMachine_USCOREgetSharedFolders *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetCurrentStateModifiedResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetCurrentStateModifiedResponse(soap, (_vbox__IMachine_USCOREgetCurrentStateModifiedResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetCurrentStateModified:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetCurrentStateModified(soap, (_vbox__IMachine_USCOREgetCurrentStateModified *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSnapshotCountResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetSnapshotCountResponse(soap, (_vbox__IMachine_USCOREgetSnapshotCountResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSnapshotCount:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetSnapshotCount(soap, (_vbox__IMachine_USCOREgetSnapshotCount *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetCurrentSnapshotResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetCurrentSnapshotResponse(soap, (_vbox__IMachine_USCOREgetCurrentSnapshotResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetCurrentSnapshot:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetCurrentSnapshot(soap, (_vbox__IMachine_USCOREgetCurrentSnapshot *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetLogFolderResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetLogFolderResponse(soap, (_vbox__IMachine_USCOREgetLogFolderResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetLogFolder:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetLogFolder(soap, (_vbox__IMachine_USCOREgetLogFolder *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetStateFilePathResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetStateFilePathResponse(soap, (_vbox__IMachine_USCOREgetStateFilePathResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetStateFilePath:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetStateFilePath(soap, (_vbox__IMachine_USCOREgetStateFilePath *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetLastStateChangeResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetLastStateChangeResponse(soap, (_vbox__IMachine_USCOREgetLastStateChangeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetLastStateChange:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetLastStateChange(soap, (_vbox__IMachine_USCOREgetLastStateChange *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetStateResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetStateResponse(soap, (_vbox__IMachine_USCOREgetStateResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetState:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetState(soap, (_vbox__IMachine_USCOREgetState *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSessionPidResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetSessionPidResponse(soap, (_vbox__IMachine_USCOREgetSessionPidResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSessionPid:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetSessionPid(soap, (_vbox__IMachine_USCOREgetSessionPid *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSessionTypeResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetSessionTypeResponse(soap, (_vbox__IMachine_USCOREgetSessionTypeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSessionType:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetSessionType(soap, (_vbox__IMachine_USCOREgetSessionType *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSessionStateResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetSessionStateResponse(soap, (_vbox__IMachine_USCOREgetSessionStateResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSessionState:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetSessionState(soap, (_vbox__IMachine_USCOREgetSessionState *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSettingsModifiedResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetSettingsModifiedResponse(soap, (_vbox__IMachine_USCOREgetSettingsModifiedResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSettingsModified:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetSettingsModified(soap, (_vbox__IMachine_USCOREgetSettingsModified *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSettingsFileVersionResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetSettingsFileVersionResponse(soap, (_vbox__IMachine_USCOREgetSettingsFileVersionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSettingsFileVersion:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetSettingsFileVersion(soap, (_vbox__IMachine_USCOREgetSettingsFileVersion *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSettingsFilePathResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetSettingsFilePathResponse(soap, (_vbox__IMachine_USCOREgetSettingsFilePathResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSettingsFilePath:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetSettingsFilePath(soap, (_vbox__IMachine_USCOREgetSettingsFilePath *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetStorageControllersResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetStorageControllersResponse(soap, (_vbox__IMachine_USCOREgetStorageControllersResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetStorageControllers:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetStorageControllers(soap, (_vbox__IMachine_USCOREgetStorageControllers *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetAudioAdapterResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetAudioAdapterResponse(soap, (_vbox__IMachine_USCOREgetAudioAdapterResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetAudioAdapter:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetAudioAdapter(soap, (_vbox__IMachine_USCOREgetAudioAdapter *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetUSBControllerResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetUSBControllerResponse(soap, (_vbox__IMachine_USCOREgetUSBControllerResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetUSBController:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetUSBController(soap, (_vbox__IMachine_USCOREgetUSBController *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetFloppyDriveResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetFloppyDriveResponse(soap, (_vbox__IMachine_USCOREgetFloppyDriveResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetFloppyDrive:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetFloppyDrive(soap, (_vbox__IMachine_USCOREgetFloppyDrive *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetDVDDriveResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetDVDDriveResponse(soap, (_vbox__IMachine_USCOREgetDVDDriveResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetDVDDrive:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetDVDDrive(soap, (_vbox__IMachine_USCOREgetDVDDrive *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHardDiskAttachmentsResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetHardDiskAttachmentsResponse(soap, (_vbox__IMachine_USCOREgetHardDiskAttachmentsResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHardDiskAttachments:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetHardDiskAttachments(soap, (_vbox__IMachine_USCOREgetHardDiskAttachments *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetVRDPServerResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetVRDPServerResponse(soap, (_vbox__IMachine_USCOREgetVRDPServerResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetVRDPServer:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetVRDPServer(soap, (_vbox__IMachine_USCOREgetVRDPServer *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetSnapshotFolderResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetSnapshotFolderResponse(soap, (_vbox__IMachine_USCOREsetSnapshotFolderResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetSnapshotFolder:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetSnapshotFolder(soap, (_vbox__IMachine_USCOREsetSnapshotFolder *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSnapshotFolderResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetSnapshotFolderResponse(soap, (_vbox__IMachine_USCOREgetSnapshotFolderResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetSnapshotFolder:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetSnapshotFolder(soap, (_vbox__IMachine_USCOREgetSnapshotFolder *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetPAEEnabledResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetPAEEnabledResponse(soap, (_vbox__IMachine_USCOREsetPAEEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetPAEEnabled:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetPAEEnabled(soap, (_vbox__IMachine_USCOREsetPAEEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetPAEEnabledResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetPAEEnabledResponse(soap, (_vbox__IMachine_USCOREgetPAEEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetPAEEnabled:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetPAEEnabled(soap, (_vbox__IMachine_USCOREgetPAEEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetHWVirtExVPIDEnabledResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetHWVirtExVPIDEnabledResponse(soap, (_vbox__IMachine_USCOREsetHWVirtExVPIDEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetHWVirtExVPIDEnabled:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetHWVirtExVPIDEnabled(soap, (_vbox__IMachine_USCOREsetHWVirtExVPIDEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHWVirtExVPIDEnabledResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetHWVirtExVPIDEnabledResponse(soap, (_vbox__IMachine_USCOREgetHWVirtExVPIDEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHWVirtExVPIDEnabled:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetHWVirtExVPIDEnabled(soap, (_vbox__IMachine_USCOREgetHWVirtExVPIDEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabledResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabledResponse(soap, (_vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabled:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabled(soap, (_vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabledResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabledResponse(soap, (_vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabled:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabled(soap, (_vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetHWVirtExEnabledResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetHWVirtExEnabledResponse(soap, (_vbox__IMachine_USCOREsetHWVirtExEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetHWVirtExEnabled:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetHWVirtExEnabled(soap, (_vbox__IMachine_USCOREsetHWVirtExEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHWVirtExEnabledResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetHWVirtExEnabledResponse(soap, (_vbox__IMachine_USCOREgetHWVirtExEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHWVirtExEnabled:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetHWVirtExEnabled(soap, (_vbox__IMachine_USCOREgetHWVirtExEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetBIOSSettingsResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetBIOSSettingsResponse(soap, (_vbox__IMachine_USCOREgetBIOSSettingsResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetBIOSSettings:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetBIOSSettings(soap, (_vbox__IMachine_USCOREgetBIOSSettings *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetMonitorCountResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetMonitorCountResponse(soap, (_vbox__IMachine_USCOREsetMonitorCountResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetMonitorCount:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetMonitorCount(soap, (_vbox__IMachine_USCOREsetMonitorCount *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetMonitorCountResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetMonitorCountResponse(soap, (_vbox__IMachine_USCOREgetMonitorCountResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetMonitorCount:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetMonitorCount(soap, (_vbox__IMachine_USCOREgetMonitorCount *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetAccelerate3DEnabledResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetAccelerate3DEnabledResponse(soap, (_vbox__IMachine_USCOREsetAccelerate3DEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetAccelerate3DEnabled:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetAccelerate3DEnabled(soap, (_vbox__IMachine_USCOREsetAccelerate3DEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetAccelerate3DEnabledResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetAccelerate3DEnabledResponse(soap, (_vbox__IMachine_USCOREgetAccelerate3DEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetAccelerate3DEnabled:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetAccelerate3DEnabled(soap, (_vbox__IMachine_USCOREgetAccelerate3DEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetVRAMSizeResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetVRAMSizeResponse(soap, (_vbox__IMachine_USCOREsetVRAMSizeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetVRAMSize:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetVRAMSize(soap, (_vbox__IMachine_USCOREsetVRAMSize *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetVRAMSizeResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetVRAMSizeResponse(soap, (_vbox__IMachine_USCOREgetVRAMSizeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetVRAMSize:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetVRAMSize(soap, (_vbox__IMachine_USCOREgetVRAMSize *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetStatisticsUpdateIntervalResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetStatisticsUpdateIntervalResponse(soap, (_vbox__IMachine_USCOREsetStatisticsUpdateIntervalResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetStatisticsUpdateInterval:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetStatisticsUpdateInterval(soap, (_vbox__IMachine_USCOREsetStatisticsUpdateInterval *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetStatisticsUpdateIntervalResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetStatisticsUpdateIntervalResponse(soap, (_vbox__IMachine_USCOREgetStatisticsUpdateIntervalResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetStatisticsUpdateInterval:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetStatisticsUpdateInterval(soap, (_vbox__IMachine_USCOREgetStatisticsUpdateInterval *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetMemoryBalloonSizeResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetMemoryBalloonSizeResponse(soap, (_vbox__IMachine_USCOREsetMemoryBalloonSizeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetMemoryBalloonSize:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetMemoryBalloonSize(soap, (_vbox__IMachine_USCOREsetMemoryBalloonSize *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetMemoryBalloonSizeResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetMemoryBalloonSizeResponse(soap, (_vbox__IMachine_USCOREgetMemoryBalloonSizeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetMemoryBalloonSize:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetMemoryBalloonSize(soap, (_vbox__IMachine_USCOREgetMemoryBalloonSize *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetMemorySizeResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetMemorySizeResponse(soap, (_vbox__IMachine_USCOREsetMemorySizeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetMemorySize:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetMemorySize(soap, (_vbox__IMachine_USCOREsetMemorySize *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetMemorySizeResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetMemorySizeResponse(soap, (_vbox__IMachine_USCOREgetMemorySizeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetMemorySize:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetMemorySize(soap, (_vbox__IMachine_USCOREgetMemorySize *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetCPUCountResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetCPUCountResponse(soap, (_vbox__IMachine_USCOREsetCPUCountResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetCPUCount:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetCPUCount(soap, (_vbox__IMachine_USCOREsetCPUCount *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetCPUCountResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetCPUCountResponse(soap, (_vbox__IMachine_USCOREgetCPUCountResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetCPUCount:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetCPUCount(soap, (_vbox__IMachine_USCOREgetCPUCount *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetHardwareVersionResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetHardwareVersionResponse(soap, (_vbox__IMachine_USCOREsetHardwareVersionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetHardwareVersion:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetHardwareVersion(soap, (_vbox__IMachine_USCOREsetHardwareVersion *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHardwareVersionResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetHardwareVersionResponse(soap, (_vbox__IMachine_USCOREgetHardwareVersionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetHardwareVersion:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetHardwareVersion(soap, (_vbox__IMachine_USCOREgetHardwareVersion *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetOSTypeIdResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetOSTypeIdResponse(soap, (_vbox__IMachine_USCOREsetOSTypeIdResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetOSTypeId:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetOSTypeId(soap, (_vbox__IMachine_USCOREsetOSTypeId *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetOSTypeIdResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetOSTypeIdResponse(soap, (_vbox__IMachine_USCOREgetOSTypeIdResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetOSTypeId:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetOSTypeId(soap, (_vbox__IMachine_USCOREgetOSTypeId *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetIdResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetIdResponse(soap, (_vbox__IMachine_USCOREgetIdResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetId:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetId(soap, (_vbox__IMachine_USCOREgetId *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetDescriptionResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetDescriptionResponse(soap, (_vbox__IMachine_USCOREsetDescriptionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetDescription:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetDescription(soap, (_vbox__IMachine_USCOREsetDescription *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetDescriptionResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetDescriptionResponse(soap, (_vbox__IMachine_USCOREgetDescriptionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetDescription:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetDescription(soap, (_vbox__IMachine_USCOREgetDescription *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetNameResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetNameResponse(soap, (_vbox__IMachine_USCOREsetNameResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREsetName:
		soap_serialize_PointerTo_vbox__IMachine_USCOREsetName(soap, (_vbox__IMachine_USCOREsetName *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetNameResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetNameResponse(soap, (_vbox__IMachine_USCOREgetNameResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetName:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetName(soap, (_vbox__IMachine_USCOREgetName *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetAccessErrorResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetAccessErrorResponse(soap, (_vbox__IMachine_USCOREgetAccessErrorResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetAccessError:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetAccessError(soap, (_vbox__IMachine_USCOREgetAccessError *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetAccessibleResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetAccessibleResponse(soap, (_vbox__IMachine_USCOREgetAccessibleResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetAccessible:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetAccessible(soap, (_vbox__IMachine_USCOREgetAccessible *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetParentResponse:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetParentResponse(soap, (_vbox__IMachine_USCOREgetParentResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IMachine_USCOREgetParent:
		soap_serialize_PointerTo_vbox__IMachine_USCOREgetParent(soap, (_vbox__IMachine_USCOREgetParent *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetPXEDebugEnabledResponse:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREsetPXEDebugEnabledResponse(soap, (_vbox__IBIOSSettings_USCOREsetPXEDebugEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetPXEDebugEnabled:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREsetPXEDebugEnabled(soap, (_vbox__IBIOSSettings_USCOREsetPXEDebugEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetPXEDebugEnabledResponse:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREgetPXEDebugEnabledResponse(soap, (_vbox__IBIOSSettings_USCOREgetPXEDebugEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetPXEDebugEnabled:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREgetPXEDebugEnabled(soap, (_vbox__IBIOSSettings_USCOREgetPXEDebugEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetTimeOffsetResponse:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREsetTimeOffsetResponse(soap, (_vbox__IBIOSSettings_USCOREsetTimeOffsetResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetTimeOffset:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREsetTimeOffset(soap, (_vbox__IBIOSSettings_USCOREsetTimeOffset *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetTimeOffsetResponse:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREgetTimeOffsetResponse(soap, (_vbox__IBIOSSettings_USCOREgetTimeOffsetResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetTimeOffset:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREgetTimeOffset(soap, (_vbox__IBIOSSettings_USCOREgetTimeOffset *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetIOAPICEnabledResponse:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREsetIOAPICEnabledResponse(soap, (_vbox__IBIOSSettings_USCOREsetIOAPICEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetIOAPICEnabled:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREsetIOAPICEnabled(soap, (_vbox__IBIOSSettings_USCOREsetIOAPICEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetIOAPICEnabledResponse:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREgetIOAPICEnabledResponse(soap, (_vbox__IBIOSSettings_USCOREgetIOAPICEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetIOAPICEnabled:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREgetIOAPICEnabled(soap, (_vbox__IBIOSSettings_USCOREgetIOAPICEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetACPIEnabledResponse:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREsetACPIEnabledResponse(soap, (_vbox__IBIOSSettings_USCOREsetACPIEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetACPIEnabled:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREsetACPIEnabled(soap, (_vbox__IBIOSSettings_USCOREsetACPIEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetACPIEnabledResponse:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREgetACPIEnabledResponse(soap, (_vbox__IBIOSSettings_USCOREgetACPIEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetACPIEnabled:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREgetACPIEnabled(soap, (_vbox__IBIOSSettings_USCOREgetACPIEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetBootMenuModeResponse:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREsetBootMenuModeResponse(soap, (_vbox__IBIOSSettings_USCOREsetBootMenuModeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetBootMenuMode:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREsetBootMenuMode(soap, (_vbox__IBIOSSettings_USCOREsetBootMenuMode *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetBootMenuModeResponse:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREgetBootMenuModeResponse(soap, (_vbox__IBIOSSettings_USCOREgetBootMenuModeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetBootMenuMode:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREgetBootMenuMode(soap, (_vbox__IBIOSSettings_USCOREgetBootMenuMode *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetLogoImagePathResponse:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREsetLogoImagePathResponse(soap, (_vbox__IBIOSSettings_USCOREsetLogoImagePathResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetLogoImagePath:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREsetLogoImagePath(soap, (_vbox__IBIOSSettings_USCOREsetLogoImagePath *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetLogoImagePathResponse:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREgetLogoImagePathResponse(soap, (_vbox__IBIOSSettings_USCOREgetLogoImagePathResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetLogoImagePath:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREgetLogoImagePath(soap, (_vbox__IBIOSSettings_USCOREgetLogoImagePath *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetLogoDisplayTimeResponse:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREsetLogoDisplayTimeResponse(soap, (_vbox__IBIOSSettings_USCOREsetLogoDisplayTimeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetLogoDisplayTime:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREsetLogoDisplayTime(soap, (_vbox__IBIOSSettings_USCOREsetLogoDisplayTime *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetLogoDisplayTimeResponse:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREgetLogoDisplayTimeResponse(soap, (_vbox__IBIOSSettings_USCOREgetLogoDisplayTimeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetLogoDisplayTime:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREgetLogoDisplayTime(soap, (_vbox__IBIOSSettings_USCOREgetLogoDisplayTime *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetLogoFadeOutResponse:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREsetLogoFadeOutResponse(soap, (_vbox__IBIOSSettings_USCOREsetLogoFadeOutResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetLogoFadeOut:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREsetLogoFadeOut(soap, (_vbox__IBIOSSettings_USCOREsetLogoFadeOut *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetLogoFadeOutResponse:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREgetLogoFadeOutResponse(soap, (_vbox__IBIOSSettings_USCOREgetLogoFadeOutResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetLogoFadeOut:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREgetLogoFadeOut(soap, (_vbox__IBIOSSettings_USCOREgetLogoFadeOut *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetLogoFadeInResponse:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREsetLogoFadeInResponse(soap, (_vbox__IBIOSSettings_USCOREsetLogoFadeInResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREsetLogoFadeIn:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREsetLogoFadeIn(soap, (_vbox__IBIOSSettings_USCOREsetLogoFadeIn *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetLogoFadeInResponse:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREgetLogoFadeInResponse(soap, (_vbox__IBIOSSettings_USCOREgetLogoFadeInResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IBIOSSettings_USCOREgetLogoFadeIn:
		soap_serialize_PointerTo_vbox__IBIOSSettings_USCOREgetLogoFadeIn(soap, (_vbox__IBIOSSettings_USCOREgetLogoFadeIn *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualSystemDescription_USCOREaddDescriptionResponse:
		soap_serialize_PointerTo_vbox__IVirtualSystemDescription_USCOREaddDescriptionResponse(soap, (_vbox__IVirtualSystemDescription_USCOREaddDescriptionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualSystemDescription_USCOREaddDescription:
		soap_serialize_PointerTo_vbox__IVirtualSystemDescription_USCOREaddDescription(soap, (_vbox__IVirtualSystemDescription_USCOREaddDescription *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualSystemDescription_USCOREsetFinalValuesResponse:
		soap_serialize_PointerTo_vbox__IVirtualSystemDescription_USCOREsetFinalValuesResponse(soap, (_vbox__IVirtualSystemDescription_USCOREsetFinalValuesResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualSystemDescription_USCOREsetFinalValues:
		soap_serialize_PointerTo_vbox__IVirtualSystemDescription_USCOREsetFinalValues(soap, (_vbox__IVirtualSystemDescription_USCOREsetFinalValues *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualSystemDescription_USCOREgetValuesByTypeResponse:
		soap_serialize_PointerTo_vbox__IVirtualSystemDescription_USCOREgetValuesByTypeResponse(soap, (_vbox__IVirtualSystemDescription_USCOREgetValuesByTypeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualSystemDescription_USCOREgetValuesByType:
		soap_serialize_PointerTo_vbox__IVirtualSystemDescription_USCOREgetValuesByType(soap, (_vbox__IVirtualSystemDescription_USCOREgetValuesByType *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualSystemDescription_USCOREgetDescriptionByTypeResponse:
		soap_serialize_PointerTo_vbox__IVirtualSystemDescription_USCOREgetDescriptionByTypeResponse(soap, (_vbox__IVirtualSystemDescription_USCOREgetDescriptionByTypeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualSystemDescription_USCOREgetDescriptionByType:
		soap_serialize_PointerTo_vbox__IVirtualSystemDescription_USCOREgetDescriptionByType(soap, (_vbox__IVirtualSystemDescription_USCOREgetDescriptionByType *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualSystemDescription_USCOREgetDescriptionResponse:
		soap_serialize_PointerTo_vbox__IVirtualSystemDescription_USCOREgetDescriptionResponse(soap, (_vbox__IVirtualSystemDescription_USCOREgetDescriptionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualSystemDescription_USCOREgetDescription:
		soap_serialize_PointerTo_vbox__IVirtualSystemDescription_USCOREgetDescription(soap, (_vbox__IVirtualSystemDescription_USCOREgetDescription *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualSystemDescription_USCOREgetCountResponse:
		soap_serialize_PointerTo_vbox__IVirtualSystemDescription_USCOREgetCountResponse(soap, (_vbox__IVirtualSystemDescription_USCOREgetCountResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualSystemDescription_USCOREgetCount:
		soap_serialize_PointerTo_vbox__IVirtualSystemDescription_USCOREgetCount(soap, (_vbox__IVirtualSystemDescription_USCOREgetCount *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREgetWarningsResponse:
		soap_serialize_PointerTo_vbox__IAppliance_USCOREgetWarningsResponse(soap, (_vbox__IAppliance_USCOREgetWarningsResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREgetWarnings:
		soap_serialize_PointerTo_vbox__IAppliance_USCOREgetWarnings(soap, (_vbox__IAppliance_USCOREgetWarnings *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREwriteResponse:
		soap_serialize_PointerTo_vbox__IAppliance_USCOREwriteResponse(soap, (_vbox__IAppliance_USCOREwriteResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREwrite:
		soap_serialize_PointerTo_vbox__IAppliance_USCOREwrite(soap, (_vbox__IAppliance_USCOREwrite *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREcreateVFSExplorerResponse:
		soap_serialize_PointerTo_vbox__IAppliance_USCOREcreateVFSExplorerResponse(soap, (_vbox__IAppliance_USCOREcreateVFSExplorerResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREcreateVFSExplorer:
		soap_serialize_PointerTo_vbox__IAppliance_USCOREcreateVFSExplorer(soap, (_vbox__IAppliance_USCOREcreateVFSExplorer *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREimportMachinesResponse:
		soap_serialize_PointerTo_vbox__IAppliance_USCOREimportMachinesResponse(soap, (_vbox__IAppliance_USCOREimportMachinesResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREimportMachines:
		soap_serialize_PointerTo_vbox__IAppliance_USCOREimportMachines(soap, (_vbox__IAppliance_USCOREimportMachines *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREinterpretResponse:
		soap_serialize_PointerTo_vbox__IAppliance_USCOREinterpretResponse(soap, (_vbox__IAppliance_USCOREinterpretResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREinterpret:
		soap_serialize_PointerTo_vbox__IAppliance_USCOREinterpret(soap, (_vbox__IAppliance_USCOREinterpret *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREreadResponse:
		soap_serialize_PointerTo_vbox__IAppliance_USCOREreadResponse(soap, (_vbox__IAppliance_USCOREreadResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREread:
		soap_serialize_PointerTo_vbox__IAppliance_USCOREread(soap, (_vbox__IAppliance_USCOREread *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREgetVirtualSystemDescriptionsResponse:
		soap_serialize_PointerTo_vbox__IAppliance_USCOREgetVirtualSystemDescriptionsResponse(soap, (_vbox__IAppliance_USCOREgetVirtualSystemDescriptionsResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREgetVirtualSystemDescriptions:
		soap_serialize_PointerTo_vbox__IAppliance_USCOREgetVirtualSystemDescriptions(soap, (_vbox__IAppliance_USCOREgetVirtualSystemDescriptions *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREgetDisksResponse:
		soap_serialize_PointerTo_vbox__IAppliance_USCOREgetDisksResponse(soap, (_vbox__IAppliance_USCOREgetDisksResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREgetDisks:
		soap_serialize_PointerTo_vbox__IAppliance_USCOREgetDisks(soap, (_vbox__IAppliance_USCOREgetDisks *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREgetPathResponse:
		soap_serialize_PointerTo_vbox__IAppliance_USCOREgetPathResponse(soap, (_vbox__IAppliance_USCOREgetPathResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IAppliance_USCOREgetPath:
		soap_serialize_PointerTo_vbox__IAppliance_USCOREgetPath(soap, (_vbox__IAppliance_USCOREgetPath *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREremoveResponse:
		soap_serialize_PointerTo_vbox__IVFSExplorer_USCOREremoveResponse(soap, (_vbox__IVFSExplorer_USCOREremoveResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREremove:
		soap_serialize_PointerTo_vbox__IVFSExplorer_USCOREremove(soap, (_vbox__IVFSExplorer_USCOREremove *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREexistsResponse:
		soap_serialize_PointerTo_vbox__IVFSExplorer_USCOREexistsResponse(soap, (_vbox__IVFSExplorer_USCOREexistsResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREexists:
		soap_serialize_PointerTo_vbox__IVFSExplorer_USCOREexists(soap, (_vbox__IVFSExplorer_USCOREexists *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREentryListResponse:
		soap_serialize_PointerTo_vbox__IVFSExplorer_USCOREentryListResponse(soap, (_vbox__IVFSExplorer_USCOREentryListResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREentryList:
		soap_serialize_PointerTo_vbox__IVFSExplorer_USCOREentryList(soap, (_vbox__IVFSExplorer_USCOREentryList *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREcdUpResponse:
		soap_serialize_PointerTo_vbox__IVFSExplorer_USCOREcdUpResponse(soap, (_vbox__IVFSExplorer_USCOREcdUpResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREcdUp:
		soap_serialize_PointerTo_vbox__IVFSExplorer_USCOREcdUp(soap, (_vbox__IVFSExplorer_USCOREcdUp *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREcdResponse:
		soap_serialize_PointerTo_vbox__IVFSExplorer_USCOREcdResponse(soap, (_vbox__IVFSExplorer_USCOREcdResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREcd:
		soap_serialize_PointerTo_vbox__IVFSExplorer_USCOREcd(soap, (_vbox__IVFSExplorer_USCOREcd *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREupdateResponse:
		soap_serialize_PointerTo_vbox__IVFSExplorer_USCOREupdateResponse(soap, (_vbox__IVFSExplorer_USCOREupdateResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREupdate:
		soap_serialize_PointerTo_vbox__IVFSExplorer_USCOREupdate(soap, (_vbox__IVFSExplorer_USCOREupdate *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREgetTypeResponse:
		soap_serialize_PointerTo_vbox__IVFSExplorer_USCOREgetTypeResponse(soap, (_vbox__IVFSExplorer_USCOREgetTypeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREgetType:
		soap_serialize_PointerTo_vbox__IVFSExplorer_USCOREgetType(soap, (_vbox__IVFSExplorer_USCOREgetType *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREgetPathResponse:
		soap_serialize_PointerTo_vbox__IVFSExplorer_USCOREgetPathResponse(soap, (_vbox__IVFSExplorer_USCOREgetPathResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVFSExplorer_USCOREgetPath:
		soap_serialize_PointerTo_vbox__IVFSExplorer_USCOREgetPath(soap, (_vbox__IVFSExplorer_USCOREgetPath *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREremoveDHCPServerResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREremoveDHCPServerResponse(soap, (_vbox__IVirtualBox_USCOREremoveDHCPServerResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREremoveDHCPServer:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREremoveDHCPServer(soap, (_vbox__IVirtualBox_USCOREremoveDHCPServer *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREfindDHCPServerByNetworkNameResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREfindDHCPServerByNetworkNameResponse(soap, (_vbox__IVirtualBox_USCOREfindDHCPServerByNetworkNameResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREfindDHCPServerByNetworkName:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREfindDHCPServerByNetworkName(soap, (_vbox__IVirtualBox_USCOREfindDHCPServerByNetworkName *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREcreateDHCPServerResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREcreateDHCPServerResponse(soap, (_vbox__IVirtualBox_USCOREcreateDHCPServerResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREcreateDHCPServer:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREcreateDHCPServer(soap, (_vbox__IVirtualBox_USCOREcreateDHCPServer *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREsaveSettingsWithBackupResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREsaveSettingsWithBackupResponse(soap, (_vbox__IVirtualBox_USCOREsaveSettingsWithBackupResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREsaveSettingsWithBackup:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREsaveSettingsWithBackup(soap, (_vbox__IVirtualBox_USCOREsaveSettingsWithBackup *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREsaveSettingsResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREsaveSettingsResponse(soap, (_vbox__IVirtualBox_USCOREsaveSettingsResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREsaveSettings:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREsaveSettings(soap, (_vbox__IVirtualBox_USCOREsaveSettings *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREwaitForPropertyChangeResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREwaitForPropertyChangeResponse(soap, (_vbox__IVirtualBox_USCOREwaitForPropertyChangeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREwaitForPropertyChange:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREwaitForPropertyChange(soap, (_vbox__IVirtualBox_USCOREwaitForPropertyChange *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenExistingSessionResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREopenExistingSessionResponse(soap, (_vbox__IVirtualBox_USCOREopenExistingSessionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenExistingSession:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREopenExistingSession(soap, (_vbox__IVirtualBox_USCOREopenExistingSession *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenRemoteSessionResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREopenRemoteSessionResponse(soap, (_vbox__IVirtualBox_USCOREopenRemoteSessionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenRemoteSession:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREopenRemoteSession(soap, (_vbox__IVirtualBox_USCOREopenRemoteSession *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenSessionResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREopenSessionResponse(soap, (_vbox__IVirtualBox_USCOREopenSessionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenSession:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREopenSession(soap, (_vbox__IVirtualBox_USCOREopenSession *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREsetExtraDataResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREsetExtraDataResponse(soap, (_vbox__IVirtualBox_USCOREsetExtraDataResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREsetExtraData:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREsetExtraData(soap, (_vbox__IVirtualBox_USCOREsetExtraData *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetExtraDataResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetExtraDataResponse(soap, (_vbox__IVirtualBox_USCOREgetExtraDataResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetExtraData:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetExtraData(soap, (_vbox__IVirtualBox_USCOREgetExtraData *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetNextExtraDataKeyResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetNextExtraDataKeyResponse(soap, (_vbox__IVirtualBox_USCOREgetNextExtraDataKeyResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetNextExtraDataKey:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetNextExtraDataKey(soap, (_vbox__IVirtualBox_USCOREgetNextExtraDataKey *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREremoveSharedFolderResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREremoveSharedFolderResponse(soap, (_vbox__IVirtualBox_USCOREremoveSharedFolderResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREremoveSharedFolder:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREremoveSharedFolder(soap, (_vbox__IVirtualBox_USCOREremoveSharedFolder *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREcreateSharedFolderResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREcreateSharedFolderResponse(soap, (_vbox__IVirtualBox_USCOREcreateSharedFolderResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREcreateSharedFolder:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREcreateSharedFolder(soap, (_vbox__IVirtualBox_USCOREcreateSharedFolder *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetGuestOSTypeResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetGuestOSTypeResponse(soap, (_vbox__IVirtualBox_USCOREgetGuestOSTypeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetGuestOSType:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetGuestOSType(soap, (_vbox__IVirtualBox_USCOREgetGuestOSType *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREfindFloppyImageResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREfindFloppyImageResponse(soap, (_vbox__IVirtualBox_USCOREfindFloppyImageResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREfindFloppyImage:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREfindFloppyImage(soap, (_vbox__IVirtualBox_USCOREfindFloppyImage *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetFloppyImageResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetFloppyImageResponse(soap, (_vbox__IVirtualBox_USCOREgetFloppyImageResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetFloppyImage:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetFloppyImage(soap, (_vbox__IVirtualBox_USCOREgetFloppyImage *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenFloppyImageResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREopenFloppyImageResponse(soap, (_vbox__IVirtualBox_USCOREopenFloppyImageResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenFloppyImage:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREopenFloppyImage(soap, (_vbox__IVirtualBox_USCOREopenFloppyImage *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREfindDVDImageResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREfindDVDImageResponse(soap, (_vbox__IVirtualBox_USCOREfindDVDImageResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREfindDVDImage:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREfindDVDImage(soap, (_vbox__IVirtualBox_USCOREfindDVDImage *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetDVDImageResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetDVDImageResponse(soap, (_vbox__IVirtualBox_USCOREgetDVDImageResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetDVDImage:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetDVDImage(soap, (_vbox__IVirtualBox_USCOREgetDVDImage *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenDVDImageResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREopenDVDImageResponse(soap, (_vbox__IVirtualBox_USCOREopenDVDImageResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenDVDImage:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREopenDVDImage(soap, (_vbox__IVirtualBox_USCOREopenDVDImage *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREfindHardDiskResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREfindHardDiskResponse(soap, (_vbox__IVirtualBox_USCOREfindHardDiskResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREfindHardDisk:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREfindHardDisk(soap, (_vbox__IVirtualBox_USCOREfindHardDisk *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetHardDiskResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetHardDiskResponse(soap, (_vbox__IVirtualBox_USCOREgetHardDiskResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetHardDisk:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetHardDisk(soap, (_vbox__IVirtualBox_USCOREgetHardDisk *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenHardDiskResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREopenHardDiskResponse(soap, (_vbox__IVirtualBox_USCOREopenHardDiskResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenHardDisk:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREopenHardDisk(soap, (_vbox__IVirtualBox_USCOREopenHardDisk *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREcreateHardDiskResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREcreateHardDiskResponse(soap, (_vbox__IVirtualBox_USCOREcreateHardDiskResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREcreateHardDisk:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREcreateHardDisk(soap, (_vbox__IVirtualBox_USCOREcreateHardDisk *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREcreateApplianceResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREcreateApplianceResponse(soap, (_vbox__IVirtualBox_USCOREcreateApplianceResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREcreateAppliance:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREcreateAppliance(soap, (_vbox__IVirtualBox_USCOREcreateAppliance *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREunregisterMachineResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREunregisterMachineResponse(soap, (_vbox__IVirtualBox_USCOREunregisterMachineResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREunregisterMachine:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREunregisterMachine(soap, (_vbox__IVirtualBox_USCOREunregisterMachine *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREfindMachineResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREfindMachineResponse(soap, (_vbox__IVirtualBox_USCOREfindMachineResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREfindMachine:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREfindMachine(soap, (_vbox__IVirtualBox_USCOREfindMachine *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetMachineResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetMachineResponse(soap, (_vbox__IVirtualBox_USCOREgetMachineResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetMachine:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetMachine(soap, (_vbox__IVirtualBox_USCOREgetMachine *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREregisterMachineResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREregisterMachineResponse(soap, (_vbox__IVirtualBox_USCOREregisterMachineResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREregisterMachine:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREregisterMachine(soap, (_vbox__IVirtualBox_USCOREregisterMachine *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenMachineResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREopenMachineResponse(soap, (_vbox__IVirtualBox_USCOREopenMachineResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREopenMachine:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREopenMachine(soap, (_vbox__IVirtualBox_USCOREopenMachine *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREcreateLegacyMachineResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREcreateLegacyMachineResponse(soap, (_vbox__IVirtualBox_USCOREcreateLegacyMachineResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREcreateLegacyMachine:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREcreateLegacyMachine(soap, (_vbox__IVirtualBox_USCOREcreateLegacyMachine *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREcreateMachineResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREcreateMachineResponse(soap, (_vbox__IVirtualBox_USCOREcreateMachineResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREcreateMachine:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREcreateMachine(soap, (_vbox__IVirtualBox_USCOREcreateMachine *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetDHCPServersResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetDHCPServersResponse(soap, (_vbox__IVirtualBox_USCOREgetDHCPServersResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetDHCPServers:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetDHCPServers(soap, (_vbox__IVirtualBox_USCOREgetDHCPServers *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetPerformanceCollectorResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetPerformanceCollectorResponse(soap, (_vbox__IVirtualBox_USCOREgetPerformanceCollectorResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetPerformanceCollector:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetPerformanceCollector(soap, (_vbox__IVirtualBox_USCOREgetPerformanceCollector *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetSharedFoldersResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetSharedFoldersResponse(soap, (_vbox__IVirtualBox_USCOREgetSharedFoldersResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetSharedFolders:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetSharedFolders(soap, (_vbox__IVirtualBox_USCOREgetSharedFolders *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetGuestOSTypesResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetGuestOSTypesResponse(soap, (_vbox__IVirtualBox_USCOREgetGuestOSTypesResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetGuestOSTypes:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetGuestOSTypes(soap, (_vbox__IVirtualBox_USCOREgetGuestOSTypes *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetProgressOperationsResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetProgressOperationsResponse(soap, (_vbox__IVirtualBox_USCOREgetProgressOperationsResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetProgressOperations:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetProgressOperations(soap, (_vbox__IVirtualBox_USCOREgetProgressOperations *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetFloppyImagesResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetFloppyImagesResponse(soap, (_vbox__IVirtualBox_USCOREgetFloppyImagesResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetFloppyImages:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetFloppyImages(soap, (_vbox__IVirtualBox_USCOREgetFloppyImages *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetDVDImagesResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetDVDImagesResponse(soap, (_vbox__IVirtualBox_USCOREgetDVDImagesResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetDVDImages:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetDVDImages(soap, (_vbox__IVirtualBox_USCOREgetDVDImages *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetHardDisksResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetHardDisksResponse(soap, (_vbox__IVirtualBox_USCOREgetHardDisksResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetHardDisks:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetHardDisks(soap, (_vbox__IVirtualBox_USCOREgetHardDisks *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetMachinesResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetMachinesResponse(soap, (_vbox__IVirtualBox_USCOREgetMachinesResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetMachines:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetMachines(soap, (_vbox__IVirtualBox_USCOREgetMachines *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetSystemPropertiesResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetSystemPropertiesResponse(soap, (_vbox__IVirtualBox_USCOREgetSystemPropertiesResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetSystemProperties:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetSystemProperties(soap, (_vbox__IVirtualBox_USCOREgetSystemProperties *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetHostResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetHostResponse(soap, (_vbox__IVirtualBox_USCOREgetHostResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetHost:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetHost(soap, (_vbox__IVirtualBox_USCOREgetHost *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetSettingsFormatVersionResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetSettingsFormatVersionResponse(soap, (_vbox__IVirtualBox_USCOREgetSettingsFormatVersionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetSettingsFormatVersion:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetSettingsFormatVersion(soap, (_vbox__IVirtualBox_USCOREgetSettingsFormatVersion *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetSettingsFileVersionResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetSettingsFileVersionResponse(soap, (_vbox__IVirtualBox_USCOREgetSettingsFileVersionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetSettingsFileVersion:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetSettingsFileVersion(soap, (_vbox__IVirtualBox_USCOREgetSettingsFileVersion *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetSettingsFilePathResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetSettingsFilePathResponse(soap, (_vbox__IVirtualBox_USCOREgetSettingsFilePathResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetSettingsFilePath:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetSettingsFilePath(soap, (_vbox__IVirtualBox_USCOREgetSettingsFilePath *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetHomeFolderResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetHomeFolderResponse(soap, (_vbox__IVirtualBox_USCOREgetHomeFolderResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetHomeFolder:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetHomeFolder(soap, (_vbox__IVirtualBox_USCOREgetHomeFolder *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetPackageTypeResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetPackageTypeResponse(soap, (_vbox__IVirtualBox_USCOREgetPackageTypeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetPackageType:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetPackageType(soap, (_vbox__IVirtualBox_USCOREgetPackageType *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetRevisionResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetRevisionResponse(soap, (_vbox__IVirtualBox_USCOREgetRevisionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetRevision:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetRevision(soap, (_vbox__IVirtualBox_USCOREgetRevision *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetVersionResponse:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetVersionResponse(soap, (_vbox__IVirtualBox_USCOREgetVersionResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBox_USCOREgetVersion:
		soap_serialize_PointerTo_vbox__IVirtualBox_USCOREgetVersion(soap, (_vbox__IVirtualBox_USCOREgetVersion *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREstopResponse:
		soap_serialize_PointerTo_vbox__IDHCPServer_USCOREstopResponse(soap, (_vbox__IDHCPServer_USCOREstopResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREstop:
		soap_serialize_PointerTo_vbox__IDHCPServer_USCOREstop(soap, (_vbox__IDHCPServer_USCOREstop *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREstartResponse:
		soap_serialize_PointerTo_vbox__IDHCPServer_USCOREstartResponse(soap, (_vbox__IDHCPServer_USCOREstartResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREstart:
		soap_serialize_PointerTo_vbox__IDHCPServer_USCOREstart(soap, (_vbox__IDHCPServer_USCOREstart *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREsetConfigurationResponse:
		soap_serialize_PointerTo_vbox__IDHCPServer_USCOREsetConfigurationResponse(soap, (_vbox__IDHCPServer_USCOREsetConfigurationResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREsetConfiguration:
		soap_serialize_PointerTo_vbox__IDHCPServer_USCOREsetConfiguration(soap, (_vbox__IDHCPServer_USCOREsetConfiguration *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREgetUpperIPResponse:
		soap_serialize_PointerTo_vbox__IDHCPServer_USCOREgetUpperIPResponse(soap, (_vbox__IDHCPServer_USCOREgetUpperIPResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREgetUpperIP:
		soap_serialize_PointerTo_vbox__IDHCPServer_USCOREgetUpperIP(soap, (_vbox__IDHCPServer_USCOREgetUpperIP *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREgetLowerIPResponse:
		soap_serialize_PointerTo_vbox__IDHCPServer_USCOREgetLowerIPResponse(soap, (_vbox__IDHCPServer_USCOREgetLowerIPResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREgetLowerIP:
		soap_serialize_PointerTo_vbox__IDHCPServer_USCOREgetLowerIP(soap, (_vbox__IDHCPServer_USCOREgetLowerIP *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREgetNetworkNameResponse:
		soap_serialize_PointerTo_vbox__IDHCPServer_USCOREgetNetworkNameResponse(soap, (_vbox__IDHCPServer_USCOREgetNetworkNameResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREgetNetworkName:
		soap_serialize_PointerTo_vbox__IDHCPServer_USCOREgetNetworkName(soap, (_vbox__IDHCPServer_USCOREgetNetworkName *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREgetNetworkMaskResponse:
		soap_serialize_PointerTo_vbox__IDHCPServer_USCOREgetNetworkMaskResponse(soap, (_vbox__IDHCPServer_USCOREgetNetworkMaskResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREgetNetworkMask:
		soap_serialize_PointerTo_vbox__IDHCPServer_USCOREgetNetworkMask(soap, (_vbox__IDHCPServer_USCOREgetNetworkMask *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREgetIPAddressResponse:
		soap_serialize_PointerTo_vbox__IDHCPServer_USCOREgetIPAddressResponse(soap, (_vbox__IDHCPServer_USCOREgetIPAddressResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREgetIPAddress:
		soap_serialize_PointerTo_vbox__IDHCPServer_USCOREgetIPAddress(soap, (_vbox__IDHCPServer_USCOREgetIPAddress *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREsetEnabledResponse:
		soap_serialize_PointerTo_vbox__IDHCPServer_USCOREsetEnabledResponse(soap, (_vbox__IDHCPServer_USCOREsetEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREsetEnabled:
		soap_serialize_PointerTo_vbox__IDHCPServer_USCOREsetEnabled(soap, (_vbox__IDHCPServer_USCOREsetEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREgetEnabledResponse:
		soap_serialize_PointerTo_vbox__IDHCPServer_USCOREgetEnabledResponse(soap, (_vbox__IDHCPServer_USCOREgetEnabledResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IDHCPServer_USCOREgetEnabled:
		soap_serialize_PointerTo_vbox__IDHCPServer_USCOREgetEnabled(soap, (_vbox__IDHCPServer_USCOREgetEnabled *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetNextResponse:
		soap_serialize_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetNextResponse(soap, (_vbox__IVirtualBoxErrorInfo_USCOREgetNextResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetNext:
		soap_serialize_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetNext(soap, (_vbox__IVirtualBoxErrorInfo_USCOREgetNext *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetTextResponse:
		soap_serialize_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetTextResponse(soap, (_vbox__IVirtualBoxErrorInfo_USCOREgetTextResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetText:
		soap_serialize_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetText(soap, (_vbox__IVirtualBoxErrorInfo_USCOREgetText *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetComponentResponse:
		soap_serialize_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetComponentResponse(soap, (_vbox__IVirtualBoxErrorInfo_USCOREgetComponentResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetComponent:
		soap_serialize_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetComponent(soap, (_vbox__IVirtualBoxErrorInfo_USCOREgetComponent *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceIDResponse:
		soap_serialize_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceIDResponse(soap, (_vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceIDResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceID:
		soap_serialize_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceID(soap, (_vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceID *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetResultCodeResponse:
		soap_serialize_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetResultCodeResponse(soap, (_vbox__IVirtualBoxErrorInfo_USCOREgetResultCodeResponse *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetResultCode:
		soap_serialize_PointerTo_vbox__IVirtualBoxErrorInfo_USCOREgetResultCode(soap, (_vbox__IVirtualBoxErrorInfo_USCOREgetResultCode *const*)ptr);
		break;
	case SOAP_TYPE_PointerTovbox__IRemoteDisplayInfo:
		soap_serialize_PointerTovbox__IRemoteDisplayInfo(soap, (vbox__IRemoteDisplayInfo *const*)ptr);
		break;
	case SOAP_TYPE_PointerTovbox__IHardDiskAttachment:
		soap_serialize_PointerTovbox__IHardDiskAttachment(soap, (vbox__IHardDiskAttachment *const*)ptr);
		break;
	case SOAP_TYPE_PointerTovbox__ISharedFolder:
		soap_serialize_PointerTovbox__ISharedFolder(soap, (vbox__ISharedFolder *const*)ptr);
		break;
	case SOAP_TYPE_PointerTovbox__IGuestOSType:
		soap_serialize_PointerTovbox__IGuestOSType(soap, (vbox__IGuestOSType *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__RuntimeFault:
		soap_serialize_PointerTo_vbox__RuntimeFault(soap, (_vbox__RuntimeFault *const*)ptr);
		break;
	case SOAP_TYPE_PointerTo_vbox__InvalidObjectFault:
		soap_serialize_PointerTo_vbox__InvalidObjectFault(soap, (_vbox__InvalidObjectFault *const*)ptr);
		break;
	case SOAP_TYPE__QName:
		soap_serialize_string(soap, (char*const*)&ptr);
		break;
	case SOAP_TYPE_string:
		soap_serialize_string(soap, (char*const*)&ptr);
		break;
	}
}

#ifdef __cplusplus
}
#endif
#endif

SOAP_FMAC3 void * SOAP_FMAC4 soap_instantiate(struct soap *soap, int t, const char *type, const char *arrayType, size_t *n)
{
	switch (t)
	{
#ifndef WITH_NOGLOBAL
	case SOAP_TYPE_SOAP_ENV__Detail:
		return (void*)soap_instantiate_SOAP_ENV__Detail(soap, -1, type, arrayType, n);
#endif
	case SOAP_TYPE_std__string:
		return (void*)soap_instantiate_std__string(soap, -1, type, arrayType, n);
	case SOAP_TYPE_vbox__IRemoteDisplayInfo:
		return (void*)soap_instantiate_vbox__IRemoteDisplayInfo(soap, -1, type, arrayType, n);
	case SOAP_TYPE_vbox__IGuestOSType:
		return (void*)soap_instantiate_vbox__IGuestOSType(soap, -1, type, arrayType, n);
	case SOAP_TYPE_vbox__IHardDiskAttachment:
		return (void*)soap_instantiate_vbox__IHardDiskAttachment(soap, -1, type, arrayType, n);
	case SOAP_TYPE_vbox__ISharedFolder:
		return (void*)soap_instantiate_vbox__ISharedFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetResultCode:
		return (void*)soap_instantiate__vbox__IVirtualBoxErrorInfo_USCOREgetResultCode(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetResultCodeResponse:
		return (void*)soap_instantiate__vbox__IVirtualBoxErrorInfo_USCOREgetResultCodeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceID:
		return (void*)soap_instantiate__vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceID(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceIDResponse:
		return (void*)soap_instantiate__vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceIDResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetComponent:
		return (void*)soap_instantiate__vbox__IVirtualBoxErrorInfo_USCOREgetComponent(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetComponentResponse:
		return (void*)soap_instantiate__vbox__IVirtualBoxErrorInfo_USCOREgetComponentResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetText:
		return (void*)soap_instantiate__vbox__IVirtualBoxErrorInfo_USCOREgetText(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetTextResponse:
		return (void*)soap_instantiate__vbox__IVirtualBoxErrorInfo_USCOREgetTextResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetNext:
		return (void*)soap_instantiate__vbox__IVirtualBoxErrorInfo_USCOREgetNext(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBoxErrorInfo_USCOREgetNextResponse:
		return (void*)soap_instantiate__vbox__IVirtualBoxErrorInfo_USCOREgetNextResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetEnabled:
		return (void*)soap_instantiate__vbox__IDHCPServer_USCOREgetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetEnabledResponse:
		return (void*)soap_instantiate__vbox__IDHCPServer_USCOREgetEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREsetEnabled:
		return (void*)soap_instantiate__vbox__IDHCPServer_USCOREsetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREsetEnabledResponse:
		return (void*)soap_instantiate__vbox__IDHCPServer_USCOREsetEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetIPAddress:
		return (void*)soap_instantiate__vbox__IDHCPServer_USCOREgetIPAddress(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetIPAddressResponse:
		return (void*)soap_instantiate__vbox__IDHCPServer_USCOREgetIPAddressResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetNetworkMask:
		return (void*)soap_instantiate__vbox__IDHCPServer_USCOREgetNetworkMask(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetNetworkMaskResponse:
		return (void*)soap_instantiate__vbox__IDHCPServer_USCOREgetNetworkMaskResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetNetworkName:
		return (void*)soap_instantiate__vbox__IDHCPServer_USCOREgetNetworkName(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetNetworkNameResponse:
		return (void*)soap_instantiate__vbox__IDHCPServer_USCOREgetNetworkNameResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetLowerIP:
		return (void*)soap_instantiate__vbox__IDHCPServer_USCOREgetLowerIP(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetLowerIPResponse:
		return (void*)soap_instantiate__vbox__IDHCPServer_USCOREgetLowerIPResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetUpperIP:
		return (void*)soap_instantiate__vbox__IDHCPServer_USCOREgetUpperIP(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREgetUpperIPResponse:
		return (void*)soap_instantiate__vbox__IDHCPServer_USCOREgetUpperIPResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREsetConfiguration:
		return (void*)soap_instantiate__vbox__IDHCPServer_USCOREsetConfiguration(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREsetConfigurationResponse:
		return (void*)soap_instantiate__vbox__IDHCPServer_USCOREsetConfigurationResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREstart:
		return (void*)soap_instantiate__vbox__IDHCPServer_USCOREstart(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREstartResponse:
		return (void*)soap_instantiate__vbox__IDHCPServer_USCOREstartResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREstop:
		return (void*)soap_instantiate__vbox__IDHCPServer_USCOREstop(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDHCPServer_USCOREstopResponse:
		return (void*)soap_instantiate__vbox__IDHCPServer_USCOREstopResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetVersion:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetVersion(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetVersionResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetVersionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetRevision:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetRevision(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetRevisionResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetRevisionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetPackageType:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetPackageType(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetPackageTypeResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetPackageTypeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetHomeFolder:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetHomeFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetHomeFolderResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetHomeFolderResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSettingsFilePath:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetSettingsFilePath(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSettingsFilePathResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetSettingsFilePathResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSettingsFileVersion:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetSettingsFileVersion(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSettingsFileVersionResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetSettingsFileVersionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSettingsFormatVersion:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetSettingsFormatVersion(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSettingsFormatVersionResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetSettingsFormatVersionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetHost:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetHost(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetHostResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetHostResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSystemProperties:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetSystemProperties(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSystemPropertiesResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetSystemPropertiesResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetMachines:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetMachines(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetMachinesResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetMachinesResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetHardDisks:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetHardDisks(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetHardDisksResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetHardDisksResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetDVDImages:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetDVDImages(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetDVDImagesResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetDVDImagesResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetFloppyImages:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetFloppyImages(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetFloppyImagesResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetFloppyImagesResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetProgressOperations:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetProgressOperations(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetProgressOperationsResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetProgressOperationsResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetGuestOSTypes:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetGuestOSTypes(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetGuestOSTypesResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetGuestOSTypesResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSharedFolders:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetSharedFolders(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetSharedFoldersResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetSharedFoldersResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetPerformanceCollector:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetPerformanceCollector(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetPerformanceCollectorResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetPerformanceCollectorResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetDHCPServers:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetDHCPServers(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetDHCPServersResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetDHCPServersResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateMachine:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREcreateMachine(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateMachineResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREcreateMachineResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateLegacyMachine:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREcreateLegacyMachine(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateLegacyMachineResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREcreateLegacyMachineResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenMachine:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREopenMachine(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenMachineResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREopenMachineResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREregisterMachine:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREregisterMachine(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREregisterMachineResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREregisterMachineResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetMachine:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetMachine(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetMachineResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetMachineResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindMachine:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREfindMachine(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindMachineResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREfindMachineResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREunregisterMachine:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREunregisterMachine(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREunregisterMachineResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREunregisterMachineResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateAppliance:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREcreateAppliance(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateApplianceResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREcreateApplianceResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateHardDisk:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREcreateHardDisk(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateHardDiskResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREcreateHardDiskResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenHardDisk:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREopenHardDisk(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenHardDiskResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREopenHardDiskResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetHardDisk:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetHardDisk(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetHardDiskResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetHardDiskResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindHardDisk:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREfindHardDisk(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindHardDiskResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREfindHardDiskResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenDVDImage:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREopenDVDImage(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenDVDImageResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREopenDVDImageResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetDVDImage:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetDVDImage(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetDVDImageResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetDVDImageResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindDVDImage:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREfindDVDImage(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindDVDImageResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREfindDVDImageResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenFloppyImage:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREopenFloppyImage(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenFloppyImageResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREopenFloppyImageResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetFloppyImage:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetFloppyImage(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetFloppyImageResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetFloppyImageResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindFloppyImage:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREfindFloppyImage(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindFloppyImageResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREfindFloppyImageResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetGuestOSType:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetGuestOSType(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetGuestOSTypeResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetGuestOSTypeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateSharedFolder:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREcreateSharedFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateSharedFolderResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREcreateSharedFolderResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREremoveSharedFolder:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREremoveSharedFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREremoveSharedFolderResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREremoveSharedFolderResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetNextExtraDataKey:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetNextExtraDataKey(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetNextExtraDataKeyResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetNextExtraDataKeyResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetExtraData:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetExtraData(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREgetExtraDataResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREgetExtraDataResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREsetExtraData:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREsetExtraData(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREsetExtraDataResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREsetExtraDataResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenSession:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREopenSession(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenSessionResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREopenSessionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenRemoteSession:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREopenRemoteSession(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenRemoteSessionResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREopenRemoteSessionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenExistingSession:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREopenExistingSession(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREopenExistingSessionResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREopenExistingSessionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREwaitForPropertyChange:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREwaitForPropertyChange(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREwaitForPropertyChangeResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREwaitForPropertyChangeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREsaveSettings:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREsaveSettings(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREsaveSettingsResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREsaveSettingsResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREsaveSettingsWithBackup:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREsaveSettingsWithBackup(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREsaveSettingsWithBackupResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREsaveSettingsWithBackupResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateDHCPServer:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREcreateDHCPServer(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREcreateDHCPServerResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREcreateDHCPServerResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindDHCPServerByNetworkName:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREfindDHCPServerByNetworkName(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREfindDHCPServerByNetworkNameResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREfindDHCPServerByNetworkNameResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREremoveDHCPServer:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREremoveDHCPServer(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualBox_USCOREremoveDHCPServerResponse:
		return (void*)soap_instantiate__vbox__IVirtualBox_USCOREremoveDHCPServerResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREgetPath:
		return (void*)soap_instantiate__vbox__IVFSExplorer_USCOREgetPath(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREgetPathResponse:
		return (void*)soap_instantiate__vbox__IVFSExplorer_USCOREgetPathResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREgetType:
		return (void*)soap_instantiate__vbox__IVFSExplorer_USCOREgetType(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREgetTypeResponse:
		return (void*)soap_instantiate__vbox__IVFSExplorer_USCOREgetTypeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREupdate:
		return (void*)soap_instantiate__vbox__IVFSExplorer_USCOREupdate(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREupdateResponse:
		return (void*)soap_instantiate__vbox__IVFSExplorer_USCOREupdateResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREcd:
		return (void*)soap_instantiate__vbox__IVFSExplorer_USCOREcd(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREcdResponse:
		return (void*)soap_instantiate__vbox__IVFSExplorer_USCOREcdResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREcdUp:
		return (void*)soap_instantiate__vbox__IVFSExplorer_USCOREcdUp(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREcdUpResponse:
		return (void*)soap_instantiate__vbox__IVFSExplorer_USCOREcdUpResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREentryList:
		return (void*)soap_instantiate__vbox__IVFSExplorer_USCOREentryList(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREentryListResponse:
		return (void*)soap_instantiate__vbox__IVFSExplorer_USCOREentryListResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREexists:
		return (void*)soap_instantiate__vbox__IVFSExplorer_USCOREexists(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREexistsResponse:
		return (void*)soap_instantiate__vbox__IVFSExplorer_USCOREexistsResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREremove:
		return (void*)soap_instantiate__vbox__IVFSExplorer_USCOREremove(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVFSExplorer_USCOREremoveResponse:
		return (void*)soap_instantiate__vbox__IVFSExplorer_USCOREremoveResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAppliance_USCOREgetPath:
		return (void*)soap_instantiate__vbox__IAppliance_USCOREgetPath(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAppliance_USCOREgetPathResponse:
		return (void*)soap_instantiate__vbox__IAppliance_USCOREgetPathResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAppliance_USCOREgetDisks:
		return (void*)soap_instantiate__vbox__IAppliance_USCOREgetDisks(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAppliance_USCOREgetDisksResponse:
		return (void*)soap_instantiate__vbox__IAppliance_USCOREgetDisksResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAppliance_USCOREgetVirtualSystemDescriptions:
		return (void*)soap_instantiate__vbox__IAppliance_USCOREgetVirtualSystemDescriptions(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAppliance_USCOREgetVirtualSystemDescriptionsResponse:
		return (void*)soap_instantiate__vbox__IAppliance_USCOREgetVirtualSystemDescriptionsResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAppliance_USCOREread:
		return (void*)soap_instantiate__vbox__IAppliance_USCOREread(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAppliance_USCOREreadResponse:
		return (void*)soap_instantiate__vbox__IAppliance_USCOREreadResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAppliance_USCOREinterpret:
		return (void*)soap_instantiate__vbox__IAppliance_USCOREinterpret(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAppliance_USCOREinterpretResponse:
		return (void*)soap_instantiate__vbox__IAppliance_USCOREinterpretResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAppliance_USCOREimportMachines:
		return (void*)soap_instantiate__vbox__IAppliance_USCOREimportMachines(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAppliance_USCOREimportMachinesResponse:
		return (void*)soap_instantiate__vbox__IAppliance_USCOREimportMachinesResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAppliance_USCOREcreateVFSExplorer:
		return (void*)soap_instantiate__vbox__IAppliance_USCOREcreateVFSExplorer(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAppliance_USCOREcreateVFSExplorerResponse:
		return (void*)soap_instantiate__vbox__IAppliance_USCOREcreateVFSExplorerResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAppliance_USCOREwrite:
		return (void*)soap_instantiate__vbox__IAppliance_USCOREwrite(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAppliance_USCOREwriteResponse:
		return (void*)soap_instantiate__vbox__IAppliance_USCOREwriteResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAppliance_USCOREgetWarnings:
		return (void*)soap_instantiate__vbox__IAppliance_USCOREgetWarnings(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAppliance_USCOREgetWarningsResponse:
		return (void*)soap_instantiate__vbox__IAppliance_USCOREgetWarningsResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREgetCount:
		return (void*)soap_instantiate__vbox__IVirtualSystemDescription_USCOREgetCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREgetCountResponse:
		return (void*)soap_instantiate__vbox__IVirtualSystemDescription_USCOREgetCountResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREgetDescription:
		return (void*)soap_instantiate__vbox__IVirtualSystemDescription_USCOREgetDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREgetDescriptionResponse:
		return (void*)soap_instantiate__vbox__IVirtualSystemDescription_USCOREgetDescriptionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREgetDescriptionByType:
		return (void*)soap_instantiate__vbox__IVirtualSystemDescription_USCOREgetDescriptionByType(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREgetDescriptionByTypeResponse:
		return (void*)soap_instantiate__vbox__IVirtualSystemDescription_USCOREgetDescriptionByTypeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREgetValuesByType:
		return (void*)soap_instantiate__vbox__IVirtualSystemDescription_USCOREgetValuesByType(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREgetValuesByTypeResponse:
		return (void*)soap_instantiate__vbox__IVirtualSystemDescription_USCOREgetValuesByTypeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREsetFinalValues:
		return (void*)soap_instantiate__vbox__IVirtualSystemDescription_USCOREsetFinalValues(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREsetFinalValuesResponse:
		return (void*)soap_instantiate__vbox__IVirtualSystemDescription_USCOREsetFinalValuesResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREaddDescription:
		return (void*)soap_instantiate__vbox__IVirtualSystemDescription_USCOREaddDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVirtualSystemDescription_USCOREaddDescriptionResponse:
		return (void*)soap_instantiate__vbox__IVirtualSystemDescription_USCOREaddDescriptionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetLogoFadeIn:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREgetLogoFadeIn(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetLogoFadeInResponse:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREgetLogoFadeInResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetLogoFadeIn:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREsetLogoFadeIn(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetLogoFadeInResponse:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREsetLogoFadeInResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetLogoFadeOut:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREgetLogoFadeOut(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetLogoFadeOutResponse:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREgetLogoFadeOutResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetLogoFadeOut:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREsetLogoFadeOut(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetLogoFadeOutResponse:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREsetLogoFadeOutResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetLogoDisplayTime:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREgetLogoDisplayTime(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetLogoDisplayTimeResponse:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREgetLogoDisplayTimeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetLogoDisplayTime:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREsetLogoDisplayTime(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetLogoDisplayTimeResponse:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREsetLogoDisplayTimeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetLogoImagePath:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREgetLogoImagePath(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetLogoImagePathResponse:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREgetLogoImagePathResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetLogoImagePath:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREsetLogoImagePath(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetLogoImagePathResponse:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREsetLogoImagePathResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetBootMenuMode:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREgetBootMenuMode(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetBootMenuModeResponse:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREgetBootMenuModeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetBootMenuMode:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREsetBootMenuMode(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetBootMenuModeResponse:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREsetBootMenuModeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetACPIEnabled:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREgetACPIEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetACPIEnabledResponse:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREgetACPIEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetACPIEnabled:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREsetACPIEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetACPIEnabledResponse:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREsetACPIEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetIOAPICEnabled:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREgetIOAPICEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetIOAPICEnabledResponse:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREgetIOAPICEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetIOAPICEnabled:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREsetIOAPICEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetIOAPICEnabledResponse:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREsetIOAPICEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetTimeOffset:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREgetTimeOffset(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetTimeOffsetResponse:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREgetTimeOffsetResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetTimeOffset:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREsetTimeOffset(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetTimeOffsetResponse:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREsetTimeOffsetResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetPXEDebugEnabled:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREgetPXEDebugEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREgetPXEDebugEnabledResponse:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREgetPXEDebugEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetPXEDebugEnabled:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREsetPXEDebugEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IBIOSSettings_USCOREsetPXEDebugEnabledResponse:
		return (void*)soap_instantiate__vbox__IBIOSSettings_USCOREsetPXEDebugEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetParent:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetParent(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetParentResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetParentResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetAccessible:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetAccessible(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetAccessibleResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetAccessibleResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetAccessError:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetAccessError(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetAccessErrorResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetAccessErrorResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetName:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetName(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetNameResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetNameResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetName:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetName(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetNameResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetNameResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetDescription:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetDescriptionResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetDescriptionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetDescription:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetDescriptionResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetDescriptionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetId:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetId(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetIdResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetIdResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetOSTypeId:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetOSTypeId(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetOSTypeIdResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetOSTypeIdResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetOSTypeId:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetOSTypeId(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetOSTypeIdResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetOSTypeIdResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHardwareVersion:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetHardwareVersion(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHardwareVersionResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetHardwareVersionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetHardwareVersion:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetHardwareVersion(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetHardwareVersionResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetHardwareVersionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetCPUCount:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetCPUCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetCPUCountResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetCPUCountResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetCPUCount:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetCPUCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetCPUCountResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetCPUCountResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetMemorySize:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetMemorySize(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetMemorySizeResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetMemorySizeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetMemorySize:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetMemorySize(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetMemorySizeResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetMemorySizeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetMemoryBalloonSize:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetMemoryBalloonSize(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetMemoryBalloonSizeResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetMemoryBalloonSizeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetMemoryBalloonSize:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetMemoryBalloonSize(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetMemoryBalloonSizeResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetMemoryBalloonSizeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetStatisticsUpdateInterval:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetStatisticsUpdateInterval(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetStatisticsUpdateIntervalResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetStatisticsUpdateIntervalResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetStatisticsUpdateInterval:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetStatisticsUpdateInterval(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetStatisticsUpdateIntervalResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetStatisticsUpdateIntervalResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetVRAMSize:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetVRAMSize(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetVRAMSizeResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetVRAMSizeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetVRAMSize:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetVRAMSize(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetVRAMSizeResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetVRAMSizeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetAccelerate3DEnabled:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetAccelerate3DEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetAccelerate3DEnabledResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetAccelerate3DEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetAccelerate3DEnabled:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetAccelerate3DEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetAccelerate3DEnabledResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetAccelerate3DEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetMonitorCount:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetMonitorCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetMonitorCountResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetMonitorCountResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetMonitorCount:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetMonitorCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetMonitorCountResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetMonitorCountResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetBIOSSettings:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetBIOSSettings(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetBIOSSettingsResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetBIOSSettingsResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHWVirtExEnabled:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetHWVirtExEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHWVirtExEnabledResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetHWVirtExEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetHWVirtExEnabled:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetHWVirtExEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetHWVirtExEnabledResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetHWVirtExEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabled:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabledResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabled:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabledResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHWVirtExVPIDEnabled:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetHWVirtExVPIDEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHWVirtExVPIDEnabledResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetHWVirtExVPIDEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetHWVirtExVPIDEnabled:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetHWVirtExVPIDEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetHWVirtExVPIDEnabledResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetHWVirtExVPIDEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetPAEEnabled:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetPAEEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetPAEEnabledResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetPAEEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetPAEEnabled:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetPAEEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetPAEEnabledResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetPAEEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSnapshotFolder:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetSnapshotFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSnapshotFolderResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetSnapshotFolderResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetSnapshotFolder:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetSnapshotFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetSnapshotFolderResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetSnapshotFolderResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetVRDPServer:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetVRDPServer(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetVRDPServerResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetVRDPServerResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHardDiskAttachments:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetHardDiskAttachments(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHardDiskAttachmentsResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetHardDiskAttachmentsResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetDVDDrive:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetDVDDrive(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetDVDDriveResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetDVDDriveResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetFloppyDrive:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetFloppyDrive(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetFloppyDriveResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetFloppyDriveResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetUSBController:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetUSBController(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetUSBControllerResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetUSBControllerResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetAudioAdapter:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetAudioAdapter(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetAudioAdapterResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetAudioAdapterResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetStorageControllers:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetStorageControllers(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetStorageControllersResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetStorageControllersResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSettingsFilePath:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetSettingsFilePath(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSettingsFilePathResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetSettingsFilePathResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSettingsFileVersion:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetSettingsFileVersion(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSettingsFileVersionResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetSettingsFileVersionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSettingsModified:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetSettingsModified(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSettingsModifiedResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetSettingsModifiedResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSessionState:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetSessionState(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSessionStateResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetSessionStateResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSessionType:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetSessionType(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSessionTypeResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetSessionTypeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSessionPid:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetSessionPid(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSessionPidResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetSessionPidResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetState:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetState(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetStateResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetStateResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetLastStateChange:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetLastStateChange(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetLastStateChangeResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetLastStateChangeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetStateFilePath:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetStateFilePath(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetStateFilePathResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetStateFilePathResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetLogFolder:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetLogFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetLogFolderResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetLogFolderResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetCurrentSnapshot:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetCurrentSnapshot(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetCurrentSnapshotResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetCurrentSnapshotResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSnapshotCount:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetSnapshotCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSnapshotCountResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetSnapshotCountResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetCurrentStateModified:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetCurrentStateModified(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetCurrentStateModifiedResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetCurrentStateModifiedResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSharedFolders:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetSharedFolders(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSharedFoldersResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetSharedFoldersResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetClipboardMode:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetClipboardMode(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetClipboardModeResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetClipboardModeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetClipboardMode:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetClipboardMode(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetClipboardModeResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetClipboardModeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetGuestPropertyNotificationPatterns:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetGuestPropertyNotificationPatterns(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetGuestPropertyNotificationPatternsResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetGuestPropertyNotificationPatternsResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetGuestPropertyNotificationPatterns:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetGuestPropertyNotificationPatterns(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetGuestPropertyNotificationPatternsResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetGuestPropertyNotificationPatternsResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetBootOrder:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetBootOrder(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetBootOrderResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetBootOrderResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetBootOrder:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetBootOrder(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetBootOrderResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetBootOrderResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREattachHardDisk:
		return (void*)soap_instantiate__vbox__IMachine_USCOREattachHardDisk(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREattachHardDiskResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREattachHardDiskResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHardDisk:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetHardDisk(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHardDiskResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetHardDiskResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREdetachHardDisk:
		return (void*)soap_instantiate__vbox__IMachine_USCOREdetachHardDisk(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREdetachHardDiskResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREdetachHardDiskResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHardDiskAttachmentsOfController:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetHardDiskAttachmentsOfController(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetHardDiskAttachmentsOfControllerResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetHardDiskAttachmentsOfControllerResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetNetworkAdapter:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetNetworkAdapter(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetNetworkAdapterResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetNetworkAdapterResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREaddStorageController:
		return (void*)soap_instantiate__vbox__IMachine_USCOREaddStorageController(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREaddStorageControllerResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREaddStorageControllerResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetStorageControllerByName:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetStorageControllerByName(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetStorageControllerByNameResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetStorageControllerByNameResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREremoveStorageController:
		return (void*)soap_instantiate__vbox__IMachine_USCOREremoveStorageController(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREremoveStorageControllerResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREremoveStorageControllerResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSerialPort:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetSerialPort(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSerialPortResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetSerialPortResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetParallelPort:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetParallelPort(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetParallelPortResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetParallelPortResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetNextExtraDataKey:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetNextExtraDataKey(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetNextExtraDataKeyResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetNextExtraDataKeyResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetExtraData:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetExtraData(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetExtraDataResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetExtraDataResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetExtraData:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetExtraData(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetExtraDataResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetExtraDataResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsaveSettings:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsaveSettings(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsaveSettingsResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsaveSettingsResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsaveSettingsWithBackup:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsaveSettingsWithBackup(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsaveSettingsWithBackupResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsaveSettingsWithBackupResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREdiscardSettings:
		return (void*)soap_instantiate__vbox__IMachine_USCOREdiscardSettings(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREdiscardSettingsResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREdiscardSettingsResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREdeleteSettings:
		return (void*)soap_instantiate__vbox__IMachine_USCOREdeleteSettings(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREdeleteSettingsResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREdeleteSettingsResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREexport:
		return (void*)soap_instantiate__vbox__IMachine_USCOREexport(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREexportResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREexportResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSnapshot:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetSnapshot(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetSnapshotResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetSnapshotResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREfindSnapshot:
		return (void*)soap_instantiate__vbox__IMachine_USCOREfindSnapshot(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREfindSnapshotResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREfindSnapshotResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetCurrentSnapshot:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetCurrentSnapshot(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetCurrentSnapshotResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetCurrentSnapshotResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREcreateSharedFolder:
		return (void*)soap_instantiate__vbox__IMachine_USCOREcreateSharedFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREcreateSharedFolderResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREcreateSharedFolderResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREremoveSharedFolder:
		return (void*)soap_instantiate__vbox__IMachine_USCOREremoveSharedFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREremoveSharedFolderResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREremoveSharedFolderResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREcanShowConsoleWindow:
		return (void*)soap_instantiate__vbox__IMachine_USCOREcanShowConsoleWindow(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREcanShowConsoleWindowResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREcanShowConsoleWindowResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREshowConsoleWindow:
		return (void*)soap_instantiate__vbox__IMachine_USCOREshowConsoleWindow(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREshowConsoleWindowResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREshowConsoleWindowResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetGuestProperty:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetGuestProperty(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetGuestPropertyResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetGuestPropertyResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetGuestPropertyValue:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetGuestPropertyValue(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetGuestPropertyValueResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetGuestPropertyValueResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetGuestPropertyTimestamp:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetGuestPropertyTimestamp(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREgetGuestPropertyTimestampResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREgetGuestPropertyTimestampResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetGuestProperty:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetGuestProperty(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetGuestPropertyResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetGuestPropertyResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetGuestPropertyValue:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetGuestPropertyValue(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREsetGuestPropertyValueResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREsetGuestPropertyValueResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREenumerateGuestProperties:
		return (void*)soap_instantiate__vbox__IMachine_USCOREenumerateGuestProperties(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMachine_USCOREenumerateGuestPropertiesResponse:
		return (void*)soap_instantiate__vbox__IMachine_USCOREenumerateGuestPropertiesResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetMachine:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetMachine(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetMachineResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetMachineResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetState:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetState(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetStateResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetStateResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetGuest:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetGuest(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetGuestResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetGuestResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetKeyboard:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetKeyboard(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetKeyboardResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetKeyboardResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetMouse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetMouse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetMouseResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetMouseResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetDisplay:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetDisplay(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetDisplayResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetDisplayResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetUSBDevices:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetUSBDevices(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetUSBDevicesResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetUSBDevicesResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetRemoteUSBDevices:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetRemoteUSBDevices(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetRemoteUSBDevicesResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetRemoteUSBDevicesResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetSharedFolders:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetSharedFolders(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetSharedFoldersResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetSharedFoldersResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetRemoteDisplayInfo:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetRemoteDisplayInfo(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetRemoteDisplayInfoResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetRemoteDisplayInfoResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREpowerUp:
		return (void*)soap_instantiate__vbox__IConsole_USCOREpowerUp(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREpowerUpResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREpowerUpResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREpowerUpPaused:
		return (void*)soap_instantiate__vbox__IConsole_USCOREpowerUpPaused(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREpowerUpPausedResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREpowerUpPausedResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREpowerDown:
		return (void*)soap_instantiate__vbox__IConsole_USCOREpowerDown(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREpowerDownResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREpowerDownResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREreset:
		return (void*)soap_instantiate__vbox__IConsole_USCOREreset(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREresetResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREresetResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREpause:
		return (void*)soap_instantiate__vbox__IConsole_USCOREpause(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREpauseResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREpauseResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREresume:
		return (void*)soap_instantiate__vbox__IConsole_USCOREresume(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREresumeResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREresumeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREpowerButton:
		return (void*)soap_instantiate__vbox__IConsole_USCOREpowerButton(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREpowerButtonResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREpowerButtonResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREsleepButton:
		return (void*)soap_instantiate__vbox__IConsole_USCOREsleepButton(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREsleepButtonResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREsleepButtonResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetPowerButtonHandled:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetPowerButtonHandled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetPowerButtonHandledResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetPowerButtonHandledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetGuestEnteredACPIMode:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetGuestEnteredACPIMode(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetGuestEnteredACPIModeResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetGuestEnteredACPIModeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREsaveState:
		return (void*)soap_instantiate__vbox__IConsole_USCOREsaveState(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREsaveStateResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREsaveStateResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREadoptSavedState:
		return (void*)soap_instantiate__vbox__IConsole_USCOREadoptSavedState(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREadoptSavedStateResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREadoptSavedStateResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREforgetSavedState:
		return (void*)soap_instantiate__vbox__IConsole_USCOREforgetSavedState(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREforgetSavedStateResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREforgetSavedStateResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetDeviceActivity:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetDeviceActivity(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREgetDeviceActivityResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREgetDeviceActivityResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREattachUSBDevice:
		return (void*)soap_instantiate__vbox__IConsole_USCOREattachUSBDevice(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREattachUSBDeviceResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREattachUSBDeviceResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREdetachUSBDevice:
		return (void*)soap_instantiate__vbox__IConsole_USCOREdetachUSBDevice(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREdetachUSBDeviceResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREdetachUSBDeviceResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREfindUSBDeviceByAddress:
		return (void*)soap_instantiate__vbox__IConsole_USCOREfindUSBDeviceByAddress(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREfindUSBDeviceByAddressResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREfindUSBDeviceByAddressResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREfindUSBDeviceById:
		return (void*)soap_instantiate__vbox__IConsole_USCOREfindUSBDeviceById(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREfindUSBDeviceByIdResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREfindUSBDeviceByIdResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREcreateSharedFolder:
		return (void*)soap_instantiate__vbox__IConsole_USCOREcreateSharedFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREcreateSharedFolderResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREcreateSharedFolderResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREremoveSharedFolder:
		return (void*)soap_instantiate__vbox__IConsole_USCOREremoveSharedFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREremoveSharedFolderResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREremoveSharedFolderResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREtakeSnapshot:
		return (void*)soap_instantiate__vbox__IConsole_USCOREtakeSnapshot(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREtakeSnapshotResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREtakeSnapshotResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREdiscardSnapshot:
		return (void*)soap_instantiate__vbox__IConsole_USCOREdiscardSnapshot(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREdiscardSnapshotResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREdiscardSnapshotResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREdiscardCurrentState:
		return (void*)soap_instantiate__vbox__IConsole_USCOREdiscardCurrentState(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREdiscardCurrentStateResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREdiscardCurrentStateResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREdiscardCurrentSnapshotAndState:
		return (void*)soap_instantiate__vbox__IConsole_USCOREdiscardCurrentSnapshotAndState(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IConsole_USCOREdiscardCurrentSnapshotAndStateResponse:
		return (void*)soap_instantiate__vbox__IConsole_USCOREdiscardCurrentSnapshotAndStateResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostDVDDrive_USCOREgetName:
		return (void*)soap_instantiate__vbox__IHostDVDDrive_USCOREgetName(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostDVDDrive_USCOREgetNameResponse:
		return (void*)soap_instantiate__vbox__IHostDVDDrive_USCOREgetNameResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostDVDDrive_USCOREgetDescription:
		return (void*)soap_instantiate__vbox__IHostDVDDrive_USCOREgetDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostDVDDrive_USCOREgetDescriptionResponse:
		return (void*)soap_instantiate__vbox__IHostDVDDrive_USCOREgetDescriptionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostDVDDrive_USCOREgetUdi:
		return (void*)soap_instantiate__vbox__IHostDVDDrive_USCOREgetUdi(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostDVDDrive_USCOREgetUdiResponse:
		return (void*)soap_instantiate__vbox__IHostDVDDrive_USCOREgetUdiResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostFloppyDrive_USCOREgetName:
		return (void*)soap_instantiate__vbox__IHostFloppyDrive_USCOREgetName(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostFloppyDrive_USCOREgetNameResponse:
		return (void*)soap_instantiate__vbox__IHostFloppyDrive_USCOREgetNameResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostFloppyDrive_USCOREgetDescription:
		return (void*)soap_instantiate__vbox__IHostFloppyDrive_USCOREgetDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostFloppyDrive_USCOREgetDescriptionResponse:
		return (void*)soap_instantiate__vbox__IHostFloppyDrive_USCOREgetDescriptionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostFloppyDrive_USCOREgetUdi:
		return (void*)soap_instantiate__vbox__IHostFloppyDrive_USCOREgetUdi(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostFloppyDrive_USCOREgetUdiResponse:
		return (void*)soap_instantiate__vbox__IHostFloppyDrive_USCOREgetUdiResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetName:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetName(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetNameResponse:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetNameResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetId:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetId(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIdResponse:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetIdResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetNetworkName:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetNetworkName(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetNetworkNameResponse:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetNetworkNameResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetDhcpEnabled:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetDhcpEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetDhcpEnabledResponse:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetDhcpEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIPAddress:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetIPAddress(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIPAddressResponse:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetIPAddressResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetNetworkMask:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetNetworkMask(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetNetworkMaskResponse:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetNetworkMaskResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIPV6Supported:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetIPV6Supported(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIPV6SupportedResponse:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetIPV6SupportedResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIPV6Address:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetIPV6Address(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIPV6AddressResponse:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetIPV6AddressResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLength:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLength(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLengthResponse:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLengthResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetHardwareAddress:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetHardwareAddress(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetHardwareAddressResponse:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetHardwareAddressResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetMediumType:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetMediumType(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetMediumTypeResponse:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetMediumTypeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetStatus:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetStatus(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetStatusResponse:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetStatusResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetInterfaceType:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetInterfaceType(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREgetInterfaceTypeResponse:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREgetInterfaceTypeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREenableStaticIpConfig:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREenableStaticIpConfig(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREenableStaticIpConfigResponse:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREenableStaticIpConfigResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6Response:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6Response(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREenableDynamicIpConfig:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREenableDynamicIpConfig(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREenableDynamicIpConfigResponse:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREenableDynamicIpConfigResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREdhcpRediscover:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREdhcpRediscover(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostNetworkInterface_USCOREdhcpRediscoverResponse:
		return (void*)soap_instantiate__vbox__IHostNetworkInterface_USCOREdhcpRediscoverResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetDVDDrives:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetDVDDrives(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetDVDDrivesResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetDVDDrivesResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetFloppyDrives:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetFloppyDrives(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetFloppyDrivesResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetFloppyDrivesResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetUSBDevices:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetUSBDevices(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetUSBDevicesResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetUSBDevicesResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetUSBDeviceFilters:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetUSBDeviceFilters(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetUSBDeviceFiltersResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetUSBDeviceFiltersResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetNetworkInterfaces:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetNetworkInterfaces(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetNetworkInterfacesResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetNetworkInterfacesResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorCount:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetProcessorCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorCountResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetProcessorCountResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorOnlineCount:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetProcessorOnlineCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorOnlineCountResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetProcessorOnlineCountResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetMemorySize:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetMemorySize(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetMemorySizeResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetMemorySizeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetMemoryAvailable:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetMemoryAvailable(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetMemoryAvailableResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetMemoryAvailableResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetOperatingSystem:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetOperatingSystem(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetOperatingSystemResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetOperatingSystemResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetOSVersion:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetOSVersion(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetOSVersionResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetOSVersionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetUTCTime:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetUTCTime(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetUTCTimeResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetUTCTimeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetAcceleration3DAvailable:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetAcceleration3DAvailable(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetAcceleration3DAvailableResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetAcceleration3DAvailableResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorSpeed:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetProcessorSpeed(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorSpeedResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetProcessorSpeedResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorFeature:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetProcessorFeature(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorFeatureResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetProcessorFeatureResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorDescription:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetProcessorDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREgetProcessorDescriptionResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREgetProcessorDescriptionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREcreateHostOnlyNetworkInterface:
		return (void*)soap_instantiate__vbox__IHost_USCOREcreateHostOnlyNetworkInterface(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREcreateHostOnlyNetworkInterfaceResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREcreateHostOnlyNetworkInterfaceResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREremoveHostOnlyNetworkInterface:
		return (void*)soap_instantiate__vbox__IHost_USCOREremoveHostOnlyNetworkInterface(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREremoveHostOnlyNetworkInterfaceResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREremoveHostOnlyNetworkInterfaceResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREcreateUSBDeviceFilter:
		return (void*)soap_instantiate__vbox__IHost_USCOREcreateUSBDeviceFilter(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREcreateUSBDeviceFilterResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREcreateUSBDeviceFilterResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREinsertUSBDeviceFilter:
		return (void*)soap_instantiate__vbox__IHost_USCOREinsertUSBDeviceFilter(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREinsertUSBDeviceFilterResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREinsertUSBDeviceFilterResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREremoveUSBDeviceFilter:
		return (void*)soap_instantiate__vbox__IHost_USCOREremoveUSBDeviceFilter(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREremoveUSBDeviceFilterResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREremoveUSBDeviceFilterResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREfindHostDVDDrive:
		return (void*)soap_instantiate__vbox__IHost_USCOREfindHostDVDDrive(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREfindHostDVDDriveResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREfindHostDVDDriveResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREfindHostFloppyDrive:
		return (void*)soap_instantiate__vbox__IHost_USCOREfindHostFloppyDrive(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREfindHostFloppyDriveResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREfindHostFloppyDriveResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREfindHostNetworkInterfaceByName:
		return (void*)soap_instantiate__vbox__IHost_USCOREfindHostNetworkInterfaceByName(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREfindHostNetworkInterfaceByNameResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREfindHostNetworkInterfaceByNameResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREfindHostNetworkInterfaceById:
		return (void*)soap_instantiate__vbox__IHost_USCOREfindHostNetworkInterfaceById(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREfindHostNetworkInterfaceByIdResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREfindHostNetworkInterfaceByIdResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREfindHostNetworkInterfacesOfType:
		return (void*)soap_instantiate__vbox__IHost_USCOREfindHostNetworkInterfacesOfType(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREfindHostNetworkInterfacesOfTypeResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREfindHostNetworkInterfacesOfTypeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREfindUSBDeviceById:
		return (void*)soap_instantiate__vbox__IHost_USCOREfindUSBDeviceById(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREfindUSBDeviceByIdResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREfindUSBDeviceByIdResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREfindUSBDeviceByAddress:
		return (void*)soap_instantiate__vbox__IHost_USCOREfindUSBDeviceByAddress(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHost_USCOREfindUSBDeviceByAddressResponse:
		return (void*)soap_instantiate__vbox__IHost_USCOREfindUSBDeviceByAddressResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMinGuestRAM:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetMinGuestRAM(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMinGuestRAMResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetMinGuestRAMResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxGuestRAM:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetMaxGuestRAM(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxGuestRAMResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetMaxGuestRAMResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMinGuestVRAM:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetMinGuestVRAM(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMinGuestVRAMResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetMinGuestVRAMResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxGuestVRAM:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetMaxGuestVRAM(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxGuestVRAMResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetMaxGuestVRAMResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMinGuestCPUCount:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetMinGuestCPUCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMinGuestCPUCountResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetMinGuestCPUCountResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxGuestCPUCount:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetMaxGuestCPUCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxGuestCPUCountResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetMaxGuestCPUCountResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxVDISize:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetMaxVDISize(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxVDISizeResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetMaxVDISizeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetNetworkAdapterCount:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetNetworkAdapterCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetNetworkAdapterCountResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetNetworkAdapterCountResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetSerialPortCount:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetSerialPortCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetSerialPortCountResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetSerialPortCountResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetParallelPortCount:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetParallelPortCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetParallelPortCountResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetParallelPortCountResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxBootPosition:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetMaxBootPosition(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetMaxBootPositionResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetMaxBootPositionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetDefaultMachineFolder:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetDefaultMachineFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetDefaultMachineFolderResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetDefaultMachineFolderResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetDefaultMachineFolder:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREsetDefaultMachineFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetDefaultMachineFolderResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREsetDefaultMachineFolderResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetDefaultHardDiskFolder:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetDefaultHardDiskFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetDefaultHardDiskFolderResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetDefaultHardDiskFolderResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetDefaultHardDiskFolder:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREsetDefaultHardDiskFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetDefaultHardDiskFolderResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREsetDefaultHardDiskFolderResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetHardDiskFormats:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetHardDiskFormats(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetHardDiskFormatsResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetHardDiskFormatsResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetDefaultHardDiskFormat:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetDefaultHardDiskFormat(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetDefaultHardDiskFormatResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetDefaultHardDiskFormatResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetDefaultHardDiskFormat:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREsetDefaultHardDiskFormat(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetDefaultHardDiskFormatResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREsetDefaultHardDiskFormatResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibrary:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibrary(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibraryResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibraryResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibrary:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibrary(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibraryResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibraryResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetWebServiceAuthLibrary:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetWebServiceAuthLibrary(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetWebServiceAuthLibraryResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetWebServiceAuthLibraryResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetWebServiceAuthLibrary:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREsetWebServiceAuthLibrary(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetWebServiceAuthLibraryResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREsetWebServiceAuthLibraryResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetLogHistoryCount:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetLogHistoryCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetLogHistoryCountResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetLogHistoryCountResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetLogHistoryCount:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREsetLogHistoryCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREsetLogHistoryCountResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREsetLogHistoryCountResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetDefaultAudioDriver:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetDefaultAudioDriver(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISystemProperties_USCOREgetDefaultAudioDriverResponse:
		return (void*)soap_instantiate__vbox__ISystemProperties_USCOREgetDefaultAudioDriverResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IGuest_USCOREgetOSTypeId:
		return (void*)soap_instantiate__vbox__IGuest_USCOREgetOSTypeId(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IGuest_USCOREgetOSTypeIdResponse:
		return (void*)soap_instantiate__vbox__IGuest_USCOREgetOSTypeIdResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IGuest_USCOREgetAdditionsActive:
		return (void*)soap_instantiate__vbox__IGuest_USCOREgetAdditionsActive(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IGuest_USCOREgetAdditionsActiveResponse:
		return (void*)soap_instantiate__vbox__IGuest_USCOREgetAdditionsActiveResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IGuest_USCOREgetAdditionsVersion:
		return (void*)soap_instantiate__vbox__IGuest_USCOREgetAdditionsVersion(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IGuest_USCOREgetAdditionsVersionResponse:
		return (void*)soap_instantiate__vbox__IGuest_USCOREgetAdditionsVersionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IGuest_USCOREgetSupportsSeamless:
		return (void*)soap_instantiate__vbox__IGuest_USCOREgetSupportsSeamless(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IGuest_USCOREgetSupportsSeamlessResponse:
		return (void*)soap_instantiate__vbox__IGuest_USCOREgetSupportsSeamlessResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IGuest_USCOREgetSupportsGraphics:
		return (void*)soap_instantiate__vbox__IGuest_USCOREgetSupportsGraphics(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IGuest_USCOREgetSupportsGraphicsResponse:
		return (void*)soap_instantiate__vbox__IGuest_USCOREgetSupportsGraphicsResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IGuest_USCOREgetMemoryBalloonSize:
		return (void*)soap_instantiate__vbox__IGuest_USCOREgetMemoryBalloonSize(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IGuest_USCOREgetMemoryBalloonSizeResponse:
		return (void*)soap_instantiate__vbox__IGuest_USCOREgetMemoryBalloonSizeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IGuest_USCOREsetMemoryBalloonSize:
		return (void*)soap_instantiate__vbox__IGuest_USCOREsetMemoryBalloonSize(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IGuest_USCOREsetMemoryBalloonSizeResponse:
		return (void*)soap_instantiate__vbox__IGuest_USCOREsetMemoryBalloonSizeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IGuest_USCOREgetStatisticsUpdateInterval:
		return (void*)soap_instantiate__vbox__IGuest_USCOREgetStatisticsUpdateInterval(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IGuest_USCOREgetStatisticsUpdateIntervalResponse:
		return (void*)soap_instantiate__vbox__IGuest_USCOREgetStatisticsUpdateIntervalResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IGuest_USCOREsetStatisticsUpdateInterval:
		return (void*)soap_instantiate__vbox__IGuest_USCOREsetStatisticsUpdateInterval(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IGuest_USCOREsetStatisticsUpdateIntervalResponse:
		return (void*)soap_instantiate__vbox__IGuest_USCOREsetStatisticsUpdateIntervalResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IGuest_USCOREsetCredentials:
		return (void*)soap_instantiate__vbox__IGuest_USCOREsetCredentials(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IGuest_USCOREsetCredentialsResponse:
		return (void*)soap_instantiate__vbox__IGuest_USCOREsetCredentialsResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IGuest_USCOREgetStatistic:
		return (void*)soap_instantiate__vbox__IGuest_USCOREgetStatistic(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IGuest_USCOREgetStatisticResponse:
		return (void*)soap_instantiate__vbox__IGuest_USCOREgetStatisticResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetId:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetId(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetIdResponse:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetIdResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetDescription:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetDescriptionResponse:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetDescriptionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetInitiator:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetInitiator(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetInitiatorResponse:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetInitiatorResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetCancelable:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetCancelable(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetCancelableResponse:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetCancelableResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetPercent:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetPercent(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetPercentResponse:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetPercentResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetTimeRemaining:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetTimeRemaining(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetTimeRemainingResponse:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetTimeRemainingResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetCompleted:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetCompleted(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetCompletedResponse:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetCompletedResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetCanceled:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetCanceled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetCanceledResponse:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetCanceledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetResultCode:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetResultCode(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetResultCodeResponse:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetResultCodeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetErrorInfo:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetErrorInfo(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetErrorInfoResponse:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetErrorInfoResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetOperationCount:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetOperationCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetOperationCountResponse:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetOperationCountResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetOperation:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetOperation(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetOperationResponse:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetOperationResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetOperationDescription:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetOperationDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetOperationDescriptionResponse:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetOperationDescriptionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetOperationPercent:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetOperationPercent(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREgetOperationPercentResponse:
		return (void*)soap_instantiate__vbox__IProgress_USCOREgetOperationPercentResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREwaitForCompletion:
		return (void*)soap_instantiate__vbox__IProgress_USCOREwaitForCompletion(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREwaitForCompletionResponse:
		return (void*)soap_instantiate__vbox__IProgress_USCOREwaitForCompletionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREwaitForOperationCompletion:
		return (void*)soap_instantiate__vbox__IProgress_USCOREwaitForOperationCompletion(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREwaitForOperationCompletionResponse:
		return (void*)soap_instantiate__vbox__IProgress_USCOREwaitForOperationCompletionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREcancel:
		return (void*)soap_instantiate__vbox__IProgress_USCOREcancel(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IProgress_USCOREcancelResponse:
		return (void*)soap_instantiate__vbox__IProgress_USCOREcancelResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetId:
		return (void*)soap_instantiate__vbox__ISnapshot_USCOREgetId(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetIdResponse:
		return (void*)soap_instantiate__vbox__ISnapshot_USCOREgetIdResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetName:
		return (void*)soap_instantiate__vbox__ISnapshot_USCOREgetName(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetNameResponse:
		return (void*)soap_instantiate__vbox__ISnapshot_USCOREgetNameResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISnapshot_USCOREsetName:
		return (void*)soap_instantiate__vbox__ISnapshot_USCOREsetName(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISnapshot_USCOREsetNameResponse:
		return (void*)soap_instantiate__vbox__ISnapshot_USCOREsetNameResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetDescription:
		return (void*)soap_instantiate__vbox__ISnapshot_USCOREgetDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetDescriptionResponse:
		return (void*)soap_instantiate__vbox__ISnapshot_USCOREgetDescriptionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISnapshot_USCOREsetDescription:
		return (void*)soap_instantiate__vbox__ISnapshot_USCOREsetDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISnapshot_USCOREsetDescriptionResponse:
		return (void*)soap_instantiate__vbox__ISnapshot_USCOREsetDescriptionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetTimeStamp:
		return (void*)soap_instantiate__vbox__ISnapshot_USCOREgetTimeStamp(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetTimeStampResponse:
		return (void*)soap_instantiate__vbox__ISnapshot_USCOREgetTimeStampResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetOnline:
		return (void*)soap_instantiate__vbox__ISnapshot_USCOREgetOnline(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetOnlineResponse:
		return (void*)soap_instantiate__vbox__ISnapshot_USCOREgetOnlineResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetMachine:
		return (void*)soap_instantiate__vbox__ISnapshot_USCOREgetMachine(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetMachineResponse:
		return (void*)soap_instantiate__vbox__ISnapshot_USCOREgetMachineResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetParent:
		return (void*)soap_instantiate__vbox__ISnapshot_USCOREgetParent(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetParentResponse:
		return (void*)soap_instantiate__vbox__ISnapshot_USCOREgetParentResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetChildren:
		return (void*)soap_instantiate__vbox__ISnapshot_USCOREgetChildren(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISnapshot_USCOREgetChildrenResponse:
		return (void*)soap_instantiate__vbox__ISnapshot_USCOREgetChildrenResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREgetId:
		return (void*)soap_instantiate__vbox__IMedium_USCOREgetId(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREgetIdResponse:
		return (void*)soap_instantiate__vbox__IMedium_USCOREgetIdResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREgetDescription:
		return (void*)soap_instantiate__vbox__IMedium_USCOREgetDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREgetDescriptionResponse:
		return (void*)soap_instantiate__vbox__IMedium_USCOREgetDescriptionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREsetDescription:
		return (void*)soap_instantiate__vbox__IMedium_USCOREsetDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREsetDescriptionResponse:
		return (void*)soap_instantiate__vbox__IMedium_USCOREsetDescriptionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREgetState:
		return (void*)soap_instantiate__vbox__IMedium_USCOREgetState(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREgetStateResponse:
		return (void*)soap_instantiate__vbox__IMedium_USCOREgetStateResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREgetLocation:
		return (void*)soap_instantiate__vbox__IMedium_USCOREgetLocation(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREgetLocationResponse:
		return (void*)soap_instantiate__vbox__IMedium_USCOREgetLocationResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREsetLocation:
		return (void*)soap_instantiate__vbox__IMedium_USCOREsetLocation(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREsetLocationResponse:
		return (void*)soap_instantiate__vbox__IMedium_USCOREsetLocationResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREgetName:
		return (void*)soap_instantiate__vbox__IMedium_USCOREgetName(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREgetNameResponse:
		return (void*)soap_instantiate__vbox__IMedium_USCOREgetNameResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREgetSize:
		return (void*)soap_instantiate__vbox__IMedium_USCOREgetSize(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREgetSizeResponse:
		return (void*)soap_instantiate__vbox__IMedium_USCOREgetSizeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREgetLastAccessError:
		return (void*)soap_instantiate__vbox__IMedium_USCOREgetLastAccessError(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREgetLastAccessErrorResponse:
		return (void*)soap_instantiate__vbox__IMedium_USCOREgetLastAccessErrorResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREgetMachineIds:
		return (void*)soap_instantiate__vbox__IMedium_USCOREgetMachineIds(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREgetMachineIdsResponse:
		return (void*)soap_instantiate__vbox__IMedium_USCOREgetMachineIdsResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREgetSnapshotIds:
		return (void*)soap_instantiate__vbox__IMedium_USCOREgetSnapshotIds(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREgetSnapshotIdsResponse:
		return (void*)soap_instantiate__vbox__IMedium_USCOREgetSnapshotIdsResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCORElockRead:
		return (void*)soap_instantiate__vbox__IMedium_USCORElockRead(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCORElockReadResponse:
		return (void*)soap_instantiate__vbox__IMedium_USCORElockReadResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREunlockRead:
		return (void*)soap_instantiate__vbox__IMedium_USCOREunlockRead(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREunlockReadResponse:
		return (void*)soap_instantiate__vbox__IMedium_USCOREunlockReadResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCORElockWrite:
		return (void*)soap_instantiate__vbox__IMedium_USCORElockWrite(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCORElockWriteResponse:
		return (void*)soap_instantiate__vbox__IMedium_USCORElockWriteResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREunlockWrite:
		return (void*)soap_instantiate__vbox__IMedium_USCOREunlockWrite(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREunlockWriteResponse:
		return (void*)soap_instantiate__vbox__IMedium_USCOREunlockWriteResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREclose:
		return (void*)soap_instantiate__vbox__IMedium_USCOREclose(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMedium_USCOREcloseResponse:
		return (void*)soap_instantiate__vbox__IMedium_USCOREcloseResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetFormat:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREgetFormat(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetFormatResponse:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREgetFormatResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetType:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREgetType(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetTypeResponse:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREgetTypeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREsetType:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREsetType(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREsetTypeResponse:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREsetTypeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetParent:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREgetParent(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetParentResponse:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREgetParentResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetChildren:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREgetChildren(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetChildrenResponse:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREgetChildrenResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetRoot:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREgetRoot(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetRootResponse:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREgetRootResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetReadOnly:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREgetReadOnly(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetReadOnlyResponse:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREgetReadOnlyResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetLogicalSize:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREgetLogicalSize(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetLogicalSizeResponse:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREgetLogicalSizeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetAutoReset:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREgetAutoReset(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetAutoResetResponse:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREgetAutoResetResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREsetAutoReset:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREsetAutoReset(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREsetAutoResetResponse:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREsetAutoResetResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetProperty:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREgetProperty(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetPropertyResponse:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREgetPropertyResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREsetProperty:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREsetProperty(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREsetPropertyResponse:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREsetPropertyResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetProperties:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREgetProperties(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREgetPropertiesResponse:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREgetPropertiesResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREsetProperties:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREsetProperties(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREsetPropertiesResponse:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREsetPropertiesResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREcreateBaseStorage:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREcreateBaseStorage(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREcreateBaseStorageResponse:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREcreateBaseStorageResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREdeleteStorage:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREdeleteStorage(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREdeleteStorageResponse:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREdeleteStorageResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREcreateDiffStorage:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREcreateDiffStorage(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREcreateDiffStorageResponse:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREcreateDiffStorageResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREmergeTo:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREmergeTo(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREmergeToResponse:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREmergeToResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREcloneTo:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREcloneTo(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREcloneToResponse:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREcloneToResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREcompact:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREcompact(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREcompactResponse:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREcompactResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREreset:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREreset(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDisk_USCOREresetResponse:
		return (void*)soap_instantiate__vbox__IHardDisk_USCOREresetResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREgetId:
		return (void*)soap_instantiate__vbox__IHardDiskFormat_USCOREgetId(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREgetIdResponse:
		return (void*)soap_instantiate__vbox__IHardDiskFormat_USCOREgetIdResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREgetName:
		return (void*)soap_instantiate__vbox__IHardDiskFormat_USCOREgetName(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREgetNameResponse:
		return (void*)soap_instantiate__vbox__IHardDiskFormat_USCOREgetNameResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREgetFileExtensions:
		return (void*)soap_instantiate__vbox__IHardDiskFormat_USCOREgetFileExtensions(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREgetFileExtensionsResponse:
		return (void*)soap_instantiate__vbox__IHardDiskFormat_USCOREgetFileExtensionsResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREgetCapabilities:
		return (void*)soap_instantiate__vbox__IHardDiskFormat_USCOREgetCapabilities(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREgetCapabilitiesResponse:
		return (void*)soap_instantiate__vbox__IHardDiskFormat_USCOREgetCapabilitiesResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREdescribeProperties:
		return (void*)soap_instantiate__vbox__IHardDiskFormat_USCOREdescribeProperties(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHardDiskFormat_USCOREdescribePropertiesResponse:
		return (void*)soap_instantiate__vbox__IHardDiskFormat_USCOREdescribePropertiesResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREgetState:
		return (void*)soap_instantiate__vbox__IDVDDrive_USCOREgetState(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREgetStateResponse:
		return (void*)soap_instantiate__vbox__IDVDDrive_USCOREgetStateResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREgetPassthrough:
		return (void*)soap_instantiate__vbox__IDVDDrive_USCOREgetPassthrough(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREgetPassthroughResponse:
		return (void*)soap_instantiate__vbox__IDVDDrive_USCOREgetPassthroughResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREsetPassthrough:
		return (void*)soap_instantiate__vbox__IDVDDrive_USCOREsetPassthrough(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREsetPassthroughResponse:
		return (void*)soap_instantiate__vbox__IDVDDrive_USCOREsetPassthroughResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREmountImage:
		return (void*)soap_instantiate__vbox__IDVDDrive_USCOREmountImage(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREmountImageResponse:
		return (void*)soap_instantiate__vbox__IDVDDrive_USCOREmountImageResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREcaptureHostDrive:
		return (void*)soap_instantiate__vbox__IDVDDrive_USCOREcaptureHostDrive(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREcaptureHostDriveResponse:
		return (void*)soap_instantiate__vbox__IDVDDrive_USCOREcaptureHostDriveResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREunmount:
		return (void*)soap_instantiate__vbox__IDVDDrive_USCOREunmount(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREunmountResponse:
		return (void*)soap_instantiate__vbox__IDVDDrive_USCOREunmountResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREgetImage:
		return (void*)soap_instantiate__vbox__IDVDDrive_USCOREgetImage(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREgetImageResponse:
		return (void*)soap_instantiate__vbox__IDVDDrive_USCOREgetImageResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREgetHostDrive:
		return (void*)soap_instantiate__vbox__IDVDDrive_USCOREgetHostDrive(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDVDDrive_USCOREgetHostDriveResponse:
		return (void*)soap_instantiate__vbox__IDVDDrive_USCOREgetHostDriveResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREgetEnabled:
		return (void*)soap_instantiate__vbox__IFloppyDrive_USCOREgetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREgetEnabledResponse:
		return (void*)soap_instantiate__vbox__IFloppyDrive_USCOREgetEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREsetEnabled:
		return (void*)soap_instantiate__vbox__IFloppyDrive_USCOREsetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREsetEnabledResponse:
		return (void*)soap_instantiate__vbox__IFloppyDrive_USCOREsetEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREgetState:
		return (void*)soap_instantiate__vbox__IFloppyDrive_USCOREgetState(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREgetStateResponse:
		return (void*)soap_instantiate__vbox__IFloppyDrive_USCOREgetStateResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREmountImage:
		return (void*)soap_instantiate__vbox__IFloppyDrive_USCOREmountImage(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREmountImageResponse:
		return (void*)soap_instantiate__vbox__IFloppyDrive_USCOREmountImageResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREcaptureHostDrive:
		return (void*)soap_instantiate__vbox__IFloppyDrive_USCOREcaptureHostDrive(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREcaptureHostDriveResponse:
		return (void*)soap_instantiate__vbox__IFloppyDrive_USCOREcaptureHostDriveResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREunmount:
		return (void*)soap_instantiate__vbox__IFloppyDrive_USCOREunmount(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREunmountResponse:
		return (void*)soap_instantiate__vbox__IFloppyDrive_USCOREunmountResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREgetImage:
		return (void*)soap_instantiate__vbox__IFloppyDrive_USCOREgetImage(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREgetImageResponse:
		return (void*)soap_instantiate__vbox__IFloppyDrive_USCOREgetImageResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREgetHostDrive:
		return (void*)soap_instantiate__vbox__IFloppyDrive_USCOREgetHostDrive(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IFloppyDrive_USCOREgetHostDriveResponse:
		return (void*)soap_instantiate__vbox__IFloppyDrive_USCOREgetHostDriveResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IKeyboard_USCOREputScancode:
		return (void*)soap_instantiate__vbox__IKeyboard_USCOREputScancode(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IKeyboard_USCOREputScancodeResponse:
		return (void*)soap_instantiate__vbox__IKeyboard_USCOREputScancodeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IKeyboard_USCOREputScancodes:
		return (void*)soap_instantiate__vbox__IKeyboard_USCOREputScancodes(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IKeyboard_USCOREputScancodesResponse:
		return (void*)soap_instantiate__vbox__IKeyboard_USCOREputScancodesResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IKeyboard_USCOREputCAD:
		return (void*)soap_instantiate__vbox__IKeyboard_USCOREputCAD(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IKeyboard_USCOREputCADResponse:
		return (void*)soap_instantiate__vbox__IKeyboard_USCOREputCADResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMouse_USCOREgetAbsoluteSupported:
		return (void*)soap_instantiate__vbox__IMouse_USCOREgetAbsoluteSupported(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMouse_USCOREgetAbsoluteSupportedResponse:
		return (void*)soap_instantiate__vbox__IMouse_USCOREgetAbsoluteSupportedResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMouse_USCOREputMouseEvent:
		return (void*)soap_instantiate__vbox__IMouse_USCOREputMouseEvent(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMouse_USCOREputMouseEventResponse:
		return (void*)soap_instantiate__vbox__IMouse_USCOREputMouseEventResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMouse_USCOREputMouseEventAbsolute:
		return (void*)soap_instantiate__vbox__IMouse_USCOREputMouseEventAbsolute(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IMouse_USCOREputMouseEventAbsoluteResponse:
		return (void*)soap_instantiate__vbox__IMouse_USCOREputMouseEventAbsoluteResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDisplay_USCOREgetWidth:
		return (void*)soap_instantiate__vbox__IDisplay_USCOREgetWidth(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDisplay_USCOREgetWidthResponse:
		return (void*)soap_instantiate__vbox__IDisplay_USCOREgetWidthResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDisplay_USCOREgetHeight:
		return (void*)soap_instantiate__vbox__IDisplay_USCOREgetHeight(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDisplay_USCOREgetHeightResponse:
		return (void*)soap_instantiate__vbox__IDisplay_USCOREgetHeightResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDisplay_USCOREgetBitsPerPixel:
		return (void*)soap_instantiate__vbox__IDisplay_USCOREgetBitsPerPixel(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDisplay_USCOREgetBitsPerPixelResponse:
		return (void*)soap_instantiate__vbox__IDisplay_USCOREgetBitsPerPixelResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDisplay_USCOREsetVideoModeHint:
		return (void*)soap_instantiate__vbox__IDisplay_USCOREsetVideoModeHint(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDisplay_USCOREsetVideoModeHintResponse:
		return (void*)soap_instantiate__vbox__IDisplay_USCOREsetVideoModeHintResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDisplay_USCOREsetSeamlessMode:
		return (void*)soap_instantiate__vbox__IDisplay_USCOREsetSeamlessMode(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDisplay_USCOREsetSeamlessModeResponse:
		return (void*)soap_instantiate__vbox__IDisplay_USCOREsetSeamlessModeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDisplay_USCOREtakeScreenShotSlow:
		return (void*)soap_instantiate__vbox__IDisplay_USCOREtakeScreenShotSlow(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDisplay_USCOREtakeScreenShotSlowResponse:
		return (void*)soap_instantiate__vbox__IDisplay_USCOREtakeScreenShotSlowResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDisplay_USCOREinvalidateAndUpdate:
		return (void*)soap_instantiate__vbox__IDisplay_USCOREinvalidateAndUpdate(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDisplay_USCOREinvalidateAndUpdateResponse:
		return (void*)soap_instantiate__vbox__IDisplay_USCOREinvalidateAndUpdateResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDisplay_USCOREresizeCompleted:
		return (void*)soap_instantiate__vbox__IDisplay_USCOREresizeCompleted(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDisplay_USCOREresizeCompletedResponse:
		return (void*)soap_instantiate__vbox__IDisplay_USCOREresizeCompletedResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDisplay_USCOREupdateCompleted:
		return (void*)soap_instantiate__vbox__IDisplay_USCOREupdateCompleted(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IDisplay_USCOREupdateCompletedResponse:
		return (void*)soap_instantiate__vbox__IDisplay_USCOREupdateCompletedResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetAdapterType:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREgetAdapterType(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetAdapterTypeResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREgetAdapterTypeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetAdapterType:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREsetAdapterType(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetAdapterTypeResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREsetAdapterTypeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetSlot:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREgetSlot(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetSlotResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREgetSlotResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetEnabled:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREgetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetEnabledResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREgetEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetEnabled:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREsetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetEnabledResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREsetEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetMACAddress:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREgetMACAddress(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetMACAddressResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREgetMACAddressResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetMACAddress:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREsetMACAddress(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetMACAddressResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREsetMACAddressResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetAttachmentType:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREgetAttachmentType(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetAttachmentTypeResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREgetAttachmentTypeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetHostInterface:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREgetHostInterface(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetHostInterfaceResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREgetHostInterfaceResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetHostInterface:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREsetHostInterface(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetHostInterfaceResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREsetHostInterfaceResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetInternalNetwork:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREgetInternalNetwork(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetInternalNetworkResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREgetInternalNetworkResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetInternalNetwork:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREsetInternalNetwork(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetInternalNetworkResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREsetInternalNetworkResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetNATNetwork:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREgetNATNetwork(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetNATNetworkResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREgetNATNetworkResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetNATNetwork:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREsetNATNetwork(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetNATNetworkResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREsetNATNetworkResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetCableConnected:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREgetCableConnected(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetCableConnectedResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREgetCableConnectedResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetCableConnected:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREsetCableConnected(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetCableConnectedResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREsetCableConnectedResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetLineSpeed:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREgetLineSpeed(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetLineSpeedResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREgetLineSpeedResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetLineSpeed:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREsetLineSpeed(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetLineSpeedResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREsetLineSpeedResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetTraceEnabled:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREgetTraceEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetTraceEnabledResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREgetTraceEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetTraceEnabled:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREsetTraceEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetTraceEnabledResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREsetTraceEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetTraceFile:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREgetTraceFile(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREgetTraceFileResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREgetTraceFileResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetTraceFile:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREsetTraceFile(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREsetTraceFileResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREsetTraceFileResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREattachToNAT:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREattachToNAT(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREattachToNATResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREattachToNATResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREattachToBridgedInterface:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREattachToBridgedInterface(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREattachToBridgedInterfaceResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREattachToBridgedInterfaceResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREattachToInternalNetwork:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREattachToInternalNetwork(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREattachToInternalNetworkResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREattachToInternalNetworkResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREattachToHostOnlyInterface:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREattachToHostOnlyInterface(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREattachToHostOnlyInterfaceResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREattachToHostOnlyInterfaceResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREdetach:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREdetach(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__INetworkAdapter_USCOREdetachResponse:
		return (void*)soap_instantiate__vbox__INetworkAdapter_USCOREdetachResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetSlot:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREgetSlot(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetSlotResponse:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREgetSlotResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetEnabled:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREgetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetEnabledResponse:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREgetEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetEnabled:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREsetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetEnabledResponse:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREsetEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetIOBase:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREgetIOBase(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetIOBaseResponse:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREgetIOBaseResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetIOBase:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREsetIOBase(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetIOBaseResponse:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREsetIOBaseResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetIRQ:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREgetIRQ(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetIRQResponse:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREgetIRQResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetIRQ:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREsetIRQ(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetIRQResponse:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREsetIRQResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetHostMode:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREgetHostMode(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetHostModeResponse:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREgetHostModeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetHostMode:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREsetHostMode(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetHostModeResponse:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREsetHostModeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetServer:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREgetServer(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetServerResponse:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREgetServerResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetServer:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREsetServer(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetServerResponse:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREsetServerResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetPath:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREgetPath(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREgetPathResponse:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREgetPathResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetPath:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREsetPath(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISerialPort_USCOREsetPathResponse:
		return (void*)soap_instantiate__vbox__ISerialPort_USCOREsetPathResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetSlot:
		return (void*)soap_instantiate__vbox__IParallelPort_USCOREgetSlot(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetSlotResponse:
		return (void*)soap_instantiate__vbox__IParallelPort_USCOREgetSlotResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetEnabled:
		return (void*)soap_instantiate__vbox__IParallelPort_USCOREgetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetEnabledResponse:
		return (void*)soap_instantiate__vbox__IParallelPort_USCOREgetEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IParallelPort_USCOREsetEnabled:
		return (void*)soap_instantiate__vbox__IParallelPort_USCOREsetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IParallelPort_USCOREsetEnabledResponse:
		return (void*)soap_instantiate__vbox__IParallelPort_USCOREsetEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetIOBase:
		return (void*)soap_instantiate__vbox__IParallelPort_USCOREgetIOBase(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetIOBaseResponse:
		return (void*)soap_instantiate__vbox__IParallelPort_USCOREgetIOBaseResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IParallelPort_USCOREsetIOBase:
		return (void*)soap_instantiate__vbox__IParallelPort_USCOREsetIOBase(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IParallelPort_USCOREsetIOBaseResponse:
		return (void*)soap_instantiate__vbox__IParallelPort_USCOREsetIOBaseResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetIRQ:
		return (void*)soap_instantiate__vbox__IParallelPort_USCOREgetIRQ(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetIRQResponse:
		return (void*)soap_instantiate__vbox__IParallelPort_USCOREgetIRQResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IParallelPort_USCOREsetIRQ:
		return (void*)soap_instantiate__vbox__IParallelPort_USCOREsetIRQ(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IParallelPort_USCOREsetIRQResponse:
		return (void*)soap_instantiate__vbox__IParallelPort_USCOREsetIRQResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetPath:
		return (void*)soap_instantiate__vbox__IParallelPort_USCOREgetPath(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IParallelPort_USCOREgetPathResponse:
		return (void*)soap_instantiate__vbox__IParallelPort_USCOREgetPathResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IParallelPort_USCOREsetPath:
		return (void*)soap_instantiate__vbox__IParallelPort_USCOREsetPath(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IParallelPort_USCOREsetPathResponse:
		return (void*)soap_instantiate__vbox__IParallelPort_USCOREsetPathResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBController_USCOREgetEnabled:
		return (void*)soap_instantiate__vbox__IUSBController_USCOREgetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBController_USCOREgetEnabledResponse:
		return (void*)soap_instantiate__vbox__IUSBController_USCOREgetEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBController_USCOREsetEnabled:
		return (void*)soap_instantiate__vbox__IUSBController_USCOREsetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBController_USCOREsetEnabledResponse:
		return (void*)soap_instantiate__vbox__IUSBController_USCOREsetEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBController_USCOREgetEnabledEhci:
		return (void*)soap_instantiate__vbox__IUSBController_USCOREgetEnabledEhci(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBController_USCOREgetEnabledEhciResponse:
		return (void*)soap_instantiate__vbox__IUSBController_USCOREgetEnabledEhciResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBController_USCOREsetEnabledEhci:
		return (void*)soap_instantiate__vbox__IUSBController_USCOREsetEnabledEhci(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBController_USCOREsetEnabledEhciResponse:
		return (void*)soap_instantiate__vbox__IUSBController_USCOREsetEnabledEhciResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBController_USCOREgetUSBStandard:
		return (void*)soap_instantiate__vbox__IUSBController_USCOREgetUSBStandard(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBController_USCOREgetUSBStandardResponse:
		return (void*)soap_instantiate__vbox__IUSBController_USCOREgetUSBStandardResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBController_USCOREgetDeviceFilters:
		return (void*)soap_instantiate__vbox__IUSBController_USCOREgetDeviceFilters(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBController_USCOREgetDeviceFiltersResponse:
		return (void*)soap_instantiate__vbox__IUSBController_USCOREgetDeviceFiltersResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBController_USCOREcreateDeviceFilter:
		return (void*)soap_instantiate__vbox__IUSBController_USCOREcreateDeviceFilter(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBController_USCOREcreateDeviceFilterResponse:
		return (void*)soap_instantiate__vbox__IUSBController_USCOREcreateDeviceFilterResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBController_USCOREinsertDeviceFilter:
		return (void*)soap_instantiate__vbox__IUSBController_USCOREinsertDeviceFilter(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBController_USCOREinsertDeviceFilterResponse:
		return (void*)soap_instantiate__vbox__IUSBController_USCOREinsertDeviceFilterResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBController_USCOREremoveDeviceFilter:
		return (void*)soap_instantiate__vbox__IUSBController_USCOREremoveDeviceFilter(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBController_USCOREremoveDeviceFilterResponse:
		return (void*)soap_instantiate__vbox__IUSBController_USCOREremoveDeviceFilterResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetId:
		return (void*)soap_instantiate__vbox__IUSBDevice_USCOREgetId(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetIdResponse:
		return (void*)soap_instantiate__vbox__IUSBDevice_USCOREgetIdResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetVendorId:
		return (void*)soap_instantiate__vbox__IUSBDevice_USCOREgetVendorId(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetVendorIdResponse:
		return (void*)soap_instantiate__vbox__IUSBDevice_USCOREgetVendorIdResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetProductId:
		return (void*)soap_instantiate__vbox__IUSBDevice_USCOREgetProductId(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetProductIdResponse:
		return (void*)soap_instantiate__vbox__IUSBDevice_USCOREgetProductIdResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetRevision:
		return (void*)soap_instantiate__vbox__IUSBDevice_USCOREgetRevision(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetRevisionResponse:
		return (void*)soap_instantiate__vbox__IUSBDevice_USCOREgetRevisionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetManufacturer:
		return (void*)soap_instantiate__vbox__IUSBDevice_USCOREgetManufacturer(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetManufacturerResponse:
		return (void*)soap_instantiate__vbox__IUSBDevice_USCOREgetManufacturerResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetProduct:
		return (void*)soap_instantiate__vbox__IUSBDevice_USCOREgetProduct(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetProductResponse:
		return (void*)soap_instantiate__vbox__IUSBDevice_USCOREgetProductResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetSerialNumber:
		return (void*)soap_instantiate__vbox__IUSBDevice_USCOREgetSerialNumber(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetSerialNumberResponse:
		return (void*)soap_instantiate__vbox__IUSBDevice_USCOREgetSerialNumberResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetAddress:
		return (void*)soap_instantiate__vbox__IUSBDevice_USCOREgetAddress(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetAddressResponse:
		return (void*)soap_instantiate__vbox__IUSBDevice_USCOREgetAddressResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetPort:
		return (void*)soap_instantiate__vbox__IUSBDevice_USCOREgetPort(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetPortResponse:
		return (void*)soap_instantiate__vbox__IUSBDevice_USCOREgetPortResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetVersion:
		return (void*)soap_instantiate__vbox__IUSBDevice_USCOREgetVersion(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetVersionResponse:
		return (void*)soap_instantiate__vbox__IUSBDevice_USCOREgetVersionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetPortVersion:
		return (void*)soap_instantiate__vbox__IUSBDevice_USCOREgetPortVersion(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetPortVersionResponse:
		return (void*)soap_instantiate__vbox__IUSBDevice_USCOREgetPortVersionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetRemote:
		return (void*)soap_instantiate__vbox__IUSBDevice_USCOREgetRemote(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDevice_USCOREgetRemoteResponse:
		return (void*)soap_instantiate__vbox__IUSBDevice_USCOREgetRemoteResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetName:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREgetName(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetNameResponse:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREgetNameResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetName:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREsetName(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetNameResponse:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREsetNameResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetActive:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREgetActive(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetActiveResponse:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREgetActiveResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetActive:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREsetActive(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetActiveResponse:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREsetActiveResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetVendorId:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREgetVendorId(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetVendorIdResponse:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREgetVendorIdResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetVendorId:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREsetVendorId(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetVendorIdResponse:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREsetVendorIdResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetProductId:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREgetProductId(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetProductIdResponse:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREgetProductIdResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetProductId:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREsetProductId(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetProductIdResponse:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREsetProductIdResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetRevision:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREgetRevision(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetRevisionResponse:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREgetRevisionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetRevision:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREsetRevision(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetRevisionResponse:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREsetRevisionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetManufacturer:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREgetManufacturer(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetManufacturerResponse:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREgetManufacturerResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetManufacturer:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREsetManufacturer(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetManufacturerResponse:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREsetManufacturerResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetProduct:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREgetProduct(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetProductResponse:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREgetProductResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetProduct:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREsetProduct(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetProductResponse:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREsetProductResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetSerialNumber:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREgetSerialNumber(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetSerialNumberResponse:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREgetSerialNumberResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetSerialNumber:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREsetSerialNumber(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetSerialNumberResponse:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREsetSerialNumberResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetPort:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREgetPort(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetPortResponse:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREgetPortResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetPort:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREsetPort(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetPortResponse:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREsetPortResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetRemote:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREgetRemote(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetRemoteResponse:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREgetRemoteResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetRemote:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREsetRemote(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetRemoteResponse:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREsetRemoteResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetMaskedInterfaces:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREgetMaskedInterfaces(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREgetMaskedInterfacesResponse:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREgetMaskedInterfacesResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetMaskedInterfaces:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREsetMaskedInterfaces(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IUSBDeviceFilter_USCOREsetMaskedInterfacesResponse:
		return (void*)soap_instantiate__vbox__IUSBDeviceFilter_USCOREsetMaskedInterfacesResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostUSBDevice_USCOREgetState:
		return (void*)soap_instantiate__vbox__IHostUSBDevice_USCOREgetState(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostUSBDevice_USCOREgetStateResponse:
		return (void*)soap_instantiate__vbox__IHostUSBDevice_USCOREgetStateResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostUSBDeviceFilter_USCOREgetAction:
		return (void*)soap_instantiate__vbox__IHostUSBDeviceFilter_USCOREgetAction(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostUSBDeviceFilter_USCOREgetActionResponse:
		return (void*)soap_instantiate__vbox__IHostUSBDeviceFilter_USCOREgetActionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostUSBDeviceFilter_USCOREsetAction:
		return (void*)soap_instantiate__vbox__IHostUSBDeviceFilter_USCOREsetAction(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IHostUSBDeviceFilter_USCOREsetActionResponse:
		return (void*)soap_instantiate__vbox__IHostUSBDeviceFilter_USCOREsetActionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREgetEnabled:
		return (void*)soap_instantiate__vbox__IAudioAdapter_USCOREgetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREgetEnabledResponse:
		return (void*)soap_instantiate__vbox__IAudioAdapter_USCOREgetEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREsetEnabled:
		return (void*)soap_instantiate__vbox__IAudioAdapter_USCOREsetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREsetEnabledResponse:
		return (void*)soap_instantiate__vbox__IAudioAdapter_USCOREsetEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREgetAudioController:
		return (void*)soap_instantiate__vbox__IAudioAdapter_USCOREgetAudioController(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREgetAudioControllerResponse:
		return (void*)soap_instantiate__vbox__IAudioAdapter_USCOREgetAudioControllerResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREsetAudioController:
		return (void*)soap_instantiate__vbox__IAudioAdapter_USCOREsetAudioController(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREsetAudioControllerResponse:
		return (void*)soap_instantiate__vbox__IAudioAdapter_USCOREsetAudioControllerResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREgetAudioDriver:
		return (void*)soap_instantiate__vbox__IAudioAdapter_USCOREgetAudioDriver(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREgetAudioDriverResponse:
		return (void*)soap_instantiate__vbox__IAudioAdapter_USCOREgetAudioDriverResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREsetAudioDriver:
		return (void*)soap_instantiate__vbox__IAudioAdapter_USCOREsetAudioDriver(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IAudioAdapter_USCOREsetAudioDriverResponse:
		return (void*)soap_instantiate__vbox__IAudioAdapter_USCOREsetAudioDriverResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetEnabled:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREgetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetEnabledResponse:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREgetEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetEnabled:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREsetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetEnabledResponse:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREsetEnabledResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetPort:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREgetPort(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetPortResponse:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREgetPortResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetPort:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREsetPort(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetPortResponse:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREsetPortResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetNetAddress:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREgetNetAddress(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetNetAddressResponse:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREgetNetAddressResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetNetAddress:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREsetNetAddress(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetNetAddressResponse:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREsetNetAddressResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetAuthType:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREgetAuthType(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetAuthTypeResponse:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREgetAuthTypeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetAuthType:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREsetAuthType(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetAuthTypeResponse:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREsetAuthTypeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetAuthTimeout:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREgetAuthTimeout(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetAuthTimeoutResponse:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREgetAuthTimeoutResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetAuthTimeout:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREsetAuthTimeout(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetAuthTimeoutResponse:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREsetAuthTimeoutResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetAllowMultiConnection:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREgetAllowMultiConnection(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetAllowMultiConnectionResponse:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREgetAllowMultiConnectionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetAllowMultiConnection:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREsetAllowMultiConnection(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetAllowMultiConnectionResponse:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREsetAllowMultiConnectionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetReuseSingleConnection:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREgetReuseSingleConnection(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREgetReuseSingleConnectionResponse:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREgetReuseSingleConnectionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetReuseSingleConnection:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREsetReuseSingleConnection(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IVRDPServer_USCOREsetReuseSingleConnectionResponse:
		return (void*)soap_instantiate__vbox__IVRDPServer_USCOREsetReuseSingleConnectionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISession_USCOREgetState:
		return (void*)soap_instantiate__vbox__ISession_USCOREgetState(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISession_USCOREgetStateResponse:
		return (void*)soap_instantiate__vbox__ISession_USCOREgetStateResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISession_USCOREgetType:
		return (void*)soap_instantiate__vbox__ISession_USCOREgetType(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISession_USCOREgetTypeResponse:
		return (void*)soap_instantiate__vbox__ISession_USCOREgetTypeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISession_USCOREgetMachine:
		return (void*)soap_instantiate__vbox__ISession_USCOREgetMachine(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISession_USCOREgetMachineResponse:
		return (void*)soap_instantiate__vbox__ISession_USCOREgetMachineResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISession_USCOREgetConsole:
		return (void*)soap_instantiate__vbox__ISession_USCOREgetConsole(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISession_USCOREgetConsoleResponse:
		return (void*)soap_instantiate__vbox__ISession_USCOREgetConsoleResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISession_USCOREclose:
		return (void*)soap_instantiate__vbox__ISession_USCOREclose(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__ISession_USCOREcloseResponse:
		return (void*)soap_instantiate__vbox__ISession_USCOREcloseResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetName:
		return (void*)soap_instantiate__vbox__IStorageController_USCOREgetName(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetNameResponse:
		return (void*)soap_instantiate__vbox__IStorageController_USCOREgetNameResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetMaxDevicesPerPortCount:
		return (void*)soap_instantiate__vbox__IStorageController_USCOREgetMaxDevicesPerPortCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetMaxDevicesPerPortCountResponse:
		return (void*)soap_instantiate__vbox__IStorageController_USCOREgetMaxDevicesPerPortCountResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetMinPortCount:
		return (void*)soap_instantiate__vbox__IStorageController_USCOREgetMinPortCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetMinPortCountResponse:
		return (void*)soap_instantiate__vbox__IStorageController_USCOREgetMinPortCountResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetMaxPortCount:
		return (void*)soap_instantiate__vbox__IStorageController_USCOREgetMaxPortCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetMaxPortCountResponse:
		return (void*)soap_instantiate__vbox__IStorageController_USCOREgetMaxPortCountResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetInstance:
		return (void*)soap_instantiate__vbox__IStorageController_USCOREgetInstance(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetInstanceResponse:
		return (void*)soap_instantiate__vbox__IStorageController_USCOREgetInstanceResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCOREsetInstance:
		return (void*)soap_instantiate__vbox__IStorageController_USCOREsetInstance(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCOREsetInstanceResponse:
		return (void*)soap_instantiate__vbox__IStorageController_USCOREsetInstanceResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetPortCount:
		return (void*)soap_instantiate__vbox__IStorageController_USCOREgetPortCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetPortCountResponse:
		return (void*)soap_instantiate__vbox__IStorageController_USCOREgetPortCountResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCOREsetPortCount:
		return (void*)soap_instantiate__vbox__IStorageController_USCOREsetPortCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCOREsetPortCountResponse:
		return (void*)soap_instantiate__vbox__IStorageController_USCOREsetPortCountResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetBus:
		return (void*)soap_instantiate__vbox__IStorageController_USCOREgetBus(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetBusResponse:
		return (void*)soap_instantiate__vbox__IStorageController_USCOREgetBusResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetControllerType:
		return (void*)soap_instantiate__vbox__IStorageController_USCOREgetControllerType(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCOREgetControllerTypeResponse:
		return (void*)soap_instantiate__vbox__IStorageController_USCOREgetControllerTypeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCOREsetControllerType:
		return (void*)soap_instantiate__vbox__IStorageController_USCOREsetControllerType(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCOREsetControllerTypeResponse:
		return (void*)soap_instantiate__vbox__IStorageController_USCOREsetControllerTypeResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCOREGetIDEEmulationPort:
		return (void*)soap_instantiate__vbox__IStorageController_USCOREGetIDEEmulationPort(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCOREGetIDEEmulationPortResponse:
		return (void*)soap_instantiate__vbox__IStorageController_USCOREGetIDEEmulationPortResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCORESetIDEEmulationPort:
		return (void*)soap_instantiate__vbox__IStorageController_USCORESetIDEEmulationPort(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IStorageController_USCORESetIDEEmulationPortResponse:
		return (void*)soap_instantiate__vbox__IStorageController_USCORESetIDEEmulationPortResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IManagedObjectRef_USCOREgetInterfaceName:
		return (void*)soap_instantiate__vbox__IManagedObjectRef_USCOREgetInterfaceName(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IManagedObjectRef_USCOREgetInterfaceNameResponse:
		return (void*)soap_instantiate__vbox__IManagedObjectRef_USCOREgetInterfaceNameResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IManagedObjectRef_USCORErelease:
		return (void*)soap_instantiate__vbox__IManagedObjectRef_USCORErelease(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IManagedObjectRef_USCOREreleaseResponse:
		return (void*)soap_instantiate__vbox__IManagedObjectRef_USCOREreleaseResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IWebsessionManager_USCORElogon:
		return (void*)soap_instantiate__vbox__IWebsessionManager_USCORElogon(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IWebsessionManager_USCORElogonResponse:
		return (void*)soap_instantiate__vbox__IWebsessionManager_USCORElogonResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IWebsessionManager_USCOREgetSessionObject:
		return (void*)soap_instantiate__vbox__IWebsessionManager_USCOREgetSessionObject(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IWebsessionManager_USCOREgetSessionObjectResponse:
		return (void*)soap_instantiate__vbox__IWebsessionManager_USCOREgetSessionObjectResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IWebsessionManager_USCORElogoff:
		return (void*)soap_instantiate__vbox__IWebsessionManager_USCORElogoff(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IWebsessionManager_USCORElogoffResponse:
		return (void*)soap_instantiate__vbox__IWebsessionManager_USCORElogoffResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetMetricName:
		return (void*)soap_instantiate__vbox__IPerformanceMetric_USCOREgetMetricName(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetMetricNameResponse:
		return (void*)soap_instantiate__vbox__IPerformanceMetric_USCOREgetMetricNameResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetObject:
		return (void*)soap_instantiate__vbox__IPerformanceMetric_USCOREgetObject(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetObjectResponse:
		return (void*)soap_instantiate__vbox__IPerformanceMetric_USCOREgetObjectResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetDescription:
		return (void*)soap_instantiate__vbox__IPerformanceMetric_USCOREgetDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetDescriptionResponse:
		return (void*)soap_instantiate__vbox__IPerformanceMetric_USCOREgetDescriptionResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetPeriod:
		return (void*)soap_instantiate__vbox__IPerformanceMetric_USCOREgetPeriod(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetPeriodResponse:
		return (void*)soap_instantiate__vbox__IPerformanceMetric_USCOREgetPeriodResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetCount:
		return (void*)soap_instantiate__vbox__IPerformanceMetric_USCOREgetCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetCountResponse:
		return (void*)soap_instantiate__vbox__IPerformanceMetric_USCOREgetCountResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetUnit:
		return (void*)soap_instantiate__vbox__IPerformanceMetric_USCOREgetUnit(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetUnitResponse:
		return (void*)soap_instantiate__vbox__IPerformanceMetric_USCOREgetUnitResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetMinimumValue:
		return (void*)soap_instantiate__vbox__IPerformanceMetric_USCOREgetMinimumValue(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetMinimumValueResponse:
		return (void*)soap_instantiate__vbox__IPerformanceMetric_USCOREgetMinimumValueResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetMaximumValue:
		return (void*)soap_instantiate__vbox__IPerformanceMetric_USCOREgetMaximumValue(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceMetric_USCOREgetMaximumValueResponse:
		return (void*)soap_instantiate__vbox__IPerformanceMetric_USCOREgetMaximumValueResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREgetMetricNames:
		return (void*)soap_instantiate__vbox__IPerformanceCollector_USCOREgetMetricNames(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREgetMetricNamesResponse:
		return (void*)soap_instantiate__vbox__IPerformanceCollector_USCOREgetMetricNamesResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREgetMetrics:
		return (void*)soap_instantiate__vbox__IPerformanceCollector_USCOREgetMetrics(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREgetMetricsResponse:
		return (void*)soap_instantiate__vbox__IPerformanceCollector_USCOREgetMetricsResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREsetupMetrics:
		return (void*)soap_instantiate__vbox__IPerformanceCollector_USCOREsetupMetrics(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREsetupMetricsResponse:
		return (void*)soap_instantiate__vbox__IPerformanceCollector_USCOREsetupMetricsResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREenableMetrics:
		return (void*)soap_instantiate__vbox__IPerformanceCollector_USCOREenableMetrics(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREenableMetricsResponse:
		return (void*)soap_instantiate__vbox__IPerformanceCollector_USCOREenableMetricsResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREdisableMetrics:
		return (void*)soap_instantiate__vbox__IPerformanceCollector_USCOREdisableMetrics(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREdisableMetricsResponse:
		return (void*)soap_instantiate__vbox__IPerformanceCollector_USCOREdisableMetricsResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREqueryMetricsData:
		return (void*)soap_instantiate__vbox__IPerformanceCollector_USCOREqueryMetricsData(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__IPerformanceCollector_USCOREqueryMetricsDataResponse:
		return (void*)soap_instantiate__vbox__IPerformanceCollector_USCOREqueryMetricsDataResponse(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__InvalidObjectFault:
		return (void*)soap_instantiate__vbox__InvalidObjectFault(soap, -1, type, arrayType, n);
	case SOAP_TYPE__vbox__RuntimeFault:
		return (void*)soap_instantiate__vbox__RuntimeFault(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBoxErrorInfo_USCOREgetResultCode:
		return (void*)soap_instantiate___vbox__IVirtualBoxErrorInfo_USCOREgetResultCode(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceID:
		return (void*)soap_instantiate___vbox__IVirtualBoxErrorInfo_USCOREgetInterfaceID(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBoxErrorInfo_USCOREgetComponent:
		return (void*)soap_instantiate___vbox__IVirtualBoxErrorInfo_USCOREgetComponent(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBoxErrorInfo_USCOREgetText:
		return (void*)soap_instantiate___vbox__IVirtualBoxErrorInfo_USCOREgetText(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBoxErrorInfo_USCOREgetNext:
		return (void*)soap_instantiate___vbox__IVirtualBoxErrorInfo_USCOREgetNext(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDHCPServer_USCOREgetEnabled:
		return (void*)soap_instantiate___vbox__IDHCPServer_USCOREgetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDHCPServer_USCOREsetEnabled:
		return (void*)soap_instantiate___vbox__IDHCPServer_USCOREsetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDHCPServer_USCOREgetIPAddress:
		return (void*)soap_instantiate___vbox__IDHCPServer_USCOREgetIPAddress(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDHCPServer_USCOREgetNetworkMask:
		return (void*)soap_instantiate___vbox__IDHCPServer_USCOREgetNetworkMask(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDHCPServer_USCOREgetNetworkName:
		return (void*)soap_instantiate___vbox__IDHCPServer_USCOREgetNetworkName(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDHCPServer_USCOREgetLowerIP:
		return (void*)soap_instantiate___vbox__IDHCPServer_USCOREgetLowerIP(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDHCPServer_USCOREgetUpperIP:
		return (void*)soap_instantiate___vbox__IDHCPServer_USCOREgetUpperIP(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDHCPServer_USCOREsetConfiguration:
		return (void*)soap_instantiate___vbox__IDHCPServer_USCOREsetConfiguration(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDHCPServer_USCOREstart:
		return (void*)soap_instantiate___vbox__IDHCPServer_USCOREstart(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDHCPServer_USCOREstop:
		return (void*)soap_instantiate___vbox__IDHCPServer_USCOREstop(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetVersion:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetVersion(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetRevision:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetRevision(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetPackageType:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetPackageType(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetHomeFolder:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetHomeFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetSettingsFilePath:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetSettingsFilePath(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetSettingsFileVersion:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetSettingsFileVersion(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetSettingsFormatVersion:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetSettingsFormatVersion(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetHost:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetHost(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetSystemProperties:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetSystemProperties(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetMachines:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetMachines(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetHardDisks:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetHardDisks(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetDVDImages:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetDVDImages(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetFloppyImages:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetFloppyImages(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetProgressOperations:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetProgressOperations(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetGuestOSTypes:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetGuestOSTypes(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetSharedFolders:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetSharedFolders(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetPerformanceCollector:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetPerformanceCollector(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetDHCPServers:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetDHCPServers(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREcreateMachine:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREcreateMachine(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREcreateLegacyMachine:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREcreateLegacyMachine(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREopenMachine:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREopenMachine(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREregisterMachine:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREregisterMachine(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetMachine:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetMachine(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREfindMachine:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREfindMachine(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREunregisterMachine:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREunregisterMachine(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREcreateAppliance:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREcreateAppliance(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREcreateHardDisk:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREcreateHardDisk(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREopenHardDisk:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREopenHardDisk(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetHardDisk:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetHardDisk(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREfindHardDisk:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREfindHardDisk(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREopenDVDImage:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREopenDVDImage(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetDVDImage:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetDVDImage(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREfindDVDImage:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREfindDVDImage(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREopenFloppyImage:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREopenFloppyImage(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetFloppyImage:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetFloppyImage(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREfindFloppyImage:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREfindFloppyImage(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetGuestOSType:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetGuestOSType(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREcreateSharedFolder:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREcreateSharedFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREremoveSharedFolder:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREremoveSharedFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetNextExtraDataKey:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetNextExtraDataKey(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREgetExtraData:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREgetExtraData(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREsetExtraData:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREsetExtraData(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREopenSession:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREopenSession(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREopenRemoteSession:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREopenRemoteSession(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREopenExistingSession:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREopenExistingSession(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREwaitForPropertyChange:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREwaitForPropertyChange(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREsaveSettings:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREsaveSettings(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREsaveSettingsWithBackup:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREsaveSettingsWithBackup(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREcreateDHCPServer:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREcreateDHCPServer(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREfindDHCPServerByNetworkName:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREfindDHCPServerByNetworkName(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualBox_USCOREremoveDHCPServer:
		return (void*)soap_instantiate___vbox__IVirtualBox_USCOREremoveDHCPServer(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVFSExplorer_USCOREgetPath:
		return (void*)soap_instantiate___vbox__IVFSExplorer_USCOREgetPath(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVFSExplorer_USCOREgetType:
		return (void*)soap_instantiate___vbox__IVFSExplorer_USCOREgetType(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVFSExplorer_USCOREupdate:
		return (void*)soap_instantiate___vbox__IVFSExplorer_USCOREupdate(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVFSExplorer_USCOREcd:
		return (void*)soap_instantiate___vbox__IVFSExplorer_USCOREcd(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVFSExplorer_USCOREcdUp:
		return (void*)soap_instantiate___vbox__IVFSExplorer_USCOREcdUp(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVFSExplorer_USCOREentryList:
		return (void*)soap_instantiate___vbox__IVFSExplorer_USCOREentryList(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVFSExplorer_USCOREexists:
		return (void*)soap_instantiate___vbox__IVFSExplorer_USCOREexists(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVFSExplorer_USCOREremove:
		return (void*)soap_instantiate___vbox__IVFSExplorer_USCOREremove(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IAppliance_USCOREgetPath:
		return (void*)soap_instantiate___vbox__IAppliance_USCOREgetPath(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IAppliance_USCOREgetDisks:
		return (void*)soap_instantiate___vbox__IAppliance_USCOREgetDisks(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IAppliance_USCOREgetVirtualSystemDescriptions:
		return (void*)soap_instantiate___vbox__IAppliance_USCOREgetVirtualSystemDescriptions(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IAppliance_USCOREread:
		return (void*)soap_instantiate___vbox__IAppliance_USCOREread(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IAppliance_USCOREinterpret:
		return (void*)soap_instantiate___vbox__IAppliance_USCOREinterpret(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IAppliance_USCOREimportMachines:
		return (void*)soap_instantiate___vbox__IAppliance_USCOREimportMachines(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IAppliance_USCOREcreateVFSExplorer:
		return (void*)soap_instantiate___vbox__IAppliance_USCOREcreateVFSExplorer(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IAppliance_USCOREwrite:
		return (void*)soap_instantiate___vbox__IAppliance_USCOREwrite(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IAppliance_USCOREgetWarnings:
		return (void*)soap_instantiate___vbox__IAppliance_USCOREgetWarnings(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualSystemDescription_USCOREgetCount:
		return (void*)soap_instantiate___vbox__IVirtualSystemDescription_USCOREgetCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualSystemDescription_USCOREgetDescription:
		return (void*)soap_instantiate___vbox__IVirtualSystemDescription_USCOREgetDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualSystemDescription_USCOREgetDescriptionByType:
		return (void*)soap_instantiate___vbox__IVirtualSystemDescription_USCOREgetDescriptionByType(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualSystemDescription_USCOREgetValuesByType:
		return (void*)soap_instantiate___vbox__IVirtualSystemDescription_USCOREgetValuesByType(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualSystemDescription_USCOREsetFinalValues:
		return (void*)soap_instantiate___vbox__IVirtualSystemDescription_USCOREsetFinalValues(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVirtualSystemDescription_USCOREaddDescription:
		return (void*)soap_instantiate___vbox__IVirtualSystemDescription_USCOREaddDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREgetLogoFadeIn:
		return (void*)soap_instantiate___vbox__IBIOSSettings_USCOREgetLogoFadeIn(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREsetLogoFadeIn:
		return (void*)soap_instantiate___vbox__IBIOSSettings_USCOREsetLogoFadeIn(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREgetLogoFadeOut:
		return (void*)soap_instantiate___vbox__IBIOSSettings_USCOREgetLogoFadeOut(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREsetLogoFadeOut:
		return (void*)soap_instantiate___vbox__IBIOSSettings_USCOREsetLogoFadeOut(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREgetLogoDisplayTime:
		return (void*)soap_instantiate___vbox__IBIOSSettings_USCOREgetLogoDisplayTime(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREsetLogoDisplayTime:
		return (void*)soap_instantiate___vbox__IBIOSSettings_USCOREsetLogoDisplayTime(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREgetLogoImagePath:
		return (void*)soap_instantiate___vbox__IBIOSSettings_USCOREgetLogoImagePath(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREsetLogoImagePath:
		return (void*)soap_instantiate___vbox__IBIOSSettings_USCOREsetLogoImagePath(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREgetBootMenuMode:
		return (void*)soap_instantiate___vbox__IBIOSSettings_USCOREgetBootMenuMode(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREsetBootMenuMode:
		return (void*)soap_instantiate___vbox__IBIOSSettings_USCOREsetBootMenuMode(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREgetACPIEnabled:
		return (void*)soap_instantiate___vbox__IBIOSSettings_USCOREgetACPIEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREsetACPIEnabled:
		return (void*)soap_instantiate___vbox__IBIOSSettings_USCOREsetACPIEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREgetIOAPICEnabled:
		return (void*)soap_instantiate___vbox__IBIOSSettings_USCOREgetIOAPICEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREsetIOAPICEnabled:
		return (void*)soap_instantiate___vbox__IBIOSSettings_USCOREsetIOAPICEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREgetTimeOffset:
		return (void*)soap_instantiate___vbox__IBIOSSettings_USCOREgetTimeOffset(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREsetTimeOffset:
		return (void*)soap_instantiate___vbox__IBIOSSettings_USCOREsetTimeOffset(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREgetPXEDebugEnabled:
		return (void*)soap_instantiate___vbox__IBIOSSettings_USCOREgetPXEDebugEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IBIOSSettings_USCOREsetPXEDebugEnabled:
		return (void*)soap_instantiate___vbox__IBIOSSettings_USCOREsetPXEDebugEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetParent:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetParent(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetAccessible:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetAccessible(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetAccessError:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetAccessError(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetName:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetName(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsetName:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsetName(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetDescription:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsetDescription:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsetDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetId:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetId(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetOSTypeId:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetOSTypeId(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsetOSTypeId:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsetOSTypeId(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetHardwareVersion:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetHardwareVersion(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsetHardwareVersion:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsetHardwareVersion(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetCPUCount:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetCPUCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsetCPUCount:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsetCPUCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetMemorySize:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetMemorySize(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsetMemorySize:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsetMemorySize(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetMemoryBalloonSize:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetMemoryBalloonSize(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsetMemoryBalloonSize:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsetMemoryBalloonSize(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetStatisticsUpdateInterval:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetStatisticsUpdateInterval(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsetStatisticsUpdateInterval:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsetStatisticsUpdateInterval(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetVRAMSize:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetVRAMSize(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsetVRAMSize:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsetVRAMSize(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetAccelerate3DEnabled:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetAccelerate3DEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsetAccelerate3DEnabled:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsetAccelerate3DEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetMonitorCount:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetMonitorCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsetMonitorCount:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsetMonitorCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetBIOSSettings:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetBIOSSettings(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetHWVirtExEnabled:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetHWVirtExEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsetHWVirtExEnabled:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsetHWVirtExEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabled:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetHWVirtExNestedPagingEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabled:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsetHWVirtExNestedPagingEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetHWVirtExVPIDEnabled:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetHWVirtExVPIDEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsetHWVirtExVPIDEnabled:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsetHWVirtExVPIDEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetPAEEnabled:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetPAEEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsetPAEEnabled:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsetPAEEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetSnapshotFolder:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetSnapshotFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsetSnapshotFolder:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsetSnapshotFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetVRDPServer:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetVRDPServer(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetHardDiskAttachments:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetHardDiskAttachments(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetDVDDrive:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetDVDDrive(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetFloppyDrive:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetFloppyDrive(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetUSBController:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetUSBController(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetAudioAdapter:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetAudioAdapter(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetStorageControllers:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetStorageControllers(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetSettingsFilePath:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetSettingsFilePath(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetSettingsFileVersion:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetSettingsFileVersion(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetSettingsModified:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetSettingsModified(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetSessionState:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetSessionState(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetSessionType:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetSessionType(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetSessionPid:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetSessionPid(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetState:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetState(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetLastStateChange:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetLastStateChange(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetStateFilePath:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetStateFilePath(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetLogFolder:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetLogFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetCurrentSnapshot:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetCurrentSnapshot(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetSnapshotCount:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetSnapshotCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetCurrentStateModified:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetCurrentStateModified(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetSharedFolders:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetSharedFolders(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetClipboardMode:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetClipboardMode(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsetClipboardMode:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsetClipboardMode(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetGuestPropertyNotificationPatterns:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetGuestPropertyNotificationPatterns(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsetGuestPropertyNotificationPatterns:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsetGuestPropertyNotificationPatterns(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsetBootOrder:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsetBootOrder(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetBootOrder:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetBootOrder(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREattachHardDisk:
		return (void*)soap_instantiate___vbox__IMachine_USCOREattachHardDisk(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetHardDisk:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetHardDisk(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREdetachHardDisk:
		return (void*)soap_instantiate___vbox__IMachine_USCOREdetachHardDisk(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetHardDiskAttachmentsOfController:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetHardDiskAttachmentsOfController(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetNetworkAdapter:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetNetworkAdapter(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREaddStorageController:
		return (void*)soap_instantiate___vbox__IMachine_USCOREaddStorageController(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetStorageControllerByName:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetStorageControllerByName(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREremoveStorageController:
		return (void*)soap_instantiate___vbox__IMachine_USCOREremoveStorageController(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetSerialPort:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetSerialPort(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetParallelPort:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetParallelPort(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetNextExtraDataKey:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetNextExtraDataKey(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetExtraData:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetExtraData(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsetExtraData:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsetExtraData(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsaveSettings:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsaveSettings(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsaveSettingsWithBackup:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsaveSettingsWithBackup(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREdiscardSettings:
		return (void*)soap_instantiate___vbox__IMachine_USCOREdiscardSettings(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREdeleteSettings:
		return (void*)soap_instantiate___vbox__IMachine_USCOREdeleteSettings(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREexport:
		return (void*)soap_instantiate___vbox__IMachine_USCOREexport(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetSnapshot:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetSnapshot(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREfindSnapshot:
		return (void*)soap_instantiate___vbox__IMachine_USCOREfindSnapshot(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsetCurrentSnapshot:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsetCurrentSnapshot(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREcreateSharedFolder:
		return (void*)soap_instantiate___vbox__IMachine_USCOREcreateSharedFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREremoveSharedFolder:
		return (void*)soap_instantiate___vbox__IMachine_USCOREremoveSharedFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREcanShowConsoleWindow:
		return (void*)soap_instantiate___vbox__IMachine_USCOREcanShowConsoleWindow(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREshowConsoleWindow:
		return (void*)soap_instantiate___vbox__IMachine_USCOREshowConsoleWindow(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetGuestProperty:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetGuestProperty(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetGuestPropertyValue:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetGuestPropertyValue(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREgetGuestPropertyTimestamp:
		return (void*)soap_instantiate___vbox__IMachine_USCOREgetGuestPropertyTimestamp(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsetGuestProperty:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsetGuestProperty(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREsetGuestPropertyValue:
		return (void*)soap_instantiate___vbox__IMachine_USCOREsetGuestPropertyValue(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMachine_USCOREenumerateGuestProperties:
		return (void*)soap_instantiate___vbox__IMachine_USCOREenumerateGuestProperties(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREgetMachine:
		return (void*)soap_instantiate___vbox__IConsole_USCOREgetMachine(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREgetState:
		return (void*)soap_instantiate___vbox__IConsole_USCOREgetState(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREgetGuest:
		return (void*)soap_instantiate___vbox__IConsole_USCOREgetGuest(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREgetKeyboard:
		return (void*)soap_instantiate___vbox__IConsole_USCOREgetKeyboard(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREgetMouse:
		return (void*)soap_instantiate___vbox__IConsole_USCOREgetMouse(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREgetDisplay:
		return (void*)soap_instantiate___vbox__IConsole_USCOREgetDisplay(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREgetUSBDevices:
		return (void*)soap_instantiate___vbox__IConsole_USCOREgetUSBDevices(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREgetRemoteUSBDevices:
		return (void*)soap_instantiate___vbox__IConsole_USCOREgetRemoteUSBDevices(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREgetSharedFolders:
		return (void*)soap_instantiate___vbox__IConsole_USCOREgetSharedFolders(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREgetRemoteDisplayInfo:
		return (void*)soap_instantiate___vbox__IConsole_USCOREgetRemoteDisplayInfo(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREpowerUp:
		return (void*)soap_instantiate___vbox__IConsole_USCOREpowerUp(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREpowerUpPaused:
		return (void*)soap_instantiate___vbox__IConsole_USCOREpowerUpPaused(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREpowerDown:
		return (void*)soap_instantiate___vbox__IConsole_USCOREpowerDown(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREreset:
		return (void*)soap_instantiate___vbox__IConsole_USCOREreset(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREpause:
		return (void*)soap_instantiate___vbox__IConsole_USCOREpause(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREresume:
		return (void*)soap_instantiate___vbox__IConsole_USCOREresume(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREpowerButton:
		return (void*)soap_instantiate___vbox__IConsole_USCOREpowerButton(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREsleepButton:
		return (void*)soap_instantiate___vbox__IConsole_USCOREsleepButton(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREgetPowerButtonHandled:
		return (void*)soap_instantiate___vbox__IConsole_USCOREgetPowerButtonHandled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREgetGuestEnteredACPIMode:
		return (void*)soap_instantiate___vbox__IConsole_USCOREgetGuestEnteredACPIMode(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREsaveState:
		return (void*)soap_instantiate___vbox__IConsole_USCOREsaveState(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREadoptSavedState:
		return (void*)soap_instantiate___vbox__IConsole_USCOREadoptSavedState(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREforgetSavedState:
		return (void*)soap_instantiate___vbox__IConsole_USCOREforgetSavedState(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREgetDeviceActivity:
		return (void*)soap_instantiate___vbox__IConsole_USCOREgetDeviceActivity(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREattachUSBDevice:
		return (void*)soap_instantiate___vbox__IConsole_USCOREattachUSBDevice(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREdetachUSBDevice:
		return (void*)soap_instantiate___vbox__IConsole_USCOREdetachUSBDevice(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREfindUSBDeviceByAddress:
		return (void*)soap_instantiate___vbox__IConsole_USCOREfindUSBDeviceByAddress(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREfindUSBDeviceById:
		return (void*)soap_instantiate___vbox__IConsole_USCOREfindUSBDeviceById(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREcreateSharedFolder:
		return (void*)soap_instantiate___vbox__IConsole_USCOREcreateSharedFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREremoveSharedFolder:
		return (void*)soap_instantiate___vbox__IConsole_USCOREremoveSharedFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREtakeSnapshot:
		return (void*)soap_instantiate___vbox__IConsole_USCOREtakeSnapshot(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREdiscardSnapshot:
		return (void*)soap_instantiate___vbox__IConsole_USCOREdiscardSnapshot(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREdiscardCurrentState:
		return (void*)soap_instantiate___vbox__IConsole_USCOREdiscardCurrentState(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IConsole_USCOREdiscardCurrentSnapshotAndState:
		return (void*)soap_instantiate___vbox__IConsole_USCOREdiscardCurrentSnapshotAndState(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostDVDDrive_USCOREgetName:
		return (void*)soap_instantiate___vbox__IHostDVDDrive_USCOREgetName(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostDVDDrive_USCOREgetDescription:
		return (void*)soap_instantiate___vbox__IHostDVDDrive_USCOREgetDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostDVDDrive_USCOREgetUdi:
		return (void*)soap_instantiate___vbox__IHostDVDDrive_USCOREgetUdi(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostFloppyDrive_USCOREgetName:
		return (void*)soap_instantiate___vbox__IHostFloppyDrive_USCOREgetName(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostFloppyDrive_USCOREgetDescription:
		return (void*)soap_instantiate___vbox__IHostFloppyDrive_USCOREgetDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostFloppyDrive_USCOREgetUdi:
		return (void*)soap_instantiate___vbox__IHostFloppyDrive_USCOREgetUdi(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetName:
		return (void*)soap_instantiate___vbox__IHostNetworkInterface_USCOREgetName(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetId:
		return (void*)soap_instantiate___vbox__IHostNetworkInterface_USCOREgetId(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetNetworkName:
		return (void*)soap_instantiate___vbox__IHostNetworkInterface_USCOREgetNetworkName(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetDhcpEnabled:
		return (void*)soap_instantiate___vbox__IHostNetworkInterface_USCOREgetDhcpEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetIPAddress:
		return (void*)soap_instantiate___vbox__IHostNetworkInterface_USCOREgetIPAddress(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetNetworkMask:
		return (void*)soap_instantiate___vbox__IHostNetworkInterface_USCOREgetNetworkMask(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetIPV6Supported:
		return (void*)soap_instantiate___vbox__IHostNetworkInterface_USCOREgetIPV6Supported(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetIPV6Address:
		return (void*)soap_instantiate___vbox__IHostNetworkInterface_USCOREgetIPV6Address(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLength:
		return (void*)soap_instantiate___vbox__IHostNetworkInterface_USCOREgetIPV6NetworkMaskPrefixLength(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetHardwareAddress:
		return (void*)soap_instantiate___vbox__IHostNetworkInterface_USCOREgetHardwareAddress(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetMediumType:
		return (void*)soap_instantiate___vbox__IHostNetworkInterface_USCOREgetMediumType(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetStatus:
		return (void*)soap_instantiate___vbox__IHostNetworkInterface_USCOREgetStatus(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREgetInterfaceType:
		return (void*)soap_instantiate___vbox__IHostNetworkInterface_USCOREgetInterfaceType(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREenableStaticIpConfig:
		return (void*)soap_instantiate___vbox__IHostNetworkInterface_USCOREenableStaticIpConfig(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6:
		return (void*)soap_instantiate___vbox__IHostNetworkInterface_USCOREenableStaticIpConfigV6(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREenableDynamicIpConfig:
		return (void*)soap_instantiate___vbox__IHostNetworkInterface_USCOREenableDynamicIpConfig(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostNetworkInterface_USCOREdhcpRediscover:
		return (void*)soap_instantiate___vbox__IHostNetworkInterface_USCOREdhcpRediscover(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREgetDVDDrives:
		return (void*)soap_instantiate___vbox__IHost_USCOREgetDVDDrives(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREgetFloppyDrives:
		return (void*)soap_instantiate___vbox__IHost_USCOREgetFloppyDrives(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREgetUSBDevices:
		return (void*)soap_instantiate___vbox__IHost_USCOREgetUSBDevices(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREgetUSBDeviceFilters:
		return (void*)soap_instantiate___vbox__IHost_USCOREgetUSBDeviceFilters(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREgetNetworkInterfaces:
		return (void*)soap_instantiate___vbox__IHost_USCOREgetNetworkInterfaces(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREgetProcessorCount:
		return (void*)soap_instantiate___vbox__IHost_USCOREgetProcessorCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREgetProcessorOnlineCount:
		return (void*)soap_instantiate___vbox__IHost_USCOREgetProcessorOnlineCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREgetMemorySize:
		return (void*)soap_instantiate___vbox__IHost_USCOREgetMemorySize(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREgetMemoryAvailable:
		return (void*)soap_instantiate___vbox__IHost_USCOREgetMemoryAvailable(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREgetOperatingSystem:
		return (void*)soap_instantiate___vbox__IHost_USCOREgetOperatingSystem(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREgetOSVersion:
		return (void*)soap_instantiate___vbox__IHost_USCOREgetOSVersion(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREgetUTCTime:
		return (void*)soap_instantiate___vbox__IHost_USCOREgetUTCTime(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREgetAcceleration3DAvailable:
		return (void*)soap_instantiate___vbox__IHost_USCOREgetAcceleration3DAvailable(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREgetProcessorSpeed:
		return (void*)soap_instantiate___vbox__IHost_USCOREgetProcessorSpeed(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREgetProcessorFeature:
		return (void*)soap_instantiate___vbox__IHost_USCOREgetProcessorFeature(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREgetProcessorDescription:
		return (void*)soap_instantiate___vbox__IHost_USCOREgetProcessorDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREcreateHostOnlyNetworkInterface:
		return (void*)soap_instantiate___vbox__IHost_USCOREcreateHostOnlyNetworkInterface(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREremoveHostOnlyNetworkInterface:
		return (void*)soap_instantiate___vbox__IHost_USCOREremoveHostOnlyNetworkInterface(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREcreateUSBDeviceFilter:
		return (void*)soap_instantiate___vbox__IHost_USCOREcreateUSBDeviceFilter(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREinsertUSBDeviceFilter:
		return (void*)soap_instantiate___vbox__IHost_USCOREinsertUSBDeviceFilter(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREremoveUSBDeviceFilter:
		return (void*)soap_instantiate___vbox__IHost_USCOREremoveUSBDeviceFilter(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREfindHostDVDDrive:
		return (void*)soap_instantiate___vbox__IHost_USCOREfindHostDVDDrive(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREfindHostFloppyDrive:
		return (void*)soap_instantiate___vbox__IHost_USCOREfindHostFloppyDrive(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREfindHostNetworkInterfaceByName:
		return (void*)soap_instantiate___vbox__IHost_USCOREfindHostNetworkInterfaceByName(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREfindHostNetworkInterfaceById:
		return (void*)soap_instantiate___vbox__IHost_USCOREfindHostNetworkInterfaceById(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREfindHostNetworkInterfacesOfType:
		return (void*)soap_instantiate___vbox__IHost_USCOREfindHostNetworkInterfacesOfType(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREfindUSBDeviceById:
		return (void*)soap_instantiate___vbox__IHost_USCOREfindUSBDeviceById(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHost_USCOREfindUSBDeviceByAddress:
		return (void*)soap_instantiate___vbox__IHost_USCOREfindUSBDeviceByAddress(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetMinGuestRAM:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREgetMinGuestRAM(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetMaxGuestRAM:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREgetMaxGuestRAM(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetMinGuestVRAM:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREgetMinGuestVRAM(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetMaxGuestVRAM:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREgetMaxGuestVRAM(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetMinGuestCPUCount:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREgetMinGuestCPUCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetMaxGuestCPUCount:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREgetMaxGuestCPUCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetMaxVDISize:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREgetMaxVDISize(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetNetworkAdapterCount:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREgetNetworkAdapterCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetSerialPortCount:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREgetSerialPortCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetParallelPortCount:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREgetParallelPortCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetMaxBootPosition:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREgetMaxBootPosition(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetDefaultMachineFolder:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREgetDefaultMachineFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREsetDefaultMachineFolder:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREsetDefaultMachineFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetDefaultHardDiskFolder:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREgetDefaultHardDiskFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREsetDefaultHardDiskFolder:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREsetDefaultHardDiskFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetHardDiskFormats:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREgetHardDiskFormats(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetDefaultHardDiskFormat:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREgetDefaultHardDiskFormat(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREsetDefaultHardDiskFormat:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREsetDefaultHardDiskFormat(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibrary:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREgetRemoteDisplayAuthLibrary(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibrary:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREsetRemoteDisplayAuthLibrary(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetWebServiceAuthLibrary:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREgetWebServiceAuthLibrary(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREsetWebServiceAuthLibrary:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREsetWebServiceAuthLibrary(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetLogHistoryCount:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREgetLogHistoryCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREsetLogHistoryCount:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREsetLogHistoryCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISystemProperties_USCOREgetDefaultAudioDriver:
		return (void*)soap_instantiate___vbox__ISystemProperties_USCOREgetDefaultAudioDriver(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IGuest_USCOREgetOSTypeId:
		return (void*)soap_instantiate___vbox__IGuest_USCOREgetOSTypeId(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IGuest_USCOREgetAdditionsActive:
		return (void*)soap_instantiate___vbox__IGuest_USCOREgetAdditionsActive(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IGuest_USCOREgetAdditionsVersion:
		return (void*)soap_instantiate___vbox__IGuest_USCOREgetAdditionsVersion(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IGuest_USCOREgetSupportsSeamless:
		return (void*)soap_instantiate___vbox__IGuest_USCOREgetSupportsSeamless(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IGuest_USCOREgetSupportsGraphics:
		return (void*)soap_instantiate___vbox__IGuest_USCOREgetSupportsGraphics(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IGuest_USCOREgetMemoryBalloonSize:
		return (void*)soap_instantiate___vbox__IGuest_USCOREgetMemoryBalloonSize(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IGuest_USCOREsetMemoryBalloonSize:
		return (void*)soap_instantiate___vbox__IGuest_USCOREsetMemoryBalloonSize(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IGuest_USCOREgetStatisticsUpdateInterval:
		return (void*)soap_instantiate___vbox__IGuest_USCOREgetStatisticsUpdateInterval(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IGuest_USCOREsetStatisticsUpdateInterval:
		return (void*)soap_instantiate___vbox__IGuest_USCOREsetStatisticsUpdateInterval(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IGuest_USCOREsetCredentials:
		return (void*)soap_instantiate___vbox__IGuest_USCOREsetCredentials(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IGuest_USCOREgetStatistic:
		return (void*)soap_instantiate___vbox__IGuest_USCOREgetStatistic(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IProgress_USCOREgetId:
		return (void*)soap_instantiate___vbox__IProgress_USCOREgetId(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IProgress_USCOREgetDescription:
		return (void*)soap_instantiate___vbox__IProgress_USCOREgetDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IProgress_USCOREgetInitiator:
		return (void*)soap_instantiate___vbox__IProgress_USCOREgetInitiator(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IProgress_USCOREgetCancelable:
		return (void*)soap_instantiate___vbox__IProgress_USCOREgetCancelable(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IProgress_USCOREgetPercent:
		return (void*)soap_instantiate___vbox__IProgress_USCOREgetPercent(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IProgress_USCOREgetTimeRemaining:
		return (void*)soap_instantiate___vbox__IProgress_USCOREgetTimeRemaining(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IProgress_USCOREgetCompleted:
		return (void*)soap_instantiate___vbox__IProgress_USCOREgetCompleted(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IProgress_USCOREgetCanceled:
		return (void*)soap_instantiate___vbox__IProgress_USCOREgetCanceled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IProgress_USCOREgetResultCode:
		return (void*)soap_instantiate___vbox__IProgress_USCOREgetResultCode(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IProgress_USCOREgetErrorInfo:
		return (void*)soap_instantiate___vbox__IProgress_USCOREgetErrorInfo(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IProgress_USCOREgetOperationCount:
		return (void*)soap_instantiate___vbox__IProgress_USCOREgetOperationCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IProgress_USCOREgetOperation:
		return (void*)soap_instantiate___vbox__IProgress_USCOREgetOperation(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IProgress_USCOREgetOperationDescription:
		return (void*)soap_instantiate___vbox__IProgress_USCOREgetOperationDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IProgress_USCOREgetOperationPercent:
		return (void*)soap_instantiate___vbox__IProgress_USCOREgetOperationPercent(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IProgress_USCOREwaitForCompletion:
		return (void*)soap_instantiate___vbox__IProgress_USCOREwaitForCompletion(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IProgress_USCOREwaitForOperationCompletion:
		return (void*)soap_instantiate___vbox__IProgress_USCOREwaitForOperationCompletion(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IProgress_USCOREcancel:
		return (void*)soap_instantiate___vbox__IProgress_USCOREcancel(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISnapshot_USCOREgetId:
		return (void*)soap_instantiate___vbox__ISnapshot_USCOREgetId(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISnapshot_USCOREgetName:
		return (void*)soap_instantiate___vbox__ISnapshot_USCOREgetName(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISnapshot_USCOREsetName:
		return (void*)soap_instantiate___vbox__ISnapshot_USCOREsetName(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISnapshot_USCOREgetDescription:
		return (void*)soap_instantiate___vbox__ISnapshot_USCOREgetDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISnapshot_USCOREsetDescription:
		return (void*)soap_instantiate___vbox__ISnapshot_USCOREsetDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISnapshot_USCOREgetTimeStamp:
		return (void*)soap_instantiate___vbox__ISnapshot_USCOREgetTimeStamp(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISnapshot_USCOREgetOnline:
		return (void*)soap_instantiate___vbox__ISnapshot_USCOREgetOnline(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISnapshot_USCOREgetMachine:
		return (void*)soap_instantiate___vbox__ISnapshot_USCOREgetMachine(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISnapshot_USCOREgetParent:
		return (void*)soap_instantiate___vbox__ISnapshot_USCOREgetParent(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISnapshot_USCOREgetChildren:
		return (void*)soap_instantiate___vbox__ISnapshot_USCOREgetChildren(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMedium_USCOREgetId:
		return (void*)soap_instantiate___vbox__IMedium_USCOREgetId(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMedium_USCOREgetDescription:
		return (void*)soap_instantiate___vbox__IMedium_USCOREgetDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMedium_USCOREsetDescription:
		return (void*)soap_instantiate___vbox__IMedium_USCOREsetDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMedium_USCOREgetState:
		return (void*)soap_instantiate___vbox__IMedium_USCOREgetState(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMedium_USCOREgetLocation:
		return (void*)soap_instantiate___vbox__IMedium_USCOREgetLocation(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMedium_USCOREsetLocation:
		return (void*)soap_instantiate___vbox__IMedium_USCOREsetLocation(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMedium_USCOREgetName:
		return (void*)soap_instantiate___vbox__IMedium_USCOREgetName(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMedium_USCOREgetSize:
		return (void*)soap_instantiate___vbox__IMedium_USCOREgetSize(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMedium_USCOREgetLastAccessError:
		return (void*)soap_instantiate___vbox__IMedium_USCOREgetLastAccessError(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMedium_USCOREgetMachineIds:
		return (void*)soap_instantiate___vbox__IMedium_USCOREgetMachineIds(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMedium_USCOREgetSnapshotIds:
		return (void*)soap_instantiate___vbox__IMedium_USCOREgetSnapshotIds(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMedium_USCORElockRead:
		return (void*)soap_instantiate___vbox__IMedium_USCORElockRead(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMedium_USCOREunlockRead:
		return (void*)soap_instantiate___vbox__IMedium_USCOREunlockRead(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMedium_USCORElockWrite:
		return (void*)soap_instantiate___vbox__IMedium_USCORElockWrite(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMedium_USCOREunlockWrite:
		return (void*)soap_instantiate___vbox__IMedium_USCOREunlockWrite(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMedium_USCOREclose:
		return (void*)soap_instantiate___vbox__IMedium_USCOREclose(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDisk_USCOREgetFormat:
		return (void*)soap_instantiate___vbox__IHardDisk_USCOREgetFormat(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDisk_USCOREgetType:
		return (void*)soap_instantiate___vbox__IHardDisk_USCOREgetType(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDisk_USCOREsetType:
		return (void*)soap_instantiate___vbox__IHardDisk_USCOREsetType(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDisk_USCOREgetParent:
		return (void*)soap_instantiate___vbox__IHardDisk_USCOREgetParent(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDisk_USCOREgetChildren:
		return (void*)soap_instantiate___vbox__IHardDisk_USCOREgetChildren(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDisk_USCOREgetRoot:
		return (void*)soap_instantiate___vbox__IHardDisk_USCOREgetRoot(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDisk_USCOREgetReadOnly:
		return (void*)soap_instantiate___vbox__IHardDisk_USCOREgetReadOnly(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDisk_USCOREgetLogicalSize:
		return (void*)soap_instantiate___vbox__IHardDisk_USCOREgetLogicalSize(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDisk_USCOREgetAutoReset:
		return (void*)soap_instantiate___vbox__IHardDisk_USCOREgetAutoReset(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDisk_USCOREsetAutoReset:
		return (void*)soap_instantiate___vbox__IHardDisk_USCOREsetAutoReset(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDisk_USCOREgetProperty:
		return (void*)soap_instantiate___vbox__IHardDisk_USCOREgetProperty(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDisk_USCOREsetProperty:
		return (void*)soap_instantiate___vbox__IHardDisk_USCOREsetProperty(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDisk_USCOREgetProperties:
		return (void*)soap_instantiate___vbox__IHardDisk_USCOREgetProperties(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDisk_USCOREsetProperties:
		return (void*)soap_instantiate___vbox__IHardDisk_USCOREsetProperties(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDisk_USCOREcreateBaseStorage:
		return (void*)soap_instantiate___vbox__IHardDisk_USCOREcreateBaseStorage(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDisk_USCOREdeleteStorage:
		return (void*)soap_instantiate___vbox__IHardDisk_USCOREdeleteStorage(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDisk_USCOREcreateDiffStorage:
		return (void*)soap_instantiate___vbox__IHardDisk_USCOREcreateDiffStorage(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDisk_USCOREmergeTo:
		return (void*)soap_instantiate___vbox__IHardDisk_USCOREmergeTo(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDisk_USCOREcloneTo:
		return (void*)soap_instantiate___vbox__IHardDisk_USCOREcloneTo(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDisk_USCOREcompact:
		return (void*)soap_instantiate___vbox__IHardDisk_USCOREcompact(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDisk_USCOREreset:
		return (void*)soap_instantiate___vbox__IHardDisk_USCOREreset(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDiskFormat_USCOREgetId:
		return (void*)soap_instantiate___vbox__IHardDiskFormat_USCOREgetId(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDiskFormat_USCOREgetName:
		return (void*)soap_instantiate___vbox__IHardDiskFormat_USCOREgetName(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDiskFormat_USCOREgetFileExtensions:
		return (void*)soap_instantiate___vbox__IHardDiskFormat_USCOREgetFileExtensions(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDiskFormat_USCOREgetCapabilities:
		return (void*)soap_instantiate___vbox__IHardDiskFormat_USCOREgetCapabilities(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHardDiskFormat_USCOREdescribeProperties:
		return (void*)soap_instantiate___vbox__IHardDiskFormat_USCOREdescribeProperties(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDVDDrive_USCOREgetState:
		return (void*)soap_instantiate___vbox__IDVDDrive_USCOREgetState(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDVDDrive_USCOREgetPassthrough:
		return (void*)soap_instantiate___vbox__IDVDDrive_USCOREgetPassthrough(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDVDDrive_USCOREsetPassthrough:
		return (void*)soap_instantiate___vbox__IDVDDrive_USCOREsetPassthrough(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDVDDrive_USCOREmountImage:
		return (void*)soap_instantiate___vbox__IDVDDrive_USCOREmountImage(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDVDDrive_USCOREcaptureHostDrive:
		return (void*)soap_instantiate___vbox__IDVDDrive_USCOREcaptureHostDrive(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDVDDrive_USCOREunmount:
		return (void*)soap_instantiate___vbox__IDVDDrive_USCOREunmount(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDVDDrive_USCOREgetImage:
		return (void*)soap_instantiate___vbox__IDVDDrive_USCOREgetImage(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDVDDrive_USCOREgetHostDrive:
		return (void*)soap_instantiate___vbox__IDVDDrive_USCOREgetHostDrive(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IFloppyDrive_USCOREgetEnabled:
		return (void*)soap_instantiate___vbox__IFloppyDrive_USCOREgetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IFloppyDrive_USCOREsetEnabled:
		return (void*)soap_instantiate___vbox__IFloppyDrive_USCOREsetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IFloppyDrive_USCOREgetState:
		return (void*)soap_instantiate___vbox__IFloppyDrive_USCOREgetState(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IFloppyDrive_USCOREmountImage:
		return (void*)soap_instantiate___vbox__IFloppyDrive_USCOREmountImage(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IFloppyDrive_USCOREcaptureHostDrive:
		return (void*)soap_instantiate___vbox__IFloppyDrive_USCOREcaptureHostDrive(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IFloppyDrive_USCOREunmount:
		return (void*)soap_instantiate___vbox__IFloppyDrive_USCOREunmount(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IFloppyDrive_USCOREgetImage:
		return (void*)soap_instantiate___vbox__IFloppyDrive_USCOREgetImage(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IFloppyDrive_USCOREgetHostDrive:
		return (void*)soap_instantiate___vbox__IFloppyDrive_USCOREgetHostDrive(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IKeyboard_USCOREputScancode:
		return (void*)soap_instantiate___vbox__IKeyboard_USCOREputScancode(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IKeyboard_USCOREputScancodes:
		return (void*)soap_instantiate___vbox__IKeyboard_USCOREputScancodes(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IKeyboard_USCOREputCAD:
		return (void*)soap_instantiate___vbox__IKeyboard_USCOREputCAD(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMouse_USCOREgetAbsoluteSupported:
		return (void*)soap_instantiate___vbox__IMouse_USCOREgetAbsoluteSupported(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMouse_USCOREputMouseEvent:
		return (void*)soap_instantiate___vbox__IMouse_USCOREputMouseEvent(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IMouse_USCOREputMouseEventAbsolute:
		return (void*)soap_instantiate___vbox__IMouse_USCOREputMouseEventAbsolute(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDisplay_USCOREgetWidth:
		return (void*)soap_instantiate___vbox__IDisplay_USCOREgetWidth(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDisplay_USCOREgetHeight:
		return (void*)soap_instantiate___vbox__IDisplay_USCOREgetHeight(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDisplay_USCOREgetBitsPerPixel:
		return (void*)soap_instantiate___vbox__IDisplay_USCOREgetBitsPerPixel(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDisplay_USCOREsetVideoModeHint:
		return (void*)soap_instantiate___vbox__IDisplay_USCOREsetVideoModeHint(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDisplay_USCOREsetSeamlessMode:
		return (void*)soap_instantiate___vbox__IDisplay_USCOREsetSeamlessMode(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDisplay_USCOREtakeScreenShotSlow:
		return (void*)soap_instantiate___vbox__IDisplay_USCOREtakeScreenShotSlow(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDisplay_USCOREinvalidateAndUpdate:
		return (void*)soap_instantiate___vbox__IDisplay_USCOREinvalidateAndUpdate(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDisplay_USCOREresizeCompleted:
		return (void*)soap_instantiate___vbox__IDisplay_USCOREresizeCompleted(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IDisplay_USCOREupdateCompleted:
		return (void*)soap_instantiate___vbox__IDisplay_USCOREupdateCompleted(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREgetAdapterType:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREgetAdapterType(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREsetAdapterType:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREsetAdapterType(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREgetSlot:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREgetSlot(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREgetEnabled:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREgetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREsetEnabled:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREsetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREgetMACAddress:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREgetMACAddress(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREsetMACAddress:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREsetMACAddress(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREgetAttachmentType:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREgetAttachmentType(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREgetHostInterface:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREgetHostInterface(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREsetHostInterface:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREsetHostInterface(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREgetInternalNetwork:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREgetInternalNetwork(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREsetInternalNetwork:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREsetInternalNetwork(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREgetNATNetwork:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREgetNATNetwork(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREsetNATNetwork:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREsetNATNetwork(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREgetCableConnected:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREgetCableConnected(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREsetCableConnected:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREsetCableConnected(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREgetLineSpeed:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREgetLineSpeed(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREsetLineSpeed:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREsetLineSpeed(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREgetTraceEnabled:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREgetTraceEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREsetTraceEnabled:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREsetTraceEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREgetTraceFile:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREgetTraceFile(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREsetTraceFile:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREsetTraceFile(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREattachToNAT:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREattachToNAT(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREattachToBridgedInterface:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREattachToBridgedInterface(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREattachToInternalNetwork:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREattachToInternalNetwork(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREattachToHostOnlyInterface:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREattachToHostOnlyInterface(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__INetworkAdapter_USCOREdetach:
		return (void*)soap_instantiate___vbox__INetworkAdapter_USCOREdetach(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISerialPort_USCOREgetSlot:
		return (void*)soap_instantiate___vbox__ISerialPort_USCOREgetSlot(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISerialPort_USCOREgetEnabled:
		return (void*)soap_instantiate___vbox__ISerialPort_USCOREgetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISerialPort_USCOREsetEnabled:
		return (void*)soap_instantiate___vbox__ISerialPort_USCOREsetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISerialPort_USCOREgetIOBase:
		return (void*)soap_instantiate___vbox__ISerialPort_USCOREgetIOBase(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISerialPort_USCOREsetIOBase:
		return (void*)soap_instantiate___vbox__ISerialPort_USCOREsetIOBase(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISerialPort_USCOREgetIRQ:
		return (void*)soap_instantiate___vbox__ISerialPort_USCOREgetIRQ(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISerialPort_USCOREsetIRQ:
		return (void*)soap_instantiate___vbox__ISerialPort_USCOREsetIRQ(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISerialPort_USCOREgetHostMode:
		return (void*)soap_instantiate___vbox__ISerialPort_USCOREgetHostMode(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISerialPort_USCOREsetHostMode:
		return (void*)soap_instantiate___vbox__ISerialPort_USCOREsetHostMode(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISerialPort_USCOREgetServer:
		return (void*)soap_instantiate___vbox__ISerialPort_USCOREgetServer(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISerialPort_USCOREsetServer:
		return (void*)soap_instantiate___vbox__ISerialPort_USCOREsetServer(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISerialPort_USCOREgetPath:
		return (void*)soap_instantiate___vbox__ISerialPort_USCOREgetPath(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISerialPort_USCOREsetPath:
		return (void*)soap_instantiate___vbox__ISerialPort_USCOREsetPath(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IParallelPort_USCOREgetSlot:
		return (void*)soap_instantiate___vbox__IParallelPort_USCOREgetSlot(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IParallelPort_USCOREgetEnabled:
		return (void*)soap_instantiate___vbox__IParallelPort_USCOREgetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IParallelPort_USCOREsetEnabled:
		return (void*)soap_instantiate___vbox__IParallelPort_USCOREsetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IParallelPort_USCOREgetIOBase:
		return (void*)soap_instantiate___vbox__IParallelPort_USCOREgetIOBase(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IParallelPort_USCOREsetIOBase:
		return (void*)soap_instantiate___vbox__IParallelPort_USCOREsetIOBase(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IParallelPort_USCOREgetIRQ:
		return (void*)soap_instantiate___vbox__IParallelPort_USCOREgetIRQ(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IParallelPort_USCOREsetIRQ:
		return (void*)soap_instantiate___vbox__IParallelPort_USCOREsetIRQ(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IParallelPort_USCOREgetPath:
		return (void*)soap_instantiate___vbox__IParallelPort_USCOREgetPath(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IParallelPort_USCOREsetPath:
		return (void*)soap_instantiate___vbox__IParallelPort_USCOREsetPath(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBController_USCOREgetEnabled:
		return (void*)soap_instantiate___vbox__IUSBController_USCOREgetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBController_USCOREsetEnabled:
		return (void*)soap_instantiate___vbox__IUSBController_USCOREsetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBController_USCOREgetEnabledEhci:
		return (void*)soap_instantiate___vbox__IUSBController_USCOREgetEnabledEhci(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBController_USCOREsetEnabledEhci:
		return (void*)soap_instantiate___vbox__IUSBController_USCOREsetEnabledEhci(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBController_USCOREgetUSBStandard:
		return (void*)soap_instantiate___vbox__IUSBController_USCOREgetUSBStandard(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBController_USCOREgetDeviceFilters:
		return (void*)soap_instantiate___vbox__IUSBController_USCOREgetDeviceFilters(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBController_USCOREcreateDeviceFilter:
		return (void*)soap_instantiate___vbox__IUSBController_USCOREcreateDeviceFilter(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBController_USCOREinsertDeviceFilter:
		return (void*)soap_instantiate___vbox__IUSBController_USCOREinsertDeviceFilter(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBController_USCOREremoveDeviceFilter:
		return (void*)soap_instantiate___vbox__IUSBController_USCOREremoveDeviceFilter(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDevice_USCOREgetId:
		return (void*)soap_instantiate___vbox__IUSBDevice_USCOREgetId(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDevice_USCOREgetVendorId:
		return (void*)soap_instantiate___vbox__IUSBDevice_USCOREgetVendorId(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDevice_USCOREgetProductId:
		return (void*)soap_instantiate___vbox__IUSBDevice_USCOREgetProductId(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDevice_USCOREgetRevision:
		return (void*)soap_instantiate___vbox__IUSBDevice_USCOREgetRevision(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDevice_USCOREgetManufacturer:
		return (void*)soap_instantiate___vbox__IUSBDevice_USCOREgetManufacturer(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDevice_USCOREgetProduct:
		return (void*)soap_instantiate___vbox__IUSBDevice_USCOREgetProduct(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDevice_USCOREgetSerialNumber:
		return (void*)soap_instantiate___vbox__IUSBDevice_USCOREgetSerialNumber(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDevice_USCOREgetAddress:
		return (void*)soap_instantiate___vbox__IUSBDevice_USCOREgetAddress(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDevice_USCOREgetPort:
		return (void*)soap_instantiate___vbox__IUSBDevice_USCOREgetPort(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDevice_USCOREgetVersion:
		return (void*)soap_instantiate___vbox__IUSBDevice_USCOREgetVersion(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDevice_USCOREgetPortVersion:
		return (void*)soap_instantiate___vbox__IUSBDevice_USCOREgetPortVersion(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDevice_USCOREgetRemote:
		return (void*)soap_instantiate___vbox__IUSBDevice_USCOREgetRemote(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREgetName:
		return (void*)soap_instantiate___vbox__IUSBDeviceFilter_USCOREgetName(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREsetName:
		return (void*)soap_instantiate___vbox__IUSBDeviceFilter_USCOREsetName(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREgetActive:
		return (void*)soap_instantiate___vbox__IUSBDeviceFilter_USCOREgetActive(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREsetActive:
		return (void*)soap_instantiate___vbox__IUSBDeviceFilter_USCOREsetActive(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREgetVendorId:
		return (void*)soap_instantiate___vbox__IUSBDeviceFilter_USCOREgetVendorId(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREsetVendorId:
		return (void*)soap_instantiate___vbox__IUSBDeviceFilter_USCOREsetVendorId(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREgetProductId:
		return (void*)soap_instantiate___vbox__IUSBDeviceFilter_USCOREgetProductId(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREsetProductId:
		return (void*)soap_instantiate___vbox__IUSBDeviceFilter_USCOREsetProductId(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREgetRevision:
		return (void*)soap_instantiate___vbox__IUSBDeviceFilter_USCOREgetRevision(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREsetRevision:
		return (void*)soap_instantiate___vbox__IUSBDeviceFilter_USCOREsetRevision(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREgetManufacturer:
		return (void*)soap_instantiate___vbox__IUSBDeviceFilter_USCOREgetManufacturer(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREsetManufacturer:
		return (void*)soap_instantiate___vbox__IUSBDeviceFilter_USCOREsetManufacturer(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREgetProduct:
		return (void*)soap_instantiate___vbox__IUSBDeviceFilter_USCOREgetProduct(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREsetProduct:
		return (void*)soap_instantiate___vbox__IUSBDeviceFilter_USCOREsetProduct(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREgetSerialNumber:
		return (void*)soap_instantiate___vbox__IUSBDeviceFilter_USCOREgetSerialNumber(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREsetSerialNumber:
		return (void*)soap_instantiate___vbox__IUSBDeviceFilter_USCOREsetSerialNumber(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREgetPort:
		return (void*)soap_instantiate___vbox__IUSBDeviceFilter_USCOREgetPort(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREsetPort:
		return (void*)soap_instantiate___vbox__IUSBDeviceFilter_USCOREsetPort(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREgetRemote:
		return (void*)soap_instantiate___vbox__IUSBDeviceFilter_USCOREgetRemote(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREsetRemote:
		return (void*)soap_instantiate___vbox__IUSBDeviceFilter_USCOREsetRemote(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREgetMaskedInterfaces:
		return (void*)soap_instantiate___vbox__IUSBDeviceFilter_USCOREgetMaskedInterfaces(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IUSBDeviceFilter_USCOREsetMaskedInterfaces:
		return (void*)soap_instantiate___vbox__IUSBDeviceFilter_USCOREsetMaskedInterfaces(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostUSBDevice_USCOREgetState:
		return (void*)soap_instantiate___vbox__IHostUSBDevice_USCOREgetState(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostUSBDeviceFilter_USCOREgetAction:
		return (void*)soap_instantiate___vbox__IHostUSBDeviceFilter_USCOREgetAction(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IHostUSBDeviceFilter_USCOREsetAction:
		return (void*)soap_instantiate___vbox__IHostUSBDeviceFilter_USCOREsetAction(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IAudioAdapter_USCOREgetEnabled:
		return (void*)soap_instantiate___vbox__IAudioAdapter_USCOREgetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IAudioAdapter_USCOREsetEnabled:
		return (void*)soap_instantiate___vbox__IAudioAdapter_USCOREsetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IAudioAdapter_USCOREgetAudioController:
		return (void*)soap_instantiate___vbox__IAudioAdapter_USCOREgetAudioController(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IAudioAdapter_USCOREsetAudioController:
		return (void*)soap_instantiate___vbox__IAudioAdapter_USCOREsetAudioController(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IAudioAdapter_USCOREgetAudioDriver:
		return (void*)soap_instantiate___vbox__IAudioAdapter_USCOREgetAudioDriver(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IAudioAdapter_USCOREsetAudioDriver:
		return (void*)soap_instantiate___vbox__IAudioAdapter_USCOREsetAudioDriver(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVRDPServer_USCOREgetEnabled:
		return (void*)soap_instantiate___vbox__IVRDPServer_USCOREgetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVRDPServer_USCOREsetEnabled:
		return (void*)soap_instantiate___vbox__IVRDPServer_USCOREsetEnabled(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVRDPServer_USCOREgetPort:
		return (void*)soap_instantiate___vbox__IVRDPServer_USCOREgetPort(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVRDPServer_USCOREsetPort:
		return (void*)soap_instantiate___vbox__IVRDPServer_USCOREsetPort(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVRDPServer_USCOREgetNetAddress:
		return (void*)soap_instantiate___vbox__IVRDPServer_USCOREgetNetAddress(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVRDPServer_USCOREsetNetAddress:
		return (void*)soap_instantiate___vbox__IVRDPServer_USCOREsetNetAddress(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVRDPServer_USCOREgetAuthType:
		return (void*)soap_instantiate___vbox__IVRDPServer_USCOREgetAuthType(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVRDPServer_USCOREsetAuthType:
		return (void*)soap_instantiate___vbox__IVRDPServer_USCOREsetAuthType(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVRDPServer_USCOREgetAuthTimeout:
		return (void*)soap_instantiate___vbox__IVRDPServer_USCOREgetAuthTimeout(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVRDPServer_USCOREsetAuthTimeout:
		return (void*)soap_instantiate___vbox__IVRDPServer_USCOREsetAuthTimeout(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVRDPServer_USCOREgetAllowMultiConnection:
		return (void*)soap_instantiate___vbox__IVRDPServer_USCOREgetAllowMultiConnection(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVRDPServer_USCOREsetAllowMultiConnection:
		return (void*)soap_instantiate___vbox__IVRDPServer_USCOREsetAllowMultiConnection(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVRDPServer_USCOREgetReuseSingleConnection:
		return (void*)soap_instantiate___vbox__IVRDPServer_USCOREgetReuseSingleConnection(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IVRDPServer_USCOREsetReuseSingleConnection:
		return (void*)soap_instantiate___vbox__IVRDPServer_USCOREsetReuseSingleConnection(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISession_USCOREgetState:
		return (void*)soap_instantiate___vbox__ISession_USCOREgetState(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISession_USCOREgetType:
		return (void*)soap_instantiate___vbox__ISession_USCOREgetType(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISession_USCOREgetMachine:
		return (void*)soap_instantiate___vbox__ISession_USCOREgetMachine(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISession_USCOREgetConsole:
		return (void*)soap_instantiate___vbox__ISession_USCOREgetConsole(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__ISession_USCOREclose:
		return (void*)soap_instantiate___vbox__ISession_USCOREclose(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IStorageController_USCOREgetName:
		return (void*)soap_instantiate___vbox__IStorageController_USCOREgetName(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IStorageController_USCOREgetMaxDevicesPerPortCount:
		return (void*)soap_instantiate___vbox__IStorageController_USCOREgetMaxDevicesPerPortCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IStorageController_USCOREgetMinPortCount:
		return (void*)soap_instantiate___vbox__IStorageController_USCOREgetMinPortCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IStorageController_USCOREgetMaxPortCount:
		return (void*)soap_instantiate___vbox__IStorageController_USCOREgetMaxPortCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IStorageController_USCOREgetInstance:
		return (void*)soap_instantiate___vbox__IStorageController_USCOREgetInstance(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IStorageController_USCOREsetInstance:
		return (void*)soap_instantiate___vbox__IStorageController_USCOREsetInstance(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IStorageController_USCOREgetPortCount:
		return (void*)soap_instantiate___vbox__IStorageController_USCOREgetPortCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IStorageController_USCOREsetPortCount:
		return (void*)soap_instantiate___vbox__IStorageController_USCOREsetPortCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IStorageController_USCOREgetBus:
		return (void*)soap_instantiate___vbox__IStorageController_USCOREgetBus(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IStorageController_USCOREgetControllerType:
		return (void*)soap_instantiate___vbox__IStorageController_USCOREgetControllerType(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IStorageController_USCOREsetControllerType:
		return (void*)soap_instantiate___vbox__IStorageController_USCOREsetControllerType(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IStorageController_USCOREGetIDEEmulationPort:
		return (void*)soap_instantiate___vbox__IStorageController_USCOREGetIDEEmulationPort(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IStorageController_USCORESetIDEEmulationPort:
		return (void*)soap_instantiate___vbox__IStorageController_USCORESetIDEEmulationPort(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IManagedObjectRef_USCOREgetInterfaceName:
		return (void*)soap_instantiate___vbox__IManagedObjectRef_USCOREgetInterfaceName(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IManagedObjectRef_USCORErelease:
		return (void*)soap_instantiate___vbox__IManagedObjectRef_USCORErelease(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IWebsessionManager_USCORElogon:
		return (void*)soap_instantiate___vbox__IWebsessionManager_USCORElogon(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IWebsessionManager_USCOREgetSessionObject:
		return (void*)soap_instantiate___vbox__IWebsessionManager_USCOREgetSessionObject(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IWebsessionManager_USCORElogoff:
		return (void*)soap_instantiate___vbox__IWebsessionManager_USCORElogoff(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IPerformanceMetric_USCOREgetMetricName:
		return (void*)soap_instantiate___vbox__IPerformanceMetric_USCOREgetMetricName(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IPerformanceMetric_USCOREgetObject:
		return (void*)soap_instantiate___vbox__IPerformanceMetric_USCOREgetObject(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IPerformanceMetric_USCOREgetDescription:
		return (void*)soap_instantiate___vbox__IPerformanceMetric_USCOREgetDescription(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IPerformanceMetric_USCOREgetPeriod:
		return (void*)soap_instantiate___vbox__IPerformanceMetric_USCOREgetPeriod(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IPerformanceMetric_USCOREgetCount:
		return (void*)soap_instantiate___vbox__IPerformanceMetric_USCOREgetCount(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IPerformanceMetric_USCOREgetUnit:
		return (void*)soap_instantiate___vbox__IPerformanceMetric_USCOREgetUnit(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IPerformanceMetric_USCOREgetMinimumValue:
		return (void*)soap_instantiate___vbox__IPerformanceMetric_USCOREgetMinimumValue(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IPerformanceMetric_USCOREgetMaximumValue:
		return (void*)soap_instantiate___vbox__IPerformanceMetric_USCOREgetMaximumValue(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IPerformanceCollector_USCOREgetMetricNames:
		return (void*)soap_instantiate___vbox__IPerformanceCollector_USCOREgetMetricNames(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IPerformanceCollector_USCOREgetMetrics:
		return (void*)soap_instantiate___vbox__IPerformanceCollector_USCOREgetMetrics(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IPerformanceCollector_USCOREsetupMetrics:
		return (void*)soap_instantiate___vbox__IPerformanceCollector_USCOREsetupMetrics(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IPerformanceCollector_USCOREenableMetrics:
		return (void*)soap_instantiate___vbox__IPerformanceCollector_USCOREenableMetrics(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IPerformanceCollector_USCOREdisableMetrics:
		return (void*)soap_instantiate___vbox__IPerformanceCollector_USCOREdisableMetrics(soap, -1, type, arrayType, n);
	case SOAP_TYPE___vbox__IPerformanceCollector_USCOREqueryMetricsData:
		return (void*)soap_instantiate___vbox__IPerformanceCollector_USCOREqueryMetricsData(soap, -1, type, arrayType, n);
#ifndef WITH_NOGLOBAL
	case SOAP_TYPE_SOAP_ENV__Header:
		return (void*)soap_instantiate_SOAP_ENV__Header(soap, -1, type, arrayType, n);
#endif
#ifndef WITH_NOGLOBAL
	case SOAP_TYPE_SOAP_ENV__Code:
		return (void*)soap_instantiate_SOAP_ENV__Code(soap, -1, type, arrayType, n);
#endif
#ifndef WITH_NOGLOBAL
	case SOAP_TYPE_SOAP_ENV__Reason:
		return (void*)soap_instantiate_SOAP_ENV__Reason(soap, -1, type, arrayType, n);
#endif
#ifndef WITH_NOGLOBAL
	case SOAP_TYPE_SOAP_ENV__Fault:
		return (void*)soap_instantiate_SOAP_ENV__Fault(soap, -1, type, arrayType, n);
#endif
	case SOAP_TYPE_std__vectorTemplateOfunsignedByte:
		return (void*)soap_instantiate_std__vectorTemplateOfunsignedByte(soap, -1, type, arrayType, n);
	case SOAP_TYPE_std__vectorTemplateOfint:
		return (void*)soap_instantiate_std__vectorTemplateOfint(soap, -1, type, arrayType, n);
	case SOAP_TYPE_std__vectorTemplateOfvbox__DataType:
		return (void*)soap_instantiate_std__vectorTemplateOfvbox__DataType(soap, -1, type, arrayType, n);
	case SOAP_TYPE_std__vectorTemplateOfunsignedLONG64:
		return (void*)soap_instantiate_std__vectorTemplateOfunsignedLONG64(soap, -1, type, arrayType, n);
	case SOAP_TYPE_std__vectorTemplateOfPointerTovbox__IHardDiskAttachment:
		return (void*)soap_instantiate_std__vectorTemplateOfPointerTovbox__IHardDiskAttachment(soap, -1, type, arrayType, n);
	case SOAP_TYPE_std__vectorTemplateOfbool:
		return (void*)soap_instantiate_std__vectorTemplateOfbool(soap, -1, type, arrayType, n);
	case SOAP_TYPE_std__vectorTemplateOfvbox__VirtualSystemDescriptionType:
		return (void*)soap_instantiate_std__vectorTemplateOfvbox__VirtualSystemDescriptionType(soap, -1, type, arrayType, n);
	case SOAP_TYPE_std__vectorTemplateOfunsignedInt:
		return (void*)soap_instantiate_std__vectorTemplateOfunsignedInt(soap, -1, type, arrayType, n);
	case SOAP_TYPE_std__vectorTemplateOfPointerTovbox__ISharedFolder:
		return (void*)soap_instantiate_std__vectorTemplateOfPointerTovbox__ISharedFolder(soap, -1, type, arrayType, n);
	case SOAP_TYPE_std__vectorTemplateOfPointerTovbox__IGuestOSType:
		return (void*)soap_instantiate_std__vectorTemplateOfPointerTovbox__IGuestOSType(soap, -1, type, arrayType, n);
	case SOAP_TYPE_std__vectorTemplateOfstd__string:
		return (void*)soap_instantiate_std__vectorTemplateOfstd__string(soap, -1, type, arrayType, n);
	}
	return NULL;
}

